package com.huoran.api;

import com.huoran.api.impl.NakadaiClientImpl;
import com.huoran.common.constant.ServiceNameConstant;
import com.huoran.common.entity.School;
import com.huoran.common.entity.to.JudgmentPointDataTo;
import com.huoran.common.response.R;
import com.huoran.config.FeignSimpleEncoderConfig;
import feign.Headers;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * @Author cheney
 * @DATE 2021/7/15 10:09
 * @Version 1.0
 * 远程调用中台
 */
@FeignClient(value = ServiceNameConstant.NAKADAI,
        fallbackFactory = NakadaiClientImpl.class, configuration = FeignSimpleEncoderConfig.class)
@Headers("Accept: application/json")
public interface NakadaiClient {

    /**
     * 查询客户订单
     */
    @Headers({"Content-Type: application/json", "Accept: application/json"})
    @GetMapping(value = "/nakadai/nakadai/order/getCustomerOrder", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
    R getCustomerOrder(@RequestParam("customerId") Integer customerId);

    /**
     * 查询订单其他信息
     */
    @Headers({"Content-Type: application/json", "Accept: application/json"})
    @GetMapping(value = "/nakadai/nakadai/orderOther/getOrderOther", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
    R getOrderOther(@RequestParam("orderId") Integer orderId, @RequestParam("authority") Integer authority);

    /**
     * 删除多个文件
     *
     * @param keys 多个文件完整名称集合
     * @return 返回多个删除的文件名
     */
    // @DeleteMapping("/nakadai/oss/fileDeletion")
    // R fileDeletion(@RequestParam("keys") List<String> keys);

    /**
     * 上传文件
     *
     * @param file 文件
     * @return 文件路径及相关信息
     */
    @PostMapping("/nakadai/nakadai/oss/fileUpload")
    R fileUpload(@RequestParam("file") MultipartFile file);

    /**
     * 根据学校id获取客户id
     *
     * @param schoolId
     * @return
     */
    @GetMapping("/nakadai/nakadai/customer/getCustomerBySchoolId")
    Integer getCustomerBySchoolId(@RequestParam("schoolId") Integer schoolId);

    /**
     * 根据客户id获取所有的系统id
     *
     * @param customerId
     * @return
     */
    @GetMapping("/nakadai/nakadai/curriculum/getSystemIdByCustomerId")
    List<Integer> getSystemIdByCustomerId(@RequestParam("customerId") Integer customerId);

    /**
     * 根据系统id获取系统名称
     *
     * @param SystemId
     * @return
     */
    @GetMapping("/nakadai/nakadai/serviceConfiguration/getSystemNameBySystemId")
    String getSystemNameBySystemId(@RequestParam("SystemId") Integer SystemId);

    /**
     * @Description :  根据学校id获取学校购买的课程id(此处返回逗号拼接格式)
     * @Param schoolId
     * @Author Rong---2021/11/8
     */
    @GetMapping("/nakadai/nakadai/curriculum/getSchoolCourse")
    String getSchoolCourse(@RequestParam("schoolId") Integer schoolId);

    /**
     * 查询客户订单信息
     */
    @PostMapping("/nakadai/nakadai/orderOther/getCustomerOrderOther")
    JudgmentPointDataTo getCustomerOrderOther(@RequestParam("schoolId") Integer schoolId, @RequestParam("systemId") Integer systemId, @RequestParam("name") String name);

    @DeleteMapping("/nakadai/nakadai/oss/fileDeletion")
    R fileDeletion(@RequestParam("keys")List<String> keys,@RequestParam("request") HttpServletRequest request);


    @GetMapping("/nakadai/nakadai/school/getTheSchoolNameBySchoolId")
    School getTheSchoolNameBySchoolId(@RequestParam("schoolId") Integer schoolId);

    @GetMapping("/nakadai/nakadai/school/getTheIdBySchoolName")
    School getTheIdBySchoolName(@RequestParam("schoolName") String schoolName);

    @PostMapping("/nakadai/message/refreshPageNotification")
    void refreshPageNotification(@RequestParam("username") String username, @RequestParam("to") String to, @RequestParam("content") String content);
}
