package com.huoran.api;

import com.huoran.api.impl.OccupationlabImpl;
import com.huoran.common.constant.ServiceNameConstant;
import com.huoran.common.entity.ExperimentalReport;
import com.huoran.common.entity.ProjectManage;
import com.huoran.common.entity.UserScore;
import com.huoran.common.entity.to.JudgmentPointDataTo;
import com.huoran.common.entity.to.ProjectDataTo;
import com.huoran.common.response.R;
import io.swagger.annotations.ApiOperation;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @description 职站远程调用客户端
 * @author: Mr.JK
 * @create: 2021-09-10 15:03
 **/
@FeignClient(value = ServiceNameConstant.OCCUPATIONLAB, fallbackFactory = OccupationlabImpl.class)
public interface OccupationlabClient {

    /**
     * 添加实验报告
     *
     * @param experimentalReport
     * @return 添加成功返回主键id
     */
    @PostMapping("/occupationlab/occupationlab/experimentalReport/addExperimentalReport")
    Integer addExperimentalReport(@RequestBody ExperimentalReport experimentalReport);

    /**
     * 添加用户分数 单条
     *
     * @param userScore
     * @return 添加成功返回主键id
     */
    @PostMapping("/occupationlab/occupationlab/userScore/addUserScore")
    Integer addUserScore(@RequestBody UserScore userScore);

    /**
     * 获取项目管理、判分点关系表分数
     *
     * @param projectId  项目id
     * @param JudgmentId 判分点id
     * @return 分数
     */
    @GetMapping("/occupationlab/occupationlab/projectJudgment/getJudgmentScore")
    Integer getJudgmentScore(@RequestParam("projectId") Integer projectId, @RequestParam("JudgmentId") Integer JudgmentId);

    /**
     * 根据系统id、项目权限获取系统内置项目
     *
     * @param systemId
     * @param permissions
     * @return
     */
    @GetMapping("/occupationlab/occupationlab/projectManage/getInternalProjectBySystemId")
    List<ProjectManage> getInternalProjectBySystemId(@RequestParam Integer systemId, @RequestParam Integer permissions);

    @ApiOperation(value = "根据系统id获取项目(获取课程下与该学校下的所有项目)")
    @GetMapping("/occupationlab/occupationlab/projectManage/getProjectBySystemId")
    R getProjectBySystemId(@RequestParam String systemId, @RequestHeader String token, @RequestParam Integer permissions, @RequestParam Integer cId, @RequestParam Integer mallId);

    /**
     * @Description :   创建客户复制题库
     * @Param schoolId
     * @Author Rong---2021/11/2
     */
    @PostMapping("/occupationlab/occupationlab/questions/copyQuestion")
    boolean copyQuestion(@RequestParam Integer schoolId, @RequestParam Integer accountId);

    /**
     * @Description :   新增客户新增学生管理初始数据
     * @Param req
     * @Param accountId
     * @Author Rong---2021/11/2
     */
    @PostMapping("/occupationlab/occupationlab/architecture/saveData")
    boolean saveData(@RequestParam("schoolId") Integer schoolId);

    /**
     * @Description :  添加课程初始数据
     * @Param schoolId
     * @Param accountId
     * @Author Rong---2021/11/2
     */
    @PostMapping("/occupationlab/occupationlab/management/edu/course/addCourseData")
    boolean addCourseData(@RequestParam("schoolId") Integer schoolId, @RequestParam("accountId") Integer accountId);

    /**
     * 添加赛事样例数据
     *
     * @param schoolId
     * @param accountId
     * @return
     */
    @PostMapping("/occupationlab/occupationlab/enterprise/match/contest/eventSampleData")
    boolean eventSampleData(@RequestParam("schoolId") Integer schoolId, @RequestParam("accountId") Integer accountId);

    /***
     * @Description :   添加资讯内置数据
     * @Param
     * @Author Rong---2021/11/3
     */
    @PostMapping("/occupationlab/occupationlab/information/article/addArticleData")
    boolean addArticleData(@RequestParam("accountId") Integer accountId, @RequestParam("schoolId") Integer schoolId);

    /**
     * @Description :   创建客户复制测评规则
     * @Param schoolId
     * @Author Rong---2021/11/2
     */
    @PostMapping("/occupationlab/occupationlab/questions/copyEvaluationRules")
    boolean copyEvaluationRules(@RequestParam Integer schoolId);

    /**
     * @Description :   创建客户添加内置员工组织架构数据
     * @Param schoolId
     * @Author Rong---2021/11/11
     */
    @PostMapping("/occupationlab/occupationlab/staff/saveStaffData")
    boolean saveStaffData(@RequestParam Integer schoolId);

    @PostMapping("/occupationlab/occupationlab/assessment/submitExam")
    boolean submitExam(@RequestParam("assessmentId") Integer assessmentId,
                       @RequestParam("classId") Integer classId,
                       @RequestParam("accountId") Integer accountId,
                       @RequestParam("score") Integer score);

    @GetMapping("/occupationlab/occupationlab/assessment/getTeacherIdByAssessmentId")
    Integer getTeacherIdByAssessmentId(@RequestParam("assessmentId") Integer assessmentId);

    /**
     * 根据项目id查询判分点
     */
    @PostMapping("/occupationlab/occupationlab/projectManage/getProjectPoints")
    JudgmentPointDataTo getProjectPoints(@RequestBody ProjectDataTo projectDataTo);


    @PostMapping("/occupationlab/occupationlab/theoreticalCourseClassification/builtInCourseClassification")
    boolean builtInCourseClassification(@RequestParam("schoolId") Integer schoolId,@RequestParam("accountId") Integer accountId);


    @GetMapping("/occupationlab/occupationlab/projectManage/getProjectBySystemIdRemoteCall")
    List<ProjectManage>  getProjectBySystemIdRemoteCall(
            @RequestParam String systemId,
            @RequestParam Integer permissions,
            @RequestParam Integer cId,
            @RequestParam Integer accountId);



    /**
     * 根据班级id查询班级详情
     */
    @GetMapping("/occupationlab/occupationlab/class/queryClassDetails")
     boolean queryClassDetails(@RequestParam("id")  Integer id);

}
