package com.huoran.api;

import com.huoran.api.impl.UserClientImpl;
import com.huoran.common.constant.ServiceNameConstant;
import com.huoran.common.entity.Role;
import com.huoran.common.entity.UserRole;
import com.huoran.common.response.R;
import feign.Headers;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * <p>
 * 远程调用用户服务
 * </p>
 *
 * @author lr
 * @since 2021-08-24
 */
@FeignClient(value = ServiceNameConstant.USERS, fallbackFactory = UserClientImpl.class)
@Headers("Accept: application/json")
public interface UserClient {

    /**
     * 根据账号id获取当前账号id所在学校id
     */
    @Headers({"Content-Type: application/json", "Accept: application/json"})
    @GetMapping(value = "/users/users/userAccount/getSchoolIdByAccountId", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
    Integer getSchoolIdByAccountId(@RequestParam("accountId") String accountId);

    /**
     * 删除职站用户
     */
    @Headers({"Content-Type: application/json", "Accept: application/json"})
    @PostMapping(value = "/users/users/occupationlab/user/delStudent", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
    void delStudent(@RequestParam("accountIds") List<Integer> accountIds);

    /**
     * 删除用户、账户信息表
     */
    @Headers({"Content-Type: application/json", "Accept: application/json"})
    @PostMapping(value = "/users/users/occupationlab/user/delAccountAndUser", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
    void delAccountAndUser(@RequestParam("accountId") Integer accountId);

    /**
     * 账号赋予角色
     *
     * @param accountId
     * @param roleIds
     * @return
     */
    @Headers({"Content-Type: application/json", "Accept: application/json"})
    @PostMapping(value = "/users/users/user-role/rolePermissionService", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
    R rolePermissionService(@RequestParam("accountId") Integer accountId, @RequestParam("roleIds") List<Integer> roleIds,@RequestParam(value = "teamId",required = false) Integer teamId);

    /**
     * @Description :删除某用户下的某个角色
     * @Param roleId
     * @Param accountId
     * @Author Rong---2021/10/15
     */
    @Headers({"Content-Type: application/json", "Accept: application/json"})
    @PostMapping(value = "/users/users/role/delRoleByAccountId", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
    R delRoleByAccountId(@RequestParam("roleId") Integer roleId, @RequestParam("accountId") Integer accountId);

    /**
     * 为客户添加超管角色等相关操作
     *
     * @param schoolId
     * @param accountId
     * @return
     */
    @Headers({"Content-Type: application/json", "Accept: application/json"})
    @PostMapping("/users/users/role/addCustomerAdmin")
    boolean addCustomerAdmin(@RequestParam("schoolId") Integer schoolId, @RequestParam("accountId") String accountId);

    /**
     * @Description :   判断该角色是否存在
     * @Param roleName
     * @Param request
     * @Author Rong---2021/10/18
     */
    @Headers({"Content-Type: application/json", "Accept: application/json"})
    @PostMapping(value = "/users/users/role/checkRoleIsExist", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
    Role checkRoleIsExist(@RequestParam("roleName") String roleName, @RequestParam("schoolId") Integer schoolId,@RequestParam("platformId")Integer platformId);

    /**
     * @Description :   校验手机号是否存在
     * @Param phone
     * @Author Rong---2021/10/18
     */
    @Headers({"Content-Type: application/json", "Accept: application/json"})
    @GetMapping(value = "/users/users/userInfo/checkPhoneOrEmailExist", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
    boolean checkPhoneOrEmailExist(@RequestParam("phone") String phone, @RequestParam("email") String email, @RequestParam("type") Integer type);

    @Headers({"Content-Type: application/json", "Accept: application/json"})
    @PostMapping(value = "/users/users/role/checkIsSuperTube", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
    boolean checkIsSuperTube(@RequestParam("accountId") String accountId);

    /**
     * 根据平台获取该用户在该平台的角色
     *
     * @param accountId
     * @param platformId
     * @return
     */
    @Headers({"Content-Type: application/json", "Accept: application/json"})
    @PostMapping(value = "/users/users/role/getUserAllRole", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
    String getUserAllRole(@RequestParam("accountId") String accountId, @RequestParam("platformId") Integer platformId);

    /**
     * @Description :   根据账号id查询账户名称
     * @Param accountId
     * @Author Rong---2021/12/30
     */
    @GetMapping(value = "/users/users/userInfo/getUserName")
    String getUserName(@RequestParam("accountId") Integer accountId);


    @GetMapping(value = "/users/users/user/accountIsDisabled")
    boolean accountIsDisabled(@RequestParam("token") String token);


    @PostMapping(value = "/users/users/role/getAccountIdsBySchoolId")
    List<Integer> getAccountIdsBySchoolId( @RequestParam("schoolId") Integer schoolId,  @RequestParam("platformId") Integer platformId);

}
