package com.huoran.common.aop.aspect;

import com.huoran.common.aop.annotation.NoRepeatSubmit;
import com.huoran.common.exception.CustomException;
import com.huoran.common.response.R;
import com.huoran.common.utils.JwtUtils;
import lombok.Synchronized;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.util.concurrent.TimeUnit;

import static com.huoran.common.exception.ExceptionEnum.LOGIN_INVALID;


/**
 * @Description aop解析注解
 */

@Aspect
@Component
public class NoRepeatSubmitAop {

    private Log logger = LogFactory.getLog(getClass());

    @Autowired
    private RedisServiceAop redisService;

    @Synchronized
    @Around("execution(* com.huoran.*.controller.*.*(..)) && @annotation(noRepeatSubmit)")
    public Object around(ProceedingJoinPoint pjp, NoRepeatSubmit noRepeatSubmit) throws Throwable {

        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();
        String token = request.getHeader("token");
        //如果header中不存在token，则从参数中获取token
        if (StringUtils.isEmpty(token)) {
            throw new CustomException(LOGIN_INVALID);
        }
        //  校验并解析token，如果token过期或者篡改，则会返回null
        boolean isVerify = JwtUtils.checkToken(token);
        if (!isVerify) {
            throw new CustomException(LOGIN_INVALID);
        }
        String key = token + "-" + request.getServletPath();
        if ( !redisService.haskey(key) ) {// 如果缓存中有这个url视为重复提交
            Object o = pjp.proceed();
            redisService.setCacheObject(key, 0, 1, TimeUnit.SECONDS);
            return o;
        } else {
            redisService.setCacheObject(key, 0, 1, TimeUnit.SECONDS);//点了同样的URL继续限制，直到2次点击中间间隔超过了限制
            return R.error("请勿重复提交或者操作过于频繁！");
        }

    }


}
