package com.huoran.common.entity;

import com.baomidou.mybatisplus.annotation.*;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;

import java.io.Serializable;
import java.util.Date;

/**
 * <p>
 * python储存实验运行记录
 * </p>
 *
 * @author lr
 * @since 2021-09-06
 */
@Data
@EqualsAndHashCode(callSuper = false)
@Accessors(chain = true)
@TableName("python_run_code")
@ApiModel(value = "RunCode对象", description = "python储存实验运行记录")
public class RunCode implements Serializable {

    private static final long serialVersionUID = 1L;

    @ApiModelProperty(value = "实验代码主键Id", hidden = true)
    @TableId(value = "id", type = IdType.AUTO)
    private Integer id;

    @ApiModelProperty(value = "实验代码", required = true, example = "IMPORT random WORDS = [ \"Python\", \"jumble\", \"easy\", \"difficult\", \"answer\", \"continue\", \"phone\", \"position\", \"pose\", \"game\" ] print ( \"\"\"" +
            "欢迎参加猜单词游戏" +
            "\"\"\" )" +
            "WHILE" +
            "TRUE : # 从序列中随机挑出一个单词" +
            "word = random.choice ( WORDS ) # 判断玩家是否猜对的变量" +
            "correct = word # 保存乱序后的单词" +
            "jumble = \"\"" +
            "WHILE" +
            "word : # 根据 word 长度产生word的随机位置" +
            "position = random.randrange ( len ( word ) ) # 将position位置的字母组合乱序后单词" +
            "jumble += word [ position ] # 通过切片将position位置的字母从原有单词中删除" +
            "word = word [: position ] + word [ ( position + 1 ) :] print ( \"打乱顺序后的单词:\", jumble ) guess = 'q'" +
            "IF" +
            "guess != 'q' :" +
            "WHILE" +
            "guess != correct " +
            "AND guess != \"\" : print ( \"对不起，不正确!\" ) guess = EXIT ( \"validing:继续猜:\" )" +
            "IF" +
            "guess == correct : print ( \"恭喜，猜对了!\" ) elif guess == 'q' : print ( '再见!' ) break")
    private String code;

    @ApiModelProperty(value = "绑定的项目id", example = "1", required = true)
    private Integer projectId;

    @ApiModelProperty(value = "判分点主键Id", example = "1")
    private Integer judgmentPointsId;

    @ApiModelProperty(value = "提交状态:默认0未提交,1为已提交", example = "0", hidden = true)
    private Integer isSubmit;

    @ApiModelProperty(value = "运行结果", example = "欢迎参加猜单词游戏 打乱顺序后的单词: aersnw 再见!")
    private String runResult;

    @ApiModelProperty(value = "运行有没有出现错误(0、运行正确 1、运行错误)", hidden = true)
    private Integer isError;

    @ApiModelProperty(value = "绑定账号id", hidden = true)
    private Integer accountId;

    @ApiModelProperty(value = "逻辑删除—>默认0未删除,1为已删除", hidden = true)
    @TableLogic(value = "0", delval = "1")
    private Integer isDel;

    @ApiModelProperty(value = "运行时间", hidden = true)
    @TableField(fill = FieldFill.INSERT)
    private Date createTime;

    @ApiModelProperty(value = "修改时间", hidden = true)
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private Date updateTime;

    @ApiModelProperty(value = "实验报告id(ol_experimental_report)", hidden = true)
    private Integer reportId;

    @ApiModelProperty(value = "运行的图片")
    private String runPhotoUrl;

}
