package com.huoran.common.exception;

import com.google.common.collect.Maps;
import com.huoran.common.response.R;
import io.jsonwebtoken.ExpiredJwtException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import java.util.HashMap;
import java.util.concurrent.atomic.AtomicReference;

/**
 * @description 自定义异常控制器    用于自定义异常抛出后，@ControllerAdvice 默认拦截全部Controller
 *                               如果发生异常将异常信息返回(异常状态码,异常消息,异常发生时间)
 * @author: Mr.JK
 * @create: 2021-06-07 10:49
 **/
@Slf4j
@RestControllerAdvice
public class CustomExceptionHandler {

    /**
     * 拦截token异常- ExpiredJwtException
     */
   @ExceptionHandler(ExpiredJwtException.class)
   public R handleException(ExpiredJwtException e){
       log.error("token已过期");
       return R.error(e.getMessage());
   }

    @ExceptionHandler(CustomException.class)
    public R handleException(CustomException e){
        ExceptionEnum em = e.getExceptionEnum();
        return R.error(em.getCode(),em.getMsg());
    }

    /**
     * 处理参数格式校验异常
     */
    @ExceptionHandler(value = MethodArgumentNotValidException.class)
    public R handleValidException(MethodArgumentNotValidException e){
        log.error("参数格式校验异常");
        BindingResult bindingResult = e.getBindingResult();
        HashMap<String, String> errorMap = Maps.newHashMap();
        AtomicReference<String> errorMsg = new AtomicReference<>("");
        bindingResult.getFieldErrors().forEach(fieldError -> {
            errorMsg.set(fieldError.getDefaultMessage());
            errorMap.put(fieldError.getField(),fieldError.getDefaultMessage());
        });
        return R.error(ExceptionEnum.VALID_EXCEPTION.getCode(), errorMsg).put("data",errorMap);
    }

}
