package com.huoran.common.exception;

import lombok.*;

/**
 * @description 异常枚举类
 * @author: Mr.JK
 * @create: 2021-06-07 10:30
 * 异常代码枚举
 * 异常代码列表
 * 10 中台主项目
 * 001 参数格式校验
 * 002 密码无改变
 * 003 账号已存在
 * 004 验证码已过期
 * 005 验证码错误
 * 006 密码错误
 * 007 账号禁用
 * 008 账号不存在
 * 009 客户已存在
 * 010 客户无订单
 * 20 数据平台
 * 001
 * 30 用户管理
 * 001 手机号未绑定
 **/
@Getter
@ToString
@NoArgsConstructor
@AllArgsConstructor
public enum ExceptionEnum {


    UNKNOWN_EXCEPTION(10000, "未知异常，请联系管理员解决！"),
    VALID_EXCEPTION(10001, "参数格式校验异常！"),
    NO_CHANGE_EXCEPTION(10002, "密码无改变，更新失败！"),
    EXIST_ACCOUNT(10003, "账号已存在，请重新设置！"),
    CODE_EXPIRED(10004, "验证码已过期"),
    CODE_ERROR(10005, "验证码错误"),
    WRONG_PASSWORD(10006, "密码错误"),
    ACCOUNT_DISABLED(10007, "账号已禁用"),
    ACCOUNT_NOT_EXIST(10008, "账号不存在"),
    NOT_AUTHORITY(10008, "无该平台登录权限！"),
    PHONE_NOT_EXIST(10008, "手机号不存在"),
    MAIL_NOT_EXIST(10008, "邮箱不存在"),
    NOT_ALLOWED(10008, "超管只支持账号进行登录！"),
    EXIST_CUSTOMER(10009, "客户已存在！"),
    NO_ORDER(10010, "该账号下没有订单，请下订单后进行登录！"),
    NO_SHIPPED(10011, "订单还未发货，请发货后进行登录！"),
    NO_ENABLE(10012, "订单还未启用，请启用后进行登录！"),
    MATURITY(10013, "订单到期，请联系管理员续费后进行登录！"),
    USER_NOT_EXIST(10014, "用户不存在"),
    ACCOUNT_AVALIABLE(10015, "可使用账号数量不足，无法启用账号！"),
    SERVICE_CALL_FAILED(10016, "服务调用失败，请稍后再试！"),
    TOO_MANY_REQUEST(10017, "请求流量过大，请稍后再试！"),
    SERVICE_FAIL(10018, "服务调用失败，请稍后再试！"),
    REPEAT_ORDERS(10019, "该时间段与现有订单有效期重叠，请勿重复下单！"),
    STUDENT_ID_NOT_EXIST(10020, "学号不存在"),
    JOB_ID_NOT_EXIST(10021, "工号不存在"),
    DATA_PLATFORM_ACCOUNT_AVALIABLE_FAIL(10022, "当前用户所在的院校在数据平台的可用账号数量不足，是否继续！"),
    INVITATION_CODE_EXPIRED(10023, "邀请码已失效"),
    USER_ALREADY_EXISTS_GROUP(10024, "用户已存在该组织"),

    USER_ALREADY_EXISTS_OTHER_GROUP(10024, "无法受邀加入该平台组织，账号已在其他平台存在组织信息！"),
    CANT_JOIN_TEAM(10025, "无法加入自己的团队"),
    CANNOT_ADDED_REPEATEDLY(10026, "该用户已经是团队管理员，无法重复添加！"),
    DEFAULT_ROLE_NOTEXIST(10027, "默认角色不存在"),
    TEAM_NOT_EXIST(10028, "团队不存在"),
    PARTNER_REMOVAL(10029, "该用户是组织超管，请转让超管后再移除！"),
    SUPERVISING_ROLE(10030, "该用户存在超管角色，请转移后进行删除！"),
    VERIFICATION_CODE_HAS_BEEN_SENT(10031, "验证码已发，请勿重复操作！"),
    LOGIN_INVALID(401, "登录已失效,请重新登录"),
    ASSOCIATED_PRODUCT(10031, "已关联产品，请在商城管理下架对应产品后再删除!"),

    COURSE_PRODUCTS_HAVE_BEEN_REMOVED(10032, "保存失败，关联课程产品已删除，请更换关联产品!"),
    COURSE_PRODUCTS_HAVE_BEEN_REMOVED_FROM_SALE(10033, "保存失败，关联课程产品已下架，请更换关联产品!"),
    DATA_PRODUCTS_HAVE_BEEN_REMOVED(10032, "保存失败，关联数据产品已删除，请更换关联产品!"),
    DATA_PRODUCTS_HAVE_BEEN_REMOVED_FROM_SALE(10033, "保存失败，关联数据产品已下架，请更换关联产品!"),

    CLASSIFICATION_HAS_NO_MODEL_DATA(10034,"当前选择的分类下的没有满足条件的模型数据可供导入！"),
    REDIS_LIMIT(10035,"当前人数较多，请稍后再试！"),

    //数据平台
    DATA_NOT_FOUND(20001, "数据不存在！"),
    INVALID_PARAM(20002, "参数错误！"),
    TABLE_NO_FOUND(20003, "该表不存在！"),
    TABLE_DOWNLOAD_FAIL(20004, "表下载量更新失败！"),
    SELECT_TOTAL_IS_ZERO(20005, "查询总数为零！"),
    QUERY_TABLE_DATA_FAIL(20006, "查询表数据失败！"),
    TABLE_CLICK_FAIL(20007, "表点击量更新失败！"),
    SYMBOL_OF_ILLEGAL(20008, "符号非法！"),

    //用户管理
    NO_BIND_PHONE(30001, "未绑定手机号，请绑定后再登录！"),

    //职站
    QUESTION_NUM_INVALID(40001, "测评题目数量设置超出范围！"),
    QUESTION_EXISTS(40002, "此题已存在！"),
    QUESTION_TYPE_INVALID(40003, "题型错误！"),
    EXCEL_INVALID(40004, "excel表内容错误！"),
    EXCEL_FILE_INVALID(40005, "上传excel文件错误！"),
    EXCEL_FILE_NULL(40006, "导入失败，导入数据为空！"),
    EXCEL_FILE_FORMAT_ERROR(40007, "请根据模板使用说明录入正确的题库信息！"),
    ASSESSMENT_ENABLE_CODE_NULL(40008, "请输入正确的邀请码！"),
    ADD_COLUMNS_FAILED(40009, "栏目最多可添加10个！"),
    INSERT_PROJECT_MANAGE_FAIL(40010, "新增项目管理失败！"),
    UPDATE_PROJECT_MANAGE_FAIL(40010, "更新项目管理失败！"),
    SAVE_ASSESSMENT_FAIL(40011, "新增考核失败！"),
    UPDATE_ASSESSMENT_FAIL(40012, "修改考核失败！"),
    ENTER_EXAM_FAIL(40013, "进入考试失败！"),
    DELETE_JUDGMENT_FAIL(40014, "删除判分点失败"),
    EMPTY_SCHOOL(40015,"请选择所属学校"),
    EMPTY_ACCOUNT(40015,"账号不能为空"),
    EMPTY_WORK_NUMBER(40015,"学号/工号不能为空"),
    EMPTY_PARAMETER(40016,"登录参数不完整！"),

    TEAM_LIMIT1(40017,"报名失败,超过团队人数限制！"),
    TEAM_LIMIT2(40018,"报名失败,超过赛事团队数上限！"),
    TEAM_LIMIT3(40019,"报名失败,超过赛事人数上限！"),

    CODE(40019,"报名失败,报名邀请码错误！"),
    HAVE_BEEN_REGISTERED(40020,"你已经报过名啦！"),
    UNABLE_TO_REGISTER(40021,"你所在院校不符合参赛要求，无法报名！"),
    ENTER_CORRECT_EMAIL(40022,"请输入正确邮箱！"),
    THE_REMOTE_FILE_TYPE_IS_UNRECOGNIZED(40023,"远程文件类型无法识别！"),

    REFRESH_THE_LIST(40024,"当前项目状态已被更改，即将刷新列表!"),

    DELETE_REFRESH_LIST(40025,"当前项目已被删除，请刷新列表!"),


    ACTIVITY_REGISTRATION_FAILURE(40026,"报名失败,超过项目人数上限！"),

    //判分点
    DELETE_JUDGMENT_POINT_FAIL(50001, "删除判分点失败！"),
    DELETE_JUDGMENT_POINT_RULE_FAIL(50002, "删除判分点规则失败！"),
    ADD_JUDGMENT_POINT_FAIL(50003, "添加判分点失败！"),
    ADD_JUDGMENT_POINT_RULE_FAIL(50004, "添加判分点规则失败！"),
    SUBJECT_TYPE_IS_NULL(50005, "题目类型为空！"),
    UPDATE_JUDGMENT_POINT_FAIL(50006, "判分点更新失败！"),
    UPDATE_JUDGMENT_POINT_RULE_FAIL(50007, "判分点规则更新失败！"),

    //python
    RUN_TIMEOUT(60001, "当前代码运行超时，请重新再试！！"),
    //银行
    ADD_EXPERIMENTAL_REPORT_FAIL(70001, "添加实验报告失败！"),

    REPEAT_SUBMISSION(70004, "重复提交考核！"),
    ADD_USER_SCORE_FAIL(70002, "添加用户分数失败！"),
    SELECT_JUDGMENT_SCORE_FAIL(70003, "查询判分点分数失败！"),
    FALSE_CODE(70005, "非法代码，请检查是否调用系统命令！"),


    ;

    private int code;
    private String msg;
}
