package com.huoran.common.response;

import com.huoran.common.exception.ExceptionEnum;
import org.apache.http.HttpStatus;
import org.springframework.lang.Nullable;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Author chen
 * @DATE 2020/9/30 11:18
 * @Version 1.0
 */
public class R<T> extends HashMap<String, Object> {

    private static final long serialVersionUID = 1L;

    public R() {
        put("status", 200);
        put("message", "success");
    }

    public static R error() {
        return error(HttpStatus.SC_INTERNAL_SERVER_ERROR, "未知异常，请联系管理员");
    }

    public static R error(String msg) {
        return error(HttpStatus.SC_INTERNAL_SERVER_ERROR, msg);
    }

    public static R error(int code, String msg) {
        R r = new R();
        r.put("status", code);
        r.put("message", msg);
        return r;
    }

    @Deprecated
    public static <T> R<T> error(int code, @Nullable T msg) {
        R r = new R();
        r.put("status", code);
        r.put("message", msg);
        return r;
    }

    public static R ok(String msg) {
        R r = new R();
        r.put("message", msg);
        return r;
    }

    public static R ok(Map<String, Object> map) {
        R r = new R();
        r.putAll(map);
        return r;
    }

    public static R ok(List list) {
        R r = new R();
        r.put("list",list);
        return r;
    }

    public static R ok() {
        return new R();
    }

    @Deprecated
    public static <T> R ok(@Nullable T msg) {
        R r = new R();
        r.put("message", msg);
        return r;
    }

    public static R error(ExceptionEnum exceptionEnum) {
        R r = new R();
        r.put("status",exceptionEnum.getCode());
        r.put("message",exceptionEnum.getMsg());
        return r;
    }

    @Override
    public R put(String key, Object value) {
        super.put(key, value);
        return this;
    }

    public static R toR(int rows){
        return rows > 0 ? R.ok() : R.error();
    }

    public static R toR(boolean result){
        return result ? R.ok() : R.error();
    }
}
