package com.huoran.common.utils;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.util.IdUtil;
import com.aliyun.oss.ClientException;
import com.aliyun.oss.OSS;
import com.aliyun.oss.OSSClientBuilder;
import com.aliyun.oss.OSSException;
import com.aliyun.oss.model.*;

import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.vod.model.v20170321.DeleteVideoRequest;
import com.aliyuncs.vod.model.v20170321.DeleteVideoResponse;
import com.aliyuncs.vod.model.v20170321.GetVideoPlayAuthRequest;
import com.aliyuncs.vod.model.v20170321.GetVideoPlayAuthResponse;
import com.huoran.common.entity.FilesResult;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.util.List;

public class AliyunOssUtil {

    /**
     * 视频上传
     * @param file 视频文件
     * @return 返回文件信息
     * @throws IOException IO异常
     */
    /*public static FilesResult vodUpload(MultipartFile file) throws IOException {
        FilesResult filesResult = new FilesResult();
        //获取文件流
        InputStream inputStream = file.getInputStream();
        String originalFilename = file.getOriginalFilename();
        assert originalFilename != null;
        String title = originalFilename.substring(0, originalFilename.lastIndexOf("."));
        // 获取文件尾缀
        String fileType = originalFilename.substring(originalFilename.lastIndexOf(".")+1);
        // 文件类型
        filesResult.setFileType(fileType);

        //上传流请求
        UploadStreamRequest request = new UploadStreamRequest(ConstantPropertiesUtil.ACCESS_KEY_ID,
                ConstantPropertiesUtil.ACCESS_KEY_SECRET, title, originalFilename, inputStream);

        *//* 是否使用默认水印(可选)，指定模板组ID时，根据模板组配置确定是否使用默认水印*//*
        //request.setShowWaterMark(true);
        *//* 设置上传完成后的回调URL(可选)，建议通过点播控制台配置消息监听事件，参见文档 https://help.aliyun.com/document_detail/57029.html *//*
        //request.setCallback("http://callback.sample.com");
        *//* 自定义消息回调设置，参数说明参考文档 https://help.aliyun.com/document_detail/86952.html#UserData *//*
        //request.setUserData(""{\"Extend\":{\"test\":\"www\",\"localId\":\"xxxx\"},\"MessageCallback\":{\"CallbackURL\":\"http://test.test.com\"}}"");
        *//* 视频分类ID(可选) *//*
        //request.setCateId(0);
        *//* 视频标签,多个用逗号分隔(可选) *//*
        //request.setTags("标签1,标签2");
        *//* 视频描述(可选) *//*
        //request.setDescription("视频描述");
        *//* 封面图片(可选) *//*
        //request.setCoverURL("http://cover.sample.com/sample.jpg");
        *//* 模板组ID(可选) *//*
        //request.setTemplateGroupId("8c4792cbc8694e7084fd5330e56a33d");
        *//* 工作流ID(可选) *//*
        //request.setWorkflowId("d4430d07361f0*be1339577859b0177b");
        *//* 存储区域(可选) *//*
        //request.setStorageLocation("in-201703232118266-5sejdln9o.oss-cn-shanghai.aliyuncs.com");
        *//* 开启默认上传进度回调 *//*
        // request.setPrintProgress(true);
        *//* 设置自定义上传进度回调 (必须继承 VoDProgressListener) *//*
        // request.setProgressListener(new PutObjectProgressListener());
        *//* 设置应用ID*//*
        //request.setAppId("app-1000000");
        *//* 点播服务接入点 *//*
        //request.setApiRegionId("cn-shanghai");
        *//* ECS部署区域*//*
        // request.setEcsRegionId("cn-shanghai");

        UploadVideoImpl uploader = new UploadVideoImpl();
        UploadStreamResponse response = uploader.uploadStream(request);

        //如果设置回调URL无效，不影响视频上传，可以返回VideoId同时会返回错误码。
        // 其他情况上传失败时，VideoId为空，此时需要根据返回错误码分析具体错误原因
        String videoId = response.getVideoId();
        if (!response.isSuccess()) {
            String errorMessage = "阿里云视频上传错误：" + "code：" + response.getCode() + ", message：" + response.getMessage();
            if(StringUtils.isEmpty(videoId)){
                throw new CustomException(ExceptionEnum.UNKNOWN_EXCEPTION);
            }
        }
        filesResult.setFileId(videoId);
        return filesResult;
    }*/

    /**
     * 根据视频id删除视频
     *
     * @param videoId 视频id
     */
    public static void removeVideo(String videoId) throws com.aliyuncs.exceptions.ClientException {

        DefaultAcsClient client = AliyunVodSDKUtil.initVodClient(
                ConstantPropertiesUtil.ACCESS_KEY_ID,
                ConstantPropertiesUtil.ACCESS_KEY_SECRET);

        DeleteVideoRequest request = new DeleteVideoRequest();
        request.setVideoIds(videoId);
        DeleteVideoResponse response = client.getAcsResponse(request);
        System.out.print("RequestId = " + response.getRequestId() + "\n");
    }

    /**
     * 根据视频id批量删除视频
     *
     * @param videoIdList 视频id
     */
    public static void removeMoreVideo(List<String> videoIdList) throws com.aliyuncs.exceptions.ClientException {
        DefaultAcsClient client = AliyunVodSDKUtil.initVodClient(
                ConstantPropertiesUtil.ACCESS_KEY_ID,
                ConstantPropertiesUtil.ACCESS_KEY_SECRET);
        DeleteVideoRequest request = new DeleteVideoRequest();
        //使用工具类将集合转化成指定的字符串
        String videos = StringUtils.join(videoIdList, ",");
        request.setVideoIds(videos);
        DeleteVideoResponse response = client.getAcsResponse(request);
        System.out.print("RequestId = " + response.getRequestId() + "\n");
    }

    /**
     * 根据视频id获取播放凭证
     *
     * @param videoId 视频id
     * @return 返回凭证
     */
    public static String getPlayAuth(String videoId) throws com.aliyuncs.exceptions.ClientException {

        DefaultAcsClient client = AliyunVodSDKUtil.initVodClient(ConstantPropertiesUtil.ACCESS_KEY_ID, ConstantPropertiesUtil.ACCESS_KEY_SECRET);

        GetVideoPlayAuthRequest request = new GetVideoPlayAuthRequest();

        request.setVideoId(videoId);

        GetVideoPlayAuthResponse response = client.getAcsResponse(request);
        //得到播放凭证并返回
        return response.getPlayAuth();
    }

    /**
     * 创建存储空间
     *
     * @param ossClient oss客户端
     */
    public static void createBucket(OSS ossClient) {
        // 判断存储空间是否存在，不存在，则创建
        if (!ossClient.doesBucketExist(ConstantPropertiesUtil.BUCKET_NAME)) {
            // 创建CreateBucketRequest对象,设置仓库名称
            CreateBucketRequest bucketRequest = new CreateBucketRequest(ConstantPropertiesUtil.BUCKET_NAME);
            // 设置存储空间的权限为公共读，默认是私有。
            bucketRequest.setCannedACL(CannedAccessControlList.PublicRead);
            // 此处以设置存储空间的存储类型为标准存储为例。
            // createBucketRequest.setStorageClass(StorageClass.Standard);
            // 默认情况下，数据容灾类型为本地冗余存储，即DataRedundancyType.LRS。如果需要设置数据容灾类型为同城冗余存储，请替换为DataRedundancyType.ZRS。
            // createBucketRequest.setDataRedundancyType(DataRedundancyType.ZRS)
            // 创建存储空间。
            ossClient.createBucket(bucketRequest);

        }
    }

    public static String getFormatSize(double size) {
        double kiloByte = size / 1024;
        if (kiloByte < 1) {
            return size + "Byte(s)";
        }

        double megaByte = kiloByte / 1024;
        if (megaByte < 1) {
            BigDecimal result1 = new BigDecimal(Double.toString(kiloByte));
            return result1.setScale(2, BigDecimal.ROUND_HALF_UP).toPlainString() + "KB";
        }

        double gigaByte = megaByte / 1024;
        if (gigaByte < 1) {
            BigDecimal result2 = new BigDecimal(Double.toString(megaByte));
            return result2.setScale(2, BigDecimal.ROUND_HALF_UP).toPlainString() + "MB";
        }

        double teraBytes = gigaByte / 1024;
        if (teraBytes < 1) {
            BigDecimal result3 = new BigDecimal(Double.toString(gigaByte));
            return result3.setScale(2, BigDecimal.ROUND_HALF_UP).toPlainString() + "GB";
        }
        BigDecimal result4 = new BigDecimal(teraBytes);
        return result4.setScale(2, BigDecimal.ROUND_HALF_UP).toPlainString() + "TB";
    }

    public static void main(String[] args) {
        System.out.println(getFormatSize(68160));
    }

    /**
     * 上传文件
     *
     * @param file 文件
     * @return 返回文件上传结果
     * @throws OSSException    oss异常
     * @throws ClientException 客户端异常
     * @throws IOException     io异常
     */
    public static FilesResult uploadFile(MultipartFile file) throws OSSException, ClientException, IOException {

        //获取访问key
        String endPoint = ConstantPropertiesUtil.END_POINT;
        String accessKeyId = ConstantPropertiesUtil.ACCESS_KEY_ID;
        String accessKeySecret = ConstantPropertiesUtil.ACCESS_KEY_SECRET;
        //创建ossClient
        OSS ossClient = new OSSClientBuilder().build(endPoint, accessKeyId, accessKeySecret);

        FilesResult result = new FilesResult();
        // 创建存储空间
        createBucket(ossClient);

        //获取上传文件流
        InputStream inputStream = file.getInputStream();
        //获取当前日期构建日期路径：avatar/2019/02/26/文件名
        String currentDate = new DateTime().toString("yyyyMMdd");
        //获取上传的文件名称
        String fileName = file.getOriginalFilename();
        result.setOriginalFileName(fileName);//原文件名

        // 获取文件尾缀
        assert fileName != null;
        String ext = fileName.substring(fileName.lastIndexOf("."));
        // 获取文件类型，大小
        String fileType = getFileType(ext);
        long fileSize = file.getSize();
        //获取随机值
        String snowflakeId = IdUtil.getSnowflake(6, 4).nextIdStr();
        //文件名：雪花算法.文件扩展名
        String newName = snowflakeId + ext;
        String sort = ext.substring(ext.lastIndexOf(".") + 1);
        result.setFileType(sort);
        result.setFileSize(String.valueOf(fileSize));
        //组合新的文件名
        String fileUrl = currentDate + "/" + sort + "/" + newName;
        result.setOssFileName(fileUrl);
        // 设置上传时存储类型与访问权限
        ObjectMetadata metadata = new ObjectMetadata();
        // 指定该Object被下载时的网页的缓存行为
        metadata.setCacheControl("no-cache");
        // 指定该Object下设置Header
        metadata.setHeader("Pragma", "no-cache");
        // 指定该Object被下载时的内容编码格式
        metadata.setContentEncoding("utf-8");
        // 文件的MIME，定义文件的类型及网页编码，决定浏览器将以什么形式、什么编码读取文件。如果用户没有指定则根据Key或文件名的扩展名生成，
        // 如果没有扩展名则填默认值application/octet-stream
        metadata.setContentType(fileType);
        // 指定该Object被下载时的名称（指示MINME用户代理如何显示附加的文件，打开或下载，及文件名称）
        metadata.setContentDisposition("filename/filesize=" + file.getName() + "/" + fileSize + "Byte.");

        //上传文件 (上传文件流的形式)
        ossClient.putObject(ConstantPropertiesUtil.BUCKET_NAME, fileUrl, inputStream, metadata);
        // 关闭OSSClient。
        ossClient.shutdown();

        // 拼接文件url地址，返回控制层
        String tempEndpoint = ConstantPropertiesUtil.END_POINT;
        String endpoint = tempEndpoint.substring(tempEndpoint.lastIndexOf("/") + 1, tempEndpoint.length());
        String url = "https://" + ConstantPropertiesUtil.BUCKET_NAME + "." + endpoint + "/" + fileUrl;

        result.setFileUrl(url);

        // 关闭OSSClient。
        ossClient.shutdown();

        return result;
    }

    /**
     * 文件下载
     *
     * @param response   响应对象
     * @param ossClient  oss客户端
     * @param objectName 文件名
     */
    public static void downloadFiles(HttpServletResponse response, OSS ossClient,
                                     String objectName) throws IOException {

        // ossObject包含文件所在的存储空间名称、文件名称、文件元信息以及一个输入流。
        OSSObject ossObject = ossClient.getObject(ConstantPropertiesUtil.BUCKET_NAME, objectName);
        InputStream input = null;
        OutputStream outputStream = null;
        try {
            // 获取文件名
            String fileName = objectName.substring(objectName.lastIndexOf("/") + 1);
            //获取输入流
            input = ossObject.getObjectContent();
            // 获取OutputStream输出流
            outputStream = response.getOutputStream();
            byte[] buffer = new byte[input.available()];

            response.setHeader("content-Type", "application/octet-stream");
            response.setHeader("Content-disposition", "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8"));
            response.flushBuffer();

            for (int length = 0; (length = input.read(buffer)) > 0; ) {
                outputStream.write(buffer, 0, length);
            }

        } catch (IOException e) {
            e.printStackTrace();
            throw e;
        } finally {
            if (null != outputStream) {
                try {
                    outputStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (null != input) {
                try {
                    input.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            // 关闭OSSClient。
            ossClient.shutdown();
        }

    }

    /**
     * *** 谨慎操作 ***
     * 删除存储空间
     *
     * @param bucketName 存储空间名称
     */
    public static void deleteBucket(String bucketName) {
        String endPoint = ConstantPropertiesUtil.END_POINT;
        String accessKeyId = ConstantPropertiesUtil.ACCESS_KEY_ID;
        String accessKeySecret = ConstantPropertiesUtil.ACCESS_KEY_SECRET;
        // 创建OSSClient实例。
        OSS ossClient = new OSSClientBuilder().build(endPoint, accessKeyId, accessKeySecret);
        // 删除存储空间。
        ossClient.deleteBucket(bucketName);
        // 关闭OSSClient。
        ossClient.shutdown();
    }

    /**
     * 创建文件夹
     *
     * @param ossClient  oss连接
     * @param bucketName 存储空间
     * @param folder     文件夹名，如 "qj_nanjing/"
     * @return 返回创建的文件夹名
     */
    public static String createFolder(OSS ossClient, String bucketName, String folder) {
        // 判断文件夹是否存在，不存在则创建
        if (!ossClient.doesObjectExist(bucketName, folder)) {
            // 创建文件夹
            ossClient.putObject(bucketName, folder, new ByteArrayInputStream(new byte[0]));
            // 得到文件夹名
            OSSObject object = ossClient.getObject(bucketName, folder);
            return object.getKey();
        }
        return folder;
    }

    /**
     * 批量删除文件。key等同于ObjectName，表示删除OSS文件时需要指定包含文件后缀在内的完整路径，例如abc/efg/123.jpg。
     *
     * @param keys 文件完整路径
     */
    public static List<String> deleteFiles(List<String> keys) {
        String endPoint = ConstantPropertiesUtil.END_POINT;
        String accessKeyId = ConstantPropertiesUtil.ACCESS_KEY_ID;
        String accessKeySecret = ConstantPropertiesUtil.ACCESS_KEY_SECRET;
        String bucketName = ConstantPropertiesUtil.BUCKET_NAME;
        // 创建OSSClient实例。
        OSS ossClient = new OSSClientBuilder().build(endPoint, accessKeyId, accessKeySecret);

        DeleteObjectsResult deleteObjectsResult = ossClient.deleteObjects(new DeleteObjectsRequest(bucketName).withKeys(keys));
        List<String> deletedObjects = deleteObjectsResult.getDeletedObjects();
        // 关闭OSSClient。
        ossClient.shutdown();
        //返回删除成功文件列表
        return deletedObjects;
    }

    /**
     * 获取文件类型
     *
     * @param fileExtension 文件后缀
     * @return 返回类型
     */
    public static String getFileType(String fileExtension) {
        if (".bmp".equalsIgnoreCase(fileExtension)) {
            return "image/bmp";
        }
        if (".gif".equalsIgnoreCase(fileExtension)) {
            return "image/gif";
        }
        if (".jpeg".equalsIgnoreCase(fileExtension) || ".jpg".equalsIgnoreCase(fileExtension)
                || ".png".equalsIgnoreCase(fileExtension)) {
            return "image/jpg";
        }
        if (".html".equalsIgnoreCase(fileExtension)) {
            return "text/html";
        }
        if (".txt".equalsIgnoreCase(fileExtension)) {
            return "text/plain";
        }
        if (".vsd".equalsIgnoreCase(fileExtension)) {
            return "application/vnd.visio";
        }
        if (".ppt".equalsIgnoreCase(fileExtension) || ".pptx".equalsIgnoreCase(fileExtension)) {
            return "application/vnd.ms-powerpoint";
        }
        if (".doc".equalsIgnoreCase(fileExtension) || ".docx".equalsIgnoreCase(fileExtension)) {
            return "application/msword";
        }
        if (".xml".equalsIgnoreCase(fileExtension)) {
            return "text/xml";
        }
        if (".mp4".equalsIgnoreCase(fileExtension) || ".avi".equalsIgnoreCase(fileExtension)) {
            return "video/mp4";
        }
        if (".mpg".equalsIgnoreCase(fileExtension) || ".mpeg".equalsIgnoreCase(fileExtension)) {
            return "video/mpeg";
        }
        if ("pdf".equalsIgnoreCase(fileExtension)) {
            return "application/pdf";
        }
        return "application/octet-stream";
    }

}
