package com.huoran.competition.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huoran.common.aop.annotation.NoRepeatSubmit;
import com.huoran.common.constant.DelConstant;
import com.huoran.common.response.R;
import com.huoran.common.utils.TokenUtils;
import com.huoran.competition.entity.CompetitionAnnouncement;
import com.huoran.competition.entity.CompetitionAnnouncementAnnex;
import com.huoran.competition.service.CompetitionAnnouncementAnnexService;
import com.huoran.competition.service.CompetitionAnnouncementService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * <p>
 * 活动公告表 前端控制器
 * </p>
 *
 * @author cheney
 * @since 2022-11-16
 */
@Api(tags = "kindergarten-活动公告管理")
@RestController
@RequestMapping("/competition/announcement")
public class CompetitionAnnouncementController {

    @Autowired
    public CompetitionAnnouncementService service;//公告

    @Autowired
    public CompetitionAnnouncementAnnexService annexService;//活动公告附件


    @PostMapping("/queryAnnouncementDetails")
    @ApiOperation(value = "查询公告详情",response = CompetitionAnnouncement.class)
    public R findById(
            @RequestParam("id") @ApiParam(value = "公告主键") Integer id) {
        CompetitionAnnouncement competitionAnnouncement = service.getById(id);
        QueryWrapper<CompetitionAnnouncementAnnex> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("announcement_id",id);
        queryWrapper.eq("is_del", DelConstant.NOT_DEL);
        List<CompetitionAnnouncementAnnex> annexList = annexService.list(queryWrapper);
        if (annexList.size()>0){
            competitionAnnouncement.setAnnouncementAnnexList(annexList);
        }
        return R.ok().put("data", competitionAnnouncement);
    }

    @NoRepeatSubmit
    @PostMapping("/addAnnouncement")
    @ApiOperation(value = "新增公告",response = CompetitionAnnouncement.class)
    public R save(@RequestBody CompetitionAnnouncement competitionAnnouncement, HttpServletRequest request) {
        String accountId = TokenUtils.getIdByJwtToken(request);
        competitionAnnouncement.setAccountId(Integer.valueOf(accountId));
        boolean save = service.save(competitionAnnouncement);
        if (competitionAnnouncement.getAnnouncementAnnexList().size() > 0) {
            competitionAnnouncement.getAnnouncementAnnexList().forEach(competitionAnnex -> {
                competitionAnnex.setAnnouncementId(competitionAnnouncement.getId());
                boolean saveCompetitionAnnex = annexService.save(competitionAnnex);//保存活动附件
            });
        }
        return save ? R.ok() : R.error();
    }

    @NoRepeatSubmit
    @PostMapping("/amendmentAnnouncement")
    @ApiOperation(value = "修改公告",response = CompetitionAnnouncement.class)
    public R update(@RequestBody CompetitionAnnouncement competitionAnnouncement) {
        boolean update = service.updateById(competitionAnnouncement);
        return update ? R.ok() : R.error();
    }

    @NoRepeatSubmit
    @ApiOperation(value = "是否禁用公告")
    @PutMapping("/disableAnnouncement")
    public R disableAnnouncement(@ApiParam(name = "id", value = "公告主键", required = true) @RequestParam Integer id,
                                 @ApiParam(name = "isDisable", value = "是否开启(0开启 1未开启 默认0)", required = true)@RequestParam Integer isDisable) {
        if (isDisable==1 || isDisable==0) {
            CompetitionAnnouncement CompetitionAnnouncement = new CompetitionAnnouncement();
            CompetitionAnnouncement.setId(id);
            CompetitionAnnouncement.setIsOpen(isDisable);
            boolean update = service.updateById(CompetitionAnnouncement);
            return update ? R.ok() : R.error();
        } else {
            return R.error("启用状态错误");
        }
    }
    @NoRepeatSubmit
    @PostMapping("/deleteAnnouncement")
    @ApiOperation(value = "删除公告",response = CompetitionAnnouncement.class)
    public R delete(@RequestParam("id") @ApiParam(value = "公告主键") Integer id) {
        boolean del = service.removeById(id);
        return del ? R.ok() : R.error();
    }


    @PostMapping("/queryAnnouncementByCompetitionId")
    @ApiOperation(value = "根据活动id查询公告列表(分页)",response = CompetitionAnnouncement.class)
    public R queryAnnouncementByCompetitionId(
            @RequestParam("competitionId") @ApiParam(value = "活动id") String competitionId,
            @ApiParam(value = "当前页数", required = true) @RequestParam("pageNum") Integer pageNum,
            @ApiParam(value = "当前页需要显示的数量", required = true) @RequestParam("pageSize") Integer pageSize) {
        return service.queryAnnouncementByCompetitionId(competitionId, pageNum, pageSize);
    }
}

