package com.huoran.competition.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.huoran.api.NakadaiClient;
import com.huoran.api.UserClient;
import com.huoran.common.aop.annotation.NoRepeatSubmit;
import com.huoran.common.response.R;
import com.huoran.common.utils.TokenUtils;
import com.huoran.competition.entity.*;
import com.huoran.competition.entity.req.AfterLoginPageCompetitionListReq;
import com.huoran.competition.entity.req.DisableEnableCompetitionReq;
import com.huoran.competition.entity.req.PageCompetitionListReq;
import com.huoran.competition.entity.vo.CompetitionVO;
import com.huoran.competition.service.*;
import com.huoran.competition.service.impl.CompetitionContentServiceImpl;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.List;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;

/**
 * <p>
 * 活动信息表 前端控制器
 * </p>
 *
 * @author cheney
 * @since 2022-11-16
 */
@Api(tags = "kindergarten-活动信息管理")
@RestController
@RequestMapping("/competition/management")
public class CompetitionController {
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private UserClient userClient;


    @Autowired
    public CompetitionClassificationService competitionClassificationService;
    @Autowired
    private CompetitionService competitionService;

    @Autowired
    private CompetitionRegistrationService competitionRegistrationService;

    @Autowired
    private CompetitionRangeService competitionRangeService;

    @Autowired
    private CompetitionAnnexService competitionAnnexService;

    @Autowired
    private CompetitionTeamService competitionTeamService;

    @Autowired
    private CompetitionRuleService competitionRuleService;

    @Autowired
    private CompleteCompetitionSetupService completeCompetitionSetupService;

    @Autowired
    private CompetitionStageContentSettingService contentSettingService;

    @Autowired
    private CompetitionReleaseTimeService competitionReleaseTimeService;

    @Autowired
    CompetitionContentServiceImpl competitionContentService;


    @Autowired
    CompetitionStageFileService stageFileService;

    @Autowired
    private NakadaiClient nakadaiClient;


    @NoRepeatSubmit
    @ApiOperation(value = "创建活动")
    @PostMapping("/addCompetition")
    public R addCompetition(@ApiParam(name = "Competition", value = "活动对象", required = true) @RequestBody Competition competition, HttpServletRequest request) {
        String accountId = TokenUtils.getIdByJwtToken(request);
        Integer schoolId = userClient.getSchoolIdByAccountId(accountId);
        competition.setFounderId(Integer.valueOf(accountId));
        competition.setFounderName(userClient.getUserName(Integer.valueOf(accountId)));
        return competitionService.addCompetition(competition);
    }


    @NoRepeatSubmit
    @ApiOperation(value = "修改活动")
    @PostMapping("/editCompetition")
    public R editCompetition(@ApiParam(name = "competition", value = "活动对象", required = true) @RequestBody Competition competition) {

        QueryWrapper<CompetitionAnnex> annexUpdateWrapper = new QueryWrapper<>();
        annexUpdateWrapper.eq("competition_id", competition.getId());
        competitionAnnexService.remove(annexUpdateWrapper);

        if (competition.getCompetitionAnnexList().size() > 0) {
            competition.getCompetitionAnnexList().forEach(competitionAnnex -> {
                competitionAnnex.setCompetitionId(competition.getId());
                competitionAnnexService.save(competitionAnnex); // Save competition annex
            });
        }

        boolean update = competitionService.updateById(competition);


        return update ? R.ok() : R.error();
    }

    @NoRepeatSubmit
    @ApiOperation(value = "根据id删除活动")
    @PostMapping("/deleteCompetition")
    public R deleteCompetition(@ApiParam(name = "competitionId", value = "活动ID", required = true) @RequestParam Integer competitionId) {
        //删除活动下的定时
        competitionReleaseTimeService.remove(new QueryWrapper<CompetitionReleaseTime>().eq("competition_id", competitionId));
        boolean remove = competitionService.removeCompetition(competitionId);
        return remove ? R.ok() : R.error();
    }

    @NoRepeatSubmit
    @ApiOperation(value = "批量删除活动")
    @PostMapping("/batchDeleteCompetition")
    public R deleteCompetition(@ApiParam(name = "competitionIds", value = "活动ID", required = true) @RequestParam List<Integer> competitionIds) {
        competitionIds.forEach(competitionId -> {
            //删除活动下的定时
            competitionReleaseTimeService.remove(new QueryWrapper<CompetitionReleaseTime>().eq("competition_id", competitionId));
            competitionService.removeCompetition(competitionId);
        });
        return R.ok();
    }

    @ApiOperation(value = "活动详情", response = Competition.class)
    @PostMapping("/getCompetition")
    public R getCompetition(@ApiParam(name = "competitionId", value = "活动ID", required = true) @RequestParam("competitionId") String competitionId, HttpServletRequest request) {
        String accountId = TokenUtils.getIdByJwtToken(request);
        Competition competition = competitionService.getById(competitionId);

//        查询活动报名表
        QueryWrapper<CompetitionRegistration> wrapper = new QueryWrapper<>();
        wrapper.eq("competition_id", competitionId);
        wrapper.eq("account_id", accountId);
        CompetitionRegistration registration = competitionRegistrationService.getOne(wrapper);
        competition.setApplicantNum(competitionRegistrationService.registrationPopulation(competitionId));


        if (registration != null) {
            CompetitionTeam team = competitionTeamService.getById(registration.getTeamId());
            if (team != null) {
                if (team.getCaptain() == 0) {
                    registration.setTeamId(team.getTeamId());
                } else {
                    registration.setTeamId(team.getPid());
                }
            }
        }
        competition.setCompetitionRegistration(registration);
        List<CompetitionAnnex> CompetitionAnnexList = competitionAnnexService.getCompetitionAnnexByCompetitionId(competitionId);
        List<CompetitionRange> CompetitionRangeList = competitionRangeService.getCompetitionRangeByCompetitionId(competitionId);
        /*if (CompetitionRangeList.size() > 0) {
            competition.setCompetitionRangeList(CompetitionRangeList);
        }*/
        if (CompetitionAnnexList.size() > 0) {
            competition.setCompetitionAnnexList(CompetitionAnnexList);
        }

        //活动范围(1:全平台 5:幼儿园，6：供应商)
        /*if (competition.getCompetitionScope() == 1) {
            competition.setRange("全平台");
        } else if (competition.getCompetitionScope() == 5) {
            competition.setRange("幼儿园");
        }else if (competition.getCompetitionScope() == 6) {
            competition.setRange("供应商");
        }*/
//        if (competition.getReleaseType()==1){
        /*QueryWrapper<CompleteCompetitionSetup> setupQueryWrapper = new QueryWrapper<>();
        setupQueryWrapper.eq("competition_id", competitionId);
        CompleteCompetitionSetup competitionSetup = completeCompetitionSetupService.getOne(setupQueryWrapper);
        competition.setCompleteCompetitionSetup(competitionSetup);*/


       /* QueryWrapper<CompetitionRule> ruleQueryWrapper = new QueryWrapper<>();
        ruleQueryWrapper.eq("setup_id", competitionSetup.getSetupId());
        CompetitionRule one = competitionRuleService.getOne(ruleQueryWrapper);
        if (one != null) {
            competition.setRule(one.getRule());
        }*/
//        }
      /*  //查询活动阶段内容数据
        List<CompetitionStage> stages = competitionService.queryCompetitionStage(competitionId, accountId);



        R competitionContent = competitionContentService.getCompetitionContent(competitionId);
        List<CompetitionContent> contentList = (List<CompetitionContent>) competitionContent.get("contentList");
        competition.setContentList(contentList);
        competition.setCompetitionStage(stages);
        if (contentList.size() > 0) {

            for (CompetitionStage stage: stages) {
                QueryWrapper<CompetitionStageContentSetting> queryWrapper1 = new QueryWrapper<>();
                queryWrapper1.eq("competition_id", competitionId);
                queryWrapper1.eq("stage_id", stage.getStageId());
                CompetitionStageContentSetting stageContentSetting = contentSettingService.getOne(queryWrapper1);
                if (!ObjectUtil.isEmpty(stageContentSetting)) {
                    //赋值线下内容设置
                    stage.setCompetitionStageContentSetting(stageContentSetting);

                    QueryWrapper<CompetitionStageFile> queryWrapper2 = new QueryWrapper<>();
                    queryWrapper2.eq("competition_id",competitionId);
                    queryWrapper2.eq("stage_id", stage.getStageId());
                    queryWrapper2.eq("account_id",accountId);
                    CompetitionStageFile myFile =  stageFileService.getOne(queryWrapper2);
                    if (!ObjectUtil.isEmpty(myFile)){
                        stage.setCompetitionStageFile(myFile);
                    }
                }

            }

        }*/
        return R.ok().put("competition", competition);
    }

    @NoRepeatSubmit
    @ApiOperation(value = "发布活动")
    @PostMapping("/publishCompetition")
    public R publishCompetition(@ApiParam(name = "competitionId", value = "活动ID", required = true) @RequestParam("competitionId") Integer competitionId, @ApiParam(name = "publishStatus", value = "发布状态", required = true) @RequestParam("publishStatus") Integer publishStatus) {
        Competition getDetail = competitionService.getById(competitionId);
        getDetail.setPublishStatus(publishStatus);
        /* getDetail.setZtOpen(0);*/
        boolean ret = competitionService.updateById(getDetail);
        return ret ? R.ok() : R.error();
    }

    @NoRepeatSubmit
    @ApiOperation(value = "活动禁启用", response = CompetitionRange.class)
    @PostMapping("/disabledEvents")
    public R disabledEvents(@RequestBody DisableEnableCompetitionReq req) {

        Competition getDetail = competitionService.getById(req.getCompetitionId());
        Competition competition = new Competition();
        competition.setId(req.getCompetitionId());
        competition.setIsOpen(req.getIsOpen());


        boolean ret = competitionService.updateById(competition);
        return ret ? R.ok() : R.error();
    }


  /*  @ApiOperation(value = "职站——活动列表(教师端活动管理)")
    @PostMapping("/competitionPageConditionQueryByOccupationlab")
    public R CompetitionPageConditionQueryByOccupationlab(@RequestBody PageCompetitionListReq pageCompetitionListReq, HttpServletRequest request) {
        String accountId = TokenUtils.getIdByJwtToken(request);
        Integer schoolId = userClient.getSchoolIdByAccountId(accountId);
        pageCompetitionListReq.setSchoolId(schoolId);
        if (!ObjectUtils.isEmpty(accountId)) {
            pageCompetitionListReq.setAccountId(Integer.valueOf(accountId));
        }
        return competitionService.competitionPageConditionQueryByOccupationlab(pageCompetitionListReq);

    }*/


    @ApiOperation(value = "后台活动列表")
    @PostMapping("/backgroundActivityList")
    public R backgroundActivityList(@RequestBody PageCompetitionListReq pageCompetitionListReq, HttpServletRequest request) {
        return competitionService.backgroundActivityList(pageCompetitionListReq);
    }

    @ApiOperation(value = "查询活动状态(返回为false时表示当前活动在教师端已禁用)")
    @PostMapping("/checkTheStatusOfTheCompetition")
    public boolean checkTheStatusOfTheCompetition(@ApiParam(name = "competitionId", value = "活动ID", required = true) @RequestParam String competitionId) {
        Competition competition = competitionService.getById(competitionId);
        //是否开启(0开启 1未开启 默认0)
        return competition.getIsOpen() != 1;
    }

   /* @ApiOperation(value = "职站(学生端)——登录前活动", response = CompetitionVO.class)
    @PostMapping("/notLoggedInBeforeStudentEvents")
    public R notLoggedInBeforeStudentEvents(@RequestBody PageCompetitionListReq pageCompetitionListReq, HttpServletRequest request) throws ExecutionException, InterruptedException {
        String accountId = TokenUtils.getIdByJwtToken(request);
        Integer schoolId = userClient.getSchoolIdByAccountId(accountId);
        if (!ObjectUtils.isEmpty(accountId)) {
            pageCompetitionListReq.setSchoolId(schoolId);
            pageCompetitionListReq.setAccountId(Integer.valueOf(accountId));
        } else {
            pageCompetitionListReq.setAccountId(0);
        }
        Future<IPage<CompetitionVO>> pageFuture = competitionService.notLoggedInBeforeStudentEvents(pageCompetitionListReq);
        IPage<CompetitionVO> pageList = pageFuture.get();
        return R.ok().put("data", pageList);

    }

    */

    @ApiOperation(value = "小程序—登录后活动", response = CompetitionVO.class)
    @PostMapping("/postLoginActivity")
    public R postLoginActivity(@RequestBody AfterLoginPageCompetitionListReq pageCompetitionListReq, HttpServletRequest request) throws ExecutionException, InterruptedException {
        String accountId = TokenUtils.getIdByJwtToken(request);
        Integer schoolId = userClient.getSchoolIdByAccountId(accountId);
        if (!ObjectUtils.isEmpty(accountId)) {
            pageCompetitionListReq.setSchoolId(schoolId);
            pageCompetitionListReq.setAccountId(Integer.valueOf(accountId));
            Integer role = Integer.valueOf(competitionService.queryRole(accountId));
            if (!ObjectUtils.isEmpty(role)) {
                pageCompetitionListReq.setRole(role);
            }
        }
        Future<IPage<CompetitionVO>> pageFuture = competitionService.competitionAfterLogin(pageCompetitionListReq);
        IPage<CompetitionVO> pageList = pageFuture.get();
        return R.ok().put("data", pageList);
    }


    @ApiOperation(value = "获取当前时间")
    @GetMapping("/getCurrentTime")
    public R getCurrentTime() {
        return R.ok().put("currentTime", new Date());
    }

}


