package com.huoran.competition.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huoran.common.aop.annotation.NoRepeatSubmit;
import com.huoran.common.response.R;
import com.huoran.competition.entity.CompetitionProgress;
import com.huoran.competition.service.CompetitionProgressService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

/**
 * <p>
 * 活动进展表 前端控制器
 * </p>
 *
 * @author cheney
 * @since 2022-11-16
 */
@Api(tags = "kindergarten-活动进展管理")
@RestController
@RequestMapping("/competition/progress")
public class CompetitionProgressController {

    @Autowired
    private CompetitionProgressService competitionProgressService;
    @NoRepeatSubmit
    @ApiOperation(value = "添加活动进展")
    @PostMapping("/addCompetitionProgress")
    public R addCompetitionProgress(
            @ApiParam(name = "competitionProgress", value = "活动进展对象", required = true)
            @RequestBody CompetitionProgress competitionProgress) {
        //判重
        boolean isRepeat = competitionProgressService.repeat(competitionProgress);
        if (isRepeat) {
            return R.error("活动进展已存在");
        }
        boolean save = competitionProgressService.save(competitionProgress);
        return save ? R.ok() : R.error();
    }

    @ApiOperation(value = "根据活动id查询活动进展")
    @GetMapping("/getCompetitionProgress")
    public R getCompetitionProgress(
            @ApiParam(name = "competitionId" , value = "賽事id")
            @RequestParam("competitionId") String competitionId) {
        QueryWrapper<CompetitionProgress> wrapper = new QueryWrapper<>();
        wrapper.eq("competition_id", competitionId);
        List<CompetitionProgress> competitionProgressList = competitionProgressService.list(wrapper);
        return R.ok().put("competitionProgressList", competitionProgressList);
    }
    @NoRepeatSubmit
    @ApiOperation(value = "根据id删除活动进展")
    @DeleteMapping("/deleteCompetitionProgress")
    public R deleteCompetitionProgress(
            @ApiParam(name = "competitionProgressId", value = "活动进展ID", required = true)
            @RequestParam Integer competitionProgressId) {
        boolean remove = competitionProgressService.removeById(competitionProgressId);
        return remove ? R.ok() : R.error();
    }
    @NoRepeatSubmit
    @ApiOperation(value = "修改活动进展")
    @PutMapping("/editCompetitionProgress")
    public R editCompetitionProgress(
            @ApiParam(name = "competitionProgress", value = "活动进展对象", required = true)
            @RequestBody CompetitionProgress competitionProgress) {
        boolean update = competitionProgressService.updateById(competitionProgress);
        return update ? R.ok() : R.error();
    }

}

