package com.huoran.competition.entity;

import com.baomidou.mybatisplus.annotation.*;

import java.util.Date;
import java.io.Serializable;
import java.util.List;

import com.huoran.competition.entity.resp.CompetitionRangeResp;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;

/**
 * <p>
 * 活动信息表
 * </p>
 *
 * @author cheney
 * @since 2022-11-16
 */
@Data
@EqualsAndHashCode(callSuper = false)
@Accessors(chain = true)
@TableName("c_competition")
@ApiModel(value="Competition对象", description="活动信息表")
public class Competition implements Serializable {

    private static final long serialVersionUID = 1L;

    @ApiModelProperty(value = "主键")
    @TableId(value = "id", type = IdType.AUTO)
    private Integer id;

    @ApiModelProperty(value = "创建人Id", example = "1")
    private Integer founderId;

    @ApiModelProperty(value = "创建人名称", example = "张三李四王五")
    private String founderName;

    @ApiModelProperty(value = "活动名称", example = "数据库数据库色即是空")
    private String name;

    @ApiModelProperty(value = "活动封面图", example = "http://localhost:9400/occupationlab/doc.html")
    private String coverUrl;

    @ApiModelProperty(value = "活动轮播图", example = "http://localhost:9400/occupationlab/doc.html")
    private String carouselUrl;

    @ApiModelProperty(value = "主办方", example = "或然科技")
    private String sponsor;

    @ApiModelProperty(value = "承办方", example = "或然科技")
    private String undertaker;

    @ApiModelProperty(value = "活动描述", example = "或然科技活动")
    private String description;

    @ApiModelProperty(value = "发布状态（0未发布，1已发布）", example = "0")
    private Integer publishStatus;

    @ApiModelProperty(value = "活动开始时间", example = "2021-09-09 16:24:24")
    private Date playStartTime;

    @ApiModelProperty(value = "活动结束时间", example = "2021-09-20 16:24:24")
    private Date playEndTime;

    @ApiModelProperty(value = "报名开始时间", example = "2021-09-10 16:24:24")
    private Date signUpStartTime;

    @ApiModelProperty(value = "报名结束时间", example = "2021-09-20 16:24:24")
    private Date signUpEndTime;

    @ApiModelProperty(value = "平台来源(5:幼儿园，6：供应商)", example = "5")
    private Integer platformSource;

    @ApiModelProperty(value = "活动范围(1:全平台 5:幼儿园，6：供应商)", example = "0")
    private String competitionScope;

    @ApiModelProperty(value = "是否开启(0开启 1未开启 默认0)", example = "0")
    private Integer isOpen;

    @ApiModelProperty(value = "发布类型（0发布信息，1完整活动）默认为0")
    private Integer releaseType;

    @ApiModelProperty(value = "是否展示报名人员(0否，1是)")
    private String whetherToShowApplicants;

    @ApiModelProperty(value = "举办地点")
    private String reportingPlace;


    @ApiModelProperty(value = "创建来源(0默认平台创建 1...待定)")
    private Integer createSource;


    @ApiModelProperty(value = "创建时间")
    @TableField(fill = FieldFill.INSERT)
    private Date createTime;

    @ApiModelProperty(value = "更新时间")
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private Date updateTime;


    @ApiModelProperty(value = "数量上限限制")
    private Integer quantityLimit;

    @ApiModelProperty(value = "活动分类id")
    private Integer classificationId;


    @ApiModelProperty(value = "邀请码")
    private String invitationCode;

    @ApiModelProperty(value = "是否需要邀请码（0不需要，1需要）默认为0")
    private Integer isNeedCode;



    //======非数据库字段===================
    @ApiModelProperty(value = "活动附件")
    @TableField(exist = false)
    private List<CompetitionAnnex> competitionAnnexList;



    @ApiModelProperty(value = "报名人数")
    @TableField(exist = false)
    private Integer applicantNum;

    @ApiModelProperty(value = "赛制")
    @TableField(exist = false)
    private Integer rule;


    @ApiModelProperty(value = "活动报名信息")
    @TableField(exist = false)
    private CompetitionRegistration competitionRegistration;

    @ApiModelProperty(value = "活动分类名称")
    @TableField(exist = false)
    private String classificationName;

}
