package com.huoran.competition.entity;

import com.baomidou.mybatisplus.annotation.*;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;

import java.io.Serializable;
import java.util.Date;

/**
 * <p>
 * 活动阶段设置表
 * </p>
 *
 * @author cheney
 * @since 2022-11-16
 */
@Data
@EqualsAndHashCode(callSuper = false)
@Accessors(chain = true)
@TableName("c_competition_stage")
@ApiModel(value="CompetitionStage对象", description="活动阶段设置表")
public class CompetitionStage implements Serializable {

    private static final long serialVersionUID = 1L;

    @ApiModelProperty(value = "主键")
    @TableId(value = "stage_id", type = IdType.AUTO)
    private Integer stageId;

    @ApiModelProperty(value = "规则id（c_competition_rule表主键）")
    private Integer ruleId;

    @ApiModelProperty(value = "阶段名称")
    private String stageName;

    @ApiModelProperty(value = "序号（第几阶段）")
    private Integer number;

    @ApiModelProperty(value = "活动方式（0实操，1理论，2线下）默认为0")
    private Integer method;

    @ApiModelProperty(value = "权重（积分赛，所有阶段权和为1）")
    private Double pointWeight;

    @ApiModelProperty(value = "排名晋级人数限制（淘汰赛）")
    private String peopleLimit;

    @ApiModelProperty(value = "排名晋级百分比限制（淘汰赛）")
    private Double percentageLimit;

    @ApiModelProperty(value = "排名晋级分数限制（淘汰赛）")
    private String scoreLimit;

    @ApiModelProperty(value = "成绩公布时间")
    @TableField(strategy = FieldStrategy.IGNORED)
    private Integer resultAnnouncementTime;

    @ApiModelProperty(value = "是否公布成绩详情（0为公布，1不公布）默认为1")
    private Integer resultsDetails;

    @ApiModelProperty(value = "参数标识，大于0表示参加过活动")
    @TableField(exist = false)
    private Integer count;

    @ApiModelProperty(value = "团队参赛人数限制（0表示不限制）默认为0")
    private Integer teamNumLimit;

    @ApiModelProperty(value = "团队成绩计算方式（0：最高分，1：平均分，2：总分）")
    private String teamCalculationMethod;

    @ApiModelProperty(value = "参赛人员名称（用户姓名）")
    @TableField(exist = false)
    private String teamParticipantIds;

    @ApiModelProperty(value = "参赛人员ids(账号id)")
    @TableField(exist = false)
    private String participantAccountIds;

    @ApiModelProperty(value = "创建时间")
    @TableField(fill = FieldFill.INSERT)
    private Date createTime;

    @ApiModelProperty(value = "更新时间")
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private Date updateTime;

    @ApiModelProperty(value = "主键id")
    @TableField(exist = false)
    private Integer contentId;

    @ApiModelProperty(value = "活动开始时间")
    @TableField(exist = false)
    private Date startTime;

    @ApiModelProperty(value = "活动结束时间")
    @TableField(exist = false)
    private Date endTime;

    @ApiModelProperty(value = "课程id")
    @TableField(exist = false)
    private Integer cid;

    @ApiModelProperty(value = "商品id")
    @TableField(exist = false)
    private Integer mallId;

    @ApiModelProperty(value = "考核配置id")
    @TableField(exist = false)
    private Integer assessmentId;

    @ApiModelProperty(value = "系统id")
    @TableField(exist = false)
    private Integer systemId;

    @ApiModelProperty(value = "项目管理表id")
    @TableField(exist = false)
    private Integer projectId;

    @ApiModelProperty(value = "线上地址")
    @TableField(exist = false)
    private String onlineAddress;

    @ApiModelProperty(value = "线下地址")
    @TableField(exist = false)
    private String offlineAddress;

    @ApiModelProperty(value = "线上地址展示（默认为0不展示）")
    @TableField(exist = false)
    private Integer onlineButton;

    @ApiModelProperty(value = "线下地址展示（默认为0不展示）")
    @TableField(exist = false)
    private Integer offlineButton;

    @ApiModelProperty(value = "活动内容描述")
    @TableField(exist = false)
    private String contentDescription;

    @ApiModelProperty(value = "评分规则")
    @TableField(exist = false)
    private String scoreRule;

    @ApiModelProperty(value = "分数")
    @TableField(exist = false)
    private Double score;

    @ApiModelProperty(value = "分数报告id")
    @TableField(exist = false)
    private Integer reportId;

    @ApiModelProperty(value = "课程系统名称")
    @TableField(exist = false)
    private String systemName;

    @ApiModelProperty(value = "活动内容")
    @TableField(exist = false)
    private CompetitionContent competitionContent;

    @ApiModelProperty(value = "线下内容设置信息")
    @TableField(exist = false)
    private CompetitionStageContentSetting competitionStageContentSetting;

    @ApiModelProperty(value = "学生上传的线下文件")
    @TableField(exist = false)
    private CompetitionStageFile competitionStageFile;

    @ApiModelProperty(value = "系统链接")
    @TableField(exist = false)
    private String systemLink;

    @ApiModelProperty(value = "说明")
    @TableField(exist = false)
    private String stageExplain;

    @ApiModelProperty(value = "是否支持上传文件(默认0:不支持,1:支持)")
    @TableField(exist = false)
    private Integer whetherToUploadFiles;

    @ApiModelProperty(value = "文件路径")
    @TableField(exist = false)
    private String fileUrl;

    @ApiModelProperty(value = "文件路径")
    @TableField(exist = false)
    private String fileName;

    @ApiModelProperty(value = "用户文件名称")
    @TableField(exist = false)
    private String userFileName;

    @ApiModelProperty(value = "文件大小")
    @TableField(exist = false)
    private String fileSize;

    @ApiModelProperty(value = "文件格式")
    @TableField(exist = false)
    private String fileFormat;

    @ApiModelProperty(value = "oss文件名")
    @TableField(exist = false)
    private String ossFileName;

    @ApiModelProperty(value = "文件路径")
    @TableField(exist = false)
    private String filePath;

    @ApiModelProperty(value = "用户文件id")
    @TableField(exist = false)
    private Integer userFileId;
}