package com.huoran.competition.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huoran.common.entity.School;
import com.huoran.competition.entity.CompetitionRange;
import com.huoran.competition.entity.resp.CompetitionRangeResp;
import com.huoran.competition.entity.resp.SchoolResp;
import com.huoran.competition.mapper.CompetitionRangeMapper;
import com.huoran.competition.service.CompetitionRangeService;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * <p>
 * 活动范围表 服务实现类
 * </p>
 *
 * @author cheney
 * @since 2022-11-16
 */
@Service
public class CompetitionRangeServiceImpl extends ServiceImpl<CompetitionRangeMapper, CompetitionRange> implements CompetitionRangeService {

    @Override
    public List<CompetitionRange> getCompetitionRangeByCompetitionId(String competitionId) {
        QueryWrapper<CompetitionRange> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("competition_id", competitionId);
        return baseMapper.selectList(queryWrapper);
    }

    @Override
    public List<CompetitionRangeResp> getRangeByCompetitionId(Integer id) {
        return baseMapper.getRangeByCompetitionId(id);
    }

    @Override
    public boolean checkWhetherToParticipate(School school, Integer competitionId) {
        //校验地区范围为学校的标准
        QueryWrapper<CompetitionRange> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("type", 0);
        queryWrapper1.eq("is_del", 0);
        queryWrapper1.eq("school_id", school.getSchoolId());
        queryWrapper1.eq("competition_id", competitionId);
        List<CompetitionRange> schoolList1 = baseMapper.selectList(queryWrapper1);

        //校验地区范围为省的标准
        QueryWrapper<CompetitionRange> queryWrapper2 = new QueryWrapper<>();
        queryWrapper2.eq("competition_id", competitionId);
        queryWrapper2.eq("type", 1);
        queryWrapper2.eq("is_del", 0);
        queryWrapper2.eq("province_id", school.getProvinceId());
        List<CompetitionRange> schoolList2 = baseMapper.selectList(queryWrapper2);

        //校验地区范围为市的标准
        QueryWrapper<CompetitionRange> queryWrapper3 = new QueryWrapper<>();
        queryWrapper3.eq("type", 1);
        queryWrapper3.eq("is_del", 0);
        queryWrapper3.eq("province_id", school.getProvinceId());
        queryWrapper3.eq("city_id", school.getCityId());
        queryWrapper3.eq("competition_id", competitionId);
        List<CompetitionRange> schoolList3 = baseMapper.selectList(queryWrapper3);

        if (schoolList1.size() <= 0 && schoolList2.size() <= 0 && schoolList3.size() <= 0) {
            return true; //你所在院校不符合参赛要求，无法报名！
        }

        return false;
    }

    @Override
    public List<SchoolResp> getCity(Integer cityId,Integer provinceId) {
        return baseMapper.getCity(cityId,provinceId);
    }

    @Override
    public List<SchoolResp> getProvince(Integer provinceId) {
        return baseMapper.getProvince(provinceId);
    }

    @Override
    public List<SchoolResp> getSchool(Integer schoolId) {
        return baseMapper.getSchool(schoolId);
    }
}
