package com.huoran.competition.service.impl;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import cn.afterturn.easypoi.excel.entity.enmus.ExcelType;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huoran.common.constant.DelConstant;
import com.huoran.common.constant.TeamCalculationMethodConstant;
import com.huoran.common.entity.School;
import com.huoran.common.exception.CustomException;
import com.huoran.common.exception.ExceptionEnum;
import com.huoran.common.response.R;
import com.huoran.competition.entity.CompetitionRanking;
import com.huoran.competition.entity.CompetitionRule;
import com.huoran.competition.entity.CompetitionStage;
import com.huoran.competition.entity.CompleteCompetitionSetup;
import com.huoran.competition.entity.req.*;
import com.huoran.competition.entity.resp.*;
import com.huoran.competition.entity.vo.ImportCompetitionRankingFailureVO;
import com.huoran.competition.entity.vo.ImportIndividualRankingFailureVO;
import com.huoran.competition.mapper.*;
import com.huoran.competition.service.CompetitionRankingService;
import com.huoran.competition.utils.ExcelImporEventResultstHelper;
import com.huoran.competition.utils.ExcelStyleUtil;
import com.huoran.competition.utils.poi.CommonCode;
import com.huoran.competition.utils.poi.ExceptionCast;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLEncoder;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * <p>
 * 活动排名表 服务实现类
 * </p>
 *
 * @author cheney
 * @since 2023-01-03
 */
@Service
public class CompetitionRankingServiceImpl extends ServiceImpl<CompetitionRankingMapper, CompetitionRanking> implements CompetitionRankingService {
    @Autowired
    private UserScoreMapper userScoreMapper;

    @Resource
    StringRedisTemplate stringRedisTemplate;
    @Autowired
    private CompetitionTeamMapper teamMapper;
    @Autowired
    CompleteCompetitionSetupMapper competitionSetupMapper;
    @Autowired
    CompetitionRuleMapper ruleMapper;
    @Autowired
    CompetitionStageMapper stageMapper;

    /**
     * 团队批量导入排名
     *
     * @param file
     * @param competitionId
     * @param stageId
     * @param isOverallRanking
     * @return
     * @throws IOException
     */
    @Override
    public Map<String, Object> batchImportRanking(MultipartFile file, Integer competitionId, Integer stageId, Integer isOverallRanking, Integer schoolId) throws IOException {
        List<ExcelImpRankReq> impGradeReqList = ExcelImporEventResultstHelper.readRanking(file);
        List<CompetitionRanking> returnedSet = new ArrayList<>();
        if (impGradeReqList.size() <= 0) {
            //小于等于0为上传空模板的情况下 抛出异常
            throw new CustomException(ExceptionEnum.EXCEL_FILE_FORMAT_ERROR);
        }
        int count = 0;
        List<ImportCompetitionRankingFailureVO> failVo1 = new ArrayList<ImportCompetitionRankingFailureVO>();
        // 参数合法性校验，只能上传.xlsx后缀的文件
        if (org.apache.commons.lang3.StringUtils.isBlank(file.getOriginalFilename()) || !file.getOriginalFilename().substring(file.getOriginalFilename().lastIndexOf(".")).equals(".xlsx")) {
            ExceptionCast.cast(CommonCode.EXCEL_FILE_INVALID);
        }
        int size = impGradeReqList.size();
        Map<String, Object> map = new HashMap<>();
        Long ii = 1L;//用于记录序列号
        Integer countNum = 0;//用于失败原因排序 eg:1、XXXXX 2、XXXXXXX
        try {
            Integer countSuccess = 0;//统计成功次数
            Integer countError = 0;//统计失败次数

            List<String> workNumberList = new ArrayList<>();//用来储存新增前用户在excel中输入的所有工号用于excel中的数据判重
            for (int i = 0; i < impGradeReqList.size(); i++) {
                boolean ret = true;
                ++ii;
                ExcelImpRankReq req = impGradeReqList.get(i);

                ImportCompetitionRankingFailureVO vo1 = new ImportCompetitionRankingFailureVO();

                String errMsg = "";
                //去除Excel中的全角半角空格

                //团队名称
                String getTeamName = req.getTeamName().replaceAll("\\u00A0", "").trim();
                //队长名称
                String teamLeader = req.getUserName().replaceAll("\\u00A0", "").trim();
                String getSchoolName = req.getSchoolName().replaceAll("\\u00A0", "").trim();
                String getScore = req.getScore().replaceAll("\\u00A0", "").trim();
                String getTimeSum = req.getTimeSum().replaceAll("\\u00A0", "").trim();


                vo1.setIndex(ii);


                //检验团队名称
                if (getTeamName.equals("")) {
                    ++countNum;
                    vo1.setTeamName(getTeamName + " *必填项:(团队不能为空) ");
                    errMsg += countNum + "必填项:(团队不能为空)";
                    ret = false;
                    countError++;
                } else {
                    vo1.setTeamName(getTeamName);
                }

                //检验姓名
                if (teamLeader.equals("")) {
                    ++countNum;
                    vo1.setTeamLeader(teamLeader + " *必填项:(队长姓名不能为空) ");
                    errMsg += countNum + "必填项:(队长姓名不能为空)";
                    ret = false;
                    countError++;
                } else {
                    vo1.setTeamLeader(teamLeader);
                }


                //检验学校名称
                if (getSchoolName.equals("")) {
                    ++countNum;
                    vo1.setSchoolName(getSchoolName + " *必填项:(学校不能为空) ");
                    errMsg += countNum + "必填项:(学校不能为空)";
                    ret = false;
                    countError++;
                } else {
                    vo1.setSchoolName(getSchoolName);
                }


                School school = baseMapper.obtainSchoolInformation(getSchoolName);
                if (school == null) {
                    ++countNum;
                    vo1.setSchoolName(getSchoolName + " *必填项:(该学校不存在) ");
                    errMsg += countNum + "必填项:(该学校不存在)";
                    ret = false;
                    countError++;
                }


                //检验分数
                if (getScore.equals("")) {
                    ++countNum;
                    vo1.setScore(getScore + " *必填项:(分数不能为空) ");
                    errMsg += countNum + "必填项:(分数不能为空)";
                    ret = false;
                    countError++;
                } else {
                    vo1.setScore(getScore);
                }


                //检验耗时
                if (getTimeSum.equals("")) {
                    ++countNum;
                    vo1.setScore(getTimeSum + " *必填项:(耗时不能为空) ");
                    errMsg += countNum + "必填项:(耗时不能为空)";
                    ret = false;
                    countError++;
                } else {
                    vo1.setTimeSum(getTimeSum);
                }


                if (!getTimeSum.equals("")) {
                    if (isLong(getTimeSum) == false) {
                        ++countNum;
                        vo1.setTimeSum(getTimeSum + " *必填项:(耗时请填写整数,不含小数点) ");
                        errMsg += countNum + "必填项:(耗时请填写整数,不含小数点)";
                        ret = false;
                        countError++;
                    } else {
                        vo1.setTimeSum(getTimeSum);
                    }
                }
                TeamAndLeaderResp resp = null;

                //检验团队信息和队长以及活动
                if (!getTeamName.equals("") && !teamLeader.equals("")) {
                    TeamAndLeaderResp query = new TeamAndLeaderResp();
                    query.setCompetitionId(competitionId);
                    query.setTeamLeader(teamLeader);
                    query.setTeamName(getTeamName);
                    resp = baseMapper.queryTeamLeaderAndTeamInformation(query);

                    if (resp == null) {
                        ++countNum;
                        vo1.setTeamLeader(teamLeader + " *必填项:(团队信息和队长信息不匹配) ");
                        vo1.setTeamName(getTeamName + " *必填项:(团队信息和队长信息不匹配) ");
                        errMsg += countNum + "必填项:(团队信息和队长信息不匹配，请输入正确的团队以及队长信息)";
                        ret = false;
                        countError++;
                    } else {
                        vo1.setTeamLeader(teamLeader);
                        vo1.setTeamName(getTeamName);
                    }


                }


                vo1.setFailureMsg(errMsg);
                countNum = 0;

                failVo1.add(vo1);

                //ret判断 新增成功的插入到ol_user_score表 否则则不新增至数据库
                if (ret) {
                    /*//执行新增到操作
                    CompetitionRanking ranking = new CompetitionRanking();
                    ranking.setScore(getScore);
                    ranking.setCompetitionId(competitionId);//活动id
                    ranking.setTimeSum(Integer.valueOf(getTimeSum));//耗时
                    ranking.setStageId(stageId);
                    ranking.setIsOverallRanking(isOverallRanking);
                    if (school.getSchoolId() != null) {
                        ranking.setSchoolId(school.getSchoolId());
                    }
                    baseMapper.insert(ranking);
                    //countSuccess：统计累计成功次数
                    countSuccess++;*/


                    //notice：存到list中返回给前端 点击确认发布排名后传入list再发布(批量导入后不直接存入数据库要再次确认才发布)
                    CompetitionRanking ranking = new CompetitionRanking();
                    ranking.setScore(getScore);
                    ranking.setCompetitionId(competitionId);//活动id
                    ranking.setTimeSum(getTimeSum);//耗时
                    ranking.setStageId(stageId);
                    ranking.setPublicationType(DelConstant.PUBLICATION_TYPE_MANUALOPERATION);

                    ranking.setIsOverallRanking(isOverallRanking);
                    if (resp != null) {
                        ranking.setTeamId(resp.getTeamId());
                        ranking.setLeaderName(resp.getTeamLeader());
                        ranking.setTeamName(resp.getTeamName());
                    }

                    if (school.getSchoolId() != null) {
                        ranking.setSchoolId(school.getSchoolId());
                        ranking.setSchoolName(resp.getSchoolName());
                    }
                    returnedSet.add(ranking);
                }
            }
            String exportCode = "";


            Integer numberOfSuccesses = 0;
            if (returnedSet != null) {
                //降序排列
                //多条件-先年龄升序、后分数升序
                Collections.sort(returnedSet, new Comparator<CompetitionRanking>() {
                    public int compare(CompetitionRanking s1, CompetitionRanking s2) {
                        int i = Double.valueOf(s2.getScore()).compareTo(Double.valueOf(s1.getScore()));
                        if (i == 0) {
                            i = s1.getTimeSum().compareTo(s2.getTimeSum());
                        }
                        return i;
                    }
                });

                for (CompetitionRanking r : returnedSet) {
                    Integer ret = baseMapper.insert(r);
                    if (ret == 1) {
                        numberOfSuccesses++;
                    }
                }


            }


            // 有导入失败的数据，才会存入redis
            if (countError > 0) {
                //生成token
                exportCode = "FAILURE_IMPORT" + UUID.randomUUID().toString().replace("-", "");
                ValueOperations<String, String> ops = stringRedisTemplate.opsForValue();

                List<ImportCompetitionRankingFailureVO> collect = failVo1.stream().filter(importStaffFailureVo -> {
                    return !importStaffFailureVo.getFailureMsg().equals("");
                }).collect(Collectors.toList());

                String failureVOJson = JSON.toJSONString(collect);
                ops.set(exportCode, failureVOJson, 30 * 60, TimeUnit.SECONDS);
            }


            map.put("exportCode", exportCode);//返回导出code
            map.put("successNum", numberOfSuccesses + "");//本次新增成功数量
            map.put("failureNum", countError);//本次新增失败数量
            map.put("data", returnedSet);
            return map;
        } catch (RuntimeException e) {
            System.err.println(e.getMessage());
            throw new RuntimeException();
        }
    }

    @Override
    public Map<String, Object> importIndividualRankingsInBulk(MultipartFile file, Integer competitionId, Integer stageId, Integer isOverallRanking, Integer schoolId) throws IOException {
        List<ExcelImpRankReq> impGradeReqList = ExcelImporEventResultstHelper.readIndividualRanking(file);

        if (impGradeReqList.size() <= 0) {
            //小于等于0为上传空模板的情况下 抛出异常
            throw new CustomException(ExceptionEnum.EXCEL_FILE_FORMAT_ERROR);
        }
        int count = 0;
        List<ImportIndividualRankingFailureVO> failVo1 = new ArrayList<ImportIndividualRankingFailureVO>();
        // 参数合法性校验，只能上传.xlsx后缀的文件
        if (org.apache.commons.lang3.StringUtils.isBlank(file.getOriginalFilename()) || !file.getOriginalFilename().substring(file.getOriginalFilename().lastIndexOf(".")).equals(".xlsx")) {
            ExceptionCast.cast(CommonCode.EXCEL_FILE_INVALID);
        }

        List<CompetitionRanking> returnedSet = new ArrayList<>();
        int size = impGradeReqList.size();
        Map<String, Object> map = new HashMap<>();
        Long ii = 1L;//用于记录序列号
        Integer countNum = 0;//用于失败原因排序 eg:1、XXXXX 2、XXXXXXX
        try {
            Integer countSuccess = 0;//统计成功次数
            Integer countError = 0;//统计失败次数

            List<String> workNumberList = new ArrayList<>();//用来储存新增前用户在excel中输入的所有工号用于excel中的数据判重
            for (int i = 0; i < impGradeReqList.size(); i++) {
                boolean ret = true;
                ++ii;
                ExcelImpRankReq req = impGradeReqList.get(i);

                ImportIndividualRankingFailureVO vo1 = new ImportIndividualRankingFailureVO();

                String errMsg = "";
                //去除Excel中的全角半角空格
                String getUserName = req.getUserName().replaceAll("\\u00A0", "").trim();
                String getWorkNumber = req.getWorkNumber().replaceAll("\\u00A0", "").trim();
                String getSchoolName = req.getSchoolName().replaceAll("\\u00A0", "").trim();
                String getScore = req.getScore().replaceAll("\\u00A0", "").trim();
                String getTimeSum = req.getTimeSum().replaceAll("\\u00A0", "").trim();


                vo1.setIndex(ii);

                if (!workNumberList.contains(req.getWorkNumber())) {
                    workNumberList.add(req.getWorkNumber());
                } else {
                    if (!getWorkNumber.equals("")) {
                        //包含excel数据中存在相同工号
                        ++countNum;
                        vo1.setWorkNumber(getWorkNumber + " *必填项:(当前Excel数据中该工号已存在重复的!)");
                        errMsg += countNum + " *必填项:(当前Excel数据中该工号已存在重复的)";
                        ret = false;
                    }

                }


                //检验姓名
                if (getUserName.equals("")) {
                    ++countNum;
                    vo1.setUserName(getUserName + " *必填项:(姓名不能为空) ");
                    errMsg += countNum + "必填项:(姓名不能为空)";
                    ret = false;
                    countError++;
                } else {
                    vo1.setUserName(getUserName);
                }


                //检验学校名称
                if (getSchoolName.equals("")) {
                    ++countNum;
                    vo1.setSchoolName(getSchoolName + " *必填项:(学校不能为空) ");
                    errMsg += countNum + "必填项:(学校不能为空)";
                    ret = false;
                    countError++;
                } else {
                    vo1.setSchoolName(getSchoolName);
                }


                //校验学号是否存在汉字
                if (!getWorkNumber.trim().equals("")) {
                    if (ExcelImporEventResultstHelper.isContainChinese(getWorkNumber)) {
                        ++countNum;
                        vo1.setWorkNumber(getWorkNumber + "* 学号不能存在汉字!");
                        errMsg += countNum + " * 学号不能存在汉字!";
                        ret = false;
                        countError++;
                    }
                }


                //检验工号
                if (getWorkNumber.equals("")) {
                    ++countNum;
                    vo1.setWorkNumber(getWorkNumber + " *必填项:(学号不能为空)");
                    errMsg += countNum + " *必填项:(学号不能为空)";
                    ret = false;
                    countError++;
                } else {
                    vo1.setWorkNumber(getWorkNumber);
                }


                //检验姓名
                if (getUserName.equals("")) {
                    ++countNum;
                    vo1.setUserName(getUserName + " *必填项:(姓名不能为空) ");
                    errMsg += countNum + "必填项:(姓名不能为空)";
                    ret = false;
                    countError++;
                } else {
                    vo1.setUserName(getUserName);
                }

                School school = baseMapper.obtainSchoolInformation(getSchoolName);
                if (school == null) {
                    ++countNum;
                    vo1.setSchoolName(getSchoolName + " *必填项:(该学校不存在) ");
                    errMsg += countNum + "必填项:(该学校不存在)";
                    ret = false;
                    countError++;
                }

                UserAccountResp checkStudentNumberAndName = null;
                //校验学号和姓名是否为同一个人 如果是同一人则获取到账号id
                if (!getUserName.equals("") && !getWorkNumber.equals("")) {
                    //checkStudentNumberAndName检测用户名和学号以及学校是否是同一人
                    if (school!=null){
                        checkStudentNumberAndName = userScoreMapper.checkStudentNumberAndName(getUserName, getWorkNumber, school.getSchoolId());
                    }


                    if (checkStudentNumberAndName == null) {
                        if (!getUserName.equals("") && !getWorkNumber.equals("")) {
                            ++countNum;
                            vo1.setWorkNumber(getWorkNumber + " *必填项:(学号与学生姓名不匹配)");
                            vo1.setUserName(getUserName + " *必填项:(学生姓名与学号不匹配!)");
                            errMsg += countNum + " *必填项:(学号与学生姓名不匹配!)";
                            ret = false;
                            countError++;
                        }
                    }
                }


                //检验分数
                if (getScore.equals("")) {
                    ++countNum;
                    vo1.setScore(getScore + " *必填项:(分数不能为空) ");
                    errMsg += countNum + "必填项:(分数不能为空)";
                    ret = false;
                    countError++;
                } else {
                    vo1.setScore(getScore);
                }


                //检验耗时
                if (getTimeSum.equals("")) {
                    ++countNum;
                    vo1.setScore(getTimeSum + " *必填项:(耗时不能为空) ");
                    errMsg += countNum + "必填项:(耗时不能为空)";
                    ret = false;
                    countError++;
                } else {
                    vo1.setTimeSum(getTimeSum);
                }


                if (!getTimeSum.equals("")) {
                    if (isLong(getTimeSum) == false) {
                        ++countNum;
                        vo1.setTimeSum(getTimeSum + " *必填项:(耗时请填写整数,不含小数点) ");
                        errMsg += countNum + "必填项:(耗时请填写整数,不含小数点)";
                        ret = false;
                        countError++;
                    } else {
                        vo1.setTimeSum(getTimeSum);
                    }
                }


                vo1.setFailureMsg(errMsg);
                countNum = 0;

                failVo1.add(vo1);

                //ret判断 新增成功的插入到ol_user_score表 否则则不新增至数据库
                if (ret) {
                    /*
                     //执行新增到操作
                    CompetitionRanking ranking = new CompetitionRanking();
                    ranking.setScore(getScore);
                    ranking.setCompetitionId(competitionId);//活动id
                    ranking.setTimeSum(Integer.valueOf(getTimeSum));//耗时
                    ranking.setStageId(stageId);
                    ranking.setIsOverallRanking(isOverallRanking);
                    ranking.setAccountId(checkStudentNumberAndName.getId());
                    if (school.getSchoolId() != null) {
                        ranking.setSchoolId(school.getSchoolId());
                    }
                    baseMapper.insert(ranking);
                    //countSuccess：统计累计成功次数
                    countSuccess++;*/

                    //notice：存到list中返回给前端 点击确认发布排名后传入list再发布(批量导入后不直接存入数据库要再次确认才发布)
                    CompetitionRanking ranking = new CompetitionRanking();
                    ranking.setScore(getScore);
                    ranking.setCompetitionId(competitionId);//活动id
                    ranking.setTimeSum(getTimeSum);//耗时
                    ranking.setStageId(stageId);
                    ranking.setIsOverallRanking(isOverallRanking);
                    ranking.setAccountId(checkStudentNumberAndName.getId());
                    //ranking.setIsRelease(1);//是否发布(默认0未发布 1已发布)
                    ranking.setPublicationType(1);
                    if (school.getSchoolId() != null) {
                        ranking.setSchoolId(school.getSchoolId());
                    }


                    returnedSet.add(ranking);

                }
            }
            String exportCode = "";

            // 有导入失败的数据，才会存入redis
            if (countError > 0) {
                //生成token
                exportCode = "FAILURE_IMPORT" + UUID.randomUUID().toString().replace("-", "");
                ValueOperations<String, String> ops = stringRedisTemplate.opsForValue();

                List<ImportIndividualRankingFailureVO> collect = failVo1.stream().filter(importStaffFailureVo -> {
                    return !importStaffFailureVo.getFailureMsg().equals("");
                }).collect(Collectors.toList());

                String failureVOJson = JSON.toJSONString(collect);
                ops.set(exportCode, failureVOJson, 30 * 60, TimeUnit.SECONDS);
            }

            Integer numberOfSuccesses = 0;
            if (returnedSet != null) {
                //Collections.sort(returnedSet, Comparator.comparingInt(CompetitionRanking::getScore));
                //降序排列
                //多条件-先年龄升序、后分数升序
                /*Collections.sort(returnedSet, new Comparator<CompetitionRanking>() {
                    public int compare(CompetitionRanking s1, CompetitionRanking s2) {
                        int  i = (int)(Double.valueOf(s2.getScore()) -Double.valueOf(s1.getScore()));
                       // int i = s2.getScore().compareTo(s1.getScore());
                        if (i == 0) {
                            //i = (int)Double.valueOf(s1.getTimeSum())-(Double.valueOf(s2.getTimeSum()));
                            i = s1.getTimeSum() - s2.getTimeSum();
                        }
                        return i;
                    }
                });
*/


                returnedSet.sort(new Comparator<CompetitionRanking>() {
                    @Override
                    public int compare(CompetitionRanking o1, CompetitionRanking o2) {
                        if ((Double.valueOf(o2.getScore()) - Double.valueOf(o1.getScore())) > 0) {
                            return 1;
                        } else if (Double.valueOf(o1.getScore()) - Double.valueOf(o2.getScore()) == 0) {
                            if (Double.valueOf(o2.getTimeSum()) - Double.valueOf(o1.getTimeSum()) < 0) {
                                return 1;
                            }
                        }
                        return -1;
                    }


                });


                for (CompetitionRanking r : returnedSet) {
                    Integer ret = baseMapper.insert(r);
                    if (ret == 1) {
                        numberOfSuccesses++;
                    }
                }


            }
            map.put("exportCode", exportCode);//返回导出code
            map.put("successNum", numberOfSuccesses + "");//本次新增成功数量
            map.put("failureNum", countError);//本次新增失败数量
            map.put("data", returnedSet);//本次要上传确认的数据
            return map;
        } catch (RuntimeException e) {
            System.err.println(e.getMessage());
            throw new RuntimeException();
        }
    }


    /**
     * 判断字符串是否是long类型
     */
    public static boolean isLong(String value) {
        try {
            Long.valueOf(value);
            return true;
        } catch (NumberFormatException e) {
            return false;
        }
    }

    public static void main(String[] args) {
        System.out.println(isLong("12."));
    }


    @Override
    public void exportFailureRecord(HttpServletResponse response, String token, Integer competitionId) throws Exception {
        if (org.apache.commons.lang3.StringUtils.isEmpty(token)) {
            return;
        }
        ValueOperations<String, String> ops = stringRedisTemplate.opsForValue();
        //获取数据
        String record = ops.get(token);
        if (org.apache.commons.lang3.StringUtils.isEmpty(record)) {
            return;
        }

        // 告诉浏览器用什么软件可以打开此文件
        response.setHeader("content-Type", "application/vnd.ms-excel");
        // 下载文件的默认名称
        response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode("活动排名导入失败数据表", "UTF-8") + ".xls");
        //编码
        response.setCharacterEncoding("UTF-8");
        Workbook workbook = null;
        CompleteCompetitionSetup competitionSetup = competitionSetupMapper.getRaceSettingsByRaceId(competitionId);
        //活动类型（0为个人赛，1为团队赛）
        if (competitionSetup.getCompetitionType() == 1) {
            List<ImportCompetitionRankingFailureVO> parse = JSON.parseObject(record, new TypeReference<List<ImportCompetitionRankingFailureVO>>() {
            });
            parse.sort(Comparator.comparing(ImportCompetitionRankingFailureVO::getIndex));
            workbook = ExcelExportUtil.exportExcel(new ExportParams(), ImportCompetitionRankingFailureVO.class, parse);
        } else {
            List<ImportIndividualRankingFailureVO> parse = JSON.parseObject(record, new TypeReference<List<ImportIndividualRankingFailureVO>>() {
            });
            parse.sort(Comparator.comparing(ImportIndividualRankingFailureVO::getIndex));
            workbook = ExcelExportUtil.exportExcel(new ExportParams(), ImportIndividualRankingFailureVO.class, parse);
        }
        ExportParams exportParams = new ExportParams(null, "delivery", ExcelType.XSSF);
        exportParams.setStyle(ExcelStyleUtil.class);
        workbook.write(response.getOutputStream());
    }

    @Override
    public R frontOfficeCompetitionRanking(FrontDeskStageTeamScoreDetailsReq req) {
        List<ListOfStageResultsResp> frontOfficeCompetitionRanking = baseMapper.frontOfficeCompetitionRanking(req);
        return R.ok(frontOfficeCompetitionRanking);
    }


    /**
     * 前台活动阶段
     *
     * @param req
     * @return
     */
   /* @Override
    public List<ListOfStageResultsResp> frontStageCompetition(FrontStageCompetitionReq req) {
        //获取本活动下的阶段信息以及权重比
        List<CompetitionRuleResp> acquisitionWeightRatio = ruleMapper.acquisitionWeightRatio(req.getCompetitionId());
        //查看活动类型是个人赛还是团队活动


        List<ListOfStageResultsResp> newList = new ArrayList<>();
        List<ListOfStageResultsResp> list = new ArrayList<>();
        CompleteCompetitionSetup competitionSetup = competitionSetupMapper.getRaceSettingsByRaceId(req.getCompetitionId());
        for (CompetitionRuleResp ruleResp : acquisitionWeightRatio) {
            // 获取阶段信息
            CompetitionStage competitionStage = stageMapper.selectById(ruleResp.getStageId());
            FrontStageCompetitionResp frontStageCompetitionResp = new FrontStageCompetitionResp();
            frontStageCompetitionResp.setPointWeight(ruleResp.getPointWeight());
            //复制属性
            BeanUtils.copyProperties(competitionStage, frontStageCompetitionResp);
            //活动类型（0为个人赛，1为团队赛）默认为0
            //个人赛
            if (competitionSetup.getCompetitionType() == TeamCalculationMethodConstant.TYPE_OF_EVENT_INDIVIDUAL_COMPETITION) {

                //活动方式（0实操，1理论，2线下）
                if (competitionStage.getMethod() == 2) {
                    list = teamMapper.offlineIndividualPointsCompetition(new StageRankingReq().
                            setStageId(frontStageCompetitionResp.getStageId()).
                            setCompetitionId(req.getCompetitionId()));
                } else {
                    list = teamMapper.onlineIndividualPointsCompetition(new StageRankingReq().
                            setStageId(frontStageCompetitionResp.getStageId()).
                            setCompetitionId(req.getCompetitionId()));
                }

            }
            newList.addAll(list);
        }

        //个人赛
        List<ListOfStageResultsResp> individualCompetition = new ArrayList<>();
        if (competitionSetup.getCompetitionType()==1){

        }else{

            Map<String, List<ListOfStageResultsResp>> listMap = newList.stream().collect(Collectors.groupingBy(ListOfStageResultsResp::getAccountId));
            List<ListOfStageResultsResp> retList = new ArrayList<>();

            for (String accountId : listMap.keySet()) {
                System.out.println("key的值是：" + accountId + "=============" + "value的值是：" + listMap.get(accountId));
                List<ListOfStageResultsResp> resultsRespList = listMap.get(accountId);
                retList = merge(resultsRespList);
                individualCompetition.addAll(retList);
            }
            individualCompetition.sort(new Comparator<ListOfStageResultsResp>() {
                @Override
                public int compare(ListOfStageResultsResp o1, ListOfStageResultsResp o2) {
                    return (int) (Double.valueOf(o2.getScore()) - Double.valueOf(o1.getScore()));
                }
            });
        }
        return individualCompetition;

    }

    */

    /**
     * 将id进行合并nums, sums 相加道回合并后的集合使用Java8的流进行处理
     *//*

    public static List<ListOfStageResultsResp> merge(List<ListOfStageResultsResp> list) {
        List<ListOfStageResultsResp> result = list.stream()
                .collect(Collectors.toMap(ListOfStageResultsResp::getAccountId, a -> a, (o1, o2) -> {
                    Double score = Double.valueOf(o1.getScore()) + Double.valueOf(o2.getScore());
                    Integer time = Integer.valueOf(o1.getTimeSum()) + Integer.valueOf(o2.getTimeSum());
                    o1.setScore(score + "");
                    o1.setTimeSum(time + "");
                    o1.setTeamId(null);
                    return o1;
                })).values().stream().collect(Collectors.toList());

        return result;

    }*/
    @Override
    public R frontDeskOverallRanking(OverallStandingsInThePointsRaceReq req) {
        CompetitionRule competitionRule = ruleMapper.getTheRulesOfTheRaceByTheRaceId(req.getCompetitionId());
        List<OverallEventRankingResp> overallEventRankingRespList = new ArrayList<>();
        //查看活动类型是个人赛还是团队活动
        CompleteCompetitionSetup competitionSetup = competitionSetupMapper.getRaceSettingsByRaceId(req.getCompetitionId());
        for (Integer stageId : req.getStageIds()) {
            OverallEventRankingResp rankingResp = new OverallEventRankingResp();
            // 获取阶段信息
            CompetitionStage competitionStage = stageMapper.selectById(stageId);

            FrontDeskStageRankingReq frontDeskStageRankingReq = new FrontDeskStageRankingReq();
            frontDeskStageRankingReq.setCompetitionId(req.getCompetitionId());
            frontDeskStageRankingReq.setCompetitionStage(competitionStage);
            frontDeskStageRankingReq.setCompetitionSetup(competitionSetup);
            frontDeskStageRankingReq.setPageNum(req.getPageNum());
            frontDeskStageRankingReq.setPageSize(req.getPageSize());
            frontDeskStageRankingReq.setStageId(competitionStage.getStageId());
            //获取每个阶段的团队、个人成绩列表
            IPage<ListOfStageResultsResp> list = pointRace(frontDeskStageRankingReq);
            rankingResp.setStageName(competitionStage.getStageName());
            rankingResp.setStageId(competitionStage.getStageId());
            rankingResp.setStageTeamInformation(list.getRecords());
            rankingResp.setPointWeight(competitionStage.getPointWeight());
            overallEventRankingRespList.add(rankingResp);

        }

        //成绩计算方式（0：各阶段加权求和，1：各阶段成绩求和，2：各阶段成绩平均值
        String teamScore = "0";

        List<ListOfStageResultsResp> returnTotalSet = new ArrayList<>();//返回的集合

        for (OverallEventRankingResp rankingResp : overallEventRankingRespList) {
            double calculateScores = 0.0;
            CompetitionStageServiceImpl.evaluationAtEachStage(competitionRule, returnTotalSet, rankingResp);
        }

        Map<Integer, List<String>> map = new HashMap<>();
        for (OverallEventRankingResp item : overallEventRankingRespList) {
            for (ListOfStageResultsResp l : item.getStageTeamInformation()) {
                List<String> scoreList = new ArrayList<>();
                if (map.get(l.getTeamId()) != null) {
                    scoreList = map.get(l.getTeamId());
                }
                scoreList.add(l.getScore());
                map.put(l.getTeamId(), scoreList);
            }
        }
        List<ListOfStageResultsResp> stageTeamInformation = overallEventRankingRespList.get(overallEventRankingRespList.size() - 1).getStageTeamInformation();
        Set<Integer> set = map.keySet();
        for (ListOfStageResultsResp l : stageTeamInformation) {
            for (Integer str : set) {
                if (str == l.getTeamId()) {
                    List<String> in = map.get(str);
                    List<Double> dList = in.stream().map((d) -> {
                        return Double.valueOf(d + "");
                    }).collect(Collectors.toList());

                    CompetitionStageServiceImpl.ScoreProcessing(competitionRule, l, dList);
                }

            }
        }


        stageTeamInformation.sort(new Comparator<ListOfStageResultsResp>() {
            @Override
            public int compare(ListOfStageResultsResp o1, ListOfStageResultsResp o2) {
                return (int) (Double.valueOf(o2.getScore()) - Double.valueOf(o1.getScore()));
            }
        });

        //总数
        int total = stageTeamInformation.size();
        //总页数
        int pageSum = total % req.getPageSize() == 0 ? total / req.getPageSize() : total / req.getPageSize() + 1;

        //分页
        List<ListOfStageResultsResp> subList = stageTeamInformation.stream().skip((req.getPageNum() - 1) * req.getPageSize()).limit(req.getPageSize()).collect(Collectors.toList());
        return R.ok().put("page", subList).put("pageSum", pageSum).put("total", total);
    }

    @Override
    public R detailsOfTotalTeamScores(DetailsOfTotalTeamScoresReq req) {

        List<DetailsOfTotalTeamScoresResp> teamScoresRespList = new ArrayList<>();
        for (Integer stageId : req.getStageIds()) {
            DetailsOfTotalTeamScoresResp resp = new DetailsOfTotalTeamScoresResp();
            CompetitionStage competitionStage = stageMapper.selectById(stageId);
            resp.setStageName(competitionStage.getStageName());
            resp.setPointWeight(competitionStage.getPointWeight());
            resp.setTeamCalculationMethod(competitionStage.getTeamCalculationMethod());
            resp.setStageId(stageId);
            req.setStageId(stageId);
            List<ListOfStageResultsResp> information = stageMapper.stageTeamInformation(req);
            resp.setStageTeamInformation(information);
            teamScoresRespList.add(resp);
        }
        return R.ok().put("data", teamScoresRespList).put("resultCalculationMethod", ruleMapper.obtainTheRulesOfTheEvent(req.getCompetitionId()).getResultCalculationMethod());
    }

    @Override
    public R manuallyRankTheUploadList(BatchUploadListCompetitionReq req) {

        Page<ListOfStageResultsResp> page = new Page<ListOfStageResultsResp>(req.getPageNum(), req.getPageSize());
        IPage<ListOfStageResultsResp> pageList = baseMapper.manuallyRankTheUploadList(page, req);

        WhetherToPublishParameterReq parameterReq = new WhetherToPublishParameterReq();
        parameterReq.setIsRelease(DelConstant.POST_STATUS);
        parameterReq.setCompetitionId(req.getCompetitionId());
        parameterReq.setPublicationType(DelConstant.PUBLICATION_TYPE_MANUALOPERATION);
        parameterReq.setIsOverallRanking(req.getIsOverallRanking());
        if (req.getStageId() != null) {
            parameterReq.setStageId(req.getStageId());
        }


        //返回判断是否默认发布过
        List<CompetitionRanking> rankingList = baseMapper.whetherToPublishTheOverallScoreRanking(parameterReq);
        boolean publishStatus = false;
        if (rankingList.size() > 0) {
            //发布过
            publishStatus = true;
        }

        return R.ok(pageList).put("publishStatus", publishStatus).put("page",pageList.getRecords());
    }

    @Override
    public List<CompetitionRanking> whetherToPublish(Integer competitionId, Integer stageId, Integer isRelease, Integer publicationType, Integer isOverallRanking) {

        WhetherToPublishParameterReq parameterReq = new WhetherToPublishParameterReq();
        parameterReq.setIsRelease(isRelease);
        parameterReq.setCompetitionId(competitionId);
        parameterReq.setPublicationType(publicationType);
        parameterReq.setIsOverallRanking(isOverallRanking);
        parameterReq.setStageId(stageId);
        return baseMapper.whetherToPublish(parameterReq);
    }

    @Override
    public List<CompetitionRanking> whetherToPublishTheOverallScoreRanking(Integer competitionId, Integer isRelease, Integer publicationType, Integer isOverallRanking) {
        WhetherToPublishParameterReq parameterReq = new WhetherToPublishParameterReq();
        parameterReq.setIsRelease(isRelease);
        parameterReq.setCompetitionId(competitionId);
        parameterReq.setPublicationType(publicationType);
        parameterReq.setIsOverallRanking(isOverallRanking);
        return baseMapper.whetherToPublishTheOverallScoreRanking(parameterReq);
    }

    @Override
    public R publishRanking(List<CompetitionRanking> rankingList) {
        if (rankingList.size() <= 0) {
            return R.error("请选择要发布的数据!");
        }

        Integer publicationType = rankingList.get(0).getPublicationType();//publicationType 发布类型(0默认发布 1批量手动导入)
        Integer getCompetitionId = rankingList.get(0).getCompetitionId();
        Integer getIsOverallRanking = rankingList.get(0).getIsOverallRanking();//是否为总分排名(1为是 默认0不是)
        Integer getStageId = rankingList.get(0).getStageId();

        QueryWrapper<CompetitionRanking> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("competition_id", getCompetitionId);
        queryWrapper.eq("is_release", DelConstant.POST_STATUS);//已发布
        //如果为总分排名时,默认删除上一次发布的总分排名
        if (getIsOverallRanking == 1) {
            queryWrapper.eq("is_overall_ranking", getIsOverallRanking);
        } else {
            //如果为阶段排名时候，则删除上一次的阶段排名
            if (publicationType == 0) {
                queryWrapper.eq("publication_type", 1);
            } else {
                queryWrapper.eq("publication_type", 0);
            }
            queryWrapper.eq("stage_id", getStageId);
        }
        baseMapper.delete(queryWrapper);
        for (CompetitionRanking ranking : rankingList) {
            ranking.setIsRelease(DelConstant.POST_STATUS);
            baseMapper.insert(ranking);
        }
        return R.ok();
    }

    public IPage<ListOfStageResultsResp> pointRace(FrontDeskStageRankingReq req) {
        IPage<ListOfStageResultsResp> pageList = null;
        //活动类型（0为个人赛，1为团队赛）默认为0
        if (req.getCompetitionSetup().getCompetitionType() == TeamCalculationMethodConstant.TYPE_OF_EVENT_INDIVIDUAL_COMPETITION) {//个人赛
            Page<ListOfStageResultsResp> page = new Page<ListOfStageResultsResp>(req.getPageNum(), req.getPageSize());
            pageList = teamMapper.frontDeskIndividualStageRanking(page, req);
            return pageList;
        } else if (req.getCompetitionSetup().getCompetitionType() == TeamCalculationMethodConstant.TYPE_OF_EVENT_TEAM_EVENT) {//团队赛
            //团队排名：用时以及分数依据当前阶段成绩的计算方式(最高分、平均分、求和)取
            //如：当前阶段活动成绩计算方式为取最高分，则用户用时以及分数取得是当前团队下的最高分当前成绩的用时以及分数
            //团队成绩计算方式（0：最高分，1：平均分，2：求和）

            //1.获取参与该活动的团队
            String teamIds = teamMapper.getTeamInformation(req.getCompetitionId());
            if (teamIds != null || teamIds.length() >= 0) {
                req.setTeamIds(teamIds);
                Page<ListOfStageResultsResp> page = new Page<ListOfStageResultsResp>(req.getPageNum(), req.getPageSize());
                pageList = teamMapper.frontOfficeTeamStageRanking(page, req);
                for (ListOfStageResultsResp resp : pageList.getRecords()) {
                    //团队成绩计算方式（0：最高分，1：平均分，2：总分）
                    if (req.getCompetitionStage().getTeamCalculationMethod().equals("0")) {
                        resp.setScore(resp.getMaxScore());
                    } else if (req.getCompetitionStage().getTeamCalculationMethod().equals("1")) {
                        resp.setScore(resp.getAvgScore());
                    } else if (req.getCompetitionStage().getTeamCalculationMethod().equals("2")) {
                        resp.setScore(resp.getSumScore());
                    }
                }

                return pageList;


            }
        }
        return null;
    }
}
