package com.huoran.competition.service.impl;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import cn.afterturn.easypoi.excel.entity.enmus.ExcelType;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huoran.api.UserClient;
import com.huoran.common.entity.School;
import com.huoran.common.exception.CustomException;
import com.huoran.common.exception.ExceptionEnum;
import com.huoran.common.response.R;
import com.huoran.competition.entity.Competition;
import com.huoran.competition.entity.CompetitionRegistration;
import com.huoran.competition.entity.req.PageRegistrationStaffReq;
import com.huoran.competition.entity.vo.RegistrationNewInformationBroughtOutVO;
import com.huoran.competition.entity.vo.RegistrationVO;
import com.huoran.competition.mapper.*;
import com.huoran.competition.service.CompetitionRangeService;
import com.huoran.competition.service.CompetitionRegistrationService;
import com.huoran.competition.service.CompetitionTeamService;
import com.huoran.competition.service.TeamParticipantService;
import com.huoran.competition.utils.ExcelStyleUtil;
import com.huoran.competition.utils.HandleNameAsteriskReplacementUtil;
import org.apache.poi.ss.usermodel.Workbook;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ObjectUtils;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLEncoder;
import java.util.List;

/**
 * <p>
 * 活动报名人员表 服务实现类
 * </p>
 *
 * @author cheney
 * @since 2022-11-16
 */
@Service
public class CompetitionRegistrationServiceImpl extends ServiceImpl<CompetitionRegistrationMapper, CompetitionRegistration> implements CompetitionRegistrationService {

    @Autowired
    private UserClient userClient;

    @Autowired
    CompetitionMapper competitionMapper;

    @Autowired
    CompetitionTeamMapper competitionTeamMapper;

    @Autowired
    CompleteCompetitionSetupMapper completeCompetitionSetupMapper;

    @Autowired
    CompetitionStageMapper competitionStageMapper;

    @Autowired
    CompetitionRangeService competitionRangeService;

    @Autowired
    private TeamParticipantService teamParticipantService;
    @Autowired
    private CompetitionTeamService competitionTeamService;

    @Override
    public School getSchool(Integer schoolId) {
        return baseMapper.selectSchool(schoolId);
    }

    @Override
    public R registrationPageConditionQuery(PageRegistrationStaffReq req) {


        Page<RegistrationVO> page = new Page<RegistrationVO>(req.getPageNum(), req.getPageSize());
        IPage<RegistrationVO> pageList = baseMapper.selectRegistrationByCondition(page, req);

        //默认设置后台

        if (ObjectUtils.isEmpty(req.getQuerySource())) {
            req.setQuerySource(2);
        }
        if (req.getQuerySource() == 1) {
            pageList.getRecords().forEach(registrationVO -> {
                registrationVO.setUsername(HandleNameAsteriskReplacementUtil.desensitizedName(registrationVO.getUsername()));
            });
        }

        return R.ok().put("data", pageList);
    }

    @Override
    public boolean disableContests(List<String> ids) {
        int updates = baseMapper.disableByIds(ids);
        return updates > 0;
    }

    @Override
    @Transactional
    public R cancelRegistration(String contestId, Integer accountId) {
        //删除团队
        /*QueryWrapper<CompleteCompetitionSetup> setupQueryWrapper = new QueryWrapper<>();
        setupQueryWrapper.eq("competition_id", contestId);
        CompleteCompetitionSetup setup = completeCompetitionSetupMapper.selectOne(setupQueryWrapper);
        //团队赛类型
        if (setup.getCompetitionType() == 1) {
            QueryWrapper<CompetitionTeam> teamQueryWrapper = new QueryWrapper<>();
            teamQueryWrapper.eq("account_id", accountId);
            teamQueryWrapper.eq("competition_id", contestId);
            CompetitionTeam one = competitionTeamService.getOne(teamQueryWrapper);
            if (one != null) {
                if (one.getCaptain() == 0) {
                    QueryWrapper<CompetitionTeam> competitionTeamQueryWrapper = new QueryWrapper<>();
                    competitionTeamQueryWrapper.eq("pid", one.getTeamId());
                    competitionTeamQueryWrapper.eq("competition_id", contestId);
                    List<CompetitionTeam> list = competitionTeamService.list(competitionTeamQueryWrapper);
                    if (list != null && list.size() > 0) {
                        return R.error("取消失败,需要把成员移除后取消!");
                    }
                    competitionTeamService.removeById(one);
                }
                competitionTeamService.removeById(one);
            }
        }*/
        boolean ret = baseMapper.cancelRegistration(contestId, accountId);

        return ret ? R.ok() : R.error();
    }

    @Override
    public void export(HttpServletResponse response, PageRegistrationStaffReq req) throws IOException {


        Page<RegistrationVO> page = new Page<RegistrationVO>(req.getPageNum(), req.getPageSize());
        IPage<RegistrationVO> pageList = baseMapper.selectRegistrationByCondition(page, req);


        for (RegistrationVO vo : pageList.getRecords()) {
            //组织类型/平台来源(5:幼儿园，6：供应商)
            switch (vo.getPlatformId()) {
                case "5":
                    vo.setPlatformId("幼儿园");
                    break;
                case "6":
                    vo.setPlatformId("供应商");
                    break;
            }


            //签到状态(0.待签到 1.已签到 2.未签到:活动结束后没签到的)
            switch (vo.getSignInStatus()) {
                case "0":
                    vo.setSignInStatus("待签到");
                    break;
                case "1":
                    vo.setSignInStatus("已签到");
                    break;
                case "2":
                    vo.setSignInStatus("未签到");
                    break;
            }

        }

        // 告诉浏览器用什么软件可以打开此文件
        response.setHeader("content-Type", "application/vnd.ms-excel");
        // 下载文件的默认名称
        response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode("报名人员导出", "UTF-8") + ".xls");
        //编码
        response.setCharacterEncoding("UTF-8");

        Workbook workbook = ExcelExportUtil.exportExcel(new ExportParams(), RegistrationVO.class, pageList.getRecords());

        ExportParams exportParams = new ExportParams(null, "delivery", ExcelType.XSSF);
        exportParams.setStyle(ExcelStyleUtil.class);
        workbook.write(response.getOutputStream());
    }

    @Override
    public void exportDataInBatches(List<RegistrationVO> listOfExportSub, HttpServletResponse response) throws IOException {

        if (!listOfExportSub.isEmpty()) {
            for (RegistrationVO vo : listOfExportSub) {
                //组织类型/平台来源(5:幼儿园，6：供应商)
                switch (vo.getPlatformId()) {
                    case "5":
                        vo.setPlatformId("幼儿园");
                        break;
                    case "6":
                        vo.setPlatformId("供应商");
                        break;
                }


                //签到状态(0.待签到 1.已签到 2.未签到:活动结束后没签到的)
                switch (vo.getSignInStatus()) {
                    case "0":
                        vo.setSignInStatus("待签到");
                        break;
                    case "1":
                        vo.setSignInStatus("已签到");
                        break;
                    case "2":
                        vo.setSignInStatus("未签到");
                        break;
                }
            }
        }

        // 告诉浏览器用什么软件可以打开此文件
        response.setHeader("content-Type", "application/vnd.ms-excel");
        // 下载文件的默认名称
        response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode("报名人员导出", "UTF-8") + ".xls");
        //编码
        response.setCharacterEncoding("UTF-8");
        Workbook workbook = ExcelExportUtil.exportExcel(new ExportParams(), RegistrationVO.class, listOfExportSub);
        ExportParams exportParams = new ExportParams(null, "delivery", ExcelType.XSSF);
        exportParams.setStyle(ExcelStyleUtil.class);
        workbook.write(response.getOutputStream());
    }

    @Override
    @Transactional
    public R apply(CompetitionRegistration competitionRegistration) {

        Competition competition = competitionMapper.selectById(competitionRegistration.getCompetitionId());


        //个人赛人数限制
        if (competition.getQuantityLimit() != null) {
            QueryWrapper<CompetitionRegistration> wrapper = new QueryWrapper<>();
            wrapper.eq("competition_id", competitionRegistration.getCompetitionId());
            List<CompetitionRegistration> competitionRegistrations = baseMapper.selectList(wrapper);
            if (competitionRegistrations != null && competitionRegistrations.size() >= competition.getQuantityLimit()) {
                throw new CustomException(ExceptionEnum.TEAM_LIMIT3);
            }
        }

        //内部添加，一路绿灯
        QueryWrapper<CompetitionRegistration> wrapper = getCompetitionRegistrationQueryWrapper(competitionRegistration, competition);
        int count = baseMapper.selectCount(wrapper);
        //添加前判断是否已报名
        if (count > 0) {
            throw new CustomException(ExceptionEnum.HAVE_BEEN_REGISTERED);
        } else {
            int save = baseMapper.insert(competitionRegistration);
            return save > 0 ? R.ok() : R.error();
        }
    }

    @NotNull
    private static QueryWrapper<CompetitionRegistration> getCompetitionRegistrationQueryWrapper(CompetitionRegistration competitionRegistration, Competition competition) {
        if (competitionRegistration.getIdentification() == null || competitionRegistration.getIdentification() != 1) {
            //是否需要报名邀请码
            Integer code = competition.getIsNeedCode();
            if (code == 1) {
                //验证报名邀请码
                if (!competition.getInvitationCode().equals(competitionRegistration.getRegistrationInvitationCode())) {
                    throw new CustomException(ExceptionEnum.CODE);
                }
            }
        }

        Integer accountId = competitionRegistration.getAccountId();
        Integer competitionId = competitionRegistration.getCompetitionId();

        QueryWrapper<CompetitionRegistration> wrapper = new QueryWrapper<>();
        wrapper.eq("account_id", accountId);
        wrapper.eq("competition_id", competitionId);
        return wrapper;
    }

    @Override
    public CompetitionRegistration enquireWhetherToRegister(Integer accountId, String competitionId) {
        QueryWrapper<CompetitionRegistration> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("competition_id", competitionId);
        queryWrapper.eq("account_id", accountId);
        CompetitionRegistration registration = baseMapper.selectOne(queryWrapper);
        return registration;
    }

    @Override
    public Integer registrationPopulation(String competitionId) {
      /*  QueryWrapper<CompetitionRegistration> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("competition_id", competitionId);
        List<CompetitionRegistration> competitionRegistrations = baseMapper.selectList(queryWrapper);*/
//        return competitionRegistrations.size();
        return baseMapper.countTheNumberOfApplicants(Integer.valueOf(competitionId));
    }

    @Override
    public R getOrganizationalInformation(String phone, Integer accountId) {
        List<RegistrationNewInformationBroughtOutVO> voList=baseMapper.getOrganizationalInformation(phone, accountId);
        if (!ObjectUtils.isEmpty(phone)){
            if (voList.size() == 0){
                return R.error("当前用户尚未注册，无法报名活动!");
            }
        }

        return R.ok().put("data",voList );
    }

    @Override
    public R detailsOfApplicants(Integer accountId, String competitionId) {
        RegistrationNewInformationBroughtOutVO informationBroughtOutVO = baseMapper.detailsOfApplicants(accountId, competitionId);
        return R.ok().put("data", informationBroughtOutVO);
    }
}
