package com.huoran.competition.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huoran.common.response.R;
import com.huoran.competition.entity.*;
import com.huoran.competition.mapper.*;
import com.huoran.competition.service.CompetitionRuleService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * <p>
 * 活动规则表 服务实现类
 * </p>
 *
 * @author cheney
 * @since 2022-11-16
 */
@Service
public class CompetitionRuleServiceImpl extends ServiceImpl<CompetitionRuleMapper, CompetitionRule> implements CompetitionRuleService {

    @Autowired
    CompetitionStageMapper competitionStageMapper;

    @Autowired
    CompleteCompetitionSetupMapper completeCompetitionSetupMapper;

    @Autowired
    TeamParticipantMapper teamParticipantMapper;

    @Autowired
    CompetitionContentMapper competitionContentMapper;

    @Autowired
    CompetitionStageContentSettingMapper competitionStageContentSettingMapper;

    @Autowired
    CompetitionStageFileMapper competitionStageFileMapper;

    @Autowired
    CompetitionRankingMapper competitionRankingMapper;

    @Autowired
    CompetitionReleaseTimeMapper competitionReleaseTimeMapper;

    @Override
    public R addCompetitionRule(CompetitionRule competitionRule) {
//        添加活动规则
        int insert = baseMapper.insert(competitionRule);
        Integer ruleId = competitionRule.getRuleId();
        if (!competitionRule.getCompetitionStageList().isEmpty()){
            List<CompetitionStage> competitionStageList = competitionRule.getCompetitionStageList();
            competitionStageList.forEach(competitionStage -> {
                competitionStage.setRuleId(ruleId);
                competitionStageMapper.insert(competitionStage);
            });
        }
        return insert>0 ? R.ok(): R.error();
    }

    @Override
    public R getCompetitionRule(String competitionId) {
        QueryWrapper<CompleteCompetitionSetup> setupQueryWrapper = new QueryWrapper<>();
        setupQueryWrapper.eq("competition_id",competitionId);
        CompleteCompetitionSetup setup = completeCompetitionSetupMapper.selectOne(setupQueryWrapper);

        CompetitionRule competitionRule = null;
        if (ObjectUtil.isNotNull(setup)){
            QueryWrapper<CompetitionRule> ruleQueryWrapper = new QueryWrapper<>();
            ruleQueryWrapper.eq("setup_id",setup.getSetupId());
            competitionRule = baseMapper.selectOne(ruleQueryWrapper);
            if (ObjectUtil.isNotNull(competitionRule)){
                QueryWrapper<CompetitionStage> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("rule_id",competitionRule.getRuleId());
                List<CompetitionStage> competitionStages = competitionStageMapper.selectList(queryWrapper);
                competitionRule.setCompetitionStageList(competitionStages);
            }
        }

        return R.ok().put("competitionRule",competitionRule);
    }

    @Override
    public R editCompetitionRule(CompetitionRule competitionRule) {
        // 更新活动规则
        int update = baseMapper.updateById(competitionRule);
//        更新阶段数据
        List<CompetitionStage> updatedStages  = competitionRule.getCompetitionStageList();

        //查询当前活动的所有阶段
        List<CompetitionStage> existingStages = competitionStageMapper.selectList(
                new QueryWrapper<CompetitionStage>().eq("rule_id",competitionRule.getRuleId())
        );
        // 新增或更新阶段
        updatedStages .forEach(competitionStage -> {
            if (competitionStage.getStageId()!=null){
                //存在进行编辑
                competitionStageMapper.updateById(competitionStage);
            }else {
                //新增添加阶段
                competitionStageMapper.insert(competitionStage);
            }
        });

        // 删除阶段及相关联数据
        for (CompetitionStage existingStage : existingStages) {
            boolean exists = updatedStages.stream().anyMatch(s -> s.getStageId().equals(existingStage.getStageId()));
            if (!exists) {
                // 如果更新的阶段数据中没有该阶段，则删除
                competitionStageMapper.deleteById(existingStage.getStageId());
                //删除阶段内容数据c_competition_content
                competitionContentMapper.delete(
                        new QueryWrapper<CompetitionContent>().eq("stage_id",existingStage.getStageId()));
                //删除活动阶段内容设置c_competition_stage_content_setting
                competitionStageContentSettingMapper.delete(
                        new QueryWrapper<CompetitionStageContentSetting>().eq("stage_id",existingStage.getStageId()));
                //活动阶段文件c_competition_stage_file
                competitionStageFileMapper.delete(
                        new QueryWrapper<CompetitionStageFile>().eq("stage_id",existingStage.getStageId()));
                //活动排名表c_competition_ranking
                competitionRankingMapper.delete(
                        new QueryWrapper<CompetitionRanking>().eq("stage_id",existingStage.getStageId()));
                //活动排名定时发布表c_competition_release_time
                competitionReleaseTimeMapper.delete(
                        new QueryWrapper<CompetitionReleaseTime>().eq("stage_id",existingStage.getStageId()));
                //团队阶段参赛人员表c_team_participant
                teamParticipantMapper.delete(
                        new QueryWrapper<TeamParticipant>().eq("stage_id",existingStage.getStageId()));
            }
        }

        return update>0 ? R.ok(): R.error();
    }
}
