package com.huoran.competition.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huoran.common.response.R;
import com.huoran.competition.entity.*;
import com.huoran.competition.entity.req.*;
import com.huoran.competition.entity.resp.ListOfStageResultsResp;
import com.huoran.competition.entity.vo.CompetitionVO;
import com.huoran.competition.mapper.*;
import com.huoran.competition.service.CompetitionService;
import com.huoran.competition.service.CompetitionStageFileService;
import com.huoran.competition.service.TeamParticipantService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.AsyncResult;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;

import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import javax.script.ScriptException;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.Future;
import java.util.stream.Collectors;

/**
 * <p>
 * 活动信息表 服务实现类
 * </p>
 *
 * @author cheney
 * @since 2022-11-16
 */
@Service
public class CompetitionServiceImpl extends ServiceImpl<CompetitionMapper, Competition> implements CompetitionService {

    @Autowired
    CompetitionStageContentSettingMapper competitionStageContentSettingMapper;
    @Autowired
    CompetitionRegistrationMapper competitionRegistrationMapper;

    @Autowired
    CompetitionProgressMapper competitionProgressMapper;

    @Autowired
    CompetitionRangeMapper competitionRangeMapper;

    @Autowired
    CompetitionAnnexMapper competitionAnnexMapper;

    @Autowired
    CompetitionRuleMapper competitionRuleMapper;

    @Autowired
    CompleteCompetitionSetupMapper completeCompetitionSetupMapper;

    @Autowired
    CompetitionStageMapper competitionStageMapper;

    @Autowired
    CompetitionRankingServiceImpl competitionStageService;

    @Autowired
    CompetitionTeamMapper competitionTeamMapper;

    @Autowired
    CompetitionContentMapper competitionContentMapper;

    @Autowired
    CompetitionContentServiceImpl competitionContentService;

    @Autowired
    CompetitionStageFileService stageFileService;

    @Autowired
    TeamParticipantService teamParticipantService;

    @Autowired
    ThreadPoolTaskExecutor competitionExecutor;

    @Override
    public boolean repeat(Competition competition) {
        QueryWrapper<Competition> wrapper = new QueryWrapper<>();
        wrapper.eq("name", competition.getName());
        //活动范围(0:本校内 1:全平台 2:指定区域、院校)
        /*switch (competition.getCompetitionScope()) {
            case 0:
                wrapper.eq("school_id", competition.getSchoolId());
                break;
        }*/
        if (competition.getId() != null) {
            wrapper.last(" and id !=  " + competition.getId());
        }
        Integer count = baseMapper.selectCount(wrapper);
        return count > 0;
    }

    @Override
    public boolean removeCompetition(Integer competitionId) {
        Competition competition = baseMapper.selectById(competitionId);

        // 删除该活动报名人员数
        QueryWrapper<CompetitionRegistration> applicantWrapper = new QueryWrapper<>();
        applicantWrapper.eq("competition_id", competitionId);
        applicantWrapper.select("id");
        List<CompetitionRegistration> applicants = competitionRegistrationMapper.selectList(applicantWrapper);
        if (applicants != null && applicants.size() != 0) {
            List<Integer> applicantIds = applicants.stream().map(CompetitionRegistration::getId).collect(Collectors.toList());
            competitionRegistrationMapper.deleteBatchIds(applicantIds);
        }
        // 删除活动进展
        QueryWrapper<CompetitionProgress> contestProgressWrapper = new QueryWrapper<>();
        contestProgressWrapper.eq("competition_id", competitionId);
        contestProgressWrapper.select("id");
        List<CompetitionProgress> contestProgressList = competitionProgressMapper.selectList(contestProgressWrapper);
        if (contestProgressList != null && contestProgressList.size() != 0) {
            List<Integer> contestProgressIds = contestProgressList.stream().map(CompetitionProgress::getId).collect(Collectors.toList());
            competitionProgressMapper.deleteBatchIds(contestProgressIds);
        }

        if (competition.getReleaseType() == 1) {
            //删除完整活动其他信息
            QueryWrapper<CompleteCompetitionSetup> setupQueryWrapper = new QueryWrapper<>();
            setupQueryWrapper.eq("competition_id", competitionId);
            CompleteCompetitionSetup completeCompetitionSetup = completeCompetitionSetupMapper.selectOne(setupQueryWrapper);
            completeCompetitionSetupMapper.delete(setupQueryWrapper);

            if (ObjectUtil.isNotNull(completeCompetitionSetup)) {
                QueryWrapper<CompetitionRule> ruleQueryWrapper = new QueryWrapper<>();
                ruleQueryWrapper.eq("setup_id", completeCompetitionSetup.getSetupId());
                CompetitionRule competitionRule = competitionRuleMapper.selectOne(ruleQueryWrapper);
                competitionRuleMapper.delete(ruleQueryWrapper);

                if (ObjectUtil.isNotNull(competitionRule)) {
                    QueryWrapper<CompetitionStage> stageQueryWrapper = new QueryWrapper<>();
                    stageQueryWrapper.eq("rule_id", competitionRule.getRuleId());
                    List<CompetitionStage> competitionStages = competitionStageMapper.selectList(stageQueryWrapper);
                    if (ObjectUtil.isNotNull(competitionStages)) {
                        competitionStages.forEach(competitionStage -> {
                            QueryWrapper<CompetitionContent> competitionContentQueryWrapper = new QueryWrapper<>();
                            competitionContentQueryWrapper.eq("stage_id", competitionStage.getStageId());
                            competitionContentMapper.delete(competitionContentQueryWrapper);
                        });
                    }
                    competitionStageMapper.delete(stageQueryWrapper);
                }
            }
        }

        // 删除活动
        int delete = baseMapper.deleteById(competitionId);
        return delete > 0;
    }

    @Override
    public R competitionPageConditionQueryByOccupationlab(PageCompetitionListReq pageCompetitionListReq) {
        Page<CompetitionVO> page = new Page<>(pageCompetitionListReq.getPageNum(), pageCompetitionListReq.getPageSize());
        IPage<CompetitionVO> pageList = baseMapper.contestPageConditionQueryByOccupationlab(page, pageCompetitionListReq);
        return getR(pageList);
    }

    @Override
    public R backgroundActivityList(PageCompetitionListReq pageCompetitionListReq) {

        Page<CompetitionVO> page = new Page<>(pageCompetitionListReq.getPageNum(), pageCompetitionListReq.getPageSize());
        IPage<CompetitionVO> pageList = baseMapper.backgroundActivityList(page, pageCompetitionListReq);
        pageList.getRecords().forEach(c -> {
            if (c.getCompetitionScope().equals("1")) {
                c.setCompetitionScope("全平台");
            } else {
                c.setCompetitionScope(baseMapper.obtainPlatformInformation(c.getCompetitionScope()));
            }


        });
        return getR(pageList);
    }

    @Override
    @Async(value = "competitionExecutor")
    public Future<IPage<CompetitionVO>> notLoggedInBeforeStudentEvents(PageCompetitionListReq pageCompetitionListReq) {
        Page<CompetitionVO> page = new Page<>(pageCompetitionListReq.getPageNum(), pageCompetitionListReq.getPageSize());
        IPage<CompetitionVO> pageList = null;
        //活动范围(0:本校内 1:全平台 2.指定区域、院校) 3表示不限
        if ("2".equals(pageCompetitionListReq.getCompetitionScope())) {
            pageList = baseMapper.designatedRangeCompetitionByStu(page, pageCompetitionListReq);
        } else {
            pageList = baseMapper.allContestsByStu(page, pageCompetitionListReq);
        }
        Integer accountId = 0;
        if (pageCompetitionListReq.getAccountId() != null) {
            accountId = pageCompetitionListReq.getAccountId();
        }
        for (CompetitionVO contest : pageList.getRecords()) {
           /* if (contest.getCompetitionScope() == 1) {
                contest.setRange("全平台");
            } else if (contest.getCompetitionScope() == 2) {
                contest.setRange("指定范围");
            } else if (contest.getCompetitionScope() == 0) {
                contest.setRange("本校内");
            }*/
            //查询活动阶段数据
            List<CompetitionStage> stages = baseMapper.selectCompetitionStageNew(contest.getRuleId(), accountId);
            contest.setCompetitionStageList(stages);
        }

        return new AsyncResult<>(pageList);
    }


//    @Override
//    public R competitionAfterLogin1(AfterLoginPageCompetitionListReq pageCompetitionListReq) {
//        Page<CompetitionVO> page = new Page<>(pageCompetitionListReq.getPageNum(), pageCompetitionListReq.getPageSize());
//        IPage<CompetitionVO> pageList = null;
//        //活动类型(1.本校活动/2.活动广场/3.已报名活动)
//        switch (pageCompetitionListReq.getEventType()) {
//            case 1:
//                pageList = baseMapper.schoolCompetition(page, pageCompetitionListReq);
//                break;
//            case 2:
//                if (pageCompetitionListReq.getCompetitionScope().equals("3")) {
//                    pageCompetitionListReq.setCompetitionScope(null);
//                }
//                pageList = baseMapper.tournamentSquare(page, pageCompetitionListReq);
//                break;
//            case 3:
//
//                switch (pageCompetitionListReq.getCompetitionScope()) {//活动范围(0:本校内 1:全平台 2.指定区域、院校 3.不限)
//                    //已报名活动——修改为依据学生报名时间排序
//                    case "0":
//                    case "1":
//                    case "3":
//                        if (pageCompetitionListReq.getCompetitionScope().equals("3")) {
//                            pageCompetitionListReq.setCompetitionScope(null);
//                        }
//                        pageList = baseMapper.registeredBySchoolAndAllPlatforms(page, pageCompetitionListReq);
//                        break;
//                    case "2":
//                        pageList = baseMapper.registeredBySpecifiedRange(page, pageCompetitionListReq);
//                        break;
//                }
//                break;
//        }
//
//        return getR(pageList, pageCompetitionListReq.getAccountId());
//    }


    @Override
    @Async
    public Future<IPage<CompetitionVO>> competitionAfterLogin(AfterLoginPageCompetitionListReq pageCompetitionListReq) {
        return CompletableFuture.supplyAsync(() -> {
            Page<CompetitionVO> page = new Page<>(pageCompetitionListReq.getPageNum(), pageCompetitionListReq.getPageSize());
            IPage<CompetitionVO> pageList = null;
            //读取的列表(0:首页活动/1.已报名活动)
            switch (pageCompetitionListReq.getListType()) {
                case 0:
                    //查询未删除且启用发布状态为发布后的
                    pageList = baseMapper.foregroundAppletActivityList(page, pageCompetitionListReq);
                    break;
                case 1:
                    //查询我报名的列表
                    pageList = baseMapper.postRegistrationList(page, pageCompetitionListReq);
                    break;

            }


            //活动类型(1.本校活动/2.活动广场/3.已报名活动)
           /* switch (pageCompetitionListReq.getEventType()) {
                case 1:
                    pageList = baseMapper.schoolCompetition(page, pageCompetitionListReq);
                    break;
                case 2:
                    if (pageCompetitionListReq.getCompetitionScope().equals("3")) {
                        pageCompetitionListReq.setCompetitionScope(null);
                    }
                    pageList = baseMapper.tournamentSquareNew(page, pageCompetitionListReq);
                    break;
                case 3:
                    //活动范围(0:本校内 1:全平台 2.指定区域、院校 3.不限)
                    //已报名活动——修改为依据学生报名时间排序
                    if ("2".equals(pageCompetitionListReq.getCompetitionScope())) {
                        pageList = baseMapper.registeredBySpecifiedRange(page, pageCompetitionListReq);
                    } else {
                        if (pageCompetitionListReq.getCompetitionScope().equals("3")) {
                            pageCompetitionListReq.setCompetitionScope(null);
                        }
                        pageList = baseMapper.registeredBySchoolAndAllPlatforms(page, pageCompetitionListReq);
                    }
                break;
            }*/

            assert pageList != null;
            /*pageList.getRecords().forEach(contest -> {
                if (contest.getCompetitionScope() == 1) {
                    contest.setRange("全平台");
                } else if (contest.getCompetitionScope() == 2) {
                    contest.setRange("指定范围");
                } else if (contest.getCompetitionScope() == 0) {
                    contest.setRange("本校内");
                }
                //查询活动阶段数据
                List<CompetitionStage> stages = baseMapper.selectCompetitionStageNew(contest.getRuleId(), pageCompetitionListReq.getAccountId());
                contest.setCompetitionStageList(stages);
            });*/
            return pageList;
        }, competitionExecutor);
    }

    private R getR(IPage<CompetitionVO> pageList) {
        pageList.getRecords().forEach(contest -> {
           /* if (contest.getCompetitionScope() == 1) {
                contest.setRange("全平台");
            } else if (contest.getCompetitionScope() == 2) {
                contest.setRange("指定范围");
                //依据活动id查询指定范围的省、市、学校
                contest.setContestRangeRespList(competitionRangeMapper.getRangeByContestId(contest.getId()));
            } else if (contest.getCompetitionScope() == 0) {
                contest.setRange("本校内");
            }*/
            //查询活动阶段数据
            List<CompetitionStage> stages = baseMapper.selectCompetitionStage(contest.getId(), 0);

            R competitionContent = competitionContentService.getCompetitionContent(contest.getId());
            List<CompetitionContent> contentList = (List<CompetitionContent>) competitionContent.get("contentList");

            contest.setCompetitionStageList(stages);
            contest.setContentList(contentList);

            if (contentList.size() > 0) {

                for (CompetitionStage stage : stages) {
                    QueryWrapper<CompetitionStageContentSetting> queryWrapper1 = new QueryWrapper<>();
                    queryWrapper1.eq("competition_id", contest.getId());
                    queryWrapper1.eq("stage_id", stage.getStageId());
                    CompetitionStageContentSetting stageContentSetting = competitionStageContentSettingMapper.selectOne(queryWrapper1);
                    if (!ObjectUtil.isEmpty(stageContentSetting)) {
                        //赋值线下内容设置
                        stage.setCompetitionStageContentSetting(stageContentSetting);

                        QueryWrapper<CompetitionStageFile> queryWrapper2 = new QueryWrapper<>();
                        queryWrapper2.eq("competition_id", contest.getId());
                        queryWrapper2.eq("stage_id", stage.getStageId());
                        queryWrapper2.eq("account_id", 0);
                        CompetitionStageFile myFile = stageFileService.getOne(queryWrapper2);
                        if (!ObjectUtil.isEmpty(myFile)) {
                            stage.setCompetitionStageFile(myFile);
                        }

                    }

                }

            }

            //查询完整活动设置
            QueryWrapper<CompleteCompetitionSetup> wrapper = new QueryWrapper<>();
            wrapper.eq("competition_id", contest.getId());
            CompleteCompetitionSetup setup = completeCompetitionSetupMapper.selectOne(wrapper);
            if (setup != null) {
                setup.setInvitationCode(null);
                contest.setSetup(setup);
                //查询用户当前活动的团队id
                if (setup.getCompetitionType() == 1) {
                    QueryWrapper<CompetitionTeam> teamQueryWrapper = new QueryWrapper<>();
                    teamQueryWrapper.eq("competition_id", contest.getId());
                    teamQueryWrapper.eq("account_id", 0);
                    CompetitionTeam team = competitionTeamMapper.selectOne(teamQueryWrapper);
                    if (team != null) {
                        if (team.getCaptain() == 0) {
                            contest.setTeamId(team.getTeamId());
                        } else {
                            contest.setTeamId(team.getPid());
                        }
                    }
                }
                //查询赛制
                QueryWrapper<CompetitionRule> ruleQueryWrapper = new QueryWrapper<>();
                ruleQueryWrapper.eq("setup_id", setup.getSetupId());
                CompetitionRule competitionRule = competitionRuleMapper.selectOne(ruleQueryWrapper);
                if (competitionRule != null) {
                    contest.setRule(competitionRule.getRule());
                }
            }
        });
        return R.ok().put("data", pageList);
    }

    @Override
    public R addCompetition(Competition competition) {
        boolean isRepeat = repeat(competition);
        if (isRepeat) {
            return R.error("活动已经存在");
        }

        int insert = baseMapper.insert(competition);

        if (competition.getCompetitionAnnexList().size() > 0) {
            competition.getCompetitionAnnexList().forEach(competitionAnnex -> {
                competitionAnnex.setCompetitionId(competition.getId());
                competitionAnnexMapper.insert(competitionAnnex); // Save competition annex
            });
        }

        /*if (competition.getCompetitionRangeList().size() > 0) {
            competition.getCompetitionRangeList().forEach(competitionRange -> {
                Integer getCompetitionId = competition.getId();
                competitionRange.setCompetitionId(getCompetitionId);
                competitionRangeMapper.insert(competitionRange); // Save activity range
            });
        }*/

        Integer setupId = 0;
        return insert > 0 ? R.ok().put("setupId", setupId).put("competitionId", competition.getId()) : R.error();
    }



    /*@Override
    public R detailsOfCompetitionStage(CompetitionDetailReq req) {
        //1.本次活动参加了该阶段的全部团队，用于与参加本阶段活动的数据作对比
        QueryWrapper<CompetitionTeam> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("competition_id", req.getContestId());
        queryWrapper.eq("captain", 0);
        queryWrapper.eq("is_del", 0);
        List<CompetitionTeam> competitionTeams = competitionTeamMapper.selectList(queryWrapper);
        Page<CompetitionDetailsVO> page = new Page<CompetitionDetailsVO>(req.getPageNum(), req.getPageSize());
        IPage<CompetitionDetailsVO> pageList = baseMapper.detailsOfCompetitionStage(page, req);
        return R.ok().put("page", pageList).put("applicantSize", competitionTeams.size());
    }
*/


    @Override
    public R allowedParticipateCompetition(AllowedParticipateCompetitionReq req) throws ScriptException {
        //第一阶段无需处理，第二阶段查询第一阶段淘汰规则，以此类推
        if (req.getNumber() == 1) {
            return R.ok();
        }

        CompetitionStage competitionStage = competitionStageMapper.selectById(req.getStageId() - 1);
        String peopleLimit = competitionStage.getPeopleLimit();//人数限制
        Double percentageLimit = competitionStage.getPercentageLimit();//百分比限制
        String scoreLimit = competitionStage.getScoreLimit();//分数限制

        //获取阶段信息
        FrontDeskStageTeamScoreDetailsReq detailsReq = new FrontDeskStageTeamScoreDetailsReq();
        detailsReq.setCompetitionId(req.getCompetitionId());
        detailsReq.setStageId(req.getStageId() - 1);
        detailsReq.setIsOverallRanking(0);
        //获取阶段信息
        R r = competitionStageService.frontOfficeCompetitionRanking(detailsReq);
        List<ListOfStageResultsResp> respList = (List<ListOfStageResultsResp>) r.get("list");
        if (respList != null && respList.size() > 0) {
            //查询完整活动设置
            QueryWrapper<CompleteCompetitionSetup> wrapper = new QueryWrapper<>();
            wrapper.eq("competition_id", req.getCompetitionId());
            CompleteCompetitionSetup setup = completeCompetitionSetupMapper.selectOne(wrapper);

            int limitPeo = 0;
            //查询团队淘汰规则
            if (StringUtils.isNotBlank(peopleLimit) && Integer.parseInt(peopleLimit) <= respList.size()) {
                limitPeo = Integer.parseInt(peopleLimit);
            }
            //排名
            int limitPer = 0;
            if (percentageLimit != null) {
                double v = (percentageLimit / 100) * respList.size();
                limitPer = (int) Math.round(v);
            }

            int limitS = 0;
            if (StringUtils.isNotBlank(scoreLimit)) {
                for (int i = 0; i < respList.size(); i++) {
                    String s = "a" + scoreLimit;
                    ScriptEngineManager manager = new ScriptEngineManager();
                    ScriptEngine engine = manager.getEngineByName("js");
                    String score = respList.get(i).getScore();
                    engine.put("a", score);
                    boolean result = (boolean) engine.eval(s);
                    if (result) {
                        limitS = i + 1;
                    }
                }
            }
            int min = this.minimum(limitPeo, limitPer, limitS);

            List<ListOfStageResultsResp> sub = CollectionUtil.sub(respList, 0, min);

            //活动类型（0为个人赛，1为团队赛）
            if (setup.getCompetitionType() == 0) {
                List<String> list = new ArrayList<>();
                for (ListOfStageResultsResp resultsResp : sub) {
                    if (resultsResp.getAccountId() != null) {
                        list.add(resultsResp.getAccountId());
                    }
                }
                boolean contains = list.contains(req.getAccountId());
                return contains ? R.ok() : R.error("无法参加活动，你已被淘汰！");
            } else {
                List<Integer> list = new ArrayList<>();
                for (ListOfStageResultsResp resultsResp : sub) {
                    if (resultsResp.getTeamId() != null) {
                        list.add(resultsResp.getTeamId());
                    }
                }
                boolean contains = list.contains(req.getTeamId());
                return contains ? R.ok() : R.error("无法参加活动，你所在团队已被淘汰！");
            }
        }
        return R.error("晋级名单尚未发布，请等发布后再进入!");
    }

    /* 返回三个数中的非0最小值，全为零则返回0 */
    int minimum(int num1, int num2, int num3) {
        if (num1 == 0) {
            if (num2 == 0) {
                if (num3 == 0) return 0;
                else return num3;
            } else {
                if (num3 == 0) return num2;
                else return (num2 < num3) ? num2 : num3;
            }
        } else {
            if (num2 == 0) {
                if (num3 == 0) return num1;
                else return (num1 < num3) ? num1 : num3;
            } else {
                if (num3 == 0) return (num1 < num2) ? num1 : num2;
                else {
                    int min = (num1 < num2) ? num1 : num2;
                    min = (min < num3) ? min : num3;

                    return min;
                }
            }
        }

    }

    public static void main(String[] args) throws ScriptException {

        String str = "a>=5";
        ScriptEngineManager manager = new ScriptEngineManager();
        ScriptEngine engine = manager.getEngineByName("js");
        engine.put("a", 5);
        Object result = engine.eval(str);
        System.out.println("结果类型:" + result.getClass().getName() + ",计算结果:" + result);

    }

    @Override
    public List<CompetitionStage> queryCompetitionStage(String competitionId, String accountId) {
        if (StringUtils.isEmpty(accountId)) {
            accountId = "0";
        }
        //查询活动阶段数据
        return baseMapper.selectCompetitionStage(competitionId, Integer.valueOf(accountId));
    }

    @Override
    public R isParticipant(IsParticipantReq req) {
        QueryWrapper<CompetitionTeam> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("pid", req.getTeamId());
        List<CompetitionTeam> competitionTeams = competitionTeamMapper.selectList(queryWrapper);
        int teamSize = competitionTeams.size() + 1;

        QueryWrapper<CompleteCompetitionSetup> setupQueryWrapper = new QueryWrapper<>();
        setupQueryWrapper.eq("competition_id", req.getCompetitionId());
        CompleteCompetitionSetup setup = completeCompetitionSetupMapper.selectOne(setupQueryWrapper);
        if (teamSize < setup.getMinTeamSize()) {
            return R.error("团队人数不满足要求，无法参加活动！");
        }
        QueryWrapper<CompetitionTeam> teamQueryWrapper = new QueryWrapper<>();
        teamQueryWrapper.eq("competition_id", req.getCompetitionId());
        teamQueryWrapper.eq("account_id", req.getAccountId());
        CompetitionTeam team = competitionTeamMapper.selectOne(teamQueryWrapper);
        //查询参赛人员是否有数据
        QueryWrapper<TeamParticipant> wrapper = new QueryWrapper<>();
        wrapper.eq("team_id", req.getTeamId());
        wrapper.eq("competition_id", req.getCompetitionId());
        wrapper.eq("stage_id", req.getStageId());
        List<TeamParticipant> list = teamParticipantService.list(wrapper);
        if (list != null && list.size() > 0) {
            //有团队限制
            for (TeamParticipant teamParticipant : list) {
                if (teamParticipant.getAccountId() == Integer.parseInt(req.getAccountId())) {
                    return R.ok();
                }
            }
            return R.error("不是当前阶段指定参赛人员，无法参加活动！");
        } else {
            //如果阶段有设置团队成员参赛限制人数的，没有指派就不能参加，如果阶段没有设置限制人数的，没有指派，设置指定才可以参加
            CompetitionStage competitionStage = competitionStageMapper.selectById(req.getStageId());
            if (competitionStage.getTeamNumLimit() != null && competitionStage.getTeamNumLimit() == 0) {
                if (team.getCaptain() == 0) {
                    return R.error("无法参加活动，前往参数信息设置指定团队成员参赛！");
                }
                return R.error("无法参加活动");
            }
        }
        if (team.getCaptain() == 0) {
            return R.error("无法参加活动，前往参数信息设置指定团队成员参赛！");
        }
        return R.error("无法参加活动");
    }

    @Override
    public String queryRole(String accountId) {
        return baseMapper.obtainRole(Integer.valueOf(accountId));
    }
}
