package com.huoran.competition.service.impl;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import cn.afterturn.easypoi.excel.entity.enmus.ExcelType;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huoran.api.NakadaiClient;
import com.huoran.api.UserClient;
import com.huoran.common.constant.DelConstant;
import com.huoran.common.constant.TeamCalculationMethodConstant;
import com.huoran.common.response.R;
import com.huoran.competition.entity.*;
import com.huoran.competition.entity.req.*;
import com.huoran.competition.entity.resp.*;
import com.huoran.competition.entity.vo.PersonageRankingExportVO;
import com.huoran.competition.entity.vo.RankingExportVO;
import com.huoran.competition.entity.vo.RegistrationVO;
import com.huoran.competition.mapper.*;
import com.huoran.competition.service.CompetitionStageService;
import com.huoran.competition.utils.ExcelStyleUtil;
import com.huoran.competition.utils.RankUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import javax.script.ScriptException;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.net.URLEncoder;
import java.text.DecimalFormat;
import java.util.*;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.stream.Collectors;

/**
 * <p>
 * 活动阶段设置表 服务实现类
 * </p>
 *
 * @author cheney
 * @since 2022-11-16
 */
@Service
public class CompetitionStageServiceImpl extends ServiceImpl<CompetitionStageMapper, CompetitionStage> implements CompetitionStageService {

    @Autowired
    CompleteCompetitionSetupMapper competitionSetupMapper;

    @Autowired
    CompetitionContentMapper competitionContentMapper;

    @Autowired
    CompetitionTeamMapper teamMapper;

    @Autowired
    CompetitionRuleMapper ruleMapper;

    @Autowired
    CompetitionRankingMapper rankingMapper;

    @Autowired
    CompetitionRegistrationMapper competitionRegistrationMapper;

    @Autowired
    CompetitionStageContentSettingMapper competitionStageContentSettingMapper;

    @Autowired
    UserClient userClient;
    @Autowired
    NakadaiClient nakadaiClient;

    @Override
    public R queryCompetitionStageBySetupId(Integer setupId) {
        List<CompetitionStage> competitionStages = baseMapper.selectCompetitionStageBySetupId(setupId);


        competitionStages.forEach(competitionStage -> {
            QueryWrapper<CompetitionContent> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("stage_id", competitionStage.getStageId());
            CompetitionContent competitionContent = competitionContentMapper.selectOne(queryWrapper);
            if (competitionContent != null) {
                String systemName = competitionContentMapper.selectSystem(competitionContent.getSystemId());
                String projectName = competitionContentMapper.selectProjectName(competitionContent.getProjectId());
                competitionContent.setSystemName(systemName);
                competitionContent.setProjectName(projectName);
                competitionStage.setCompetitionContent(competitionContent);


                QueryWrapper<CompetitionStageContentSetting> queryWrapper1 = new QueryWrapper<>();
                queryWrapper1.eq("competition_id", setupId);
                queryWrapper1.eq("stage_id", competitionStage.getStageId());
                CompetitionStageContentSetting stageContentSetting = competitionStageContentSettingMapper.selectOne(queryWrapper1);
                if (!ObjectUtil.isEmpty(stageContentSetting)) {
                    //赋值线下内容设置
                    competitionContent.setCompetitionStageContentSetting(stageContentSetting);
                }

            }
        });
        return R.ok().put("competitionStages", competitionStages);
    }

    /**
     * 阶段活动排名
     *
     * @param competitionId
     * @param stageId
     * @return
     */
    @Override
    public R stageRaceRanking(Integer competitionId, Integer stageId, Integer pageNum, Integer pageSize, Integer publicationType) throws ScriptException {

        //查看活动类型是个人赛还是团队活动
        CompleteCompetitionSetup competitionSetup = competitionSetupMapper.getRaceSettingsByRaceId(competitionId);
        //获取阶段信息
        CompetitionStage competitionStage = baseMapper.selectById(stageId);


        //1.本次活动参加了该阶段的全部团队，用于与参加本阶段活动的数据作对比
        QueryWrapper<CompetitionTeam> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("competition_id", competitionId);
        queryWrapper.eq("captain", 0);
        queryWrapper.eq("is_del", 0);
        List<CompetitionTeam> competitionTeams = teamMapper.selectList(queryWrapper);

        //2.本阶段参赛的数据
        List<ListOfStageResultsResp> list = pointRace(competitionId, stageId, competitionSetup, competitionStage);


        boolean publishStatus = false;

        //返回判断是否默认发布过（用于给前端判断显示是否发布/取消按钮）
        WhetherToPublishParameterReq parameterReq = new WhetherToPublishParameterReq();
        parameterReq.setIsRelease(DelConstant.POST_STATUS);
        parameterReq.setCompetitionId(competitionId);
        parameterReq.setPublicationType(publicationType);
        parameterReq.setStageId(stageId);
        parameterReq.setIsOverallRanking(0);
        List<CompetitionRanking> rankingList = rankingMapper.whetherToPublish(parameterReq);
        if (rankingList.size() > 0) {
            //发布过
            publishStatus = true;
        }
        if (competitionTeams.size() == 0 && list == null) {
            //总数
            int total = competitionTeams.size();
            //总页数
            int pageSum = total % pageSize == 0 ? total / pageSize : total / pageSize + 1;
            //分页
            List<CompetitionTeam> subList = competitionTeams.stream().skip((pageNum - 1) * pageSize).limit(pageSize).collect(Collectors.toList());
            return R.ok().put("page", subList).put("pageSum", pageSum).put("total", total).put("publishStatus", publishStatus);
        }


        //判断当前阶段活动是否开始 (防止出现活动还没开始数据就已经存在了)
        QueryWrapper<CompetitionContent> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("stage_id", stageId);
        queryWrapper1.last(" AND unix_timestamp( start_time ) <= unix_timestamp( NOW( ) ) " + " AND unix_timestamp( end_time ) <= unix_timestamp( NOW( ) )");
        CompetitionContent competitionContent = competitionContentMapper.selectOne(queryWrapper1);


        if (competitionContent == null) {//阶段赛未结束前的处理(没有进入活动的人员 分数为—)
            if (competitionSetup.getCompetitionType() == 0) {

                //个人赛
                PageRegistrationStaffReq registrationStaffReq = new PageRegistrationStaffReq();
                registrationStaffReq.setCompetitionId(competitionId + "");
                Page<RegistrationVO> page = new Page<RegistrationVO>(1, 10000);
                List<RegistrationVO> applicantVOList = competitionRegistrationMapper.selectApplicantByCondition(page, registrationStaffReq).getRecords();
                list = new RankUtil().singleStageMatchingSetPersonage(applicantVOList, list);
            } else {
                list = new RankUtil().singleStageMatchingSet(competitionTeams, list);
            }

        } else { //阶段赛已完成后的处理
            //活动类型（0为个人赛，1为团队赛）默认为0
            if (competitionSetup.getCompetitionType() == 1) {
                // 1循环报名的全部团队 2循环阶段团队数据对比报名团队，没有对应的团队时，生成一条该团队数据，并给分数为0
                list = new RankUtil().matchingSet(competitionTeams, list);
                //处理团队赛的耗时


            } else if (competitionSetup.getCompetitionType() == 0) {


                PageRegistrationStaffReq registrationStaffReq = new PageRegistrationStaffReq();
                registrationStaffReq.setCompetitionId(competitionId + "");
                Page<RegistrationVO> page = new Page<RegistrationVO>(1, 10000);
                List<RegistrationVO> applicantVOList = competitionRegistrationMapper.selectApplicantByCondition(page, registrationStaffReq).getRecords();
                list = new RankUtil().matchingSetPersonage(applicantVOList, list);
            }

            //对淘汰赛校验 超过第二阶段的情况下才需要校验去除被淘汰的队伍
            if (competitionStage.getNumber() != 1) {
                competitionStage = baseMapper.selectById(stageId - 1);
                String peopleLimit = competitionStage.getPeopleLimit();//人数限制
                Double percentageLimit = competitionStage.getPercentageLimit();//百分比限制
                String scoreLimit = competitionStage.getScoreLimit();//分数限制
                //查看活动赛制 赛制（0为积分赛，1为淘汰赛，2为单项赛）
                QueryWrapper<CompetitionRule> ruleQueryWrapper = new QueryWrapper<>();
                ruleQueryWrapper.eq("setup_id", competitionSetup.getSetupId());
                CompetitionRule rule = ruleMapper.selectOne(ruleQueryWrapper);
                if (rule.getRule() == 1) {//rule 赛制（0为积分赛，1为淘汰赛，2为单项赛）
                    if (list != null && list.size() > 0) {
                        //查询完整活动设置
                        QueryWrapper<CompleteCompetitionSetup> wrapper = new QueryWrapper<>();
                        wrapper.eq("competition_id", competitionId);

                        int limitPeo = 0;
                        //查询团队淘汰规则
                        if (StringUtils.isNotBlank(peopleLimit) && Integer.parseInt(peopleLimit) <= list.size()) {
                            limitPeo = Integer.parseInt(peopleLimit);
                        }
                        //排名
                        int limitPer = 0;
                        if (percentageLimit != null) {
                            double v = (percentageLimit / 100) * list.size();
                            limitPer = (int) Math.round(v);
                        }

                        int limitS = 0;
                        if (StringUtils.isNotBlank(scoreLimit)) {
                            for (int i = 0; i < list.size(); i++) {
                                String s = "a" + scoreLimit;
                                ScriptEngineManager manager = new ScriptEngineManager();
                                ScriptEngine engine = manager.getEngineByName("js");
                                String score = list.get(i).getScore();
                                engine.put("a", score);
                                boolean result = (boolean) engine.eval(s);
                                if (result) {
                                    limitS = i + 1;
                                }
                            }
                        }
                        int min = this.minimum(limitPeo, limitPer, limitS);

                        list = CollectionUtil.sub(list, 0, min);


                    }
                }
            }

        }

            /*list = list.stream().sorted(Comparator.comparing(resp -> Double.parseDouble(resp.getScore().toString())))
                    .collect(Collectors.toList());*/
            /*list.stream().sorted(Comparator.comparing(ListOfStageResultsResp::getScore,Comparator.comparingDouble(Double::parseDouble))
                    .reversed()).sorted(Comparator.comparing(ListOfStageResultsResp::getTimeSum,Comparator.comparingDouble(Double::parseDouble)))
                    .collect(Collectors.toList());*/


        if (list != null) {
            //总数
            int total = list.size();
            //总页数
            int pageSum = total % pageSize == 0 ? total / pageSize : total / pageSize + 1;


            //是否考试(0未考 1已考)
               /* list.stream()
                        .peek(r -> r.setTestOrNot(r.getReportId() == null ? 0 : 1))
                        .collect(Collectors.toList());



                List<ListOfStageResultsResp> subList = list.stream()
                        .sorted(Comparator.comparing(ListOfStageResultsResp::getScore, Comparator.comparingDouble(Double::parseDouble))
                                .reversed().thenComparing(ListOfStageResultsResp::getTimeSum, Comparator.comparingDouble(Double::parseDouble)))
                        .collect(Collectors.toList()).stream().skip((pageNum - 1) * pageSize).limit(pageSize).collect(Collectors.toList());
    */

            //是否考试(0未考 1已考)
               /* list.stream().peek(r -> r.setTestOrNot(r.getReportId() == null ? 0 : 1)).collect(Collectors.toList()).sort(
                        Comparator.comparing(ListOfStageResultsResp::getTestOrNot, Comparator.comparingInt(Integer::intValue).reversed()).
                                thenComparing(Comparator.comparing(ListOfStageResultsResp::getScore, Comparator.comparingDouble(Double::parseDouble).reversed())
                                        .thenComparing(ListOfStageResultsResp::getTimeSum, Comparator.comparingDouble(Double::parseDouble)))
                );*/


            if (!list.stream().peek(r -> r.setTestOrNot(r.getReportId() == null ? 0 : 1)).collect(Collectors.toList()).contains("-")) {
                return processingPaging(pageNum, pageSize, list, publishStatus, total, pageSum);
            } else {
                //是否考试(0未考 1已考)
                list.stream().peek(r -> r.setTestOrNot(r.getReportId() == null ? 0 : 1)).collect(Collectors.toList());
                //分数降序、耗时升序
                list = list.stream().sorted(Comparator.comparing(ListOfStageResultsResp::getTestOrNot, Comparator.comparingInt(Integer::intValue).reversed()).thenComparing(ListOfStageResultsResp::getScore, Comparator.comparingDouble(Double::parseDouble).reversed()).thenComparing(ListOfStageResultsResp::getTimeSum, Comparator.comparingDouble(Double::parseDouble))).collect(Collectors.toList());
            }


                /*//是否考试(0未考 1已考)
                list.stream().peek(r -> r.setTestOrNot(r.getReportId() == null ? 0 : 1)).collect(Collectors.toList());
                //分数降序、耗时升序
                list = list.stream().sorted(Comparator.comparing(ListOfStageResultsResp::getTestOrNot, Comparator.comparingInt(Integer::intValue).reversed()).thenComparing(ListOfStageResultsResp::getScore, Comparator.comparingDouble(Double::parseDouble).reversed()).thenComparing(ListOfStageResultsResp::getTimeSum, Comparator.comparingDouble(Double::parseDouble))).collect(Collectors.toList());

*/
            //分页
            return processingPaging(pageNum, pageSize, list, publishStatus, total, pageSum);
        }
        return R.ok();

    }


    private R processingPaging(Integer pageNum, Integer pageSize, List<ListOfStageResultsResp> list, boolean publishStatus, int total, int pageSum) {
        int startIndex = (pageNum - 1) * pageSize;
        int endIndex = startIndex + pageSize;
        List<ListOfStageResultsResp> subList = list.stream().skip(startIndex).limit(pageSize).collect(Collectors.toList());
        return R.ok().put("page", subList).put("pageSum", pageSum).put("total", total).put("publishStatus", publishStatus);
    }


        /*private List<ListOfStageResultsResp> dataProcessing(List<CompetitionTeam> competitionTeams, List<ListOfStageResultsResp> pointRace, Integer stageId) {


            //参赛全部队伍 A集合
            List<Integer> allTeam = competitionTeams.stream().map(CompetitionTeam::getTeamId).collect(Collectors.toList());

            //参与本次阶段,有成绩的队伍 B集合
            List<Integer> teamWithResults = pointRace.stream().map(ListOfStageResultsResp::getTeamId).collect(Collectors.toList());


            //falseData：假数据,用于区分活动没有参加团队的但是要给一条数据
            List<ListOfStageResultsResp> falseData = new ArrayList<>();

            // 对比
            for (Integer id : allTeam) {
                if (!teamWithResults.contains(id)) {
                    // A集合中，B集合没有的对象id
                    System.err.println(id);
                    //判断B中没有的团队id,则加入到要返回的list中

                    ListOfStageResultsResp addTeamId = new ListOfStageResultsResp();
                    addTeamId.setTeamId(id);
                    addTeamId.setScore("-");
                    addTeamId.setTimeSum("-");
                    falseData.add(addTeamId);
                }
            }

            pointRace.addAll(falseData);

            for (ListOfStageResultsResp l : pointRace) {
                CompetitionTeam team = teamMapper.selectById(l.getTeamId());
                if (team != null) {
                    if (l.getTeamName() == null) {

                        l.setTeamName(team.getTeamName());
                    }

                    if (l.getLeaderName() == null) {
                        l.setLeaderName(userClient.getUserName(team.getAccountId()));
                    }

                    if (l.getSchoolId() == null) {
                        Integer schoolId = userClient.getSchoolIdByAccountId(team.getAccountId() + "");
                        l.setSchoolName(nakadaiClient.getTheSchoolNameBySchoolId(schoolId).getSchoolName());
                    }
                }
            }
            return pointRace;
        }*/


    /**
     * 找出list2中不存在的元素并新增该记录
     * （list1中的挪过去）
     *
     * @return 处理后的List2
     */
    private List<ListOfStageResultsResp> addNonExistData(List<CompetitionTeam> competitionTeams, List<RegistrationVO> selectApplicantByCondition, List<ListOfStageResultsResp> pointRace, Integer stageId, Integer competitionType) {


        if (competitionType == 0) { //0:个人赛
            //报名本次活动的全部人员 A集合
            List<Integer> allApplicants = selectApplicantByCondition.stream().map(RegistrationVO::getAccountId).collect(Collectors.toList());
            //参与本次阶段,有成绩的队伍 B集合
            List<String> teamWithResults = pointRace.stream().map(ListOfStageResultsResp::getAccountId).collect(Collectors.toList());
            List<Integer> intClusterIds = teamWithResults.stream().mapToInt(Integer::parseInt).boxed().collect(Collectors.toList());
            //falseData：假数据,用于区分活动没有参加团队的但是要给一条数据
            List<ListOfStageResultsResp> falseData = new ArrayList<>();

            // 对比
            for (Integer id : allApplicants) {
                if (!intClusterIds.contains(id)) {
                    // A集合中，B集合没有的对象id
                    ////System.err.println(id);
                    //判断B中没有的账号id,则加入到要返回的list中
                    ListOfStageResultsResp addTeamId = new ListOfStageResultsResp();
                    addTeamId.setScore("0");
                    addTeamId.setTimeSum("0");
                    addTeamId.setAccountId(id + "");
                    falseData.add(addTeamId);
                }
            }
            if (pointRace != null) {
                pointRace.addAll(falseData);

                for (ListOfStageResultsResp l : pointRace) {
                    if (l.getSchoolId() == null) {
                        Integer schoolId = userClient.getSchoolIdByAccountId(l.getAccountId() + "");
                        l.setSchoolId(schoolId);
                        l.setSchoolName(nakadaiClient.getTheSchoolNameBySchoolId(schoolId).getSchoolName());
                    }

                    if (l.getLeaderName() == null) {
                        l.setAccountId(l.getAccountId());
                        l.setUserName(userClient.getUserName(Integer.valueOf(l.getAccountId())));
                    }

                }
            }
        } else if (competitionType == 1) {//团队赛
            //参赛全部队伍 A集合
            List<Integer> allTeam = competitionTeams.stream().map(CompetitionTeam::getTeamId).collect(Collectors.toList());

            List<Integer> teamWithResults = null;
            if (pointRace != null) {
                //参与本次阶段,有成绩的队伍 B集合
                teamWithResults = pointRace.stream().map(ListOfStageResultsResp::getTeamId).collect(Collectors.toList());
            }

            //falseData：假数据,用于区分活动没有参加团队的但是要给一条数据
            List<ListOfStageResultsResp> falseData = new ArrayList<>();

            // 对比
            for (Integer id : allTeam) {
                if (!teamWithResults.contains(id)) {
                    // A集合中，B集合没有的对象id
                    System.err.println(id);
                    //判断B中没有的团队id,则加入到要返回的list中

                    ListOfStageResultsResp addTeamId = new ListOfStageResultsResp();
                    addTeamId.setTeamId(id);
                    addTeamId.setScore("0");
                    addTeamId.setTimeSum("0");
                    falseData.add(addTeamId);
                }
            }


            if (pointRace != null) {
                pointRace.addAll(falseData);

                for (ListOfStageResultsResp l : pointRace) {
                    CompetitionTeam team = teamMapper.selectById(l.getTeamId());
                    if (team != null) {
                        if (l.getTeamName() == null) {

                            l.setTeamName(team.getTeamName());
                        }

                        if (l.getLeaderName() == null) {
                            l.setLeaderName(userClient.getUserName(team.getAccountId()));
                            l.setAccountId(team.getAccountId() + "");
                        }

                        if (l.getSchoolId() == null) {
                            Integer schoolId = userClient.getSchoolIdByAccountId(team.getAccountId() + "");
                            l.setSchoolName(nakadaiClient.getTheSchoolNameBySchoolId(schoolId).getSchoolName());
                            l.setSchoolId(schoolId);
                        }
                    }
                }
            }

        }
        return pointRace;
    }

    @Override
    public R overallRanking(OverallStandingsInThePointsRaceReq req) {
        //获取活动总分排名设置规则
        CompetitionRule competitionRule = ruleMapper.getTheRulesOfTheRaceByTheRaceId(req.getCompetitionId());


        List<OverallEventRankingResp> overallEventRankingRespList = new ArrayList<>();
        //todo 1循环报名的全部团队 2循环阶段团队数据对比报名团队，没有对应的团队时，生成一条该团队数据，并给分数为0
        QueryWrapper<CompetitionTeam> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("captain", 0);
        queryWrapper.eq("is_del", 0);
        queryWrapper.eq("competition_id", req.getCompetitionId());
        List<CompetitionTeam> competitionTeams = teamMapper.selectList(queryWrapper);
        //查看活动类型是个人赛还是团队活动
        CompleteCompetitionSetup competitionSetup = competitionSetupMapper.getRaceSettingsByRaceId(req.getCompetitionId());
        Integer competitionType = competitionSetup.getCompetitionType();

        for (Integer stageId : req.getStageIds()) {
            OverallEventRankingResp rankingResp = new OverallEventRankingResp();
            // 获取阶段信息
            CompetitionStage competitionStage = baseMapper.selectById(stageId);
            //获取每个阶段的团队、个人成绩列表
            List<ListOfStageResultsResp> pointRace = pointRace(req.getCompetitionId(), stageId, competitionSetup, competitionStage);

            //判断当前阶段活动是否开始 (防止出现活动还没开始数据就已经存在了)
            QueryWrapper<CompetitionContent> queryWrapper1 = new QueryWrapper<>();
            queryWrapper1.eq("stage_id", stageId);
            queryWrapper1.last(" AND start_time <= now()");
            CompetitionContent competitionContent = competitionContentMapper.selectOne(queryWrapper1);

            //个人赛
            PageRegistrationStaffReq registrationStaffReq = new PageRegistrationStaffReq();
            registrationStaffReq.setCompetitionId(req.getCompetitionId() + "");
            Page<RegistrationVO> page = new Page<RegistrationVO>(1, 10000);
            List<RegistrationVO> applicantVOList = competitionRegistrationMapper.selectApplicantByCondition(page, registrationStaffReq).getRecords();

            if (competitionContent != null) {
                pointRace = addNonExistData(competitionTeams, applicantVOList, pointRace, stageId, competitionType);
            }

            rankingResp.setStageName(competitionStage.getStageName());
            rankingResp.setStageId(competitionStage.getStageId());
            if (pointRace != null) {
                rankingResp.setStageTeamInformation(pointRace);
            }

            rankingResp.setPointWeight(competitionStage.getPointWeight());
            overallEventRankingRespList.add(rankingResp);

        }

        //成绩计算方式（0：各阶段加权求和，1：各阶段成绩求和，2：各阶段成绩平均值
        String teamScore = "0";

        List<ListOfStageResultsResp> returnTotalSet = new ArrayList<>();//返回的集合

        //总成绩计算方式（0：各阶段加权求和，1：各阶段成绩求和，2：各阶段成绩平均值）
        Integer resultCalculationMethod = competitionRule.getResultCalculationMethod();
        for (OverallEventRankingResp rankingResp : overallEventRankingRespList) {
            if (rankingResp.getStageTeamInformation() != null) {
                //各阶段求值 权重、求和、平均等等
                totalStageScoreCalculation(competitionRule, returnTotalSet, rankingResp);
            }
        }


        if (competitionType == 1) {
            Map<Integer, CompetitionRangeTeamCountResp> map = new HashMap<>();
            for (OverallEventRankingResp item : overallEventRankingRespList) {
                if (item.getStageTeamInformation() != null) {
                    for (ListOfStageResultsResp l : item.getStageTeamInformation()) {

                        CompetitionRangeTeamCountResp rangeTeamCountResp = new CompetitionRangeTeamCountResp();

                        List<String> scoreList = new ArrayList<>();
                        List<String> timeSum = new ArrayList<>();
                        if (map.get(l.getTeamId()) != null) {
                            CompetitionRangeTeamCountResp rangeTeamCountResp2 = map.get(l.getTeamId());
                            scoreList = rangeTeamCountResp2.getScoreList();
                            timeSum = rangeTeamCountResp2.getTimeSum();
                        }

                        scoreList.add(l.getScore());
                        timeSum.add(l.getTimeSum());

                        rangeTeamCountResp.setScoreList(scoreList);
                        rangeTeamCountResp.setTimeSum(timeSum);
                        map.put(l.getTeamId(), rangeTeamCountResp);
                    }
                }
            }
            //List<ListOfStageResultsResp> stageTeamInformation = overallEventRankingRespList.get(overallEventRankingRespList.size() - 1).getStageTeamInformation();
            List<ListOfStageResultsResp> stageTeamInformation = null;

            for (OverallEventRankingResp res : overallEventRankingRespList) {
                stageTeamInformation = res.getStageTeamInformation();
                Set<Integer> set = map.keySet();
                if (stageTeamInformation != null) {
                    for (ListOfStageResultsResp l : stageTeamInformation) {
                        for (Integer str : set) {
                            if (str == l.getTeamId()) {
                                CompetitionRangeTeamCountResp in = map.get(str);

                                //统计各个多阶段分数
                                List<Double> dList = in.getScoreList().stream().map((d) -> {
                                    return Double.valueOf(d + "");
                                }).collect(Collectors.toList());

                                ScoreProcessing(competitionRule, l, dList);


                                //统计耗时
                                List<Double> timeList = in.getTimeSum().stream().map((d) -> {
                                    return Double.valueOf(d + "");
                                }).collect(Collectors.toList());
                                Double time = 0.0;
                                /**根据活动规则选择的总成绩计算方式处理耗时*/
                                time = timeList.stream().mapToDouble(Double::doubleValue).sum();
                                //总成绩计算方式（0：各阶段加权求和，1：各阶段成绩求和，2：各阶段成绩平均值）
                                switch (resultCalculationMethod) {
                                    case 0:

                                        break;
                                    case 1:
                                        time = time;
                                        break;
                                    case 2:
                                        time = Double.valueOf(time) / timeList.size();
                                        break;
                                }


                                //处理保留两位小数

                                DecimalFormat decimal = new DecimalFormat("0.00");
                                decimal.setRoundingMode(RoundingMode.DOWN);
                                String strTime = decimal.format(Double.valueOf(time));

                                l.setTimeSum(strTime + "");

                            }

                        }
                    }
                    stageTeamInformation.sort((o1, o2) -> (int) (Double.valueOf(o2.getScore()) - Double.valueOf(o1.getScore())));
                    //拿到报名名单的全部数据，对比每个阶段的分数数据，

                    //总数
                    int total = stageTeamInformation.size();
                    //总页数
                    int pageSum = total % req.getPageSize() == 0 ? total / req.getPageSize() : total / req.getPageSize() + 1;


                    //分页
                    //                    List<ListOfStageResultsResp> subList = stageTeamInformation.stream().skip((req.getPageNum() - 1) * req.getPageSize()).limit(req.getPageSize()).collect(Collectors.toList());

                    //分页
                    List<ListOfStageResultsResp> subList = stageTeamInformation.stream().sorted(Comparator.comparing(ListOfStageResultsResp::getScore, Comparator.comparingDouble(Double::parseDouble)).reversed().thenComparing(ListOfStageResultsResp::getTimeSum, Comparator.comparingDouble(Double::parseDouble))).collect(Collectors.toList()).stream().skip((req.getPageNum() - 1) * req.getPageSize()).limit(req.getPageSize()).collect(Collectors.toList());

                    WhetherToPublishParameterReq parameterReq = getToPublishParameterReq(req);

                    //返回判断是否默认发布过
                    List<CompetitionRanking> rankingList = rankingMapper.whetherToPublishTheOverallScoreRanking(parameterReq);

                    boolean publishStatus = false;
                    if (rankingList.size() > 0) {
                        //发布过
                        publishStatus = true;
                    }
                    //处理保留两位小数
                    subList.forEach(data -> {
                        DecimalFormat decimal = new DecimalFormat("0.00");
                        decimal.setRoundingMode(RoundingMode.DOWN);
                        String str = decimal.format(Double.valueOf(data.getScore()));
                        data.setScore(str);
                    });
                    /*subList = new RankUtil().matchingSet(competitionTeams, subList);*/
                    return R.ok().put("page", subList).put("pageSum", pageSum).put("total", total).put("publishStatus", publishStatus);
                }
            }


            return R.ok();

        } else {
            //个人赛
            List<ListOfStageResultsResp> individualCompetition = new ArrayList<>();
            Map<String, List<ListOfStageResultsResp>> listMap = returnTotalSet.stream().collect(Collectors.groupingBy(ListOfStageResultsResp::getAccountId));
            List<ListOfStageResultsResp> retList = new ArrayList<>();

            for (String accountId : listMap.keySet()) {
                //                System.out.println("key的值是：" + accountId + "=============" + "value的值是：" + listMap.get(accountId));
                List<ListOfStageResultsResp> resultsRespList = listMap.get(accountId);
                retList = merge(resultsRespList);

                individualCompetition.addAll(retList);

            }

            //            /*return R.ok(individualCompetition);*/

            PageRegistrationStaffReq registrationStaffReq = new PageRegistrationStaffReq();
            registrationStaffReq.setCompetitionId(req.getCompetitionId() + "");
            Page<RegistrationVO> page = new Page<RegistrationVO>(1, 10000);
            List<RegistrationVO> applicantVOList = competitionRegistrationMapper.selectApplicantByCondition(page, registrationStaffReq).getRecords();
            individualCompetition = new RankUtil().matchingSetPersonage(applicantVOList, individualCompetition);

            individualCompetition.sort(new Comparator<ListOfStageResultsResp>() {
                @Override
                public int compare(ListOfStageResultsResp o1, ListOfStageResultsResp o2) {
                    return (int) (Double.valueOf(o2.getScore()) - Double.valueOf(o1.getScore()));
                }
            });


            //

            //总数
            int total = individualCompetition.size();
            //总页数
            int pageSum = total % req.getPageSize() == 0 ? total / req.getPageSize() : total / req.getPageSize() + 1;

            //分页
            List<ListOfStageResultsResp> subList = individualCompetition.stream().skip((req.getPageNum() - 1) * req.getPageSize()).limit(req.getPageSize()).collect(Collectors.toList());


            WhetherToPublishParameterReq parameterReq = new WhetherToPublishParameterReq();
            parameterReq.setIsRelease(DelConstant.POST_STATUS);
            parameterReq.setCompetitionId(req.getCompetitionId());
            //parameterReq.setPublicationType(DelConstant.PUBLICATION_TYPE_MANUALOPERATION);
            parameterReq.setIsOverallRanking(1);
            parameterReq.setPublicationType(req.getPublicationType());
            if (req.getLocationStageId() != null) {
                parameterReq.setStageId(req.getLocationStageId());
            }


            //返回判断是否默认发布过
            List<CompetitionRanking> rankingList = rankingMapper.whetherToPublishTheOverallScoreRanking(parameterReq);

            boolean publishStatus = false;
            if (rankingList.size() > 0) {
                //发布过
                publishStatus = true;
            }
            if (subList.size() > 0) {
                //处理保留两位小数
                subList.forEach(data -> {
                    double parseDouble = Double.parseDouble(data.getScore());
                    DecimalFormat df = new DecimalFormat("######0.00");
                    String str = df.format(parseDouble);
                    data.setScore(str);
                });
            }
            return R.ok().put("page", subList).put("pageSum", pageSum).put("total", total).put("publishStatus", publishStatus);
        }

    }

    static void ScoreProcessing(CompetitionRule competitionRule, ListOfStageResultsResp l, List<Double> dList) {
        switch (competitionRule.getResultCalculationMethod()) {
            case 0://0：各阶段加权求和
            case 1://1：各阶段成绩求和
                // java8 集合求和
                Double sum = dList.stream().mapToDouble(Double::doubleValue).sum();
                l.setScore(sum + "");
                break;
            case 2://2：各阶段成绩平均值
                DoubleSummaryStatistics statistics = dList.stream().mapToDouble(Number::doubleValue).summaryStatistics();
                Double avg = statistics.getAverage();
                l.setScore(avg + "");
                break;
        }
    }

    private static WhetherToPublishParameterReq getToPublishParameterReq(OverallStandingsInThePointsRaceReq req) {
        WhetherToPublishParameterReq parameterReq = new WhetherToPublishParameterReq();
        parameterReq.setIsRelease(DelConstant.POST_STATUS);
        parameterReq.setCompetitionId(req.getCompetitionId());
        parameterReq.setPublicationType(req.getPublicationType());
        parameterReq.setIsOverallRanking(1);
        if (req.getLocationStageId() != null) {
            parameterReq.setStageId(req.getLocationStageId());
        }
        return parameterReq;
    }


    //总分阶段成绩计算
    static void totalStageScoreCalculation(CompetitionRule competitionRule, List<ListOfStageResultsResp> returnTotalSet, OverallEventRankingResp rankingResp) {
        String teamScore;
        String teamTime;
        switch (competitionRule.getResultCalculationMethod()) {
            case 0://0：各阶段加权求和
                for (ListOfStageResultsResp list : rankingResp.getStageTeamInformation()) {
                    //获取权重比乘以分数得到本团队该阶段的分数
                    teamScore = (rankingResp.getPointWeight() / 100) * Double.valueOf(list.getScore()) + "";
                    teamTime = (rankingResp.getPointWeight() / 100) * Double.valueOf(list.getTimeSum()) + "";

                    list.setScore(teamScore);
                    list.setTimeSum(teamTime);
                    returnTotalSet.add(list);
                }
                break;
            case 1://1：各阶段成绩求和

                break;

            case 2://2：各阶段成绩平均值
                for (ListOfStageResultsResp list : rankingResp.getStageTeamInformation()) {
                    //获取权重比乘以分数得到本团队该阶段的分数
                    list.setScore(list.getScore());
                    list.setTimeSum(list.getTimeSum());
                    returnTotalSet.add(list);
                }
                break;
        }
    }


    static void evaluationAtEachStage(CompetitionRule competitionRule, List<ListOfStageResultsResp> returnTotalSet, OverallEventRankingResp rankingResp) {
        String teamScore;
        String teamTime;
        switch (competitionRule.getResultCalculationMethod()) {
            case 0://0：各阶段加权求和
                for (ListOfStageResultsResp list : rankingResp.getStageTeamInformation()) {
                    //获取权重比乘以分数得到本团队该阶段的分数
                    teamScore = (rankingResp.getPointWeight() / 100) * Double.valueOf(list.getScore()) + "";
                    teamTime = (rankingResp.getPointWeight() / 100) * Double.valueOf(list.getTimeSum()) + "";

                    list.setScore(teamScore);
                    list.setTimeSum(teamTime);
                    returnTotalSet.add(list);
                }
                break;
            case 1://1：各阶段成绩求和
            case 2://2：各阶段成绩平均值
                for (ListOfStageResultsResp list : rankingResp.getStageTeamInformation()) {
                    //获取权重比乘以分数得到本团队该阶段的分数
                    list.setScore(list.getScore());
                    list.setTimeSum(list.getTimeSum());
                    returnTotalSet.add(list);
                }
                break;
        }
    }

    /**
     * 将id进行合并nums, sums 相加道回合并后的集合使用Java8的流进行处理
     */

    public static List<ListOfStageResultsResp> merge(List<ListOfStageResultsResp> list) {
        List<ListOfStageResultsResp> result = list.stream().collect(Collectors.toMap(ListOfStageResultsResp::getAccountId, a -> a, (o1, o2) -> {
            Double score = Double.valueOf(o1.getScore()) + Double.valueOf(o2.getScore());
            Double time = Double.valueOf(o1.getTimeSum()) + Double.valueOf(o2.getTimeSum());
            o1.setScore(score + "");
            o1.setTimeSum(time + "");
            o1.setTeamId(null);
            return o1;
        })).values().stream().collect(Collectors.toList());

        return result;

    }


    /**
     * 积分赛
     * 个人赛:
     * 当前阶段的排名就是按照个人在当前阶段的成绩来（成绩一样比用时）
     * 当前阶段的总分就还是按照它本身设置的阶段之和还是阶段平均什么的来
     *
     * @param competitionId
     * @param stageId
     * @param competitionSetup
     * @return
     */
    public List<ListOfStageResultsResp> pointRace(Integer competitionId, Integer stageId, CompleteCompetitionSetup competitionSetup, CompetitionStage competitionStage) {

        //活动类型（0为个人赛，1为团队赛）默认为0
        if (competitionSetup.getCompetitionType() == TeamCalculationMethodConstant.TYPE_OF_EVENT_INDIVIDUAL_COMPETITION) {//个人赛

            if (competitionStage.getMethod() == 2) { //活动方式（0实操，1理论，2线下）
                List<ListOfStageResultsResp> offlineIndividualPointsCompetition = teamMapper.offlineIndividualPointsCompetition(new StageRankingReq().setStageId(stageId).setCompetitionId(competitionId));
                offlineIndividualPointsCompetition.sort(new Comparator<ListOfStageResultsResp>() {
                    @Override
                    public int compare(ListOfStageResultsResp o1, ListOfStageResultsResp o2) {
                        return (int) (Double.valueOf(o2.getScore()) - Double.valueOf(o1.getScore()));
                    }
                });
                return offlineIndividualPointsCompetition;
            } else {
                List<ListOfStageResultsResp> onlineIndividualPointsCompetition = teamMapper.onlineIndividualPointsCompetition(new StageRankingReq().setStageId(stageId).setCompetitionId(competitionId));

                onlineIndividualPointsCompetition.sort(new Comparator<ListOfStageResultsResp>() {
                    @Override
                    public int compare(ListOfStageResultsResp o1, ListOfStageResultsResp o2) {
                        return (int) (Double.valueOf(o2.getScore()) - Double.valueOf(o1.getScore()));
                    }
                });
                return onlineIndividualPointsCompetition;
            }
        } else if (competitionSetup.getCompetitionType() == TeamCalculationMethodConstant.TYPE_OF_EVENT_TEAM_EVENT) {//团队赛

            //团队排名：用时以及分数依据当前阶段成绩的计算方式(最高分、平均分、求和)取
            //如：当前阶段活动成绩计算方式为取最高分，则用户用时以及分数取得是当前团队下的最高分当前成绩的用时以及分数
            //团队成绩计算方式（0：最高分，1：平均分，2：求和）

            //1.获取参与该活动的团队
            String teamIds = teamMapper.getTeamInformation(competitionId);
            if (teamIds != null) {
                List<ListOfStageResultsResp> offLineTeamPointsCompetition;
                if (competitionStage.getMethod() == 2) { //活动方式为线下时候查询的ol_user_score表统计分数，线下活动是没有实验报告
                    //团队积分赛线下 ———— 求各个团队的最高分、平均分、总和
                    offLineTeamPointsCompetition = teamMapper.offLineTeamPointsCompetition(new StageRankingReq().setStageId(stageId).setCompetitionId(competitionId).setTeamIds(teamIds));


                    for (ListOfStageResultsResp resp : offLineTeamPointsCompetition) {

                        //计算耗时
                        TimeConsumingCalculationResp timeConsuming = teamMapper.functionAcquisitionTime(new StageRankingReq().setStageId(stageId).setCompetitionId(competitionId).setTeamId(resp.getTeamId()));

                        //团队成绩计算方式（0：最高分，1：平均分，2：总分）stageName = "决赛，不限制人，取最高。公布"
                        if (competitionStage.getTeamCalculationMethod().equals("0")) {
                            resp.setScore(resp.getMaxScore());
                            if (!ObjectUtil.isEmpty(timeConsuming)) {
                                resp.setTimeSum(timeConsuming.getMaximumTimeConsuming());
                            }

                        } else if (competitionStage.getTeamCalculationMethod().equals("1")) {
                            resp.setScore(resp.getAvgScore());

                            if (!ObjectUtil.isEmpty(timeConsuming)) {
                                resp.setTimeSum(timeConsuming.getAverageTimeConsuming());
                            }
                        } else if (competitionStage.getTeamCalculationMethod().equals("2")) {
                            resp.setScore(resp.getSumScore());

                            if (!ObjectUtil.isEmpty(timeConsuming)) {
                                resp.setTimeSum(timeConsuming.getTotalTimeSpent());
                            }

                        }

                    }

                    //                    System.out.println("5.2.按学号降序、年龄降序 (先以年龄升序,再进行年龄升序,结果进行(学号&年龄)降序)");
                       /* offLineTeamPointsCompetition
                                .stream()
                                .sorted(Comparator.comparingDouble(ListOfStageResultsResp::getScore)
                                        .thenComparingInt(ListOfStageResultsResp::getTimeSum)
                                        .reversed())
                                .collect(Collectors.toList())
                                .forEach(System.out::println);*/

                        /*offLineTeamPointsCompetition.sort(new Comparator<ListOfStageResultsResp>() {
                            @Override
                            public int compare(ListOfStageResultsResp o1, ListOfStageResultsResp o2) {
                                return (int) (Double.valueOf(o2.getScore()) - Double.valueOf(o1.getScore()));
                            }

                            @Override
                            public int compare(ListOfStageResultsResp o1, ListOfStageResultsResp o2) {
                                return (int) (Double.valueOf(o2.getTimeSum()) - Double.valueOf(o1.getTimeSum()));
                            }
                        });*/


                } else {
                    //团队积分赛线上 ———— 求各个在队伍线上的最高分、平均分、总和
                    offLineTeamPointsCompetition = teamMapper.onlineTeamPointsContest(new StageRankingReq().setStageId(stageId).setCompetitionId(competitionId).setTeamIds(teamIds));

                    for (ListOfStageResultsResp resp : offLineTeamPointsCompetition) {

                        //计算耗时
                        TimeConsumingCalculationResp timeConsuming = teamMapper.functionAcquisitionTime(new StageRankingReq().setStageId(stageId).setCompetitionId(competitionId).setTeamId(resp.getTeamId()));

                        //团队成绩计算方式（0：最高分，1：平均分，2：总分）
                        if (competitionStage.getTeamCalculationMethod().equals("0")) {
                            resp.setScore(resp.getMaxScore());

                            //查询最高分，最短耗时的分数
                            TimeConsumingCalculationResp maxTime = teamMapper.highestScoreTakesTheLeastTime(new StageRankingReq().setStageId(stageId).setCompetitionId(competitionId).setTeamId(resp.getTeamId()));


                            resp.setTimeSum(maxTime.getMaximumTimeConsuming());

                        } else if (competitionStage.getTeamCalculationMethod().equals("1")) {
                            resp.setScore(resp.getAvgScore());
                            resp.setTimeSum(timeConsuming.getAverageTimeConsuming());
                        } else if (competitionStage.getTeamCalculationMethod().equals("2")) {


                            double score = teamMapper.teamSum(new StageRankingReq().setStageId(stageId).setCompetitionId(competitionId).setTeamId(resp.getTeamId()));
                            resp.setScore(score + "");
                            resp.setTimeSum(timeConsuming.getTotalTimeSpent());

                        }
                    }
                    //顺序不能更改，上面分数需要赋值 否则异常
                       /* offLineTeamPointsCompetition.sort(new Comparator<ListOfStageResultsResp>() {
                            @Override
                            public int compare(ListOfStageResultsResp o1, ListOfStageResultsResp o2) {
                                return (int) (Double.valueOf(o2.getScore()) - Double.valueOf(o1.getScore()));
                            }
                        });*/
                    offLineTeamPointsCompetition = offLineTeamPointsCompetition.stream().sorted(Comparator.comparing(ListOfStageResultsResp::getScore).thenComparing(Comparator.comparing(ListOfStageResultsResp::getTimeSum))).collect(Collectors.toList());


                }
                return offLineTeamPointsCompetition;
            }
        }
        return null;
    }


    /**
     * 单个阶段团队得分详情
     * 需要区分线上、线下情况
     *
     * @param req
     * @return
     */
    @Override
    public R stageTeamScoreDetails(StageTeamScoreDetailsReq req) {
        // 获取阶段信息
        CompetitionStage competitionStage = baseMapper.selectById(req.getStageId());
        //活动方式（0实操，1理论，2线下）默认为0"
        Page<ListOfStageResultsResp> page = new Page<ListOfStageResultsResp>(req.getPageNum(), req.getPageSize());
        IPage<ListOfStageResultsResp> pageList = null;
        switch (competitionStage.getMethod()) {
            case 0:
            case 1:
                pageList = teamMapper.stageTeamScoreDetails(page, req);
                break;
            case 2:
                pageList = teamMapper.offlineTeamScores(page, req);
                break;

        }
        return R.ok().put("page", pageList);

    }

    /**
     * 多阶段
     *
     * @param req
     * @return
     */
    @Override
    public R multipleStageTeamScoreDetails(StageTeamScoreDetailsReq req) {
        Page<ListOfStageResultsResp> page = new Page<ListOfStageResultsResp>(req.getPageNum(), req.getPageSize());
        IPage<ListOfStageResultsResp> pageList = teamMapper.multipleStageTeamScoreDetails(page, req);
        return R.ok().put("page", pageList);
    }

    /**
     * 总分排名得分详情
     *
     * @param req
     * @return
     */
    @Override
    public R totalRankingScoreDetails(StageTeamScoreDetailsReq req) {
        List<DetailsOfTotalTeamScoresResp> stageTeamInformation = new ArrayList<DetailsOfTotalTeamScoresResp>();
        AtomicBoolean isPointWeight = new AtomicBoolean(false);
        req.getStageIds().forEach(stageId -> {
            //获取阶段信息
            DetailsOfTotalTeamScoresResp resp = new DetailsOfTotalTeamScoresResp();
            CompetitionStage competitionStage = baseMapper.selectById(stageId);
            resp.setStageName(competitionStage.getStageName());
            resp.setPointWeight(competitionStage.getPointWeight());
            resp.setTeamCalculationMethod(competitionStage.getTeamCalculationMethod());
            resp.setStageId(stageId);
            req.setStageId(stageId);
            //req.setStageIds(Collections.singletonList(stageId));

            Page<ListOfStageResultsResp> page = new Page<ListOfStageResultsResp>(1, 1000);
            IPage<ListOfStageResultsResp> information = null;
            //获取线上线下的团队得分成绩
            switch (competitionStage.getMethod()) {
                case 0:
                case 1:
                    information = teamMapper.stageTeamScoreDetails(page, req);


                    break;
                case 2:
                    information = teamMapper.offlineTeamScores(page, req);
                    break;
            }

            resp.setStageTeamInformation(information.getRecords());
            stageTeamInformation.add(resp);
            Double score = 0.0;
            List<Double> upSort = new ArrayList<>();
            if (information.getRecords().size() > 0) {
                //累计分数
                Integer count = 0;
                for (ListOfStageResultsResp resultsResp : information.getRecords()) {
                    score += new Double(resultsResp.getScore());
                    count++;
                    upSort.add(new Double(resultsResp.getScore()));
                }

                double fractionalValue = 0.0;
                double teamScore = 0.0;


                //团队成绩计算方式（0：最高分，1：平均分，2：总分(求和)
                if (!ObjectUtil.isEmpty(competitionStage.getPointWeight())) {
                    //团队成绩计算方式（0：最高分，1：平均分，2：总分(求和)
                    switch (competitionStage.getTeamCalculationMethod()) {
                        case "0":
                            Optional<Double> max = upSort.stream().max(Comparator.comparingDouble(Double::longValue));
                            Double maxValue = max.get();
                            fractionalValue = maxValue * (resp.getPointWeight() / 100);
                            break;
                        case "1":


                            fractionalValue = div(score, count, 2) * (resp.getPointWeight() / 100);


                            break;
                        case "2":
                            fractionalValue = score * (resp.getPointWeight() / 100);
                            break;
                    }
                    BigDecimal bigDecimal = new BigDecimal(fractionalValue);
                    teamScore = bigDecimal.setScale(2, BigDecimal.ROUND_DOWN).doubleValue();
                    isPointWeight.set(true);
                } else {
                    switch (competitionStage.getTeamCalculationMethod()) {
                        case "0":
                            Optional<Double> max = upSort.stream().max(Comparator.comparingDouble(Double::longValue));
                            Double maxValue = max.get();
                            teamScore = maxValue;
                            break;
                        case "1":
                            teamScore = score / count;
                            break;
                        case "2":
                            teamScore = score;
                            break;
                    }
                    isPointWeight.set(false);
                }


                resp.setTeamScore(teamScore);
            }
        });


        return R.ok().put("data", stageTeamInformation).put("isPointWeight", isPointWeight);

    }

    /**
     * Double精确的除法运算, 当出现除不尽的情况时, 精确到小数点以后10位, 以后的数字四舍五入
     *
     * @param d1    被除数
     * @param d2    除数
     * @param scale 表示需要精确到小数点的后几位
     * @return 两个参数的商
     */
    public static double div(double d1, double d2, int scale) {
        if (scale < 0) {
            throw new IllegalArgumentException("参数[scale]必须是正整数或者零");
        }
        BigDecimal value1 = new BigDecimal(Double.toString(d1));
        BigDecimal value2 = new BigDecimal(Double.toString(d2));
        return value1.divide(value2, scale, BigDecimal.ROUND_DOWN).doubleValue();
    }

    @Override
    public void derivedRanking(List<RankingExportVO> list, HttpServletResponse response) throws IOException {
        // 告诉浏览器用什么软件可以打开此文件
        response.setHeader("content-Type", "application/vnd.ms-excel");
        // 下载文件的默认名称
        response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode("活动阶段成绩导出", "UTF-8") + ".xls");
        //编码
        response.setCharacterEncoding("UTF-8");
        Workbook workbook = null;
        workbook = ExcelExportUtil.exportExcel(new ExportParams(), RankingExportVO.class, list);
        ExportParams exportParams = new ExportParams("团队成绩排名表", "sheet1", ExcelType.XSSF);
        exportParams.setStyle(ExcelStyleUtil.class);
        workbook.write(response.getOutputStream());
    }

    @Override
    public void individualDerivedRanking(List<PersonageRankingExportVO> list, HttpServletResponse response) throws IOException {
        // 告诉浏览器用什么软件可以打开此文件
        response.setHeader("content-Type", "application/vnd.ms-excel");
        // 下载文件的默认名称
        response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode("活动阶段成绩导出", "UTF-8") + ".xls");
        //编码
        response.setCharacterEncoding("UTF-8");
        Workbook workbook = null;
        workbook = ExcelExportUtil.exportExcel(new ExportParams(), PersonageRankingExportVO.class, list);
        ExportParams exportParams = new ExportParams("个人成绩排名表", "sheet1", ExcelType.XSSF);
        exportParams.setStyle(ExcelStyleUtil.class);
        workbook.write(response.getOutputStream());
    }


    /* 返回三个数中的非0最小值，全为零则返回0 */
    int minimum(int num1, int num2, int num3) {
        if (num1 == 0) {
            if (num2 == 0) {
                if (num3 == 0) return 0;
                else return num3;
            } else {
                if (num3 == 0) return num2;
                else return (num2 < num3) ? num2 : num3;
            }
        } else {
            if (num2 == 0) {
                if (num3 == 0) return num1;
                else return (num1 < num3) ? num1 : num3;
            } else {
                if (num3 == 0) return (num1 < num2) ? num1 : num2;
                else {
                    int min = (num1 < num2) ? num1 : num2;
                    min = (min < num3) ? min : num3;

                    return min;
                }
            }
        }

    }
}


