package com.huoran.competition.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Maps;
import com.huoran.common.response.R;
import com.huoran.competition.entity.*;
import com.huoran.competition.entity.req.BatchUploadListCompetitionReq;
import com.huoran.competition.entity.req.FrontDeskStageTeamScoreDetailsReq;
import com.huoran.competition.entity.req.TransferTeamReq;
import com.huoran.competition.entity.resp.ListOfStageResultsResp;
import com.huoran.competition.entity.resp.ScoreReportResp;
import com.huoran.competition.entity.vo.PersonalDetailVO;
import com.huoran.competition.mapper.CompetitionRuleMapper;
import com.huoran.competition.mapper.CompetitionTeamMapper;
import com.huoran.competition.mapper.CompleteCompetitionSetupMapper;
import com.huoran.competition.mapper.TeamInstructorMapper;
import com.huoran.competition.service.CompetitionRankingService;
import com.huoran.competition.service.CompetitionStageService;
import com.huoran.competition.service.CompetitionTeamService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.script.ScriptException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Objects;

/**
 * <p>
 * 活动团队表 服务实现类
 * </p>
 *
 * @author cheney
 * @since 2022-12-02
 */
@Service
public class CompetitionTeamServiceImpl extends ServiceImpl<CompetitionTeamMapper, CompetitionTeam> implements CompetitionTeamService {

    @Autowired
    TeamInstructorMapper teamInstructorMapper;

    @Autowired
    CompetitionRuleMapper competitionRuleMapper;

    @Autowired
    CompleteCompetitionSetupMapper completeCompetitionSetupMapper;

    @Autowired
    CompetitionRankingServiceImpl competitionRankingService;

    @Autowired
    UserScoreServiceImpl userScoreService;

    @Autowired
    private CompetitionStageService stagService;

    @Autowired
    private CompetitionRankingService rankingService;

    @Override
    public R captainOfTransfer(TransferTeamReq transferTeamReq) {
        //队长id
        Integer captainId = transferTeamReq.getCaptainId();
        //队员id
        Integer playerId = transferTeamReq.getPlayerId();

        CompetitionTeam team = baseMapper.selectById(captainId);
        CompetitionTeam team1 = baseMapper.selectById(playerId);
        Integer accountId = team.getAccountId();
        Integer accountId1 = team1.getAccountId();
        //账号id互换
        team.setAccountId(accountId1);
        baseMapper.updateById(team);
        team1.setAccountId(accountId);
        int update = baseMapper.updateById(team1);
        return update > 0 ? R.ok() : R.error();
    }

    @Override
    public R entryInformation(Integer competitionId,String accountId,int background) throws ScriptException {
        HashMap<String, Object> data = Maps.newHashMap();
//        查询活动类型
        QueryWrapper<CompleteCompetitionSetup> setupQueryWrapper = new QueryWrapper<>();
        setupQueryWrapper.eq("competition_id",competitionId);
        CompleteCompetitionSetup setup = completeCompetitionSetupMapper.selectOne(setupQueryWrapper);
//      活动类型（0为个人赛，1为团队赛）默认为0
        Integer competitionType = setup.getCompetitionType();
        //查询团队是否有限制成员参加一项阶段活动
        QueryWrapper<CompetitionRule> ruleQueryWrapper = new QueryWrapper<>();
        ruleQueryWrapper.eq("setup_id",setup.getSetupId());
        CompetitionRule competitionRule = competitionRuleMapper.selectOne(ruleQueryWrapper);

        Integer teamId = 0;
        if (competitionType == 1){
//        查询团队基本信息、团队成员
            QueryWrapper<CompetitionTeam> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("competition_id",competitionId);
            queryWrapper.eq("account_id",accountId);
            CompetitionTeam team = baseMapper.selectOne(queryWrapper);
            if (ObjectUtil.isNotNull(team)){

                Integer captain = team.getCaptain();
//        队长（0是队长，1为队员）默认为0,查询全队的账号信息
                List<Integer> accountIds = new ArrayList<>();
                if (captain==0){
                    accountIds.add(team.getAccountId());
                    QueryWrapper<CompetitionTeam> queryWrapper2 = new QueryWrapper<>();
                    queryWrapper2.eq("pid",team.getTeamId());
                    teamId = team.getTeamId();
                    List<CompetitionTeam> competitionTeams = baseMapper.selectList(queryWrapper2);
                    if (!competitionTeams.isEmpty()){
                        for (CompetitionTeam competitionTeam : competitionTeams) {
                            accountIds.add(competitionTeam.getAccountId());
                        }
                    }
                }else {
                    CompetitionTeam team1 = baseMapper.selectById(team.getPid());
                    accountIds.add(team1.getAccountId());
                    QueryWrapper<CompetitionTeam> queryWrapper2 = new QueryWrapper<>();
                    if (background == 1){
                        team = team1;
                        queryWrapper2.eq("pid",team.getTeamId());
                        teamId = team.getTeamId();
                    }else {
                        queryWrapper2.eq("pid",team.getPid());
                        teamId = team.getPid();
                        team.setInvitationCode(team1.getInvitationCode());
                        team.setTeamName(team1.getTeamName());
                    }
                    List<CompetitionTeam> competitionTeams = baseMapper.selectList(queryWrapper2);
                    if (!competitionTeams.isEmpty()){
                        for (CompetitionTeam competitionTeam : competitionTeams) {
                            accountIds.add(competitionTeam.getAccountId());
                        }
                    }
                }
//            团队详情信息
                List<CompetitionTeam> competitionTeams =  baseMapper.teamDetail(accountIds,competitionId);
                data.put("teamDetail",competitionTeams);
            }

//            限制提示
            boolean teamLimit = competitionRule.getTeamLimit() == 1;

            data.put("teamId",teamId);
            data.put("team",team);
            data.put("teamLimit",teamLimit);

        }else {
//            个人信息
            PersonalDetailVO personalDetail = baseMapper.personalDetail(accountId);
            data.put("personalDetail",personalDetail);
        }

        QueryWrapper<TeamInstructor> queryWrapper3 = new QueryWrapper<>();
        if (competitionType == 0){
            queryWrapper3.eq("competition_id",competitionId);
            queryWrapper3.eq("account_id",accountId);
        }else {
            queryWrapper3.eq("competition_id",competitionId);
            queryWrapper3.eq("team_id",teamId);
        }
//        查询指导老师
        List<TeamInstructor> teamInstructors = teamInstructorMapper.selectList(queryWrapper3);

//        活动阶段信息
        List<CompetitionStage> stages = null;
        if (competitionType == 0){
            stages = baseMapper.competitionStage(competitionId,accountId,null);
        }else {
            stages = baseMapper.competitionStage(competitionId,accountId,teamId);
        }

        //阶段展示成绩分数
        for (CompetitionStage stage : stages) {
            //查询成绩、报告id需要判重，学生可能没资格参加  b是否公布成绩
            boolean b = userScoreService.publicationResult(stage.getStageId());
            if (competitionType == 0){
                teamId = null;
            }

            if (b){
                //积分赛制，查询总分
                if (competitionRule.getRule()==0){
                    List<CompetitionStage> stageList = baseMapper.competitionStage(competitionId, accountId, teamId);
                    StringBuffer sb = new StringBuffer();
//                    List<Integer> stages = new ArrayList<>();
                    if (stageList!=null && stageList.size()>0){
                        for (CompetitionStage competitionStage : stageList) {
                            sb.append(competitionStage.getStageId()).append(",");
//                            stages.add(competitionStage.getStageId());
                        }
                        sb.deleteCharAt(sb.length()-1);
                    }

                    FrontDeskStageTeamScoreDetailsReq req = new FrontDeskStageTeamScoreDetailsReq();
                    req.setStageIds(sb.toString());
                    req.setCompetitionId(competitionId);
                    req.setIsOverallRanking(1);
                    if (competitionType == 0){
                        req.setType(1);
                        req.setAccountId(accountId);
                    }else {
                        req.setTeamId(teamId);
                    }
                    req.setPageNum(1);
                    req.setPageSize(1000);
                    R r = competitionRankingService.frontOfficeCompetitionRanking(req);
                    double totalScore = 0;
                    List<ListOfStageResultsResp> respList =(List<ListOfStageResultsResp>) r.get("list");
                    //排名发布数据，无发布数据取系统数据
                    if (respList.size()>0){
                        for (ListOfStageResultsResp resultsResp : respList) {
                            totalScore += Double.valueOf(resultsResp.getScore());
                        }
                    }

                    data.put("totalScore",totalScore);
                }

                //团队赛返回
                if (competitionType == 1){

                    double score = 0;

                    //查询手动上传数据
                    BatchUploadListCompetitionReq req = new BatchUploadListCompetitionReq();
                    req.setCompetitionId(competitionId);
                    req.setStageId(stage.getStageId());
                    req.setPageSize(100);
                    req.setPageNum(1);
                    req.setIsOverallRanking(0);
                    R r1 = rankingService.manuallyRankTheUploadList(req);
                    boolean publishStatus = (boolean) r1.get("publishStatus");
                    if (publishStatus){
                        List<ListOfStageResultsResp> respList = (List<ListOfStageResultsResp>) r1.get("page");
                        for (ListOfStageResultsResp resultsResp : respList) {
                            if (resultsResp.getTeamId().equals(teamId)){
                                score += Double.valueOf(resultsResp.getScore());
                                if (stage.getResultsDetails()==0){
                                    stage.setReportId(resultsResp.getReportId());
                                }
                            }
                        }
                    }else {
                        //没有发布数据取系统
                        R r = stagService.stageRaceRanking(competitionId, stage.getStageId(), 1, 100, 0);
                        List<ListOfStageResultsResp> subList1 = (List<ListOfStageResultsResp>) r.get("page");
                        for (ListOfStageResultsResp resultsResp : subList1) {
                            if (resultsResp.getTeamId().equals(teamId)){
                                score += Double.valueOf(resultsResp.getScore());
                                if (stage.getResultsDetails()==0){
                                    stage.setReportId(resultsResp.getReportId());
                                }
                            }
                        }
                    }
                    stage.setScore(score);

                }else {
                    //查询成绩、成绩报告id(发布排名数据)
                    ScoreReportResp reportResp = baseMapper.queryResults(stage.getStageId(),competitionId,accountId,teamId);
                    //公布成绩，排名表无数据，查询系统数据
                    if (reportResp!=null && reportResp.getScore()!=null){
                        stage.setScore(reportResp.getScore().doubleValue());
                        if (stage.getResultsDetails()==0){
                            stage.setReportId(reportResp.getReportId());
                        }
                    }else {
                        R r = stagService.stageRaceRanking(competitionId, stage.getStageId(), 1, 100, 0);
                        List<ListOfStageResultsResp> subList1 = (List<ListOfStageResultsResp>) r.get("page");
                        double score = 0;
                        if (subList1.size()>0){
                            for (ListOfStageResultsResp resultsResp : subList1) {
                                if (Objects.equals(resultsResp.getAccountId(), accountId)){
                                    score += Double.valueOf(resultsResp.getScore());
                                    if (stage.getResultsDetails()==0){
                                        stage.setReportId(resultsResp.getReportId());
                                    }
                                }
                            }
                        }
                        stage.setScore(score);
                    }
                }

            }else {
                stage.setScore(-1d);
            }
        }

        data.put("stages",stages);
        data.put("teamInstructors",teamInstructors);
        return R.ok().put("entryInformation",data);
    }
}
