package com.huoran.competition.service.impl;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import cn.afterturn.easypoi.excel.entity.enmus.ExcelType;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huoran.api.NakadaiClient;
import com.huoran.common.entity.School;
import com.huoran.common.entity.UserScore;
import com.huoran.common.exception.CustomException;
import com.huoran.common.exception.ExceptionEnum;
import com.huoran.common.response.R;
import com.huoran.competition.entity.CompetitionContent;
import com.huoran.competition.entity.CompetitionStage;
import com.huoran.competition.entity.CompetitionTeam;
import com.huoran.competition.entity.req.ExcelImpGradeReq;
import com.huoran.competition.entity.req.PageGradeListReq;
import com.huoran.competition.entity.resp.FractionalSegmentResp;
import com.huoran.competition.entity.resp.ListOfStageResultsResp;
import com.huoran.competition.entity.resp.UserAccountResp;
import com.huoran.competition.entity.vo.GradeExportVO;
import com.huoran.competition.entity.vo.ImportCompetitionGradeFailureVO;
import com.huoran.competition.mapper.CompetitionContentMapper;
import com.huoran.competition.mapper.CompetitionStageMapper;
import com.huoran.competition.mapper.CompetitionTeamMapper;
import com.huoran.competition.mapper.UserScoreMapper;
import com.huoran.competition.service.UserScoreService;
import com.huoran.competition.utils.DateUtil;
import com.huoran.competition.utils.ExcelImporEventResultstHelper;
import com.huoran.competition.utils.ExcelStyleUtil;
import com.huoran.competition.utils.ScoreGroupingUtil;
import com.huoran.competition.utils.poi.CommonCode;
import com.huoran.competition.utils.poi.EasyPoiUtil;
import com.huoran.competition.utils.poi.ExceptionCast;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLEncoder;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * @描述：用户分数表 服务类
 * @作者: Rong
 * @日期: 2022-12-08
 */
@Service
public class UserScoreServiceImpl extends ServiceImpl<UserScoreMapper, UserScore> implements UserScoreService {
    @Resource
    StringRedisTemplate stringRedisTemplate;
    @Autowired
    private UserScoreMapper mapper;

    @Autowired
    private CompetitionTeamMapper teamMapper;

    @Autowired
    private CompetitionStageMapper competitionStageMapper;

    @Autowired
    private CompetitionContentMapper competitionContentMapper;

    @Autowired
    private NakadaiClient nakadaiClient;

    @Override
    public Map<String, String> batchImportGrades(MultipartFile file, Integer competitionId, Integer systemId, Integer stageId, Integer schoolId) throws IOException {
        List<ExcelImpGradeReq> impGradeReqList = ExcelImporEventResultstHelper.readGrade(file);
        if (impGradeReqList.size() <= 0) {
            //小于等于0为上传空模板的情况下 抛出异常
            throw new CustomException(ExceptionEnum.EXCEL_FILE_FORMAT_ERROR);
        }
        int count = 0;
        List<ImportCompetitionGradeFailureVO> failVo1 = new ArrayList<ImportCompetitionGradeFailureVO>();
        // 参数合法性校验，只能上传.xlsx后缀的文件
        if (org.apache.commons.lang3.StringUtils.isBlank(file.getOriginalFilename()) || !file.getOriginalFilename().substring(file.getOriginalFilename().lastIndexOf(".")).equals(".xlsx")) {
            ExceptionCast.cast(CommonCode.EXCEL_FILE_INVALID);
        }
        int size = impGradeReqList.size();
        Map<String, String> map = new HashMap<>();
        Long ii = 1L;//用于记录序列号
        Integer countNum = 0;//用于失败原因排序 eg:1、XXXXX 2、XXXXXXX
        try {
            Integer countSuccess = 0;//统计成功次数
            Integer countError = 0;//统计失败次数

            List<String> workNumberList = new ArrayList<>();//用来储存新增前用户在excel中输入的所有工号用于excel中的数据判重
            for (int i = 0; i < impGradeReqList.size(); i++) {
                boolean ret = true;
                ++ii;
                ExcelImpGradeReq req = impGradeReqList.get(i);

                ImportCompetitionGradeFailureVO vo1 = new ImportCompetitionGradeFailureVO();

                String errMsg = "";
                //去除Excel中的全角半角空格
                String getTeamName = req.getTeamName().replaceAll("\\u00A0", "").trim();
                String getUserName = req.getUserName().replaceAll("\\u00A0", "").trim();
                String getWorkNumber = req.getWorkNumber().replaceAll("\\u00A0", "").trim();
                String getScore = req.getScore().replaceAll("\\u00A0", "").trim();
                String getTimeSum = req.getTimeSum().replaceAll("\\u00A0", "").trim();
                String getSchoolName = req.getSchoolName().replaceAll("\\u00A0", "").trim();
                //todo 增加学校字段 (根据学校名称查询学校id)
                School school = nakadaiClient.getTheIdBySchoolName(getSchoolName);
                if (school != null) {
                    schoolId = school.getSchoolId();
                }

//                Date getSubmitTime = req.getSubmitTime();
//                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
//                String format = sdf.format(getSubmitTime);
                if (!workNumberList.contains(req.getWorkNumber())) {
                    workNumberList.add(req.getWorkNumber());
                } else {
                    if (!getWorkNumber.equals("")) {
                        //包含excel数据中存在相同工号
                        ++countNum;
                        vo1.setWorkNumber(getWorkNumber + " *必填项:(当前Excel数据中该学号已存在重复的!)");
                        errMsg += countNum + " *必填项:(当前Excel数据中该学号已存在重复的)";
                        ret = false;
                    }

                }


                vo1.setIndex(ii);

                //校验学号是否存在汉字
                if (!getWorkNumber.trim().equals("")) {
                    if (isContainChinese(getWorkNumber)) {
                        ++countNum;
                        vo1.setWorkNumber(getWorkNumber + "* 学号不能存在汉字!");
                        errMsg += countNum + " * 学号不能存在汉字!";
                        ret = false;
                        countError++;
                    }
                }


                //检验工号
                if (getWorkNumber.equals("")) {
                    ++countNum;
                    vo1.setWorkNumber(getWorkNumber + " *必填项:(学号不能为空)");
                    errMsg += countNum + " *必填项:(学号不能为空)";
                    ret = false;
                    countError++;
                } else {
                    vo1.setWorkNumber(getWorkNumber);
                }


                //检验姓名
                if (getUserName.equals("")) {
                    ++countNum;
                    vo1.setUserName(getUserName + " *必填项:(姓名不能为空) ");
                    errMsg += countNum + "必填项:(姓名不能为空)";
                    ret = false;
                    countError++;
                } else {
                    vo1.setUserName(getUserName);
                }

                UserAccountResp checkStudentNumberAndName = null;
                //校验学号和姓名是否为同一个人 如果是同一人则获取到账号id
                if (!getUserName.equals("") && !getWorkNumber.equals("")) {
                    //checkStudentNumberAndName检测用户名和学号是否是同一人
                    checkStudentNumberAndName = baseMapper.checkStudentNumberAndName(getUserName, getWorkNumber, schoolId);
                    if (checkStudentNumberAndName == null) {
                        if (!getUserName.equals("") && !getWorkNumber.equals("")) {
                            ++countNum;
                            vo1.setWorkNumber(getWorkNumber + " *必填项:(学号与学生姓名不匹配)");
                            vo1.setUserName(getUserName + " *必填项:(学生姓名与学号不匹配!)");
                            errMsg += countNum + " *必填项:(学号与学生姓名不匹配!)";
                            ret = false;
                            countError++;
                            /*if (!getTeamName.equals("")) {
                                ++countNum;
                                vo1.setTeamName(getTeamName + "* 请填入正确的用户信息以及团队信息!");
                                errMsg += countNum + " * 请填入正确的用户信息以及团队信息!";
                                ret = false;
                                countError++;
                            }else {
                                vo1.setTeamName(getTeamName);
                            }*/
                        }
                    }
                }
                CompetitionTeam verifyThatStudentsAreInTeam = null;


                //校验依据活动查团队信息以及该用户是否存在该团队中
                if (!getTeamName.equals("")) {
                    if (checkStudentNumberAndName != null) {
                        CompetitionTeam team = new CompetitionTeam();
                        team.setAccountId(checkStudentNumberAndName.getId());
                        team.setCompetitionId(competitionId);

                        verifyThatStudentsAreInTeam = teamMapper.verifyThatStudentsAreInTeam(team);
                        //判断改团队是否存在
                        String teamName = teamMapper.getTheTeamName(competitionId, getTeamName);

                        if (ObjectUtils.isEmpty(verifyThatStudentsAreInTeam)) {
                            ++countNum;
                            vo1.setTeamName(getTeamName + "* 该用户不在此团队中!");
                            errMsg += countNum + " * 该用户不在此团队中!";
                            ret = false;
                            countError++;
                        } else {
                            vo1.setTeamName(getTeamName);
                        }


                        if ( ObjectUtils.isEmpty(teamName)) {
                            if (!teamName.equals(verifyThatStudentsAreInTeam.getTeamName())) {
                                ++countNum;
                                vo1.setTeamName(getTeamName + "* 该用户不在此团队中!");
                                errMsg += countNum + " -* 该用户不在此团队中!";
                                ret = false;
                                countError++;

                            } else {
                                vo1.setTeamName(getTeamName);
                            }

                        }
                    } else {
                        vo1.setTeamName(getTeamName);
                    }

                }


                //检验分数
                if (getScore.equals("")) {
                    ++countNum;
                    vo1.setScore(getScore + " *必填项:(分数不能为空) ");
                    errMsg += countNum + "必填项:(分数不能为空)";
                    ret = false;
                    countError++;
                } else {
                    vo1.setScore(getScore);
                }


                //检验耗时
                if (getTimeSum.equals("")) {
                    ++countNum;
                    vo1.setScore(getTimeSum + " *必填项:(耗时不能为空) ");
                    errMsg += countNum + "必填项:(耗时不能为空)";
                    ret = false;
                    countError++;
                } else {
                    vo1.setTimeSum(getTimeSum);
                }

                //检验提交时间
                /*if (getSubmitTime == null || getSubmitTime.equals("")) {
                    ++countNum;

                    vo1.setScore(format + " *必填项:(提交时间不能为空) ");
                    errMsg += countNum + "必填项:(提交时间不能为空)";
                    ret = false;
                    countError++;
                } else {
                    vo1.setSubmitTime(format);
                }*/


                vo1.setFailureMsg(errMsg);
                countNum = 0;

                failVo1.add(vo1);

                //ret判断 新增成功的插入到ol_user_score表 否则则不新增至数据库
                if (ret) {
                    //执行新增到操作

                    UserScore userScore = new UserScore();
                    userScore.setScore(Integer.valueOf(getScore));
                    if (verifyThatStudentsAreInTeam != null) {
                        if (verifyThatStudentsAreInTeam.getCaptain() == 0) {
                            userScore.setTeamId(verifyThatStudentsAreInTeam.getTeamId());
                        } else {
                           /* QueryWrapper<CompetitionTeam> queryWrapper = new QueryWrapper<>();
                            queryWrapper.eq("id",verifyThatStudentsAreInTeam.getPid());
                            queryWrapper.eq("is_del", DelConstant.NOT_DEL);*/
                            CompetitionTeam team = teamMapper.selectById(verifyThatStudentsAreInTeam.getPid());
                            userScore.setTeamId(team.getTeamId());
                        }

                    }


                    userScore.setAccountId(checkStudentNumberAndName.getId());
                    userScore.setSystemId(systemId);
                    userScore.setCompetitionId(competitionId);//活动id
                    userScore.setTimeSum(Integer.valueOf(getTimeSum));//耗时

                    String getsTheStartTimeOfThePhase = baseMapper.getsTheStartTimeOfThePhase(stageId);
                    Date time = DateUtil.addDateMinute(getsTheStartTimeOfThePhase, Integer.valueOf(getTimeSum));
                    userScore.setSubmitTime(time);//提交时间

                    userScore.setStageId(stageId);
                    userScore.setCreateTime(new Date());
                    userScore.setUpdateTime(new Date());

                    //上传前判断是都存在 存在则覆盖老数据
                    QueryWrapper<UserScore> queryWrapper = new QueryWrapper<>();
                    queryWrapper.eq("competition_id", competitionId);
                    queryWrapper.eq("stage_id", userScore.getStageId());
                    queryWrapper.eq("account_id", userScore.getAccountId());
                    if (userScore.getTeamId() != null) {
                        queryWrapper.eq("team_id", userScore.getTeamId());
                    }
                    UserScore checkWhetherItExists = baseMapper.selectOne(queryWrapper);
                    if (checkWhetherItExists != null) {
                        //删除老的数据
                        baseMapper.deleteById(checkWhetherItExists.getId());
                    }

                    baseMapper.insert(userScore);


                    //countSuccess：统计累计成功次数
                    countSuccess++;
                }/* else {
                    //countError:累计失败次数
                    countError++;
                }*/
            }
            String exportCode = "";

            // 有导入失败的数据，才会存入redis
            if (countError > 0) {
                //生成token
                exportCode = "FAILURE_IMPORT" + UUID.randomUUID().toString().replace("-", "");
                ValueOperations<String, String> ops = stringRedisTemplate.opsForValue();

                List<ImportCompetitionGradeFailureVO> collect = failVo1.stream().filter(importStaffFailureVo -> {
                    return !importStaffFailureVo.getFailureMsg().equals("");
                }).collect(Collectors.toList());

                String failureVOJson = JSON.toJSONString(collect);
                ops.set(exportCode, failureVOJson, 30 * 60, TimeUnit.SECONDS);
            }
            map.put("exportCode", exportCode);//返回导出code
            map.put("successNum", countSuccess + "");//本次新增成功数量
            map.put("failureNum", countError + "");//本次新增失败数量
            return map;
        } catch (RuntimeException e) {
            System.err.println(e.getMessage());
            throw new RuntimeException();
        }
    }

    @Override
    public void exportFailureRecord(HttpServletResponse response, String token) throws Exception {
        if (org.apache.commons.lang3.StringUtils.isEmpty(token)) {
            return;
        }
        ValueOperations<String, String> ops = stringRedisTemplate.opsForValue();
        //获取数据
        String record = ops.get(token);
        if (org.apache.commons.lang3.StringUtils.isEmpty(record)) {
            return;
        }
        List<ImportCompetitionGradeFailureVO> parse = JSON.parseObject(record, new TypeReference<List<ImportCompetitionGradeFailureVO>>() {
        });

        parse.sort(Comparator.comparing(ImportCompetitionGradeFailureVO::getIndex));

        // 告诉浏览器用什么软件可以打开此文件
        response.setHeader("content-Type", "application/vnd.ms-excel");
        // 下载文件的默认名称
        response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode("活动成绩导入失败数据表", "UTF-8") + ".xls");
        //编码
        response.setCharacterEncoding("UTF-8");
        Workbook workbook = ExcelExportUtil.exportExcel(new ExportParams(), ImportCompetitionGradeFailureVO.class, parse);
        ExportParams exportParams = new ExportParams(null, "delivery", ExcelType.XSSF);
        exportParams.setStyle(ExcelStyleUtil.class);
        workbook.write(response.getOutputStream());
    }

    @Override
    public R stageGradeManagementList(PageGradeListReq req) {
        if (req.getIsNakadai() == 0) {
            boolean result = publicationResult(req.getStageId());
            if (result) {
                req.setIsNakadai(1);
            }
        }
        Page<ListOfStageResultsResp> page = new Page<ListOfStageResultsResp>(req.getPageNum(), req.getPageSize());
        PageGradeListReq statistics = new PageGradeListReq();
        BeanUtils.copyProperties(req, statistics);
        statistics.setKeyWord(null);
        Integer setTotalNumber = Math.toIntExact(mapper.stageGradeManagementList(page, req).getTotal());

        IPage<ListOfStageResultsResp> scoreGrouping = mapper.stageGradeManagementList(page, statistics);
        FractionalSegmentResp data = ScoreGroupingUtil.scoreGrouping(scoreGrouping);
        data.setTotalNumber(setTotalNumber);
        IPage<ListOfStageResultsResp> pageList = mapper.stageGradeManagementList(page, req);
        pageList.getRecords().sort(new Comparator<ListOfStageResultsResp>() {
            @Override
            public int compare(ListOfStageResultsResp o1, ListOfStageResultsResp o2) {
                return (int) (Double.valueOf(o2.getScore()) - Double.valueOf(o1.getScore()));
            }
        });
        return R.ok().put("page", pageList).put("data", data);


    }

    /**
     * 添加日期小时
     */
    public Date addDateHour(Date date, int hour) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        cal.add(Calendar.HOUR, hour);// 24小时制
//        cal.add(Calendar.MINUTE, hour);// 分钟
        date = cal.getTime();
        //释放对象
        cal = null;
        return date;
    }

    /**
     * 是否公布成绩
     */
    public boolean publicationResult(Integer stageId) {
        CompetitionStage competitionStage = competitionStageMapper.selectById(stageId);
        if (competitionStage.getResultAnnouncementTime()==null){
            return false;
        }
        //成绩公布时间
        int resultAnnouncementTime = competitionStage.getResultAnnouncementTime();
        QueryWrapper<CompetitionContent> wrapper = new QueryWrapper<>();
        wrapper.eq("stage_id", stageId);
        CompetitionContent competitionContent = competitionContentMapper.selectOne(wrapper);
        if (competitionContent != null) {
            Date endTime = competitionContent.getEndTime();

            Date date = addDateHour(endTime, resultAnnouncementTime);
            //公布时间
            long publicationTime = date.getTime();
            //当前系统时间
            long currentTimeMillis = System.currentTimeMillis();
            return currentTimeMillis >= publicationTime;
        }
        return false;
    }


    @Override
    public R stageReportOfExperimentList(PageGradeListReq req) {
        if (req.getIsNakadai() == 0) {
            boolean result = publicationResult(req.getStageId());
            if (result) {
                req.setIsNakadai(1);
            }
        }

        Page<ListOfStageResultsResp> page = new Page<ListOfStageResultsResp>(req.getPageNum(), req.getPageSize());
        PageGradeListReq statistics = new PageGradeListReq();
        BeanUtils.copyProperties(req, statistics);
        statistics.setKeyWord(null);
        Integer setTotalNumber = Math.toIntExact(mapper.stageReportOfExperimentList(page, statistics).getTotal());
        IPage<ListOfStageResultsResp> scoreGrouping = mapper.stageReportOfExperimentList(page, statistics);

        Page<ListOfStageResultsResp> page1 = new Page<ListOfStageResultsResp>(1, setTotalNumber);
        IPage<ListOfStageResultsResp> allScoreGrouping = mapper.stageReportOfExperimentList(page1, statistics);



        FractionalSegmentResp data = ScoreGroupingUtil.scoreGrouping(allScoreGrouping);
        IPage<ListOfStageResultsResp> pageList = mapper.stageReportOfExperimentList(page, req);
        data.setTotalNumber(setTotalNumber);
        pageList.getRecords().sort(new Comparator<ListOfStageResultsResp>() {
            @Override
            public int compare(ListOfStageResultsResp o1, ListOfStageResultsResp o2) {
                return (int) (Double.valueOf(o2.getScore()) - Double.valueOf(o1.getScore()));
            }
        });
        return R.ok().put("page", pageList).put("data", data);
    }

    @Override
    public List<ListOfStageResultsResp> getUnpaginatedEventResults(PageGradeListReq req) {
        Page<ListOfStageResultsResp> page = new Page<ListOfStageResultsResp>(req.getPageNum(), req.getPageSize());
        PageGradeListReq statistics = new PageGradeListReq();
        BeanUtils.copyProperties(req, statistics);
        statistics.setKeyWord(null);
        Integer setTotalNumber = Math.toIntExact(mapper.stageReportOfExperimentList(page, statistics).getTotal());
        IPage<ListOfStageResultsResp> scoreGrouping = mapper.stageReportOfExperimentList(page, statistics);
        FractionalSegmentResp data = ScoreGroupingUtil.scoreGrouping(scoreGrouping);
        IPage<ListOfStageResultsResp> pageList = mapper.stageReportOfExperimentList(page, req);
        data.setTotalNumber(setTotalNumber);
        pageList.getRecords().sort(new Comparator<ListOfStageResultsResp>() {
            @Override
            public int compare(ListOfStageResultsResp o1, ListOfStageResultsResp o2) {
                return (int) (Double.valueOf(o2.getScore()) - Double.valueOf(o1.getScore()));
            }
        });
        return pageList.getRecords();
    }


    @Override
    public void exportExperimentalResultsInBatch(List<GradeExportVO> list, HttpServletResponse response) throws IOException {
        // 告诉浏览器用什么软件可以打开此文件
        response.setHeader("content-Type", "application/vnd.ms-excel");
        // 下载文件的默认名称
        response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode("活动阶段成绩导出", "UTF-8") + ".xls");
        //编码
        response.setCharacterEncoding("UTF-8");
        /**
         *  动态隐藏列
         */
        Integer count =0;
        for (GradeExportVO item : list) {
            item.setSerialNumber(++count);
            EasyPoiUtil<GradeExportVO> easyPoiUtil = new EasyPoiUtil<>();
            easyPoiUtil.t = item;
            try {
                if (item.getTeamName() == null || "".equals(item.getTeamName())) {
                    //隐藏bean 字段为teamName的列
                    easyPoiUtil.hihdColumn("teamName", true);
                } else {
                    easyPoiUtil.hihdColumn("teamName", false);
                }
            } catch (Exception e) {
                System.out.println("列隐藏转换失败：{}" + e.getMessage());
                e.printStackTrace();
            }
        }
        Workbook workbook = null;
        workbook = ExcelExportUtil.exportExcel(new ExportParams(), GradeExportVO.class, list);
        ExportParams exportParams = new ExportParams("阶段成绩管理表", "sheet1", ExcelType.XSSF);
        exportParams.setStyle(ExcelStyleUtil.class);
        workbook.write(response.getOutputStream());
    }

    @Override
    public boolean batchDeletionOfExperimentalReports(List<Integer> ids) {
        Integer ret = baseMapper.batchDeletionOfExperimentalReports(ids);
        return ret > 0;
    }


    /**
     * @Description :校验是否存在文字 为true表示存在 false表示不存在
     * @Param str
     * @Author Rong---2021/11/4
     */
    public static boolean isContainChinese(String str) {
        Pattern p = Pattern.compile("[\u4e00-\u9fa5]");
        Matcher m = p.matcher(str);
        if (m.find()) {
            return true;
        }
        return false;
    }
}



