/**
 * Copyright (c) 2016-2019 人人开源 All rights reserved.
 * <p>
 * https://www.renren.io
 * <p>
 * 版权所有，侵权必究！
 */

package com.huoran.competition.utils;

/**
 * 常量
 *
 * @author Rong
 */
public class Constant {

    /**
     * 测评是否通过 1、通过/0、未通过
     */
    public static final String EVALUATIONN_PASSED = "通过";
    public static final String EVALUATIONN_NOT_PASSED = "未通过";

    /**
     * 及格分数
     */
    public static final int PASSING_SCORE = 60;

    /**
     * 当前题目答案是否正确 1、正确/0、错误
     */
    public enum QuestionIsTure {

        TRUE(1, "正确"),
        FALSE(0, "错误");

        private Integer type;
        private String desc;

        QuestionIsTure(Integer type, String desc) {
            this.type = type;
            this.desc = desc;
        }

        public Integer getType() {
            return type;
        }

        public void setType(Integer type) {
            this.type = type;
        }

        public String getDesc() {
            return desc;
        }

        public void setDesc(String desc) {
            this.desc = desc;
        }
    }

    /**
     * 试题类型
     */
    public enum QuestionType {
        /**
         * 单选题
         */
        SINGLE_CHOICE("1", "单选题"),
        /**
         * 多选题
         */
        MULTIPLE_CHOICE("2", "多选题"),
        /**
         * 判断题
         */
        TRUE_OR_FALSE("3", "判断题");

        private String type;    //类型
        private String desc;    //描述

        QuestionType(String type, String desc) {
            this.type = type;
            this.desc = desc;
        }

        public String getType() {
            return type;
        }

        public void setType(String type) {
            this.type = type;
        }

        public String getDesc() {
            return desc;
        }

        public void setDesc(String desc) {
            this.desc = desc;
        }

    }

    /**
     * 项目是否提交1：已提交/0：未提交
     */
    public enum Submit {

        SUBMIT(1, "已提交"),
        NOT_SUBMIT(0, "未提交");


        private Integer type;
        private String desc;

        Submit(Integer type, String desc) {
            this.type = type;
            this.desc = desc;
        }

        public Integer getType() {
            return type;
        }

        public void setType(Integer type) {
            this.type = type;
        }

        public String getDesc() {
            return desc;
        }

        public void setDesc(String desc) {
            this.desc = desc;
        }
    }

    /**
     * 是否删除：0使用，1删除，默认0使用
     */
    public enum IsDel {
        /**
         * 删除
         */
        DEL(1, "已删除"),
        /**
         * 不删除
         */
        NOT_DEL(0, "不删除");

        private Integer type;    //类型
        private String desc;    //描述

        IsDel(Integer type, String desc) {
            this.type = type;
            this.desc = desc;
        }

        public Integer getType() {
            return type;
        }

        public void setType(Integer type) {
            this.type = type;
        }

        public String getDesc() {
            return desc;
        }

        public void setDesc(String desc) {
            this.desc = desc;
        }
    }


    /**
     * 是否启用
     */
    public enum IsEnable {
        /**
         * 启用
         */
        ENABLE(1, "启用"),
        /**
         * 不启用
         */
        NOT_ENABLE(0, "不启用");

        private Integer type;    //类型
        private String desc;    //描述

        IsEnable(Integer type, String desc) {
            this.type = type;
            this.desc = desc;
        }

        public Integer getType() {
            return type;
        }

        public void setType(Integer type) {
            this.type = type;
        }

        public String getDesc() {
            return desc;
        }

        public void setDesc(String desc) {
            this.desc = desc;
        }
    }

    /**
     * 评测规则类型
     */
    public enum RulesType {
        /**
         * 随机
         */
        RANDOM(1, "随机"),
        /**
         * 自定义
         */
        CUSTOMIZE(0, "自定义");

        private Integer type;    //类型
        private String desc;    //描述

        RulesType(Integer type, String desc) {
            this.type = type;
            this.desc = desc;
        }

        public Integer getType() {
            return type;
        }

        public void setType(Integer type) {
            this.type = type;
        }

        public String getDesc() {
            return desc;
        }

        public void setDesc(String desc) {
            this.desc = desc;
        }
    }


    /**
     * 当前题目状态 1、已作/0、未作
     */
    public enum QuestionStatus {

        MADE(1, "已作"),
        NOT_MADE(0, "未作");

        private Integer type;
        private String desc;

        QuestionStatus(Integer type, String desc) {
            this.type = type;
            this.desc = desc;
        }

        public Integer getType() {
            return type;
        }

        public void setType(Integer type) {
            this.type = type;
        }

        public String getDesc() {
            return desc;
        }

        public void setDesc(String desc) {
            this.desc = desc;
        }
    }

    /**
     * 试题选项A
     */
    public static final String A = "A";
    /**
     * 试题选项B
     */
    public static final String B = "B";
    /**
     * 试题选项C
     */
    public static final String C = "C";
    /**
     * 试题选项D
     */
    public static final String D = "D";
    /**
     * 试题选项E
     */
    public static final String E = "E";
    /**
     * 试题选项F
     */
    public static final String F = "F";

    /**
     * 通过excel批量导入试题数据时起始行
     */
    public static final int CELL_INDEX = 0;
    /**
     * 需要提取的样式所在的行号
     */
    public static final int STYLE_INDEX = 2;


    /**
     * 通过excel批量导入试题数据时起始行(专门用于练习，考核报告的）
     */
    public static final int ROW_INDEX_REPORT = 3;


    /**
     * 通过excel批量导入试题数据时起始行
     */
    public static final int ROW_INDEX = 2;


    /**
     * 测评规则表id
     */
    public static final int EVALUATION_RULES_ID = 1;
}