package com.huoran.competition.utils;

import com.huoran.common.exception.CustomException;
import com.huoran.common.exception.ExceptionEnum;
import com.huoran.competition.entity.req.ExcelImpGradeReq;
import com.huoran.competition.entity.req.ExcelImpRankReq;
import org.apache.commons.lang.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class ExcelImporEventResultstHelper {

    /**
     * 读取Excel列表成绩
     *
     * @param file
     * @return
     */
    public static List<ExcelImpGradeReq> readGrade(MultipartFile file) {
        List<ExcelImpGradeReq> list = new ArrayList<ExcelImpGradeReq>();

        Workbook workbook = getWorkbook(file);
        ExcelImpGradeReq excelImpStaffReq = null;
        // 循环工作表Sheet
        for (int numSheet = 0; numSheet < workbook.getNumberOfSheets(); numSheet++) {
            Sheet hssfSheet = workbook.getSheetAt(numSheet);
            if (hssfSheet == null) {
                continue;
            }

            int count = 1;
            // 循环行Row//开始行2
            for (int rowNum = 2; rowNum <= hssfSheet.getLastRowNum(); rowNum++) {
                Row row = hssfSheet.getRow(rowNum);

                if (isRowEmpty(row)) {
                    count++;
                    if (count == hssfSheet.getLastRowNum()) {
                        throw new CustomException(ExceptionEnum.EXCEL_FILE_NULL);
                    }
                    continue;
                }

                Cell schoolName;//学校名称
                Cell teamName;//团队名称
                Cell userName;//学生姓名
                Cell workNumber;//学号
                Cell score;//分数
                Cell timeSum;//耗时
//                Cell submitTime;//提交时间
                if (row != null) {

                    excelImpStaffReq = new ExcelImpGradeReq();


                    if (row.getCell(5) != null) {
                        row.getCell(5).setCellType(CellType.STRING);
                        schoolName = row.getCell(5);
                    } else {
                        schoolName = row.createCell(5);
                    }


                    if (row.getCell(4) != null) {
                        row.getCell(4).setCellType(CellType.STRING);
                        timeSum = row.getCell(4);
                    } else {
                        timeSum = row.createCell(4);
                    }

                    if (row.getCell(3) != null) {
                        row.getCell(3).setCellType(CellType.STRING);
                        score = row.getCell(3);
                    } else {
                        score = row.createCell(3);
                    }

                    if (row.getCell(2) != null) {
                        row.getCell(2).setCellType(CellType.STRING);
                        workNumber = row.getCell(2);
                    } else {
                        workNumber = row.createCell(2);
                    }

                    if (row.getCell(1) != null) {
                        row.getCell(1).setCellType(CellType.STRING);
                        userName = row.getCell(1);
                    } else {
                        userName = row.createCell(1);
                    }


                    if (row.getCell(0) != null) {
                        row.getCell(0).setCellType(CellType.STRING);
                        teamName = row.getCell(0);
                    } else {
                        teamName = row.createCell(0);
                    }

                    excelImpStaffReq.setTeamName(teamName.getStringCellValue());
                    excelImpStaffReq.setUserName(userName.getStringCellValue());
                    excelImpStaffReq.setWorkNumber(workNumber.getStringCellValue());
                    excelImpStaffReq.setScore(score.getStringCellValue());
                    excelImpStaffReq.setTimeSum(timeSum.getStringCellValue());
                    excelImpStaffReq.setSchoolName(schoolName.getStringCellValue());

                    /*if (!submitTime.getStringCellValue().equals("")) {
                        Date date = HSSFDateUtil.getJavaDate(Double.parseDouble(submitTime.getStringCellValue()));
                        excelImpStaffReq.setSubmitTime(date);
                    }*/
                    list.add(excelImpStaffReq);
                }
            }

        }

        return list;
    }


    /**
     * 读取Excel列表排名
     *
     * @param file
     * @return
     */
    public static List<ExcelImpRankReq> readRanking(MultipartFile file) {
        List<ExcelImpRankReq> list = new ArrayList<ExcelImpRankReq>();

        Workbook workbook = getWorkbook(file);
        ExcelImpRankReq excelImpStaffReq = null;
        // 循环工作表Sheet
        for (int numSheet = 0; numSheet < workbook.getNumberOfSheets(); numSheet++) {
            Sheet hssfSheet = workbook.getSheetAt(numSheet);
            if (hssfSheet == null) {
                continue;
            }

            int count = 1;
            // 循环行Row//开始行2
            for (int rowNum = 2; rowNum <= hssfSheet.getLastRowNum(); rowNum++) {
                Row row = hssfSheet.getRow(rowNum);

                if (isRowEmpty(row)) {
                    count++;
                    if (count == hssfSheet.getLastRowNum()) {
                        throw new CustomException(ExceptionEnum.EXCEL_FILE_NULL);
                    }
                    continue;
                }

                Cell teamName;//团队名称
                Cell userName;//学生姓名
//                Cell workNumber;//学号
                Cell schoolName;//学校
                Cell score;//分数
                Cell timeSum;//耗时
                if (row != null) {

                    excelImpStaffReq = new ExcelImpRankReq();




                    if (row.getCell(4) != null) {
                        row.getCell(4).setCellType(CellType.STRING);
                        score = row.getCell(4);
                    } else {
                        score = row.createCell(4);
                    }

                    if (row.getCell(3) != null) {
                        row.getCell(3).setCellType(CellType.STRING);
                        timeSum = row.getCell(3);
                    } else {
                        timeSum = row.createCell(3);
                    }

                    if (row.getCell(2) != null) {
                        row.getCell(2).setCellType(CellType.STRING);
                        schoolName = row.getCell(2);
                    } else {
                        schoolName = row.createCell(2);
                    }

                    if (row.getCell(1) != null) {
                        row.getCell(1).setCellType(CellType.STRING);
                        userName = row.getCell(1);
                    } else {
                        userName = row.createCell(1);
                    }


                    if (row.getCell(0) != null) {
                        row.getCell(0).setCellType(CellType.STRING);
                        teamName = row.getCell(0);
                    } else {
                        teamName = row.createCell(0);
                    }

                    excelImpStaffReq.setTeamName(teamName.getStringCellValue());
                    excelImpStaffReq.setUserName(userName.getStringCellValue());
//                    excelImpStaffReq.setWorkNumber(workNumber.getStringCellValue());
                    excelImpStaffReq.setSchoolName(schoolName.getStringCellValue());
                    excelImpStaffReq.setScore(score.getStringCellValue());
                    excelImpStaffReq.setTimeSum(timeSum.getStringCellValue());


                    list.add(excelImpStaffReq);
                }
            }

        }

        return list;
    }





    /**
     * 读取Excel个人列表排名
     *
     * @param file
     * @return
     */
    public static List<ExcelImpRankReq> readIndividualRanking(MultipartFile file) {
        List<ExcelImpRankReq> list = new ArrayList<ExcelImpRankReq>();

        Workbook workbook = getWorkbook(file);
        ExcelImpRankReq excelImpStaffReq = null;
        // 循环工作表Sheet
        for (int numSheet = 0; numSheet < workbook.getNumberOfSheets(); numSheet++) {
            Sheet hssfSheet = workbook.getSheetAt(numSheet);
            if (hssfSheet == null) {
                continue;
            }

            int count = 1;
            // 循环行Row//开始行2
            for (int rowNum = 2; rowNum <= hssfSheet.getLastRowNum(); rowNum++) {
                Row row = hssfSheet.getRow(rowNum);

                if (isRowEmpty(row)) {
                    count++;
                    if (count == hssfSheet.getLastRowNum()) {
                        throw new CustomException(ExceptionEnum.EXCEL_FILE_NULL);
                    }
                    continue;
                }

                Cell userName;//学生姓名
                Cell workNumber;//学号
                Cell schoolName;//学校
                Cell score;//分数
                Cell timeSum;//耗时
                if (row != null) {

                    excelImpStaffReq = new ExcelImpRankReq();




                    if (row.getCell(4) != null) {
                        row.getCell(4).setCellType(CellType.STRING);
                        score = row.getCell(4);
                    } else {
                        score = row.createCell(4);
                    }

                    if (row.getCell(3) != null) {
                        row.getCell(3).setCellType(CellType.STRING);
                        timeSum = row.getCell(3);
                    } else {
                        timeSum = row.createCell(2);
                    }

                    if (row.getCell(2) != null) {
                        row.getCell(2).setCellType(CellType.STRING);
                        schoolName = row.getCell(2);
                    } else {
                        schoolName = row.createCell(2);
                    }

                    if (row.getCell(1) != null) {
                        row.getCell(1).setCellType(CellType.STRING);
                        workNumber = row.getCell(1);
                    } else {
                        workNumber = row.createCell(1);
                    }

                    if (row.getCell(0) != null) {
                        row.getCell(0).setCellType(CellType.STRING);
                        userName = row.getCell(0);
                    } else {
                        userName = row.createCell(0);
                    }



                    excelImpStaffReq.setUserName(userName.getStringCellValue());
                    excelImpStaffReq.setSchoolName(schoolName.getStringCellValue());
                    excelImpStaffReq.setScore(score.getStringCellValue());
                    excelImpStaffReq.setTimeSum(timeSum.getStringCellValue());
                    excelImpStaffReq.setWorkNumber(workNumber.getStringCellValue());


                    list.add(excelImpStaffReq);
                }
            }

        }

        return list;
    }


    //判断row是否为空
    public static boolean isRowEmpty(Row row) {
        if (null == row) {
            return true;
        }
        int firstCellNum = row.getFirstCellNum();   //第一个列位置
        int lastCellNum = row.getLastCellNum();     //最后一列位置
        int nullCellNum = 0;    //空列数量
        for (int c = firstCellNum; c < lastCellNum; c++) {
            Cell cell = row.getCell(c);
            if (null == cell || CellType.BLANK == cell.getCellType()) {
                nullCellNum++;
                continue;
            }
            cell.setCellType(CellType.STRING);
            String cellValue = cell.getStringCellValue().trim();
            if (StringUtils.isEmpty(cellValue)) {
                nullCellNum++;
            }
        }
        //所有列都为空
        if (nullCellNum == (lastCellNum - firstCellNum)) {
            return true;
        }
        return false;
    }

    private static Workbook getWorkbook(MultipartFile file) {
        String fileName = file.getOriginalFilename();
        Workbook workbook = null;

        if (fileName.endsWith("xlsx")) {
            try {
                workbook = new XSSFWorkbook(file.getInputStream());
            } catch (IOException e) {
                e.printStackTrace();
            }
        } else if (fileName.endsWith("xls")) {
            try {
                workbook = new HSSFWorkbook(file.getInputStream());
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return workbook;
    }


    /**
     * @Description :校验是否存在文字 为true表示存在 false表示不存在
     * @Param str
     * @Author Rong---2021/11/4
     */
    public static boolean isContainChinese(String str) {
        Pattern p = Pattern.compile("[\u4e00-\u9fa5]");
        Matcher m = p.matcher(str);
        if (m.find()) {
            return true;
        }
        return false;
    }

}
