package com.huoran.competition.utils;

import com.huoran.api.NakadaiClient;
import com.huoran.api.UserClient;
import com.huoran.competition.entity.CompetitionTeam;
import com.huoran.competition.entity.resp.ListOfStageResultsResp;
import com.huoran.competition.entity.vo.RegistrationVO;
import com.huoran.competition.mapper.CompetitionRegistrationMapper;
import com.huoran.competition.mapper.CompetitionTeamMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

@Component
public class RankUtil {
    @Autowired
    UserClient userClient;
    @Autowired
    NakadaiClient nakadaiClient;

    @Autowired
    private CompetitionTeamMapper teamMapper;

    @Autowired
    private CompetitionRegistrationMapper registrationMapper;


    public static RankUtil rankUtil;


    @PostConstruct
    public void init() {
        rankUtil = this;
        rankUtil.teamMapper = this.teamMapper;
        rankUtil.nakadaiClient = this.nakadaiClient;
        rankUtil.userClient = this.userClient;
        rankUtil.registrationMapper = this.registrationMapper;
    }


    /**
     * 团队赛是拿参比活动的全部队伍与参加活动并且获得成绩的团队比对
     *
     * @param competitionTeams
     * @param list
     * @return
     */
    public List<ListOfStageResultsResp> matchingSet(List<CompetitionTeam> competitionTeams, List<ListOfStageResultsResp> list) {
        //参赛全部队伍 A集合
        List<Integer> allTeam = competitionTeams.stream().map(CompetitionTeam::getTeamId).collect(Collectors.toList());

        //参与本次阶段,有成绩的队伍 B集合
        List<Integer> teamWithResults = list.stream().map(ListOfStageResultsResp::getTeamId).collect(Collectors.toList());

        //falseData：假数据,用于区分活动没有参加团队的但是要给一条数据
        List<ListOfStageResultsResp> falseData = new ArrayList<>();

        // 对比
        for (Integer id : allTeam) {
            if (!teamWithResults.contains(id)) {
                // A集合中，B集合没有的对象id
                //System.err.println(id);
                //判断B中没有的团队id,则加入到要返回的list中

                ListOfStageResultsResp addTeamId = new ListOfStageResultsResp();
                addTeamId.setTeamId(id);
                addTeamId.setScore("0");
                addTeamId.setTimeSum("0");
                falseData.add(addTeamId);
            }
        }

        list.addAll(falseData);

        for (ListOfStageResultsResp l : list) {
            CompetitionTeam team = rankUtil.teamMapper.selectById(l.getTeamId());
            if (team != null) {
                if (l.getTeamName() == null) {

                    l.setTeamName(team.getTeamName());
                }

                if (l.getLeaderName() == null) {
                    l.setLeaderName(rankUtil.userClient.getUserName(team.getAccountId()));
                    l.setAccountId(team.getAccountId() + "");
                }

                if (l.getSchoolId() == null) {
                    Integer schoolId = rankUtil.userClient.getSchoolIdByAccountId(team.getAccountId() + "");
                    l.setSchoolName(rankUtil.nakadaiClient.getTheSchoolNameBySchoolId(schoolId).getSchoolName());
//                    String schoolName = registrationMapper.selectSchoolName(l.getAccountId());
//                    l.setSchoolName(schoolName);
                    l.setSchoolId(schoolId);
                }
            }
        }

        return list;
    }


    public List<ListOfStageResultsResp> matchingSetPersonage(List<RegistrationVO> selectApplicantByCondition, List<ListOfStageResultsResp> list) {
        //报名本次活动的全部人员 A集合
        List<Integer> allApplicants = selectApplicantByCondition.stream().map(RegistrationVO::getAccountId).collect(Collectors.toList());

        //参与本次阶段,有成绩的队伍 B集合
        List<String> teamWithResults = list.stream().map(ListOfStageResultsResp::getAccountId).collect(Collectors.toList());

        List<Integer> intClusterIds = teamWithResults.stream().mapToInt(Integer::parseInt).boxed().collect(Collectors.toList());


        //falseData：假数据,用于区分活动没有参加团队的但是要给一条数据
        List<ListOfStageResultsResp> falseData = new ArrayList<>();

        // 对比
        for (Integer id : allApplicants) {
            if (!intClusterIds.contains(id)) {
                // A集合中，B集合没有的对象id
                //System.err.println(id);
                //判断B中没有的账号id,则加入到要返回的list中
                ListOfStageResultsResp addTeamId = new ListOfStageResultsResp();
                addTeamId.setScore("0");
                addTeamId.setTimeSum("0");
                addTeamId.setAccountId(id + "");
                falseData.add(addTeamId);
            }
        }

        list.addAll(falseData);

        for (ListOfStageResultsResp l : list) {
            if (l.getSchoolId() == null) {
                Integer schoolId = rankUtil.userClient.getSchoolIdByAccountId(l.getAccountId() + "");
                l.setSchoolName(rankUtil.nakadaiClient.getTheSchoolNameBySchoolId(schoolId).getSchoolName());
/*                String schoolName = registrationMapper.selectSchoolName(l.getAccountId());
                l.setSchoolName(schoolName);*/
                l.setSchoolId(schoolId);
            }

            if (l.getLeaderName() == null) {
                l.setUserName(rankUtil.userClient.getUserName(Integer.valueOf(l.getAccountId())));
            }

        }

        Collections.sort(list, new Comparator<ListOfStageResultsResp>() {
            public int compare(ListOfStageResultsResp s1, ListOfStageResultsResp s2) {
                int i = Double.valueOf(s2.getScore()).compareTo(Double.valueOf(s1.getScore()));
                if (i == 0) {
                    i = s1.getTimeSum().compareTo(s2.getTimeSum());
                }
                return i;
            }
        });

        return list;
    }


    public List<ListOfStageResultsResp> singleStageMatchingSetPersonage(List<RegistrationVO> selectApplicantByCondition, List<ListOfStageResultsResp> list) {
        //报名本次活动的全部人员 A集合
        List<Integer> allApplicants = selectApplicantByCondition.stream().map(RegistrationVO::getAccountId).collect(Collectors.toList());

        //参与本次阶段,有成绩的队伍 B集合
        List<String> teamWithResults = list.stream().map(ListOfStageResultsResp::getAccountId).collect(Collectors.toList());

        List<Integer> intClusterIds = teamWithResults.stream().mapToInt(Integer::parseInt).boxed().collect(Collectors.toList());


        //falseData：假数据,用于区分活动没有参加团队的但是要给一条数据
        List<ListOfStageResultsResp> falseData = new ArrayList<>();

        // 对比
        for (Integer id : allApplicants) {
            if (!intClusterIds.contains(id)) {
                // A集合中，B集合没有的对象id
                ////System.err.println(id);
                //判断B中没有的账号id,则加入到要返回的list中
                ListOfStageResultsResp addTeamId = new ListOfStageResultsResp();
                addTeamId.setScore("—");
                addTeamId.setTimeSum("—");
                addTeamId.setAccountId(id + "");
                falseData.add(addTeamId);
            }
        }

        list.addAll(falseData);

        for (ListOfStageResultsResp l : list) {
            if (l.getSchoolId() == null) {
                Integer schoolId = rankUtil.userClient.getSchoolIdByAccountId(l.getAccountId() + "");
                l.setSchoolName(rankUtil.nakadaiClient.getTheSchoolNameBySchoolId(schoolId).getSchoolName());
/*                String schoolName = registrationMapper.selectSchoolName(l.getAccountId());
                l.setSchoolName(schoolName);*/
            }

            if (l.getLeaderName() == null) {
                l.setUserName(rankUtil.userClient.getUserName(Integer.valueOf(l.getAccountId())));
                l.setAccountId(l.getAccountId());
            }

        }

        return list;
    }


    public List<ListOfStageResultsResp> singleStageMatchingSet(List<CompetitionTeam> competitionTeams, List<ListOfStageResultsResp> list) {
        //参赛全部队伍 A集合
        List<Integer> allTeam = competitionTeams.stream().map(CompetitionTeam::getTeamId).collect(Collectors.toList());

        //参与本次阶段,有成绩的队伍 B集合
        List<Integer> teamWithResults = list.stream().map(ListOfStageResultsResp::getTeamId).collect(Collectors.toList());

        //falseData：假数据,用于区分活动没有参加团队的但是要给一条数据
        List<ListOfStageResultsResp> falseData = new ArrayList<>();

        // 对比
        for (Integer id : allTeam) {
            if (!teamWithResults.contains(id)) {
                // A集合中，B集合没有的对象id
                //System.err.println(id);
                //判断B中没有的团队id,则加入到要返回的list中

                ListOfStageResultsResp addTeamId = new ListOfStageResultsResp();
                addTeamId.setTeamId(id);
                addTeamId.setScore("—");
                addTeamId.setTimeSum("—");
                falseData.add(addTeamId);
            }
        }

        list.addAll(falseData);

        for (ListOfStageResultsResp l : list) {
            CompetitionTeam team = rankUtil.teamMapper.selectById(l.getTeamId());
            if (team != null) {
                if (l.getTeamName() == null) {

                    l.setTeamName(team.getTeamName());
                }

                if (l.getLeaderName() == null) {
                    l.setLeaderName(rankUtil.userClient.getUserName(team.getAccountId()));
                    l.setAccountId(team.getAccountId() + "");
                }

                if (l.getSchoolId() == null) {
                    Integer schoolId = rankUtil.userClient.getSchoolIdByAccountId(team.getAccountId() + "");
                    l.setSchoolId(schoolId);
                    l.setSchoolName(rankUtil.nakadaiClient.getTheSchoolNameBySchoolId(schoolId).getSchoolName());
                }
            }
        }

        return list;
    }


}
