package com.huoran.competition.utils;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huoran.competition.entity.CompetitionStageFile;
import com.huoran.competition.service.CompetitionStageFileService;
import com.huoran.competition.service.impl.CompetitionStageFileServiceImpl;
import io.lettuce.core.dynamic.annotation.Command;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

@Slf4j
@Component
public class ZipUtil {

    private static Logger logger = LoggerFactory.getLogger(ZipUtil.class);
    /**
     * 需要压缩的文件夹
     */
    public final static String downloadDir = "download";
    /**
     * 打包后的文件夹
     */
    public final static String downloadZip = "downloadZip";


    @Resource
    private CompetitionStageFileService service;

    public static ZipUtil zipUtil;

    @PostConstruct
    public void init() {
        zipUtil = this;
    }

    /**
     * 通过图片url下载图片到指定文件夹
     *
     * @param downloadUrl 图片url
     */
    public void downloadFile(String downloadUrl) {
        InputStream inputStream = null;
        OutputStream outputStream = null;
        try {
            //获取连接
            URL url = new URL(downloadUrl);
            HttpURLConnection connection = (HttpURLConnection) url.openConnection();
            connection.setConnectTimeout(3 * 1000);
            //设置请求头
            connection.setRequestProperty("User-Agent", "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/57.0.2987.110 Safari/537.36");
            //获取输入流
            inputStream = connection.getInputStream();

            File fileDir = new File(downloadDir);
            if (!fileDir.exists()) {//如果文件夹不存在
                fileDir.mkdir();//创建文件夹
            }

//            String delFile = downloadDir + downloadUrl;


            //截取文件名称，可以把 / 换成需要的规则
            String filePath = downloadDir + "/" + downloadUrl.substring(downloadUrl.lastIndexOf("/"));


            String processFileName = downloadUrl.substring(downloadUrl.lastIndexOf("/"));
            System.err.printf("处理后的路径:" + processFileName + "\n");

            QueryWrapper<CompetitionStageFile> queryWrapper = new QueryWrapper<>();
            queryWrapper.like("file_path", processFileName);
            CompetitionStageFile stageFile = zipUtil.service.getOne(queryWrapper);

            String userName = zipUtil.service.getStudentName(stageFile.getAccountId());
            String newPath = downloadDir + "/" + userName + "-" + stageFile.getFileName();

            File file = new File(newPath);

            file.createNewFile();//创建文件，存在覆盖

            outputStream = new FileOutputStream(file);
            int len = 0;
            byte[] buf = new byte[1024];
            while ((len = inputStream.read(buf, 0, 1024)) != -1) {
                outputStream.write(buf, 0, len);
            }
            outputStream.flush();


           /* String downloadDirFilePath =  filePath.replace("//","\\");
            //删除download文件下的文件
            deleteFile(downloadDirFilePath);*/

        } catch (Exception e) {
            logger.error("文件下载出错：" + e);
        } finally {
            try {
                if (inputStream != null) {
                    inputStream.close();
                }
                if (outputStream != null) {
                    outputStream.close();
                }
            } catch (IOException e) {
                logger.error("关闭流出错：" + e);
            }
        }
    }


    // 删除文件夹
    public static void deleteDirectory(File file) {
        if (file.isFile()) {// 表示该文件不是文件夹
            file.delete();
        } else {
            // 首先得到当前的路径
            String[] childFilePaths = file.list();
            for (String childFilePath : childFilePaths) {
                File childFile = new File(file.getAbsolutePath() + "/" + childFilePath);
                deleteDirectory(childFile);
            }
            file.delete();
        }
    }


    public static void main(String[] args) {
        String s = "/1724641147097735168.png";
        System.out.printf(s.substring(0, s.length() - 1));
    }

    /**
     * 下载压缩包
     */
    public void downloadZip(HttpServletRequest request, HttpServletResponse res, String zipName) throws IOException {
        OutputStream out = null;
        File zip = null;
        try {
            //多个文件进行压缩，批量打包下载文件
            //创建压缩文件需要的空的zip包
            //String zipName = "测试压缩包.zip";
            String zipFilePath = downloadZip + File.separator + zipName;


            File fileDir = new File(downloadZip);
            if (!fileDir.exists()) {//如果文件夹不存在
                fileDir.mkdir();//创建文件夹
            }

            //压缩文件
            zip = new File(zipFilePath);
            zip.createNewFile();//创建文件，存在覆盖

            //创建zip文件输出流
            ZipOutputStream zos = new ZipOutputStream(new FileOutputStream(zip));
            this.zipFile(downloadDir, zos);

            zos.close();

            //将打包后的文件写到客户端，输出的方法同上，使用缓冲流输出
            BufferedInputStream bis = new BufferedInputStream(new FileInputStream(zipFilePath));
            byte[] buff = new byte[bis.available()];
            bis.read(buff);
            bis.close();

            //IO流实现下载的功能
            res.setCharacterEncoding("UTF-8"); //设置编码字符
            res.setContentType("application/octet-stream;charset=UTF-8"); //设置下载内容类型application/octet-stream（二进制流，未知文件类型）；
            //防止文件名乱码
            String userAgent = request.getHeader("USER-AGENT");
            if (userAgent.contains("Firefox") || userAgent.contains("firefox")) {//火狐浏览器
                zipName = new String(zipName.getBytes(), "ISO8859-1");
            } else {
                zipName = URLEncoder.encode(zipName, "UTF-8");//其他浏览器
            }

//            res.setHeader("Content-disposition", "attachment;filename=" + zipName);//设置下载的压缩文件名称

            // 告诉浏览器用什么软件可以打开此文件
            res.setHeader("content-Type", "application/vnd.ms-excel");
            // 下载文件的默认名称
            res.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(zipName));
            //编码
            res.setCharacterEncoding("UTF-8");
            out = res.getOutputStream();   //创建页面返回方式为输出流，会自动弹出下载框
            out.write(buff);//输出数据文件

            //删除zip压缩文件
            System.out.printf("zipFilePath:" + zipFilePath);
            deleteFile(zipFilePath);

        } catch (Exception e) {
            logger.error("压缩包下载出错：" + e);
        } finally {
            if (out != null) {
                out.flush();//释放缓存
                out.close();//关闭输出流
            }
        }
    }

    /**
     * 压缩文件
     *
     * @param filePath 需要压缩的文件夹
     * @param zos      zip文件输出流
     */
    private void zipFile(String filePath, ZipOutputStream zos) throws IOException {
        File inputFile = new File(filePath);  //根据文件路径创建文件
        if (inputFile.exists()) { //判断文件是否存在
            if (inputFile.isFile()) {  //判断是否属于文件，还是文件夹
                //创建输入流读取文件
                BufferedInputStream bis = new BufferedInputStream(new FileInputStream(inputFile));

                //将文件写入zip内，即将文件进行打包
                zos.putNextEntry(new ZipEntry(inputFile.getName()));

                //写入文件的方法，同上
                int size = 0;
                byte[] buffer = new byte[1024];  //设置读取数据缓存大小
                while ((size = bis.read(buffer)) > 0) {
                    zos.write(buffer, 0, size);
                }
                //关闭输入输出流
                zos.closeEntry();
                bis.close();

            } else {  //如果是文件夹，则使用穷举的方法获取文件，写入zip
                try {
                    File[] files = inputFile.listFiles();
                    for (File fileTem : files) {
                        zipFile(fileTem.toString(), zos);
                    }
                } catch (Exception e) {
                    logger.error("压缩文件出错：" + e);
                }
            }
        }
    }


    /**
     * 判断指定的文件删除是否成功
     *
     * @param fileName 文件路径
     * @return true or false 成功返回true，失败返回false
     */
    public static boolean deleteFile(String fileName) {
        File file = new File(fileName);//根据指定的文件名创建File对象
        if (file.exists() && file.isFile()) { //要删除的文件存在且是文件
            if (file.delete()) {
                System.out.println("文件" + fileName + "删除成功！");
                return true;
            } else {
                System.out.println("文件" + fileName + "删除失败！");
                return false;
            }
        } else {
            System.out.println("文件" + fileName + "不存在，删除失败！");
            return false;
        }


    }


}


