package com.huoran.gateway.filter;

import com.alibaba.fastjson.JSONObject;
import com.huoran.common.utils.JwtUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.GlobalFilter;
import org.springframework.context.annotation.PropertySource;
import org.springframework.core.Ordered;
import org.springframework.core.io.buffer.DataBuffer;
import org.springframework.http.HttpStatus;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.http.server.reactive.ServerHttpResponse;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import java.nio.charset.StandardCharsets;
import java.util.List;

/**
 * @Author cheney
 * @Version 1.0
 * 全局Filter，统一处理用户登录与外部不允许访问的服务
 */
@Component
@PropertySource(value = "classpath:jwt.properties")
public class AuthGlobalFilter implements GlobalFilter, Ordered {

    /**
     * 不进行token校验的请求地址
     */
    @Value("#{'${jwt.ignoreUrlList}'.split(',')}")
    public List<String> ignoreUrl;

    /*@Autowired
    private UserClient userClient;*/

    /**
     * 拦截所有的请求头
     */
    @Override
    public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
        //获取请求路径
        String requestUrl = exchange.getRequest().getPath().toString();
        //路径比较
        boolean status = CollectionUtils.contains(ignoreUrl.iterator(), requestUrl);
        if (requestUrl.contains("/nakadai/nakadai/oss/getPlayAuth")||requestUrl.contains("/nakadai/nakadai/curriculum/chapter/queryChaptersAndSubsections")){
            status = true;
        }
        if (!status){
            //获取token值
            String token = exchange.getRequest().getHeaders().getFirst("token");
            ServerHttpResponse response = exchange.getResponse();
            //没有数据
            if (StringUtils.isBlank(token)) {
                JSONObject message = new JSONObject();
                message.put("status", "401");
                message.put("message", "请先登录");
                byte[] bits = message.toString().getBytes(StandardCharsets.UTF_8);
                DataBuffer buffer = response.bufferFactory().wrap(bits);
                response.setStatusCode(HttpStatus.UNAUTHORIZED);
                response.getHeaders().add("Content-Type", "text/json;charset=UTF-8");
                return response.writeWith(Mono.just(buffer));
            //有数据
            }else {
                //校验token
                boolean isVerify = JwtUtils.checkToken(token);
                if (isVerify){
                    /*boolean disabled = userClient.accountIsDisabled(token);
                    if (disabled){
                        JSONObject message = new JSONObject();
                        message.put("message", "用户已禁用，请联系管理员进行处理！");
                        message.put("status", "402");
                        byte[] bits = message.toString().getBytes(StandardCharsets.UTF_8);
                        DataBuffer buffer = response.bufferFactory().wrap(bits);
                        response.setStatusCode(HttpStatus.UNAUTHORIZED);
                        response.getHeaders().add("Content-Type", "text/json;charset=UTF-8");
                        return response.writeWith(Mono.just(buffer));
                    }*/
                }else {
                    JSONObject message = new JSONObject();
                    message.put("message", "登录已失效,请重新登录");
                    message.put("status", "401");
                    byte[] bits = message.toString().getBytes(StandardCharsets.UTF_8);
                    DataBuffer buffer = response.bufferFactory().wrap(bits);
                    response.setStatusCode(HttpStatus.UNAUTHORIZED);
                    response.getHeaders().add("Content-Type", "text/json;charset=UTF-8");
                    return response.writeWith(Mono.just(buffer));
                }
                //将现在的request，添加当前身份
                ServerHttpRequest mutableReq = exchange.getRequest().mutate().header("Authorization", token).build();
                ServerWebExchange mutableExchange = exchange.mutate().request(mutableReq).build();
                return chain.filter(mutableExchange);
            }
        }
        return chain.filter(exchange);
    }

    @Override
    public int getOrder() {
        return 0;
    }

}
