package com.huoran.nakadai.config;

import com.huoran.common.constant.RabbitmqConstant;
import org.springframework.amqp.core.*;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.amqp.rabbit.listener.SimpleMessageListenerContainer;
import org.springframework.amqp.support.ConsumerTagStrategy;
import org.springframework.amqp.support.converter.Jackson2JsonMessageConverter;
import org.springframework.amqp.support.converter.MessageConverter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;

import java.util.UUID;

/**
 * 初始化消息队列，创建交换机、队列以及绑定关系
 */
//@Configuration
public class RabbitConfig {

    @Autowired
    RabbitTemplate rabbitTemplate;

/**
     * mq消息转换json格式
     * @return json*/
    @Bean
    public MessageConverter messageConverter(){
        return new Jackson2JsonMessageConverter();
    }

    @Bean
    public Exchange commentExchange(){
        return new DirectExchange(RabbitmqConstant.COMMENT_EXCHANGE,true,false);
    }

    @Bean
    public Queue commentQueue(){
        return new Queue(RabbitmqConstant.COMMENT_QUEUE,true,false,false);
    }

    @Bean
    public Binding commentBind(){
        return new Binding(RabbitmqConstant.COMMENT_QUEUE,
                Binding.DestinationType.QUEUE,
                RabbitmqConstant.COMMENT_EXCHANGE,
                RabbitmqConstant.COMMENT_ROUTING_KEY,null);
    }

    @Bean
    public SimpleMessageListenerContainer simpleMessageListenerContainer(ConnectionFactory connectionFactory){
        SimpleMessageListenerContainer listenerContainer = new SimpleMessageListenerContainer(connectionFactory);
        listenerContainer.setConcurrentConsumers(1); // 设置当前消费者为1
        listenerContainer.setMaxConcurrentConsumers(5);// 设置最大消费者5
        listenerContainer.setDefaultRequeueRejected(false);// 设置是否重回队列
        listenerContainer.setAcknowledgeMode(AcknowledgeMode.MANUAL);//设置接收方式，AUTO-自动接收，MANUAL-手动接收，NULL-不接收
        listenerContainer.setConsumerTagStrategy(new ConsumerTagStrategy() {  //设置消费者标签策略
            @Override
            public String createConsumerTag(String s) {
                return s+"_"+ UUID.randomUUID().toString();
            }
        });
        listenerContainer.setMessageListener(new MessageListener() {   // 监听消息
            @Override
            public void onMessage(Message message) {
/*                String msg = new String(message.getBody());
                System.out.println("消息："+msg);*/
            }
        });
        return listenerContainer;
    }
}
