package com.huoran.nakadai.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.gson.Gson;
import com.huoran.common.entity.EnterpriseCertification;
import com.huoran.common.entity.UserAccount;
import com.huoran.common.entity.UserAuthenticationInformation;
import com.huoran.common.response.R;
import com.huoran.common.utils.TokenUtils;
import com.huoran.nakadai.config.AliRealNameAuthenticationConfig;
import com.huoran.nakadai.entity.PlatformTeam;
import com.huoran.nakadai.entity.PlatformTeamAccount;
import com.huoran.nakadai.entity.PlatformTeamClassification;
import com.huoran.nakadai.entity.UserInfo;
import com.huoran.nakadai.entity.res.BusinessLicenseOCRResp;
import com.huoran.nakadai.entity.res.CreditCodeResp;
import com.huoran.nakadai.entity.res.IdCardOCRResp;
import com.huoran.nakadai.entity.res.RealNameAuthenticationResp;
import com.huoran.nakadai.mapper.PlatformTeamClassificationMapper;
import com.huoran.nakadai.mapper.PlatformTeamMapper;
import com.huoran.nakadai.service.*;
import com.huoran.nakadai.utils.ali.FaceRecognitionAidUtil;
import com.huoran.nakadai.utils.ali.HttpUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpResponse;
import org.apache.http.util.EntityUtils;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Api(value = "小程序-实名认证", tags = "小程序-实名认证")
@RestController
@RequestMapping("/users/autonym")
public class AliRealNameAuthenticationController {

    //实名认证实现类
    @Autowired
    public UserAuthenticationInformationService service;


    //企业认证信息存储
    @Autowired
    public EnterpriseCertificationService enterpriseCertificationService;

    @Autowired
    private PlatformTeamClassificationService platformTeamClassificationService;


    @Autowired
    public PlatformTeamAccountService platformTeamAccountService;

    @Autowired
    private PlatformTeamService platformTeamService;

    @ApiOperation(value = "删除平台全部认证信息")
    @PostMapping("/deletePlatformAuthenticationInformation")
    public R deletePlatformAuthenticationInformation(@ApiParam(name = "openIds", value = "微信openId", required = true) @RequestBody List<String> openIds) {

        openIds.forEach(openId -> {
            QueryWrapper<UserAuthenticationInformation> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("open_id", openId);
            service.remove(queryWrapper);


            QueryWrapper<EnterpriseCertification> queryWrapper1 = new QueryWrapper<>();
            queryWrapper1.eq("open_id", openId);
            enterpriseCertificationService.remove(queryWrapper1);
        });

        return R.ok();
    }


    @ApiOperation(value = "身份证图片验证")
    @PostMapping("/faceAuthentication")
    public R faceAuthentication(@RequestParam(name = "imgFile", required = true) @ApiParam(value = "图片url") String imgFile, @RequestParam(name = "side", required = true) @ApiParam(value = "身份证正反面类型:face/back") String side) throws IOException {
        //身份证信息返回
        String idCardInformationReturn = FaceRecognitionAidUtil.IdCardOCRRecognition(imgFile, side);
        if (ObjectUtils.isEmpty(idCardInformationReturn)) {
            return R.error("身份证图片验证失败，请重新再试！");
        }
        IdCardOCRResp resp = new Gson().fromJson(idCardInformationReturn, IdCardOCRResp.class);
        if (resp.getSuccess()) {
            return R.ok().put("data", resp);
        } else {
            return R.error("身份证图片验证失败，请重新再试！");
        }

    }


    @PostMapping("/realNameAuthentication")
    @ApiOperation(value = "输入身份证号码、姓名进行实名认证且添加认证信息", response = UserAuthenticationInformation.class)
    public R realNameAuthentication(@RequestBody @ApiParam(name = "用户实名认证信息记录对象", value = "传入json格式", required = true) UserAuthenticationInformation userAuthenticationInformation, HttpServletRequest request) throws IOException {
        String accountId = TokenUtils.getIdByJwtToken(request);
        userAuthenticationInformation.setAccountId(Integer.valueOf(accountId));

        String json = FaceRecognitionAidUtil.postData(AliRealNameAuthenticationConfig.ALI_API_APPCODE, AliRealNameAuthenticationConfig.GET_APP_CODE_LINK, userAuthenticationInformation.getRealName(), userAuthenticationInformation.getIdCardNo());
        RealNameAuthenticationResp info = new Gson().fromJson(json, RealNameAuthenticationResp.class);

        if (!info.getRespCode().equals("0000")) {
            return R.error(info.getRespMessage());
        }

        QueryWrapper<UserAuthenticationInformation> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("open_id", userAuthenticationInformation.getOpenId());
        UserAuthenticationInformation information = service.getOne(queryWrapper);

        if (!ObjectUtils.isEmpty(information)) {
            //为二次认证，删除上一次的认证信息
            service.removeById(information.getId());
        }


        //设置用户当前认证时间
        userAuthenticationInformation.setRealNameAuthenticationTime(new Date());
        userAuthenticationInformation.setProvince(info.getProvince());
        userAuthenticationInformation.setCity(info.getCity());
        userAuthenticationInformation.setCounty(info.getCounty());

        UserAccount userAccount = enterpriseCertificationService.queryAccountInformation(userAuthenticationInformation.getOpenId(), 7);

        userAccount.setAccount(userAuthenticationInformation.getRealName());
        enterpriseCertificationService.updateAccountInfo(userAccount);


        UserInfo userInfo = new UserInfo();
        userInfo.setUserName(userAuthenticationInformation.getRealName());
        userInfo.setUserId(userAccount.getUserId());
        enterpriseCertificationService.updateUserInfo(userInfo);

        boolean addState = service.save(userAuthenticationInformation);
        return addState ? R.ok() : R.error("认证失败");
    }


    @PostMapping("/authenticationOrNot")
    @ApiOperation(value = "个人是否认证", response = UserAuthenticationInformation.class)
    public R authenticationOrNot(HttpServletRequest request) {

        String accountId = TokenUtils.getIdByJwtToken(request);
        QueryWrapper<UserAuthenticationInformation> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("account_id", accountId);
        UserAuthenticationInformation userAuthenticationInformation = service.getOne(queryWrapper);
        if (ObjectUtils.isEmpty(userAuthenticationInformation)) {
            return R.ok().put("data", false).put("msg", "未认证");
        }
        return R.ok().put("data", true).put("authenticationInformation", userAuthenticationInformation);
    }


    @ApiOperation(value = "营业执照图片验证")
    @PostMapping("/businessLicensePictureVerification")
    public R businessLicensePictureVerification(@RequestParam(name = "imgFile", required = true) @ApiParam(value = "图片url") String imgFile) throws IOException {
        String method = "POST";
        Map<String, String> headers = new HashMap<String, String>();
        //最后在header中的格式(中间是英文空格)为Authorization:APPCODE 83359fd73fe94948385f570e3c139105
//        headers.put("Authorization", "APPCODE " + AliRealNameAuthenticationConfig.ALI_API_APPCODE);
        headers.put("Authorization", "APPCODE " + "bd4ad25d0e41466eabb31d9baa16a79f");
        //根据API的要求，定义相对应的Content-Type
        headers.put("Content-Type", "application/json; charset=UTF-8");
        Map<String, String> querys = new HashMap<String, String>();
        String bodys = "{\"image\":\"" + imgFile + "\"}";
        try {
            HttpResponse response = HttpUtils.doPost(AliRealNameAuthenticationConfig.ALI_API_BUSINESS_LICENSE_HOST, AliRealNameAuthenticationConfig.ALI_API_BUSINESS_LICENSE_PATH, method, headers, querys, bodys);
            BusinessLicenseOCRResp info = new Gson().fromJson(EntityUtils.toString(response.getEntity()), BusinessLicenseOCRResp.class);
            if (info.getSuccess()) {
                return R.ok().put("data", info);
            } else {
                //("营业执照图片验证失败，请重新再试！");
                return R.error("营业执照图片验证失败，请重新再试！");
            }
        } catch (Exception e) {
            e.printStackTrace();
            return R.error("营业执照图片验证失败，请重新再试！");
        }
    }


    @ApiOperation(value = "统一社会信用代码认证(更改审核状态未提交为待审核)", response = CreditCodeResp.class)
    @PostMapping("/creditCodeAuthentication")
    public R creditCodeAuthentication(@RequestBody EnterpriseCertification enterpriseCertification, HttpServletRequest request) throws IOException {


        String accountId = TokenUtils.getIdByJwtToken(request);
        enterpriseCertification.setAccountId(Integer.valueOf(accountId));
        enterpriseCertification.setSubmitTime(new Date());
        String host = "https://sys.81api.com";
        String path = "/verifyCompany/";
        String method = "GET";
        Map<String, String> headers = new HashMap<String, String>();
        //最后在header中的格式(中间是英文空格)为Authorization:APPCODE 83359fd73fe94948385f570e3c139105
        headers.put("Authorization", "APPCODE " + "bd4ad25d0e41466eabb31d9baa16a79f");//嘉龙
        Map<String, String> querys = new HashMap<String, String>();
        querys.put("ComapnyName", enterpriseCertification.getCompanyName());
        querys.put("CreditCode", enterpriseCertification.getCreditCode());
        querys.put("LegalPersonName", enterpriseCertification.getLegalPerson());

        String errorMsg = "";
        //查询认证id
        QueryWrapper<EnterpriseCertification> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("open_id", enterpriseCertification.getOpenId());
        EnterpriseCertification certification = enterpriseCertificationService.getOne(queryWrapper);
        if (certification == null) {
            //通过后台删除组织信息的
            /*return R.error("您已没有组织信息,请联系管理人员!");*/
            //重新新增认证信息，且更改认证状态
            enterpriseCertification.setAccountId(Integer.valueOf(accountId));
            if (!ObjectUtils.isEmpty(enterpriseCertificationService.checkWhetherItExists(enterpriseCertification.getCompanyName(), null, null))) {
                return R.error("当前企业名称已存在!");
            }

            enterpriseCertification.setAuditStatus(1);
            enterpriseCertification.setAuthenticationStatus(1);
            enterpriseCertificationService.save(enterpriseCertification);

            //新增团队信息
            PlatformTeamAccount platformTeamAccount1 = platformTeamAccountService.getOne(new QueryWrapper<PlatformTeamAccount>().
                    eq("account_id", accountId).
                    eq("platform_id", enterpriseCertification.getPlatformSource()));
            String teamName = "";
            Integer classificationId = null;
            if (platformTeamAccount1 == null) {
                // 添加供应商/幼儿园信息
                PlatformTeamAccount platformTeamAccount = new PlatformTeamAccount();
                platformTeamAccount.setAccountId(Integer.valueOf(accountId));
                platformTeamAccount.setPlatformId(Integer.valueOf(enterpriseCertification.getPlatformSource()));
                platformTeamAccountService.save(platformTeamAccount);

                // 添加所属供应商/幼儿园分类,分类为空直接新增团队
                PlatformTeamClassification platformTeamClassification = new PlatformTeamClassification();
                platformTeamClassification.setIsTeam(1);
                platformTeamClassification.setParentId(0);
                platformTeamClassification.setLevel(1);

                teamName = enterpriseCertification.getCompanyName();
                classificationId = platformTeamClassification.getId();
                platformTeamClassification.setClassificationName(teamName + "的团队");
                boolean insert = platformTeamClassificationService.save(platformTeamClassification);
                if (insert) {
                    PlatformTeam platformTeam = new PlatformTeam();
                    platformTeam.setManageId(platformTeamAccount.getManageId());
                    platformTeam.setClassificationId(platformTeamClassification.getId());
                    platformTeamService.save(platformTeam);
                }


            }

        }
            enterpriseCertification.setId(certification.getId());
            try {
                HttpResponse response = HttpUtils.doGet(host, path, method, headers, querys);
                CreditCodeResp info = new Gson().fromJson(EntityUtils.toString(response.getEntity()), CreditCodeResp.class);
                if (info.getStatus()) {
                    if (!ObjectUtils.isEmpty(enterpriseCertificationService.checkWhetherItExists(enterpriseCertification.getCompanyName(), certification.getId(), enterpriseCertification.getOpenId()))) {
                        errorMsg = "企业名称已存在";
                        return R.error(errorMsg);
                    }

                    //删除上团队信息
//                enterpriseCertificationService.deleteTeamInformation(enterpriseCertification.getAccountId());
                    enterpriseCertificationService.updateById(enterpriseCertification);

                    //更改二次验证的组织信息
                    return changeTeamInformation(enterpriseCertification);
                } else {
                    errorMsg = info.getReason();
                    return R.error().put("message", info.getReason());
                }
            } catch (Exception e) {
                e.printStackTrace();
                return R.error(errorMsg);
            }


        }

        @NotNull
        private R changeTeamInformation (@RequestBody EnterpriseCertification enterpriseCertification){
            enterpriseCertification.setSubmitTime(new Date());
            UserAccount userAccount1 = enterpriseCertificationService.queryAccountInformation(enterpriseCertification.getOpenId(), Integer.valueOf(enterpriseCertification.getPlatformSource()));

            userAccount1.setAccount(enterpriseCertification.getCompanyName());
            enterpriseCertificationService.updateAccountInfo(userAccount1);

            //删除团队信息
//        enterpriseCertificationService.deleteTeamInformation(enterpriseCertification.getAccountId());
            return R.ok("提交成功,请等待审核！");
        }

        @ApiOperation(value = "统一社会信用代码认证(auditStatus给1，新增认证状态为待审核，用于个人认证幼儿园或供应商情况下)", response = CreditCodeResp.class)
        @PostMapping("/organizationCertification")
        public R organizationCertification (@RequestBody EnterpriseCertification
        enterpriseCertification, HttpServletRequest request) throws IOException {

/*
        String accountId = TokenUtils.getIdByJwtToken(request);
        enterpriseCertification.setAccountId(Integer.valueOf(accountId));*/
            enterpriseCertification.setSubmitTime(new Date());
            String host = "https://sys.81api.com";
            String path = "/verifyCompany/";
            String method = "GET";
            Map<String, String> headers = new HashMap<String, String>();
            //最后在header中的格式(中间是英文空格)为Authorization:APPCODE 83359fd73fe94948385f570e3c139105
            headers.put("Authorization", "APPCODE " + "bd4ad25d0e41466eabb31d9baa16a79f");
            Map<String, String> querys = new HashMap<String, String>();
            querys.put("ComapnyName", enterpriseCertification.getCompanyName());
            querys.put("CreditCode", enterpriseCertification.getCreditCode());
            querys.put("LegalPersonName", enterpriseCertification.getLegalPerson());

            String errorMsg = "";

            try {
                HttpResponse response = HttpUtils.doGet(host, path, method, headers, querys);
                CreditCodeResp info = new Gson().fromJson(EntityUtils.toString(response.getEntity()), CreditCodeResp.class);
                if (info.getStatus()) {
                    if (!ObjectUtils.isEmpty(enterpriseCertificationService.checkWhetherItExists(enterpriseCertification.getCompanyName(), null, enterpriseCertification.getOpenId()))) {
                        errorMsg = "企业名称已存在";
                        return R.error(errorMsg);
                    }
                    enterpriseCertificationService.save(enterpriseCertification);

                    //更改二次验证的组织信息
                    return changeTeamInformation(enterpriseCertification);
                } else {
                    errorMsg = info.getReason();

                    //失败后删除本次生成的企业团队信息
                /*QueryWrapper<HrUserAccount> accountQueryWrapper = new QueryWrapper<>();
                accountQueryWrapper.eq("app_open_id", enterpriseCertification.getOpenId());
                accountQueryWrapper.eq("platform_id", enterpriseCertification.getPlatformSource());

                HrUserAccount userAccount = userAccountService.getOne(accountQueryWrapper);*/

                    UserAccount userAccount = enterpriseCertificationService.queryAccountInformation(enterpriseCertification.getOpenId(), Integer.valueOf(enterpriseCertification.getPlatformSource()));

                    enterpriseCertificationService.deleteUserInformation(userAccount.getUserId());
                    enterpriseCertificationService.deleteAccountInformation(userAccount.getId());
                /*userInfoService.removeById(userAccount.getUserId());
                userAccountService.removeById(userAccount);*/
                    return R.error().put("message", info.getReason());
                }
            } catch (Exception e) {
                e.printStackTrace();
                return R.error().put("message", errorMsg);
            }


        }

        public static void main (String[]args){
            String host = "https://sys.81api.com";
            String path = "/verifyCompany/";
            String method = "GET";
            Map<String, String> headers = new HashMap<String, String>();
            //最后在header中的格式(中间是英文空格)为Authorization:APPCODE 83359fd73fe94948385f570e3c139105
            headers.put("Authorization", "APPCODE " + AliRealNameAuthenticationConfig.ALI_API_APPCODE);
            Map<String, String> querys = new HashMap<String, String>();
            querys.put("ComapnyName", "荆州国瑞网络科技有限公司");
            querys.put("CreditCode", "91421000MA494BGL8W");
            querys.put("LegalPersonName", "余承文");

            String errorMsg = "";

            try {
                HttpResponse response = HttpUtils.doGet(host, path, method, headers, querys);
                CreditCodeResp info = new Gson().fromJson(EntityUtils.toString(response.getEntity()), CreditCodeResp.class);
                if (info.getStatus()) {
                /*if (!ObjectUtils.isEmpty(enterpriseCertificationService.checkWhetherItExists(enterpriseCertification.getCompanyName(), null, enterpriseCertification.getOpenId()))) {
                    errorMsg = "企业名称已存在";
                    (errorMsg);
                }
                enterpriseCertificationService.save(enterpriseCertification);*/

                } else {
                    errorMsg = info.getReason();

                    System.err.println("错误信息:" + errorMsg);
                }
            } catch (Exception e) {
                e.printStackTrace();
                errorMsg = "系统错误";
            }


        }
    }