/*
 * Copyright (c) 2018-2999 广州市蓝海创新科技有限公司 All rights reserved.
 *
 * https://www.mall4j.com/
 *
 * 未经允许，不可做商业用途！
 *
 * 版权所有，侵权必究！
 */

package com.huoran.nakadai.controller;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.huoran.common.exception.YamiShopBindException;
import com.huoran.common.response.ServerResponseEntity;
import com.huoran.common.utils.TokenUtils;
import com.huoran.nakadai.entity.app.dto.CategoryDto;
import com.huoran.nakadai.entity.model.Category;
import com.huoran.nakadai.entity.model.Product;
import com.huoran.nakadai.service.CategoryService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @author lanhai
 */
@RestController
@RequestMapping("/category")
@Api(tags = "商品分类管理")
public class CategoryController {

    @Autowired
    private CategoryService categoryService;


    /**
     * 分类信息列表接口
     */
    @GetMapping("/categoryInfo")
    @ApiOperation(value = "根据父id查询分类列表")
    public ServerResponseEntity<List<CategoryDto>> categoryInfo(
            @RequestParam(value = "parentId", defaultValue = "0") Long parentId) {
        List<Category> categories = categoryService.listByParentId(parentId);
        List<CategoryDto> categoryDtos = BeanUtil.copyToList(categories, CategoryDto.class);
        return ServerResponseEntity.success(categoryDtos);
    }


    /**
     * 获取菜单页面的表
     * @return
     */
    @GetMapping("/list")
    @ApiOperation(value = "获取店铺所有分类列表")
    public ServerResponseEntity<List<Category>> table(HttpServletRequest request){
        List<Category> categoryMenuList = categoryService.tableCategory(TokenUtils.getShopIdByJwtToken(request));
        return ServerResponseEntity.success(categoryMenuList);
    }

    /**
     * 获取分类信息
     */
    @GetMapping("/info")
    @ApiOperation(value = "获取分类信息")
    public ServerResponseEntity<Category> info(@RequestParam("categoryId") Long categoryId){
        Category category = categoryService.getById(categoryId);
        return ServerResponseEntity.success(category);
    }



    /**
     * 保存分类
     */
    @PostMapping("/save")
    @ApiOperation(value = "保存分类信息")
    public ServerResponseEntity<Void> save(@RequestBody Category category,HttpServletRequest request){
        category.setShopId(TokenUtils.getShopIdByJwtToken(request));
        category.setRecTime(new Date());
        Category categoryName = categoryService.getOne(new LambdaQueryWrapper<Category>().eq(Category::getCategoryName,category.getCategoryName())
                .eq(Category::getShopId,category.getShopId()));
        if(categoryName != null){
            throw new YamiShopBindException("类目名称已存在！");
        }
        categoryService.saveCategory(category);
        return ServerResponseEntity.success();
    }

    /**
     * 更新分类
     */
    @PostMapping("/update")
    @ApiOperation(value = "更新分类信息")
    public ServerResponseEntity<String> update(@RequestBody Category category,HttpServletRequest request){
        category.setShopId(TokenUtils.getShopIdByJwtToken(request));
        if (Objects.equals(category.getParentId(),category.getCategoryId())) {
            return ServerResponseEntity.showFailMsg("分类的上级不能是自己本身");
        }
        Category categoryName = categoryService.getOne(new LambdaQueryWrapper<Category>().eq(Category::getCategoryName,category.getCategoryName())
                .eq(Category::getShopId,category.getShopId()).ne(Category::getCategoryId,category.getCategoryId()));
        if(categoryName != null){
            throw new YamiShopBindException("类目名称已存在！");
        }
        Category categoryDb = categoryService.getById(category.getCategoryId());
        // 如果从下线改成正常，则需要判断上级的状态
        if (Objects.equals(categoryDb.getStatus(),0) && Objects.equals(category.getStatus(),1) && !Objects.equals(category.getParentId(),0L)){
            Category parentCategory = categoryService.getOne(new LambdaQueryWrapper<Category>().eq(Category::getCategoryId, category.getParentId()));
            if(Objects.isNull(parentCategory) || Objects.equals(parentCategory.getStatus(),0)){
                // 修改失败，上级分类不存在或者不为正常状态
                throw new YamiShopBindException("修改失败，上级分类不存在或者不为正常状态");
            }
        }
        categoryService.updateCategory(category);
        return ServerResponseEntity.success();
    }

    /**
     * 删除分类
     */
    @PostMapping("/delete")
    @ApiOperation(value = "删除分类信息")
    public ServerResponseEntity<String> delete(@RequestParam("categoryId") Long categoryId){
        if (categoryService.count(new LambdaQueryWrapper<Category>().eq(Category::getParentId,categoryId)) >0) {
            return ServerResponseEntity.showFailMsg("请删除子分类，再删除该分类");
        }
        categoryService.deleteCategory(categoryId);
        return ServerResponseEntity.success();
    }

    /**
     * 所有的
     */
    @GetMapping("/listCategory")
    public ServerResponseEntity<List<Category>> listCategory(HttpServletRequest request){
        return ServerResponseEntity.success(categoryService.list(new LambdaQueryWrapper<Category>()
                .le(Category::getGrade, 2)
                .eq(Category::getShopId, TokenUtils.getShopIdByJwtToken(request))
                .orderByAsc(Category::getSeq)));
    }

    /**
     * 所有的产品分类
     */
    @GetMapping("/listProdCategory")
    public ServerResponseEntity<List<Category>> listProdCategory(HttpServletRequest request){
        List<Category> categories = categoryService.treeSelect(TokenUtils.getShopIdByJwtToken(request),2);
        return ServerResponseEntity.success(categories);
    }


}
