package com.huoran.nakadai.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huoran.api.UserClient;
import com.huoran.common.response.R;
import com.huoran.common.utils.TokenUtils;
import com.huoran.nakadai.entity.CollectionProvider;
import com.huoran.nakadai.service.CollectionProviderService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * @描述：关注供应商控制类
 * @作者: Rong
 * @日期: 2024-01-12
 */
@RestController
@RequestMapping("/collectionProvider")
@Api(value = "关注供应商:CollectionProviderController", tags = "供应商关注")
public class CollectionProviderController {

    @Autowired
    public CollectionProviderService service;

    @Autowired
    private UserClient userClient;

    /*@PostMapping("/listByEntity")
    @ApiOperation(value = "分页查询", response = CollectionProvider.class)
    public R listByEntity(@RequestBody CollectionProvider collectionProvider) {

        return null;
    }*/


    @PostMapping("/findById")
    @ApiOperation(value = "查询详情", response = CollectionProvider.class)
    public R findById(@ApiParam(name = "id", value = "主键", required = true) @RequestParam Integer id) {
        CollectionProvider collectionProvider = service.getById(id);
        return R.ok().put("data", collectionProvider);
    }



    @PostMapping("/collect")
    @ApiOperation(value = "关注", response = CollectionProvider.class)
    public R collect(@RequestBody @ApiParam(name = "关注供应商对象", value = "传入json格式", required = true) CollectionProvider collectionProvider, HttpServletRequest request) {
        String accountId = TokenUtils.getIdByJwtToken(request);
        collectionProvider.setAccountId(Integer.valueOf(accountId));
        if (checkCollect(collectionProvider.getSupplierAccountId(), Integer.valueOf(accountId))){
            return R.ok("您已关注!请勿重复关注");
        }
        boolean addState = service.save(collectionProvider);
        return addState ? R.ok(collectionProvider.getId()) : R.error("新增失败");
    }

    //校验是否收藏过
    public  boolean checkCollect(Integer supplierAccountId, Integer accountId) {
        QueryWrapper<CollectionProvider> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("account_id", accountId);
        queryWrapper.eq("supplier_account_id", supplierAccountId);
        CollectionProvider collectionProvider = service.getOne(queryWrapper);
        return collectionProvider != null;
    }

   /* @PostMapping("/update")
    @ApiOperation(value = "修改", response = CollectionProvider.class)
    public R update(@RequestBody @ApiParam(name = "关注供应商对象", value = "传入json格式", required = true) CollectionProvider collectionProvider, HttpServletRequest request) {
        String accountId = TokenUtils.getIdByJwtToken(request);
        Integer schoolId = userClient.getSchoolIdByAccountId(accountId);
        collectionProvider.setAccountId(Integer.valueOf(accountId));
        boolean updateState = service.updateById(collectionProvider);
        return updateState ? R.ok() : R.error("编辑失败");
    }*/

    @PostMapping("/cancelCollection")
    @ApiOperation(value = "取消关注", response = CollectionProvider.class)
    public R cancelCollection(@ApiParam(name = "id", value = "主键", required = true) @RequestParam Integer id) {
        boolean delState = service.removeById(id);
        return delState ? R.ok() : R.error("取消失败");
    }


    @PostMapping("/batchDeletion")
    @ApiOperation(value = "批量删除", response = CollectionProvider.class)
    public R batchDeletion(@ApiParam(name = "ids", value = "主键", required = true) @RequestBody List<Integer> ids) {
        boolean delState = service.removeByIds(ids);
        return delState ? R.ok() : R.error("删除失败");
    }
}

