package com.huoran.nakadai.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huoran.common.entity.EnterpriseCertification;
import com.huoran.common.entity.OrganizationInfo;
import com.huoran.common.entity.UserAccount;
import com.huoran.common.entity.UserAuthenticationInformation;
import com.huoran.common.response.R;
import com.huoran.common.utils.TokenUtils;
import com.huoran.nakadai.entity.req.EnterpriseCertificationLIstReq;
import com.huoran.nakadai.entity.vo.WxMssVo;
import com.huoran.nakadai.mapper.PlatformTeamAccountMapper;
import com.huoran.nakadai.service.EnterpriseCertificationService;
import com.huoran.nakadai.service.UserAuthenticationInformationService;
import com.huoran.nakadai.utils.WeChatUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

/**
 * <p>
 * 企业认证审核 前端控制器
 * </p>
 *
 * @author chen
 * @since 2023-12-18
 */

@Api(value = "后台-企业实名认证相关", tags = "后台-企业实名认证相关")
@RestController
@RequestMapping("/enterprise/certification")
public class EnterpriseCertificationController {


    @Autowired
    public EnterpriseCertificationService service;

    @Autowired
    public UserAuthenticationInformationService userAuthenticationInformationService;

    @Autowired
    public PlatformTeamAccountMapper platformTeamAccountMapper;


    /**
     * 由前台申请认证后才有后台认证记录，认证状态有 待审核、已通过、未驳回三种状态，没有未提交这种状态
     *
     * @param req
     * @return
     */
    @PostMapping("/selectEnterpriseCertificationList")
    @ApiOperation(value = "企业认证列表", response = EnterpriseCertification.class)
    public R selectEnterpriseCertificationList(@RequestBody EnterpriseCertificationLIstReq req) {
        return service.selectEnterpriseCertificationList(req);
    }


    @PostMapping("/supplierList")
    @ApiOperation(value = "供应商列表", response = EnterpriseCertification.class)
    public R supplierList(@RequestBody EnterpriseCertificationLIstReq req, HttpServletRequest request) {
        String accountId = TokenUtils.getIdByJwtToken(request);
        req.setAccountId(Integer.valueOf(accountId));
        return service.supplierList(req);
    }


    @PostMapping("/enterpriseCertificationDetails")
    @ApiOperation(value = "企业认证详情", response = EnterpriseCertification.class)
    public R enterpriseCertificationDetails(@ApiParam(name = "id", value = "主键", required = true) @RequestParam Integer id) {
        return service.enterpriseCertificationDetails(id);
    }

    @PostMapping("/findById")
    @ApiOperation(value = "查询详情", response = EnterpriseCertification.class)
    public R findById(@ApiParam(name = "id", value = "主键", required = true) @RequestParam Integer id) {
        EnterpriseCertification enterpriseCertification = service.getById(id);
        return R.ok().put("data", enterpriseCertification);
    }


    @PostMapping("/enterpriseCertificationStatus")
    @ApiOperation(value = "查询企业当前认证状态", response = EnterpriseCertification.class)
    public R enterpriseCertificationStatus(@ApiParam(name = "openId", value = "微信openId", required = true) @RequestParam String openId) {
        /*String accountId = TokenUtils.getIdByJwtToken(request);*/
        QueryWrapper<EnterpriseCertification> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("open_id", openId);
        EnterpriseCertification enterpriseCertification = service.getOne(queryWrapper);
        return R.ok().put("data", enterpriseCertification);
    }


    @PostMapping("/save")
    @ApiOperation(value = "新增(注册后调用此接口，传值状态0默认未提交)", response = EnterpriseCertification.class)
    public R save(@RequestBody @ApiParam(name = "企业实名认证信息记录对象", value = "传入json格式", required = true) EnterpriseCertification enterpriseCertification, HttpServletRequest request) {
        String accountId = TokenUtils.getIdByJwtToken(request);
        enterpriseCertification.setAccountId(Integer.valueOf(accountId));
        if (!ObjectUtils.isEmpty(service.checkWhetherItExists(enterpriseCertification.getCompanyName(), null, null))) {
            /*throw new RuntimeException("企业名称已存在");*/
            return R.error("企业名称已存在");
        }
        boolean addState = service.save(enterpriseCertification);
        return addState ? R.ok() : R.error("新增失败");
    }


    @PostMapping("/update")
    @ApiOperation(value = "修改", response = EnterpriseCertification.class)
    public R update(@RequestBody @ApiParam(name = "企业实名认证信息记录对象", value = "传入json格式", required = true) EnterpriseCertification enterpriseCertification, HttpServletRequest request) {

        //增加逻辑判断：然后后台改为通过的，此时审核状态为通过。如果又改回未认证的，那审核状态为未通过。(现在是会变为未提交)
        //查询当前要更改前的企业认证信息(更改前)

        /* auditStatus: 审核状态(0默认未提交 1.待审核，2.已通过，3.已驳回)*/
        /* authenticationStatus: 认证状态(0默认未认证 1.认证中2.已认证)*/


        EnterpriseCertification beforeChange = service.getById(enterpriseCertification.getId());
        if (beforeChange.getAuthenticationStatus()==2&&beforeChange.getAuditStatus()==2){
            //判断原数据为已通过已认证的状态下，通过后台编辑后更改为‘未认证’，此时审核状态应为为‘未通过’
            if (enterpriseCertification.getAuthenticationStatus()==0&&enterpriseCertification.getAuditStatus()==0){
                enterpriseCertification.setAuditStatus(3);
                enterpriseCertification.setAuthenticationStatus(0);
            }
        }

        boolean updateState = service.updateById(enterpriseCertification);
        return updateState ? R.ok() : R.error("编辑失败");
    }


    @PostMapping("/informationAudit")
    @ApiOperation(value = "信息审核", response = EnterpriseCertification.class)
    public R informationAudit(@ApiParam(name = "auditStatus", value = "审核状态(2.通过，3.不通过)", required = true) @RequestParam(required = false) Integer auditStatus,
                              @ApiParam(name = "id", value = "主键", required = true) @RequestParam Integer id,
                              @ApiParam(name = "openId", value = "openid", required = true) @RequestParam(required = true) String openId,
                              @ApiParam(name = "remark", value = "备注", required = false) @RequestParam(required = false) String remark) {

        EnterpriseCertification details = service.getById(id);

        EnterpriseCertification enterpriseCertification = new EnterpriseCertification();
        enterpriseCertification.setAuditStatus(auditStatus);
        enterpriseCertification.setId(id);

        String authenticationStatus = "";

        WxMssVo wxMssVo = new WxMssVo();
        //设置模板id
        wxMssVo.setTemplate_id("ZB6wTenlv13mivxXwi-DWwjgVoNJ6eUr1vBvNdQGbJw");
        //跳转小程序类型：developer为开发版；trial为体验版；formal为正式版；默认为正式版
        wxMssVo.setMiniprogram_state("trial");
        wxMssVo.setLang("zh_CN");
        String accessToken = null;
        try {
            //设置openId
            wxMssVo.setTouser(openId);
            //设置accessToken
            accessToken = WeChatUtil.getAccessToken();
        } catch (IOException e) {
            e.printStackTrace();
        }
        //设置小程序 跳转页面
        wxMssVo.setPage("/pages/person/person");

        wxMssVo.setRequest_url("https://api.weixin.qq.com/cgi-bin/message/subscribe/send?access_token=" + accessToken);
        //将模板中的值一一赋值 发送到小程序的数据要转化为json格式 可以使用下面的方法直接拼接成JSONObject
        //使用map结合模板 后续再加遍历拼接成JSONObject
        HashMap<String, Object> map = new HashMap<>(4);
        map.put("thing1", details.getCompanyName());//公司名称

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String format = sdf.format(new Date());
        map.put("time5", format);//申请时间

        switch (auditStatus) {
            case 2:
                //通过:更改认证状态 认证状态(0默认未认证 1.认证中2.已认证)
                enterpriseCertification.setAuthenticationStatus(2);
                authenticationStatus = "通过";
                map.put("thing3", "无");//备注
                break;
            case 3:
                //不通过
                authenticationStatus = "不通过";
                map.put("thing3", remark);//备注
                enterpriseCertification.setAuthenticationStatus(0);
                break;
        }

        map.put("phrase2", authenticationStatus);//认证状态
        wxMssVo.setMap(map);
        WeChatUtil.sendTemplateMessage(wxMssVo);
        //更新状态
        boolean updateState = service.updateById(enterpriseCertification);
        return updateState ? R.ok() : R.error("编辑失败");
    }

    @PostMapping("/batchDeletion")
    @ApiOperation(value = "批量删除", response = EnterpriseCertification.class)
    public R batchDeletion(@ApiParam(name = "ids", value = "主键", required = true) @RequestBody List<Integer> ids) {

        for (Integer id : ids) {
            //

            EnterpriseCertification enterpriseCertification = service.getById(id);

            //删除企业认证后同步删除团队、用户、账户信息


            //删除账号表
            /*QueryWrapper<HrUserAccount> accountQueryWrapper1 = new QueryWrapper<>();
            accountQueryWrapper1.eq("app_open_id", enterpriseCertification.getOpenId());
            accountQueryWrapper1.eq("platform_id",enterpriseCertification.getPlatformSource());
            HrUserAccount userAccount = userAccountService.getOne(accountQueryWrapper1);*/

            //删除团队表
            UserAccount userAccount = service.queryAccountInformation(enterpriseCertification.getOpenId(), Integer.valueOf(enterpriseCertification.getPlatformSource()));
            service.deleteAccountInformation(userAccount.getId());

            //删除用户表
            service.deleteUserInformation(userAccount.getUserId());

            //删除团队信息表
//            service.deleteTeamInformation(userAccount.getId());
            //删除平台全部认证信息
            QueryWrapper<UserAuthenticationInformation> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("open_id", enterpriseCertification.getOpenId());
            userAuthenticationInformationService.remove(queryWrapper);

            QueryWrapper<EnterpriseCertification> queryWrapper1 = new QueryWrapper<>();
            queryWrapper1.eq("open_id", enterpriseCertification.getOpenId());
            service.remove(queryWrapper1);
            //删除组织信息，及组织下的用户信息
            List<OrganizationInfo> organizationInfoList = platformTeamAccountMapper.selectOrganizationInfo(enterpriseCertification.getOpenId());
            if (organizationInfoList != null && organizationInfoList.size() > 0) {
                for (OrganizationInfo organizationInfo : organizationInfoList) {
                    //如果是团队负责人，则删除自身团队及成员团队信息
                    if (organizationInfo.getIsTeam() == 1) {
                        ArrayList<Integer> getTeamIds = platformTeamAccountMapper.selectTeamIdByClassificationId(organizationInfo.getTeamId().toString());
                        for (Integer teamId : getTeamIds) {
                            platformTeamAccountMapper.deleteTeam(teamId);
                        }
                    } else {
                        //删除团队信息
                        platformTeamAccountMapper.deleteOrganization(organizationInfo.getAccountId());
                    }
                }
            }
        }

        boolean delState = service.removeByIds(ids);
        return delState ? R.ok() : R.error("删除失败");
    }

}

