package com.huoran.nakadai.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huoran.api.UserClient;
import com.huoran.common.response.R;
import com.huoran.common.utils.TokenUtils;
import com.huoran.nakadai.entity.ExplosiveRecommendation;
import com.huoran.nakadai.entity.req.ExplosiveRecommendationReq;
import com.huoran.nakadai.service.ExplosiveRecommendationService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

/**
 * @描述：控制类
 * @作者: Rong
 * @日期: 2023-12-28
 */
@RestController
@RequestMapping("/explosiveRecommendation")
@Api(value = "营销管理/banner管理:ExplosiveRecommendationController", tags = "营销管理/banner管理")
public class ExplosiveRecommendationController {

    @Autowired
    public ExplosiveRecommendationService service;

    @Autowired
    private UserClient userClient;

    @PostMapping("/explosiveList")
    @ApiOperation(value = "爆款推荐列表", response = ExplosiveRecommendation.class)
    public R explosiveList(@RequestBody ExplosiveRecommendationReq explosiveRecommendation) {
        IPage<ExplosiveRecommendation> iPage = new Page<>(explosiveRecommendation.getPageNum(), explosiveRecommendation.getPageSize());
        QueryWrapper<ExplosiveRecommendation> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("type",explosiveRecommendation.getType());
        if (!ObjectUtils.isEmpty(explosiveRecommendation.getEnabledState())) {
            queryWrapper.eq("enabled_state", explosiveRecommendation.getEnabledState());
        }
        queryWrapper.orderByDesc("create_time");
        IPage<ExplosiveRecommendation> pageList = service.page(iPage, queryWrapper);
        return R.ok().put("data", pageList);
    }


    //排序
    @PostMapping("/sort")
    @ApiOperation(value = "排序", response = ExplosiveRecommendation.class)
    public R sort(@RequestBody LinkedList<ExplosiveRecommendation> explosiveRecommendation) {

        //获取类型(0.爆款推荐 1.banner管理)
        ExplosiveRecommendation firstRecommendation = explosiveRecommendation.stream()
                .findFirst()
                .orElse(null);
        Integer type = firstRecommendation.getType();

        QueryWrapper<ExplosiveRecommendation> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("type", type);
        //删除原集合
        service.remove(queryWrapper);

        //todo 还需要对中间表进行删除信息

        Integer serialNumber = 0;
        for (ExplosiveRecommendation item : explosiveRecommendation) {
            item.setSequence(++serialNumber);
        }
        //添加排序后的集合
        service.saveBatch(explosiveRecommendation);

        return R.ok();
    }


    @PostMapping("/findById")
    @ApiOperation(value = "查询详情", response = ExplosiveRecommendation.class)
    public R findById(@ApiParam(name = "id", value = "主键", required = true) @RequestParam Integer id) {
        ExplosiveRecommendation explosiveRecommendation = service.getById(id);
        return R.ok().put("data", explosiveRecommendation);
    }


    @PostMapping("/save")
    @ApiOperation(value = "新增", response = ExplosiveRecommendation.class)
    public R save(@RequestBody @ApiParam(name = "对象", value = "传入json格式", required = true) ExplosiveRecommendation explosiveRecommendation, HttpServletRequest request) {
        String accountId = TokenUtils.getIdByJwtToken(request);
        Integer schoolId = userClient.getSchoolIdByAccountId(accountId);
        boolean addState = service.save(explosiveRecommendation);
        return addState ? R.ok() : R.error("新增失败");
    }


    @PostMapping("/update")
    @ApiOperation(value = "修改", response = ExplosiveRecommendation.class)
    public R update(@RequestBody @ApiParam(name = "对象", value = "传入json格式", required = true) ExplosiveRecommendation explosiveRecommendation, HttpServletRequest request) {
        String accountId = TokenUtils.getIdByJwtToken(request);
        boolean updateState = service.updateById(explosiveRecommendation);
        return updateState ? R.ok() : R.error("编辑失败");
    }


    @PostMapping("/batchDeletion")
    @ApiOperation(value = "批量删除", response = ExplosiveRecommendation.class)
    public R batchDeletion(@ApiParam(name = "ids", value = "主键", required = true) @RequestBody List<Integer> ids) {
        boolean delState = service.removeByIds(ids);
        return delState ? R.ok() : R.error("删除失败");
    }


}

