/*
 * Copyright (c) 2018-2999 广州市蓝海创新科技有限公司 All rights reserved.
 *
 * https://www.mall4j.com/
 *
 * 未经允许，不可做商业用途！
 *
 * 版权所有，侵权必究！
 */

package com.huoran.nakadai.controller;

import cn.hutool.core.bean.BeanUtil;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.huoran.common.entity.UserAccount;
import com.huoran.common.exception.YamiShopBindException;
import com.huoran.common.response.ServerResponseEntity;
import com.huoran.common.utils.SecurityUtils;
import com.huoran.common.entity.EnterpriseCertification;
import com.huoran.common.entity.UserAuthenticationInformation;
import com.huoran.nakadai.entity.app.dto.MyOrderDto;
import com.huoran.nakadai.entity.app.dto.OrderCountData;
import com.huoran.nakadai.entity.app.dto.OrderItemDto;
import com.huoran.nakadai.entity.app.dto.OrderShopDto;
import com.huoran.nakadai.entity.enums.OrderStatus;
import com.huoran.nakadai.entity.model.Order;
import com.huoran.nakadai.entity.model.OrderItem;
import com.huoran.nakadai.entity.model.ShopDetail;
import com.huoran.nakadai.entity.req.OrderPageReq;
import com.huoran.nakadai.mapper.PlatformTeamAccountMapper;
import com.huoran.nakadai.mapper.PlatformTeamClassificationMapper;
import com.huoran.nakadai.service.*;
import com.huoran.nakadai.utils.Arith;
import com.huoran.nakadai.utils.PageParam;
import io.swagger.annotations.Api;

import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Collections;

import java.util.List;
import java.util.Objects;

/**
 * @author lanhai
 */
@RestController
@RequestMapping("/myOrder")
@Api(tags = "小程序我的订单接口(用户采购信息)")
@AllArgsConstructor
public class MyOrderController {

    private final OrderService orderService;

    private final ProductService productService;

    private final SkuService skuService;

    private final MyOrderService myOrderService;

    @Autowired
    private PlatformTeamClassificationMapper platformTeamClassificationMapper;

    @Autowired
    public EnterpriseCertificationService enterpriseCertificationService;

    @Autowired
    public UserAuthenticationInformationService userAuthenticationInformationService;

    @Autowired
    private PlatformTeamAccountMapper platformTeamAccountMapper;

    private final ShopDetailService shopDetailService;

    private final OrderItemService orderItemService;


    /**
     * 订单详情信息接口
     */
    @GetMapping("/orderDetail")
    @ApiOperation(value = "根据订单号获取订单详情信息")
    public ServerResponseEntity<OrderShopDto> orderDetail(@RequestParam(value = "orderNumber") String orderNumber) {

//        String userId = SecurityUtils.getCurrentInfo().getAccountId();
        OrderShopDto orderShopDto = new OrderShopDto();

        Order order = orderService.getOrderByOrderNumber(orderNumber);

        if (order == null) {
            throw new RuntimeException("该订单不存在");
        }
//        if (!Objects.equals(order.getUserId(), userId)) {
//            throw new RuntimeException("你没有权限获取该订单信息");
//        }

        ShopDetail shopDetail = shopDetailService.getShopDetailByShopId(order.getShopId());
//        UserAddrOrder userAddrOrder = userAddrOrderService.getById(order.getAddrOrderId());
//        UserAddrDto userAddrDto = BeanUtil.copyProperties(userAddrOrder, UserAddrDto.class);
        List<OrderItem> orderItems = orderItemService.getOrderItemsByOrderNumber(orderNumber);
        List<OrderItemDto> orderItemList = BeanUtil.copyToList(orderItems, OrderItemDto.class);

        orderShopDto.setShopId(shopDetail.getShopId());
        orderShopDto.setShopName(shopDetail.getShopName());
        orderShopDto.setShopLogo(shopDetail.getShopLogo());
        orderShopDto.setActualTotal(order.getActualTotal());
//        orderShopDto.setUserAddrDto(userAddrDto);
        orderShopDto.setOrderItemDtos(orderItemList);
        orderShopDto.setTransfee(order.getFreightAmount());
        orderShopDto.setReduceAmount(order.getReduceAmount());
        orderShopDto.setCreateTime(order.getCreateTime());
        orderShopDto.setRemarks(order.getRemarks());
        orderShopDto.setStatus(order.getStatus());
        orderShopDto.setOrderType(order.getOrderType());

        double total = 0.0;
        Integer totalNum = 0;
        for (OrderItemDto orderItem : orderShopDto.getOrderItemDtos()) {
            if (orderItem.getProductTotalAmount()!=null){
                total = Arith.add(total, orderItem.getProductTotalAmount());
            }
            totalNum += orderItem.getProdCount();
        }
        orderShopDto.setTotal(total);
        orderShopDto.setTotalNum(totalNum);

        //获取采购方信息，根据accountId查询
        UserAccount userAccount = platformTeamAccountMapper.selectAccountPlatform(order.getUserId());
        List<UserAccount> userAccountList = platformTeamClassificationMapper.selectAccountList(userAccount.getAppOpenId());
        for (UserAccount account : userAccountList) {
            if (account.getPhone()!=null){
                orderShopDto.setPhone(account.getPhone());
            }
            if ("7".equals(account.getPlatformId())){
                QueryWrapper<UserAuthenticationInformation> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("account_id", account.getId());
                UserAuthenticationInformation userAuthenticationInformation = userAuthenticationInformationService.getOne(queryWrapper);
                orderShopDto.setAuthentication("未认证");
                orderShopDto.setPurchasingPerson(account.getAccount());
                if (userAuthenticationInformation!=null){
                    orderShopDto.setAuthentication("已认证");
                    orderShopDto.setPurchasingPerson(userAuthenticationInformation.getRealName());
                }
            }else {
                QueryWrapper<EnterpriseCertification> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("open_id", account.getAppOpenId());
                EnterpriseCertification enterpriseCertification = enterpriseCertificationService.getOne(queryWrapper);
                orderShopDto.setAuthentication("未认证");
                orderShopDto.setKindergarten(account.getAccount());
                if (enterpriseCertification!=null){
                    orderShopDto.setAuthentication("已认证");
                    orderShopDto.setKindergarten(enterpriseCertification.getCompanyName());
                }
            }
        }

        return ServerResponseEntity.success(orderShopDto);
    }


    /**
     * 订单列表接口
     */
    @GetMapping("/myOrder")
    @ApiOperation(value = "根据订单状态获取订单列表信息，状态为0时获取所有订单 1:待付款 2:待发货 3:待收货 4:待评价 5:成功 6:失败")
    public ServerResponseEntity<IPage<Order>> myOrder(OrderPageReq orderParam) {
        PageParam<Order> page = new PageParam<>();
        if (orderParam.getCurrent()!= null){
            page.setCurrent(orderParam.getCurrent());
        }
        if (orderParam.getSize()!= null){
            page.setSize(orderParam.getSize());
        }
        orderParam.setUserId(SecurityUtils.getCurrentInfo().getAccountId());
        IPage<Order> myOrderDtoPage = myOrderService.pageMyOrderByUserIdAndStatus(page,orderParam);
        return ServerResponseEntity.success(myOrderDtoPage);
    }

    /**
     * 取消订单
     */
    @PostMapping("/cancel")
    @ApiOperation(value = "根据订单号取消订单")
    public ServerResponseEntity<String> cancel(@RequestParam("orderNumber") String orderNumber) {
        String userId = SecurityUtils.getCurrentInfo().getAccountId();
        Order order = orderService.getOrderByOrderNumber(orderNumber);
        if (!Objects.equals(order.getUserId(), userId)) {
            throw new YamiShopBindException("你没有权限获取该订单信息");
        }
        if (!Objects.equals(order.getStatus(), OrderStatus.UNPAY.value())) {
            throw new YamiShopBindException("订单已支付，无法取消订单");
        }
        List<OrderItem> orderItems = orderItemService.getOrderItemsByOrderNumber(orderNumber);
        order.setOrderItems(orderItems);
        // 取消订单
        orderService.cancelOrders(Collections.singletonList(order));

        // 清除缓存
        for (OrderItem orderItem : orderItems) {
            productService.removeProductCacheByProdId(orderItem.getProdId());
            skuService.removeSkuCacheBySkuId(orderItem.getSkuId(), orderItem.getProdId());
        }
        return ServerResponseEntity.success();
    }


    /**
     * 确认收货
     */
    @PostMapping("/receipt")
    @ApiOperation(value = "根据订单号确认收货")
    public ServerResponseEntity<String> receipt(@RequestParam("orderNumber") String orderNumber) {
        String userId = SecurityUtils.getCurrentInfo().getAccountId();
        Order order = orderService.getOrderByOrderNumber(orderNumber);
        if (!Objects.equals(order.getUserId(), userId)) {
            throw new YamiShopBindException("你没有权限获取该订单信息");
        }
        if (!Objects.equals(order.getStatus(), OrderStatus.CONSIGNMENT.value())) {
            throw new YamiShopBindException("订单未发货，无法确认收货");
        }
        List<OrderItem> orderItems = orderItemService.getOrderItemsByOrderNumber(orderNumber);
        order.setOrderItems(orderItems);
        // 确认收货
        orderService.confirmOrder(Collections.singletonList(order));

        for (OrderItem orderItem : orderItems) {
            productService.removeProductCacheByProdId(orderItem.getProdId());
            skuService.removeSkuCacheBySkuId(orderItem.getSkuId(), orderItem.getProdId());
        }
        return ServerResponseEntity.success();
    }

    /**
     * 删除订单
     */
    @PostMapping("/deleteOrder")
    @ApiOperation(value = "根据订单号删除订单")
    public ServerResponseEntity<String> delete(@RequestParam("orderNumber") String orderNumber) {
        String userId = SecurityUtils.getCurrentInfo().getAccountId();

        Order order = orderService.getOrderByOrderNumber(orderNumber);
        if (order == null) {
            throw new YamiShopBindException("该订单不存在");
        }
        if (!Objects.equals(order.getUserId(), userId)) {
            throw new YamiShopBindException("你没有权限获取该订单信息");
        }
        if (!Objects.equals(order.getStatus(), OrderStatus.SUCCESS.value()) && !Objects.equals(order.getStatus(), OrderStatus.CLOSE.value())) {
            throw new YamiShopBindException("订单未完成或未关闭，无法删除订单");
        }

        // 删除订单
        orderService.deleteOrders(Collections.singletonList(order));

        return ServerResponseEntity.success("删除成功");
    }

    /**
     * 获取我的订单订单数量
     */
    @GetMapping("/orderCount")
    @ApiOperation(value = "获取我的订单订单数量")
    public ServerResponseEntity<OrderCountData> getOrderCount() {
        String userId = SecurityUtils.getCurrentInfo().getAccountId();
        OrderCountData orderCountMap = orderService.getOrderCount(userId);
        return ServerResponseEntity.success(orderCountMap);
    }


}
