/*
 * Copyright (c) 2018-2999 广州市蓝海创新科技有限公司 All rights reserved.
 *
 * https://www.mall4j.com/
 *
 * 未经允许，不可做商业用途！
 *
 * 版权所有，侵权必究！
 */

package com.huoran.nakadai.controller;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.huoran.common.exception.YamiShopBindException;
import com.huoran.common.response.ResponseEnum;
import com.huoran.common.response.ServerResponseEntity;
import com.huoran.common.utils.TokenUtils;
import com.huoran.nakadai.entity.model.PickAddr;

import com.huoran.nakadai.service.PickAddrService;
import com.huoran.nakadai.utils.PageParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.Arrays;
import java.util.Objects;


@RestController
@RequestMapping("/shop/pickAddr")
public class PickAddrController {

    @Autowired
    private PickAddrService pickAddrService;

	/**
	 * 分页获取
	 */
    @GetMapping("/page")
	public ServerResponseEntity<IPage<PickAddr>> page(PickAddr pickAddr, PageParam<PickAddr> page){
		IPage<PickAddr> pickAddrs = pickAddrService.page(page,new LambdaQueryWrapper<PickAddr>()
													.like(StrUtil.isNotBlank(pickAddr.getAddrName()),PickAddr::getAddrName,pickAddr.getAddrName())
													.orderByDesc(PickAddr::getAddrId));
		return ServerResponseEntity.success(pickAddrs);
	}

    /**
	 * 获取信息
	 */
	@GetMapping("/info/{id}")
	public ServerResponseEntity<PickAddr> info(@PathVariable("id") Long id){
		PickAddr pickAddr = pickAddrService.getById(id);
		return ServerResponseEntity.success(pickAddr);
	}

	/**
	 * 保存
	 */
	@PostMapping
	public ServerResponseEntity<Void> save(@Valid @RequestBody PickAddr pickAddr,HttpServletRequest request){
		pickAddr.setShopId(TokenUtils.getShopIdByJwtToken(request));
		pickAddrService.save(pickAddr);
		return ServerResponseEntity.success();
	}

	/**
	 * 修改
	 */
	@PutMapping
	public ServerResponseEntity<Void> update(@Valid @RequestBody PickAddr pickAddr, HttpServletRequest request){
		PickAddr dbPickAddr = pickAddrService.getById(pickAddr.getAddrId());

		if (!Objects.equals(dbPickAddr.getShopId(), TokenUtils.getShopIdByJwtToken(request))) {
			throw new YamiShopBindException(ResponseEnum.UNAUTHORIZED);
		}
		pickAddrService.updateById(pickAddr);
		return ServerResponseEntity.success();
	}

	/**
	 * 删除
	 */
	@DeleteMapping
	public ServerResponseEntity<Void> delete(@RequestBody Long[] ids){
		pickAddrService.removeByIds(Arrays.asList(ids));
		return ServerResponseEntity.success();
	}
}
