package com.huoran.nakadai.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huoran.common.entity.UserAccount;
import com.huoran.common.exception.CustomException;
import com.huoran.common.exception.ExceptionEnum;
import com.huoran.common.response.R;
import com.huoran.common.utils.TokenUtils;
import com.huoran.nakadai.entity.*;
import com.huoran.nakadai.entity.req.*;
import com.huoran.nakadai.entity.res.PartnerAccountResp;
import com.huoran.nakadai.entity.res.PlatformTeamAccountResp;
import com.huoran.nakadai.entity.vo.TeamClassificationVO;
import com.huoran.nakadai.service.*;
import com.huoran.nakadai.utils.GenerateCodeUtil;
import com.huoran.nakadai.utils.ali.HttpUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpResponse;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.TimeUnit;

/**
 * <p>
 * 幼教产业平台团队账号表
 * </p>
 *
 * @author chen
 * @since 2023-12-12
 */
@Api(tags = "幼教产业平台幼儿园与供应商团队账号管理")
@RestController
@RequestMapping("/kindergarten/platformTeamAccount")
public class PlatformTeamAccountController {

    @Autowired
    public PlatformTeamAccountService platformTeamAccountService;

    @Autowired
    public PlatformTeamService platformTeamService;

    @Autowired
    public CollectionProviderService collectionProviderService;

    @Autowired
    public PlatformTeamClassificationService platformTeamClassificationService;

    @Autowired
    public SupplierClassificationConfigService supplierClassificationConfigService;

    @Autowired
    private RedisTemplate<String, String> redisTemplate;


    @PostMapping("/generateInvitationCode")
    @ApiOperation(value = "生成不同平台专属邀请码")
    public R generateInvitationCode(@ApiParam(name = "platformId", value = "平台id", required = true) @RequestParam("platformId") String platformId, HttpServletRequest request) {
        //平台唯一标识码
        String uniqueIdentification = platformId + TokenUtils.getIdByJwtToken(request) + "kindergartenInvitationCode";
        //存在直接返回邀请码
        String code = redisTemplate.opsForValue().get(uniqueIdentification);
        if (StringUtils.isEmpty(code)) {
            code = GenerateCodeUtil.generateCode();
            redisTemplate.opsForValue().set(uniqueIdentification, code, 7, TimeUnit.DAYS);
        }
        //过期时间
        Long expireTime = redisTemplate.opsForValue().getOperations().getExpire(uniqueIdentification);
        return Objects.requireNonNull(R.ok().put("invitationCode", code)).put("expireTime", expireTime);
    }

    @ApiOperation(value = "平台邀请二维码生成")
    @GetMapping(value = "/generationQrCode", produces = "image/png")
    public void generationQrCode(@ApiParam(name = "url", value = "链接", required = true) @RequestParam String url, HttpServletResponse response) {
        platformTeamAccountService.generationQrCode(url, response);
    }


//    @PostMapping("/platformTeamAccountMergeList")
//    @ApiOperation(value = "平台团队账号列表--合并数据查询(以账号为维度查询合并-后台调用)", response = PlatformTeamAccountResp.class)
//    public R partnerAccountMergeList(@RequestBody @Valid PlatformTeamAccountListReq req) {
//        return platformTeamAccountService.platformTeamAccountMergeList(req);
//    }

//    @PostMapping("/teamPartnerList")
//    @ApiOperation(value = "团队合伙人列表(以团队维度查询-小程序调用)", response = PlatformTeamAccountResp.class)
//    public R teamPartnerList(@RequestBody @Valid PlatformTeamAccountListReq req) {
//        return platformTeamAccountService.teamPartnerList(req);
//    }

//    @PostMapping("/queryTeamMembers")
//    @ApiOperation(value = "查询团队成员", response = PlatformTeamAccountResp.class)
//    public R queryTeamMembers(@RequestBody TeamMemberVo teamMemberVo) {
//        return platformTeamAccountService.queryTeamMembers(teamMemberVo);
//    }


//    @GetMapping("/partnerAccountDetail")
//    // @ApiOperation(value = "合伙人账号详情", response = PlatformTeamAccountResp.class)
//    public R partnerAccountDetail(@ApiParam(name = "accountId", value = "合伙人账号账号id")
//                                  @RequestParam Integer accountId) {
//        return platformTeamAccountService.partnerAccountDetail(accountId);
//    }


//    @PostMapping("/modifyPartnerAccount")
//    // @ApiOperation(value = "编辑合伙人账号")
//    public R modifyPartnerAccount(@RequestBody @Valid PartnerAccountUpdateReq updateReq) {
//        return platformTeamAccountService.modifyPartnerAccount(updateReq);
//    }


//    @PostMapping("/generateTransferCode")
//    @ApiOperation(value = "生成转让邀请码")
//    public R generateTransferCode(@RequestParam("accountId") String accountId) {
//        //存在直接返回邀请码
//        String code = redisTemplate.opsForValue().get(accountId+"transferCode");
//        if (StringUtils.isEmpty(code)){
//            code = GenerateCodeUtil.generateCode();
//            redisTemplate.opsForValue().set(accountId+"transferCode",code,1, TimeUnit.DAYS);
//        }
//        //过期时间
//        Long expireTime = redisTemplate.opsForValue().getOperations().getExpire(accountId+"transferCode");
//        return R.ok().put("transferCode",code).put("expireTime",expireTime);
//    }

//    @PostMapping("/transferAdmin")
//    @ApiOperation(value = "新增转让超管")
//    public R transferAdmin(@RequestBody @Valid TransferAdminReq transferAdminReq) {
//        return platformTeamAccountService.transferAdmin(transferAdminReq);
//    }

    @PostMapping("/transferTeam")
    @ApiOperation(value = "团队内转让超管")
    public R transferTeam(@RequestBody @Valid TransferTeamReq transferTeamReq) {
        return platformTeamAccountService.transferTeam(transferTeamReq);
    }


    @GetMapping("/removeOrganization")
    @ApiOperation(value = "移除组织")
    public R removeOrganization(@ApiParam(name = "teamId", value = "团队id") @RequestParam Integer teamId, @ApiParam(name = "manageId", value = "组织id") @RequestParam Integer manageId) {
        PlatformTeamClassification teamClassification = platformTeamClassificationService.getById(teamId);
        if (teamClassification.getIsTeam() == 1) {
            throw new CustomException(ExceptionEnum.PARTNER_REMOVAL);
        }
        //删除团队成员
        platformTeamClassificationService.removeById(teamId);
        //TODO 删除该团队相关角色信息
        platformTeamAccountService.removeById(manageId);
        boolean remove = platformTeamService.remove(new QueryWrapper<PlatformTeam>().eq("manage_id", manageId).eq("classification_id", teamId));
        return remove ? R.ok() : R.error();
    }

    @ApiOperation(value = "用户详情更新团队状态，禁用无法查询到该组织，同步禁用用户账号")
    @PostMapping("/updateTeamStatus")
    public R updateTeamStatus(@RequestParam("teamId") @ApiParam(value = "团队id") Integer teamId, @RequestParam("isEnable") @ApiParam(value = "是否禁用（1->禁用，0->启用）") Integer isEnable) {
        PlatformTeamClassification platformTeamClassification = new PlatformTeamClassification();
        platformTeamClassification.setId(teamId);
        platformTeamClassification.setIsEnable(isEnable);
        boolean update = platformTeamClassificationService.updateById(platformTeamClassification);
        //查询组织平台账号
        PlatformTeam platformTeam = platformTeamService.getOne(new QueryWrapper<PlatformTeam>().eq("classification_id", teamId));
        PlatformTeamAccount platformTeamAccount = platformTeamAccountService.getById(platformTeam.getManageId());
        int status = 1;
        if (isEnable == 1) {
            status = 0;
        }
        platformTeamClassificationService.updateByAccountId(platformTeamAccount.getAccountId(), status);
        return update ? R.ok() : R.error();
    }

    @ApiOperation(value = "用户列表更新账号状态，禁用整个账号禁止登录，不管是哪个端，哪个组织")
    @PostMapping("/updateAccountStatus")
    public R updateAccountStatus(@RequestParam("openId") @ApiParam(value = "openId") String openId, @RequestParam("isEnable") @ApiParam(value = "是否禁用（1->启用，0->禁用）") Integer isEnable) {
        //根据openId查询账号详情List
        List<UserAccount> userAccountList = platformTeamClassificationService.selectAccountList(openId);
        for (UserAccount userAccount : userAccountList) {
            //更新账号信息
            platformTeamClassificationService.updateByAccountId(userAccount.getId(), isEnable);
            if (Integer.parseInt(userAccount.getPlatformId()) != 7) {
                //查询账号绑定的组织信息
                List<PlatformTeamAccount> platformTeamAccounts = platformTeamAccountService.list(new QueryWrapper<PlatformTeamAccount>().eq("account_id", userAccount.getId()));
                for (PlatformTeamAccount platformTeamAccount : platformTeamAccounts) {
                    //查询组织平台账号
                    PlatformTeam platformTeam = platformTeamService.getOne(new QueryWrapper<PlatformTeam>().eq("manage_id", platformTeamAccount.getManageId()));
                    PlatformTeamClassification platformTeamClassification = new PlatformTeamClassification();
                    platformTeamClassification.setId(platformTeam.getClassificationId());
                    int status = 1;
                    if (isEnable == 1) {
                        status = 0;
                    }
                    platformTeamClassification.setIsEnable(status);
                    //更新账号绑定的组织信息
                    platformTeamClassificationService.updateById(platformTeamClassification);
                }

            }
        }
        return R.ok();
    }

    //================================以下接口为小程序调用===================================
    @ApiOperation(value = "幼教产品微信小程序回调登录")
    @PostMapping("/weChatAppletCallbackByKindergarten")
    public R weChatAppletCallbackByKindergarten(@RequestBody @Valid AppletCallbackReq appletCallbackReq, HttpServletResponse response) throws Exception {
        Map<String, Object> result = platformTeamAccountService.kindergartenWeChatAccountApplication(appletCallbackReq);
        return R.ok().put("data", result);
    }


    @ApiOperation(value = "幼教产品微信小程序账号注册（根据openid）")
    @PostMapping("/kindergartenWeChatApplication")
    public R kindergartenWeChatApplication(@RequestBody @Valid AppletCallbackReq appletCallbackReq) {
        return platformTeamAccountService.kindergartenWeChatApplication(appletCallbackReq);
    }

    @ApiOperation(value = "幼教产品微信小程序查询手机号登录")
    @PostMapping("/loginWithPhoneNumber")
    public R loginWithPhoneNumber(@RequestBody @Valid AppletCallbackReq appletCallbackReq) {
        return platformTeamAccountService.getUserAccountByPhone(appletCallbackReq);
    }

    @ApiOperation(value = "获取当前平台用户的团队信息(接口弃用，组织信息从用户信息中获取)", response = PartnerAccountResp.class)
    @GetMapping("/getTeamsByPlatformId")
    public R getTeamsByPlatformId(@ApiParam(name = "platformId", value = "平台id") @RequestParam Integer platformId, HttpServletRequest request) {
        String accountId = platformTeamAccountService.getCurrentPlatformAccount(TokenUtils.getIdByJwtToken(request), platformId);
        return platformTeamAccountService.getTeamsByPlatformId(accountId, platformId);
    }

    @PostMapping("/platformTeamAccountList")
    @ApiOperation(value = "小程序根据不同平台查询分类下所有团队及成员", response = PlatformTeamAccountResp.class)
    public R platformTeamAccountList(@RequestBody @Valid PlatformTeamAccountListReq req) {
        return platformTeamAccountService.pagePlatformTeamAccountList(req);
    }

    @ApiOperation(value = "小程序团队信息维护")
    @PostMapping("/updateTeamInfo")
    @CacheEvict(cacheNames = "shop_detail", key = "#teamInfo.id")
    public R updateTeamInfo(@RequestBody TeamInfo teamInfo) {
        PlatformTeamClassification platformTeamClassification = new PlatformTeamClassification();
        BeanUtils.copyProperties(teamInfo, platformTeamClassification);
        // 供应商角色下编辑分类
        // 查询供应商的分类信息(只有当前团队为供应商才有supplierClassification数据)

        Integer accountId = platformTeamService.getAccountIdByTeamId(teamInfo.getId());


        QueryWrapper<SupplierClassificationConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("account_id", accountId);
        //删除供应商原有分类重新新增分类配置信息
        List<SupplierClassificationConfig> classificationConfigs = supplierClassificationConfigService.list(queryWrapper);

        if (!classificationConfigs.isEmpty()){
            supplierClassificationConfigService.remove(queryWrapper);
        }


        if (teamInfo.getSupplierClassificationConfigs()!=null) {
            for (SupplierClassificationConfig config : teamInfo.getSupplierClassificationConfigs()) {
                //获取teamInfo.getSupplierClassificationConfigs()第一个账号id
                config.setAccountId(accountId);
            }
            //新增分类配置信息
            supplierClassificationConfigService.saveBatch(teamInfo.getSupplierClassificationConfigs());
        }


        boolean update = platformTeamClassificationService.updateById(platformTeamClassification);
        return update ? R.ok() : R.error();
    }


    @ApiOperation(value = "查询团队信息")
    @GetMapping("/queryTeamInfo")
    public R queryTeamInfo(@ApiParam(value = "团队id") @RequestParam("teamId") Integer teamId, HttpServletRequest request) {
        String accountId = TokenUtils.getIdByJwtToken(request);
        PlatformTeamClassification teamInfo = platformTeamClassificationService.getById(teamId);
        //查询供应商的分类信息(只有当前团队为供应商才有supplierClassification数据)
        List<TeamClassificationVO> supplierClassification = platformTeamService.getTeamClassificationVOByTeamId(teamId);

        CollectionProvider collectionProviders = collectionProviderService.getCollectionProviderByUserIdAndProviderId(teamId, Integer.valueOf(accountId));
        return R.ok().put("teamInfo", teamInfo).put("supplierClassification", supplierClassification).put("collectionInformation",collectionProviders);

    }

    @PostMapping("/inviteJoinOrganization")
    @ApiOperation(value = "邀请加入不同平台组织")
    public R inviteJoinOrganization(@RequestBody @Valid InviteJoinInfo inviteJoinInfo) {
        return platformTeamAccountService.inviteJoinOrganization(inviteJoinInfo);
    }

    public static void main(String[] args) {
        String host = "https://gpsyl.market.alicloudapi.com";
        String path = "/ai_market/kg/gu_piao_shi_ying_lv/v1";
        String method = "GET";
        String appcode = "4c22c2b34b2d4b5aaa57186a49aa3a4c";
        Map<String, String> headers = new HashMap<String, String>();
        //最后在header中的格式(中间是英文空格)为Authorization:APPCODE 83359fd73fe94948385f570e3c139105
        headers.put("Authorization", "APPCODE " + appcode);
        Map<String, String> querys = new HashMap<String, String>();
        querys.put("PRICE", "6.5899");
        querys.put("PROFIT", "1");


        try {
            /**
             * 重要提示如下:
             * HttpUtils请从
             * https://github.com/aliyun/api-gateway-demo-sign-java/blob/master/src/main/java/com/aliyun/api/gateway/demo/util/HttpUtils.java
             * 下载
             *
             * 相应的依赖请参照
             * https://github.com/aliyun/api-gateway-demo-sign-java/blob/master/pom.xml
             */
            HttpResponse response = HttpUtils.doGet(host, path, method, headers, querys);
            System.out.println(response.toString());
            //获取response的body
            //System.out.println(EntityUtils.toString(response.getEntity()));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}

