/*
 * Copyright (c) 2018-2999 广州市蓝海创新科技有限公司 All rights reserved.
 *
 * https://www.mall4j.com/
 *
 * 未经允许，不可做商业用途！
 *
 * 版权所有，侵权必究！
 */

package com.huoran.nakadai.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.huoran.common.exception.YamiShopBindException;
import com.huoran.common.response.R;
import com.huoran.common.response.ServerResponseEntity;
import com.huoran.common.utils.SecurityUtils;
import com.huoran.common.utils.TokenUtils;
import com.huoran.nakadai.entity.model.Product;
import com.huoran.nakadai.entity.model.Sku;
import com.huoran.nakadai.entity.param.ProductParam;
import com.huoran.nakadai.entity.req.ProductPageReq;
import com.huoran.nakadai.entity.res.PlatformTeamAccountResp;
import com.huoran.nakadai.entity.vo.ProductCount;
import com.huoran.nakadai.service.BasketService;
import com.huoran.nakadai.service.ProdTagReferenceService;
import com.huoran.nakadai.service.ProductService;
import com.huoran.nakadai.service.SkuService;
import com.huoran.nakadai.utils.Json;
import com.huoran.nakadai.utils.PageParam;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;


@Api(tags = "商品列表、商品发布管理")
@RestController
@RequestMapping("/prod")
public class ProductController {

    @Autowired
    private ProductService productService;

    @Autowired
    private SkuService skuService;

    @Autowired
    private ProdTagReferenceService prodTagReferenceService;

    @Autowired
    private BasketService basketService;

    @GetMapping("/list")
    @ApiOperation(value = "采购查询店铺中所有产品规则列表展示")
    public ServerResponseEntity<List<Sku>> list(@RequestParam("shopId") Integer shopId) {
        List<Product> productList = productService.list(
                new QueryWrapper<Product>().eq("shop_id", shopId).eq("status", 1));
        List<Sku> skuList = new ArrayList<>();
        for (Product product : productList) {
            List<Sku> list = skuService.list(new QueryWrapper<Sku>().eq("prod_id", product.getProdId()).
                    eq("is_delete", 0));
            for (Sku sku : list) {
                sku.setProdName(product.getProdName());
            }
            skuList.addAll(list);
        }
        return ServerResponseEntity.success(skuList);
    }

    @GetMapping("/page")
    @ApiOperation(value = "分页获取商品信息", response = Product.class)
    public R page(ProductPageReq productPageReq, HttpServletRequest request) {
        PageParam<Product> page = new PageParam<>();
        if (productPageReq.getCurrent()!= null){
            page.setCurrent(productPageReq.getCurrent());
        }
        if (productPageReq.getSize()!= null){
            page.setSize(productPageReq.getSize());
        }
        Integer shopId = SecurityUtils.getCurrentInfo().getShopId();
        if (productPageReq.getShopId()!= null){
            shopId =  productPageReq.getShopId();
        }
        IPage<Product> products = productService.page(page,
                new LambdaQueryWrapper<Product>()
                        .like(StrUtil.isNotBlank(productPageReq.getProdName()), Product::getProdName, productPageReq.getProdName())
                        .eq(Product::getShopId, shopId)
                        .eq(productPageReq.getStatus() != null, Product::getStatus, productPageReq.getStatus())
                        .orderByDesc(Product::getPutawayTime));
        List<ProductCount> productCount = productService.selectCount(TokenUtils.getShopIdByJwtToken(request));
        return R.ok().put("page", products).put("productCount", productCount);
    }


    @GetMapping("/info")
    @ApiOperation(value = "获取商品信息", response = Product.class)
    public ServerResponseEntity<Product> info(@RequestParam("prodId") Integer prodId, HttpServletRequest request) {
        Product prod = productService.getProductByProdId(prodId);
//        if (!Objects.equals(prod.getShopId(), TokenUtils.getShopIdByJwtToken(request))) {
//            throw new YamiShopBindException("没有权限获取该商品规格信息");
//        }
        List<Sku> skuList = skuService.listByProdId(prodId);
        prod.setSkuList(skuList);

        //获取分组标签
//        List<Long> listTagId = prodTagReferenceService.listTagIdByProdId(prodId);
//        prod.setTagList(listTagId);
        return ServerResponseEntity.success(prod);
    }

    @PostMapping("/save")
    @ApiOperation(value = "保存商品信息")
    public ServerResponseEntity<String> save(@Valid @RequestBody ProductParam productParam,HttpServletRequest request) {
//        productParam.setShopId(TokenUtils.getShopIdByJwtToken(request));
        checkParam(productParam);
        Product product = BeanUtil.copyProperties(productParam, Product.class);
        product.setShopId(TokenUtils.getShopIdByJwtToken(request));
//        product.setDeliveryMode(Json.toJsonString(productParam.getDeliveryModeVo()));
        product.setUpdateTime(new Date());
        if (product.getStatus() == 1) {
            product.setPutawayTime(new Date());
        }
        product.setCreateTime(new Date());
        productService.saveProduct(product);
        return ServerResponseEntity.success();
    }


    @PostMapping("/update")
    @ApiOperation(value = "修改商品信息")
    public ServerResponseEntity<String> update(@Valid @RequestBody ProductParam productParam,HttpServletRequest request) {
//        productParam.setShopId(TokenUtils.getShopIdByJwtToken(request));
        checkParam(productParam);
        Product dbProduct = productService.getProductByProdId(productParam.getProdId());
        if (!Objects.equals(dbProduct.getShopId(), TokenUtils.getShopIdByJwtToken(request))) {
            return ServerResponseEntity.showFailMsg("无法修改非本店铺商品信息");
        }

        List<Sku> dbSkus = skuService.listByProdId(dbProduct.getProdId());

        Product product = BeanUtil.copyProperties(productParam, Product.class);
//        product.setDeliveryMode(Json.toJsonString(productParam.getDeliveryModeVo()));
        product.setUpdateTime(new Date());

        if (productParam.getStatus() == 1) {
            product.setPutawayTime(new Date());
        }
        dbProduct.setSkuList(dbSkus);
        productService.updateProduct(product, dbProduct);


//        List<String> userIds = basketService.listUserIdByProdId(product.getProdId());
//
//        for (String userId : userIds) {
//            basketService.removeShopCartItemsCacheByUserId(userId);
//        }
        for (Sku sku : dbSkus) {
            skuService.removeSkuCacheBySkuId(sku.getSkuId(), sku.getProdId());
        }
        return ServerResponseEntity.success();
    }

    /**
     * 删除
     */
    public ServerResponseEntity<Void> delete(Integer prodId,HttpServletRequest request) {
        Product dbProduct = productService.getProductByProdId(prodId);
//        if (!Objects.equals(dbProduct.getShopId(), TokenUtils.getShopIdByJwtToken(request))) {
//            throw new YamiShopBindException("无法获取非本店铺商品信息");
//        }
        List<Sku> dbSkus = skuService.listByProdId(dbProduct.getProdId());
        // 删除商品
        productService.removeProductByProdId(prodId);

        for (Sku sku : dbSkus) {
            skuService.removeSkuCacheBySkuId(sku.getSkuId(), sku.getProdId());
        }


//        List<String> userIds = basketService.listUserIdByProdId(prodId);
//
//        for (String userId : userIds) {
//            basketService.removeShopCartItemsCacheByUserId(userId);
//        }

        return ServerResponseEntity.success();
    }

    @PostMapping("/batchDelete")
    @ApiOperation(value = "批量删除商品信息")
    public ServerResponseEntity<Void> batchDelete(@RequestBody Integer[] prodIds,HttpServletRequest request) {
        for (Integer prodId : prodIds) {
            delete(prodId,request);
        }
        return ServerResponseEntity.success();
    }


    @PostMapping("/prodStatus")
    @ApiOperation(value = "更新商品状态")
    public ServerResponseEntity<Void> shopStatus(
            @ApiParam(value = "商品id")
            @RequestParam Integer prodId,
            @ApiParam(value = "prodStatus:默认是1，表示上架, 2下架，3草稿，4审核,0为删除")
            @RequestParam Integer prodStatus) {
        Product product = new Product();
        product.setProdId(prodId);
        product.setStatus(prodStatus);
        if (prodStatus == 1) {
            product.setPutawayTime(new Date());
        }
        productService.updateById(product);
        productService.removeProductCacheByProdId(prodId);
//        List<String> userIds = basketService.listUserIdByProdId(prodId);
//
//        for (String userId : userIds) {
//            basketService.removeShopCartItemsCacheByUserId(userId);
//        }
        return ServerResponseEntity.success();
    }

    private void checkParam(ProductParam productParam) {
        QueryWrapper<Product> productQueryWrapper = new QueryWrapper<>();
        productQueryWrapper.eq("prod_name",productParam.getProdName());
        if (productParam.getProdId()!= null){
            productQueryWrapper.notIn("prod_id",productParam.getProdId());
        }
        Product one = productService.getOne(productQueryWrapper);
        if (one != null){
            throw new YamiShopBindException("商品名称已存在");
        }
//        if (CollectionUtil.isEmpty(productParam.getTagList())) {
//            throw new YamiShopBindException("请选择产品分组");
//        }

//        Product.DeliveryModeVO deliveryMode = productParam.getDeliveryModeVo();
//        boolean hasDeliverMode = deliveryMode != null
//                && (deliveryMode.getHasShopDelivery() || deliveryMode.getHasUserPickUp());
//        if (!hasDeliverMode) {
//            throw new YamiShopBindException("请选择配送方式");
//        }
        List<Sku> skuList = productParam.getSkuList();
        List<String> stringList = skuList.stream().map(Sku::getSkuName)
                .collect(Collectors.toList());
        long count = stringList.stream().distinct().count();
        if (stringList.size() != count) {
            throw new YamiShopBindException("商品规格名称重复");
        }
//        boolean isAllUnUse = true;
//        for (Sku sku : skuList) {
//            if (sku.getStatus() == 1) {
//                isAllUnUse = false;
//            }
//        }
//        if (isAllUnUse) {
//            throw new YamiShopBindException("至少要启用一种商品规格");
//        }
    }
}
