package com.huoran.nakadai.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huoran.common.constant.DelConstant;
import com.huoran.common.response.R;
import com.huoran.nakadai.entity.City;
import com.huoran.nakadai.entity.Province;
import com.huoran.nakadai.entity.School;
import com.huoran.nakadai.service.ICityService;
import com.huoran.nakadai.service.IProvinceService;
import com.huoran.nakadai.service.ISchoolService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * <p>
 * 学校表 前端控制器
 * </p>
 *
 * @author huoran
 * @since 2021-06-29
 */
@Api(value = "学校管理", tags = "学校管理")
@RestController
@RequestMapping("/nakadai/school")
public class SchoolController {

    @Autowired
    private ISchoolService iSchoolService;

    @Autowired
    private ICityService iCityService;

    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private IProvinceService iProvinceService;


    @ApiOperation(value = "查询学校列表", notes = "查询学校列表")
    @GetMapping("/querySchool")
    public R querySchool(@RequestParam(required = false) String schoolName, @RequestParam(required = false) Integer provinceId, @RequestParam(required = false) Integer cityId) {
        List<School> data = iSchoolService.querySchool(schoolName, provinceId, cityId);
        return R.ok(data);
    }


    @ApiOperation(value = "按省市份获取学校", notes = "按省市获取学校")
    @GetMapping("/getSchoolsByProvince")
    public R getSchoolsByProvince(@RequestParam(required = false) String schoolName, @RequestParam(required = false) Integer provinceId, @RequestParam(required = false) Integer cityId) {
        List<School> data = iSchoolService.getSchoolsByProvince(schoolName, provinceId, cityId);
        return R.ok(data);
    }


    @ApiOperation(value = "按学校id获取学校名称", notes = "按学校id获取学校名称")
    @GetMapping("/getTheSchoolNameBySchoolId")
    public School getTheSchoolNameBySchoolId(@RequestParam(required = false) Integer schoolId) {

        School school = iSchoolService.getById(schoolId);
        return school;
    }

    @ApiOperation(value = "按学校名称获取id", notes = "按学校名称获取id")
    @GetMapping("/getTheIdBySchoolName")
    public School getTheIdBySchoolName(@RequestParam(required = false) String schoolName) {
        QueryWrapper<School> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("school_name", schoolName);
        queryWrapper.eq("is_del", DelConstant.NOT_DEL);
        School school = iSchoolService.getOne(queryWrapper);
        return school;
    }


    @ApiOperation(value = "自行新增客户或者学校", notes = "自行新增客户或者学校")
    @CacheEvict(value = "n_school_", key = "'querySchool'")
    @GetMapping("/addNewCustomersByOneself")
    public R addNewCustomersByOneself(@RequestParam(required = false) String schoolName, @RequestParam(required = false) String provinceName, @RequestParam(required = false) String cityName) {
        QueryWrapper<School> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("school_name", schoolName);
        queryWrapper.eq("is_del", DelConstant.NOT_DEL);
        School school = iSchoolService.getOne(queryWrapper);


        QueryWrapper<Province> provinceQueryWrapper = new QueryWrapper<>();
        provinceQueryWrapper.eq("province_name", provinceName);
        provinceQueryWrapper.eq("is_del", DelConstant.NOT_DEL);
        Province province = iProvinceService.getOne(provinceQueryWrapper);


        QueryWrapper<City> cityQueryWrapper = new QueryWrapper<>();
        cityQueryWrapper.eq("city_name", cityName);
        cityQueryWrapper.eq("is_del", DelConstant.NOT_DEL);
        City city = iCityService.getOne(cityQueryWrapper);


        if (school == null) {
            //新增
            if (province != null && city != null) {
                School school1 = new School();
                school1.setCityId(city.getCityId());
                school1.setProvinceId(province.getProvinceId());
                school1.setSchoolName(schoolName);
                school1.setProvinceName(provinceName);
                school1.setCityName(cityName);
                school1.setLevel(0);
                school1.setIsDel("0");
                //删除缓存 n_school_::querySchool
                redisTemplate.delete("querySchool");

                return iSchoolService.save(school1) ? R.ok().put("schoolId",school1.getSchoolId()) : R.error();
            }
        }
        return R.error("当前客户/学校已存在!");
    }
}
