/*
 * Copyright (c) 2018-2999 广州市蓝海创新科技有限公司 All rights reserved.
 *
 * https://www.mall4j.com/
 *
 * 未经允许，不可做商业用途！
 *
 * 版权所有，侵权必究！
 */

package com.huoran.nakadai.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.huoran.common.response.ServerResponseEntity;
import com.huoran.common.utils.TokenUtils;
import com.huoran.nakadai.entity.model.ShopDetail;
import com.huoran.nakadai.entity.param.ShopDetailParam;
import com.huoran.nakadai.service.ShopDetailService;
import com.huoran.nakadai.utils.PageParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;



/**
 *
 * @author lgh on 2018/08/29.
 */
@RestController
@RequestMapping("/shop/shopDetail")
public class ShopDetailController {

    @Autowired
    private ShopDetailService shopDetailService;


	/**
	 * 修改分销开关
	 */
	@PutMapping("/isDistribution")
	public ServerResponseEntity<Void> updateIsDistribution(@RequestParam Integer isDistribution, HttpServletRequest request){
		ShopDetail shopDetail=new ShopDetail();
		Integer shopId = TokenUtils.getShopIdByJwtToken(request);
		shopDetail.setShopId(shopId);
		shopDetail.setIsDistribution(isDistribution);
		shopDetailService.updateById(shopDetail);
		// 更新完成后删除缓存
		shopDetailService.removeShopDetailCacheByShopId(shopDetail.getShopId());
		return ServerResponseEntity.success();
	}
	/**
	 * 获取信息
	 */
	@GetMapping("/info")
	public ServerResponseEntity<ShopDetail> info(HttpServletRequest request){
		ShopDetail shopDetail = shopDetailService.getShopDetailByShopId(TokenUtils.getShopIdByJwtToken(request));
		return ServerResponseEntity.success(shopDetail);
	}


	/**
	 * 分页获取
	 */
    @GetMapping("/page")
	public ServerResponseEntity<IPage<ShopDetail>> page(ShopDetail shopDetail, PageParam<ShopDetail> page){
		IPage<ShopDetail> shopDetails = shopDetailService.page(page,
				new LambdaQueryWrapper<ShopDetail>()
						.like(StrUtil.isNotBlank(shopDetail.getShopName()),ShopDetail::getShopName,shopDetail.getShopName())
						.orderByDesc(ShopDetail::getShopId));
		return ServerResponseEntity.success(shopDetails);
	}

	/**
	 * 获取信息
	 */
	@GetMapping("/info/{shopId}")
	public ServerResponseEntity<ShopDetail> info(@PathVariable("shopId") Integer shopId){
		ShopDetail shopDetail = shopDetailService.getShopDetailByShopId(shopId);
		// 店铺图片
		return ServerResponseEntity.success(shopDetail);
	}

	/**
	 * 保存
	 */
	@PostMapping
	public ServerResponseEntity<Void> save(@Valid ShopDetailParam shopDetailParam){
		ShopDetail shopDetail = BeanUtil.copyProperties(shopDetailParam, ShopDetail.class);
		shopDetail.setCreateTime(new Date());
		shopDetail.setShopStatus(1);
		shopDetailService.save(shopDetail);
		return ServerResponseEntity.success();
	}

	/**
	 * 修改
	 */
	@PutMapping
	public ServerResponseEntity<Void> update(@Valid ShopDetailParam shopDetailParam){
		ShopDetail daShopDetail = shopDetailService.getShopDetailByShopId(shopDetailParam.getShopId());
		ShopDetail shopDetail = BeanUtil.copyProperties(shopDetailParam, ShopDetail.class);
		shopDetail.setUpdateTime(new Date());
		shopDetailService.updateShopDetail(shopDetail,daShopDetail);
		return ServerResponseEntity.success();
	}

	/**
	 * 删除
	 */
	@DeleteMapping("/{id}")
	public ServerResponseEntity<Void> delete(@PathVariable Integer id){
		shopDetailService.deleteShopDetailByShopId(id);
		return ServerResponseEntity.success();
	}

	/**
	 * 更新店铺状态
	 */
	@PutMapping("/shopStatus")
	public ServerResponseEntity<Void> shopStatus(@RequestParam Integer shopId,@RequestParam Integer shopStatus){
		ShopDetail shopDetail = new ShopDetail();
		shopDetail.setShopId(shopId);
		shopDetail.setShopStatus(shopStatus);
		shopDetailService.updateById(shopDetail);
		// 更新完成后删除缓存
		shopDetailService.removeShopDetailCacheByShopId(shopDetail.getShopId());
		return ServerResponseEntity.success();
	}


	/**
	 * 获取所有的店铺名称
	 */
//    @GetMapping("/listShopName")
//	public ServerResponseEntity<List<ShopDetail>> listShopName(){
//		List<ShopDetail> list = shopDetailService.list().stream().map((dbShopDetail) ->{
//			ShopDetail shopDetail = new ShopDetail();
//			shopDetail.setShopId(dbShopDetail.getShopId());
//			shopDetail.setShopName(dbShopDetail.getShopName());
//			return shopDetail;
//		}).collect(Collectors.toList());
//		return ServerResponseEntity.success(list);
//	}
}
