package com.huoran.nakadai.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huoran.api.UserClient;
import com.huoran.common.response.R;
import com.huoran.common.utils.TokenUtils;
import com.huoran.nakadai.entity.SupplierClassification;
import com.huoran.nakadai.entity.req.SupplierClassificationReq;
import com.huoran.nakadai.service.SupplierClassificationService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

/**
 * @描述：供应商分类信息控制类
 * @作者: Rong
 * @日期: 2023-12-27
 */
@RestController
@RequestMapping("/supplierClassification")
@Api(value = "供应商分类信息:SupplierClassificationController", tags = "供应商分类信息")
public class SupplierClassificationController {

    @Autowired
    public SupplierClassificationService service;

    @Autowired
    private UserClient userClient;

    
    @PostMapping("/treeStructureList")
    @ApiOperation(value = "树结构列表", response = SupplierClassification.class)
    public R treeStructureList() {
        /*QueryWrapper<SupplierClassification> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("parent_id", supplierClassification.getParentId());*/
        List<SupplierClassification> list = service.list(null);


        return R.ok().put("list", buildTree(list));
    }

    public List<SupplierClassification> buildTree(List<SupplierClassification> list) {
        List<SupplierClassification> treeList = new ArrayList<SupplierClassification>();
        List<SupplierClassification> children;

        for (SupplierClassification address : list) {
            if (address.getParentId() == 0) {
                treeList.add(address);
            }
            children = new ArrayList<SupplierClassification>();
            for (SupplierClassification node : list) {
                if (node.getParentId() == address.getId()) {
                    children.add(node);
                }
            }
            address.setChildren(children);
        }

        return treeList;
    }
 

    @PostMapping("/findById")
    @ApiOperation(value = "查询详情", response = SupplierClassification.class)
    public R findById(@ApiParam(name = "id", value = "主键", required = true) @RequestParam Integer id) {
        SupplierClassification supplierClassification = service.getById(id);
        return R.ok().put("data", supplierClassification);
    }


    @PostMapping("/save")
    @ApiOperation(value = "新增", response = SupplierClassification.class)
    public R save(@RequestBody @ApiParam(name = "供应商分类信息对象", value = "传入json格式", required = true) SupplierClassification supplierClassification, HttpServletRequest request) {
        String accountId = TokenUtils.getIdByJwtToken(request);
        Integer schoolId = userClient.getSchoolIdByAccountId(accountId);
        supplierClassification.setAccountId(Integer.valueOf(accountId));
        boolean addState = service.save(supplierClassification);
        return addState ? R.ok() : R.error("新增失败");
    }


    @PostMapping("/update")
    @ApiOperation(value = "修改", response = SupplierClassification.class)
    public R update(@RequestBody @ApiParam(name = "供应商分类信息对象", value = "传入json格式", required = true) SupplierClassification supplierClassification, HttpServletRequest request) {
        String accountId = TokenUtils.getIdByJwtToken(request);
        Integer schoolId = userClient.getSchoolIdByAccountId(accountId);
        supplierClassification.setAccountId(Integer.valueOf(accountId));
        boolean updateState = service.updateById(supplierClassification);
        return updateState ? R.ok() : R.error("编辑失败");
    }


    @PostMapping("/batchDeletion")
    @ApiOperation(value = "批量删除", response = SupplierClassification.class)
    public R batchDeletion(@ApiParam(name = "ids", value = "主键", required = true) @RequestBody List<Integer> ids) {
        boolean delState = service.removeByIds(ids);
        return delState ? R.ok() : R.error("删除失败");
    }
}

