package com.huoran.nakadai.entity;

import com.baomidou.mybatisplus.annotation.*;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

/**
 * <p>
 * 商城管理表
 * </p>
 *
 * @author chen
 * @since 2023-04-25
 */
@Data
@EqualsAndHashCode(callSuper = false)
@Accessors(chain = true)
@TableName("hr_mall")
@ApiModel(value = "Mall对象", description = "商城管理表")
public class Mall implements Serializable {

    private static final long serialVersionUID = 1L;

    @ApiModelProperty(value = "商城主键id")
    @TableId(value = "mall_id", type = IdType.AUTO)
    private Integer mallId;

    @ApiModelProperty(value = "关联产品（产品表为的课程、数据产品、职站增值模块）")
    private Integer associatedProduct;

    @ApiModelProperty(value = "是否关联产品，默认为0关联，1不关联")
    private Integer isAssociatedProduct;

    @ApiModelProperty(value = "关联产品名称")
    private String associatedProductName;

    @ApiModelProperty(value = "区分产品权限（0为数据平台权限，1为课程权限，2为职站增值模块，3为实训课程（非集成），4为实验工具，对应id为商城管理中的产品）")
    private Integer authority;


    @ApiModelProperty(value = "主题id")
    private Integer themeId;

    @ApiModelProperty(value = "产品名称")
    private String productName;

    @ApiModelProperty(value = "匹配课程")
    private String matchingCourse;

    @ApiModelProperty(value = "课程耗时")
    private String courseHours;

    @ApiModelProperty(value = "适用场景")
    private String applicationScenario;

    @ApiModelProperty(value = "封面图")
    private String coverDrawing;

    @ApiModelProperty(value = "界面图（支持上传4张，使用逗号分隔）")
    private String interfaceDiagram;

    @ApiModelProperty(value = "小程序图标")
    private String appletIcon;

    @ApiModelProperty(value = "市场建议单价")
    private BigDecimal marketUnitPrice;

    @ApiModelProperty(value = "产品简介")
    private String productIntroduction;

    @ApiModelProperty(value = "详情介绍")
    private String detailedIntroduction;

    @ApiModelProperty(value = "是否删除(0、未删除 1、已删除)")
    @TableLogic(value = "0", delval = "1")
    private int isDel;

    @ApiModelProperty(value = "上下架（0上架 1下架 默认0上架）")
    private int isShelves;

    @ApiModelProperty(value = "编辑人")
    private String accountId;

    @ApiModelProperty(value = "是否精选，默认0不精选，1为精选")
    private int selected;

    @ApiModelProperty(value = "商品状态，默认0正常，1为草稿")
    private int state;

    @ApiModelProperty(value = "更新时间")
    @TableField(fill = FieldFill.INSERT_UPDATE)
    @JsonFormat(locale = "zh", timezone = "GMT+8", pattern = "yyyy-MM-dd")
    private Date updateTime;

    @ApiModelProperty(value = "上架时间")
    @JsonFormat(locale = "zh", timezone = "GMT+8", pattern = "yyyy-MM-dd")
    private Date shelfTime;

    @ApiModelProperty(value = "创建时间")
    @TableField(fill = FieldFill.INSERT)
    @JsonFormat(locale = "zh", timezone = "GMT+8", pattern = "yyyy-MM-dd")
    private Date createTime;


}
