package com.huoran.nakadai.entity;

import com.baomidou.mybatisplus.annotation.*;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import javax.validation.constraints.NotNull;
import java.util.Date;
import java.util.List;

/**
 * @描述：合伙人——文章管理
 * @作者: Rong
 * @日期: 2023-03-16
 */
@Data
@ApiModel(value = "合伙人——文章管理")
@TableName("partner_article_management")
public class PartnerArticleManagement {

    private static final long serialVersionUID = 1L;

    @TableId(type = IdType.AUTO)
    @ApiModelProperty(value = "文章id")
    private Integer id;

    @ApiModelProperty(value = "文章标题(依据站点判重)")
    private String title;

    @ApiModelProperty(value = "是否删除(默认0未删除；1为已删除)")
    @TableLogic
    private Integer isDel;

    @ApiModelProperty(value = "标题图")
    private String titleImg;

    @ApiModelProperty(value = "封面图")
    private String bannerImg;

    @ApiModelProperty(value = "来源")
    private String source;

    @ApiModelProperty(value = "发布时间(不要时分秒)")
    private String releaseTime;

    @ApiModelProperty(value = "摘要")
    private String summary;

    /*@ApiModelProperty(value = "文章模板(前端自定义)")
    private Integer articleTemplate;*/

    @ApiModelProperty(value = "正文")
    private String mainBody;

    @ApiModelProperty(value = "作者")
    private String author;

    @ApiModelProperty(value = "审核")
    private String audit;

    @ApiModelProperty(value = "是否发布(0:草稿 1:已发布)")
    private Integer isRelease;

    @ApiModelProperty(value = "创建人id")
    private Integer founderId;

    @ApiModelProperty(value = "编辑人id")
    private Integer editorId;

    @ApiModelProperty(value = "文件")
    private String file;

    @ApiModelProperty(value = "创建时间")
    @TableField(fill = FieldFill.INSERT)
    private Date createTime;

    @ApiModelProperty(value = "更新时间")
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private Date updateTime;

    @ApiModelProperty(value = "文章总浏览量")
    private Integer totalBrowsing;


    @ApiModelProperty(value = "编辑")
    private String edit;

    @ApiModelProperty(value = "主题标签(partner_article_label表外键) 可多选逗号隔开")
    private String lableId;

    @ApiModelProperty(value = "所属分类(partner_article_classification表外键)")
    @NotNull(message = "请选择所属分类!")
    private Integer classificationId;

    @ApiModelProperty(value = "是否禁用(0默认，0启用 1禁用)")
    private Integer isDisable;

    /*@ApiModelProperty(value = "序号")
    private Integer sequence;*/

    @ApiModelProperty(value = "类型(1.学习2.资讯)")
    private Integer typeId;

    @ApiModelProperty(value = "文章附件")
    @TableField(exist = false)
    List<PartnerArticleFile> fileList;

    @ApiModelProperty(value = "分类名称")
    @TableField(exist = false)
    private List<String> classificationNameList;
    @ApiModelProperty(value = "标签名称")
    @TableField(exist = false)
    private List<String> labelNameList;
  /*  @Override
    public int compareTo(PartnerArticleManagement o) {
        return Integer.compare(this.sequence, o.getSequence());
    }
*/

    @ApiModelProperty(value = "是否置顶(默认为0 不置顶 1为置顶)")
    private Integer isTop;

    @ApiModelProperty(value = "置顶时间")
    private Date topTime;
}