package com.huoran.nakadai.entity.req;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;

import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.List;

/**
 * <p>
 * 课程管理
 * </p>
 *
 * @author lr
 * @since 2021-09-16
 */
@Data
@EqualsAndHashCode(callSuper = false)
@Accessors(chain = true)
@ApiModel(value = "创建课程所需参数", description = "创建课程所需参数")
public class AddCurriculumReq {


    @ApiModelProperty(value = "创建人id", hidden = true)
    private Integer accountId;

    @ApiModelProperty(value = "课程名称", required = true, example = "小贷业务实训课程")
    @NotNull(message = "课程名称不能为空！")
    private String curriculumName;

    @ApiModelProperty(value = "课程类别(0->理论 1-实训)", required = true, example = "0")
    @NotNull(message = "课程类别不能为空！")
    private Integer curriculumType;

    @ApiModelProperty(value = "学科类别id", required = true, example = "1")
    @NotNull(message = "学科类别不能为空！")
    private Integer categoryId;

    @ApiModelProperty(value = "专业类id", required = true, example = "1")
    @NotNull(message = "专业类不能为空！")
    private Integer professionalCategoryId;

    @ApiModelProperty(value = "专业id", required = true, example = "1")
    @NotNull(message = "专业不能为空！")
    private Integer professionalId;

    @ApiModelProperty(value = "预计课时", required = true, example = "64课时")
    @NotNull(message = "预计课时不能为空！")
    private String expectedCourse;

    @ApiModelProperty(value = "市场价格", required = true, example = "99.98")
//    @NotNull(message = "市场价格不能为空！")
    private BigDecimal marketPrice;

    @ApiModelProperty(value = "课程简介", required = true, example = "课程简介........")
    @NotNull(message = "课程简介不能为空！")
    private String briefIntroduction;

    @ApiModelProperty(value = "教学目标", required = true, example = "教学目标.......")
    @NotNull(message = "教学目标不能为空！")
    private String teachingObjectives;

    @ApiModelProperty(value = "练习配置")
    private List<ConfigurationReq> systemIdByPractice;

    @ApiModelProperty(value = "考核配置")
    private List<ConfigurationReq> systemIdByAssessment;

    @ApiModelProperty(value = "竞赛配置")
    private List<ConfigurationReq> systemIdByCompetition;

    @ApiModelProperty(value = "资源配置id")
    private Integer resourcesId;

    @ApiModelProperty(value = "学校id", hidden = true)
    private Integer schoolId;

//    @NotNull(message = "封面图片路径不能为空")
    @ApiModelProperty(value = "课程封面图片路径", required = true, example = "https://huoran.oss-cn-shenzhen.aliyuncs.com/20210926/jpg/1441973739251589120.jpg")
    private String coverUrl;

    @ApiModelProperty(value = "结算方式，0为单价，1为分成")
    private String settlementMethod;

    @ApiModelProperty(value = "供应厂商")
    private String supplier;

    @ApiModelProperty(value = "结算单价")
    private BigDecimal settlementPrice;

    @ApiModelProperty(value = "商务占比")
    private String businessProportion;

    @ApiModelProperty(value = "厂商占比")
    private String manufacturerProportion;

    @ApiModelProperty(value = "小程序图片地址")
    private String miniProgramPictureAddress;
}
