package com.huoran.nakadai.entity.req;

import com.baomidou.mybatisplus.annotation.*;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;
import java.io.Serializable;
import java.util.Date;

/**
 * <p>
 * 客户表
 * </p>
 *
 * @author huoran
 * @since 2021-06-28
 */
@Data
@TableName("customer")
public class CustomerReq implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 主键ID
     */
    @ApiModelProperty("主键ID")
    @TableId(value = "customer_id", type = IdType.AUTO)
    private Integer customerId;

    /**
     * 国家
     */
    @ApiModelProperty("国家")
    @NotNull(message = "国家不能为空！")
    private String countries;

    /**
     * 客户类型(1、正式 2、试用 3、到期)
     */
    @ApiModelProperty("客户类型(1、正式 2、试用 3、到期)")
    // @NotNull(message = "客户类型不能为空！")
    private Integer customerType;

    /**
     * 省份ID
     */
    @ApiModelProperty("省份ID")
    @NotNull(message = "省份不能为空！")
    private Integer provinceId;

    /**
     * 城市ID
     */
    @ApiModelProperty("城市ID")
    @NotNull(message = "城市不能为空！")
    private Integer cityId;

    /**
     * 到期时间
     */
    @ApiModelProperty("到期时间")
    private Date expireDate;

    /**
     * 客户名称(学校名称)
     */
    @ApiModelProperty("客户名称(学校名称)")
    @NotNull(message = "客户名称不能为空！")
    private String customerName;

    @ApiModelProperty("学校ID")
    @NotNull(message = "学校id不能为空！")
    private Integer schoolId;

    /**
     * 行业
     */
    @ApiModelProperty("行业(忽略)")
    private String industry;

    @ApiModelProperty("行业")
    @NotNull(message = "行业不能为空！")
    private Integer industryId;

    @ApiModelProperty("行业类型")
    @NotNull(message = "行业类型不能为空！")
    private Integer industryClassId;

    /**
     * 联系人姓名
     */
    @ApiModelProperty("联系人姓名")
    @NotBlank(message = "联系人姓名不能为空！")
    private String name;

    /**
     * 手机号
     */
    @Pattern(regexp = "^$|^1(3\\d|4[5-8]|5[0-35-9]|6[567]|7[01345-8]|8\\d|9[025-9])\\d{8}$",message = "手机号格式不正确！")
    @ApiModelProperty("手机号")
    private String phone;

    /**
     * 账号
     */
    @ApiModelProperty("账号")
    @NotNull(message = "账号不能为空！")
    private String account;

    /**
     * 密码
     */
    @ApiModelProperty("密码")
    private String password;

    /**
     * 职务
     */
    @ApiModelProperty("职务")
    private String position;

    /**
     * 邮箱
     */

    @Pattern(regexp = "^$|^([a-zA-Z]|[0-9])(\\w|\\-)+@[a-zA-Z0-9]+\\.([a-zA-Z]{2,4})$",message = "邮箱格式不正确！")
    @ApiModelProperty("邮箱")
    private String email;

    /**
     * 是否删除(0、未删除 1、已删除)
     */
    @ApiModelProperty("是否删除(0、未删除 1、已删除)")
    private Integer isDel;

    /**
     * 创建时间
     */
    @ApiModelProperty("创建时间")
    @TableField(fill = FieldFill.INSERT)
    private Date createTime;

    /**
     * 修改时间
     */
    @ApiModelProperty("修改时间")
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private Date updateTime;

}
