package com.huoran.nakadai.entity.res;

import com.baomidou.mybatisplus.annotation.*;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

/**
 * <p>
 * 订单其他信息（包含课程权限、数据平台权限、合同信息等）
 * </p>
 *
 * @author cheney
 * @since 2021-07-01
 */
@Data

@TableName("order_other")
@ApiModel(value = "OrderOther对象", description = "订单其他信息（包含课程权限、数据平台权限、合同信息等）")
public class OrderOtherResp implements Serializable {

    private static final long serialVersionUID = 6473128434681123441L;

    @ApiModelProperty(value = "主键")
    @TableId(value = "id", type = IdType.AUTO)
    private Integer id;

    @ApiModelProperty(value = "订单id")
    private Integer orderId;

    @ApiModelProperty(value = "数据产品或课程id（根据权限authority字段判断）")
    private Integer dataOrCourseId;

    @ApiModelProperty(value = "数据产品或课程名称")
    private String productName;

    @ApiModelProperty(value = "端口id（预留处理）")
    private Integer portId;

    @ApiModelProperty(value = "使用期限（以月为单位）")
    @NotEmpty(message = "期限不能为空")
    private Integer periodOfUse;

    @ApiModelProperty(value = "剩余期限（以天为单位）")
    private Integer remainingPeriod;

    @ApiModelProperty(value = "成交价（以万位单位）")
    @NotEmpty(message = "成交价不能为空")
    private BigDecimal finalPrice;

    @ApiModelProperty(value = "总金额（以万为单位）")
    @NotEmpty(message = "总金额不能为空")
    private BigDecimal totalAmount;

    @ApiModelProperty(value = "市场价（以万为单位）")
    private BigDecimal marketValue;

    @ApiModelProperty(value = "折扣率（百分比）")
    private String discountRate;

    @ApiModelProperty(value = "是否发货（0未发货，1已发货，默认不发货）")
    @Size(max = 1)
    private Integer ship;

    @ApiModelProperty(value = "区分权限（0为数据平台权限，1为课程权限，对应data_or_course_id）")
    @Size(max = 1)
    private Integer authority;

    @ApiModelProperty(value = "是否开启（0不开启，1开启，发货后，默认开启）")
    @Size(max = 1)
    private Integer isEnable;

    @ApiModelProperty(value = "账号数")
    @NotEmpty(message = "账号数不能为空")
    private Integer accountNum;

    @ApiModelProperty(value = "是否删除(0、未删除 1、已删除)")
    @TableLogic(value = "0", delval = "1")
    private Integer isDel;

    @ApiModelProperty(value = "使用开始时间")
    @TableField(fill = FieldFill.INSERT)
    @JsonFormat(locale = "zh", timezone = "GMT+8", pattern = "yyyy-MM-dd")
    private Date startTime;

    @ApiModelProperty(value = "使用结束时间")
    @JsonFormat(locale = "zh", timezone = "GMT+8", pattern = "yyyy-MM-dd")
    private Date endTime;

    @ApiModelProperty(value = "更新时间")
    @TableField(fill = FieldFill.INSERT_UPDATE)
    @JsonFormat(locale = "zh", timezone = "GMT+8", pattern = "yyyy-MM-dd")
    private Date updateTime;

    @ApiModelProperty(value = "创建时间")
    @TableField(fill = FieldFill.INSERT)
    @JsonFormat(locale = "zh", timezone = "GMT+8", pattern = "yyyy-MM-dd")
    private Date createTime;

    @ApiModelProperty(value = "剩余天数")
    private Integer remainingDay;

}
