package com.huoran.nakadai.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huoran.common.response.R;
import com.huoran.nakadai.entity.Comment;
import com.huoran.nakadai.entity.CommentLike;
import com.huoran.nakadai.entity.Notify;
import com.huoran.nakadai.entity.res.CommentRes;
import com.huoran.nakadai.mapper.CommentLikeMapper;
import com.huoran.nakadai.mapper.CommentMapper;
import com.huoran.nakadai.mapper.NotifyMapper;
import com.huoran.nakadai.service.CommentService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 课程评论回复表 服务实现类
 * </p>
 *
 * @author chen
 * @since 2023-05-25
 */
@Service
public class CommentServiceImpl extends ServiceImpl<CommentMapper, Comment> implements CommentService {

    @Autowired
    CommentLikeMapper commentLikeMapper;

    @Autowired
    NotifyMapper notifyMapper;

    @Override
    public R commentTreeList(Integer mallId, Integer schoolId,String accountId) {
        //查询所有顶级评论
        List<CommentRes> rootComments = baseMapper.findByParentId(0,mallId,schoolId,accountId);
        for (CommentRes rootComment : rootComments) {
            rootComment.setCurrentAccountId(accountId);
            rootComment.setMallId(mallId);
            rootComment.setSchoolId(schoolId);
            rootComment.setLevel(1);
            if (rootComment.getCreateRoleName() !=null && rootComment.getCreateRoleName().contains("老师")){
                rootComment.setTeacherIDOfCreateAccount(1);
            }
            if (rootComment.getCreateUsername()==null){
                rootComment.setCreateUsername("超级管理员");
            }
            if (rootComment.getReplyUsername()==null){
                rootComment.setReplyUsername("超级管理员");
            }
            setChildren(rootComment);
        }
        //查询当前账号是否为老师管理员账号
        String roleName = baseMapper.selectTeacherAdministrator(accountId);
        boolean deleteIdentity = false;
        if (roleName!=null){
            if (roleName.contains("老师")||roleName.contains("管理员")){
                deleteIdentity = true;
            }
        }
        return R.ok().put("rootComments",rootComments).put("deleteIdentity",deleteIdentity);
    }

    /**
     * 递归获取
     * @param commentRes 参数
     */
    public void setChildren(CommentRes commentRes){
        List<CommentRes> children = baseMapper.findByParentId(commentRes.getCommentId(),commentRes.getMallId(),commentRes.getSchoolId(),commentRes.getCurrentAccountId());
        if (!children.isEmpty()) {
            commentRes.setChildren(children);
            for (CommentRes child : children) {
                child.setLevel(commentRes.getLevel()+1);
                if (child.getCreateRoleName() !=null && child.getCreateRoleName().contains("老师")){
                    child.setTeacherIDOfCreateAccount(1);
                }
                if (child.getReplyRoleName()!=null && child.getReplyRoleName().contains("老师")){
                    child.setTeacherIDOfReplyAccount(1);
                }
                if (child.getCreateUsername()==null){
                    child.setCreateUsername("超级管理员");
                }
                if (child.getReplyUsername()==null){
                    child.setReplyUsername("超级管理员");
                }
                child.setCurrentAccountId(commentRes.getCurrentAccountId());
                setChildren(child);
            }
        }
    }

    /**
     * 递归删除
     * @param commentId 参数
     */
    @Override
    public R removeChildById(Integer commentId) {
        List<Integer> idList = new ArrayList<>();
        this.selectChildListById(commentId, idList);
        idList.add(commentId);

        for (Integer id : idList) {
            //删除点赞
            commentLikeMapper.delete(new QueryWrapper<CommentLike>().eq("comment_id", id));
            //删除通知
            notifyMapper.delete(new QueryWrapper<Notify>().eq("comment_id", id));
        }
        int i = baseMapper.deleteBatchIds(idList);
        return i>0 ? R.ok() : R.error();
    }

    private void selectChildListById(Integer id, List<Integer> idList) {
        List<Comment> childList = baseMapper.selectList(new QueryWrapper<Comment>().eq("pid", id).select("comment_id"));
        childList.forEach(item -> {
            idList.add(item.getCommentId());
            this.selectChildListById(item.getCommentId(), idList);
        });
    }
}
