package com.huoran.nakadai.service.impl;

import com.aliyuncs.exceptions.ClientException;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huoran.common.utils.AliyunOssUtil;
import com.huoran.common.utils.VideoUtil;
import com.huoran.nakadai.entity.CurriculumChapter;
import com.huoran.nakadai.entity.CurriculumSubsection;
import com.huoran.nakadai.entity.vo.ChapterVO;
import com.huoran.nakadai.entity.vo.SortVO;
import com.huoran.nakadai.entity.vo.SubsectionVO;
import com.huoran.nakadai.mapper.CurriculumChapterMapper;
import com.huoran.nakadai.mapper.CurriculumMapper;
import com.huoran.nakadai.mapper.CurriculumSubsectionMapper;
import com.huoran.nakadai.service.CurriculumChapterService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 企业课程章节表 服务实现类
 * </p>
 *
 * @author lr
 * @since 2021-09-26
 */
@Service
public class CurriculumChapterServiceImpl extends ServiceImpl<CurriculumChapterMapper, CurriculumChapter> implements CurriculumChapterService {
    @Autowired
    private CurriculumSubsectionMapper curriculumSubsectionMapper;

    @Override
    public List<ChapterVO> getChapterTree(String courseId) {
        // 根据课程ID查询所有章节
        QueryWrapper<CurriculumChapter> chapterWrapper = new QueryWrapper<>();
        chapterWrapper.eq("cid", courseId);
        //降序查询
        chapterWrapper.orderByAsc("sort");
        List<CurriculumChapter> chapters = baseMapper.selectList(chapterWrapper);

        // 根据课程ID查询所有小节
        QueryWrapper<CurriculumSubsection> subsectionWrapper = new QueryWrapper<>();
        subsectionWrapper.eq("cid", courseId);
        //降序查询
        subsectionWrapper.orderByAsc("sort");
        List<CurriculumSubsection> subsections = curriculumSubsectionMapper.selectList(subsectionWrapper);

        List<ChapterVO> chapterVOList = new ArrayList<>();
        // 遍历封装
        for (CurriculumChapter c : chapters) {
            ChapterVO chapterVO = new ChapterVO();
            //属性复制
            BeanUtils.copyProperties(c, chapterVO);
            chapterVOList.add(chapterVO);

            List<SubsectionVO> subsectionVOList = new ArrayList<>();
            for (CurriculumSubsection s : subsections) {
                //判断小节是否属于某一章节
                if (c.getId().equals(s.getChapterId())) {
                    SubsectionVO subsectionVO = new SubsectionVO();
                    BeanUtils.copyProperties(s, subsectionVO);
                    subsectionVOList.add(subsectionVO);
                }
            }
            //设置小节
            chapterVO.setSubsectionList(subsectionVOList);
        }
        return chapterVOList;
    }

    @Override
    public boolean removeChapter(Integer chapterId) {
        QueryWrapper<CurriculumSubsection> wrapper = new QueryWrapper<>();
        wrapper.eq("chapter_id", chapterId);
        List<CurriculumSubsection> courseSubsections = curriculumSubsectionMapper.selectList(wrapper);

        if (courseSubsections != null && courseSubsections.size() != 0) {
            //循环删除小节
            courseSubsections.forEach(subsection -> {
                //删除资源
                boolean isVideo = VideoUtil.isVideo(subsection.getFileType());
                if (isVideo) {
                    try {
                        AliyunOssUtil.removeVideo(subsection.getFileId());
                    } catch (ClientException e) {
                        e.printStackTrace();
                    }
                } else {
                    List<String> keys = new ArrayList<>();
                    keys.add(subsection.getFileName());
                    try {
                        AliyunOssUtil.removeMoreVideo(keys);
                    } catch (ClientException e) {
                        e.printStackTrace();
                    }
                }
                curriculumSubsectionMapper.deleteById(subsection.getId());
            });
        }
        int count = baseMapper.deleteById(chapterId);
        return count > 0;
    }

    @Override
    public boolean saveChapter(CurriculumChapter courseChapter) {
        //查询课程章节的总数
        QueryWrapper<CurriculumChapter> wrapper = new QueryWrapper<>();
        wrapper.eq("cid", courseChapter.getCid());
//        Long count = baseMapper.selectCount(wrapper);
//        if (count == 0) {
//            courseChapter.setSort(1);
//        } else {
//            //查询课程章节最大排序数
//            count = Long.valueOf(baseMapper.selectMaxSort());
//            courseChapter.setSort((int) (count + 1));
//        }
        int insert = baseMapper.insert(courseChapter);
        return insert > 0;
    }

    @Override
    public boolean updateSort(SortVO sortVO) {
        List<ChapterVO> chapterVOList = sortVO.getChapterVOList();
        if (chapterVOList != null && chapterVOList.size() != 0) {
            chapterVOList.forEach(chapterVO -> {
                List<SubsectionVO> subsectionList = chapterVO.getSubsectionList();
                if (subsectionList != null && subsectionList.size() != 0) {
                    subsectionList.forEach(subsectionVO -> {
                        CurriculumSubsection courseSubsection = new CurriculumSubsection();
                        BeanUtils.copyProperties(subsectionVO,courseSubsection);
                        curriculumSubsectionMapper.updateById(courseSubsection);
                    });
                }
                CurriculumChapter courseChapter = new CurriculumChapter();
                BeanUtils.copyProperties(chapterVO,courseChapter);
                baseMapper.updateById(courseChapter);
            });
            return true;
        }
        return false;
    }

    @Override
    public boolean repeat(CurriculumChapter courseChapter) {
        QueryWrapper<CurriculumChapter> wrapper = new QueryWrapper<>();
        wrapper.eq("cid", courseChapter.getCid());
        wrapper.eq("name", courseChapter.getName());
        Integer count = baseMapper.selectCount(wrapper);
        return count > 0;
    }
}
