package com.huoran.nakadai.service.impl;

import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huoran.api.OccupationlabClient;
import com.huoran.common.constant.DelConstant;
import com.huoran.common.exception.CustomException;
import com.huoran.common.exception.ExceptionEnum;
import com.huoran.common.response.R;
import com.huoran.nakadai.entity.*;
import com.huoran.nakadai.entity.req.AddCurriculumReq;
import com.huoran.nakadai.entity.req.ModifyCurriculumReq;
import com.huoran.nakadai.entity.req.PageCurriculumReq;
import com.huoran.nakadai.entity.res.CurriculumDetailRes;
import com.huoran.nakadai.entity.res.CurriculumRes;
import com.huoran.nakadai.entity.res.SchoolCourseRes;
import com.huoran.nakadai.mapper.*;
import com.huoran.nakadai.service.ICurriculumService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

/**
 * <p>
 * 课程管理 服务实现类
 * </p>
 *
 * @author lr
 * @since 2021-09-16
 */
@Service
public class CurriculumServiceImpl extends ServiceImpl<CurriculumMapper, Curriculum> implements ICurriculumService {

    @Autowired
    private OccupationlabClient occupationlabClient;
    @Autowired
    private CurriculumConfigureMapper configureMapper;
    @Autowired
    private SchoolCurriculumMapper schoolCurriculumMapper;
    @Autowired
    private OrderMapper orderMapper;
    @Autowired
    private CurriculumChapterMapper chapterMapper;

    @Autowired
    private CurriculumMapper curriculumMapper;

    @Autowired
    private MallMapper mallMapper;

    /**
     * @Description: 创建课程
     * @auther: Rong
     * @date: 2021/9/17 11:49
     */
    @Override
    public R createCurriculum(AddCurriculumReq req) {
        Curriculum curriculum = new Curriculum();
        BeanUtils.copyProperties(req, curriculum);
        int ret = baseMapper.insert(curriculum);
        //练习系统id   因为选择应用配置时可以选择多个系统的练习项目
        if (req.getSystemIdByPractice().size() > 0) {
            req.getSystemIdByPractice().stream().forEach(practice -> {
                configureMapper.insert(new CurriculumConfigure()
                        .setSort(practice.getSort())
                        .setProjectId(practice.getProjectId())
                        .setSystemId(practice.getSystemId())
                        .setCid(curriculum.getCid())
                        .setPermissions(0)
                        .setIsShow(practice.getIsShow())
                );
            });
        }
        //考核系统id   因为选择应用配置时可以选择多个系统下的考核项目
        if (req.getSystemIdByAssessment().size() > 0) {
            req.getSystemIdByAssessment().stream().forEach(assessment -> {
                configureMapper.insert(new CurriculumConfigure()
                        .setSort(assessment.getSort())
                        .setProjectId(assessment.getProjectId())
                        .setSystemId(assessment.getSystemId())
                        .setCid(curriculum.getCid())
                        .setPermissions(1)
                        .setIsShow(assessment.getIsShow())
                );
            });
        }//竞赛系统id   因为选择应用配置时可以选择多个系统下的考核项目
        if (req.getSystemIdByCompetition().size() > 0) {
            req.getSystemIdByCompetition().stream().forEach(assessment -> {
                configureMapper.insert(new CurriculumConfigure()
                        .setSort(assessment.getSort())
                        .setProjectId(assessment.getProjectId())
                        .setSystemId(assessment.getSystemId())
                        .setCid(curriculum.getCid())
                        .setPermissions(2)
                        .setIsShow(assessment.getIsShow())
                );
            });
        }
        //默认添加第一章
        CurriculumChapter chapter = new CurriculumChapter();
        chapter.setCid(curriculum.getCid());
        chapter.setSort(1);
        chapter.setName("第一章");
        chapterMapper.insert(chapter);

        Integer cid = curriculum.getCid();
        Integer schoolId = curriculum.getSchoolId();

        //学校id不为空，学校端添加课程对应学校关系
        if (req.getSchoolId() != null) {
            SchoolCurriculum schoolCurriculum = new SchoolCurriculum();
            schoolCurriculum.setCid(cid);
            schoolCurriculum.setSchoolId(schoolId);
            schoolCurriculumMapper.insert(schoolCurriculum);
        }

        return ret > 0 ? R.ok().put("cid", cid) : R.error();
    }

    @Override
    public R modifyCourse(ModifyCurriculumReq req) {
        Curriculum curriculum = new Curriculum();
        BeanUtils.copyProperties(req, curriculum);

        //当考核练习系统id不为空时，对已有的记录进行逻辑删除操作 重新新增
        //考核系统id
        if (req.getSystemIdByAssessment() != null) {
            //根据课程id删除课程配置表的记录
            UpdateWrapper<CurriculumConfigure> updateWrapper = new UpdateWrapper<>();
            updateWrapper.set("is_del", DelConstant.IS_DEL);
            updateWrapper.eq("permissions", 1);
            updateWrapper.eq("cid", req.getCid());
            configureMapper.update(new CurriculumConfigure(), updateWrapper);

            //考核系统id   因为选择应用配置时可以选择多个系统下的考核项目
            if (req.getSystemIdByAssessment().size() > 0) {
                req.getSystemIdByAssessment().stream().forEach(assessment -> {
                    configureMapper.insert(new CurriculumConfigure()
                            .setSort(assessment.getSort())
                            .setProjectId(assessment.getProjectId())
                            .setSystemId(assessment.getSystemId())
                            .setCid(curriculum.getCid())
                            .setPermissions(1)
                            .setIsShow(assessment.getIsShow())
                    );
                });
            }
        }

        //练习系统id
        if (req.getSystemIdByPractice() != null) {

            //根据课程id删除课程配置表的练习记录
            UpdateWrapper<CurriculumConfigure> updateWrapper = new UpdateWrapper<>();
            updateWrapper.set("is_del", DelConstant.IS_DEL);
            updateWrapper.eq("cid", req.getCid());
            updateWrapper.eq("permissions", 0);
            configureMapper.update(new CurriculumConfigure(), updateWrapper);
            //练习系统id   因为选择应用配置时可以选择多个系统的练习项目
            if (req.getSystemIdByPractice().size() > 0) {
                req.getSystemIdByPractice().stream().forEach(practice -> {
                    configureMapper.insert(new CurriculumConfigure()
                            .setSort(practice.getSort())
                            .setProjectId(practice.getProjectId())
                            .setSystemId(practice.getSystemId())
                            .setCid(curriculum.getCid())
                            .setPermissions(0)
                            .setIsShow(practice.getIsShow())
                    );
                });
            }
        }

        //竞赛系统id
        if (req.getSystemIdByCompetition() != null) {

            //根据课程id删除课程配置表的练习记录
            UpdateWrapper<CurriculumConfigure> updateWrapper = new UpdateWrapper<>();
            updateWrapper.set("is_del", DelConstant.IS_DEL);
            updateWrapper.eq("cid", req.getCid());
            updateWrapper.eq("permissions", 2);
            configureMapper.update(new CurriculumConfigure(), updateWrapper);
            //练习系统id   因为选择应用配置时可以选择多个系统的练习项目
            if (req.getSystemIdByCompetition().size() > 0) {
                req.getSystemIdByCompetition().stream().forEach(practice -> {
                    configureMapper.insert(new CurriculumConfigure()
                            .setSort(practice.getSort())
                            .setProjectId(practice.getProjectId())
                            .setSystemId(practice.getSystemId())
                            .setCid(curriculum.getCid())
                            .setPermissions(2)
                            .setIsShow(practice.getIsShow())
                    );
                });
            }
        }

        int ret = baseMapper.updateById(curriculum);

        return ret > 0 ? R.ok() : R.error();

    }

    @Override
    public R delCourse(List<Integer> cids) {

        //先删除课程配置信息，再删除课程信息
        if (cids.size() > 0) {
            cids.forEach(cid -> {
                //查询是否有关联上架商品，有提示无法删
                int count = baseMapper.selectAssociatedListedGoods(cid);
                if (count > 0){
                    throw new CustomException(ExceptionEnum.ASSOCIATED_PRODUCT);
                }
                UpdateWrapper<CurriculumConfigure> updateWrapper = new UpdateWrapper<>();
                updateWrapper.set("is_del", DelConstant.IS_DEL);
                updateWrapper.eq("cid", cid);
                configureMapper.update(new CurriculumConfigure(), updateWrapper);
            });
        }

        int deleteBatchIds = baseMapper.deleteBatchIds(cids);

        return deleteBatchIds > 0 ? R.ok() : R.error();
    }

    @Override
    public R deleteCoursePrompt(List<Integer> cids) {
        //先删除课程配置信息，再删除课程信息
        StringBuilder stringBuilder = new StringBuilder();
        int i = 0;
        for (Integer cid : cids) {
            //查询是否有关联上架商品，有提示无法删
            int count = baseMapper.selectAssociatedListedGoods(cid);
            if (count > 0){
                Curriculum curriculum = baseMapper.selectById(cid);
                stringBuilder.append(curriculum.getCurriculumName()+"、");
                i++;
            }
        }
        if (i>0){
            return R.error(stringBuilder.substring(0,stringBuilder.toString().length()-1)+"已关联产品，请在商城管理下架对应产品后再删除!");
        }
        return R.ok();
    }

    @Override
    public R curriculumList(PageCurriculumReq req) {
        if (req.getSupplierId().length()==1 && "1".equals(req.getSupplierId())){
            return R.ok().put("page",null);
        }
        Page<CurriculumRes> page = new Page<>(req.getPageNum(), req.getPageSize());
        //查询包含供应商配置的课程
        List<Integer> cIds = baseMapper.selectCurriculumBySupplier(req.getSupplierId());
        if (!cIds.isEmpty()){
            req.setCIds(cIds);
        }
        //查询已经关联产品的课程id
        List<Integer> ids = baseMapper.selectProductCurriculum();
        IPage<CurriculumRes> curriculumResIPage = baseMapper.curriculumList(page, req);
        for (CurriculumRes curriculumRes : curriculumResIPage.getRecords()) {
            if (ids.contains(curriculumRes.getCid())){
                curriculumRes.setAssociatedProductIdentification(true);
            }
        }
        return R.ok().put("page", curriculumResIPage);
    }

    @Override
    public R schoolCourse(Integer schoolId) {

//        List<SchoolCourseRes> schoolCourse = orderMapper.schoolCourse(schoolId);
        /*for (SchoolCourseRes res : schoolCourse) {
            res.setSystemId(configureMapper.getConfigureSystemId(res.getCid()));
        }*/
        return R.ok().put("data", null);
    }

    @Override
    public R getSchoolEffectiveCourse(Integer schoolId) {
//        List<SchoolCourseRes> schoolCourse = orderMapper.getSchoolEffectiveCourse(schoolId);
        return R.ok().put("data", null);
    }

    @Override
    public R schoolCourseByAchievement(Integer schoolId) {

//        List<SchoolCourseRes> schoolCourse = orderMapper.schoolCourseByAchievement(schoolId);
//        for (SchoolCourseRes res : schoolCourse) {
//            res.setSystemId(configureMapper.getConfigureSystemId(res.getCid()));
//        }
        return R.ok().put("data", null);
    }

    /**
     * @Description: 课程详情
     * @auther: Rong
     * @date: 2021/9/29 10:45
     */
    @Override
    public R curriculumDetail(Integer cid,Integer mallId) {
        Curriculum curriculum = baseMapper.selectCurriculum(cid);
        //读取商品详情介绍
        if (mallId!=null){
            Mall mall = mallMapper.selectById(mallId);
            curriculum.setBriefIntroduction(mall.getDetailedIntroduction());
        }

        String systemIds = configureMapper.getConfigureSystemId(cid);

        curriculum.setSystemIds(systemIds);
        List<CurriculumDetailRes> practice = configureMapper.getCurriculumConfigure(cid, 0);
        List<CurriculumDetailRes> assessment = configureMapper.getCurriculumConfigure(cid, 1);
//        竞赛配置
        List<CurriculumDetailRes> competition = configureMapper.getCurriculumConfigure(cid, 2);

        curriculum.setPracticeConfig(practice);
        curriculum.setAssessmentConfig(assessment);
        curriculum.setCompetitionConfig(competition);
        return R.ok().put("data", curriculum);
    }

    @Override
    public List<Integer> getSystemIdByCustomerId(Integer customerId) {
        Set<Integer> systemIds = curriculumMapper.getSystemIdByCustomerId(customerId);
        return new ArrayList<>(systemIds);
    }

    @Override
    public String getSchoolCourse(Integer schoolId) {
        String str = "";
//        List<SchoolCourseRes> list = orderMapper.schoolCourse(schoolId);
//        for (SchoolCourseRes s : list) {
//            str += s.getCid() + ",";
//        }
        return str.substring(0, str.length() - 1);
    }

    @Override
    public R whetherToRenewTheFee(Integer schoolId, Integer mallId) {
//        SchoolCourseRes res = orderMapper.whetherToRenewTheFee(schoolId,mallId);
//        //返回是否续费 0表示没有该月订单的课程(显示续约按钮) 1表示正常显示课程
//        if (res==null){
//            return R.ok().put("isRenew",0);
//        }
        return R.ok().put("isRenew",1);
    }
}
