package com.huoran.nakadai.service.impl;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import cn.afterturn.easypoi.excel.entity.enmus.ExcelType;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huoran.api.OccupationlabClient;
import com.huoran.api.UserClient;
import com.huoran.common.entity.UserAccount;
import com.huoran.common.exception.CustomException;
import com.huoran.common.exception.ExceptionEnum;
import com.huoran.common.response.R;
import com.huoran.common.utils.MD5;
import com.huoran.common.utils.PageUtils;
import com.huoran.nakadai.entity.*;
import com.huoran.nakadai.entity.req.CustomerListReq;
import com.huoran.nakadai.entity.req.CustomerReq;
import com.huoran.nakadai.entity.req.ListOfClientCommercialManagersReq;
import com.huoran.nakadai.entity.req.SalesProgressReq;
import com.huoran.nakadai.entity.res.*;
import com.huoran.nakadai.mapper.*;
import com.huoran.nakadai.service.ICustomerService;
import com.huoran.nakadai.utils.ExcelStyleUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.ibatis.annotations.Param;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.util.HashSet;
import java.util.List;
import java.util.UUID;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ThreadPoolExecutor;

/**
 * <p>
 * 客户表 服务实现类
 * </p>
 *
 * @author huoran
 * @since 2021-06-28
 */
@Service
public class CustomerServiceImpl extends ServiceImpl<CustomerMapper, Customer> implements ICustomerService {

    @Autowired
    private HrContractInformationMapper hrContractInformationMapper;

    @Autowired
    private HrCoursePermissionsMapper hrCoursePermissionsMapper;

    @Autowired
    private HrDataPermissionsMapper hrDataPermissionsMapper;

    @Autowired
    private OrderOtherMapper orderOtherMapper;

    @Autowired
    private OrderMapper orderMapper;

    @Autowired
    private UserClient userClient;

    @Resource
    private RedisTemplate stringRedisTemplate;

    @Autowired
    private OccupationlabClient occupationlabClient;

    @Autowired
    private SchoolMapper schoolMapper;

    @Autowired
    private HrStaffMapper staffMapper;

    @Autowired
    private CurriculumMapper curriculumMapper;

    @Autowired
    ThreadPoolExecutor executor;
    @Autowired
    private PartnerTeamMapper teamMapper;

    @Override
    public int addCustomer(CustomerReq customerReq) throws ExecutionException, InterruptedException {
        //账号去重
        int count = baseMapper.queryAccountExists(customerReq.getAccount());
        if (count > 0) {
            throw new CustomException(ExceptionEnum.EXIST_ACCOUNT);
        }

        //添加客户、账号基本信息
        CompletableFuture<Integer> customerAccountFuture = CompletableFuture.supplyAsync(() -> {
            //添加客户默认用户信息
            UserInfo userInfo = new UserInfo();
            // userInfo.setUserName();用户姓名默认为空
            /*if (customerReq.getPhone()==null){
                userInfo.setPhone("18888888888");
            }else {
                userInfo.setPhone(customerReq.getPhone());
            }*/
            userInfo.setUniqueIdentification(UUID.randomUUID().toString());
            userInfo.setUserName("学校超管");//设置客户默认名称
            staffMapper.addUserInfo(userInfo);

            Customer customer = new Customer();
            BeanUtils.copyProperties(customerReq, customer);
            baseMapper.insert(customer);

            //将账号密码以及学校id添加到账号表
            UserAccount account = new UserAccount();
            account.setAccount(customerReq.getAccount());
            account.setPassword(MD5.encrypt("1122aa"));
            account.setSchoolId(customerReq.getSchoolId());
            account.setUserId(userInfo.getUserId());
            account.setCustomerId(customer.getCustomerId());
            account.setPlatformId("1,2");
            customerReq.setCustomerId(customer.getCustomerId());
            customerReq.setSchoolId(customer.getSchoolId());
            baseMapper.insertUserAccount(account);

            return account.getId();
        }, executor);

        //为客户添加各模块之间数据
        CompletableFuture<Void> questionFuture = customerAccountFuture.thenAcceptAsync(res -> {
            //添加测评管理数据
            occupationlabClient.copyQuestion(customerReq.getSchoolId(), res);
        }, executor);

        CompletableFuture<Void> studentFuture = customerAccountFuture.thenAcceptAsync(res -> {
            //添加学生管理数据
            occupationlabClient.saveData(customerReq.getSchoolId());
        }, executor);

        CompletableFuture<Void> courseFuture = customerAccountFuture.thenAcceptAsync(res -> {
            //添加理论课程管理数据
            occupationlabClient.addCourseData(customerReq.getSchoolId(), res);
        }, executor);

        CompletableFuture<Void> articleFuture = customerAccountFuture.thenAcceptAsync(res -> {
            //添加咨询管理数据
            occupationlabClient.addArticleData(res, customerReq.getSchoolId());
        }, executor);

        CompletableFuture<Void> sampleFuture = customerAccountFuture.thenAcceptAsync(res -> {
            //添加赛事管理数据
            occupationlabClient.eventSampleData(customerReq.getSchoolId(), res);
        }, executor);

        CompletableFuture<Void> evaluationFuture = customerAccountFuture.thenAcceptAsync(res -> {
            //增加测评规则内置数据
            occupationlabClient.copyEvaluationRules(customerReq.getSchoolId());
        }, executor);

        CompletableFuture<Void> authorityFuture = customerAccountFuture.thenAcceptAsync(res -> {
            //添加职站及数据平台权限
            userClient.addCustomerAdmin(customerReq.getSchoolId(), res.toString());
        }, executor);

        CompletableFuture<Void> staffFuture = customerAccountFuture.thenAcceptAsync(res -> {
            //添加员工组织架构内置数据
            occupationlabClient.saveStaffData(customerReq.getSchoolId());
        }, executor);

        CompletableFuture<Void> builtInCourseClassification = customerAccountFuture.thenAcceptAsync(res -> {
            //添加理论课程分类内置数据
            occupationlabClient.builtInCourseClassification(customerReq.getSchoolId(), res);
        }, executor);

        //等待所有任务都完成
        CompletableFuture.allOf(questionFuture, studentFuture, courseFuture, articleFuture, sampleFuture, evaluationFuture, authorityFuture, staffFuture, builtInCourseClassification).get();

        return customerAccountFuture.get();
    }

    @Override
    public PageUtils queryPage(Integer page, Integer size, CustomerListReq customerListReq) {
        //分页参数
        if (page <= 0) {
            page = 1;
        }
        if (size <= 0) {
            size = 10;
        }
        if (!StringUtils.isEmpty(customerListReq.getSupplierId())) {
            //查询包含供应商配置的课程
            List<Integer> cIds = curriculumMapper.selectCurriculumBySupplier(customerListReq.getSupplierId());
            //根据数据权限查询客户
            //查询课程的数据权限订单id
            HashSet<Integer> orderIds = new HashSet<>();
//            if (!cIds.isEmpty()) {
//                orderIds = orderMapper.dataPermissionOrderId(cIds);
//            }
//            //查询数据产品的数据权限订单id
//            HashSet<Integer> dataProductOrderIds = orderMapper.dataPermissionOrderIdByDataProduct(customerListReq.getSupplierId());
//            orderIds.addAll(dataProductOrderIds);
//            //查询增值权限订单id
//            HashSet<Integer> valueOrderIds = orderMapper.valuePermissionOrderIds();
//            orderIds.addAll(valueOrderIds);
//            List<Integer> customerIds = baseMapper.selectDataPermissionCustomer(orderIds);
            customerListReq.setCustomerIds(null);
        }
        IPage<CustomerListRes> positionVO = baseMapper.pageByCondition(new Page(page, size), customerListReq);
        return new PageUtils(positionVO);
    }

    @Override
    public int resetPwd(Integer customerId) {
        //根据客户id修改所有的账号密码
        UserAccount hrUserAccount = new UserAccount();
        hrUserAccount.setCustomerId(customerId);
        hrUserAccount.setPassword(MD5.encrypt("1122aa"));
        return baseMapper.updatePwdByCustomerId(hrUserAccount);
    }

    @Override
    public int delCustomer(Integer customerId) {

        //根据客户id查询学校id,根据学校id删除 数据操作记录表
        Customer customer1 = this.baseMapper.selectOne(new QueryWrapper<Customer>().eq("customer_id", customerId));
        boolean a = this.baseMapper.delDataRecords(customer1.getSchoolId());

        boolean c = this.baseMapper.updateUserInfo(customer1.getSchoolId());

        boolean d = this.baseMapper.updateAccount(customer1.getSchoolId());

        Customer customer = new Customer();
        customer.setIsDel(1);
        //删除所有客户对应的账号
        boolean b = this.baseMapper.updateDelById(customerId);
        return baseMapper.update(customer, new QueryWrapper<Customer>().eq("customer_id", customerId));
    }

    @Override
    public CustomerDetailRes queryCustomerDetails(Integer customerId) {
        //根据customerId获取customer
        CustomerRes customer = this.baseMapper.selectCustomer(customerId);

        //根据客户id查询绑定的合同信息
        List<ContractInformationsRes> contractList = hrContractInformationMapper.selectContractInfo(customerId);

        //根据客户id查询绑定的课程权限
        List<CoursePermissionssRes> courseList = orderOtherMapper.selectCoursePermissions(customerId);

        //根据客户id查询绑定的数据平台权限
        List<DataPermissionssRes> dataPlatformList = orderOtherMapper.selectDataPermissions(customerId);

        CustomerDetailRes res = new CustomerDetailRes();
        res.setCustomer(customer);
        res.setContractInformations(contractList);
        res.setCoursePermissionss(courseList);
        res.setDataPermissionss(dataPlatformList);
        return res;
    }

    @Override
    public int updateCourseEnabled(Integer coursePermissionsId, Integer isEnabled) {
        HrCoursePermissions hrCoursePermissions = new HrCoursePermissions();
        hrCoursePermissions.setIsEnabled(isEnabled);
        hrCoursePermissions.setCoursePermissionsId(coursePermissionsId);
        return hrCoursePermissionsMapper.updateById(hrCoursePermissions);
    }

    @Override
    public int updateDataEnabled(Integer dataPermissionsId, Integer isEnabled) {
        HrDataPermissions hrDataPermissions = new HrDataPermissions();
        hrDataPermissions.setIsEnabled(isEnabled);
        hrDataPermissions.setDataPermissionsId(dataPermissionsId);
        return hrDataPermissionsMapper.updateById(hrDataPermissions);
    }

    @Override
    public int updateCustomer(CustomerReq customerReq) {

        Customer customer = new Customer();
        BeanUtils.copyProperties(customerReq, customer);
        //根据客户id修改账号表的账号
        boolean b = this.baseMapper.updateAccountById(customer.getCustomerId(), customerReq.getAccount());

        return this.baseMapper.updateById(customer);
    }

    @Override
    public int updateCustomerByRoleId(@Param("customerId") Integer customerId, @Param("roleId") String roleId) {
        return this.baseMapper.updateCustomerByRoleId(customerId, roleId);
    }

    @Override
    public int updateEnabled(Integer id, Integer isEnable) {
        OrderOther orderOther = new OrderOther();
        orderOther.setIsEnable(isEnable);
        orderOther.setId(id);
        return orderOtherMapper.updateById(orderOther);
    }

    @Override
    public Integer queryCustomerIsExists(Integer schoolId) {

        return this.baseMapper.selectCount(new QueryWrapper<Customer>().eq("is_del", 0).eq("school_id", schoolId));
    }

    @Override
    @CacheEvict(value = "n_school_", key = "'querySchool'")
    public int delCustomers(List<Integer> ids) {
        //根据客户id查询学校id,根据学校id删除 数据操作记录表
        List<Integer> schoolIds = this.baseMapper.selectSchoolIds(ids);
        //删除学校表该学校 然后新增一条一样的数据(为了避免出现删除该校后再新增该校还会有之前该校的数据)
        for (Integer schoolId : schoolIds) {
            //同时再新增一个学校信息
            School school = schoolMapper.selectById(schoolId);
            if (ObjectUtil.isNotNull(school)) {
                schoolMapper.insert(school);
            }
            //删除学校
            schoolMapper.deleteById(schoolId);
        }

        boolean a = this.baseMapper.delDataRecordsBySchoolIds(schoolIds);

        boolean c = this.baseMapper.updateUserInfoBySchoolIds(schoolIds);

        boolean d = this.baseMapper.updateAccountBySchoolIds(schoolIds);

        //根据key删除redis中的缓存
        for (Integer schoolId : schoolIds) {
            System.out.println("清空缓存~~~~");
            stringRedisTemplate.delete("databaseInfo:" + schoolId + "+1000-01-01 01:01:01+9999-01-01 01:01:01");
        }

        //删除所有客户对应的账号
        boolean b = this.baseMapper.delAccountsById(ids);
        return baseMapper.delCustomers(ids);
    }

    @Override
    public List<Customer> checkEmailOrPhone(Customer customer) {
        return baseMapper.checkEmailOrPhone(customer);
    }

    @Override
    public R getCustomerListBasedOnBusinessManagerId(@Param("req") ListOfClientCommercialManagersReq req) {
        Page<AppletsMyClientsForBusinessResp> page = new Page<AppletsMyClientsForBusinessResp>(req.getPageNum(), req.getPageSize());
        IPage<AppletsMyClientsForBusinessResp> pageList = baseMapper.getCustomerListBasedOnBusinessManagerId(page, req);
        return R.ok().put("data", pageList);
    }

    @Override
    public Integer newCustomerOnOrder(SalesProgressReq req) {
        return baseMapper.getNewCustomerOnOrder(req);
    }

    @Override
    public Integer newProductAddedToOrder(SalesProgressReq req) {
        return baseMapper.getNewProductAddedToOrder(req);
    }

    @Override
    public AnnualOperatingAnalysisResp annualOperatingAnalysis(SalesProgressReq req) {
        //查询费率
        QueryWrapper<PartnerTeam> queryWrapper = new QueryWrapper<>();

        queryWrapper.eq("classification_id", req.getTeamId());
//        queryWrapper.eq("partner_id", req.getBusinessManagerId());
        PartnerTeam partnerTeam = teamMapper.selectOne(queryWrapper);


        /**
         * 1、查询当前用户所在团队本人本年内的已完成的订单，已删除的订单不计算在内；
         * 2、总成交金额：全年订单产品的成交价之和
         * 3、总结算金额：全年订单产品的结算价之和；
         * 4、总市场服务费：市场服务费率项目抽成的金额，为产品全年销售金额（即成交金额之和）*所填入的费率；
         * 5、项目收益：总成交金额-总结算金额-总市场服务费。
         */
        AnnualOperatingAnalysisResp resp = baseMapper.annualOperatingAnalysis(req);
        if (resp != null) {
            if (partnerTeam != null) {
                if (partnerTeam.getAnnualMarketingFee() != null) {
                    BigDecimal bigDecimal = resp.getFinalPrice().multiply((partnerTeam.getAnnualMarketingFee().divide(new BigDecimal(100))));
                    //市场服务费 = 即成交金额之和  * 所填入的费率/100；
                    resp.setMarketingServiceCharge(bigDecimal);
                } else {
                    resp.setMarketingServiceCharge(new BigDecimal(0));
                }

                if (resp.getMarketingServiceCharge() != null) {
                    //项目收益：总成交金额-总结算金额-总市场服务费
                    resp.setProjectBenefit(resp.getFinalPrice().subtract(resp.getSettlementPrice()).subtract(resp.getMarketingServiceCharge()));
                }
            }
        }

        /*AnnualOperatingAnalysisToStringResp toStringResp = new AnnualOperatingAnalysisToStringResp();
        if (resp != null) {


            if (resp.getMarketingServiceCharge() != null) {
                toStringResp.setMarketingServiceCharge(RelativeNumberFormatTool.relativeNumberFormat(resp.getMarketingServiceCharge(), "PY"));
            }

            if (resp.getFinalPrice() != null) {
                toStringResp.setFinalPrice(RelativeNumberFormatTool.relativeNumberFormat(resp.getFinalPrice(), "PY"));
            }

            if (resp.getProjectBenefit() != null) {
                toStringResp.setProjectBenefit(RelativeNumberFormatTool.relativeNumberFormat(resp.getProjectBenefit(), "PY"));
            }

            if (resp.getSettlementPrice() != null) {
                toStringResp.setSettlementPrice(RelativeNumberFormatTool.relativeNumberFormat(resp.getSettlementPrice(), "PY"));
            }

        }*/

        return resp;
    }

    @Override
    public void exportClient(CustomerListReq customerListReq, HttpServletResponse response) throws Exception {


        if (!StringUtils.isEmpty(customerListReq.getSupplierId())) {
            //查询包含供应商配置的课程
            List<Integer> cIds = curriculumMapper.selectCurriculumBySupplier(customerListReq.getSupplierId());
            //根据数据权限查询客户
            //查询课程的数据权限订单id
            HashSet<Integer> orderIds = new HashSet<>();
//            if (!cIds.isEmpty()) {
//                orderIds = orderMapper.dataPermissionOrderId(cIds);
//            }
//            //查询数据产品的数据权限订单id
//            HashSet<Integer> dataProductOrderIds = orderMapper.dataPermissionOrderIdByDataProduct(customerListReq.getSupplierId());
//            orderIds.addAll(dataProductOrderIds);
//            //查询增值权限订单id
//            HashSet<Integer> valueOrderIds = orderMapper.valuePermissionOrderIds();
//            orderIds.addAll(valueOrderIds);
//            List<Integer> customerIds = baseMapper.selectDataPermissionCustomer(orderIds);
            customerListReq.setCustomerIds(null);
        }
//        IPage<CustomerListRes> positionVO = baseMapper.pageByCondition(new Page(customerListReq.getPage(), customerListReq.getSize()), customerListReq);


        Page<CustomerListRes> assessPage = new Page<CustomerListRes>(1, 100000);
        customerListReq.setSize(10000);
        customerListReq.setPage(1);
        IPage<CustomerListRes> positionVO = baseMapper.pageByCondition(assessPage, customerListReq);
        // 告诉浏览器用什么软件可以打开此文件
        response.setHeader("content-Type", "application/vnd.ms-excel");

        // 下载文件的默认名称
        response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode("客户列表", "UTF-8") + ".xls");
        //编码
        response.setCharacterEncoding("UTF-8");

        Workbook workbook = ExcelExportUtil.exportExcel(new ExportParams(), CustomerListRes.class, positionVO.getRecords());
        ExportParams exportParams = new ExportParams(null, "客户列表", ExcelType.XSSF);
        exportParams.setStyle(ExcelStyleUtil.class);
        workbook.write(response.getOutputStream());
    }


}
