package com.huoran.nakadai.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huoran.common.constant.DelConstant;
import com.huoran.common.entity.Architecture;
import com.huoran.common.response.R;
import com.huoran.nakadai.entity.HrStaffAccountArchitecture;
import com.huoran.nakadai.entity.req.ArchitectureAddReq;
import com.huoran.nakadai.mapper.HrStaffAccountArchitectureMapper;
import com.huoran.nakadai.service.HrStaffAccountArchitectureService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 中台账号组织架构 服务实现类
 * </p>
 *
 * @author chen
 * @since 2022-03-22
 */
@Service
public class HrStaffAccountArchitectureServiceImpl extends ServiceImpl<HrStaffAccountArchitectureMapper, HrStaffAccountArchitecture> implements HrStaffAccountArchitectureService {

    @Override
    public boolean checkRepeat(HrStaffAccountArchitecture architecture) {
        QueryWrapper<HrStaffAccountArchitecture> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_del", DelConstant.NOT_DEL);
        queryWrapper.eq("level", architecture.getLevel());
        queryWrapper.eq("organization_name", architecture.getOrganizationName());
        queryWrapper.eq("parent_id", architecture.getParentId());
        if (architecture.getId() != null) {
            queryWrapper.last(" and id  != " + architecture.getId());
        }
        List<HrStaffAccountArchitecture> architectureList = baseMapper.selectList(queryWrapper);
        return architectureList.size() > 0;
    }

    @Override
    public List<HrStaffAccountArchitecture> treeList() {
        //查询父级的
        QueryWrapper<HrStaffAccountArchitecture> wrapper = new QueryWrapper<>();
        wrapper.eq("is_del", DelConstant.NOT_DEL);
        List<HrStaffAccountArchitecture> architectureList = baseMapper.selectList(wrapper);
        return build(architectureList);
    }

    /**
     * 使用递归方法构建权限菜单
     */
    public List<HrStaffAccountArchitecture> build(List<HrStaffAccountArchitecture> treeNodes) {
        List<HrStaffAccountArchitecture> trees = new ArrayList<>();
        for (HrStaffAccountArchitecture treeNode : treeNodes) {
            if (treeNode.getParentId() == 1) {
                trees.add(findChildren(treeNode, treeNodes));
            }
        }
        return trees;
    }

    /**
     * 递归查找子节点
     */
    public HrStaffAccountArchitecture findChildren(HrStaffAccountArchitecture treeNode, List<HrStaffAccountArchitecture> treeNodes) {
        treeNode.setChildren(new ArrayList<>());

        for (HrStaffAccountArchitecture it : treeNodes) {

            if (treeNode.getId().equals(it.getParentId())) {
                if (treeNode.getChildren() == null) {
                    treeNode.setChildren(new ArrayList<>());
                }
                treeNode.getChildren().add(findChildren(it, treeNodes));
            }
        }
        return treeNode;
    }

    @Override
    public boolean removeOrganization(Integer id) {
        //根据父id查询以下所有子级
        List<Integer> ids = baseMapper.selectIdByPid(id);
        ids.add(id);
        //批量进行删除
        int i = baseMapper.deleteBatchIds(ids);
        return i>0;
    }


}
