package com.huoran.nakadai.service.impl;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import cn.afterturn.easypoi.excel.entity.enmus.ExcelType;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Maps;
import com.huoran.api.UserClient;
import com.huoran.common.constant.DelConstant;
import com.huoran.common.entity.Role;
import com.huoran.common.entity.UserAccount;
import com.huoran.common.exception.CustomException;
import com.huoran.common.exception.ExceptionEnum;
import com.huoran.common.response.R;
import com.huoran.common.utils.MD5;
import com.huoran.nakadai.entity.*;
import com.huoran.nakadai.entity.req.ExcelImpStaffReq;
import com.huoran.nakadai.entity.req.StaffAddReq;
import com.huoran.nakadai.entity.req.StaffListReq;
import com.huoran.nakadai.entity.req.StaffUpdateReq;
import com.huoran.nakadai.entity.res.StaffResp;
import com.huoran.nakadai.entity.vo.ImportStaffFailureVo;
import com.huoran.nakadai.mapper.HrStaffAccountArchitectureMapper;
import com.huoran.nakadai.mapper.HrStaffDepartmentMapper;
import com.huoran.nakadai.mapper.HrStaffMapper;
import com.huoran.nakadai.service.HrStaffService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huoran.nakadai.utils.*;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLEncoder;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * <p>
 * 中台后台员工账号表 服务实现类
 * </p>
 *
 * @author chen
 * @since 2022-03-22
 */
@Service
public class HrStaffServiceImpl extends ServiceImpl<HrStaffMapper, HrStaff> implements HrStaffService {

    @Autowired
    private UserClient userClient;

    @Autowired
    private HrStaffDepartmentMapper staffDepartmentMapper;

    @Autowired
    private HrStaffAccountArchitectureMapper staffAccountArchitectureMapper;

    @Resource
    StringRedisTemplate stringRedisTemplate;

    @Override
    public R saveStaff(StaffAddReq staffReq) {
        try {
            //用户相关信息表
            UserInfo userInfo = new UserInfo();
            BeanUtils.copyProperties(staffReq, userInfo);
            int addUserInfo = baseMapper.addUserInfo(userInfo);

            //账户相关信息表，账号判重
            //中台判重
            Integer count = baseMapper.checkAccount(staffReq.getAccount());
            if (count>0){
                throw new CustomException(ExceptionEnum.EXIST_ACCOUNT);
            }
            UserAccount userAccount = new UserAccount();
            BeanUtils.copyProperties(staffReq, userAccount);
            userAccount.setPassword(MD5.encrypt("1122aa"));
            userAccount.setUserId(userInfo.getUserId());
            userAccount.setType(2);//平台端区分：0->教师端 1->学生端 2->无端
            userAccount.setPlatformId("3");//1表示职站
            // userAccount.setRoleId("0");//设置为0 表示该表角色id暂时无用 acl_user_role:用户角色表有账号所属角色
            int addAccount = baseMapper.addUserAccount(userAccount);

            //添加员工信息
            HrStaff staff = new HrStaff();
            staff.setAccountId(userAccount.getId());
            // staff.setStaffArchitectureId(staffReq.getStaffArchitectureId());
            int insert = baseMapper.insert(staff);

            //添加员工部门
            List<Integer> staffArchitectureId = staffReq.getStaffArchitectureId();
            if (staffArchitectureId!=null && staffArchitectureId.size()>0){
                staffArchitectureId.forEach(id -> {
                    HrStaffDepartment staffDepartment = new HrStaffDepartment();
                    staffDepartment.setStaffId(staff.getStaffId());
                    staffDepartment.setDepartmentId(id);
                    staffDepartmentMapper.insert(staffDepartment);
                });
            }

            //账号赋予角色
            userClient.rolePermissionService(userAccount.getId(), staffReq.getRoleList(),null);
            return addUserInfo > 0 && addAccount > 0 && insert > 0 ? R.ok() : R.error();
        } catch (Exception e) {
            return R.error(e.getMessage());
        }
    }

    @Override
    public R pageStaffList(StaffListReq req) {
        //查询未加入部门的员工，部门id一定清空
        if (req.getType()==2){
            req.setStaffArchitectureId(null);
        }

        Page<StaffResp> page = new Page<StaffResp>(req.getPageNum(), req.getPageSize());
        IPage<StaffResp> staffList = baseMapper.staffList(page, req);
        return R.ok().put("page", staffList);
    }

    @Override
    public R staffDetail(Integer accountId) {
        StaffResp resp = baseMapper.staffDetail(accountId);
        return R.ok().put("data", resp);
    }

    @Override
    public R modifyStaff(StaffUpdateReq updateReq) {
        //根据账号id获取userId
        Integer userId = baseMapper.getUserId(updateReq.getAccountId());

        //更新员工相关信息表
        if (updateReq.getRoleList() != null) {

            //删除该账号原有角色
            baseMapper.delRoleByAccountId(updateReq.getAccountId());

            //更新员工信息,删除之前部门
            QueryWrapper<HrStaffDepartment> staffDepartmentQueryWrapper = new QueryWrapper<>();
            staffDepartmentQueryWrapper.eq("staff_id", updateReq.getStaffId());
            int delete = staffDepartmentMapper.delete(staffDepartmentQueryWrapper);

            //添加员工部门
            List<Integer> idList = updateReq.getStaffArchitectureId();
            if (idList!=null && idList.size()>0){
                idList.forEach(id -> {
                    HrStaffDepartment staffDepartment = new HrStaffDepartment();
                    staffDepartment.setStaffId(updateReq.getStaffId());
                    staffDepartment.setDepartmentId(id);
                    staffDepartmentMapper.insert(staffDepartment);
                });
            }

            //账号赋予角色
            userClient.rolePermissionService(updateReq.getAccountId(), updateReq.getRoleList(),null);
        }

        //编辑账户相关信息表
        UserAccount userAccount = new UserAccount();
        BeanUtils.copyProperties(updateReq, userAccount);
        userAccount.setId(updateReq.getAccountId());
        boolean updateAccount = baseMapper.modifyUserAccount(userAccount);

        //编辑用户相关信息表
        UserInfo userInfo = new UserInfo();
        BeanUtils.copyProperties(updateReq, userInfo);
        userInfo.setUserId(userId);
        boolean updateUserInfo = baseMapper.modifyUserInfo(userInfo);
        return updateUserInfo && updateAccount ? R.ok() : R.error();
    }

    @Override
    public R delStaff(List<Integer> accountIds) {
        Integer ret = 0;
        if (accountIds.size() > 0) {
            for (Integer accountId : accountIds) {
                QueryWrapper<HrStaff> staffQueryWrapper = new QueryWrapper<>();
                staffQueryWrapper.eq("account_id", accountId);
                HrStaff hrStaff = baseMapper.selectOne(staffQueryWrapper);

                //删除账号、用户表数据  修改用户表逻辑删除状态 修改账号表逻辑删除状态
                userClient.delAccountAndUser(accountId);

                //删除现有员工信息
                UpdateWrapper<HrStaff> updateWrapper = new UpdateWrapper<>();
                updateWrapper.set("is_del", DelConstant.IS_DEL);
                updateWrapper.eq("account_id", accountId);
                ret = baseMapper.update(new HrStaff(), updateWrapper);

                //删除员工部门
                QueryWrapper<HrStaffDepartment> staffDepartmentQueryWrapper = new QueryWrapper<>();
                staffDepartmentQueryWrapper.eq("staff_id", hrStaff.getStaffId());
                staffDepartmentMapper.delete(staffDepartmentQueryWrapper);
            }

        }
        return ret > 0 ? R.ok() : R.error();
    }

    @Override
    @Transactional
    public Map<String, String> importStaff(MultipartFile file) {
        List<ExcelImpStaffReq> impStaffReqList = ExcelImportHelper.readStaff(file);
        if (impStaffReqList.size() <= 0) {
            //小于等于0为上传空模板的情况下 抛出异常
            throw new CustomException(ExceptionEnum.EXCEL_FILE_FORMAT_ERROR);
        }
        int count = 0;
        List<ImportStaffFailureVo> failVo1 = new ArrayList<ImportStaffFailureVo>();
        // 参数合法性校验，只能上传.xlsx后缀的文件
        if (org.apache.commons.lang3.StringUtils.isBlank(file.getOriginalFilename())
                || !file.getOriginalFilename().substring(file.getOriginalFilename().lastIndexOf(".")).equals(".xlsx")) {
            ExceptionCast.cast(CommonCode.EXCEL_FILE_INVALID);
        }
        int size = impStaffReqList.size();
        Map<String, String> map = new HashMap<>();
        Long ii = 1L;//用于记录序列号
        Integer countNum = 0;//用于失败原因排序 eg:1、XXXXX 2、XXXXXXX
        try {
            Integer countSuccess = 0;//统计成功次数
            Integer countError = 0;//统计失败次数

            List<String> workNumberList = new ArrayList<>();//用来储存新增前用户在excel中输入的所有学号用于excel中的数据判重
            List<String> accountList = new ArrayList<>();//用来储存新增前用户在excel中输入的所有账号用于excel中的数据判重
            for (int i = 0; i < impStaffReqList.size(); i++) {
                boolean ret = true;
                ++ii;
                ExcelImpStaffReq req = impStaffReqList.get(i);

                ImportStaffFailureVo vo1 = new ImportStaffFailureVo();

                String errMsg = "";
                //去除Excel中的全角半角空格
                String getUserName = req.getUserName().replaceAll("\\u00A0", "").trim();
                String getAccount = req.getAccount().replaceAll("\\u00A0", "").trim();
                String getRoleName = req.getRoleName().replaceAll("\\u00A0", "").trim();
                String getWorkNumber = req.getWorkNumber().replaceAll("\\u00A0", "").trim();
                String dept = req.getDept().replaceAll("\\u00A0", "").trim();
                String getPhone = req.getPhone().replaceAll("\\u00A0", "").trim();
                String getEmail = req.getEmail().replaceAll("\\u00A0", "").trim();

                if (!workNumberList.contains(req.getWorkNumber())) {
                    workNumberList.add(req.getWorkNumber());
                } else {
                    //包含excel数据中存在相同学号
                    ++countNum;
                    vo1.setWorkNumber(getWorkNumber + " *必填项:(当前Excel数据中该工号已存在重复的!)");
                    errMsg += countNum + " *必填项:(当前Excel数据中该工号号已存在重复的)";
                    ret = false;
                }

                if (!accountList.contains(req.getAccount())) {
                    accountList.add(req.getAccount());
                } else {
                    //包含excel数据中存在相同账号
                    ++countNum;
                    vo1.setAccount(getAccount + " *必填项:(当前Excel数据中该账号已存在重复的)");
                    errMsg += countNum + " *必填项:(当前Excel数据中该账号已存在重复的)";
                    ret = false;
                }

                vo1.setIndex(ii);
                //检验姓名
                if ("".equals(getUserName)) {
                    ++countNum;
                    vo1.setUserName(getUserName + " *必填项:(姓名不能为空) ");
                    errMsg += countNum + "必填项:(姓名不能为空)";
                    ret = false;
                } else {
                    vo1.setUserName(getUserName);
                }
                Integer accountId1 = 0;
                Integer userId1 = 0;
                //检验账号
                if ("".equals(getAccount)) {
                    ++countNum;
                    vo1.setAccount(getAccount + " *必填项:(账号不能为空)");
                    errMsg += countNum + " *必填项:(账号不能为空)";
                    ret = false;
                } else {
                    UserAccount checkAccount = new UserAccount();
                    checkAccount.setAccount(getAccount);
                    List<UserAccount> checkUserAccount = baseMapper.checkUserAccount(checkAccount);
                    //账号存在，验证该账号相关信息
                    if (checkUserAccount.size() > 0) {
                        for (UserAccount userAccount: checkUserAccount){
                            String account = userAccount.getAccount();
                            Integer userId = userAccount.getUserId();
                            String workNumber = userAccount.getWorkNumber();
                            String userName = baseMapper.selectUserNameById(userId);
                            if (getAccount.equals(account)&&getWorkNumber.equals(workNumber)&&getUserName.equals(userName)){
                                accountId1=userAccount.getId();
                                userId1=userId;
                                break;
                            }else {
                                ++countNum;
                                vo1.setAccount(getAccount + " *必填项:(账号已有重复的)");
                                errMsg += countNum + " *必填项:(账号已有重复的)";
                                ret = false;
                            }
                        }

                    } else {
                        vo1.setAccount(getAccount + "");
                    }
                }

                List<Integer> roleIds = new ArrayList<>();

                //检验角色
                if ("".equals(getRoleName)) {
                    ++countNum;
                    vo1.setAccount(getRoleName + " *必填项:(角色不能为空)");
                    errMsg += countNum + " *必填项:(角色不能为空)";
                    ret = false;
                } else {
                    //分隔多个角色
                    String[] roleNames = getRoleName.split("/");
                    for (int r = 0; r < roleNames.length; r++) {
                        //  查询多个角色是否存在
                        Role checkRole = userClient.checkRoleIsExist(roleNames[r], 0,3);
                        if (checkRole != null && !"超级管理员".equals(checkRole.getRoleName())) {
                            roleIds.add(checkRole.getId());
                            //存在
                            vo1.setRoleName(getRoleName);
                        } else {
                            //不存在
                            vo1.setRoleName(getRoleName + " *必填项:(请输入系统中存在的角色)");
                            errMsg += countNum + " *必填项:(请输入系统中存在的角色)";
                            ret = false;
                        }
                    }

                }

                //检验工号
                if ("".equals(getWorkNumber)) {
                    ++countNum;
                    vo1.setWorkNumber(getWorkNumber + " *必填项:(工号不能为空)");
                    errMsg += countNum + " *必填项:(工号不能为空)";
                    ret = false;
                } else {
                    UserAccount workNum = new UserAccount();
                    workNum.setWorkNumber(getWorkNumber);
                    workNum.setType(2);
                    workNum.setPlatformId("3");
                    List<UserAccount> checkUserWorkNum = baseMapper.checkUserAccount(workNum);
                    if (checkUserWorkNum.size() > 0) {
                        ++countNum;
                        vo1.setWorkNumber(getWorkNumber + " *必填项:(工号已经存在重复的)");
                        errMsg += countNum + " *必填项:(工号已经存在重复的)";
                        ret = false;
                    } else {
                        vo1.setWorkNumber(getWorkNumber + "");
                    }
                }

                if (!"".equals(getPhone)) {
                    //校验手机号格式
                    if (!validPhoneNum("0", getPhone)) {
                        ++countNum;
                        vo1.setPhone(getPhone + " *请输入正确的手机号!");
                        errMsg += countNum + " *请输入正确的手机号!";
                        ret = false;
                    } else {
                        //用户账号存在，手机号邮箱则跳过效验
                        if (accountId1==0&&userId1==0){
                            //检验手机号 type 1为手机号 2为校验邮箱
                            boolean checkPhone = userClient.checkPhoneOrEmailExist(getPhone, getEmail, 1);
                            if (checkPhone) {
                                ++countNum;
                                vo1.setPhone(getPhone + " *该手机号已被绑定!");
                                errMsg += countNum + " *该手机号已被绑定!";
                                ret = false;
                            }
                            vo1.setPhone(getPhone);
                        }
                    }

                } else {
                    vo1.setPhone(getPhone);
                }

                if (!"".equals(getEmail)) {
                    //校验邮箱格式
                    if (!isValidEmail(getEmail)) {
                        ++countNum;
                        vo1.setEmail(getEmail + "*请输入正确的邮箱格式!");
                        errMsg += countNum + " *请输入正确的邮箱格式!";
                        ret = false;
                    } else {
                        if (accountId1==0&&userId1==0){
                            //检验手机号 type  2为校验邮箱
                            boolean checkEmail = userClient.checkPhoneOrEmailExist(getPhone, getEmail, 2);
                            if (checkEmail) {
                                ++countNum;
                                vo1.setEmail(getEmail + " *该邮箱已被绑定!");
                                errMsg += countNum + " *该邮箱已被绑定!";
                                ret = false;
                            }
                            vo1.setEmail(getEmail);
                        }
                    }

                } else {
                    vo1.setEmail(getEmail);
                }
                
                ArrayList<Integer> architectureIds = new ArrayList<>();
                //检验所在部门
                if ("".equals(dept)) {
                    ++countNum;
                    vo1.setDept(dept + " *必填项:(请输入正确的员工架构！)");
                    errMsg += countNum + " *必填项:(请输入正确的员工架构!)";
                    ret = false;
                } else {
                    if (ret){
                        //检验是否有多部门
                        String[] multi = dept.split(",");
                        //多部门进行循环添加每个部门
                        if (multi.length>1){
                            for (int d = 0; d < multi.length; d++) {
                                HashMap<String, Object> hashMap = automaticAddArchitecture(multi[d], countNum, dept, vo1);

                                Object countNum1 = hashMap.get("countNum");
                                //存在错误数据
                                if (countNum1!=null){
                                    countNum = (Integer) countNum1;
                                    Object vo11 = hashMap.get("vo1");
                                    vo1 = (ImportStaffFailureVo) vo11;
                                    Object errMsg1 = hashMap.get("errMsg");
                                    errMsg = (String) errMsg1;
                                    Object ret1 = hashMap.get("ret");
                                    ret = (boolean) ret1;
                                }else {
                                    //保存架构id
                                    Object architectureId = hashMap.get("architectureId");
                                    Integer id = 0;
                                    if (architectureId!=null){
                                        id = (Integer) architectureId;
                                        architectureIds.add(id);
                                    }
                                }
                            }

                        }else{
                            HashMap<String, Object> hashMap = automaticAddArchitecture(dept, countNum, dept, vo1);

                            Object countNum1 = hashMap.get("countNum");
                            //存在错误数据
                            if (countNum1!=null){
                                countNum = (Integer) countNum1;
                                Object vo11 = hashMap.get("vo1");
                                vo1 = (ImportStaffFailureVo) vo11;
                                Object errMsg1 = hashMap.get("errMsg");
                                errMsg = (String) errMsg1;
                                Object ret1 = hashMap.get("ret");
                                ret = (boolean) ret1;
                            }else {
                                //保存架构id
                                Object architectureId = hashMap.get("architectureId");
                                Integer id = 0;
                                if (architectureId!=null){
                                    id = (Integer) architectureId;
                                    architectureIds.add(id);
                                }
                            }
                        }
                    }
                }

                vo1.setFailureMsg(errMsg);
                countNum = 0;

                failVo1.add(vo1);

                //ret判断 新增成功的插入到数据库 否则则不新增至数据库
                if (ret) {
                    //用户账号不存在
                    if (accountId1==0&&userId1==0){

                        //执行新增操作
                        // 工号（本校内）——————账号（全平台）判重 ——————手机号、邮箱根据平台id判重
                        UserInfo userInfo = new UserInfo();
                        userInfo.setRoleId("0");
                        userInfo.setUniqueIdentification(String.valueOf(System.currentTimeMillis()));
                        BeanUtils.copyProperties(req, userInfo);
                        baseMapper.addUserInfo(userInfo);

                        UserAccount account = new UserAccount();
                        account.setPassword(MD5.encrypt("1122aa"));
                        account.setPlatformId("3");//所属平台id 1.职站
                        account.setType(2);//平台端区分：0->教师端 1->学生端 2->无端
                        account.setUserId(userInfo.getUserId());
                        account.setRoleId("0");
                        BeanUtils.copyProperties(req, account);
                        baseMapper.addUserAccount(account);

                        accountId1 = account.getId();

                        // 新增至员工信息表
                        HrStaff staff = new HrStaff().setAccountId(accountId1);
                        baseMapper.insert(staff);
                        //添加员工部门
                        if (architectureIds.size()>0){
                            architectureIds.forEach(id -> {
                                HrStaffDepartment staffDepartment = new HrStaffDepartment();
                                staffDepartment.setStaffId(staff.getStaffId());
                                staffDepartment.setDepartmentId(id);
                                staffDepartmentMapper.insert(staffDepartment);
                            });
                        }

                        //  同时新增至用户角色表
                        //账号赋予角色
                        userClient.rolePermissionService(accountId1, roleIds,null);

                    }else {
                        HrStaff staff = baseMapper.selectOne(new QueryWrapper<HrStaff>().eq("account_id", accountId1));
                        //导入新部门、赋予新角色
                        List<HrStaffDepartment> staffDepartments = staffDepartmentMapper.
                                selectList(new QueryWrapper<HrStaffDepartment>().eq("staff_id", staff.getStaffId()));

                        ArrayList<Integer> staffDepartmentIds = new ArrayList<>();
                        for (HrStaffDepartment staffDepartment : staffDepartments) {
                            staffDepartmentIds.add(staffDepartment.getDepartmentId());
                        }
                        for (Integer architectureId : architectureIds) {
                            if (!staffDepartmentIds.contains(architectureId)){
                                HrStaffDepartment hrStaffDepartment = new HrStaffDepartment();
                                hrStaffDepartment.setStaffId(staff.getStaffId());
                                hrStaffDepartment.setDepartmentId(architectureId);
                                staffDepartmentMapper.insert(hrStaffDepartment);
                            }
                        }
                        //补充部门
                        /*if (staffDepartmentIds.size()!=architectureIds.size()){
                            Collection differentNoDuplicate = CollectionUtil.getDifferentNoDuplicate(architectureIds, staffDepartmentIds);
                            ArrayList<Integer> arrayList = new ArrayList<Integer>(differentNoDuplicate);
                            if (arrayList.size()>0){
                                for (Integer id : arrayList) {
                                    HrStaffDepartment hrStaffDepartment = new HrStaffDepartment();
                                    hrStaffDepartment.setStaffId(staff.getStaffId());
                                    hrStaffDepartment.setDepartmentId(id);
                                    staffDepartmentMapper.insert(hrStaffDepartment);
                                }
                            }
                        }*/

                        //补充角色
                        List<Integer> roles = baseMapper.selectRolesByAccountId(accountId1);

                        for (Integer roleId : roleIds) {
                            if (!roles.contains(roleId)){
                                List<Integer> addRoleId = new ArrayList<>();
                                addRoleId.add(roleId);
                                //账号赋予角色
                                userClient.rolePermissionService(accountId1, addRoleId,null);
                            }
                        }
                        /*if (roleIds.size()!=roles.size()){
                            Collection differentNoDuplicate = CollectionUtil.getDifferentNoDuplicate(roleIds, roles);
                            ArrayList<Integer> arrayList = new ArrayList<Integer>(differentNoDuplicate);
                            if (arrayList.size()>0){
                                for (Integer id : arrayList) {
                                    List<Integer> addRoleId = new ArrayList<>();
                                    addRoleId.add(id);
                                    //账号赋予角色
                                    userClient.rolePermissionService(accountId1, addRoleId);
                                }
                            }
                        }*/
                    }
                    //countSuccess：统计累计成功次数
                    countSuccess++;
                } else {
                    //countError:累计失败次数
                    countError++;
                }
            }
            String exportCode = "";

            // 有导入失败的数据，才会存入redis
            if (countError > 0) {
                //生成token
                exportCode = "FAILURE_IMPORT" + UUID.randomUUID().toString().replace("-", "");
                ValueOperations<String, String> ops = stringRedisTemplate.opsForValue();

                List<ImportStaffFailureVo> collect = failVo1.stream().filter(importStaffFailureVo -> {
                    return !importStaffFailureVo.getFailureMsg().equals("");
                }).collect(Collectors.toList());

                String failureVOJson = JSON.toJSONString(collect);
                ops.set(exportCode, failureVOJson, 30 * 60, TimeUnit.SECONDS);
            }
            map.put("exportCode", exportCode);//返回导出code
            map.put("successNum", countSuccess + "");//本次新增成功数量
            map.put("failureNum", countError + "");//本次新增失败数量
            return map;
        } catch (RuntimeException e) {
            System.err.println(e.getMessage());
            throw new RuntimeException();
        }
    }

    @Override
    public void exportFailureRecord(HttpServletResponse response, String token) throws IOException {
        if (org.apache.commons.lang3.StringUtils.isEmpty(token)) {
            return;
        }
        ValueOperations<String, String> ops = stringRedisTemplate.opsForValue();
        //获取数据
        String record = ops.get(token);
        if (org.apache.commons.lang3.StringUtils.isEmpty(record)) {
            return;
        }
        List<ImportStaffFailureVo> parse = JSON.parseObject(record, new TypeReference<List<ImportStaffFailureVo>>() {
        });

        parse.sort(Comparator.comparing(ImportStaffFailureVo::getIndex));

        // 告诉浏览器用什么软件可以打开此文件
        response.setHeader("content-Type", "application/vnd.ms-excel");
        // 下载文件的默认名称
        response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode("员工导入失败数据表", "UTF-8") + ".xls");
        //编码
        response.setCharacterEncoding("UTF-8");
        Workbook workbook = ExcelExportUtil.exportExcel(new ExportParams(), ImportStaffFailureVo.class, parse);
        ExportParams exportParams = new ExportParams(null, "delivery", ExcelType.XSSF);
        exportParams.setStyle(ExcelStyleUtil.class);
        workbook.write(response.getOutputStream());
    }


    /**
     * @Description :   校验手机号
     * @Param checkType 校验类型：0校验手机号码，1校验座机号码，2两者都校验满足其一就可
     * @Param phoneNum
     * @Author Rong---2021/10/18
     */
    public static boolean validPhoneNum(String checkType, String phoneNum) {
        boolean flag = false;
        Pattern p1 = null;
        Pattern p2 = null;
        Matcher m = null;
        p1 = Pattern.compile("^(((13[0-9]{1})|(15[0-9]{1})|(18[0-9]{1})|(17[0-9]{1}))+\\d{8})?$");
        p2 = Pattern.compile("^(0[0-9]{2,3}\\-)?([1-9][0-9]{6,7})$");
        if ("0".equals(checkType)) {
            System.out.println(phoneNum.length());
            if (phoneNum.length() != 11) {
                return false;
            } else {
                m = p1.matcher(phoneNum);
                flag = m.matches();
            }
        } else if ("1".equals(checkType)) {
            if (phoneNum.length() < 11 || phoneNum.length() >= 16) {
                return false;
            } else {
                m = p2.matcher(phoneNum);
                flag = m.matches();
            }
        } else if ("2".equals(checkType)) {
            if (!((phoneNum.length() == 11 && p1.matcher(phoneNum).matches()) || (phoneNum.length() < 16 && p2.matcher(phoneNum).matches()))) {
                return false;
            } else {
                flag = true;
            }
        }
        return flag;
    }

    /**
     * @Description :  校验邮箱
     * @Param email
     * @Author Rong---2021/10/18
     */
    public static boolean isValidEmail(String email) {
        if ((email != null) && (!email.isEmpty())) {
            return Pattern.matches("^(\\w+([-.][A-Za-z0-9]+)*){3,18}@\\w+([-.][A-Za-z0-9]+)*\\.\\w+([-.][A-Za-z0-9]+)*$", email);
        }
        return false;
    }

    /**
    * 组织架构判重,返回下一层级的父id
    *
     */
    public Integer checkArchitectureRepeat(String name ,Integer pid){
        QueryWrapper<HrStaffAccountArchitecture> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("organization_name",name).eq("parent_id",pid);
        HrStaffAccountArchitecture hrStaffAccountArchitecture = staffAccountArchitectureMapper.selectOne(queryWrapper);
        if (hrStaffAccountArchitecture!=null){
            return hrStaffAccountArchitecture.getId();
        }
        return null;
    }
    /**
     * 根据导入部门自动添加组织架构
     */
    public HashMap<String, Object> automaticAddArchitecture(String architectureName,Integer countNum,
                                                  String dept,ImportStaffFailureVo vo1){
        HashMap<String, Object> map = Maps.newHashMap();
        //获取部门层级
        String[] split = architectureName.split("/");
        if (split.length>6){
            map.put("countNum",++countNum);
            map.put("vo1",vo1.setDept(dept + " 员工架构暂不支持第7层级"));
            map.put("errMsg",countNum + " 员工架构暂不支持第7层级");
            map.put("ret",false);
            return map;
        }
        // 检验输入的一级部门是否存在
        //查询一级部门信息
        QueryWrapper<HrStaffAccountArchitecture> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("organization_name", split[0]);
        HrStaffAccountArchitecture hrStaffAccountArchitecture = staffAccountArchitectureMapper.selectOne(queryWrapper1);
        if (hrStaffAccountArchitecture == null) {
            //为空表示不存在该部门,一级部门不存在，表示子级部门都能直接新增，达底端返回部门id
            int pid = 1;
            for (int j = 0; j < split.length; j++) {
                HrStaffAccountArchitecture hrStaffAccountArchitecture1 = new HrStaffAccountArchitecture();
                hrStaffAccountArchitecture1.setOrganizationName(split[j]);
                hrStaffAccountArchitecture1.setParentId(pid);
                hrStaffAccountArchitecture1.setLevel(j+1);
                int insert = staffAccountArchitectureMapper.insert(hrStaffAccountArchitecture1);
                //添加成功后id赋值给下一层级父id
                if (insert>0){
                    pid = hrStaffAccountArchitecture1.getId();
                }
                //最后返回部门id
                if (split.length-1==j){
                    map.put("architectureId",pid);
                }
            }
        } else {
            Integer nextLevelPid = hrStaffAccountArchitecture.getId();
            //存在一级部门，循环查询下一层级，存在则继续向下判断，否则进行新增，到达底端返回部门id
            int pid = 1;
            for (int j = 1; j < split.length; j++) {
                Integer nextPid = checkArchitectureRepeat(split[j],nextLevelPid);
                //都存在，最后一次循环返回部门id
                if (split.length-1==j){
                    map.put("architectureId",nextPid);
                }
                //层级已存在，进入下次循环
                if (nextPid!=null){
                    nextLevelPid = nextPid;
                    continue;
                }
                //该层级不存在，进行新增
                HrStaffAccountArchitecture architecture = new HrStaffAccountArchitecture();
                architecture.setOrganizationName(split[j]);
                architecture.setParentId(nextLevelPid);
                architecture.setLevel(j+1);
                int insert = staffAccountArchitectureMapper.insert(architecture);
                //添加成功后id赋值给下一层级父id
                if (insert>0){
                    nextLevelPid = architecture.getId();
                }
                //最后返回部门id
                if (split.length-1==j){
                    map.put("architectureId",nextLevelPid);
                }
            }
        }
        return map;
    }

    public static void main(String[] args) {
//集合一

        List _first=new ArrayList();

        _first.add("1");

        _first.add("2");

        _first.add("3");

//集合二

        List _second=new ArrayList();

        _second.add("1");

        _second.add("2");
        _second.add("2");
        _second.add("4");

        Collection differentNoDuplicate = CollectionUtil.getDifferentNoDuplicate(_first, _second);
        ArrayList<String> arrayList = new ArrayList<>(differentNoDuplicate);
        arrayList.forEach(System.out::println);
    }

}
