package com.huoran.nakadai.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huoran.common.exception.CustomException;
import com.huoran.common.exception.ExceptionEnum;
import com.huoran.common.response.R;
import com.huoran.nakadai.entity.Discipline;
import com.huoran.nakadai.entity.MallDiscipline;
import com.huoran.nakadai.entity.Professional;
import com.huoran.nakadai.entity.ProfessionalClass;
import com.huoran.nakadai.entity.vo.DisciplineExcelVo;
import com.huoran.nakadai.mapper.DisciplineMapper;
import com.huoran.nakadai.mapper.MallDisciplineMapper;
import com.huoran.nakadai.mapper.ProfessionalClassMapper;
import com.huoran.nakadai.mapper.ProfessionalMapper;
import com.huoran.nakadai.service.MallDisciplineService;
import com.huoran.nakadai.utils.CommonCode;
import com.huoran.nakadai.utils.ExcelImportHelper;
import com.huoran.nakadai.utils.ExceptionCast;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * <p>
 * 商品学科类别关联表 服务实现类
 * </p>
 *
 * @author chen
 * @since 2023-04-25
 */
@Service
public class MallDisciplineServiceImpl extends ServiceImpl<MallDisciplineMapper, MallDiscipline> implements MallDisciplineService {

    @Autowired
    private DisciplineMapper disciplineMapper;

    @Autowired
    private ProfessionalClassMapper professionalClassMapper;

    @Autowired
    private ProfessionalMapper professionalMapper;

    @Override
    public R importSubjectCategoriesInBatches(MultipartFile file) {
        List<DisciplineExcelVo> disciplineExcelVos = ExcelImportHelper.readDiscipline(file);
        if (disciplineExcelVos.size() <= 0) {
            //小于等于0为上传空模板的情况下 抛出异常
            throw new CustomException(ExceptionEnum.EXCEL_FILE_FORMAT_ERROR);
        }
        // 参数合法性校验，只能上传.xlsx后缀的文件
        if (org.apache.commons.lang3.StringUtils.isBlank(file.getOriginalFilename())
                || !file.getOriginalFilename().substring(file.getOriginalFilename().lastIndexOf(".")).equals(".xlsx")) {
            ExceptionCast.cast(CommonCode.EXCEL_FILE_INVALID);
        }

        for (int i = 0; i < disciplineExcelVos.size(); i++) {
            DisciplineExcelVo disciplineExcelVo = disciplineExcelVos.get(i);

            //去除Excel中的全角半角空格
            String num = disciplineExcelVo.getSerialNumber().replaceAll("\\u00A0", "").trim();
            String category = disciplineExcelVo.getCategory().replaceAll(" ", "").trim();
            String majorCategory = disciplineExcelVo.getMajorCategory().replaceAll(" ", "").trim();
            String major = disciplineExcelVo.getMajor().replaceAll(" ", "").trim();
            Discipline discipline = disciplineMapper.selectOne(new QueryWrapper<Discipline>().eq("discipline_name", category));
            if (discipline!=null){
                ProfessionalClass professionalClass = professionalClassMapper.selectOne(new QueryWrapper<ProfessionalClass>().
                        eq("professional_class_name", majorCategory).eq("discipline_id", discipline.getDisciplineId()));
                if (professionalClass!=null){
                    Professional professional = professionalMapper.selectOne(new QueryWrapper<Professional>().
                            eq("professional_name", major).eq("professional_class_id", professionalClass.getProfessionalClassId()));
                    if (professional==null){
                        Professional professional1 = new Professional();
                        professional1.setProfessionalClassId(professionalClass.getProfessionalClassId());
                        professional1.setProfessionalName(major);
                        professionalMapper.insert(professional1);
                    }

                }else {
                    ProfessionalClass professionalClass1 = new ProfessionalClass();
                    professionalClass1.setDisciplineId(discipline.getDisciplineId());
                    professionalClass1.setProfessionalClassName(majorCategory);
                    professionalClassMapper.insert(professionalClass1);
                }

            }else {
                Discipline discipline1 = new Discipline();
                discipline1.setDisciplineName(category);
                disciplineMapper.insert(discipline1);
            }

        }

        return R.ok();

    }
}
