package com.huoran.nakadai.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huoran.api.OccupationlabClient;
import com.huoran.api.UserClient;
import com.huoran.common.exception.CustomException;
import com.huoran.common.exception.ExceptionEnum;
import com.huoran.common.response.R;
import com.huoran.common.utils.TokenUtils;
import com.huoran.nakadai.entity.*;
import com.huoran.nakadai.entity.model.OrderItem;
import com.huoran.nakadai.entity.req.GoodsListReq;
import com.huoran.nakadai.entity.res.GoodsRes;
import com.huoran.nakadai.entity.vo.GoodsVo;
import com.huoran.nakadai.mapper.*;
import com.huoran.nakadai.service.MallService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.stream.Collectors;

/**
 * <p>
 * 商城管理表 服务实现类
 * </p>
 *
 * @author chen
 * @since 2023-04-25
 */
@Service
public class MallServiceImpl extends ServiceImpl<MallMapper, Mall> implements MallService {

    @Autowired
    ThreadPoolExecutor executor;

    @Autowired
    private UserClient userClient;

    @Autowired
    private MallClassificationMapper mallClassificationMapper;

    @Autowired
    private MallDisciplineMapper mallDisciplineMapper;

    @Autowired
    private MallPriceMapper mallPriceMapper;

    @Autowired
    private MallTagsMapper mallTagsMapper;

    @Autowired
    private MallTypeMapper mallTypeMapper;

    @Autowired
    private MallSupplierMapper mallSupplierMapper;

    @Autowired
    private MallAnnexMapper mallAnnexMapper;

    @Autowired
    private OccupationlabClient occupationlabClient;

    @Autowired
    private CurriculumMapper curriculumMapper;

    @Autowired
    private DataProductMapper dataProductMapper;

    @Autowired
    private MallNonAssociatedLinksMapper mallNonAssociatedLinksMapper;

    @Override
    @Transactional
//    @CacheEvict(value = "n_listOfGoods", allEntries = true)
    public R addGoods(GoodsVo goodsVo) throws ExecutionException, InterruptedException {
        Mall goods = goodsVo.getMall();
        List<Integer> classificationIds = goodsVo.getClassificationIds();
        List<MallDiscipline> mallDisciplines = goodsVo.getMallDisciplines();
        List<MallPrice> mallPrices = goodsVo.getMallPrices();
        List<Integer> tagsIds = goodsVo.getTagsIds();
        List<Integer> supplierIds = goodsVo.getSupplierIds();
        List<Integer> typeIds = goodsVo.getTypeIds();
        List<MallAnnex> mallAnnex = goodsVo.getMallAnnex();
        List<MallNonAssociatedLinks> mallNonAssociatedLinks = goodsVo.getMallNonAssociatedLinks();

        CompletableFuture<Integer> goodsFuture = CompletableFuture.supplyAsync(() -> {
            //添加商品基本信息
            baseMapper.insert(goods);
            return goods.getMallId();
        }, executor);


        CompletableFuture<Void> classificationFuture = goodsFuture.thenAcceptAsync(mallId -> {
            //如果产品分类存在，则保存
            if (ObjectUtil.isNotNull(classificationIds)) {
                for (Integer classificationId : classificationIds) {
                    MallClassification mallClassification = new MallClassification();
                    mallClassification.setMallId(mallId);
                    mallClassification.setClassificationId(classificationId);
                    mallClassificationMapper.insert(mallClassification);
                }
            }
        }, executor);


        CompletableFuture<Void> disciplinesFuture = goodsFuture.thenAcceptAsync(mallId -> {
            //如果适用学科专业集存在，则保存
            if (ObjectUtil.isNotNull(mallDisciplines)) {
                for (MallDiscipline mallDiscipline : mallDisciplines) {
                    mallDiscipline.setMallId(mallId);
                    mallDisciplineMapper.insert(mallDiscipline);
                }
            }
        }, executor);


        CompletableFuture<Void> mallPriceFuture = goodsFuture.thenAcceptAsync(mallId -> {
            //如果产品分类存在，则保存
            if (ObjectUtil.isNotNull(mallPrices)) {
                for (MallPrice mallPrice : mallPrices) {
                    mallPrice.setMallId(mallId);
                    mallPriceMapper.insert(mallPrice);
                }
            }
        }, executor);

        CompletableFuture<Void> tagsIdsFuture = goodsFuture.thenAcceptAsync(mallId -> {
            //如果产品分类存在，则保存
            if (ObjectUtil.isNotNull(tagsIds)) {
                for (Integer tagsId : tagsIds) {
                    MallTags mallTags = new MallTags();
                    mallTags.setTagsId(tagsId);
                    mallTags.setMallId(mallId);
                    mallTagsMapper.insert(mallTags);
                }
            }
        }, executor);


        CompletableFuture<Void> mallSupplierFuture = goodsFuture.thenAcceptAsync(mallId -> {
            //如果产品分类存在，则保存
            if (ObjectUtil.isNotNull(supplierIds)) {
                for (Integer supplierId : supplierIds) {
                    MallSupplier mallSupplier = new MallSupplier();
                    mallSupplier.setSupplierId(supplierId);
                    mallSupplier.setMallId(mallId);
                    mallSupplierMapper.insert(mallSupplier);
                }
            }
        }, executor);


        CompletableFuture<Void> mallTypeFuture = goodsFuture.thenAcceptAsync(mallId -> {
            //如果产品分类存在，则保存
            if (ObjectUtil.isNotNull(typeIds)) {
                for (Integer typeId : typeIds) {
                    MallType mallType = new MallType();
                    mallType.setTypeId(typeId);
                    mallType.setMallId(mallId);
                    mallTypeMapper.insert(mallType);
                }
            }
        }, executor);


        CompletableFuture<Void> mallAnnexFuture = goodsFuture.thenAcceptAsync(mallId -> {
            //如果产品分类存在，则保存
            if (ObjectUtil.isNotNull(mallAnnex)) {
                for (MallAnnex annex : mallAnnex) {
                    annex.setMallId(mallId);
                    mallAnnexMapper.insert(annex);
                }
            }
        }, executor);

        CompletableFuture<Void> mallNonAssociatedLinksFuture = goodsFuture.thenAcceptAsync(mallId -> {
            //如果产品链接存在，则保存
            if (ObjectUtil.isNotNull(mallNonAssociatedLinks)) {
                for (MallNonAssociatedLinks mallNonAssociatedLink : mallNonAssociatedLinks) {
                    mallNonAssociatedLink.setMallId(mallId);
                    mallNonAssociatedLinksMapper.insert(mallNonAssociatedLink);
                }
            }
        }, executor);

        //等待所有任务都完成
        CompletableFuture.allOf(classificationFuture,
                disciplinesFuture, mallPriceFuture,
                tagsIdsFuture, mallSupplierFuture,
                mallTypeFuture,mallAnnexFuture,mallNonAssociatedLinksFuture).get();

        if (goodsFuture.get() > 0) {
            return R.ok().put("mallId", goods.getMallId());
        }
        return R.error();
    }

    @Override
//    @Cacheable(value = {"n_listOfGoods"})
    public R listOfGoods(GoodsListReq goodsListReq) {
        //默认为0
        goodsListReq.setSchoolPurchaseSign(0);
        Page<GoodsRes> page = new Page<>(goodsListReq.getPageNum(), goodsListReq.getPageSize());

        List<Integer> mallIds = new ArrayList<>();
        //查询学校购买的商品id
        if (goodsListReq.getSchoolId()!=null&& goodsListReq.getSchoolId()!=0){
            mallIds = this.schoolGoods(goodsListReq.getSchoolId());
            //查询已购买信息，未购买直接返回空
            if (mallIds==null || mallIds.size()<1){
                if (goodsListReq.getPurchaseStatus()!=null&&goodsListReq.getPurchaseStatus()==1){
                    return createEmptyPage();
                }
            }
            goodsListReq.setMallIds(mallIds);

        }
        if (goodsListReq.getPurchaseStatus()!=null&&goodsListReq.getPurchaseStatus()==1){
            //学校购买标识，购买过可以查询下架商品
            goodsListReq.setSchoolPurchaseSign(1);
        }

//        if (goodsListReq.getPurchaseStatus()!=null&&goodsListReq.getPurchaseStatus()==2){
//            //查询，所有上架商品
//            List<Integer> allMallIds = baseMapper.selectGoodsList();
//
//            //如果所有上架商品都在学校购买的商品之中，未购买直接返回空
//            if (allMallIds!=null){
//                int i = 0;
//                for (Integer allMallId : allMallIds) {
//                    if (mallIds.contains(allMallId)){
//                        i++;
//                    }
//                }
//                if (allMallIds.size()==i){
//                    return createEmptyPage();
//                }
//
//                allMallIds.removeAll(mallIds);
//            }
//            goodsListReq.setMallIds(allMallIds);
//        }
        if (goodsListReq.getPurchaseStatus()!=null&&goodsListReq.getPurchaseStatus()==2){
            //查询，所有上架商品
            List<Integer> allMallIds = baseMapper.selectGoodsList();

            //如果所有上架商品都在学校购买的商品之中，未购买直接返回空
            if (allMallIds!=null && allMallIds.stream().allMatch(mallIds::contains)){
                return createEmptyPage();
            }
            assert allMallIds != null;
            allMallIds.removeAll(mallIds);
            goodsListReq.setMallIds(allMallIds);
        }

        IPage<GoodsRes> iPage = baseMapper.goodsList(page, goodsListReq);
        for (GoodsRes goodsRes : iPage.getRecords()) {
            //我校标识
            if (mallIds!=null && mallIds.size()>0){
                if (mallIds.contains(goodsRes.getMallId())){
                    goodsRes.setLogoOfOurSchool(true);
                }
            }
            List<MallNonAssociatedLinks> mallNonAssociatedLinks = mallNonAssociatedLinksMapper.
                    selectList(new QueryWrapper<MallNonAssociatedLinks>().eq("mall_id", goodsRes.getMallId()));
            goodsRes.setMallNonAssociatedLinks(mallNonAssociatedLinks);
        }

        if (goodsListReq.getIsAssociatedProduct()!=null && goodsListReq.getIsAssociatedProduct()==1){
            //查询课程下面的系统，赛事内容调用
            ArrayList<Integer> curriculumList = new ArrayList<>();
            curriculumList.add(1);
            curriculumList.add(2);
            List<GoodsRes> list = iPage.getRecords().stream().
                    filter(goodsRes -> goodsRes.getIsAssociatedProduct() == 0 &&
                            curriculumList.contains(goodsRes.getClassificationId())).collect(Collectors.toList());
            for (GoodsRes goodsRes : list) {
                //查询关联产品对应的系统
                String systemId = baseMapper.selectMallSystem(goodsRes.getAssociatedProduct());
                goodsRes.setSystemId(systemId);
            }
            iPage.setRecords(list);
            iPage.setSize(list.size());
        }

        return R.ok().put("page",iPage);
    }

    public R createEmptyPage() {
        IPage<GoodsRes> iPage = new IPage<GoodsRes>() {

            @Override
            public List<GoodsRes> getRecords() {
                return null;
            }

            @Override
            public IPage<GoodsRes> setRecords(List<GoodsRes> list) {
                return null;
            }

            @Override
            public long getTotal() {
                return 0;
            }

            @Override
            public IPage<GoodsRes> setTotal(long l) {
                return null;
            }

            @Override
            public long getSize() {
                return 0;
            }

            @Override
            public IPage<GoodsRes> setSize(long l) {
                return null;
            }

            @Override
            public long getCurrent() {
                return 0;
            }

            @Override
            public IPage<GoodsRes> setCurrent(long l) {
                return null;
            }
        };
        return R.ok().put("page",iPage);
    }

    @Override
    public R stationShoppingMall(GoodsListReq goodsListReq) {
        //默认为0
        goodsListReq.setSchoolPurchaseSign(0);
        Page<GoodsRes> page = new Page<>(goodsListReq.getPageNum(), goodsListReq.getPageSize());

        IPage<GoodsRes> iPage = baseMapper.goodsList(page, goodsListReq);
        for (GoodsRes goodsRes : iPage.getRecords()) {
            List<MallNonAssociatedLinks> mallNonAssociatedLinks = mallNonAssociatedLinksMapper.
                    selectList(new QueryWrapper<MallNonAssociatedLinks>().eq("mall_id", goodsRes.getMallId()));
            goodsRes.setMallNonAssociatedLinks(mallNonAssociatedLinks);
        }

        if (goodsListReq.getIsAssociatedProduct()!=null && goodsListReq.getIsAssociatedProduct()==1){
            //查询课程下面的系统，赛事内容调用
            ArrayList<Integer> curriculumList = new ArrayList<>();
            curriculumList.add(1);
            curriculumList.add(2);
            List<GoodsRes> list = iPage.getRecords().stream().
                    filter(goodsRes -> goodsRes.getIsAssociatedProduct() == 0 &&
                            curriculumList.contains(goodsRes.getClassificationId())).collect(Collectors.toList());
            for (GoodsRes goodsRes : list) {
                //查询关联产品对应的系统
                String systemId = baseMapper.selectMallSystem(goodsRes.getAssociatedProduct());
                goodsRes.setSystemId(systemId);
            }
            iPage.setRecords(list);
            iPage.setSize(list.size());
        }

        return R.ok().put("page",iPage);
    }

    @Override
    public List<Integer> schoolGoods(Integer schoolId) {
        return baseMapper.schoolGoods(schoolId);
    }

    @Override
    public GoodsVo detailsOfGoods(Integer mallId,String token) {
        GoodsVo goodsVo = new GoodsVo();
        Mall mall = baseMapper.selectById(mallId);

        List<MallClassification> classifications = mallClassificationMapper.selectList(new QueryWrapper<MallClassification>().eq("mall_id", mallId));
        List<Integer> classificationIds = classifications.stream().map(MallClassification::getClassificationId).collect(Collectors.toList());

        //关联了课程产品
        if (mall.getIsAssociatedProduct()==0){
            if (classificationIds.contains(1)||classificationIds.contains(2)){
                //查询课程系统
                String systemId = baseMapper.selectSystem(mall.getAssociatedProduct(),0);
                if (systemId!=null&&token!=null){
                    goodsVo.setSystemId(systemId);
                    R r = occupationlabClient.getProjectBySystemId(systemId, token,0, mall.getAssociatedProduct(),mallId);
                    Integer projectSize = (Integer)r.get("projectSize");
                    if (projectSize!=null){
                        goodsVo.setNumberOfExperimentalItems(projectSize);
                    }
                }
                if (token==null){
                    goodsVo.setNumberOfExperimentalItems(50);
                }
            }
        }

        List<MallDiscipline> mallDisciplines = mallDisciplineMapper.selectList(new QueryWrapper<MallDiscipline>().eq("mall_id", mallId));
        List<MallPrice> mallPrices = mallPriceMapper.selectList(new QueryWrapper<MallPrice>().eq("mall_id", mallId));

        List<MallTags> mallTags = mallTagsMapper.selectList(new QueryWrapper<MallTags>().eq("mall_id", mallId));
        List<Integer> mallTagIds = mallTags.stream().map(MallTags::getTagsId).collect(Collectors.toList());

        List<MallType> mallTypes = mallTypeMapper.selectList(new QueryWrapper<MallType>().eq("mall_id", mallId));
        List<Integer> mallTypeIds = mallTypes.stream().map(MallType::getTypeId).collect(Collectors.toList());

        List<MallSupplier> mallSuppliers = mallSupplierMapper.selectList(new QueryWrapper<MallSupplier>().eq("mall_id", mallId));
        List<Integer> mallSupplierIds = mallSuppliers.stream().map(MallSupplier::getSupplierId).collect(Collectors.toList());

        List<MallAnnex> mallAnnexes = mallAnnexMapper.selectList(new QueryWrapper<MallAnnex>().eq("mall_id", mallId));
        List<MallNonAssociatedLinks> mallNonAssociatedLinks = mallNonAssociatedLinksMapper.selectList(new QueryWrapper<MallNonAssociatedLinks>().eq("mall_id", mallId));

        GoodsRes goodsRes = baseMapper.selectGoods(mallId);
        if (token!=null){
            String accountId = TokenUtils.getIdByJwtToken(token);
            Integer schoolId = userClient.getSchoolIdByAccountId(accountId);
            //查询学校购买的课程id
            if (schoolId!=null&& schoolId!=0){
                List<Integer> list = baseMapper.schoolCourseIds(schoolId);
                if (list.contains(mall.getAssociatedProduct())){
                    goodsRes.setLogoOfOurSchool(true);
                }
            }

            if (schoolId!=null&& schoolId!=0){
                List<Integer> list = this.schoolGoods(schoolId);
                if (list.contains(mallId)){
                    goodsRes.setLogoOfOurSchool(true);
                }
            }
        }

        goodsVo.setGoodsRes(goodsRes);
        goodsVo.setMall(mall);
        goodsVo.setMallPrices(mallPrices);
        goodsVo.setMallDisciplines(mallDisciplines);
        goodsVo.setTagsIds(mallTagIds);
        goodsVo.setClassificationIds(classificationIds);
        goodsVo.setTypeIds(mallTypeIds);
        goodsVo.setSupplierIds(mallSupplierIds);
        goodsVo.setMallAnnex(mallAnnexes);
        goodsVo.setMallNonAssociatedLinks(mallNonAssociatedLinks);
        return goodsVo;
    }

    @Override
    @Transactional
//    @CacheEvict(value = "n_listOfGoods", allEntries = true)
    public boolean deletionOfGoods(Integer mallId) {
        //商品相关联数据删除，再删除商品
        mallClassificationMapper.delete(new QueryWrapper<MallClassification>().eq("mall_id", mallId));
        mallDisciplineMapper.delete(new QueryWrapper<MallDiscipline>().eq("mall_id", mallId));
        mallPriceMapper.delete(new QueryWrapper<MallPrice>().eq("mall_id", mallId));
        mallTagsMapper.delete(new QueryWrapper<MallTags>().eq("mall_id", mallId));
        mallTypeMapper.delete(new QueryWrapper<MallType>().eq("mall_id", mallId));
        mallSupplierMapper.delete(new QueryWrapper<MallSupplier>().eq("mall_id", mallId));
        mallAnnexMapper.delete(new QueryWrapper<MallAnnex>().eq("mall_id", mallId));
        mallNonAssociatedLinksMapper.delete(new QueryWrapper<MallNonAssociatedLinks>().eq("mall_id", mallId));
        int delete = baseMapper.deleteById(mallId);
        return delete > 0;
    }

    @Override
    @Transactional
//    @CacheEvict(value = "n_listOfGoods", allEntries = true)
    public boolean renewalOfGoods(GoodsVo goodsVo) {
        Mall mall = goodsVo.getMall();
        int update = baseMapper.updateById(mall);
        Integer mallId = mall.getMallId();

        //关联了课程产品并上架
        if (mall.getIsAssociatedProduct()==0 && mall.getIsShelves()==0){
            if (goodsVo.getClassificationIds().contains(1)||goodsVo.getClassificationIds().contains(2)){
                //查询课程
                Curriculum curriculum = curriculumMapper.selectById(mall.getAssociatedProduct());
                if (ObjectUtil.isNull(curriculum)){
                    throw new CustomException(ExceptionEnum.COURSE_PRODUCTS_HAVE_BEEN_REMOVED);
                }else {
                    if (curriculum.getIsShelves()==0){
                        throw new CustomException(ExceptionEnum.COURSE_PRODUCTS_HAVE_BEEN_REMOVED_FROM_SALE);
                    }
                }
                //关联了数据平台
            }else if (goodsVo.getClassificationIds().contains(5)){
                DataProduct dataProduct = dataProductMapper.selectById(mall.getAssociatedProduct());
                if (ObjectUtil.isNull(dataProduct)){
                    throw new CustomException(ExceptionEnum.DATA_PRODUCTS_HAVE_BEEN_REMOVED);
                }else {
                    if (dataProduct.getStatus()==0){
                        throw new CustomException(ExceptionEnum.DATA_PRODUCTS_HAVE_BEEN_REMOVED_FROM_SALE);
                    }
                }
            }
        }

        //商品相关联数据删除，再删除商品
        mallClassificationMapper.delete(new QueryWrapper<MallClassification>().eq("mall_id", mallId));
        mallDisciplineMapper.delete(new QueryWrapper<MallDiscipline>().eq("mall_id", mallId));
        mallPriceMapper.delete(new QueryWrapper<MallPrice>().eq("mall_id", mallId));
        mallTagsMapper.delete(new QueryWrapper<MallTags>().eq("mall_id", mallId));
        mallTypeMapper.delete(new QueryWrapper<MallType>().eq("mall_id", mallId));
        mallSupplierMapper.delete(new QueryWrapper<MallSupplier>().eq("mall_id", mallId));
        mallAnnexMapper.delete(new QueryWrapper<MallAnnex>().eq("mall_id", mallId));
        mallNonAssociatedLinksMapper.delete(new QueryWrapper<MallNonAssociatedLinks>().eq("mall_id", mallId));

        //如果产品分类存在，则保存
        if (ObjectUtil.isNotNull(goodsVo.getClassificationIds())) {
            for (Integer classificationId : goodsVo.getClassificationIds()) {
                MallClassification mallClassification = new MallClassification();
                mallClassification.setMallId(mallId);
                mallClassification.setClassificationId(classificationId);
                mallClassificationMapper.insert(mallClassification);
            }
        }

        //如果适用学科专业集存在，则保存
        if (ObjectUtil.isNotNull(goodsVo.getMallDisciplines())) {
            for (MallDiscipline mallDiscipline : goodsVo.getMallDisciplines()) {
                mallDiscipline.setMallId(mallId);
                mallDisciplineMapper.insert(mallDiscipline);
            }
        }

        //如果产品分类存在，则保存
        if (ObjectUtil.isNotNull(goodsVo.getMallPrices())) {
            for (MallPrice mallPrice : goodsVo.getMallPrices()) {
                mallPrice.setMallId(mallId);
                mallPriceMapper.insert(mallPrice);
            }
        }

        //如果产品分类存在，则保存
        if (ObjectUtil.isNotNull(goodsVo.getTagsIds())) {
            for (Integer tagsId : goodsVo.getTagsIds()) {
                MallTags mallTags = new MallTags();
                mallTags.setTagsId(tagsId);
                mallTags.setMallId(mallId);
                mallTagsMapper.insert(mallTags);
            }
        }


        //如果产品分类存在，则保存
        if (ObjectUtil.isNotNull(goodsVo.getSupplierIds())) {
            for (Integer supplierId : goodsVo.getSupplierIds()) {
                MallSupplier mallSupplier = new MallSupplier();
                mallSupplier.setSupplierId(supplierId);
                mallSupplier.setMallId(mallId);
                mallSupplierMapper.insert(mallSupplier);
            }
        }

        //如果产品分类存在，则保存
        if (ObjectUtil.isNotNull(goodsVo.getTypeIds())) {
            for (Integer typeId : goodsVo.getTypeIds()) {
                MallType mallType = new MallType();
                mallType.setTypeId(typeId);
                mallType.setMallId(mallId);
                mallTypeMapper.insert(mallType);
            }
        }

        //如果产品附件存在，则保存
        if (ObjectUtil.isNotNull(goodsVo.getMallAnnex())) {
            for (MallAnnex annex : goodsVo.getMallAnnex()) {
                annex.setMallId(mallId);
                mallAnnexMapper.insert(annex);
            }
        }

        //如果产品链接存在，则保存
        if (ObjectUtil.isNotNull(goodsVo.getMallNonAssociatedLinks())) {
            for (MallNonAssociatedLinks mallNonAssociatedLink : goodsVo.getMallNonAssociatedLinks()) {
                mallNonAssociatedLink.setMallId(mallId);
                mallNonAssociatedLinksMapper.insert(mallNonAssociatedLink);
            }
        }

        return update>0;
    }


    @Override
    public List<PartnerArticleClassification> goodsSchemeClassification() {
        //查询关联方案
        List<PartnerArticleClassification> list = baseMapper.selectAssociationScheme();
        for (PartnerArticleClassification partnerArticleClassification : list) {
            //查询关联商品
            if (StringUtils.isNotEmpty(partnerArticleClassification.getSchemeId())){
                String mallId = baseMapper.selectAssociationMall(partnerArticleClassification.getSchemeId());
                partnerArticleClassification.setMallId(mallId);
            }
        }
        return list;
    }

    @Override
    public R websiteProductList(GoodsListReq goodsListReq) {

        Page<GoodsRes> page = new Page<>(goodsListReq.getPageNum(), goodsListReq.getPageSize());
        IPage<GoodsRes> iPage = baseMapper.websiteProductList(page, goodsListReq);
        for (GoodsRes goodsRes : iPage.getRecords()) {
            List<MallNonAssociatedLinks> mallNonAssociatedLinks = mallNonAssociatedLinksMapper.
                    selectList(new QueryWrapper<MallNonAssociatedLinks>().eq("mall_id", goodsRes.getMallId()));
            goodsRes.setMallNonAssociatedLinks(mallNonAssociatedLinks);
        }

        return R.ok().put("page",iPage);
    }
}
