package com.huoran.nakadai.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.huoran.common.constant.DelConstant;
import com.huoran.common.constant.ModellConstant;
import com.huoran.common.response.R;
import com.huoran.nakadai.entity.ModelReferenceCategory;
import com.huoran.nakadai.entity.ModelReferenceDemo;
import com.huoran.nakadai.entity.ModelSysCategory;
import com.huoran.nakadai.entity.ModelSysDemo;
import com.huoran.nakadai.mapper.ModelReferenceCategoryMapper;
import com.huoran.nakadai.mapper.ModelReferenceDemoMapper;
import com.huoran.nakadai.service.ModelReferenceCategoryService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 引用模型-分类表 服务实现类
 * </p>
 *
 * @author chen
 * @since 2022-03-04
 */
@Service
public class ModelReferenceCategoryServiceImpl extends ServiceImpl<ModelReferenceCategoryMapper, ModelReferenceCategory> implements ModelReferenceCategoryService {

    @Autowired
    private ModelReferenceDemoMapper referenceDemoMapper;

    /**
     * 递归查询子级
     *
     * @param menus        查询的所有菜单
     * @param firstMenuVos 第一级的菜单
     */
    private void findSubMenu(List<ModelReferenceCategory> menus, List<ModelReferenceCategory> firstMenuVos) {
        //遍历第一级
        firstMenuVos.forEach(firstMenuVo -> {
            //子级菜单集合
            List<ModelReferenceCategory> secondMenuVos = new LinkedList<>();
            //查找子级  递归的出口就是循环完毕
            for (ModelReferenceCategory menu : menus) {
                //判断当前是否为子父级关系
                if (firstMenuVo.getId().equals(menu.getParentId())) {
                    //如果是添加到子级中
                    secondMenuVos.add(menuToMenuVO(menu));
                }
                //递归调用，几级菜单都可以
                // findSubMenu(menus, secondMenuVos);
            }

            if (menus.size() != 0) {
                //递归调用，几级菜单都可以
                findSubMenu(menus, secondMenuVos);
            }
            //将子级放入一级目录
            firstMenuVo.setChildren(secondMenuVos);
        });
    }

    /**
     * 视图与实体类数据转换
     *
     * @param menu
     * @return
     */
    private ModelReferenceCategory menuToMenuVO(ModelReferenceCategory menu) {
        ModelReferenceCategory menuVo = new ModelReferenceCategory();
        BeanUtils.copyProperties(menu, menuVo);
        return menuVo;
    }

    public void selectCategoryChildById(String id, List<Integer> idList) {
        //查询菜单里面子菜单id
        QueryWrapper<ModelReferenceCategory> wrapper = new QueryWrapper<ModelReferenceCategory>();
        wrapper.eq("parent_id", id);
        wrapper.eq("is_del", 0);
        wrapper.select("id");//我们只想查询id  所以只需要查询指定列的值
        List<ModelReferenceCategory> childIdList = baseMapper.selectList(wrapper);//当前菜单的所有子菜单
        //把childIdList里面菜单id值获取出来，封装idList里面，做递归查询
        childIdList.stream().forEach(item -> {//遍历集合得到每一个对象item
            //封装idList里面
            idList.add(item.getId());
            //递归查询
            this.selectCategoryChildById(item.getId() + "", idList);
        });
    }

    @Override
    public List<ModelReferenceCategory> modelClassList(Integer systemId, Integer founder, Integer schoolId) {
        //1.查询出所有分类

        QueryWrapper<ModelReferenceCategory> queryWrapper = new QueryWrapper<ModelReferenceCategory>();
        queryWrapper.eq("is_del", DelConstant.NOT_DEL);
        queryWrapper.eq("system_id", systemId);
        queryWrapper.eq("founder", founder);
        if (founder == ModellConstant.SOURCE_INSTITUTION) {
            //创建来源为院校时候赋值学校id
            queryWrapper.eq("school_id", schoolId);
        }
        List<ModelReferenceCategory> menus = baseMapper.selectList(queryWrapper);

        //2.组装成父子分类
        //2.1找到所有的一级分类（父分类ID为0）
        //获取一级菜单
        List<ModelReferenceCategory> firstMenuVos = menus.stream().filter(menu -> menu.getParentId() == 0).collect(Collectors.toList());
        //查询子级
        findSubMenu(menus, firstMenuVos);
        return firstMenuVos;
    }

    @Override
    public R deleteModelClass(Integer categoryId) {
        //1 创建list集合，用于封装所有删除菜单id值
        List<Integer> idList = new ArrayList<>();
        //2 向idList集合设置删除菜单id
        this.selectCategoryChildById(categoryId + "", idList);//找到当前菜单的子菜单  把结果id封装到idlist里面去
        //把当前id封装到list里面
        idList.add(categoryId);//现在把当前id封装进去  之前都是子菜单的id

        for (Integer id : idList) {
            //删除当前分类以及分类下模型
            referenceDemoMapper.update(new ModelReferenceDemo(), new UpdateWrapper<ModelReferenceDemo>()
                    .set("is_del", DelConstant.IS_DEL).eq("category_id", id));
        }
        Integer retCode = baseMapper.deleteBatchIds(idList);
        return retCode > 0 ? R.ok() : R.error();
    }

    @Override
    public boolean checkRepeat(ModelReferenceCategory modelSysCategory, Integer type) {
        //增加校验 判重 同级下不能重复
        QueryWrapper<ModelReferenceCategory> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("parent_id", modelSysCategory.getParentId());
        queryWrapper.eq("category_name", modelSysCategory.getCategoryName());
        queryWrapper.eq("is_del", DelConstant.NOT_DEL);
        queryWrapper.eq("system_id", modelSysCategory.getSystemId());
        if (modelSysCategory.getFounder() == ModellConstant.SOURCE_INSTITUTION) {
            //来源为院校时候增加学校查询条件
            queryWrapper.eq("school_id", modelSysCategory.getSchoolId());
        }

        switch (type) {
            //区分编辑还是新增判重 如若为编辑判重时候除去本id
            case 1:
                //编辑
                queryWrapper.last(" and id != " + modelSysCategory.getId());
        }

        ModelReferenceCategory category = baseMapper.selectOne(queryWrapper);
        return category != null;
    }

    @Override
    public R checkIsShowBySystemId(Integer systemId) {
        QueryWrapper<ModelReferenceCategory> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("system_id", systemId);
        queryWrapper.eq("level", 0);
        queryWrapper.last(" LIMIT 0,1");
        ModelReferenceCategory category = baseMapper.selectOne(queryWrapper);
        if (category != null) {
            return R.ok().put("isShow", category.getIsShow());
        }
        return R.ok().put("isShow", 1);

    }

    @Override
    public R modifyIsShowState(Integer systemId, Integer isShow) {
        UpdateWrapper<ModelReferenceCategory> wrapper = new UpdateWrapper<>();
        wrapper.set("is_show", isShow);
        wrapper.eq("system_id", systemId);
        wrapper.eq("is_del", DelConstant.NOT_DEL);
        Integer retCode = baseMapper.update(new ModelReferenceCategory(), wrapper);
        return R.ok();
    }

    @Override
    public List<ModelReferenceCategory> builtInClassificationByNakadai(Integer systemId) {
        //1.查询出中台创建的引用模型的所有未删除的分类
        List<ModelReferenceCategory> menus = baseMapper.selectList(new QueryWrapper<ModelReferenceCategory>()
                .eq("is_del", DelConstant.NOT_DEL)
                .eq("founder", ModellConstant.SOURCE_SYSTEM)
                .eq("system_id", systemId));
        //2.组装成父子分类
        //2.1找到所有的一级分类（父分类ID为0）
        //获取一级菜单
        List<ModelReferenceCategory> firstMenuVos = menus.stream().filter(menu -> menu.getParentId() == 0).collect(Collectors.toList());
        //查询子级
        findSubMenu(menus, firstMenuVos);
        return firstMenuVos;
    }
}
