package com.huoran.nakadai.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huoran.common.constant.DelConstant;
import com.huoran.common.constant.ModellConstant;
import com.huoran.common.response.R;
import com.huoran.nakadai.entity.ModelReferenceCategory;
import com.huoran.nakadai.entity.ModelReferenceDemo;
import com.huoran.nakadai.entity.ModelSysCategory;
import com.huoran.nakadai.entity.ModelSysDemo;
import com.huoran.nakadai.entity.req.PageImportModelReq;
import com.huoran.nakadai.entity.req.PageModelReq;
import com.huoran.nakadai.mapper.ModelReferenceCategoryMapper;
import com.huoran.nakadai.mapper.ModelReferenceDemoMapper;
import com.huoran.nakadai.mapper.ModelSysCategoryMapper;
import com.huoran.nakadai.mapper.ModelSysDemoMapper;
import com.huoran.nakadai.service.ModelReferenceDemoService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Queue;

/**
 * <p>
 * 引用模型代码记录表 服务实现类
 * </p>
 *
 * @author chen
 * @since 2022-03-04
 */
@Service
public class ModelReferenceDemoServiceImpl extends ServiceImpl<ModelReferenceDemoMapper, ModelReferenceDemo> implements ModelReferenceDemoService {

    @Autowired
    private ModelSysCategoryMapper sysCategoryMapper;

    @Autowired
    private ModelSysDemoMapper sysDemoMapper;

    @Autowired
    private ModelReferenceCategoryMapper referenceCategoryMapper;

    @Override
    public R referenceDemoList(PageModelReq req) {
        Page<ModelReferenceDemo> page = new Page<ModelReferenceDemo>(req.getPageNum(), req.getPageSize());
        IPage<ModelReferenceDemo> pageList = baseMapper.referenceDemoList(page, req);
        return R.ok().put("data", pageList);
    }

    @Override
    public R sortReadingModelByTeacherSideSystem(PageModelReq req) {
        Page<ModelReferenceDemo> page = new Page<ModelReferenceDemo>(req.getPageNum(), req.getPageSize());
        IPage<ModelReferenceDemo> pageList = baseMapper.sortReadingModelByTeacherSideSystem(page, req);
        return R.ok().put("data", pageList);
    }

    /**
     * @Description :同步原始模型前删除原有数据
     * @Param systemId
     * @Author Rong---2022/3/7
     */
    @Override
    public R delModelInfoBySystemId(Integer systemId, Integer founder, Integer schoolId) {
        //同步前删除原有系统的数据
        UpdateWrapper<ModelReferenceCategory> updateWrapper = new UpdateWrapper<>();
        updateWrapper.set("is_del", DelConstant.IS_DEL);
        updateWrapper.eq("system_id", systemId);
        updateWrapper.eq("founder", founder);
        if (founder == ModellConstant.SOURCE_INSTITUTION && schoolId != null) {
            //创建来源为院校时候赋值学校id
            updateWrapper.eq("school_id", schoolId);
        }
        Integer del = referenceCategoryMapper.update(new ModelReferenceCategory(), updateWrapper);

        //同时将当前删除分类下的模型一并删除
        QueryWrapper<ModelReferenceCategory> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_del", DelConstant.IS_DEL);
        queryWrapper.eq("system_id", systemId);
        queryWrapper.eq("founder", founder);
        if (founder == ModellConstant.SOURCE_INSTITUTION && schoolId != null) {
            //创建来源为院校时候赋值学校id
            queryWrapper.eq("school_id", schoolId);
        }

        List<ModelReferenceCategory> list = referenceCategoryMapper.selectList(queryWrapper);
        for (ModelReferenceCategory modelSysCategory : list) {
            UpdateWrapper<ModelReferenceDemo> updateWrapper1 = new UpdateWrapper<>();
            updateWrapper1.set("is_del", DelConstant.IS_DEL);
            updateWrapper1.eq("category_id", modelSysCategory.getId());
            updateWrapper1.eq("system_id", systemId);
            updateWrapper1.eq("founder", founder);
            if (founder == ModellConstant.SOURCE_INSTITUTION && schoolId != null) {
                //创建来源为院校时候赋值学校id
                updateWrapper1.eq("school_id", schoolId);
            }
            baseMapper.update(new ModelReferenceDemo(), updateWrapper1);
        }
        return del != -1 ? R.ok() : R.error();
    }

    @Override
    public R getAllModelList(PageImportModelReq req) {
        Page<ModelReferenceDemo> page = new Page<ModelReferenceDemo>(req.getPageNum(), req.getPageSize());
        IPage<ModelReferenceDemo> pageList = baseMapper.getAllModelList(page, req);
        return R.ok().put("data", pageList);
    }

    @Override
    public R teacherModelList(PageImportModelReq req) {
        Page<ModelReferenceDemo> page = new Page<ModelReferenceDemo>(req.getPageNum(), req.getPageSize());
        IPage<ModelReferenceDemo> pageList = baseMapper.teacherModelList(page, req);
        return R.ok().put("data", pageList);
    }

    /**
     * @Description :  根据系统id同步数据
     * @Param systemId
     * @Param accountId
     * @Author Rong---2022/3/11
     */
    @Override
    public R synchronizationMdel(Integer systemId, Integer accountId) {
        boolean ret = addCategory(0, 0, systemId, accountId);
        return ret ? R.ok() : R.error();

    }

    @Override
    public R synchronizationMdelByOccupationlab(Integer systemId, Integer schoolId, Integer accountId) {
        boolean ret = addCategoryByOccupationlab(0, 0, systemId, schoolId, accountId);

        QueryWrapper<ModelReferenceCategory> queryWrapper1 = new QueryWrapper<ModelReferenceCategory>();
        queryWrapper1.eq("parent_id", 0);
        queryWrapper1.eq("founder", 1);
        queryWrapper1.eq("school_id", schoolId);
        queryWrapper1.eq("system_id", systemId);
        queryWrapper1.eq("is_del", DelConstant.NOT_DEL);

        //获取学校分类父类id
        ModelReferenceCategory getsTheSchoolParentClassId = referenceCategoryMapper.selectOne(queryWrapper1);
        return ret ? R.ok().put("id", getsTheSchoolParentClassId.getId()) : R.error();
    }

    @Override
    public R referenceDemoListByNakadai(PageModelReq req) {
        Page<ModelReferenceDemo> page = new Page<ModelReferenceDemo>(req.getPageNum(), req.getPageSize());
        IPage<ModelReferenceDemo> pageList = baseMapper.referenceDemoListByNakadai(page, req);
        return R.ok().put("data", pageList);
    }

    @Override
    public R studentModelList(PageImportModelReq req) {
        Page<ModelReferenceDemo> page = new Page<ModelReferenceDemo>(req.getPageNum(), req.getPageSize());
        IPage<ModelReferenceDemo> pageList = baseMapper.studentModelList(page, req);
        return R.ok().put("data", pageList);
    }

    @Override
    public R systemModelByTeacher(PageImportModelReq req) {
        Page<ModelReferenceDemo> page = new Page<ModelReferenceDemo>(req.getPageNum(), req.getPageSize());
        IPage<ModelReferenceDemo> pageList = baseMapper.systemModelByTeacher(page, req);
        return R.ok().put("data", pageList);
    }

   /* public boolean addCategoryByOccupationlab(Integer fid, Integer incrementId, Integer systemId, Integer schoolId, Integer accountId) {

        //查询中台的内置数据同步到职站引用模型表
        QueryWrapper<ModelSysCategory> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("parent_id", fid);
        queryWrapper1.eq("is_open", DelConstant.IS_OPEN);
        queryWrapper1.eq("is_del", DelConstant.NOT_DEL);
        queryWrapper1.eq("founder", 0);

        List<ModelSysCategory> list = sysCategoryMapper.selectList(queryWrapper1);//查询源表 父类id为fid的数据

        if (list.size() > 0) {
            for (ModelSysCategory modelSysCategory : list) {
                //将modelSysCategory添加进引用分类表中
                ModelReferenceCategory referenceCategory = new ModelReferenceCategory();
                referenceCategory.setCategoryName(modelSysCategory.getCategoryName());
                referenceCategory.setAccountId(accountId);
                referenceCategory.setSystemId(systemId);
                referenceCategory.setParentId(incrementId);
                referenceCategory.setFounder(1);
                referenceCategory.setSchoolId(schoolId);
                referenceCategory.setLevel(modelSysCategory.getLevel());
                referenceCategoryMapper.insert(referenceCategory);
                Integer getId = referenceCategory.getId();//拿到当前自增id

                //根据getId查询源模型代码表
                QueryWrapper<ModelSysDemo> queryWrapper2 = new QueryWrapper<>();
                queryWrapper2.eq("category_id", modelSysCategory.getId());
                queryWrapper2.eq("is_open", DelConstant.IS_OPEN);
                queryWrapper2.eq("is_del", DelConstant.NOT_DEL);
                queryWrapper2.eq("post_status", DelConstant.POST_STATUS);
                //查询已发布的
                queryWrapper2.eq("post_status", ModellConstant.PUBLICATION_STATUS_PUBLISHED);
                List<ModelSysDemo> demoList = sysDemoMapper.selectList(queryWrapper2);//查询源模型代码

                if (demoList.size() > 0) {
                    for (ModelSysDemo demo : demoList) {
                        ModelReferenceDemo referenceDemo = new ModelReferenceDemo();
                        referenceDemo.setCategoryId(getId);
                        referenceDemo.setAccountId(accountId);
                        referenceDemo.setCopyId(demo.getId());
                        referenceDemo.setSystemId(systemId);
                        referenceDemo.setFounder(1);
                        referenceDemo.setModifyAccountId(accountId);
                        referenceDemo.setSchoolId(schoolId);
                        baseMapper.insert(referenceDemo);
                    }
                }
                addCategoryByOccupationlab(modelSysCategory.getId(), getId, systemId, schoolId, accountId);
            }
            return true;
        }
        return false;
    }*/


    /**
     * @Description :同步职站源模型至模型列表
     * @Param fid 父id
     * @Param incrementId 自增id
     * @Param systemId 系统id
     * @Param accountId 账号id
     * @Author Rong---2022/3/11
     */
    public boolean addCategoryByOccupationlab(Integer fid, Integer incrementId, Integer systemId, Integer schoolId, Integer accountId) {

        QueryWrapper<ModelSysCategory> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("parent_id", fid);
        queryWrapper1.eq("is_open", DelConstant.IS_OPEN);
        queryWrapper1.eq("is_del", DelConstant.NOT_DEL);
        queryWrapper1.eq("founder", 1);
        queryWrapper1.eq("school_id", schoolId);
        List<ModelSysCategory> list = sysCategoryMapper.selectList(queryWrapper1);//查询源表 父类id为fid的数据

        if (list.size() > 0) {
            for (ModelSysCategory modelSysCategory : list) {
                //将modelSysCategory添加进引用分类表中
                ModelReferenceCategory referenceCategory = new ModelReferenceCategory();
                referenceCategory.setCategoryName(modelSysCategory.getCategoryName());
                referenceCategory.setAccountId(accountId);
                referenceCategory.setSystemId(systemId);
                referenceCategory.setParentId(incrementId);
                referenceCategory.setFounder(ModellConstant.SOURCE_INSTITUTION);
                referenceCategory.setSchoolId(schoolId);
                referenceCategory.setLevel(modelSysCategory.getLevel());
                referenceCategoryMapper.insert(referenceCategory);
                Integer getId = referenceCategory.getId();//拿到当前自增id
                //根据getId查询查询已发布的源模型代码表
                QueryWrapper<ModelSysDemo> queryWrapper2 = new QueryWrapper<>();
                queryWrapper2.eq("category_id", modelSysCategory.getId());
                queryWrapper2.eq("is_open", DelConstant.IS_OPEN);
                queryWrapper2.eq("zt_open", DelConstant.IS_OPEN);
                queryWrapper2.eq("is_del", DelConstant.NOT_DEL);
                queryWrapper2.eq("zt_del", DelConstant.NOT_DEL);
                queryWrapper2.eq("founder", ModellConstant.SOURCE_INSTITUTION);
                queryWrapper2.eq("school_id", schoolId);
                queryWrapper2.eq("post_status", ModellConstant.PUBLICATION_STATUS_PUBLISHED);
                //查询源模型代码
                List<ModelSysDemo> demoList = sysDemoMapper.selectList(queryWrapper2);

                if (demoList.size() > 0) {
                    for (ModelSysDemo demo : demoList) {
                        ModelReferenceDemo referenceDemo = new ModelReferenceDemo();
                        referenceDemo.setCategoryId(getId);
                        referenceDemo.setAccountId(accountId);
                        referenceDemo.setCopyId(demo.getId());
                        referenceDemo.setSystemId(systemId);
                        referenceDemo.setFounder(ModellConstant.SOURCE_INSTITUTION);
                        referenceDemo.setModifyAccountId(accountId);
                        referenceDemo.setSchoolId(schoolId);
                        baseMapper.insert(referenceDemo);
                    }
                }
                addCategoryByOccupationlab(modelSysCategory.getId(), getId, systemId,schoolId, accountId);
            }
            return true;
        }
        return false;
    }


    /**
     * @Description :同步原始模型列表
     * @Param fid 父id
     * @Param incrementId 自增id
     * @Param systemId 系统id
     * @Param accountId 账号id
     * @Author Rong---2022/3/11
     */
    public boolean addCategory(Integer fid, Integer incrementId, Integer systemId, Integer accountId) {

        QueryWrapper<ModelSysCategory> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("parent_id", fid);
        queryWrapper1.eq("is_open", DelConstant.IS_OPEN);
        queryWrapper1.eq("is_del", DelConstant.NOT_DEL);
        queryWrapper1.eq("founder", 0);

        List<ModelSysCategory> list = sysCategoryMapper.selectList(queryWrapper1);//查询源表 父类id为fid的数据

        if (list.size() > 0) {
            for (ModelSysCategory modelSysCategory : list) {
                //将modelSysCategory添加进引用分类表中
                ModelReferenceCategory referenceCategory = new ModelReferenceCategory();
                referenceCategory.setCategoryName(modelSysCategory.getCategoryName());
                referenceCategory.setAccountId(accountId);
                referenceCategory.setSystemId(systemId);
                referenceCategory.setParentId(incrementId);
                referenceCategory.setFounder(0);
                referenceCategory.setSchoolId(0);
                referenceCategory.setLevel(modelSysCategory.getLevel());
                referenceCategoryMapper.insert(referenceCategory);
                Integer getId = referenceCategory.getId();//拿到当前自增id

                //根据getId查询源模型代码表
                QueryWrapper<ModelSysDemo> queryWrapper2 = new QueryWrapper<>();
                queryWrapper2.eq("category_id", modelSysCategory.getId());
                queryWrapper2.eq("is_open", DelConstant.IS_OPEN);
                queryWrapper2.eq("is_del", DelConstant.NOT_DEL);
                queryWrapper2.eq("post_status", DelConstant.POST_STATUS);
                //查询已发布的
                queryWrapper2.eq("post_status", ModellConstant.PUBLICATION_STATUS_PUBLISHED);
                List<ModelSysDemo> demoList = sysDemoMapper.selectList(queryWrapper2);//查询源模型代码

                if (demoList.size() > 0) {
                    for (ModelSysDemo demo : demoList) {
                        ModelReferenceDemo referenceDemo = new ModelReferenceDemo();
                        referenceDemo.setCategoryId(getId);
                        referenceDemo.setAccountId(accountId);
                        referenceDemo.setCopyId(demo.getId());
                        referenceDemo.setSystemId(systemId);
                        referenceDemo.setFounder(0);
                        referenceDemo.setModifyAccountId(accountId);
                        referenceDemo.setSchoolId(0);
                        baseMapper.insert(referenceDemo);
                    }
                }
                addCategory(modelSysCategory.getId(), getId, systemId, accountId);
            }
            return true;
        }
        return false;
    }


}
