package com.huoran.nakadai.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.huoran.common.constant.DelConstant;
import com.huoran.common.constant.ModellConstant;
import com.huoran.common.response.R;
import com.huoran.nakadai.entity.ModelReferenceCategory;
import com.huoran.nakadai.entity.ModelSysCategory;
import com.huoran.nakadai.entity.ModelSysDemo;
import com.huoran.nakadai.entity.req.ModelSysDemoReq;
import com.huoran.nakadai.mapper.ModelSysCategoryMapper;
import com.huoran.nakadai.mapper.ModelSysDemoMapper;
import com.huoran.nakadai.service.ModelSysCategoryService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 源模型-分类表 服务实现类
 * </p>
 *
 * @author chen
 * @since 2022-03-03
 */
@Service
public class ModelSysCategoryServiceImpl extends ServiceImpl<ModelSysCategoryMapper, ModelSysCategory> implements ModelSysCategoryService {

    @Autowired
    private ModelSysCategoryMapper mapper;

    @Autowired
    private ModelSysDemoMapper sysDemoMapper;

    /**
     * 视图与实体类数据转换
     *
     * @param menu
     * @return
     */
    private ModelSysCategory menuToMenuVO(ModelSysCategory menu) {
        ModelSysCategory menuVo = new ModelSysCategory();
        BeanUtils.copyProperties(menu, menuVo);
        return menuVo;
    }

    /**
     * 递归查询子级
     *
     * @param menus        查询的所有菜单
     * @param firstMenuVos 第一级的菜单
     */
    private void findSubMenu(List<ModelSysCategory> menus, List<ModelSysCategory> firstMenuVos) {
        //遍历第一级
        firstMenuVos.forEach(firstMenuVo -> {
            //子级菜单集合
            List<ModelSysCategory> secondMenuVos = new LinkedList<>();
            //查找子级  递归的出口就是循环完毕
            for (ModelSysCategory menu : menus) {
                //判断当前是否为子父级关系
                if (firstMenuVo.getId().equals(menu.getParentId())) {
                    //如果是添加到子级中
                    secondMenuVos.add(menuToMenuVO(menu));
                }

            }

            if (menus.size() != 0) {
                //递归调用，几级菜单都可以
                findSubMenu(menus, secondMenuVos);
            }
            //将子级放入一级目录
            firstMenuVo.setChildren(secondMenuVos);
        });
    }

    @Override
    public List<ModelSysCategory> sourceModelClassification(ModelSysDemoReq req) {
        QueryWrapper<ModelSysCategory> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_del", 0);
        queryWrapper.eq("founder", req.getFounder());
        if (req.getFounder() == ModellConstant.SOURCE_INSTITUTION) {
            //查询院校
            queryWrapper.eq("school_id", req.getSchoolId());
        }


        //1.查询出所有分类
        List<ModelSysCategory> menus = mapper.selectList(queryWrapper);
        //2.组装成父子分类
        //2.1找到所有的一级分类（父分类ID为0）
        //获取一级菜单
        List<ModelSysCategory> firstMenuVos = menus.stream().filter(menu -> menu.getParentId() == 0).collect(Collectors.toList());
        //查询子级
        findSubMenu(menus, firstMenuVos);
        return firstMenuVos;
    }

    public void selectCategoryChildById(String id, List<Integer> idList) {
        //查询菜单里面子菜单id
        QueryWrapper<ModelSysCategory> wrapper = new QueryWrapper<ModelSysCategory>();
        wrapper.eq("parent_id", id);
        wrapper.eq("is_del", 0);
        wrapper.select("id");//我们只想查询id  所以只需要查询指定列的值
        List<ModelSysCategory> childIdList = baseMapper.selectList(wrapper);//当前菜单的所有子菜单
        //把childIdList里面菜单id值获取出来，封装idList里面，做递归查询
        childIdList.stream().forEach(item -> {//遍历集合得到每一个对象item
            //封装idList里面
            idList.add(item.getId());
            //递归查询
            this.selectCategoryChildById(item.getId() + "", idList);
        });
    }

    @Override
    public R deleteSourceModelCategory(Integer categoryId) {
        //1 创建list集合，用于封装所有删除菜单id值
        List<Integer> idList = new ArrayList<>();
        //2 向idList集合设置删除菜单id
        this.selectCategoryChildById(categoryId + "", idList);//找到当前菜单的子菜单  把结果id封装到idlist里面去
        //把当前id封装到list里面
        idList.add(categoryId);//现在把当前id封装进去  之前都是子菜单的id

        for (Integer id : idList) {
            //删除当前分类以及分类下模型
            sysDemoMapper.update(new ModelSysDemo(), new UpdateWrapper<ModelSysDemo>()
                    .set("is_del", DelConstant.IS_DEL).eq("category_id", id));
        }
        Integer retCode = baseMapper.deleteBatchIds(idList);
        return retCode > 0 ? R.ok() : R.error();
    }

    @Override
    public boolean checkRepeat(ModelSysCategory modelSysCategory, Integer type) {
        //增加校验 判重 同级下不能重复
        QueryWrapper<ModelSysCategory> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("category_name", modelSysCategory.getCategoryName());
        queryWrapper.eq("parent_id", modelSysCategory.getParentId());
        queryWrapper.eq("is_del", DelConstant.NOT_DEL);
        queryWrapper.eq("founder", modelSysCategory.getFounder());
        if (modelSysCategory.getFounder() == ModellConstant.SOURCE_INSTITUTION) {
            //来源为院校时候增加学校查询条件
            queryWrapper.eq("school_id", modelSysCategory.getSchoolId());
        }

        switch (type) {
            //区分编辑还是新增判重 如若为编辑判重时候除去本id
            case 1:
                //编辑
                queryWrapper.last(" and id != " + modelSysCategory.getId());
        }

        ModelSysCategory category = baseMapper.selectOne(queryWrapper);
        return category != null;
    }

    @Override
    public List<ModelSysCategory> primaryClassification() {
        return baseMapper.primaryClassification();
    }

    @Override
    public List<ModelSysCategory> classificationByPid(Integer parentId) {
        QueryWrapper<ModelSysCategory> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("parent_id", parentId);
        queryWrapper.eq("is_del", 0);
        return baseMapper.selectList(queryWrapper);
    }

}
