package com.huoran.nakadai.service.impl;

import com.aliyun.oss.ClientException;
import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.vod.model.v20170321.DeleteVideoRequest;
import com.aliyuncs.vod.model.v20170321.DeleteVideoResponse;
import com.aliyuncs.vod.model.v20170321.GetVideoPlayAuthRequest;
import com.aliyuncs.vod.model.v20170321.GetVideoPlayAuthResponse;
import com.huoran.common.entity.FilesResult;
import com.huoran.common.exception.CustomException;
import com.huoran.common.exception.ExceptionEnum;
import com.huoran.common.utils.AliyunOssUtil;
import com.huoran.common.utils.AliyunVodSDKUtil;
import com.huoran.common.utils.ConstantPropertiesUtil;
import com.huoran.nakadai.service.OSSFileService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.util.List;

/**
 * @Author chen
 * @DATE 2020/10/1 22:01
 * @Version 1.0
 */
@Slf4j
@Service
public class OSSFileServiceImpl implements OSSFileService {

    @Override
    public FilesResult upload(MultipartFile file) throws IOException {
        //判断文件是否为视频
/*        String fileName = file.getOriginalFilename();
        assert fileName != null;
        String suffix = fileName.substring(fileName.lastIndexOf(".") + 1);
        boolean isVideo = VideoUtil.isVideo(suffix);
        if (isVideo) {
            return vodUpload(file);
        }*/
        return fileUpload(file);
    }

    /**
     * 视频上传
     *
     * @param file 视频文件
     * @return 返回文件信息
     * @throws IOException IO异常
     */
    public FilesResult vodUpload(MultipartFile file) throws IOException {
        FilesResult filesResult = new FilesResult();
        //获取文件流
        InputStream inputStream = file.getInputStream();
        String originalFilename = file.getOriginalFilename();
        String fileSize = String.valueOf(file.getSize());
        assert originalFilename != null;
        String title = originalFilename.substring(0, originalFilename.lastIndexOf("."));
        // 获取文件尾缀
        String fileType = originalFilename.substring(originalFilename.lastIndexOf(".") + 1);
        // 文件类型
        filesResult.setFileType(fileType);
        //文件大小
        filesResult.setFileSize(fileSize);

        //上传流请求
//        UploadStreamRequest request = new UploadStreamRequest(ConstantPropertiesUtil.ACCESS_KEY_ID,
//                ConstantPropertiesUtil.ACCESS_KEY_SECRET, title, originalFilename, inputStream);

        /* 是否使用默认水印(可选)，指定模板组ID时，根据模板组配置确定是否使用默认水印*/
        //request.setShowWaterMark(true);
        /* 设置上传完成后的回调URL(可选)，建议通过点播控制台配置消息监听事件，参见文档 https://help.aliyun.com/document_detail/57029.html */
        //request.setCallback("http://callback.sample.com");
        /* 自定义消息回调设置，参数说明参考文档 https://help.aliyun.com/document_detail/86952.html#UserData */
        //request.setUserData(""{\"Extend\":{\"test\":\"www\",\"localId\":\"xxxx\"},\"MessageCallback\":{\"CallbackURL\":\"http://test.test.com\"}}"");
        /* 视频分类ID(可选) */
        //request.setCateId(0);
        /* 视频标签,多个用逗号分隔(可选) */
        //request.setTags("标签1,标签2");
        /* 视频描述(可选) */
        //request.setDescription("视频描述");
        /* 封面图片(可选) */
        //request.setCoverURL("http://cover.sample.com/sample.jpg");
        /* 模板组ID(可选) */
        //request.setTemplateGroupId("8c4792cbc8694e7084fd5330e56a33d");
        /* 工作流ID(可选) */
        //request.setWorkflowId("d4430d07361f0*be1339577859b0177b");
        /* 存储区域(可选) */
        //request.setStorageLocation("in-201703232118266-5sejdln9o.oss-cn-shanghai.aliyuncs.com");
        /* 开启默认上传进度回调 */
        // request.setPrintProgress(true);
        /* 设置自定义上传进度回调 (必须继承 VoDProgressListener) */
        // request.setProgressListener(new PutObjectProgressListener());
        /* 设置应用ID*/
        //request.setAppId("app-1000000");
        /* 点播服务接入点 */
        //request.setApiRegionId("cn-shanghai");
        /* ECS部署区域*/
        // request.setEcsRegionId("cn-shanghai");

//        UploadVideoImpl uploader = new UploadVideoImpl();
//        UploadStreamResponse response = uploader.uploadStream(request);
//
//        //如果设置回调URL无效，不影响视频上传，可以返回VideoId同时会返回错误码。
//        // 其他情况上传失败时，VideoId为空，此时需要根据返回错误码分析具体错误原因
//        String videoId = response.getVideoId();
//        if (!response.isSuccess()) {
//            String errorMessage = "阿里云视频上传错误：" + "code：" + response.getCode() + ", message：" + response.getMessage();
//            log.error(errorMessage);
//            if (StringUtils.isEmpty(videoId)) {
//                throw new CustomException(ExceptionEnum.UNKNOWN_EXCEPTION);
//            }
//        }
//        filesResult.setFileId(videoId);
        return filesResult;
    }

    /**
     * 文件删除
     *
     * @param keys 完整文件名
     * @return 返回成功删除文件名
     */
    @Override
    public List<String> fileDeletion(List<String> keys) {
        return AliyunOssUtil.deleteFiles(keys);
    }

    @Override
    public String getPlayAuth(String videoId) throws ClientException, com.aliyuncs.exceptions.ClientException {
        DefaultAcsClient client = AliyunVodSDKUtil.initVodClient(ConstantPropertiesUtil.ACCESS_KEY_ID, ConstantPropertiesUtil.ACCESS_KEY_SECRET);
        GetVideoPlayAuthRequest request = new GetVideoPlayAuthRequest();
        request.setVideoId(videoId);
        GetVideoPlayAuthResponse response = client.getAcsResponse(request);
        //得到播放凭证并返回
        return response.getPlayAuth();
    }

    @Override
    public void removeVideo(String videoId) throws ClientException, com.aliyuncs.exceptions.ClientException {
        DefaultAcsClient client = AliyunVodSDKUtil.initVodClient(
                ConstantPropertiesUtil.ACCESS_KEY_ID,
                ConstantPropertiesUtil.ACCESS_KEY_SECRET);

        DeleteVideoRequest request = new DeleteVideoRequest();
        request.setVideoIds(videoId);
        DeleteVideoResponse response = client.getAcsResponse(request);
        System.out.print("RequestId = " + response.getRequestId() + "\n");
    }

    /**
     * 文件上传
     *
     * @param file 上传文件
     * @return 返回文件信息
     * @throws IOException IO异常
     */
    @Override
    public FilesResult fileUpload(MultipartFile file) throws IOException {
        return AliyunOssUtil.uploadFile(file);
    }

}
