package com.huoran.nakadai.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.huoran.api.OccupationlabClient;
import com.huoran.common.constant.OrderConstant;
import com.huoran.common.entity.to.JudgmentPointDataTo;
import com.huoran.common.entity.to.ProjectDataTo;
import com.huoran.common.exception.CustomException;
import com.huoran.common.exception.ExceptionEnum;
import com.huoran.nakadai.entity.CurriculumConfigure;
import com.huoran.nakadai.entity.Customer;
import com.huoran.nakadai.entity.Order;
import com.huoran.nakadai.entity.OrderOther;
import com.huoran.nakadai.entity.req.BulkShipOrdersReq;
import com.huoran.nakadai.entity.req.QueryTimeReq;
import com.huoran.nakadai.entity.vo.OrderRenewVo;
import com.huoran.nakadai.mapper.*;
import com.huoran.nakadai.service.OrderOtherService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * <p>
 * 订单其他信息（包含课程权限、数据平台权限、合同信息等） 服务实现类
 * </p>
 *
 * @author cheney
 * @since 2021-07-01
 */
@Service
public class OrderOtherServiceImpl extends ServiceImpl<OrderOtherMapper, OrderOther> implements OrderOtherService {

    @Autowired
    private OrderMapper orderMapper;

    @Autowired
    private CustomerMapper customerMapper;

    @Autowired
    private OccupationlabClient occupationlabClient;

    @Autowired
    private CurriculumConfigureMapper configureMapper;

    @Autowired
    private DataProductMapper dataProductMapper;

    @Autowired
    private CurriculumMapper curriculumMapper;

    /**
     * 根据订单内容id修改货物发货状态及订单状态，同步修改起始时间
     * @return boolean
     */
    @Override
    public boolean ship(OrderOther orderOther) {
        int update = baseMapper.updateById(orderOther);
        //查询订单其他产品状态，都为发货则订单已完成
        QueryWrapper<OrderOther> otherQueryWrapper = new QueryWrapper<>();
        otherQueryWrapper.eq("order_id", orderOther.getOrderId());
        List<OrderOther> orderOtherList = baseMapper.selectList(otherQueryWrapper);
        List<Integer> ship = new ArrayList<>();
        orderOtherList.forEach(other -> {
            ship.add(other.getShip());
        });
        boolean contains = ship.contains(0);
//        if (!contains) {
//            Order order = new Order();
//            order.setOrderId(orderOther.getOrderId());
//            order.setOrderStatus(OrderConstant.SHIPPED);
//            orderMapper.updateById(order);
//        }
        return update > 0;
    }

    @Override
    public boolean bulkShipping(BulkShipOrdersReq bulkShipOrdersReq) {
        List<OrderOther> orderOthers = bulkShipOrdersReq.getOrderOthers();
        for (OrderOther orderOther : orderOthers) {
            baseMapper.updateById(orderOther);
            //查询订单其他产品状态，都为发货则订单已完成
            QueryWrapper<OrderOther> otherQueryWrapper = new QueryWrapper<>();
            otherQueryWrapper.eq("order_id", orderOther.getOrderId());
            List<OrderOther> orderOtherList = baseMapper.selectList(otherQueryWrapper);
            List<Integer> ship = new ArrayList<>();
            orderOtherList.forEach(other -> {
                ship.add(other.getShip());
            });
            boolean contains = ship.contains(0);
//            if (!contains) {
//                Order order = new Order();
//                order.setOrderId(orderOther.getOrderId());
//                order.setOrderStatus(OrderConstant.SHIPPED);
//                orderMapper.updateById(order);
//            }
        }
        return true;
    }

    /**
     * 根据订单内容id修改货物启用状态
     *
     * @param id   内容id
     * @param flag 标识
     * @return boolean
     */
    @Override
    public boolean enable(Integer id, Integer flag) {
        OrderOther orderOther = new OrderOther();
        orderOther.setId(id);
        orderOther.setIsEnable(flag);
        int update = baseMapper.updateById(orderOther);
        return update > 0;
    }

    /**
     * 根据起止日期以及当前日期设置剩余期限
     */
    @Override
    public void remainingPeriod() {
        // List<OrderOtherResp> orderOtherList = baseMapper.remainingDay();
        //查询所有订单产品
        List<OrderOther> orderOtherList = baseMapper.selectList(null);
        orderOtherList.forEach(orderOther -> {
            if (orderOther.getShip().equals(OrderConstant.SHIPPED)) {
                if (orderOther.getRemainingPeriod() > 0) {
                    //用于将负数全部修正，判断隔天可以删除
                    if (orderOther.getRemainingPeriod() < 0) {
                        orderOther.setRemainingPeriod(0);
                    }
                    //修改订单剩余期限，当前时间在产品时间内进行扣除
                    long start = orderOther.getStartTime().getTime();
                    long now = DateUtil.date().getTime();
                    long end = orderOther.getEndTime().getTime();
                    if (now >= start && now <= end){
                        orderOther.setRemainingPeriod(orderOther.getRemainingPeriod()-1);
                        baseMapper.updateById(orderOther);
                    }
                }else {
                    //关闭产品
                    orderOther.setIsEnable(0);
                    baseMapper.updateById(orderOther);
                }
            }
        });
    }

    @Override
    public void customerExpiration() {
        //查询到期订单
        List<OrderOther> orderOtherList = baseMapper.selectList(new QueryWrapper<OrderOther>()
                .eq("remaining_period", 0));
        if (ObjectUtil.isNotNull(orderOtherList) && orderOtherList.size()>0){
            orderOtherList.forEach(orderOther -> {
                //产品到期、查询客户所有订单产品是否都到期
//                if (orderOther.getRemainingPeriod()==0){
//                    Integer orderId = orderOther.getOrderId();
//                    Order order = orderMapper.selectById(orderId);
//                    List<Order> customerList = orderMapper.selectList(new QueryWrapper<Order>().
//                            eq("customer_id", order.getCustomerId()));
//                    List<Integer> type = new ArrayList<>();
//                    // 订单类型(1、正式 2、试用)
//                    for (Order orderInfo : customerList){
//                        type.add(orderInfo.getOrderType());
//                    }
//                    List<OrderOther> allOrderOther = new ArrayList<>();
//                    customerList.forEach(customerOrder -> {
//                        List<OrderOther> other = baseMapper.selectList(new QueryWrapper<OrderOther>()
//                                .eq("order_id", customerOrder.getOrderId()));
//                        if (ObjectUtil.isNotNull(other) && other.size()>0){
//                            allOrderOther.addAll(other);
//                        }
//                    });
//                    List<Integer> remainingPeriod = new ArrayList<>();
//                    allOrderOther.forEach(customerOrderOther -> {
//                        remainingPeriod.add(customerOrderOther.getRemainingPeriod());
//                    });
//                    Customer customer = customerMapper.selectById(order.getCustomerId());
//                    HashSet<Integer> set = new HashSet<>(remainingPeriod);
//                    //订单产品都过期，更新客户类型为到期
//                    if (set.size()==1&&set.contains(0)){
//                        customer.setCustomerType(3);
//                    }else {
//                        //有正式订单
//                        boolean contains = type.contains(1);
//                        if (contains) {
//                            customer.setCustomerType(1);
//                        }else {
//                            customer.setCustomerType(2);
//                        }
//                    }
//                    //更新客户类型
//                    customerMapper.updateById(customer);
//                }
            });
        }
    }

    @Override
    public ArrayList<OrderOther> renew(OrderRenewVo orderRenewVo) {

        Integer customerId = orderRenewVo.getCustomerId();
        Integer authority = orderRenewVo.getAuthority();
        List<Integer> productList = orderRenewVo.getProductId();
        HashMap<String, OrderOther> map = Maps.newHashMap();
        ArrayList<OrderOther> orderOthers = Lists.newArrayList();
        productList.forEach(id -> {
            //查询还未过期有此产品的订单
            QueryWrapper<OrderOther> otherQueryWrapper = new QueryWrapper<>();
            otherQueryWrapper.eq("mall_id", id).
                    eq("authority",authority).
                    ge("end_time", new Date());
            List<OrderOther> orderOtherList = baseMapper.selectList(otherQueryWrapper);
            if (!orderOtherList.isEmpty()) {
                orderOtherList.forEach(orderOther -> {
                    //查询客户产品的最新订单
                    QueryWrapper<Order> wrapper = new QueryWrapper<>();
                    wrapper.eq("customer_id", customerId).
                            eq("order_id", orderOther.getOrderId()).
                            orderByDesc("create_time").last("limit 1");
//                    Order order = orderMapper.selectOne(wrapper);
                    if (ObjectUtil.isNotNull(null)) {
                        /*//课程产品
                        if (orderOther.getAuthority() == 1) {
                            Curriculum curriculum = curriculumMapper.courseListDeletedNotDeleted(orderOther.getDataOrCourseId());
                            orderOther.setProductType(curriculum.getCurriculumType());
                            orderOther.setMiniProgramPictureAddress(curriculum.getMiniProgramPictureAddress());
                            orderOther.setOriginalSettlementPrice(curriculum.getSettlementPrice());
                            orderOther.setSettlementMethod(curriculum.getSettlementMethod());
                            orderOther.setSupplier(curriculum.getSupplier());
                            orderOther.setBusinessProportion(curriculum.getBusinessProportion());
                            orderOther.setManufacturerProportion(curriculum.getManufacturerProportion());
                            orderOther.setMarketPrice(curriculum.getMarketPrice());
                            orderOther.setMarketValue(curriculum.getMarketPrice());
                        }else if (orderOther.getAuthority() == 0){
                            //数据产品
                            DataProduct dataProduct = dataProductMapper.selectById(orderOther.getDataOrCourseId());
                            orderOther.setOriginalSettlementPrice(dataProduct.getSettlementPrice());
                            orderOther.setProductType(3);
                            orderOther.setSettlementMethod(dataProduct.getSettlementMethod());
                            orderOther.setSupplier(dataProduct.getSupplier());
                            orderOther.setBusinessProportion(dataProduct.getBusinessProportion());
                            orderOther.setManufacturerProportion(dataProduct.getManufacturerProportion());
                            orderOther.setMarketPrice(new BigDecimal(dataProduct.getMarket()));
                            orderOther.setMarketValue(new BigDecimal(dataProduct.getMarket()));
                        }else if (orderOther.getAuthority() == 2){

                        }*/
                        map.put("" + id + "", orderOther);
                    }
                });
            }
        });
        orderOthers.addAll(map.values());
        return orderOthers;
    }

    @Override
    public JudgmentPointDataTo getCustomerOrderOther(Integer schoolId,Integer systemId,String name) {
        //查询客户id
        Customer customer = customerMapper.selectOne(new QueryWrapper<Customer>().eq("school_id", schoolId));
        //查询客户订单
//        List<Order> orderList = orderMapper.selectList(new QueryWrapper<Order>().eq("customer_id", customer.getCustomerId()));
//        //查询所有订单内容
//        ArrayList<OrderOther> orderOthers = Lists.newArrayList();
//        orderList.forEach(order -> {
//            //已发货、课程权限、启用、未过期
//            List<OrderOther> orderOtherList = baseMapper.selectList(new QueryWrapper<OrderOther>()
//                    .eq("order_id", order.getOrderId())
//                    .eq("ship",1)
//                    .eq("authority",1)
//                    .eq("is_enable",1)
//                    .ge("remaining_period",0));
//            orderOthers.addAll(orderOtherList);
//        });


        ArrayList<CurriculumConfigure> curriculumConfigures = Lists.newArrayList();
        //查询购买的所有课程中系统的项目
//        orderOthers.forEach(orderOther -> {
//            //展示、未删除、系统查询
//            List<CurriculumConfigure> curriculumConfigureList = configureMapper.selectList(new QueryWrapper<CurriculumConfigure>()
//                    .eq("cid", orderOther.getDataOrCourseId())
//                    .eq("is_show", 0)
//                    .eq("is_del", 0)
//                    .eq("system_id", systemId));
//            curriculumConfigures.addAll(curriculumConfigureList);
//        });

        //获取项目集合
        HashSet<Integer> set = new HashSet<>();
        if (curriculumConfigures.size()>0){
            for (CurriculumConfigure curriculumConfigure : curriculumConfigures){
                set.add(curriculumConfigure.getProjectId());
            }
        }else {
            return null;
        }

        ProjectDataTo projectDataTo = new ProjectDataTo();
        projectDataTo.setSet(set);
        projectDataTo.setName(name);
        projectDataTo.setSystemId(systemId);
        JudgmentPointDataTo projectPoints = occupationlabClient.getProjectPoints(projectDataTo);
        if (projectPoints==null){
            throw new CustomException(ExceptionEnum.SERVICE_FAIL);
        }
        return projectPoints;
    }

    @Override
    public String getOrderOtherTime(QueryTimeReq queryTimeReq) {
        if (queryTimeReq.getStartTime()!=null){
            //查询客户下有此产品的所有订单产品是否存在时间期内
            Integer count = baseMapper.isTimePeriod(queryTimeReq);
            if (count>0){
                throw new CustomException(ExceptionEnum.REPEAT_ORDERS);
            }
            // 返回上一个订单产品的到期时间
            // return baseMapper.OrderOtherEndTime(queryTimeReq);
        }

        if (StringUtils.isNotBlank(queryTimeReq.getEndTime()) && StringUtils.isNotBlank(queryTimeReq.getStartTime())){
            //查询客户下有此产品的所有订单产品时间是否重叠
            List<OrderOther> orderOtherList = baseMapper.selectOtherList(queryTimeReq);
            orderOtherList.forEach(orderOther -> {
                long startTime1 = DateUtil.parseDate(queryTimeReq.getStartTime()).getTime();
                long endTime1 = DateUtil.parseDate(queryTimeReq.getEndTime()).getTime();
                long startTime2 = orderOther.getStartTime().getTime();
                long endTime2 = orderOther.getEndTime().getTime();
                //允许时间重叠，续费时订单时间无缝衔接
                if (startTime1 < endTime2 && endTime1 > startTime2){
                    throw new CustomException(ExceptionEnum.REPEAT_ORDERS);
                }
            });
        }

        return null;
    }
}
