package com.huoran.nakadai.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.RandomUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huoran.api.UserClient;
import com.huoran.common.constant.DelConstant;
import com.huoran.common.constant.SMSTemplate;
import com.huoran.common.entity.UserAccount;
import com.huoran.common.exception.CustomException;
import com.huoran.common.exception.ExceptionEnum;
import com.huoran.common.response.R;
import com.huoran.common.utils.JwtUtils;
import com.huoran.common.utils.MD5;
import com.huoran.nakadai.entity.*;
import com.huoran.nakadai.entity.req.*;
import com.huoran.nakadai.entity.res.PartnerAccountResp;
import com.huoran.nakadai.entity.vo.TeamMemberVo;
import com.huoran.nakadai.mapper.*;
import com.huoran.nakadai.service.PartnerAccountService;
import com.huoran.nakadai.service.PartnerClassificationService;
import com.huoran.nakadai.service.PartnerTeamService;
import com.huoran.nakadai.utils.EmailUtil;
import com.huoran.nakadai.utils.QrCodeUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 合伙人账号表 服务实现类
 * </p>
 *
 * @author chen
 * @since 2022-05-18
 */
@Service
public class PartnerAccountServiceImpl extends ServiceImpl<PartnerAccountMapper, PartnerAccount> implements PartnerAccountService {

    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Autowired
    private HrStaffMapper staffMapper;

    @Autowired
    private UserClient userClient;

    @Autowired
    private PartnerClassificationMapper partnerClassificationMapper;

    @Autowired
    private PartnerClassificationService partnerClassificationService;

    @Autowired
    private PartnerTeamMapper partnerTeamMapper;

    @Autowired
    private EmailUtil emailUtil;

    @Autowired
    private OrderMapper orderMapper;

    @Autowired
    public PartnerTeamService partnerTeamService;

    @Override
    @Transactional
    public R savePartnerAccount(PartnerAccountAddReq partnerAccountAddReq) {
        //验证邀请码是否过期
        String code = redisTemplate.opsForValue().get(partnerAccountAddReq.getId() + "invitationCode");
        if (StringUtils.isEmpty(code)) {
            throw new CustomException(ExceptionEnum.INVITATION_CODE_EXPIRED);
        }

        String key = StringUtils.join(partnerAccountAddReq.getPhone(), SMSTemplate.AUTHENTICATION_CODE);
        String value = redisTemplate.opsForValue().get(key);
        if (StringUtils.isEmpty(value)) {
            throw new CustomException(ExceptionEnum.CODE_EXPIRED);
        }
        //验证手机验证码
        if (partnerAccountAddReq.getCode().equals(value)) {
            int role = 0;
            if (partnerAccountAddReq.getPartnerClassificationId()!=null){
                //查询转让管理人员角色是否为未认证角色，为未认证角色，邀请进入的人默认为未认证角色
                List<String> roleName = baseMapper.selectUserRole(Integer.valueOf(partnerAccountAddReq.getId()), partnerAccountAddReq.getPartnerClassificationId().toString());
                //role为0，赋予超管角色，其他值为未认证角色
                if (roleName.contains("未认证")){
                    role = 1;
                }
            }

            //创建账号，用户信息，团队分类，分配角色权限
            //账户相关信息表，账号合伙人平台判重(手机号为唯一标识)
            Integer accountId = baseMapper.selectAccount(partnerAccountAddReq.getPhone());

            //邀请人为成员且账号存在，则直接拉入该团队，不可重复拉入
            if (accountId != null & partnerAccountAddReq.getIsTeam() == 0) {
                //无法加入自己的团队
                if (partnerAccountAddReq.getId().equals(accountId.toString())) {
                    throw new CustomException(ExceptionEnum.CANT_JOIN_TEAM);
                }

                //查询合伙人账号
                PartnerAccount partnerAccount = baseMapper.selectOne(new QueryWrapper<PartnerAccount>().
                        eq("account_id", accountId));

                //被邀请人有合伙人账号，判断是否已经存在当前团队中
                if (ObjectUtil.isNotNull(partnerAccount)){
                    List<PartnerTeam> partnerTeams = partnerTeamMapper.selectList(new QueryWrapper<PartnerTeam>().
                            eq("partner_id", partnerAccount.getPartnerId()));

                    //查询团队下的成员是否存在团队中
                    List<PartnerClassification> teamMeb = partnerClassificationMapper.selectList(new QueryWrapper<PartnerClassification>().
                            eq("parent_id", partnerAccountAddReq.getPartnerClassificationId()));

                    for (PartnerTeam partnerTeam : partnerTeams) {
                        for (PartnerClassification partnerClassification : teamMeb) {
                            if (partnerTeam!=null&&partnerClassification!=null){
                                if (partnerTeam.getClassificationId().equals(partnerClassification.getId())){
                                    throw new CustomException(ExceptionEnum.USER_ALREADY_EXISTS_GROUP);
                                }
                            }
                        }
                    }
                }

                PartnerClassification classification = partnerClassificationMapper.selectById(partnerAccountAddReq.getPartnerClassificationId());
                //新增团队成员
                PartnerClassification partnerClassification = new PartnerClassification();
                partnerClassification.setIsTeam(0);
                partnerClassification.setParentId(partnerAccountAddReq.getPartnerClassificationId());
                partnerClassification.setLevel(classification.getLevel() + 1);
                partnerClassification.setPartnerClassificationName(classification.getPartnerClassificationName());
                int insert = partnerClassificationMapper.insert(partnerClassification);

                //团队分类
                if (insert > 0) {
                    PartnerTeam partnerTeam = new PartnerTeam();
                    partnerTeam.setPartnerId(partnerAccount.getPartnerId());
                    partnerTeam.setClassificationId(partnerClassification.getId());
                    partnerTeam.setProvinceId(partnerAccountAddReq.getProvinceId());
                    partnerTeam.setCityId(partnerAccountAddReq.getCityId());
                    partnerTeamMapper.insert(partnerTeam);
                }
                this.assignTeamMemberRoles(accountId, role, partnerClassification.getId());
                return insert > 0 ? R.ok() : R.error();
            }

            //合伙人新增团队，已是团员，新增当前用户为新团队管理员
            if (accountId != null & partnerAccountAddReq.getIsTeam() == 1) {
                //查询当前用户只能有一个团队管理员身份
                PartnerAccount partnerAccount = baseMapper.selectOne(new QueryWrapper<PartnerAccount>().
                        eq("account_id", accountId));

                //查询团队管理员数量，count为1表示已经是一个团队的超管，无法再创建
                Integer count = baseMapper.selectTeamManager(accountId);
                if (count > 1) {
                    throw new CustomException(ExceptionEnum.CANNOT_ADDED_REPEATEDLY);
                }

                Integer level = 1;
                Integer parentId = 1;
                if (partnerAccountAddReq.getPartnerClassificationId()!=null) {
                    //分类不为空，为团队直接新增该分类下的团体，为团队成员新增为该团队下的成员
                    PartnerClassification classification = partnerClassificationMapper.selectById(partnerAccountAddReq.getPartnerClassificationId());
                    level = classification.getLevel();
                    parentId = partnerAccountAddReq.getPartnerClassificationId();
                }

                //新增分类团队
                PartnerClassification partnerClassification = new PartnerClassification();
                partnerClassification.setIsTeam(1);
                partnerClassification.setParentId(parentId);
                partnerClassification.setLevel(level);
                String teamName = partnerAccountAddReq.getUserName() + "的团队";
                //名称进行判重
                String newName = checkRepeatClassName(teamName);
                partnerClassification.setPartnerClassificationName(newName);
                int insert = partnerClassificationMapper.insert(partnerClassification);
                if (insert > 0) {
                    PartnerTeam partnerTeam = new PartnerTeam();
                    partnerTeam.setPartnerId(partnerAccount.getPartnerId());
                    partnerTeam.setClassificationId(partnerClassification.getId());
                    partnerTeam.setProvinceId(partnerAccountAddReq.getProvinceId());
                    partnerTeam.setCityId(partnerAccountAddReq.getCityId());
                    partnerTeamMapper.insert(partnerTeam);
                }
                this.assignAdministratorRoles(accountId, role, partnerClassification.getId());
                return insert > 0 ? R.ok() : R.error();
            }

            //账号平台判重
            Integer count = baseMapper.checkAccount(partnerAccountAddReq.getAccount());
            if (count > 0) {
                throw new CustomException(ExceptionEnum.EXIST_ACCOUNT);
            }

            //用户相关信息表
            UserInfo userInfo = new UserInfo();
            BeanUtils.copyProperties(partnerAccountAddReq, userInfo);
            int addUserInfo = staffMapper.addUserInfo(userInfo);

            UserAccount userAccount = new UserAccount();
            BeanUtils.copyProperties(partnerAccountAddReq, userAccount);
            userAccount.setPassword(MD5.encrypt("1122aa"));
            userAccount.setUserId(userInfo.getUserId());
            userAccount.setType(2);//平台端区分：0->教师端 1->学生端 2->无端
            userAccount.setPlatformId("4");//4表示合伙人
            userAccount.setAccount(RandomUtil.randomNumbers(6));
            int addAccount = staffMapper.addUserAccount(userAccount);

            //添加合伙人信息
            PartnerAccount partnerAccount = new PartnerAccount();
            //合伙人绑定账号id
            partnerAccount.setAccountId(userAccount.getId());
            String username = baseMapper.selectUsername(partnerAccountAddReq.getId());
            //邀请人名称
            partnerAccount.setInvitationAccount(username);
            int insert = baseMapper.insert(partnerAccount);

            //添加所属合伙人分类,分类为空直接新增团队,（0是团队成员，1为团队）
            if (partnerAccountAddReq.getPartnerClassificationId()==null) {
                //新增团队
                PartnerClassification partnerClassification = new PartnerClassification();
                partnerClassification.setIsTeam(1);
                partnerClassification.setParentId(1);
                partnerClassification.setLevel(1);
                String teamName = partnerAccountAddReq.getUserName() + "的团队";
                //名称进行判重
                String newName = checkRepeatClassName(teamName);
                partnerClassification.setPartnerClassificationName(newName);
                int insert1 = partnerClassificationMapper.insert(partnerClassification);
                if (insert1 > 0) {
                    PartnerTeam partnerTeam = new PartnerTeam();
                    partnerTeam.setPartnerId(partnerAccount.getPartnerId());
                    partnerTeam.setClassificationId(partnerClassification.getId());
                    partnerTeam.setProvinceId(partnerAccountAddReq.getProvinceId());
                    partnerTeam.setCityId(partnerAccountAddReq.getCityId());
                    partnerTeamMapper.insert(partnerTeam);
                }
                this.assignAdministratorRoles(userAccount.getId(), 0, partnerClassification.getId());
                return addUserInfo > 0 && addAccount > 0 && insert > 0 ? R.ok() : R.error();
            } else {
                //分类不为空，为团队直接新增该分类下的团体，为团队成员新增为该团队下的成员
                PartnerClassification classification = partnerClassificationMapper.selectById(partnerAccountAddReq.getPartnerClassificationId());
                if (partnerAccountAddReq.getIsTeam() == 1) {
                    //新增分类团队
                    PartnerClassification partnerClassification = new PartnerClassification();
                    partnerClassification.setIsTeam(1);
                    partnerClassification.setParentId(partnerAccountAddReq.getPartnerClassificationId());
                    partnerClassification.setLevel(classification.getLevel() + 1);
                    String teamName = partnerAccountAddReq.getUserName() + "的团队";
                    //名称进行判重
                    String newName = checkRepeatClassName(teamName);
                    partnerClassification.setPartnerClassificationName(newName);
                    int insert1 = partnerClassificationMapper.insert(partnerClassification);
                    if (insert1 > 0) {
                        PartnerTeam partnerTeam = new PartnerTeam();
                        partnerTeam.setPartnerId(partnerAccount.getPartnerId());
                        partnerTeam.setClassificationId(partnerClassification.getId());
                        partnerTeam.setProvinceId(partnerAccountAddReq.getProvinceId());
                        partnerTeam.setCityId(partnerAccountAddReq.getCityId());
                        partnerTeamMapper.insert(partnerTeam);
                    }
                    this.assignAdministratorRoles(userAccount.getId(),0, partnerClassification.getId());
                    return addUserInfo > 0 && addAccount > 0 && insert > 0 ? R.ok() : R.error();
                }
                if (partnerAccountAddReq.getIsTeam() == 0) {
                    //新增分类团队成员
                    PartnerClassification partnerClassification = new PartnerClassification();
                    partnerClassification.setIsTeam(0);
                    partnerClassification.setParentId(partnerAccountAddReq.getPartnerClassificationId());
                    partnerClassification.setLevel(classification.getLevel() + 1);
                    partnerClassification.setPartnerClassificationName(classification.getPartnerClassificationName());
                    int insert1 = partnerClassificationMapper.insert(partnerClassification);
                    if (insert1 > 0) {
                        PartnerTeam partnerTeam = new PartnerTeam();
                        partnerTeam.setPartnerId(partnerAccount.getPartnerId());
                        partnerTeam.setClassificationId(partnerClassification.getId());
                        partnerTeam.setProvinceId(partnerAccountAddReq.getProvinceId());
                        partnerTeam.setCityId(partnerAccountAddReq.getCityId());
                        partnerTeamMapper.insert(partnerTeam);
                    }
                    this.assignTeamMemberRoles(userAccount.getId(), role, partnerClassification.getId());
                    return addUserInfo > 0 && addAccount > 0 && insert > 0 ? R.ok() : R.error();
                }
            }
        }
        throw new CustomException(ExceptionEnum.CODE_ERROR);
    }

    /**
     * 小程序分配合伙人角色
     */
    public void assignAdministratorRoles(Integer accountId, Integer role, Integer teamId) {
        List<Integer> list = new ArrayList<>();
        Integer platform = 4;//合伙人平台
        Integer port = 1;//pc端角色为0、小程序端为1
        String roleName;
        if (role==0){
            //查询超管角色
            roleName = "超级管理员";
        }else {
            //查询未认证角色
            roleName = "未认证";
        }
        Integer roleId = baseMapper.selectAdmin(platform, port,roleName);

        Integer id = baseMapper.selectRoleExist(accountId, roleId, teamId);
        if (id != null) {
            System.out.println("角色已存在");
        } else {
            list.add(roleId);
            // roleList.addAll(list);
            //账号赋予角色
            userClient.rolePermissionService(accountId, list, teamId);
        }
    }

    /**
     * 小程序分配合伙人队员角色
     */
    public void assignTeamMemberRoles(Integer accountId, Integer role, Integer teamId) {
        List<Integer> list = new ArrayList<>();
        //查询团队成员
        Integer platform = 4;//合伙人平台
        Integer port = 1;//pc端角色为0、小程序端为1
        String roleName;
        if (role==0){
            //查询团队成员角色
            roleName = "团队成员";
        }else {
            //查询未认证角色
            roleName = "未认证";
        }
        Integer roleId = baseMapper.selectAdmin(platform, port,roleName);
        if (roleId == null) {
            throw new CustomException(ExceptionEnum.DEFAULT_ROLE_NOTEXIST);
        }
        Integer id = baseMapper.selectRoleExist(accountId, roleId, teamId);
        if (id != null) {
            System.out.println("角色已存在");
        } else {
            list.add(roleId);
            // roleList.addAll(list);
            //账号赋予角色
            userClient.rolePermissionService(accountId, list, teamId);
        }
    }

    @Override
    public R pagePartnerAccountList(PartnerAccountListReq req) {
        //无分类id，查询所有团队
        if (StringUtils.isEmpty(req.getPartnerClassificationId())) {
            return R.error("先选择分类/团队进行查询！");
        }
        Page<PartnerAccountResp> page = new Page<>(req.getPageNum(), req.getPageSize());
        //查询分类下所有团队id
        ArrayList<Integer> ids = baseMapper.selectTeamIdByClassificationId(req.getPartnerClassificationId());
        req.setIds(ids);

        IPage<PartnerAccountResp> partnerAccountList = baseMapper.pagePartnerAccountList(page, req);

        List<PartnerAccountResp> respList = partnerAccountList.getRecords();
        //获取负责人数据
        List<PartnerAccountResp> collect = respList.stream().filter(partnerAccountResp -> "1".equals(partnerAccountResp.getIsTeam())).collect(Collectors.toList());

        for (PartnerAccountResp partnerAccountResp : collect) {
            //成员设置省份城市及费率
            for (PartnerAccountResp accountResp : respList) {
                if (accountResp.getParentId().toString().equals(partnerAccountResp.getPartnerClassificationId())){
                    accountResp.setProvinceId(partnerAccountResp.getProvinceId());
                    accountResp.setProvinceName(partnerAccountResp.getProvinceName());
                    accountResp.setCityId(partnerAccountResp.getCityId());
                    accountResp.setCityName(partnerAccountResp.getCityName());
                    accountResp.setAnnualTeamFee(partnerAccountResp.getAnnualTeamFee());
                    accountResp.setAnnualMarketingFee(partnerAccountResp.getAnnualMarketingFee());
                }
            }
        }
        partnerAccountList.setRecords(respList);
        return R.ok().put("pageList", partnerAccountList);

    }

    @Override
    public R partnerAccountMergeList(PartnerAccountListReq req) {

        Page<PartnerAccountResp> page = new Page<>(req.getPageNum(), req.getPageSize());
        //查询团队合伙人
        IPage<PartnerAccountResp> partnerAccountList = baseMapper.partnerAccountMergeList(page, req);

        return R.ok().put("pageList", partnerAccountList);
    }

    @Override
    public R partnerAccountDetail(Integer accountId) {
        PartnerAccountResp resp = baseMapper.partnerAccountDetail(accountId);
        return R.ok().put("data", resp);
    }

    @Override
    public R modifyPartnerAccount(PartnerAccountUpdateReq updateReq) {
        //根据账号id获取userId
        Integer userId = staffMapper.getUserId(updateReq.getAccountId());

        //更新员工相关信息表
        if (updateReq.getRoleList() != null) {

            //删除该账号原有角色
            staffMapper.delRoleByAccountId(updateReq.getAccountId());

            //更新员工信息,删除之前部门
            QueryWrapper<PartnerTeam> partnerTeamQueryWrapper = new QueryWrapper<>();
            partnerTeamQueryWrapper.eq("partner_id", updateReq.getPartnerId());
            int delete = partnerTeamMapper.delete(partnerTeamQueryWrapper);

            //添加员工部门
            List<Integer> idList = updateReq.getPartnerClassificationId();
            if (idList != null && idList.size() > 0) {
                idList.forEach(id -> {
                    PartnerTeam partnerTeam = new PartnerTeam();
                    partnerTeam.setClassificationId(id);
                    partnerTeam.setPartnerId(updateReq.getPartnerId());
                    partnerTeamMapper.insert(partnerTeam);
                });
            }

            //账号赋予角色
            userClient.rolePermissionService(updateReq.getAccountId(), updateReq.getRoleList(), null);
        }

        //编辑账户相关信息表
        UserAccount userAccount = new UserAccount();
        BeanUtils.copyProperties(updateReq, userAccount);
        userAccount.setId(updateReq.getAccountId());
        boolean updateAccount = staffMapper.modifyUserAccount(userAccount);

        //编辑用户相关信息表
        UserInfo userInfo = new UserInfo();
        BeanUtils.copyProperties(updateReq, userInfo);
        userInfo.setUserId(userId);
        boolean updateUserInfo = staffMapper.modifyUserInfo(userInfo);
        return updateUserInfo && updateAccount ? R.ok() : R.error();
    }

    @Override
    public R delPartnerAccount(Integer accountId) {

        QueryWrapper<PartnerAccount> accountQueryWrapper = new QueryWrapper<>();
        accountQueryWrapper.eq("account_id", accountId);
        PartnerAccount partnerAccount = baseMapper.selectOne(accountQueryWrapper);

        int count = 0;
        List<PartnerTeam> partnerTeams = partnerTeamMapper.selectList(new QueryWrapper<PartnerTeam>().
                eq("partner_id", partnerAccount.getPartnerId()));
        if (ObjectUtil.isNotNull(partnerTeams) && partnerTeams.size() > 0) {
            for (PartnerTeam partnerTeam : partnerTeams) {
                PartnerClassification partnerClassification = partnerClassificationMapper.selectById(partnerTeam.getClassificationId());
                if (ObjectUtil.isNotNull(partnerClassification)) {
                    //已经存在团队管理员，抛出异常
                    if (partnerClassification.getIsTeam() == 1) {
                        count++;
                    }
                }
            }
        }
        if (count>0){
            throw new CustomException(ExceptionEnum.SUPERVISING_ROLE);
        }

        //删除账号、用户表数据  修改用户表逻辑删除状态 修改账号表逻辑删除状态
        userClient.delAccountAndUser(accountId);

        //删除合伙人信息
        UpdateWrapper<PartnerAccount> updateWrapper = new UpdateWrapper<>();
        updateWrapper.set("is_del", DelConstant.IS_DEL);
        updateWrapper.eq("account_id", accountId);
        int update = baseMapper.update(new PartnerAccount(), updateWrapper);

        QueryWrapper<PartnerTeam> partnerTeamQueryWrapper = new QueryWrapper<>();
        partnerTeamQueryWrapper.eq("partner_id", partnerAccount.getPartnerId());

        //查询所有分类进行删除
        List<PartnerTeam> partnerTeams1 = partnerTeamMapper.selectList(partnerTeamQueryWrapper);

        partnerTeams1.forEach(partnerTeam -> {
            partnerClassificationMapper.deleteById(partnerTeam.getClassificationId());
        });

        //删除团队关系
        partnerTeamMapper.delete(partnerTeamQueryWrapper);

        return update > 0 ? R.ok() : R.error();
    }

    /*@Override
    public R delPartnerAccount(String teamId) {
        //团队id
        PartnerClassification partnerClassification = partnerClassificationMapper.selectById(teamId);
        QueryWrapper<PartnerClassification> partnerTeamQueryWrapper = new QueryWrapper<>();
        partnerTeamQueryWrapper.eq("parent_id", teamId);

        //删除该团队所以关系
        List<PartnerClassification> partnerClassifications = partnerClassificationMapper.selectList(partnerTeamQueryWrapper);
        partnerClassifications.add(partnerClassification);
        partnerClassifications.forEach(partnerClass -> {
            partnerTeamMapper.delete(new QueryWrapper<PartnerTeam>().eq("classification_id",partnerClass.getId()));
        });

        //删除团队成员
        partnerClassificationMapper.delete(partnerTeamQueryWrapper);
        //删除团队管理员
        partnerClassificationMapper.deleteById(Integer.parseInt(teamId));

        return R.ok();
    }*/

    @Override
    public void generationQrCode(String url, HttpServletResponse response) {
        QrCodeUtil.encode(url, response);
    }

    @Override
    public R getTeamsByAccountId(Integer accountId) {
        //1.首先拿到当前用户所在团队的信息

        List<PartnerAccountResp> partnerAccountList = baseMapper.getTeamsByAccountId(accountId);
        /*for (PartnerAccountResp resp:partnerAccountList) {
        for (PartnerAccountResp resp : partnerAccountList) {
            QueryWrapper<PartnerClassification> queryWrapper = new QueryWrapper<>();
            queryWrapper.select("*").eq("id",resp.getTeamId()).eq("is_del",0);
            PartnerClassification partnerClassification=  partnerClassificationMapper.selectOne(queryWrapper);
            queryWrapper.select("parent_id").eq("id", resp.getTeamId()).eq("is_team", 1).eq("is_del", 0);
            PartnerClassification partnerClassification = partnerClassificationMapper.selectOne(queryWrapper);

            List<Integer> partnerClassificationList = partnerClassificationMapper.getBusinessManagersUnderTheSameTeam(partnerClassification.getParentId());
            resp.setBusinessManagerIds(partnerClassificationList);
        }*/
        partnerAccountList.forEach(partnerAccountResp -> {
            PartnerClassification partnerClassification = partnerClassificationMapper.getParentInformationBasedOnChild(Integer.parseInt(partnerAccountResp.getPartnerClassificationId()));
            partnerAccountResp.setPartnerClassificationList(partnerClassification);
        });
        return R.ok().put("data", partnerAccountList);



        /*List<PartnerAccountResp> partnerAccountList = baseMapper.getTeamsByAccountId(accountId);
        for (PartnerAccountResp resp:partnerAccountList) {
            QueryWrapper<PartnerClassification> queryWrapper = new QueryWrapper<>();
            queryWrapper.select("*").eq("id",resp.getTeamId()).eq("is_del",0);
            PartnerClassification partnerClassification=  partnerClassificationMapper.selectOne(queryWrapper);


            if (partnerClassification.getIsTeam()==0){

            }

            List<Integer> partnerClassificationList = partnerClassificationMapper.getBusinessManagersUnderTheSameTeam(partnerClassification.getParentId());
            resp.setBusinessManagerIds(partnerClassificationList);
        }
        return R.ok().put("data", partnerAccountList);*/
    }

    @Override
    public R transferAdmin(TransferAdminReq transferAdminReq) {
        //验证邀请码是否过期
        String code = redisTemplate.opsForValue().get(transferAdminReq.getId() + "transferCode");
        if (StringUtils.isEmpty(code)) {
            throw new CustomException(ExceptionEnum.INVITATION_CODE_EXPIRED);
        }

        String key = StringUtils.join(transferAdminReq.getPhone(), SMSTemplate.AUTHENTICATION_CODE);
        String value = redisTemplate.opsForValue().get(key);
        if (StringUtils.isEmpty(value)) {
            throw new CustomException(ExceptionEnum.CODE_EXPIRED);
        }
        //验证手机验证码
        if (transferAdminReq.getCode().equals(value)) {
            //创建账号，用户信息，团队分类，分配角色权限
            //账户相关信息表，账号合伙人平台判重(手机号为唯一标识)
            Integer accountId = baseMapper.selectAccount(transferAdminReq.getPhone());

            //邀请人账号存在
            if (accountId != null) {
                //已经是该团队超管
                if (transferAdminReq.getId().equals(accountId.toString())) {
                    throw new CustomException(ExceptionEnum.CANNOT_ADDED_REPEATEDLY);
                }

                List<PartnerClassification> partnerClassifications = partnerClassificationMapper.selectList(new QueryWrapper<PartnerClassification>().
                        eq("parent_id", transferAdminReq.getPartnerClassificationId()));

                ArrayList<Integer> partnerClassIds = new ArrayList<>();
                if (ObjectUtil.isNotNull(partnerClassifications) && partnerClassifications.size() > 0) {
                    //是否已存在团队
                    partnerClassifications.forEach(partnerClassification -> {
                        partnerClassIds.add(partnerClassification.getId());
                    });
                }
                PartnerAccount memberAccount = baseMapper.selectOne(new QueryWrapper<PartnerAccount>().
                        eq("account_id",accountId));

                List<PartnerTeam> partnerTeams = partnerTeamMapper.selectList(new QueryWrapper<PartnerTeam>().
                        eq("partner_id", memberAccount.getPartnerId()));

                boolean exist = false;
                Integer memberTeamId = 0;
                for (PartnerTeam partnerTeam : partnerTeams) {
                    if (partnerClassIds.contains(partnerTeam.getClassificationId())){
                        exist = true;
                        memberTeamId = partnerTeam.getClassificationId();
                    }
                }
                //查询管理员的团队id
                PartnerAccount adminAccount = baseMapper.selectOne(new QueryWrapper<PartnerAccount>().
                        eq("account_id",transferAdminReq.getId()));

                //存在该团队，直接进行转换，不存在该团队，进入团队后再转换
                if (exist){
                    //转移团队
                    partnerTeamMapper.update(new PartnerTeam(),new UpdateWrapper<PartnerTeam>().
                            eq("partner_id",adminAccount.getPartnerId()).eq("classification_id",transferAdminReq.getPartnerClassificationId()).
                            set("classification_id",memberTeamId));

                    partnerTeamMapper.update(new PartnerTeam(),new UpdateWrapper<PartnerTeam>().
                            eq("partner_id",memberAccount.getPartnerId()).eq("classification_id",memberTeamId).
                            set("classification_id",transferAdminReq.getPartnerClassificationId()));

                    //转移订单
//                    List<Order> orders = orderMapper.selectList(new QueryWrapper<Order>().
//                            eq("business_manager_id", adminAccount.getPartnerId()).
//                            eq("team_id", transferAdminReq.getPartnerClassificationId()));
//
//                    for (Order order : orders) {
//                        order.setBusinessManagerId(memberAccount.getPartnerId().toString());
//                        orderMapper.updateById(order);
//                    }

                    //更新团队名称
                    PartnerClassification partnerClassification = partnerClassificationMapper.selectById(transferAdminReq.getPartnerClassificationId());
                    String teamName = transferAdminReq.getUserName() + "的团队";
                    //名称进行判重
                    String newName = checkRepeatClassName(teamName);
                    partnerClassification.setPartnerClassificationName(newName);
                    partnerClassificationMapper.updateById(partnerClassification);
                    //更新团员所属团队名称
                    List<PartnerClassification> partnerClassifications1 = partnerClassificationMapper.selectList(new QueryWrapper<PartnerClassification>().
                            eq("parent_id", partnerClassification.getId()));
                    partnerClassifications1.forEach(partnerClassification1 -> {
                        partnerClassification1.setPartnerClassificationName(newName);
                        partnerClassificationMapper.updateById(partnerClassification1);
                    });
                    //查询转让管理人员角色是否为未认证角色，为未认证角色，邀请进入的人默认为未认证角色
                    List<String> roleName = baseMapper.selectUserRole(adminAccount.getAccountId(), transferAdminReq.getPartnerClassificationId());
                    //role为0，赋予超管角色，其他值为未认证角色
                    int role = 0;
                    if (roleName.contains("未认证")){
                        role = 1;
                    }

                    //删除之前账号在所在团队的角色
                    partnerTeamService.delRoleByAccountIdAndTeamId(memberAccount.getAccountId(),Integer.parseInt(transferAdminReq.getPartnerClassificationId()));
                    //删除该账号在所在团队的角色
                    partnerTeamService.delRoleByAccountIdAndTeamId(adminAccount.getAccountId(),memberTeamId);

                    //赋予新增用户为所在团队超管角色，删除之前管理员在团队中的所有角色，赋予团队成员角色
                    this.assignAdministratorRoles(memberAccount.getAccountId(), role,Integer.parseInt(transferAdminReq.getPartnerClassificationId()));

                    this.assignTeamMemberRoles(adminAccount.getAccountId(), role, memberTeamId);
                    return R.ok();
                }else {

                    PartnerClassification classification = partnerClassificationMapper.selectById(transferAdminReq.getPartnerClassificationId());
                    //新增团队成员
                    PartnerClassification partnerClassification = new PartnerClassification();
                    partnerClassification.setIsTeam(0);
                    partnerClassification.setParentId(Integer.parseInt(transferAdminReq.getPartnerClassificationId()));
                    partnerClassification.setLevel(classification.getLevel() + 1);
                    partnerClassification.setPartnerClassificationName(classification.getPartnerClassificationName());
                    int insert = partnerClassificationMapper.insert(partnerClassification);

                    //团队分类
                    if (insert > 0) {
                        PartnerTeam partnerTeam = new PartnerTeam();
                        partnerTeam.setPartnerId(memberAccount.getPartnerId());
                        partnerTeam.setClassificationId(partnerClassification.getId());
                        partnerTeam.setProvinceId(transferAdminReq.getProvinceId());
                        partnerTeam.setCityId(transferAdminReq.getCityId());
                        partnerTeamMapper.insert(partnerTeam);
                    }
                    this.assignTeamMemberRoles(accountId, 0, partnerClassification.getId());

                    //转移团队
                    partnerTeamMapper.update(new PartnerTeam(),new UpdateWrapper<PartnerTeam>().
                            eq("partner_id",adminAccount.getPartnerId()).eq("classification_id",transferAdminReq.getPartnerClassificationId()).
                            set("classification_id",partnerClassification.getId()));

                    partnerTeamMapper.update(new PartnerTeam(),new UpdateWrapper<PartnerTeam>().
                            eq("partner_id",memberAccount.getPartnerId()).eq("classification_id",partnerClassification.getId()).
                            set("classification_id",transferAdminReq.getPartnerClassificationId()));

                    //转移订单
//                    List<Order> orders = orderMapper.selectList(new QueryWrapper<Order>().
//                            eq("business_manager_id", adminAccount.getPartnerId()).
//                            eq("team_id", transferAdminReq.getPartnerClassificationId()));
//
//                    for (Order order : orders) {
//                        order.setBusinessManagerId(memberAccount.getPartnerId().toString());
//                        orderMapper.updateById(order);
//                    }

                    //更新团队名称
                    PartnerClassification partnerClassification1 = partnerClassificationMapper.selectById(transferAdminReq.getPartnerClassificationId());
                    String teamName = transferAdminReq.getUserName() + "的团队";
                    //名称进行判重
                    String newName = checkRepeatClassName(teamName);
                    partnerClassification1.setPartnerClassificationName(newName);
                    partnerClassificationMapper.updateById(partnerClassification1);
                    //更新团员所属团队名称
                    List<PartnerClassification> partnerClassifications2 = partnerClassificationMapper.selectList(new QueryWrapper<PartnerClassification>().
                            eq("parent_id", partnerClassification1.getId()));
                    partnerClassifications2.forEach(partnerClassification2 -> {
                        partnerClassification2.setPartnerClassificationName(newName);
                        partnerClassificationMapper.updateById(partnerClassification2);
                    });
                    //查询转让管理人员角色是否为未认证角色，为未认证角色，邀请进入的人默认为未认证角色
                    List<String> roleName = baseMapper.selectUserRole(adminAccount.getAccountId(), transferAdminReq.getPartnerClassificationId());
                    //role为0，赋予超管角色，其他值为未认证角色
                    int role = 0;
                    if (roleName.contains("未认证")){
                        role = 1;
                    }

                    //删除之前账号在所在团队的角色
                    partnerTeamService.delRoleByAccountIdAndTeamId(memberAccount.getAccountId(),Integer.parseInt(transferAdminReq.getPartnerClassificationId()));

                    //删除该账号在所在团队的角色
                    partnerTeamService.delRoleByAccountIdAndTeamId(adminAccount.getAccountId(),partnerClassification.getId());
                    //赋予新增用户为所在团队超管角色，删除之前管理员在团队中的所有角色，赋予团队成员角色
                    this.assignAdministratorRoles(memberAccount.getAccountId(), role,Integer.parseInt(transferAdminReq.getPartnerClassificationId()));

                    this.assignTeamMemberRoles(adminAccount.getAccountId(), role, partnerClassification.getId());

                    return R.ok();
                }
            }

            //账号平台判重
            Integer count = baseMapper.checkAccount(transferAdminReq.getAccount());
            if (count > 0) {
                throw new CustomException(ExceptionEnum.EXIST_ACCOUNT);
            }

            //用户相关信息表
            UserInfo userInfo = new UserInfo();
            BeanUtils.copyProperties(transferAdminReq, userInfo);
            int addUserInfo = staffMapper.addUserInfo(userInfo);

            UserAccount userAccount = new UserAccount();
            BeanUtils.copyProperties(transferAdminReq, userAccount);
            userAccount.setPassword(MD5.encrypt("1122aa"));
            userAccount.setUserId(userInfo.getUserId());
            userAccount.setType(2);//平台端区分：0->教师端 1->学生端 2->无端
            userAccount.setPlatformId("4");//4表示合伙人
            userAccount.setAccount(RandomUtil.randomNumbers(6));
            int addAccount = staffMapper.addUserAccount(userAccount);

            //查询管理员的团队id
            PartnerAccount adminAccount = baseMapper.selectOne(new QueryWrapper<PartnerAccount>().
                    eq("account_id",transferAdminReq.getId()));

            //添加合伙人信息
            PartnerAccount partnerAccount = new PartnerAccount();
            partnerAccount.setAccountId(userAccount.getId());
            String username = baseMapper.selectUsername(transferAdminReq.getId());
            //邀请人名称
            partnerAccount.setInvitationAccount(username);
            int insert = baseMapper.insert(partnerAccount);

            //团队新增该分类下的成员
            PartnerClassification classification = partnerClassificationMapper.selectById(transferAdminReq.getPartnerClassificationId());
            //新增分类团队成员
            PartnerClassification partnerClassification = new PartnerClassification();
            partnerClassification.setIsTeam(0);
            partnerClassification.setParentId(Integer.parseInt(transferAdminReq.getPartnerClassificationId()));
            partnerClassification.setLevel(classification.getLevel() + 1);
            partnerClassification.setPartnerClassificationName(classification.getPartnerClassificationName());
            int insert1 = partnerClassificationMapper.insert(partnerClassification);
            if (insert1 > 0) {
                PartnerTeam partnerTeam = new PartnerTeam();
                partnerTeam.setPartnerId(partnerAccount.getPartnerId());
                partnerTeam.setClassificationId(partnerClassification.getId());
                partnerTeam.setProvinceId(transferAdminReq.getProvinceId());
                partnerTeam.setCityId(transferAdminReq.getCityId());
                partnerTeamMapper.insert(partnerTeam);
            }



            //转移团队
            partnerTeamMapper.update(new PartnerTeam(),new UpdateWrapper<PartnerTeam>().
                    eq("partner_id",adminAccount.getPartnerId()).eq("classification_id",transferAdminReq.getPartnerClassificationId()).
                    set("classification_id",partnerClassification.getId()));

            partnerTeamMapper.update(new PartnerTeam(),new UpdateWrapper<PartnerTeam>().
                    eq("partner_id",partnerAccount.getPartnerId()).eq("classification_id",partnerClassification.getId()).
                    set("classification_id",transferAdminReq.getPartnerClassificationId()));

            //转移订单
//            List<Order> orders = orderMapper.selectList(new QueryWrapper<Order>().
//                    eq("business_manager_id", adminAccount.getPartnerId()).
//                    eq("team_id", transferAdminReq.getPartnerClassificationId()));
//
//            for (Order order : orders) {
//                order.setBusinessManagerId(partnerAccount.getPartnerId().toString());
//                orderMapper.updateById(order);
//            }

            //更新团队名称
            PartnerClassification partnerClassification1 = partnerClassificationMapper.selectById(transferAdminReq.getPartnerClassificationId());
            String teamName = transferAdminReq.getUserName() + "的团队";
            //名称进行判重
            String newName = checkRepeatClassName(teamName);
            partnerClassification1.setPartnerClassificationName(newName);
            partnerClassificationMapper.updateById(partnerClassification1);
            //更新团员所属团队名称
            List<PartnerClassification> partnerClassifications1 = partnerClassificationMapper.selectList(new QueryWrapper<PartnerClassification>().
                    eq("parent_id", partnerClassification1.getId()));
            partnerClassifications1.forEach(partnerClassification2 -> {
                partnerClassification2.setPartnerClassificationName(newName);
                partnerClassificationMapper.updateById(partnerClassification2);
            });
            //查询转让管理人员角色是否为未认证角色，为未认证角色，邀请进入的人默认为未认证角色
            List<String> roleName = baseMapper.selectUserRole(adminAccount.getAccountId(),transferAdminReq.getPartnerClassificationId());
            //role为0，赋予超管角色，其他值为未认证角色
            int role = 0;
            if (roleName.contains("未认证")){
                role = 1;
            }

            //删除被转让账号在所在团队的角色
            partnerTeamService.delRoleByAccountIdAndTeamId(adminAccount.getAccountId(),Integer.parseInt(transferAdminReq.getPartnerClassificationId()));
            //赋予新增用户为所在团队超管角色，删除之前管理员在团队中的所有角色，赋予团队成员角色
            this.assignAdministratorRoles(userAccount.getId(), role,Integer.parseInt(transferAdminReq.getPartnerClassificationId()));

            this.assignTeamMemberRoles(adminAccount.getAccountId(), role, partnerClassification.getId());

            return addAccount > 0 ? R.ok() : R.error();
        }
        throw new CustomException(ExceptionEnum.CODE_ERROR);
    }

    @Override
    public R transferTeam(TransferTeamReq transferTeamReq) {
        Integer adminPartnerId = transferTeamReq.getAdminPartnerId();
        Integer memberPartnerId = transferTeamReq.getMemberPartnerId();
        Integer adminTeamId = transferTeamReq.getAdminTeamId();
        Integer memberTeamId = transferTeamReq.getMemberTeamId();

        //查询管理员的团队id
        PartnerAccount adminAccount = baseMapper.selectById(adminPartnerId);
        //转移团队
        partnerTeamMapper.update(new PartnerTeam(),new UpdateWrapper<PartnerTeam>().
                eq("partner_id",adminPartnerId).eq("classification_id",adminTeamId).
                set("classification_id",memberTeamId));

        partnerTeamMapper.update(new PartnerTeam(),new UpdateWrapper<PartnerTeam>().
                eq("partner_id",memberPartnerId).eq("classification_id",memberTeamId).
                set("classification_id",adminTeamId));

        //转移订单
//        List<Order> orders = orderMapper.selectList(new QueryWrapper<Order>().
//                eq("business_manager_id", adminPartnerId).
//                eq("team_id", adminTeamId));
//
//        for (Order order : orders) {
//            order.setBusinessManagerId(memberPartnerId.toString());
//            orderMapper.updateById(order);
//        }

        //更新团队名称
        PartnerClassification partnerClassification = partnerClassificationMapper.selectById(adminTeamId);
        //查询当前用户的团队后台
        String teamName = transferTeamReq.getMemberName() + "的团队";
        //名称进行判重
        String newName = checkRepeatClassName(teamName);
        partnerClassification.setPartnerClassificationName(newName);
        partnerClassificationMapper.updateById(partnerClassification);
        //更新团员所属团队名称
        List<PartnerClassification> partnerClassifications = partnerClassificationMapper.selectList(new QueryWrapper<PartnerClassification>().
                eq("parent_id", partnerClassification.getId()));
        partnerClassifications.forEach(partnerClassification1 -> {
            partnerClassification1.setPartnerClassificationName(newName);
            partnerClassificationMapper.updateById(partnerClassification1);
        });
        //查询转让管理人员角色是否为未认证角色，为未认证角色，邀请进入的人默认为未认证角色
        List<String> roleName = baseMapper.selectUserRole(adminAccount.getAccountId(), adminTeamId.toString());
        //role为0，赋予超管角色，其他值为未认证角色
        int role = 0;
        if (roleName.contains("未认证")){
            role = 1;
        }

        PartnerAccount memberAccount = baseMapper.selectById(memberPartnerId);

        //删除之前账号在所在团队的角色
        partnerTeamService.delRoleByAccountIdAndTeamId(memberAccount.getAccountId(),adminTeamId);
        //删除该账号在所在团队的角色
        partnerTeamService.delRoleByAccountIdAndTeamId(adminAccount.getAccountId(),memberTeamId);


        //赋予转让用户为所在团队超管角色，删除之前管理员在团队中的所有角色，赋予团队成员角色
        this.assignAdministratorRoles( memberAccount.getAccountId(), role,adminTeamId);

        this.assignTeamMemberRoles(adminAccount.getAccountId(), role,memberTeamId);

        return R.ok();
    }

    @Override
    public R mailFileSend(FileSendReq fileSendReq) {
//        emailUtil.sendMessageCarryFile(fileSendReq.getMail(), fileSendReq.getCopyWriting(), fileSendReq.getUrl(),fileSendReq.getCopyWriting());
        emailUtil.sendMessageCarryFiles(fileSendReq.getMail(), fileSendReq.getCopyWriting(), fileSendReq.getCopyWriting(),fileSendReq.getUrls(),fileSendReq.getFileName());
        return R.ok();
    }

    @Override
    public String checkRepeatClassName(String teamName) {
        Integer count = 1;
        PartnerClassification partnerClassification = new PartnerClassification();
        partnerClassification.setPartnerClassificationName(teamName);
        boolean ok = partnerClassificationService.checkRepeatAll(partnerClassification);
        //如果名称重复，加完之后再调取一遍
        if (ok){
            teamName=teamName+count++;
            return checkRepeatClassName(teamName);
        }else {
            return teamName;
        }
    }

    @Override
    public R teamPartnerList(PartnerAccountListReq req) {
        Page<PartnerAccountResp> page = new Page<>(req.getPageNum(), req.getPageSize());
        //无分类id，查询所有团队
        if (StringUtils.isNotEmpty(req.getPartnerClassificationId())) {
            //查询分类下所有团队id
            ArrayList<Integer> ids = baseMapper.selectTeamIdByClassificationId(req.getPartnerClassificationId());
            req.setIds(ids);
        }
        //查询团队合伙人
        IPage<PartnerAccountResp> partnerAccountList = baseMapper.teamPartnerList(page, req);
        for (PartnerAccountResp record : partnerAccountList.getRecords()) {
            //查询区域，往上查询所有父级
            String area = baseMapper.area(record.getTeamId());
            record.setArea(area);
            //查询团队数，往下查询所有子级
            Integer teamSize = baseMapper.teamSize(record.getTeamId());
            record.setTeamSize(teamSize);
        }

        return R.ok().put("pageList", partnerAccountList);
    }

    @Override
    public R queryTeamMembers(TeamMemberVo teamMemberVo) {
        //查询团队数，往下查询所有子级
        List<Integer> ids = baseMapper.teamMembers(teamMemberVo.getTeamId());
        List<PartnerAccountResp> list = baseMapper.selectTeamMembersDetails(ids,teamMemberVo.getName());
        return R.ok().put("teamMembers", list);
    }

    @Override
    public R queryPartnerAccount(String phone) {
        //查询是否有合伙人平台账号
        Integer accountId = baseMapper.selectAccount(phone);
        //无账号数据，返回成功状态
        if (accountId==null){
            return R.ok();
        }
        //查询合伙人账号
        PartnerAccount partnerAccount = baseMapper.selectOne(new QueryWrapper<PartnerAccount>().
                eq("account_id", accountId));

        if (ObjectUtil.isNotNull(partnerAccount)){

            List<PartnerTeam> partnerTeams = partnerTeamMapper.selectList(new QueryWrapper<PartnerTeam>().
                    eq("partner_id", partnerAccount.getPartnerId()));
            //查询所在团队
            List<Integer> classIds = new ArrayList<>();
            for (PartnerTeam partnerTeam : partnerTeams) {
                classIds.add(partnerTeam.getClassificationId());
            }
            //查询团队角色
            QueryWrapper<PartnerClassification> wrapper = new QueryWrapper<>();
            wrapper.in("id",classIds);
            wrapper.eq("is_team",1);
            wrapper.orderByDesc("create_time");
            wrapper.last("limit 1");
            PartnerClassification partnerClassification1 = partnerClassificationMapper.selectOne(wrapper);
            Integer classId = 0;
            int teamId = 0;
            int parentId = 0;
            String username = baseMapper.selectUsername(accountId.toString());
            //拥有管理者信息
            if (partnerClassification1!=null){
                classId = partnerClassification1.getId();
                teamId = 1;
            }else {
                //查询最近创建成员信息
                QueryWrapper<PartnerClassification> classificationQueryWrapper = new QueryWrapper<>();
                classificationQueryWrapper.in("id",classIds);
                classificationQueryWrapper.eq("is_team",0);
                classificationQueryWrapper.orderByDesc("create_time");
                classificationQueryWrapper.last("limit 1");
                PartnerClassification partnerClassification = partnerClassificationMapper.selectOne(classificationQueryWrapper);
                if (partnerClassification==null){
                    return R.ok().put("username",username);
                }
                classId = partnerClassification.getId();
                parentId = partnerClassification.getParentId();
            }


            String accountName = baseMapper.selectAccountName(accountId.toString());
            PartnerTeam team = baseMapper.selectProvinceCity( partnerAccount.getPartnerId(),classId);
            if (team==null){
                return R.ok().put("username",username);
            }
            if (teamId==0){
                QueryWrapper<PartnerClassification> classificationQueryWrapper1 = new QueryWrapper<>();
                classificationQueryWrapper1.eq("id",parentId);
                PartnerClassification partnerClassification = partnerClassificationMapper.selectOne(classificationQueryWrapper1);
                team.setTeamName(partnerClassification.getPartnerClassificationName());
                classId = partnerClassification.getId();
            }
            team.setTeamId(classId);
            team.setUserName(username);
            team.setIsTeam(teamId);
            team.setAccountName(accountName);
            return R.ok().put("team",team);

        }

        return R.error("信息有误，请联系管理员进行处理！");
    }

    @Override
    public R partnerAccountApplication(PartnerRegisterReq registerReq) {

        //创建账号，用户信息，团队分类，分配角色权限
        //账户相关信息表，账号合伙人平台判重(手机号为唯一标识)
        Integer accountId = baseMapper.selectAccount(registerReq.getPhone());


        //合伙人新增团队，已是团员，新增当前用户为新团队管理员
        if (accountId != null ) {
            //查询当前用户只能有一个团队管理员身份
            PartnerAccount partnerAccount = baseMapper.selectOne(new QueryWrapper<PartnerAccount>().
                    eq("account_id", accountId));

            //查询团队管理员数量，count为1表示已经是一个团队的超管，无法再创建
            Integer count = baseMapper.selectTeamManager(accountId);
            if (count > 1) {
                throw new CustomException(ExceptionEnum.CANNOT_ADDED_REPEATEDLY);
            }

            Integer level = 1;
            Integer parentId = 1;

            //新增分类团队
            PartnerClassification partnerClassification = new PartnerClassification();
            partnerClassification.setIsTeam(1);
            partnerClassification.setParentId(parentId);
            partnerClassification.setLevel(level);
            String teamName = registerReq.getUserName() + "的团队";
            //名称进行判重
            String newName = checkRepeatClassName(teamName);
            partnerClassification.setPartnerClassificationName(newName);
            int insert = partnerClassificationMapper.insert(partnerClassification);
            if (insert > 0) {
                PartnerTeam partnerTeam = new PartnerTeam();
                partnerTeam.setPartnerId(partnerAccount.getPartnerId());
                partnerTeam.setClassificationId(partnerClassification.getId());
                partnerTeam.setProvinceId(registerReq.getProvinceId());
                partnerTeam.setCityId(registerReq.getCityId());
                partnerTeamMapper.insert(partnerTeam);
            }
            //role为0赋为超管角色，不为0为未认证角色
            this.assignAdministratorRoles(accountId,1 , partnerClassification.getId());

            return R.ok().put("token", "").put("teamId",partnerClassification.getId());
        }

        //账号平台判重
        Integer count = baseMapper.checkAccount(registerReq.getAccount());
        if (count > 0) {
            throw new CustomException(ExceptionEnum.EXIST_ACCOUNT);
        }

        //用户相关信息表
        UserInfo userInfo = new UserInfo();
        BeanUtils.copyProperties(registerReq, userInfo);
        userInfo.setLoginNumber(1);
        userInfo.setLastLoginTime(new Date());
        int addUserInfo = staffMapper.addUserInfo(userInfo);

        UserAccount userAccount = new UserAccount();
        BeanUtils.copyProperties(registerReq, userAccount);
        userAccount.setPassword(MD5.encrypt("1122aa"));
        userAccount.setUserId(userInfo.getUserId());
        userAccount.setType(2);//平台端区分：0->教师端 1->学生端 2->无端
        userAccount.setPlatformId("4");//4表示合伙人
        userAccount.setAppOpenId(registerReq.getAppOpenId());
        userAccount.setAccount(RandomUtil.randomNumbers(6));
        userAccount.setAppOpenId(registerReq.getOpenId());
        int addAccount = staffMapper.addUserAccount(userAccount);

        //添加合伙人信息
        PartnerAccount partnerAccount = new PartnerAccount();
        //合伙人绑定账号id
        partnerAccount.setAccountId(userAccount.getId());

        int insert = baseMapper.insert(partnerAccount);

        //添加所属合伙人分类,分类为空直接新增团队
        PartnerClassification partnerClassification = new PartnerClassification();
        partnerClassification.setIsTeam(1);
        partnerClassification.setParentId(1);
        partnerClassification.setLevel(1);
        String teamName = registerReq.getUserName() + "的团队";
        //名称进行判重
        String newName = checkRepeatClassName(teamName);
        partnerClassification.setPartnerClassificationName(newName);
        int insert1 = partnerClassificationMapper.insert(partnerClassification);
        if (insert1 > 0) {
            PartnerTeam partnerTeam = new PartnerTeam();
            partnerTeam.setPartnerId(partnerAccount.getPartnerId());
            partnerTeam.setClassificationId(partnerClassification.getId());
            partnerTeam.setProvinceId(registerReq.getProvinceId());
            partnerTeam.setCityId(registerReq.getCityId());
            partnerTeamMapper.insert(partnerTeam);
        }
        //role为0赋为超管角色，不为0为未认证角色
        this.assignAdministratorRoles(userAccount.getId(), 1, partnerClassification.getId());

        //注册成功返回token和团队id
        if (addUserInfo > 0 && addAccount > 0){
            return R.ok().put("token", "").put("teamId",partnerClassification.getId());
        }
        return R.error();
    }
}
