package com.huoran.nakadai.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huoran.common.constant.DelConstant;
import com.huoran.common.response.R;
import com.huoran.nakadai.entity.PartnerArticleManagement;
import com.huoran.nakadai.entity.PartnerArticleManagementCollect;
import com.huoran.nakadai.entity.req.PagePartnerArticleManagementReq;
import com.huoran.nakadai.entity.res.PartnerArticleManagementResp;
import com.huoran.nakadai.entity.vo.ContentHeavyTitleReqVO;
import com.huoran.nakadai.mapper.PartnerArticleManagementCollectMapper;
import com.huoran.nakadai.mapper.PartnerArticleManagementMapper;
import com.huoran.nakadai.service.PartnerArticleManagementService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @描述：合伙人——文章管理 服务类
 * @作者: Rong
 * @日期: 2023-03-16
 */
@Service
public class PartnerArticleManagementServiceImpl extends ServiceImpl<PartnerArticleManagementMapper, PartnerArticleManagement> implements PartnerArticleManagementService {


    private final static Logger logger = LoggerFactory.getLogger(PartnerArticleManagementServiceImpl.class);
    @Autowired
    private PartnerArticleManagementMapper mapper;

    @Autowired
    private PartnerArticleManagementCollectMapper collectMapper;
    @Autowired
    private StringRedisTemplate redisTemplate;

    @Override
    public R partnerOperatingList(PagePartnerArticleManagementReq req) {
        req.setPageNum((req.getPageNum() - 1) * req.getPageSize());
        req.setPageSize(req.getPageSize());
        ValueOperations<String, String> opsForString = redisTemplate.opsForValue();
        String key = "browse:accountId=" + req.getAccountId();
//        Page<PartnerArticleManagementResp> page = new Page<PartnerArticleManagementResp>(req.getPageNum(), req.getPageSize());
//        IPage<PartnerArticleManagementResp> pageList = baseMapper.partnerOperatingList(page, req);

        List<PartnerArticleManagementResp> pageList = baseMapper.partnerOperatingList(req);
        //查看缓存是否有数据
        String returnValue = opsForString.get(key);
        //使用缓存返回数据 类型(1.学习2.资讯) 只有学习返回上次阅读
        if (req.getTypeId() == 1) {
            if (returnValue != null) {
                for (PartnerArticleManagementResp r : pageList) {
                    if (r.getId() == Integer.valueOf(returnValue)) {
                        r.setLastReading(true);
                    }
                }
            }

            for (PartnerArticleManagementResp resp : pageList) {
                QueryWrapper<PartnerArticleManagementCollect> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("account_id", req.getAccountId());
                queryWrapper.eq("content_id", resp.getId());
                PartnerArticleManagementCollect courseCollect = collectMapper.selectOne(queryWrapper);


                QueryWrapper<PartnerArticleManagementCollect> queryWrapper1 = new QueryWrapper<>();
                queryWrapper1.eq("content_id", resp.getId());
                Integer collectCount = collectMapper.selectCount(queryWrapper1);

                resp.setCollectionNumber(collectCount);
                //是否收藏：(state = 1收藏,0取消收藏)
                if (courseCollect != null) {
                    //收藏过
                    resp.setCollectionStatus(1);
                } else {
                    resp.setCollectionStatus(0);
                }
            }
        }
//        return R.ok().put("data", pageList);
        return R.ok().put("page", pageList).put("total", baseMapper.partnerOperatingListCount(req));

    }

    @Override
    public boolean checkIfTheTitleIsRepeat(ContentHeavyTitleReqVO content) {
        //只对已发布的标题判重
        QueryWrapper<PartnerArticleManagement> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("title", content.getTitle());
        queryWrapper.eq("is_release", DelConstant.POST_STATUS);
        queryWrapper.eq("type_id", content.getTypeId());
        List<PartnerArticleManagement> article = mapper.selectList(queryWrapper);
        if (article.size() > 0) {
            if (content.getId() != null) {
                //判断为编辑情况下，如果编辑的id与查询出来的一致不提示
                if (content.getId().equals(article.get(0).getId())) {
                    return true;
                }
            }

            return false;
        }
        return true;
    }

   /* @Override
    public R modifiedSort(Integer sequenceNumber, Integer articleId) {
        // 占分布式锁，去redis占坑
        // 1. 分布式锁占坑
        Boolean lock = redisTemplate.opsForValue().setIfAbsent("articleSortLock", "value", 30, TimeUnit.SECONDS);
        if (lock) {
            //加锁成功...
            if (sequenceNumber <= 0) {
                redisTemplate.delete("articleSortLock");   //删除key，释放锁
                return R.error("请输入正确的序号!");
            }

            PartnerArticleManagement content = baseMapper.selectById(articleId);


            //查看当前升序情况下全部文章信息
            QueryWrapper<PartnerArticleManagement> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("type_id", content.getTypeId());
            List<PartnerArticleManagement> articles = baseMapper.selectList(queryWrapper);

            if (sequenceNumber > articles.size()) {
                sequenceNumber = articles.size();
            }

            //获取当前排序的序号
            Integer currentSerialNumber = content.getSequence();

            if (currentSerialNumber.equals(sequenceNumber)) {
                redisTemplate.delete("articleSortLock");   //删除key，释放锁
                return R.ok();
            }
            //修改当前序号
            UpdateWrapper<PartnerArticleManagement> updateWrapper = new UpdateWrapper<>();
            updateWrapper.set("sequence", sequenceNumber);
            updateWrapper.eq("id", articleId);
            //做乐观锁处理
            *//*updateWrapper.eq("sequence", currentSerialNumber);*//*
            int updateCount = baseMapper.update(new PartnerArticleManagement(), updateWrapper);

            UpdateWrapper<PartnerArticleManagement> updateWrapper1 = new UpdateWrapper<>();
            updateWrapper1.ne("id", articleId);
            //currentSerialNumber 当前文章的序号
            //sequenceNumber 要更改的序号

            *//**
     * 就要判断排序是值改小还是改大
     * 大改小比如53改5那就加一个语句“大于等于5并且小于53”
     * 如果是小改大比如3改50那就是“大于3并且小于等于50"
     *//*
            if (currentSerialNumber > sequenceNumber) {
                //改大 gt >
                updateWrapper1.setSql("  sequence = sequence + 1");
                updateWrapper1.last(" and sequence >= " + sequenceNumber + " and sequence < " + currentSerialNumber);
            } else if (currentSerialNumber < sequenceNumber) {
                //改小
                //le为<=
//                updateWrapper1.le("sequence", sequenceNumber);
                updateWrapper1.setSql("  sequence = sequence - 1");
                updateWrapper1.last(" and sequence > " + currentSerialNumber + " and sequence <= " + sequenceNumber);
                //用于判断序号为本身就为1时候就不减了
//                updateWrapper1.ne("sequence", 1);
            }

            updateWrapper1.eq("type_id", content.getTypeId());
            baseMapper.update(new PartnerArticleManagement(), updateWrapper1);
            redisTemplate.delete("articleSortLock");   //删除key，释放锁
            return R.ok();
        } else {
            return R.error("当前有用户操作排序，请稍后刷新重试!");
        }
    }
*/

}



