package com.huoran.nakadai.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huoran.common.constant.DelConstant;
import com.huoran.nakadai.entity.PartnerClassification;
import com.huoran.nakadai.mapper.PartnerClassificationMapper;
import com.huoran.nakadai.service.PartnerClassificationService;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 合伙人分类表 服务实现类
 * </p>
 *
 * @author chen
 * @since 2022-05-18
 */
@Service
public class PartnerClassificationServiceImpl extends ServiceImpl<PartnerClassificationMapper, PartnerClassification> implements PartnerClassificationService {

    @Override
    public boolean checkRepeat(PartnerClassification partnerClassification) {
        QueryWrapper<PartnerClassification> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_del", DelConstant.NOT_DEL);
        queryWrapper.eq("level", partnerClassification.getLevel());
        queryWrapper.eq("partner_classification_name", partnerClassification.getPartnerClassificationName());
        queryWrapper.eq("parent_id", partnerClassification.getParentId());
        if (partnerClassification.getId() != null) {
            queryWrapper.last(" and id  != " + partnerClassification.getId());
        }
        List<PartnerClassification> partnerClassifications = baseMapper.selectList(queryWrapper);
        return partnerClassifications.size() > 0;
    }

    @Override
    public boolean checkRepeatAll(PartnerClassification partnerClassification) {
        QueryWrapper<PartnerClassification> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_del", DelConstant.NOT_DEL);
        queryWrapper.eq("partner_classification_name", partnerClassification.getPartnerClassificationName());
        List<PartnerClassification> partnerClassifications = baseMapper.selectList(queryWrapper);
        return partnerClassifications.size() > 0;
    }

    @Override
    public boolean removeClassification(Integer id) {
        //查询当前id的父id，删除后所有的团体父id更新此父id
        PartnerClassification partnerClassification = baseMapper.selectById(id);
        Integer parentId = partnerClassification.getParentId();

        //查询当前分类下的所有子分类,0为分类，1为团队
        List<Integer> ids = baseMapper.selectIdByPid(id,0);
        ids.add(id);

        //查询当前分类下的所有团队，团队名称叠加更新
        List<Integer> team = baseMapper.selectIdByPid(id, 1);
        team.forEach(t ->{

            PartnerClassification classification = new PartnerClassification();
            classification.setParentId(parentId);
            classification.setId(t);
            baseMapper.updateById(classification);
        });

        //批量进行删除
        int i = baseMapper.deleteBatchIds(ids);
        return i>0;
    }

    @Override
    public List<PartnerClassification> treeList(Integer applet) {
        //查询父级的
        QueryWrapper<PartnerClassification> wrapper = new QueryWrapper<>();
        wrapper.eq("is_del", DelConstant.NOT_DEL);
//        小程序端只查询层级，不查询团队
        if (applet!=null&&applet==1){
            wrapper.eq("is_team", 0);
        }
        List<PartnerClassification> partnerClassifications = baseMapper.selectList(wrapper);
        return build(partnerClassifications);
    }

    @Override
    public List<Integer> queryBusinessManagerIdBasedOnTeamId(String teamId) {
        return baseMapper.queryBusinessManagerIdBasedOnTeamId(teamId);
    }

    @Override
    public void removeRole(String teamId, Integer accountId) {
        baseMapper.updateRole(teamId,accountId);
    }

    /**
     * 使用递归方法构建权限菜单
     */
    public List<PartnerClassification> build(List<PartnerClassification> treeNodes) {
        List<PartnerClassification> trees = new ArrayList<>();
        for (PartnerClassification treeNode : treeNodes) {
            if (treeNode.getParentId() == 1) {
                trees.add(findChildren(treeNode, treeNodes));
            }
        }
        return trees;
    }

    /**
     * 递归查找子节点
     */
    public PartnerClassification findChildren(PartnerClassification treeNode, List<PartnerClassification> treeNodes) {
        treeNode.setChildren(new ArrayList<>());

        for (PartnerClassification it : treeNodes) {

            if (treeNode.getId().equals(it.getParentId())) {
                if (treeNode.getChildren() == null) {
                    treeNode.setChildren(new ArrayList<>());
                }
                treeNode.getChildren().add(findChildren(it, treeNodes));
            }
        }
        return treeNode;
    }
}
