package com.huoran.nakadai.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Maps;
import com.huoran.common.exception.CustomException;
import com.huoran.common.exception.ExceptionEnum;
import com.huoran.common.response.R;
import com.huoran.common.utils.RandomUtil;
import com.huoran.nakadai.entity.PartnerClassification;
import com.huoran.nakadai.entity.PartnerTeam;
import com.huoran.nakadai.entity.req.EditProvinceCityReq;
import com.huoran.nakadai.entity.req.SalesProgressReq;
import com.huoran.nakadai.entity.res.AnnualOperatingAnalysisResp;
import com.huoran.nakadai.entity.res.PartnerAccountResp;
import com.huoran.nakadai.entity.res.UserInfoByPhoneRes;
import com.huoran.nakadai.entity.vo.CheckVo;
import com.huoran.nakadai.mapper.OrderMapper;
import com.huoran.nakadai.mapper.PartnerClassificationMapper;
import com.huoran.nakadai.mapper.PartnerTeamMapper;
import com.huoran.nakadai.service.PartnerAccountService;
import com.huoran.nakadai.service.PartnerTeamService;
import com.huoran.nakadai.utils.EmailUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * <p>
 * 合伙人团队表（合伙人账号与分类中间表） 服务实现类
 * </p>
 *
 * @author chen
 * @since 2022-05-18
 */
@Service
public class PartnerTeamServiceImpl extends ServiceImpl<PartnerTeamMapper, PartnerTeam> implements PartnerTeamService {

    @Autowired
    private PartnerClassificationMapper partnerClassificationMapper;

    @Autowired
    private OrderMapper orderMapper;

    @Autowired
    private PartnerAccountService partnerAccountService;

    @Autowired
    private PartnerTeamMapper partnerTeamMapper;

    @Autowired
    private PartnerTeamMapper teamMapper;

    @Autowired
    StringRedisTemplate stringRedisTemplate;

    @Autowired
    private EmailUtil emailUtil;

    @Override
    public List<Integer> getMembersUnderTheSameTeam(Integer classificationId) {
        return baseMapper.getMembersUnderTheSameTeam(classificationId);
    }

    @Override
    public List<PartnerAccountResp> selectTeamList(String id, int current, int pageSize, String name) {
        current = (current - 1) * pageSize;

        List<PartnerClassification> partnerClassifications = partnerClassificationMapper.
                selectList(new QueryWrapper<PartnerClassification>().
                        eq("parent_id", id));
        //合伙人id
        ArrayList<Integer> ids = new ArrayList<>();
        //查询所有团队成员id
        partnerClassifications.forEach(partnerClassification -> {
            PartnerTeam partnerTeam = baseMapper.selectOne(new QueryWrapper<PartnerTeam>().
                    eq("classification_id", partnerClassification.getId()));
            ids.add(partnerTeam.getPartnerId());
        });

        if (ids != null && ids.size() > 0) {
            List<PartnerAccountResp> teamList = baseMapper.selectTeamInfo(ids, current, pageSize, name);
            return teamList;
        }
        return null;
    }

    /**
     * 1.分成方式的
     * 产品1成交价格是10w。结算方式是5:5分成的，则此时该产品的结算价=成交价*商务分成比例（比如50%，此处读取课程管理设置的商务经理分成比例）
     * 采购成本为 = 结算价+平台服务费
     * 平台服务费=结算价*10%即：10*50%+10**50%10%=5.5w
     * 2.结算单价方式
     * 采购成本为 =结算价+平台服务费
     * 结算价=结算单价（**元/年）*购买时长*数量
     * 此处购买时长要进行转换，比如购买的为月，则要先算出每月的价格即年结算价格/12;如果为日则按照年结算价/365
     * 比如产品结算价2w/年，购买三年此处采购成本为：
     * 2*3+2*3*10%=6.6
     */
    @Override
    public BigDecimal teamIncome(String id) {

        return null;
    }

    @Override
    public BigDecimal myIncome() {
        return null;
    }

    @Override
    public R queryUserInfoByPhone(String phone, String partnerId, String teamId) {
        HashMap<String, Object> map = Maps.newHashMap();

        //查询费率
        QueryWrapper<PartnerTeam> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("classification_id", teamId);
        PartnerTeam partnerTeam = teamMapper.selectOne(queryWrapper);
        int platformId = 3;
        if (StringUtils.isNotEmpty(partnerId) || StringUtils.isNotEmpty(teamId)) {
            //根据商务经理查询已完成的订单
            SalesProgressReq req = new SalesProgressReq();
            req.setTeamId(Integer.parseInt(teamId));
//            AnnualOperatingAnalysisResp teamIncome = orderMapper.income(req);
//            if (teamIncome != null) {
//                if (partnerTeam != null) {
//                    if (partnerTeam.getAnnualMarketingFee() != null) {
//                        BigDecimal bigDecimal = teamIncome.getFinalPrice().multiply((partnerTeam.getAnnualMarketingFee().divide(new BigDecimal(100))));
//                        //市场服务费 = 即成交金额之和  * 所填入的费率/100；
//                        teamIncome.setMarketingServiceCharge(bigDecimal);
//                    }
//
//                    if (teamIncome.getMarketingServiceCharge() != null) {
//                        //项目收益：总成交金额-总结算金额-总市场服务费
//                        teamIncome.setProjectBenefit(teamIncome.getFinalPrice()
//                                .subtract(teamIncome.getSettlementPrice()).
//                                subtract(teamIncome.getMarketingServiceCharge()));
//                    }
//                }
//            }


            AnnualOperatingAnalysisResp myIncome = null;

            if (partnerId != null) {
                req.setBusinessManagerId(Integer.parseInt(partnerId));
//                myIncome = orderMapper.income(req);
//                if (myIncome != null) {
//                    if (partnerTeam != null) {
//                        if (partnerTeam.getAnnualMarketingFee() != null) {
//                            BigDecimal bigDecimal = myIncome.getFinalPrice().multiply((partnerTeam.getAnnualMarketingFee().divide(new BigDecimal(100))));
//                            //市场服务费 = 即成交金额之和  * 所填入的费率/100；
//                            myIncome.setMarketingServiceCharge(bigDecimal);
//                        }
//
//                        if (myIncome.getMarketingServiceCharge() != null) {
//                            //项目收益：总成交金额-总结算金额-总市场服务费
//                            myIncome.setProjectBenefit(myIncome.getFinalPrice()
//                                    .subtract(myIncome.getSettlementPrice())
//                                    .subtract(myIncome.getMarketingServiceCharge()));
//                        }
//                    }
//                }
            }


            BigDecimal teamIncomeMoney = BigDecimal.ZERO;
//            if (teamIncome!=null && teamIncome.getProjectBenefit() != null) {
//                teamIncomeMoney = teamIncome.getProjectBenefit();
//            }


            BigDecimal myIncomeMoney = BigDecimal.ZERO;
            if (myIncome!=null && myIncome.getProjectBenefit() != null) {
                myIncomeMoney = myIncome.getProjectBenefit();
            }

           /*



            List<Order> orders = orderMapper.selectList(new QueryWrapper<Order>().
                    eq("business_manager_id", partnerId).
                    eq("team_id", teamId).eq("order_status",1));
            //统计个人收益
            BigDecimal myIncome = new BigDecimal(0);
            for (Order order : orders) {
                if (order.getProfit()!=null){
                    myIncome = myIncome.add(new BigDecimal(order.getProfit().toString()));
                }
            }

            //统计团队收益
            BigDecimal teamIncome = BigDecimal.ZERO;
            //根据团队查询团队商务经理
            //查询所有团队成员id
            List<PartnerClassification> partnerClassifications = partnerClassificationMapper.
                    selectList(new QueryWrapper<PartnerClassification>().
                            eq("parent_id", teamId));

            for (PartnerClassification partnerClassification : partnerClassifications) {
                PartnerTeam partnerTeam = partnerTeamMapper.selectOne(new QueryWrapper<PartnerTeam>().
                        eq("classification_id", partnerClassification.getId()));
                //查询团队内商务经理的订单
                List<Order> orderList = orderMapper.selectList(new QueryWrapper<Order>().
                        eq("team_id", teamId).eq("order_status",1));

                for (Order order : orderList) {
                    if (order.getProfit()!=null){
                        teamIncome = teamIncome.add(new BigDecimal(order.getProfit().toString()));
                    }
                }
            }*/

            QueryWrapper<PartnerTeam> queryWrapper1 = new QueryWrapper<>();
            queryWrapper1.eq("classification_id", teamId);
            PartnerTeam one = baseMapper.selectOne(queryWrapper1);
            map.put("teamInfo", one);
            map.put("myIncome", myIncomeMoney);
            map.put("teamIncome", teamIncomeMoney);
            platformId = 4;
        }

        UserInfoByPhoneRes info = baseMapper.selectInfoByPlatformId(phone, platformId);
        info.setPhone(phone);
        map.put("info", info);
        return R.ok().put("my", map);
    }

    @Override
    public String getPhone(String id, int platformId) {
        return baseMapper.queryPhone(id, platformId);
    }

    @Override
    public R queryPhone(String phone) {
        UserInfoByPhoneRes info = baseMapper.selectInfo(phone);
        return R.ok().put("info", info);
    }

    @Override
    public R updateMyEmail(CheckVo checkVo) {
        //邀请码
        String opener = stringRedisTemplate.opsForValue().get(checkVo.getAccountId() + checkVo.getPlatformId() + "mailCode");
        if (opener == null) {
            throw new CustomException(ExceptionEnum.CODE_EXPIRED);
        }

        if (opener.equals(checkVo.getCode())) {
            Integer userId = baseMapper.queryUserId(checkVo.getAccountId());
            baseMapper.updateUserMail(userId, checkVo.getEmail());
            return R.ok();
        }

        return R.error("验证码错误！");
    }

    @Override
    public boolean mailCodeSend(CheckVo checkVo) {
        //存在直接返回邀请码
        String code = stringRedisTemplate.opsForValue().get(checkVo.getAccountId() + checkVo.getPlatformId() + "mailCode");
        if (StringUtils.isNotEmpty(code)) {
            throw new CustomException(ExceptionEnum.VERIFICATION_CODE_HAS_BEEN_SENT);
        }
        code = RandomUtil.getSixBitRandom();
        stringRedisTemplate.opsForValue().set(checkVo.getAccountId() + checkVo.getPlatformId() + "mailCode", code, 5, TimeUnit.MINUTES);

        emailUtil.sendMessage(checkVo.getEmail(),
                "职站",
                "尊敬的用户，欢迎使用职站，您本次的验证码是: " + code + "，有效时间为5分钟");
        return true;
    }

    @Override
    public boolean checkName(EditProvinceCityReq provinceCityReq) {
        //查询合伙人姓名是否重复
        Integer count = baseMapper.queryUsername(provinceCityReq.getAccountId(), provinceCityReq.getUserName());
        return count > 0;
    }

    @Override
    public void editUsername(EditProvinceCityReq provinceCityReq) {
        Integer userId = baseMapper.queryUserId(provinceCityReq.getAccountId().toString());
        baseMapper.editUsername(userId, provinceCityReq.getUserName());
    }

    @Override
    public void delRoleByAccountIdAndTeamId(Integer accountId, Integer classificationId) {
        baseMapper.delRoleByAccountIdAndTeamId(accountId, classificationId);
    }
}
