package com.huoran.nakadai.service.impl;


import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huoran.common.constant.SMSTemplate;
import com.huoran.common.entity.UserAccount;
import com.huoran.common.exception.CustomException;
import com.huoran.common.exception.ExceptionEnum;
import com.huoran.common.response.R;
import com.huoran.common.utils.HttpClientUtils;
import com.huoran.common.utils.JwtUtils;
import com.huoran.common.utils.MD5;
import com.huoran.common.utils.TokenUtils;
import com.huoran.nakadai.entity.*;
import com.huoran.nakadai.entity.req.*;
import com.huoran.nakadai.entity.res.PartnerAccountResp;
import com.huoran.nakadai.entity.res.PlatformTeamAccountResp;
import com.huoran.nakadai.entity.res.TeamAccountResp;
import com.huoran.nakadai.mapper.*;
import com.huoran.nakadai.service.PlatformTeamAccountService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huoran.nakadai.utils.QrCodeUtil;
import com.huoran.nakadai.utils.WeChatPropertiesUtil;
import org.apache.commons.collections4.map.HashedMap;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 幼教产业平台团队账号表 服务实现类
 * </p>
 *
 * @author chen
 * @since 2023-12-12
 */
@Service
public class PlatformTeamAccountServiceImpl extends ServiceImpl<PlatformTeamAccountMapper, PlatformTeamAccount> implements PlatformTeamAccountService {

    @Autowired
    private HrStaffMapper staffMapper;

    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Autowired
    private PartnerAccountMapper partnerAccountMapper;

    @Autowired
    private PlatformTeamClassificationMapper platformTeamClassificationMapper;

    @Autowired
    private PlatformTeamMapper platformTeamMapper;

    private static final String REQUEST_URL = "https://api.weixin.qq.com/sns/jscode2session";
    private static final String GRANT_TYPE = "authorization_code";

    @Override
    public void generationQrCode(String url, HttpServletResponse response) {
        response.setHeader("Content-Type", "image/png");
        response.setHeader("Content-Disposition", "inline;filename=qrcode.png");
        QrCodeUtil.encode(url, response);
    }

    @Override
    public R platformTeamAccountMergeList(PlatformTeamAccountListReq req) {
        Page<PlatformTeamAccountResp> page = new Page<>(req.getPageNum(), req.getPageSize());
        //查询团队合伙人
        IPage<PlatformTeamAccountResp> partnerAccountList = baseMapper.platformTeamAccountMergeList(page, req);

        return R.ok().put("pageList", partnerAccountList);
    }

    @Override
    public Map<String, Object> kindergartenWeChatAccountApplication(AppletCallbackReq appletCallbackReq) throws Exception {
        String code = appletCallbackReq.getCode();

        Map<String, Object> data = new HashedMap<>();
        JSONObject sessionKeyOrOpenId = this.getSessionKeyOrOpenIdByKindergarten(code);
        String openid =(String) sessionKeyOrOpenId.get("openid");

        // 根据openid查询用户，存在则返回登录状态
        ArrayList<Integer> arrayList = new ArrayList<>();
        arrayList.add(5);
        arrayList.add(6);
        arrayList.add(7);

        boolean userExists = false; // 添加标志位，判断用户是否存在
        UserAccount personalAccount = null; // 用于存储个人账号

        for (Integer platformId : arrayList) {
            //根据openId查询账号详情List
            List<UserAccount> userAccountList = platformTeamClassificationMapper.selectAccountList(openid);
            for (UserAccount userAccount : userAccountList) {
                if(userAccount.getPlatformId().equals(platformId.toString())){
                    if (!userAccount.getIsEnable()){
                        if (platformId == 7) { // 如果个人账号被禁用，抛出异常
                            throw new CustomException(ExceptionEnum.ACCOUNT_DISABLED);
                        }
                        data.put("tips", "你已被组织禁止登录，现为您切回个人端");
                        continue; // 如果组织账号被禁用，跳过当前循环
                    }
                    if (platformId == 7) { // 如果是个人账号，保存信息并跳过当前循环
                        personalAccount = userAccount;
                        continue;
                    }
                    if (userAccount.getPhone()== null){
                        //登录更新手机号
                        baseMapper.updateAccountPhone(appletCallbackReq.getPhone(),userAccount.getId());
                    }
                    //更新登录时间
                    baseMapper.updateUserInfo(userAccount.getUserId());
                    data.put("state", "login");
                    data.put("platformId", platformId);
                    data.put("token", JwtUtils.getJwtToken(userAccount.getId().toString(),getShopId(openid,platformId),platformId));
                    userExists = true; // 如果用户存在，将标志位设为true
                    break; // 如果找到用户，就不需要继续查找
                }
            }
        }

        // 如果用户不存在，检查是否有个人账号
        if (!userExists && personalAccount != null) {
            data.put("state", "login");
            data.put("platformId", 7);
            data.put("token", JwtUtils.getJwtToken(personalAccount.getId().toString(),0L,7));
            userExists = true;
        }

        // 如果用户不存在，返回状态为"绑定"
        if (!userExists) {
            data.put("state", "bind");
        }

        data.put("sessionKey", sessionKeyOrOpenId);
        data.put("openId", openid);
        return data;
    }


    public Long getShopId(String openId, Integer platformId) {
        return baseMapper.selectShopIdByOpenId(openId, platformId);
    }

    @Override
    public String getCurrentPlatformAccount(String accountId, Integer platformId) {
        UserAccount userAccount = baseMapper.selectAccountPlatform(accountId);
        UserAccount currentPlatformAccount = baseMapper.selectAccountByOpenId(userAccount.getAppOpenId(), platformId);
        return currentPlatformAccount.getId().toString();
    }

    @Override
    public R getUserAccountByPhone(AppletCallbackReq appletCallbackReq) {
        Map<String, Object> data = new HashedMap<String, Object>();

        ArrayList<String> arrayList = new ArrayList<>();
        arrayList.add("5");
        arrayList.add("6");
        for (String platformId : arrayList) {
            UserAccount userAccountByPhone = baseMapper.selectAccount(appletCallbackReq.getPhone(), platformId);
            //根据手机号查询用户
            if (userAccountByPhone != null){
                if (!userAccountByPhone.getIsEnable()){
                    throw new CustomException(ExceptionEnum.ACCOUNT_DISABLED);
                }
                if (userAccountByPhone.getAppOpenId().equals("0")){
                    data.put("platformId", platformId);
                    //邀请添加用户没有openid，登录更新openId
                    baseMapper.updateAccountOpenId(appletCallbackReq.getOpenId(),userAccountByPhone.getId());
                    //查询个人端账号更新openid
                    UserAccount userAccount = baseMapper.selectAccount(appletCallbackReq.getPhone(), "7");
                    baseMapper.updateAccountOpenId(appletCallbackReq.getOpenId(),userAccount.getId());
                    //更新登录时间
                    baseMapper.updateUserInfo(userAccountByPhone.getUserId());
                }
                data.put("token", JwtUtils.getJwtToken(userAccountByPhone.getId().toString(),getShopId(appletCallbackReq.getOpenId(),Integer.valueOf(platformId)),Integer.valueOf(platformId)));
                data.put("openId", appletCallbackReq.getOpenId());

                return R.ok().put("data",data);
            }
        }

        return R.ok();
    }

    @Override
    public R kindergartenWeChatApplication(AppletCallbackReq appletCallbackReq) {
        Map<String, Object> data = new HashedMap<String, Object>();
        UserAccount userAccount = baseMapper.selectAccountByOpenId(appletCallbackReq.getOpenId(), appletCallbackReq.getPlatformId());
        //该平台已经注册
        if (userAccount != null){
            data.put("registerId", userAccount.getId());
            data.put("openId", appletCallbackReq.getOpenId());
            return R.ok().put("data",data);
        }

        String randomNumber5 = cn.hutool.core.util.RandomUtil.randomNumbers(5);
        String randomNumber6 = cn.hutool.core.util.RandomUtil.randomNumbers(6);
        String randomNumber7 = cn.hutool.core.util.RandomUtil.randomNumbers(7);
        Integer accountId ;
        String userName = "";

        // 根据不同的平台ID设置用户名和账户名
        switch (appletCallbackReq.getPlatformId()) {
            case 5:
                if (StringUtils.isNotBlank(appletCallbackReq.getOrganizationName())){
                    userName = appletCallbackReq.getOrganizationName();
                }else {
                    userName = "幼儿园" + randomNumber5;
                }
                break;
            case 6:
                if (StringUtils.isNotBlank(appletCallbackReq.getOrganizationName())){
                    userName = appletCallbackReq.getOrganizationName();
                }else {
                    userName = "供应商" + randomNumber6;
                }
                break;
            case 7:
                userName = "用户" + randomNumber7;
                break;
        }
        if (appletCallbackReq.getName()!=null){
            userName = appletCallbackReq.getName();
        }
        //企业认证获取个人平台手机号进行绑定
        UserAccount account = baseMapper.selectAccountByOpenId(appletCallbackReq.getOpenId(), 7);
        if (account != null) {
            appletCallbackReq.setPhone(account.getPhone());
        }

        accountId = addUserInfo(userName, appletCallbackReq.getPlatformId(), appletCallbackReq.getOpenId(), appletCallbackReq.getPhone());

        // 如果平台ID不是7，添加供应商/幼儿园信息和团队信息，默认添加个人端信息
        if (appletCallbackReq.getPlatformId() != 7) {
            //添加个人端信息，判断是否重复，为空添加
            if (account == null){
                String personalUserName = "用户" + randomNumber7;
                addUserInfo(personalUserName, 7, appletCallbackReq.getOpenId(), appletCallbackReq.getPhone());
            }

            PlatformTeamAccount platformTeamAccount1 = baseMapper.selectOne(new QueryWrapper<PlatformTeamAccount>().
                    eq("account_id", accountId).
                    eq("platform_id", appletCallbackReq.getPlatformId()));
            String teamName = "";
            Integer classificationId = null;
            if (platformTeamAccount1 == null){
                // 添加供应商/幼儿园信息
                PlatformTeamAccount platformTeamAccount = new PlatformTeamAccount();
                platformTeamAccount.setAccountId(accountId);
                platformTeamAccount.setPlatformId(appletCallbackReq.getPlatformId());
                baseMapper.insert(platformTeamAccount);

                // 添加所属供应商/幼儿园分类,分类为空直接新增团队
                PlatformTeamClassification platformTeamClassification = new PlatformTeamClassification();
                platformTeamClassification.setIsTeam(1);
                platformTeamClassification.setParentId(0);
                platformTeamClassification.setLevel(1);
                
                if (StringUtils.isNotBlank(appletCallbackReq.getOrganizationName())){
                    teamName = appletCallbackReq.getOrganizationName();
                }else {
                    teamName = userName + "的团队";
                }
                classificationId = platformTeamClassification.getId();
                platformTeamClassification.setClassificationName(teamName);
                int insert = platformTeamClassificationMapper.insert(platformTeamClassification);
                if (insert > 0) {
                    PlatformTeam platformTeam = new PlatformTeam();
                    platformTeam.setManageId(platformTeamAccount.getManageId());
                    platformTeam.setClassificationId(platformTeamClassification.getId());
                    platformTeamMapper.insert(platformTeam);
                }
            }else {
                List<PlatformTeam> platformTeamList = platformTeamMapper.selectList(new QueryWrapper<PlatformTeam>()
                        .eq("manage_id", platformTeamAccount1.getManageId()));

                PlatformTeamClassification platformTeamClassification = platformTeamClassificationMapper.selectOne(new QueryWrapper<PlatformTeamClassification>().
                        eq("is_team", 1).
                        in("id", platformTeamList.stream().map(PlatformTeam::getClassificationId).collect(Collectors.toList())));
                classificationId = platformTeamClassification.getId();
                teamName = platformTeamClassification.getClassificationName();
            }
            data.put("teamId",classificationId);
            data.put("teamName",teamName);
        }

        // 注册成功返回token
        data.put("token", JwtUtils.getJwtToken(accountId.toString(),getShopId(appletCallbackReq.getOpenId(),appletCallbackReq.getPlatformId()),appletCallbackReq.getPlatformId()));
        data.put("registerId", accountId);
        data.put("openId", appletCallbackReq.getOpenId());
        return R.ok().put("data",data);
    }

    public Integer addUserInfo(String userName,Integer platformId,String openId,String phone){
        // 创建并设置用户信息
        UserInfo userInfo = new UserInfo();
        userInfo.setUserName(userName);
        userInfo.setLoginNumber(1);
        staffMapper.addUserInfo(userInfo);

        // 创建并设置用户账户
        UserAccount userAccount = new UserAccount();
        userAccount.setPhone(phone);
        userAccount.setPhoneBindingTime(new Date().toString());
        userAccount.setPassword(MD5.encrypt("1122aa"));
        userAccount.setUserId(userInfo.getUserId());
        userAccount.setType(2); // 平台端区分：0->教师端 1->学生端 2->无端
        userAccount.setPlatformId(platformId.toString());
        userAccount.setAppOpenId(openId);
        userAccount.setAccount(userName);
        staffMapper.addUserAccount(userAccount);
        return userAccount.getId();
    }

    @Override
    public R getTeamsByPlatformId(String accountId, Integer platformId) {
        List<TeamAccountResp> teamsByPlatformId = baseMapper.getTeamsByPlatformId(accountId,platformId);
        teamsByPlatformId.forEach(teamAccountResp -> {
            PlatformTeamClassification platformTeamClassification = platformTeamClassificationMapper.getParentInformationBasedOnChild(teamAccountResp.getClassificationId());
            teamAccountResp.setPlatformTeamClassificationList(platformTeamClassification);
        });
        return R.ok().put("data", teamsByPlatformId);
    }

    @Override
    public R pagePlatformTeamAccountList(PlatformTeamAccountListReq req) {
        Page<PlatformTeamAccountResp> page = new Page<>(req.getPageNum(), req.getPageSize());
        //查询分类下所有团队id
        ArrayList<Integer> ids = baseMapper.selectTeamIdByClassificationId(req.getClassificationId());
        req.setIds(ids);
        IPage<PlatformTeamAccountResp> platformTeamAccountList = baseMapper.pagePlatformTeamAccountList(page, req);
        return R.ok().put("pageList", platformTeamAccountList);
    }

    private JSONObject getSessionKeyOrOpenIdByKindergarten(String code) throws Exception {
        Map<String, String> requestUrlParam = new HashMap<>();
        requestUrlParam.put("appid", WeChatPropertiesUtil.KINDERGARTEN_WX_OPEN_APP_ID);
        requestUrlParam.put("secret", WeChatPropertiesUtil.KINDERGARTEN_WX_OPEN_APP_SECRET);
        // 小程序端返回的code
        requestUrlParam.put("js_code", code);
        // 默认参数
        requestUrlParam.put("grant_type", GRANT_TYPE);

        // 发送post请求读取调用微信接口获取openid用户唯一标识
        String result = HttpClientUtils.doPost(REQUEST_URL, requestUrlParam);
        return JSON.parseObject(result);
    }

    @Override
    public R inviteJoinOrganization(InviteJoinInfo inviteJoinInfo) {
        //验证邀请码是否过期
        String uniqueIdentification = inviteJoinInfo.getPlatformId()+ inviteJoinInfo.getAccountId()+"kindergartenInvitationCode";
        String code = redisTemplate.opsForValue().get(uniqueIdentification);
        if (StringUtils.isEmpty(code)) {
            throw new CustomException(ExceptionEnum.INVITATION_CODE_EXPIRED);
        }

        String key = StringUtils.join(inviteJoinInfo.getPhone(), SMSTemplate.AUTHENTICATION_CODE);
        String value = redisTemplate.opsForValue().get(key);
        if (StringUtils.isEmpty(value)) {
            throw new CustomException(ExceptionEnum.CODE_EXPIRED);
        }
        //验证手机验证码
        if (inviteJoinInfo.getCode().equals(value)) {
            if (inviteJoinInfo.getPlatformId()==5){
                // 查询邀请用户其他平台是否存在账号
                UserAccount userAccount = baseMapper.selectAccount(inviteJoinInfo.getPhone(),"6");
                if (userAccount != null) {
                    throw new CustomException(ExceptionEnum.USER_ALREADY_EXISTS_OTHER_GROUP);
                }
            }else {
                UserAccount userAccount = baseMapper.selectAccount(inviteJoinInfo.getPhone(),"5");
                if (userAccount != null) {
                    throw new CustomException(ExceptionEnum.USER_ALREADY_EXISTS_OTHER_GROUP);
                }
            }

            //查询邀请用户是否存在账号
            UserAccount userAccount = baseMapper.selectAccount(inviteJoinInfo.getPhone(),inviteJoinInfo.getPlatformId().toString());

            Integer manageId = 0;
            //邀请人为成员且账号存在，则直接拉入该团队，不可重复拉入
            if (userAccount != null) {
                //无法加入自己的团队
                if (inviteJoinInfo.getAccountId().equals(userAccount.getId().toString())) {
                    throw new CustomException(ExceptionEnum.CANT_JOIN_TEAM);
                }

                //查询合伙人账号
                PlatformTeamAccount platformTeamAccount = baseMapper.selectOne(new QueryWrapper<PlatformTeamAccount>().
                        eq("account_id", userAccount.getId()));


                //被邀请人有团队账号，判断是否已经存在当前团队中
                if (ObjectUtil.isNotNull(platformTeamAccount)){
                    manageId = platformTeamAccount.getManageId();
                    List<PlatformTeam> platformTeamList = platformTeamMapper.selectList(new QueryWrapper<PlatformTeam>().
                            eq("manage_id", platformTeamAccount.getManageId()));

                    //查询团队下的成员是否存在团队中
                    List<PlatformTeamClassification> teamMeb = platformTeamClassificationMapper.selectList(new QueryWrapper<PlatformTeamClassification>().
                            eq("parent_id", inviteJoinInfo.getClassificationId()));

                    for (PlatformTeam platformTeam : platformTeamList) {
                        for (PlatformTeamClassification platformTeamClassification : teamMeb) {
                            if (platformTeam!=null&&platformTeamClassification!=null){
                                if (platformTeam.getClassificationId().equals(platformTeamClassification.getId())){
                                    throw new CustomException(ExceptionEnum.USER_ALREADY_EXISTS_GROUP);
                                }
                            }
                        }
                    }
                    //不存在加入当前组织
                    PlatformTeamClassification platformTeamAdmin = platformTeamClassificationMapper.selectById(inviteJoinInfo.getClassificationId());
                    //新增团队成员
                    PlatformTeamClassification platformTeamMeb = new PlatformTeamClassification();
                    platformTeamMeb.setIsTeam(0);
                    platformTeamMeb.setParentId(inviteJoinInfo.getClassificationId());
                    platformTeamMeb.setLevel(platformTeamAdmin.getLevel() + 1);
                    platformTeamMeb.setClassificationName(platformTeamAdmin.getClassificationName());
                    int insert = platformTeamClassificationMapper.insert(platformTeamMeb);

                    //团队分类
                    if (insert > 0) {
                        PlatformTeam platformTeam = new PlatformTeam();
                        platformTeam.setManageId(platformTeamAccount.getManageId());
                        platformTeam.setClassificationId(platformTeamMeb.getId());
                        platformTeamMapper.insert(platformTeam);
                    }
                    return insert > 0 ? R.ok() : R.error();
                //拉入当前团队
                }else {
                    //添加团队管理账号
                    PlatformTeamAccount platformTeamAccount1 = new PlatformTeamAccount();
                    platformTeamAccount1.setAccountId(userAccount.getId());
                    platformTeamAccount1.setPlatformId(inviteJoinInfo.getPlatformId());
                    String username = partnerAccountMapper.selectUsername(inviteJoinInfo.getAccountId());
                    platformTeamAccount1.setInvitationAccount(username);
                    baseMapper.insert(platformTeamAccount1);

                    PlatformTeamClassification platformTeamAdmin = platformTeamClassificationMapper.selectById(inviteJoinInfo.getClassificationId());
                    //新增团队成员
                    PlatformTeamClassification platformTeamMeb = new PlatformTeamClassification();
                    platformTeamMeb.setIsTeam(0);
                    platformTeamMeb.setParentId(inviteJoinInfo.getClassificationId());
                    platformTeamMeb.setLevel(platformTeamAdmin.getLevel() + 1);
                    platformTeamMeb.setClassificationName(platformTeamAdmin.getClassificationName());
                    int insert = platformTeamClassificationMapper.insert(platformTeamMeb);

                    //团队分类
                    if (insert > 0) {
                        PlatformTeam platformTeam = new PlatformTeam();
                        platformTeam.setManageId(platformTeamAccount1.getManageId());
                        platformTeam.setClassificationId(platformTeamMeb.getId());
                        platformTeamMapper.insert(platformTeam);
                    }
                    return insert > 0 ? R.ok() : R.error();
                }
            }else {
                //不存在创建账号，拉入组织
                AppletCallbackReq appletCallbackReq = new AppletCallbackReq();
                appletCallbackReq.setPlatformId(inviteJoinInfo.getPlatformId());
                appletCallbackReq.setPhone(inviteJoinInfo.getPhone());
                appletCallbackReq.setOpenId("0");
                appletCallbackReq.setName(inviteJoinInfo.getName());
                R r = kindergartenWeChatApplication(appletCallbackReq);
                Map<String, Object> data =(Map<String, Object>) r.get("data");
                Integer registerId =(Integer) data.get("registerId");
                //添加团队管理账号
                PlatformTeamAccount platformTeamAccount = new PlatformTeamAccount();
                platformTeamAccount.setAccountId(registerId);
                platformTeamAccount.setPlatformId(inviteJoinInfo.getPlatformId());
                String username = partnerAccountMapper.selectUsername(inviteJoinInfo.getAccountId());
                platformTeamAccount.setInvitationAccount(username);
                baseMapper.insert(platformTeamAccount);

                manageId = platformTeamAccount.getManageId();
            }

            PlatformTeamClassification platformTeamAdmin = platformTeamClassificationMapper.selectById(inviteJoinInfo.getClassificationId());
            //新增团队成员
            PlatformTeamClassification platformTeamMeb = new PlatformTeamClassification();
            platformTeamMeb.setIsTeam(0);
            platformTeamMeb.setParentId(inviteJoinInfo.getClassificationId());
            platformTeamMeb.setLevel(platformTeamAdmin.getLevel() + 1);
            platformTeamMeb.setClassificationName(platformTeamAdmin.getClassificationName());
            int insert = platformTeamClassificationMapper.insert(platformTeamMeb);

            //团队分类
            if (insert > 0) {
                PlatformTeam platformTeam = new PlatformTeam();
                platformTeam.setManageId(manageId);
                platformTeam.setClassificationId(platformTeamMeb.getId());
                platformTeamMapper.insert(platformTeam);
            }
            return insert > 0 ? R.ok() : R.error();

        }
        throw new CustomException(ExceptionEnum.CODE_ERROR);
    }


    @Override
    public R transferTeam(TransferTeamReq transferTeamReq) {
        Integer adminManageId = transferTeamReq.getAdminManageId();
        Integer memberManageId = transferTeamReq.getMemberManageId();
        Integer adminTeamId = transferTeamReq.getAdminTeamId();
        Integer memberTeamId = transferTeamReq.getMemberTeamId();

        //转移团队
        platformTeamMapper.update(new PlatformTeam(),new UpdateWrapper<PlatformTeam>().
                eq("manage_id",adminManageId).eq("classification_id",adminTeamId).
                set("classification_id",memberTeamId));

        platformTeamMapper.update(new PlatformTeam(),new UpdateWrapper<PlatformTeam>().
                eq("manage_id",memberManageId).eq("classification_id",memberTeamId).
                set("classification_id",adminTeamId));

        return R.ok();
    }
}
