package com.huoran.nakadai.utils;

import com.huoran.common.exception.CustomException;
import com.huoran.common.exception.ExceptionEnum;
import com.huoran.nakadai.entity.req.ExcelImpStaffReq;
import com.huoran.nakadai.entity.vo.DisciplineExcelVo;
import org.apache.commons.lang.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class ExcelImportHelper {

    //判断row是否为空
    public static boolean isRowEmpty(Row row) {
        if (null == row) {
            return true;
        }
        int firstCellNum = row.getFirstCellNum();   //第一个列位置
        int lastCellNum = row.getLastCellNum();     //最后一列位置
        int nullCellNum = 0;    //空列数量
        for (int c = firstCellNum; c < lastCellNum; c++) {
            Cell cell = row.getCell(c);
            if (null == cell || CellType.BLANK == cell.getCellType()) {
                nullCellNum++;
                continue;
            }
            cell.setCellType(CellType.STRING);
            String cellValue = cell.getStringCellValue().trim();
            if (StringUtils.isEmpty(cellValue)) {
                nullCellNum++;
            }
        }
        //所有列都为空
        if (nullCellNum == (lastCellNum - firstCellNum)) {
            return true;
        }
        return false;
    }


    private static Workbook getWorkbook(MultipartFile file) {
        String fileName = file.getOriginalFilename();
        Workbook workbook = null;

        if (fileName.endsWith("xlsx")) {
            try {
                workbook = new XSSFWorkbook(file.getInputStream());
            } catch (IOException e) {
                e.printStackTrace();
            }
        } else if (fileName.endsWith("xls")) {
            try {
                workbook = new HSSFWorkbook(file.getInputStream());
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return workbook;
    }


    /**
     * @Description :   读取文件数据
     * @Param file
     * @Author Rong---2021/10/15
     */
    public static List<ExcelImpStaffReq> readStaff(MultipartFile file) {
        List<ExcelImpStaffReq> list = new ArrayList<>();

        Workbook workbook = getWorkbook(file);
        ExcelImpStaffReq excelImpStaffReq = null;
        // 循环工作表Sheet
        for (int numSheet = 0; numSheet < workbook.getNumberOfSheets(); numSheet++) {
            Sheet hssfSheet = workbook.getSheetAt(numSheet);
            if (hssfSheet == null) {
                continue;
            }

            int count = 1;
            // 循环行Row//开始行2
            for (int rowNum = 2; rowNum <= hssfSheet.getLastRowNum(); rowNum++) {
                Row row = hssfSheet.getRow(rowNum);

                if (isRowEmpty(row)) {
                    count++;
                    if (count == hssfSheet.getLastRowNum()) {
                        throw new CustomException(ExceptionEnum.EXCEL_FILE_NULL);
                    }
                    continue;
                }

                Cell userName;//姓名
                Cell account;//账号
                Cell workNumber;//工号
                Cell dept;//所在部门
                Cell roleName;//角色
                Cell phone;//手机号
                Cell email;//邮箱
                if (row != null) {

                    excelImpStaffReq = new ExcelImpStaffReq();

                    if (row.getCell(6) != null) {
                        row.getCell(6).setCellType(CellType.STRING);
                        email = row.getCell(6);
                    } else {
                        email = row.createCell(6);
                    }

                    if (row.getCell(5) != null) {
                        row.getCell(5).setCellType(CellType.STRING);
                        phone = row.getCell(5);
                    } else {
                        phone = row.createCell(5);
                    }

                    if (row.getCell(4) != null) {
                        row.getCell(4).setCellType(CellType.STRING);
                        dept = row.getCell(4);
                    } else {
                        dept = row.createCell(4);
                    }

                    if (row.getCell(3) != null) {
                        row.getCell(3).setCellType(CellType.STRING);
                        workNumber = row.getCell(3);
                    } else {
                        workNumber = row.createCell(3);
                    }

                    if (row.getCell(2) != null) {
                        row.getCell(2).setCellType(CellType.STRING);
                        roleName = row.getCell(2);
                    } else {
                        roleName = row.createCell(3);
                    }

                    if (row.getCell(1) != null) {
                        row.getCell(1).setCellType(CellType.STRING);
                        account = row.getCell(1);
                    } else {
                        account = row.createCell(1);
                    }

                    if (row.getCell(0) != null) {
                        row.getCell(0).setCellType(CellType.STRING);
                        userName = row.getCell(0);
                    } else {
                        userName = row.createCell(0);
                    }

                    excelImpStaffReq.setUserName(userName.getStringCellValue());
                    excelImpStaffReq.setAccount(account.getStringCellValue());
                    excelImpStaffReq.setWorkNumber(workNumber.getStringCellValue());
                    excelImpStaffReq.setDept(dept.getStringCellValue());
                    excelImpStaffReq.setRoleName(roleName.getStringCellValue());
                    excelImpStaffReq.setPhone(phone.getStringCellValue());
                    excelImpStaffReq.setEmail(email.getStringCellValue());

                    list.add(excelImpStaffReq);
                }
            }

        }

        return list;
    }


    public static List<DisciplineExcelVo> readDiscipline(MultipartFile file) {
        List<DisciplineExcelVo> list = new ArrayList<>();

        Workbook workbook = getWorkbook(file);
        DisciplineExcelVo disciplineExcelVo = null;
        // 循环工作表Sheet
        for (int numSheet = 0; numSheet < workbook.getNumberOfSheets(); numSheet++) {
            Sheet hssfSheet = workbook.getSheetAt(numSheet);
            if (hssfSheet == null) {
                continue;
            }

            int count = 1;
            // 循环行Row//开始行2
            for (int rowNum = 2; rowNum <= hssfSheet.getLastRowNum(); rowNum++) {
                Row row = hssfSheet.getRow(rowNum);

                if (isRowEmpty(row)) {
                    count++;
                    if (count == hssfSheet.getLastRowNum()) {
                        throw new CustomException(ExceptionEnum.EXCEL_FILE_NULL);
                    }
                    continue;
                }

                Cell serialNumber;//姓名
                Cell category;//账号
                Cell majorCategory;//工号
                Cell major;//所在部门

                if (row != null) {

                    disciplineExcelVo = new DisciplineExcelVo();


                    if (row.getCell(3) != null) {
                        row.getCell(3).setCellType(CellType.STRING);
                        major = row.getCell(3);
                    } else {
                        major = row.createCell(3);
                    }

                    if (row.getCell(2) != null) {
                        row.getCell(2).setCellType(CellType.STRING);
                        majorCategory = row.getCell(2);
                    } else {
                        majorCategory = row.createCell(3);
                    }

                    if (row.getCell(1) != null) {
                        row.getCell(1).setCellType(CellType.STRING);
                        category = row.getCell(1);
                    } else {
                        category = row.createCell(1);
                    }

                    if (row.getCell(0) != null) {
                        row.getCell(0).setCellType(CellType.STRING);
                        serialNumber = row.getCell(0);
                    } else {
                        serialNumber = row.createCell(0);
                    }

                    disciplineExcelVo.setSerialNumber(serialNumber.getStringCellValue());
                    disciplineExcelVo.setCategory(category.getStringCellValue());
                    disciplineExcelVo.setMajorCategory(majorCategory.getStringCellValue());
                    disciplineExcelVo.setMajor(major.getStringCellValue());


                    list.add(disciplineExcelVo);
                }
            }

        }

        return list;
    }

}
