package com.huoran.nakadai.utils;

import com.google.zxing.*;
import com.google.zxing.client.j2se.BufferedImageLuminanceSource;
import com.google.zxing.client.j2se.MatrixToImageWriter;
import com.google.zxing.common.BitMatrix;
import com.google.zxing.common.HybridBinarizer;
import com.google.zxing.qrcode.decoder.ErrorCorrectionLevel;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;

import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletResponse;
import java.awt.image.BufferedImage;
import java.io.File;
import java.util.Hashtable;
import java.util.Map;

/** *
 * @Author chen
 * @DATE 2022/5/23 10:13
 * @Version 1.0
 * 二维码生成工具类
 */
public class QrCodeUtil {

    /**
     * 生成二维码
     * David
     * @param url  url 网址
     * @param response   生成的二维码url: 路径+文件名
     */
    public static void encode(String url,  HttpServletResponse response) {
        int width = 500; int height = 500;

        Map<EncodeHintType, Object> hints = new Hashtable<EncodeHintType, Object>();

        // 指定纠错等级

        hints.put(EncodeHintType.ERROR_CORRECTION, ErrorCorrectionLevel.L);

        // 指定编码格式

        hints.put(EncodeHintType.CHARACTER_SET, "UTF8");

        response.setContentType(MediaType.IMAGE_PNG_VALUE);
        response.setHeader(HttpHeaders.PRAGMA, "No-cache");
        response.setHeader(HttpHeaders.CACHE_CONTROL, "No-cache");
        response.setDateHeader(HttpHeaders.EXPIRES, 0L);

        try {

            BitMatrix bitMatrix = new MultiFormatWriter().encode(url, BarcodeFormat.QR_CODE, width, height, hints);

            // MatrixToImageWriter.writeToStream(bitMatrix, "png", new FileOutputStream(imgPath));

           MatrixToImageWriter.writeToStream(bitMatrix, "png", response.getOutputStream());// 输出图像


        } catch (Exception e) {

            e.printStackTrace();

        }

    }


    /**
     * 二维码解析
     * @param imgPath   二维码图片 路径+文件名
     * @return          解析后的二维码内容
     */

    public static String decode(String imgPath) {

        BufferedImage image = null;

        Result result = null;

        try {

            image = ImageIO.read(new File(imgPath));

            if (image == null) {

                return "the decode image may be not exit";

            }

            LuminanceSource source = new BufferedImageLuminanceSource(image);

            BinaryBitmap bitmap = new BinaryBitmap(new HybridBinarizer(source));

            Map<DecodeHintType, Object> hints = new Hashtable<DecodeHintType, Object>();

            hints.put(DecodeHintType.CHARACTER_SET, "UTF8");

            result = new MultiFormatReader().decode(bitmap, hints);

            return result.getText();

        } catch (Exception e) {

            e.printStackTrace();

        }

        return null;

    }

    // 测试代码
    public static void main( String[] args ){

        // QrCodeUtil.encode("http://www.dataforward.cn/", );
//        System.out.println(QrCodeUtil.decode("D:\\David\\QrCode\\QrCode1.png"));

    }

}
