package com.huoran.nakadai.utils;

import com.huoran.common.entity.PythonCodeResp;
import com.huoran.common.utils.DeleteFilesUtil;
import com.huoran.nakadai.config.EnvironmentConfig;
import lombok.extern.slf4j.Slf4j;

import java.io.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 运行python文件工具类
 *
 * @author 或然
 */
@Slf4j
public class RunCodeHelperUtil {

    /**
     * 生成文件到本地
     *
     * @param path：路径
     * @param data：数据
     */
    private static void writeFile(String path, byte[] data) {
        FileOutputStream fos = null;
        try {
            //注意这里一定要写到文件名为止
            File file = new File(path);
            if (!file.exists()) {
                file.getParentFile().mkdir();
                file.createNewFile();
            }
            fos = new FileOutputStream(new File(path));
            fos.write(data);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (fos != null) {
                    fos.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * 运行python脚本，接收它的运行信息
     *
     * @param
     * @return
     */
    public static PythonCodeResp runPython(String code) {
        PythonCodeResp resp = new PythonCodeResp();
        //判断代码中是否包含某个函数 包含则抛出异常
        if (code.contains("os.system") ||
                code.contains("os.popen") ||
                code.contains("commands.getstatusoutput") ||
                code.contains("subprocess.Popen") ||
                code.contains("subprocess.run")) {
            return null;
        }
        InputStream input = null;
        BufferedReader in = null;
        //将代码转成字节数组
        byte[] data = code.getBytes();
        //生成本地路径
        String objectName = EnvironmentConfig.pythonURL + "python" + System.currentTimeMillis() + ".py";//测试服
        //生成文件到本地
        writeFile(objectName, data);
        //接受python脚本运行结果
        StringBuffer buff = new StringBuffer();

        Process process;
        try {
            process = Runtime.getRuntime().exec("python " + objectName);
            int ztm = process.waitFor();
            String result = null;

            if (ztm == 0) {
                input = process.getInputStream(); //正确返回值
                resp.setRetResult(1);//retResult:返回结果(1为正确 0为错误)
                if (code.contains(".savefig(")) {
                    String url = checkCode(code);
                    if (url != null && !url.equals("输入的路径有误")) {
                        resp.setPhotoUrl(url);
                    }

                }
            } else {
                input = process.getErrorStream(); //错误返回值
                resp.setRetResult(0);
            }

            in = new BufferedReader(new InputStreamReader(input, EnvironmentConfig.coding));
            //一行一行读取
            while ((result = in.readLine()) != null) {
                buff.append(result);
                buff.append("\n");
            }
            resp.setRunResult(buff.toString());
        } catch (IOException | InterruptedException e) {
            e.printStackTrace();
        } finally {
            try {
                if (in != null) {
                    in.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
            try {
                if (input != null) {
                    input.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        //运行后删除python脚本文件
        DeleteFilesUtil.deleteFile(objectName);
        return resp;

    }

    private static void writeFile1(String code, String fileName) {
        FileWriter writer;
        try {
            writer = new FileWriter(fileName);
            writer.write(code);
            writer.flush();
            writer.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * @Description :   校验输入的代码是否存在
     * @Param str
     * @Author Rong---2021/12/14
     */
    public static String checkCode(String str) {
        if (str.contains(".savefig(")) {
            String url = checkRegular(str);
            String servicePath = EnvironmentConfig.projectImgUrl;
            //String servicePath = "http://www.occupationlab.com/python/";

            //本地地址
            // String servicePath = "http://localhost:9100/nakadai/static/";
            String returnUrl = "";
            for (String path : url.split("\t")) {
                returnUrl += servicePath + path + ",";
            }
            return returnUrl.substring(0, returnUrl.length() - 1);
        }
        return null;

    }

    public static String checkRegular(String str) {
        Pattern p = Pattern.compile("plt\\.savefig\\((\\\".*?\\\")\\)");
        Matcher m = p.matcher(str);
        String url = "";
        while (m.find()) {
            String quStr = m.group();
            String quStr1 = quStr.substring(quStr.indexOf("\"") + 1, quStr.lastIndexOf("\""));
            System.out.println(quStr1);
            url += quStr1 + "\t";
        }
        System.out.println("返回总:" + url.substring(0, url.length() - 1));
        return url.substring(0, url.length() - 1);
    }
}



