/*
package com.huoran.users.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.google.gson.Gson;
import com.huoran.common.response.R;
import com.huoran.common.utils.TokenUtils;
import com.huoran.users.config.AliRealNameAuthenticationConfig;
import com.huoran.users.entity.EnterpriseCertification;
import com.huoran.users.entity.HrUserAccount;
import com.huoran.users.entity.HrUserInfo;
import com.huoran.users.entity.UserAuthenticationInformation;
import com.huoran.users.entity.res.BusinessLicenseOCRResp;
import com.huoran.users.entity.res.CreditCodeResp;
import com.huoran.users.entity.res.IdCardOCRResp;
import com.huoran.users.entity.res.RealNameAuthenticationResp;
import com.huoran.users.service.EnterpriseCertificationService;
import com.huoran.users.service.IHrUserAccountService;
import com.huoran.users.service.IHrUserInfoService;
import com.huoran.users.service.UserAuthenticationInformationService;
import com.huoran.users.utils.ali.FaceRecognitionAidUtil;
import com.huoran.users.utils.ali.HttpUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.http.HttpResponse;
import org.apache.http.util.EntityUtils;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Api(value = "小程序-实名认证", tags = "小程序-实名认证")
@RestController
@RequestMapping("/users/autonym")
public class AliRealNameAuthenticationController {

    //实名认证实现类
    @Autowired
    public UserAuthenticationInformationService service;

    @Autowired
    public IHrUserAccountService userAccountService;

    @Autowired
    public IHrUserInfoService userInfoService;
    //企业认证信息存储
    @Autowired
    public EnterpriseCertificationService enterpriseCertificationService;

    @ApiOperation(value = "删除平台全部认证信息")
    @PostMapping("/deletePlatformAuthenticationInformation")
    public R deletePlatformAuthenticationInformation(@ApiParam(name = "openIds", value = "微信openId", required = true) @RequestBody List<String> openIds) {

        openIds.forEach(openId -> {
            QueryWrapper<UserAuthenticationInformation> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("open_id", openId);
            service.remove(queryWrapper);


            QueryWrapper<EnterpriseCertification> queryWrapper1 = new QueryWrapper<>();
            queryWrapper1.eq("open_id", openId);
            enterpriseCertificationService.remove(queryWrapper1);
        });

        return R.ok();
    }


    @ApiOperation(value = "身份证图片验证")
    @PostMapping("/faceAuthentication")
    public R faceAuthentication(@RequestParam(name = "imgFile", required = true) @ApiParam(value = "图片url") String imgFile, @RequestParam(name = "side", required = true) @ApiParam(value = "身份证正反面类型:face/back") String side) throws IOException {
        //身份证信息返回
        String idCardInformationReturn = FaceRecognitionAidUtil.IdCardOCRRecognition(imgFile, side);
        if (ObjectUtils.isEmpty(idCardInformationReturn)) {
//            ("身份证图片验证失败，请重新再试！");
            return R.error("身份证图片验证失败，请重新再试！");
        }
        IdCardOCRResp resp = new Gson().fromJson(idCardInformationReturn, IdCardOCRResp.class);
        if (resp.getSuccess()) {
            return R.ok().put("data", resp);
        } else {
//            ("身份证图片验证失败，请重新再试！");
            return R.error("身份证图片验证失败，请重新再试！");
        }

    }


    @PostMapping("/realNameAuthentication")
    @ApiOperation(value = "输入身份证号码、姓名进行实名认证且添加认证信息", response = UserAuthenticationInformation.class)
    public R realNameAuthentication(@RequestBody @ApiParam(name = "用户实名认证信息记录对象", value = "传入json格式", required = true) UserAuthenticationInformation userAuthenticationInformation, HttpServletRequest request) throws IOException {
        String accountId = TokenUtils.getIdByJwtToken(request);
        userAuthenticationInformation.setAccountId(Integer.valueOf(accountId));

        String json = FaceRecognitionAidUtil.postData(AliRealNameAuthenticationConfig.ALI_API_APPCODE, AliRealNameAuthenticationConfig.GET_APP_CODE_LINK, userAuthenticationInformation.getRealName(), userAuthenticationInformation.getIdCardNo());
        RealNameAuthenticationResp info = new Gson().fromJson(json, RealNameAuthenticationResp.class);

        if (!info.getRespCode().equals("0000")) {
            // ();
            return R.error(info.getRespMessage());
        }

        QueryWrapper<UserAuthenticationInformation> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("open_id", userAuthenticationInformation.getOpenId());
        UserAuthenticationInformation information = service.getOne(queryWrapper);

        if (!ObjectUtils.isEmpty(information)) {
            //为二次认证，删除上一次的认证信息
            service.removeById(information.getId());
        }


        //设置用户当前认证时间
        userAuthenticationInformation.setRealNameAuthenticationTime(new Date());
        userAuthenticationInformation.setProvince(info.getProvince());
        userAuthenticationInformation.setCity(info.getCity());
        userAuthenticationInformation.setCounty(info.getCounty());

        QueryWrapper<HrUserAccount> accountQueryWrapper = new QueryWrapper<>();
        accountQueryWrapper.eq("app_open_id", userAuthenticationInformation.getOpenId());
        accountQueryWrapper.eq("platform_id", 7);

        HrUserAccount userAccount = userAccountService.getOne(accountQueryWrapper);

        userAccount.setAccount(userAuthenticationInformation.getRealName());
        userAccountService.updateById(userAccount);


        UpdateWrapper<HrUserInfo> userInfoUpdateWrapper = new UpdateWrapper<>();
        userInfoUpdateWrapper.set("user_name", userAuthenticationInformation.getRealName());
        userInfoUpdateWrapper.eq("user_id", userAccount.getUserId());

        userInfoService.update(new HrUserInfo(), userInfoUpdateWrapper);


        boolean addState = service.save(userAuthenticationInformation);
        return addState ? R.ok() : R.error("认证失败");
    }


    @PostMapping("/authenticationOrNot")
    @ApiOperation(value = "个人是否认证", response = UserAuthenticationInformation.class)
    public R authenticationOrNot(HttpServletRequest request) {

        String accountId = TokenUtils.getIdByJwtToken(request);
        QueryWrapper<UserAuthenticationInformation> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("account_id", accountId);
        UserAuthenticationInformation userAuthenticationInformation = service.getOne(queryWrapper);
        if (ObjectUtils.isEmpty(userAuthenticationInformation)) {
            return R.ok().put("data", false).put("msg", "未认证");
        }
        return R.ok().put("data", true).put("authenticationInformation", userAuthenticationInformation);
    }


    @ApiOperation(value = "营业执照图片验证")
    @PostMapping("/businessLicensePictureVerification")
    public R businessLicensePictureVerification(@RequestParam(name = "imgFile", required = true) @ApiParam(value = "图片url") String imgFile) throws IOException {
        String method = "POST";
        Map<String, String> headers = new HashMap<String, String>();
        //最后在header中的格式(中间是英文空格)为Authorization:APPCODE 83359fd73fe94948385f570e3c139105
        headers.put("Authorization", "APPCODE " + AliRealNameAuthenticationConfig.ALI_API_APPCODE);
        //根据API的要求，定义相对应的Content-Type
        headers.put("Content-Type", "application/json; charset=UTF-8");
        Map<String, String> querys = new HashMap<String, String>();
        String bodys = "{\"image\":\"" + imgFile + "\"}";
        try {
            HttpResponse response = HttpUtils.doPost(AliRealNameAuthenticationConfig.ALI_API_BUSINESS_LICENSE_HOST, AliRealNameAuthenticationConfig.ALI_API_BUSINESS_LICENSE_PATH, method, headers, querys, bodys);
            BusinessLicenseOCRResp info = new Gson().fromJson(EntityUtils.toString(response.getEntity()), BusinessLicenseOCRResp.class);
            if (info.getSuccess()) {
                return R.ok().put("data", info);
            } else {
                //("营业执照图片验证失败，请重新再试！");
                return R.error("营业执照图片验证失败，请重新再试！");
            }
        } catch (Exception e) {
            e.printStackTrace();
//            ("营业执照图片验证失败，请重新再试！");
            return R.error("营业执照图片验证失败，请重新再试！");
        }
    }


    @ApiOperation(value = "统一社会信用代码认证(更改审核状态未提交为待审核)", response = CreditCodeResp.class)
    @PostMapping("/creditCodeAuthentication")
    public R creditCodeAuthentication(@RequestBody EnterpriseCertification enterpriseCertification, HttpServletRequest request) throws IOException {


        String accountId = TokenUtils.getIdByJwtToken(request);
        enterpriseCertification.setAccountId(Integer.valueOf(accountId));
        enterpriseCertification.setSubmitTime(new Date());
        String host = "https://sys.81api.com";
        String path = "/verifyCompany/";
        String method = "GET";
        Map<String, String> headers = new HashMap<String, String>();
        //最后在header中的格式(中间是英文空格)为Authorization:APPCODE 83359fd73fe94948385f570e3c139105
        headers.put("Authorization", "APPCODE " + "5a7f623d06724a4da5806c1fb0544e47");
        Map<String, String> querys = new HashMap<String, String>();
        querys.put("ComapnyName", enterpriseCertification.getCompanyName());
        querys.put("CreditCode", enterpriseCertification.getCreditCode());
        querys.put("LegalPersonName", enterpriseCertification.getLegalPerson());

        String errorMsg = "";
        //查询认证id
        QueryWrapper<EnterpriseCertification> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("open_id", enterpriseCertification.getOpenId());
        EnterpriseCertification certification = enterpriseCertificationService.getOne(queryWrapper);
        enterpriseCertification.setId(certification.getId());
        try {
            HttpResponse response = HttpUtils.doGet(host, path, method, headers, querys);
            CreditCodeResp info = new Gson().fromJson(EntityUtils.toString(response.getEntity()), CreditCodeResp.class);
            if (info.getStatus()) {
                if (!ObjectUtils.isEmpty(enterpriseCertificationService.checkWhetherItExists(enterpriseCertification.getCompanyName(), certification.getId(), enterpriseCertification.getOpenId()))) {
                    errorMsg = "企业名称已存在";
//                    (errorMsg);

                    return R.error(errorMsg);
                }

                //删除上团队信息
//                enterpriseCertificationService.deleteTeamInformation(enterpriseCertification.getAccountId());
                enterpriseCertificationService.updateById(enterpriseCertification);

                //更改二次验证的组织信息
                return changeTeamInformation(enterpriseCertification);
            } else {
                errorMsg = info.getReason();
                return R.error().put("message", info.getReason());
            }
        } catch (Exception e) {
            e.printStackTrace();
//            (errorMsg);
            return R.error(errorMsg);
        }


    }

    @NotNull
    private R changeTeamInformation(@RequestBody EnterpriseCertification enterpriseCertification) {
        enterpriseCertification.setSubmitTime(new Date());
        QueryWrapper<HrUserAccount> accountQueryWrapper1 = new QueryWrapper<>();
        accountQueryWrapper1.eq("app_open_id", enterpriseCertification.getOpenId());
        accountQueryWrapper1.eq("platform_id", enterpriseCertification.getPlatformSource());
        HrUserAccount userAccount1 = userAccountService.getOne(accountQueryWrapper1);
        userAccount1.setAccount(enterpriseCertification.getCompanyName());
        userAccountService.updateById(userAccount1);
        UpdateWrapper<HrUserInfo> userInfoUpdateWrapper = new UpdateWrapper<>();
        userInfoUpdateWrapper.set("user_name", enterpriseCertification.getCompanyName());
        userInfoUpdateWrapper.eq("user_id", userAccount1.getUserId());
        userInfoService.update(new HrUserInfo(), userInfoUpdateWrapper);

        //删除团队信息
//        enterpriseCertificationService.deleteTeamInformation(enterpriseCertification.getAccountId());
        return R.ok("提交成功,请等待审核！");
    }

    @ApiOperation(value = "统一社会信用代码认证(auditStatus给1，新增认证状态为待审核，用于个人认证幼儿园或供应商情况下)", response = CreditCodeResp.class)
    @PostMapping("/organizationCertification")
    public R organizationCertification(@RequestBody EnterpriseCertification enterpriseCertification, HttpServletRequest request) throws IOException {

*/
/*
        String accountId = TokenUtils.getIdByJwtToken(request);
        enterpriseCertification.setAccountId(Integer.valueOf(accountId));*//*

        enterpriseCertification.setSubmitTime(new Date());
        String host = "https://sys.81api.com";
        String path = "/verifyCompany/";
        String method = "GET";
        Map<String, String> headers = new HashMap<String, String>();
        //最后在header中的格式(中间是英文空格)为Authorization:APPCODE 83359fd73fe94948385f570e3c139105
        headers.put("Authorization", "APPCODE " +"5a7f623d06724a4da5806c1fb0544e47");
        Map<String, String> querys = new HashMap<String, String>();
        querys.put("ComapnyName", enterpriseCertification.getCompanyName());
        querys.put("CreditCode", enterpriseCertification.getCreditCode());
        querys.put("LegalPersonName", enterpriseCertification.getLegalPerson());

        String errorMsg = "";

        try {
            HttpResponse response = HttpUtils.doGet(host, path, method, headers, querys);
            CreditCodeResp info = new Gson().fromJson(EntityUtils.toString(response.getEntity()), CreditCodeResp.class);
            if (info.getStatus()) {
                if (!ObjectUtils.isEmpty(enterpriseCertificationService.checkWhetherItExists(enterpriseCertification.getCompanyName(), null, enterpriseCertification.getOpenId()))) {
                    errorMsg = "企业名称已存在";
                    return R.error(errorMsg);
                }
                enterpriseCertificationService.save(enterpriseCertification);

                //更改二次验证的组织信息
                return changeTeamInformation(enterpriseCertification);
            } else {
                errorMsg = info.getReason();

                //失败后删除本次生成的企业团队信息
                QueryWrapper<HrUserAccount> accountQueryWrapper = new QueryWrapper<>();
                accountQueryWrapper.eq("app_open_id", enterpriseCertification.getOpenId());
                accountQueryWrapper.eq("platform_id", enterpriseCertification.getPlatformSource());

                HrUserAccount userAccount = userAccountService.getOne(accountQueryWrapper);

                userInfoService.removeById(userAccount.getUserId());
                userAccountService.removeById(userAccount);
                return R.error().put("message", info.getReason());
            }
        } catch (Exception e) {
            e.printStackTrace();
            return R.error().put("message", errorMsg);
        }


    }

    public static void main(String[] args) {
        String host = "https://sys.81api.com";
        String path = "/verifyCompany/";
        String method = "GET";
        Map<String, String> headers = new HashMap<String, String>();
        //最后在header中的格式(中间是英文空格)为Authorization:APPCODE 83359fd73fe94948385f570e3c139105
        headers.put("Authorization", "APPCODE " + AliRealNameAuthenticationConfig.ALI_API_APPCODE);
        Map<String, String> querys = new HashMap<String, String>();
        querys.put("ComapnyName", "荆州国瑞网络科技有限公司");
        querys.put("CreditCode", "91421000MA494BGL8W");
        querys.put("LegalPersonName", "余承文");

        String errorMsg = "";

        try {
            HttpResponse response = HttpUtils.doGet(host, path, method, headers, querys);
            CreditCodeResp info = new Gson().fromJson(EntityUtils.toString(response.getEntity()), CreditCodeResp.class);
            if (info.getStatus()) {
                */
/*if (!ObjectUtils.isEmpty(enterpriseCertificationService.checkWhetherItExists(enterpriseCertification.getCompanyName(), null, enterpriseCertification.getOpenId()))) {
                    errorMsg = "企业名称已存在";
                    (errorMsg);
                }
                enterpriseCertificationService.save(enterpriseCertification);*//*


            } else {
                errorMsg = info.getReason();

                System.err.println("错误信息:" + errorMsg);
            }
        } catch (Exception e) {
            e.printStackTrace();
            errorMsg = "系统错误";
        }


    }
}*/
