package com.huoran.users.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.huoran.common.constant.DelConstant;
import com.huoran.common.constant.PlatformConstant;
import com.huoran.common.response.R;
import com.huoran.common.utils.TokenUtils;
import com.huoran.users.entity.HrUserAccount;
import com.huoran.users.entity.vo.CheckVo;
import com.huoran.users.service.IHrUserAccountService;
import com.huoran.users.service.IHrUserInfoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;


@Api(value = "小程序-我的", tags = "小程序-我的")
@RestController
@RequestMapping("/users/applets/mine")
public class AppletMyController {


    @Autowired
    public IHrUserInfoService userInfoService;

    @Autowired
    public IHrUserAccountService userAccountService;


    @ApiOperation(value = "更改手机号")
    @PostMapping("/changePhoneNumber")
    public R changePhoneNumber(@RequestParam(name = "phone", required = true) @ApiParam(value = "要更改的手机号") String phone,
            @RequestParam(name = "code", required = true) @ApiParam(value = "验证码") String code, HttpServletRequest request) {
        String accountId = TokenUtils.getIdByJwtToken(request);

        CheckVo checkVo = new CheckVo();
        checkVo.setOpener(code);
        checkVo.setPhone(phone);
        return userAccountService.changePhoneNumber(checkVo,Integer.valueOf(accountId));
    }



    @ApiOperation(value = "更改账号前判断账号是否存在")
    @PostMapping("/checkIfAnAccountExists")
    public R checkIfAnAccountExists(@RequestParam(name = "account", required = true) @ApiParam(value = "账号") String account, HttpServletRequest request) {
        String accountId = TokenUtils.getIdByJwtToken(request);
        QueryWrapper<HrUserAccount> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_del", DelConstant.NOT_DEL);
        queryWrapper.eq("account",account);

        HrUserAccount userAccount = userAccountService.getOne(queryWrapper);
        if (userAccount!=null){
            if (accountId.equals(String.valueOf(userAccount.getId()))){
                return R.ok();
            }
            return R.error("账号已存在!");
        }else {
            return R.ok();
        }


    }


    @ApiOperation(value = "更改手机号前判断该手机号是否存在")
    @PostMapping("/checkIfThePhoneNumberExists")
    public R checkIfThePhoneNumberExists(@RequestParam(name = "phone", required = true) @ApiParam(value = "手机号") String phone, HttpServletRequest request) {
        String accountId = TokenUtils.getIdByJwtToken(request);
        QueryWrapper<HrUserAccount> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_del", DelConstant.NOT_DEL);
        queryWrapper.eq("phone",phone);
        queryWrapper.eq("platform_id", PlatformConstant.ACCESSORY);
        HrUserAccount userAccount = userAccountService.getOne(queryWrapper);
        if (userAccount!=null){
            /*if (String.valueOf(userAccount.getId()).equals(accountId)&&userAccount.getPhone().equals(phone)){
                return R.error("更改的号码不能为当前号码！");
            }*/
            return R.error("当前手机号已存在，请更换手机号!");
        }else {
            return R.ok();
        }


    }

    @ApiOperation(value = "更改账号")
    @PostMapping("/changeAccount")
    public R changeAccount(@RequestParam(name = "account", required = true) @ApiParam(value = "账号") String account, HttpServletRequest request) {
        String accountId = TokenUtils.getIdByJwtToken(request);
        UpdateWrapper<HrUserAccount> updateWrapper = new UpdateWrapper<>();
        updateWrapper.set("account",account);
        updateWrapper.eq("is_del", DelConstant.NOT_DEL);
        updateWrapper.eq("id",accountId);

        boolean result = userAccountService.update(new HrUserAccount(),updateWrapper);
        return result?R.ok():R.error();
    }

}
