package com.huoran.users.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.huoran.api.UserClient;
import com.huoran.common.constant.DelConstant;
import com.huoran.common.constant.PlatformConstant;
import com.huoran.common.response.R;
import com.huoran.common.utils.DateUtils;
import com.huoran.common.utils.MD5;
import com.huoran.common.utils.TokenUtils;
import com.huoran.users.entity.HrUserAccount;
import com.huoran.users.entity.HrUserInfo;
import com.huoran.users.entity.req.DataUserListReq;
import com.huoran.users.entity.req.ModifyUserInfoReq;
import com.huoran.users.entity.req.SaveUserInfoReq;
import com.huoran.users.entity.res.DataUserInfoResp;
import com.huoran.users.entity.res.UserRoleResp;
import com.huoran.users.service.IHrUserAccountService;
import com.huoran.users.service.IHrUserInfoService;
import com.huoran.users.service.UserRoleService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 用户信息表 前端控制器
 * </p>
 *
 * @author huoran
 * @since 2021-06-28
 */
@RestController
@RequestMapping("/users/data/user")
@Api(value = "API - 用户管理", tags = "数据平台-用户管理")
public class DataUserInfoController {

    @Autowired
    public IHrUserInfoService service;

    @Autowired
    public IHrUserAccountService accountService;

    @Autowired
    private UserClient userClient;

    @Autowired
    private UserRoleService userRoleService;

    /**
     * 新增用户
     *
     * @param hrUserInfo
     * @return
     */
    @PostMapping("/save")
    @ApiOperation(value = "新增用户")
    public R save(@RequestBody @Valid SaveUserInfoReq hrUserInfo, HttpServletRequest request) {

        String accountId = TokenUtils.getIdByJwtToken(request);
        Integer schoolId = userClient.getSchoolIdByAccountId(accountId);
        hrUserInfo.setSchoolId(schoolId);
        //统计该院校下可启用的账号数量
        int availableNum = accountService.getAvailableNum(hrUserInfo.getSchoolId());

        //查询现在该院校已启用的账号数量
        QueryWrapper<HrUserAccount> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("school_id", hrUserInfo.getSchoolId());
        queryWrapper.eq("is_enable", DelConstant.IS_ENABLE);
        List<HrUserAccount> getNum = accountService.list(queryWrapper);

        //设置默认密码
        hrUserInfo.setPassword(MD5.encrypt("1122aa"));
        HrUserInfo userInfo = new HrUserInfo();
        userInfo.setRoleId("0");
        BeanUtils.copyProperties(hrUserInfo, userInfo);
        HrUserAccount userAccount = new HrUserAccount();
        BeanUtils.copyProperties(hrUserInfo, userAccount);
        availableNum = availableNum - 1;//todo 减1是因为客户也包含了算启用的用户

        //判断该院校已启用账户总数量是否小于该院校可启用的账户数量
        if (getNum.size() <= availableNum) {
            userAccount.setIsEnable(DelConstant.IS_ENABLE);//设置为已启用
        } else {
            userAccount.setIsEnable(DelConstant.NO_ENABLE);//设置为未启用
        }
        boolean ret1 = service.save(userInfo);
        userAccount.setUserId(userInfo.getUserId());
        userAccount.setPlatformId(PlatformConstant.DATA_PLATFORM + "");
        userAccount.setRoleId("0");
        boolean ret2 = accountService.save(userAccount);

        //账号赋予角色
        userClient.rolePermissionService(userAccount.getId(), hrUserInfo.getRoleIds(),null);

        return ret1 && ret2 ? R.ok() : R.error("新增失败");

    }

    /**
     * 编辑用户
     *
     * @param hrUserInfo
     * @return
     */
    @PostMapping("/modify")
    @ApiOperation(value = "编辑用户")
    public R modify(@RequestBody @Valid ModifyUserInfoReq hrUserInfo, HttpServletRequest request) {

        String accountId = TokenUtils.getIdByJwtToken(request);
        Integer schoolId = userClient.getSchoolIdByAccountId(accountId);
        //当要启用用户时候判断该院校的可启用账号是否还能再启用
        if (hrUserInfo.getIsEnable() == DelConstant.IS_ENABLE) {
            //查询现在该院校已启用的账号数量
            QueryWrapper<HrUserAccount> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("school_id", schoolId);
            queryWrapper.eq("is_del", DelConstant.NOT_DEL);
            queryWrapper.eq("is_enable", DelConstant.IS_ENABLE);
            //增加限制只统计数据平台的用户已启用账户数量 2023.10.23
            queryWrapper.last(" AND FIND_IN_SET( platform_id, '2' ) ");
            List<HrUserAccount> getNum = accountService.list(queryWrapper);

            //统计该院校下可启用的账号数量
            int availableNum = accountService.getAvailableNum(Integer.valueOf(schoolId));




            availableNum = availableNum - 1;//todo 减1是因为客户也包含了算启用的用户
            if (availableNum < getNum.size()) {
                return R.error("可启用账户数量不足");
            }
        }

        //账号 用户姓名 账户角色 手机号 邮箱 学号工号
        UpdateWrapper<HrUserAccount> updateWrapper = new UpdateWrapper<>();
        updateWrapper.set("account", hrUserInfo.getAccount());
        updateWrapper.set("work_number", hrUserInfo.getWorkNumber());

        //查询原来该用户的角色信息
        List<UserRoleResp> roleRespList = userRoleService.getUserRoleByAccountId(hrUserInfo.getAccountId());

        if (roleRespList.size() > 0) {
            for (UserRoleResp roleResp : roleRespList) {
                //删除该用户原有角色
                userClient.delRoleByAccountId(roleResp.getRoleId(), hrUserInfo.getAccountId());
            }
        }

        //重新账号赋予角色
        userClient.rolePermissionService(hrUserInfo.getAccountId(), hrUserInfo.getRoleIds(),null);
        if (hrUserInfo.getIsEnable() != null) {
            updateWrapper.set("is_enable", hrUserInfo.getIsEnable());
        }
        updateWrapper.eq("id", hrUserInfo.getAccountId());
        updateWrapper.set("phone", hrUserInfo.getPhone());
        if (hrUserInfo.getPhone() != null) {
            updateWrapper.set("phone_binding_time", DateUtils.getNowTime());
        }

        /*if (hrUserInfo.getPhone() != null||!hrUserInfo.getPhone().equals("")) {
            updateWrapper.set("phone_binding_time", new Date());
        }*/
        boolean ret = accountService.update(new HrUserAccount(), updateWrapper);

        UpdateWrapper<HrUserInfo> userWrapper = new UpdateWrapper<HrUserInfo>();
        userWrapper.set("user_name", hrUserInfo.getUserName());
        userWrapper.set("email", hrUserInfo.getEmail());
        userWrapper.eq("user_id", hrUserInfo.getUserId());

        boolean retUser = service.update(new HrUserInfo(), userWrapper);

        return ret && retUser ? R.ok() : R.error();

    }

    /**
     * 校验账号以及学号或工号
     *
     * @param account
     * @param workNumber
     * @param type
     * @return
     */
    @GetMapping("/checkAccount")
    @ApiOperation(value = "校验账户")
    public R checkAccount(@RequestParam(name = "account", required = false) @ApiParam(value = "需要校验的账号") String account,
                          @RequestParam(name = "workNumber", required = false) @ApiParam(value = "需要校验的工号/学号") String workNumber,
                          @RequestParam(name = "type", required = true) @ApiParam(value = "1.账号 2.学号/工号") Integer type,
                          @RequestParam(name = "platformId", required = true) @ApiParam(value = "平台 数据平台为2") Integer platformId,
                          HttpServletRequest request) {

        String schoolId = request.getHeader("schoolId");

        String msg = "";
        QueryWrapper<HrUserAccount> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_del", DelConstant.NOT_DEL);
        queryWrapper.eq("platform_id", platformId);

        if (type == 1) {
            queryWrapper.eq("account", account);
            msg = "账号";
        } else if (type == 2) {
            queryWrapper.eq("work_number", workNumber);
            queryWrapper.eq("school_id", schoolId);
            msg = "学号/工号";
        }

        HrUserAccount accountList = accountService.getOne(queryWrapper);
        if (accountList != null) {
            return R.error("该" + msg + "已被使用，请重新设置！");
        }

        return R.ok();
    }

    /**
     * 数据平台——用户管理列表
     *
     * @param req
     * @return
     */
    @PostMapping("/getUserList")
    @ApiOperation(value = "数据平台——用户管理列表", response = DataUserInfoResp.class)
    public R getUserList(@RequestBody DataUserListReq req, HttpServletRequest request) {
        String accountId = TokenUtils.getIdByJwtToken(request);
        req.setSchoolId(userClient.getSchoolIdByAccountId(accountId));
        IPage<DataUserInfoResp> getDataUserList = accountService.getDataUserList(req);
        return R.ok().put("pageList", getDataUserList);
    }

    /**
     * 本校累计用户数量
     *
     * @param schoolId
     * @return
     */
    @GetMapping("/getUserTotalBySchoolId")
    @ApiOperation(value = "本校累计用户数量")
    public R userTotalBySchool(@RequestParam(name = "schoolId", required = true) Integer schoolId) {
        return accountService.getUserTotalBySchoolId(schoolId);
    }

    /**
     * 查看用户详情
     *
     * @param
     * @return
     */
    @PostMapping("/getDetail")
    @ApiOperation(value = "查看用户详情")
    public R getDetail(@ApiParam(name = "accountId", value = "账号id", required = true) Integer accountId) {

        //查询原来该用户的角色信息
        List<UserRoleResp> roleRespList = userRoleService.getUserRoleByAccountId(accountId);
        DataUserInfoResp detail = service.getDetail(accountId);
        detail.setRoleId(roleRespList);

        return R.ok().put("data", detail);

    }

    /**
     * 批量导入用户信息
     *
     * @param file
     * @return
     */
    @ApiOperation(value = "批量导入用户信息")
    @PostMapping("/uploadFile")
    public R uploadFile(@ApiParam(name = "file", value = "用户信息文件", required = true) MultipartFile file
            , HttpServletRequest request) throws Exception {
        String accountId = TokenUtils.getIdByJwtToken(request);
        Integer schoolId = userClient.getSchoolIdByAccountId(accountId);
        Map<String, String> upload = service.upload(file, schoolId);
        return R.ok().put("data", upload);
    }

    /**
     * 批量导入用户信息失败数据导出
     *
     * @param response
     * @param token
     * @throws Exception
     */
    @ApiOperation(value = "批量导入用户信息失败数据导出")
    @GetMapping("/export_failure")
    public void exportFailureRecord(HttpServletResponse response, @RequestParam String token) throws Exception {
        service.exportFailureRecord(response, token);
    }

    /**
     * 批量逻辑删除数据平台用户(默认情况下只针对account表删除)
     * 当 当前用户在平台只有一个账号时，删除账号的同时需要删除用户
     *
     * @param accountIds
     * @return
     */
    @PostMapping("/delete")
    @ApiOperation(value = "根据主键删除")
    public R delete(@ApiParam(name = "accountIds", value = "多个账号id", required = true) @RequestParam("accountIds") List<Integer> accountIds) {
        boolean remove = false;
        if (accountIds != null && accountIds.size() != 0) {
            for (Integer id : accountIds) {
                //remove = service.removeById(id);

                //查询原来该用户的角色信息
                List<UserRoleResp> roleRespList = userRoleService.getUserRoleByAccountId(id);

                if (roleRespList.size() > 0) {
                    for (UserRoleResp roleResp : roleRespList) {
                        //删除该用户原有角色
                        userClient.delRoleByAccountId(roleResp.getRoleId(), id);
                    }
                }

                //查询该id是否有绑定相同的userId
                QueryWrapper<HrUserAccount> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("id", id);
                queryWrapper.eq("is_del", DelConstant.NOT_DEL);
                //获取到该绑定的userId
                HrUserAccount account = accountService.getOne(queryWrapper);

                //查询相同的userId
                QueryWrapper<HrUserAccount> queryWrapper1 = new QueryWrapper<>();
                queryWrapper1.eq("user_id", account.getUserId());
                queryWrapper1.eq("is_del", DelConstant.NOT_DEL);

                //该用户绑定的账户小于等于0时 同时删除user表和account表的数据
                List<HrUserAccount> accountList = accountService.list(queryWrapper1);
                if (accountList.size() <= 1) {
                    service.update(new HrUserInfo(), new UpdateWrapper<HrUserInfo>().set("is_del", DelConstant.IS_DEL).eq("user_id", account.getUserId()));
                }

                remove = accountService.update(new HrUserAccount(), new UpdateWrapper<HrUserAccount>().set("is_del", DelConstant.IS_DEL).eq("id", id));
            }
        }

        return remove ? R.ok() : R.error("删除失败");

    }

    /**
     * @Description :   根据token获取当前账户学校id
     * @Param req
     * @Param request
     * @Author Rong---2021/11/9
     */
    @PostMapping("/getSchoolIdByToken")
    @ApiOperation(value = "根据token获取当前账户学校id")
    public R getSchoolIdByToken(HttpServletRequest request) {
        String accountId = TokenUtils.getIdByJwtToken(request);
        Integer schoolId = userClient.getSchoolIdByAccountId(accountId);
        return R.ok().put("schoolId", schoolId);
    }
}
