package com.huoran.users.controller;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.huoran.common.constant.DelConstant;
import com.huoran.common.constant.SMSTemplate;
import com.huoran.common.response.R;
import com.huoran.common.utils.*;

import com.huoran.users.entity.HrUserAccount;
import com.huoran.users.entity.HrUserInfo;
import com.huoran.users.entity.req.ResetPwdReq;
import com.huoran.users.entity.req.UserInfoReq;
import com.huoran.users.entity.res.ProfileDetailsRes;
import com.huoran.users.entity.res.UserInfoDetailsRes;
import com.huoran.users.entity.res.UserSysDetailRes;
import com.huoran.users.entity.vo.CheckVo;
import com.huoran.users.service.IHrUserAccountService;
import com.huoran.users.service.IHrUserInfoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * <p>
 * 各个平台账号表 前端控制器
 * </p>
 *
 * @author Qyq
 * @since 2021-06-18
 */
@Api(value = "用户管理", tags = "用户管理")
@RestController
@RequestMapping("/users/userAccount")
public class HrUserAccountController {

    @Autowired
    private IHrUserAccountService iHrUserAccountService;

    @Autowired
    private IHrUserInfoService iHrUserInfoService;

    /**
     * 查询用户管理列表
     */
    @ApiOperation(value = "查询用户管理列表", notes = "查询用户管理列表")
    @PostMapping("/queryUserInfoList")
    public R queryUserInfoList(@RequestBody UserInfoReq userInfoReq) {
        return iHrUserAccountService.queryPage( userInfoReq);
    }

    /**
     * 密码重置，以手机号的维度，重置所有账号密码为1122aa
     */
    @ApiOperation(value = "密码重置")
    @GetMapping("/resetPwd")
    public R resetPwd(@RequestParam("userId") Integer userId) {
        int result = iHrUserAccountService.resetPwd(userId);
        return result > 0 ? R.ok() : R.error();
    }

    /**
     * 删除用户以及账号
     */
    @ApiOperation(value = "删除用户以及账号")
    @GetMapping("/delUserAccount")
    public R delUserAccount(@RequestParam("userId") Integer userId, @RequestParam("isdel") Integer isdel) {
        int result = iHrUserAccountService.delUserAccount(userId, isdel);
        return result > 0 ? R.ok() : R.error();
    }

    /**
     * 批量删除用户以及账号
     *
     * @param ids 用户id
     * @return
     */
    @PostMapping("/delUserAccounts")
    @ApiOperation(value = "批量删除用户以及账号")
    public R delUserAccounts(@RequestBody List<String> ids) {
        return iHrUserAccountService.delUserAccounts(ids);
    }


    @PostMapping("/deletePrompt")
    @ApiOperation(value = "批量删除用户，为企业负责人，则提示：该用户为组织负责人，删除后会同步删除该组织，确定删除？")
    public R deletePrompt(@RequestBody List<String> ids) {
        String tips = null;
        for (String id : ids) {
            int team = iHrUserAccountService.selectTeam(id);
            if (team > 0){
                tips = "该用户为组织负责人，删除后会同步删除该组织，确定删除？";
            }
        }
        return R.ok().put("tips",tips);
    }

    /**
     * 查看用户系统绑定详情
     */
    @ApiOperation(value = "查看用户系统绑定详情", response = UserSysDetailRes.class)
    @GetMapping("/selectUserSysBind")
    public R selectUserSysBind(@RequestParam("userId") Integer userId, @RequestParam("platformId") Integer platformId) {
        UserSysDetailRes result = iHrUserAccountService.selectUserSysBind(userId, platformId);
        return R.ok().put("result", result);
    }

    /**
     * 查看个人账号信息详情
     */
    @ApiOperation(value = "查看个人账号信息详情", response = HrUserAccount.class)
    @GetMapping("/selectAccountDetail")
    public R selectAccountDetail(@RequestParam("accountId") Integer accountId) {
        HrUserAccount result = iHrUserAccountService.selectAccountDetail(accountId);
        return R.ok().put("result", result);
    }

    /**
     * 查看用户详情
     *
     * @return
     */
    @ApiOperation(value = "查看用户详情", response = UserInfoDetailsRes.class)
    @GetMapping("/queryUserInfoDetails")
    public R queryUserInfoDetails(@Param("userId") Integer userId, HttpServletRequest request) {
        String accountId = TokenUtils.getIdByJwtToken(request);
        if (userId == null) {
            userId = iHrUserAccountService.getUserIdByAccountId(accountId);
        }
        UserInfoDetailsRes result = iHrUserAccountService.queryUserInfoDetails(userId);
        return R.ok().put("result", result);
    }

    /**
     * 修改用户头像
     *
     * @return
     */
    @ApiOperation(value = "修改用户头像")
    @PostMapping("/updateUserAvatars")
    public R updateUserAvatars(HttpServletRequest request, MultipartFile file) {
        String accountId = TokenUtils.getIdByJwtToken(request);
        Integer userId = iHrUserAccountService.getUserIdByAccountId(accountId);
        return iHrUserAccountService.updateUserAvatars(userId, file);
    }

    /**
     * 修改账号禁用启用功能
     *
     * @param id
     * @param isEnable
     * @return
     */
    @ApiOperation(value = "修改账号禁用启用功能")
    @GetMapping("/updateAccountEnable")
    public R updateAccountEnable(@RequestParam("id") @ApiParam(value = "账号id") Integer id, @RequestParam("isEnable") Integer isEnable) {
        int result = iHrUserAccountService.updateAccountEnable(id, isEnable);
        return result > 0 ? R.ok() : R.error();
    }

    /**
     * @Description :   中台一键禁用——修改用户下所有账号禁用启用功能
     * @Param id
     * @Param isEnable
     * @Author Rong---2022/4/20
     */
    @ApiOperation(value = "中台一键禁用——修改用户下所有账号禁用启用功能")
    @GetMapping("/updateAccountAllEnable")
    public R updateAccountAllEnable(@RequestParam("id") @ApiParam(value = "用户id") Integer id,
                                    @RequestParam("isEnable") @ApiParam(value = "是否禁用（0->禁用，1->启用）") Integer isEnable) {
        int result = iHrUserAccountService.updateAccountAllEnable(id, isEnable);
        return result > 0 ? R.ok() : R.error();
    }

    /**
     * 修改个人中心
     *
     * @param req
     * @return
     */
    @ApiOperation(value = "修改个人中心")
    @PostMapping("/updatePersonCenter")
    public R updatePersonCenter(@RequestBody UserInfoDetailsRes req) {
        int result = iHrUserAccountService.updatePersonCenter(req);
        return result > 0 ? R.ok() : R.error();
    }

    /**
     * 更换手机号或邮箱--发送手机验证码
     *
     * @return
     */
    @ApiOperation(value = "更换手机号或邮箱--发送手机验证码")
    @PostMapping("/sendPhoneOrEmailCode")
    public R sendPhoneOrEmailCode(@RequestBody CheckVo checkVo) {

        R result = iHrUserInfoService.sendCheckPhoneCode(checkVo);
        return result;
    }

    /**
     * 绑定手机或邮箱
     *
     * @param checkVo
     * @return
     */
    @ApiOperation(value = "绑定手机或邮箱")
    @PostMapping("/bindPhoneOrEmail")
    public R bindPhoneOrEmail(@RequestBody CheckVo checkVo, HttpServletRequest request) {
        String accountId = TokenUtils.getIdByJwtToken(request);
        String token = "";
        //未登录情况判断,并返回token
        if (!StrUtil.isNotEmpty(accountId)) {
            accountId = iHrUserAccountService.getAccountId(checkVo);
//            token = JwtUtils.getJwtToken(accountId);
        }
        Integer userId = iHrUserAccountService.getUserIdByAccountId(accountId);
        R result = iHrUserInfoService.bindPhone(checkVo, userId);
        result.put("token", token);
        return result;
    }

    /**
     * 校验更换密码
     *
     * @param resetPwdReq
     * @return
     */
    @ApiOperation(value = "校验更换密码")
    @PostMapping("/examinePassword")
    public R examinePassword(@RequestBody ResetPwdReq resetPwdReq, HttpServletRequest request) {
        String accountId = TokenUtils.getIdByJwtToken(request);
        resetPwdReq.setAccountId(Integer.valueOf(accountId));
        R result = iHrUserAccountService.examinePassword(resetPwdReq);
        return result;
    }

    /**
     * 根据账号id获取用户id
     */
    @GetMapping("/getUserId")
    @ApiOperation(value = "根据账号id获取用户id")
    public Integer getUserIdByAccountId(@RequestParam String accountId) {
        return iHrUserAccountService.getUserIdByAccountId(accountId);
    }

    /**
     * 根据账号id获取学校id
     */
    @GetMapping("/getSchoolIdByAccountId")
    @ApiOperation(value = "根据账号id获取学校id")
    public Integer getSchoolIdByAccountId(@RequestParam("accountId") String accountId) {
        QueryWrapper<HrUserAccount> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_del", DelConstant.NOT_DEL);
        queryWrapper.eq("id", accountId);

        Integer schoolId = iHrUserAccountService.getOne(queryWrapper).getSchoolId();
        return schoolId;
    }

    /**
     * 账号注册（数据平台注册）
     */
    @ApiOperation(value = "账号注册")
    @PostMapping("/registerAccount")
    public R registerAccount(@RequestBody ResetPwdReq resetPwdReq, HttpServletRequest request) {

        return R.ok();
    }

    @ApiOperation(value = "中台查看用户详情", response = UserInfoDetailsRes.class)
    @GetMapping("/viewUserDetailsforNakadai")
    public R viewUserDetailsforNakadai(@RequestParam @Param("userId") Integer userId, HttpServletRequest request) {
        String accountId = TokenUtils.getIdByJwtToken(request);
        if (userId == null) {
            userId = iHrUserAccountService.getUserIdByAccountId(accountId);
        }
        UserInfoDetailsRes result = iHrUserAccountService.viewUserDetailsforNakadai(userId);
        return R.ok().put("result", result);
    }

    @ApiOperation(value = "查看用户详情", response = UserInfoDetailsRes.class)
    @GetMapping("/viewUserDetails")
    public R viewUserDetails(@RequestParam @NotNull(message = "openId为空查询错误") @Param("userId") String openId) {
        UserInfoDetailsRes result = iHrUserAccountService.viewUserDetails(openId);
        return R.ok().put("result", result);
    }

    @ApiOperation(value = "中台一键启用前——用户判断是否能够一键启用(判断当前用户是否有数据平台的账号绑定，需要判断院校可用账号数量)")
    @PostMapping("/oneClickEnableVerification")
    public R oneClickEnableVerification(@RequestParam("accountId") @ApiParam(value = "账号id") Integer accountId) {
        return iHrUserAccountService.oneClickEnableVerification(accountId);
    }

    @ApiOperation(value = "个人档案", response = UserInfoDetailsRes.class)
    @GetMapping("/personalFile")
    public R personalFile(@RequestParam("userId") @Param("userId") Integer userId, HttpServletRequest request) {
        String accountId = TokenUtils.getIdByJwtToken(request);
        if (userId == null) {
            userId = iHrUserAccountService.getUserIdByAccountId(accountId);
        }
        ProfileDetailsRes result = iHrUserAccountService.personalFile(userId);
        return R.ok().put("result", result);
    }

    @ApiOperation(value = "解绑手机号(该接口可用于token、账号id参数)")
    @GetMapping("/unbindMobilePhone")
    public R unbindMobilePhone(@RequestParam(value = "accountId", required = false) @ApiParam("accountId") Integer accountId, HttpServletRequest request) {
        String account_id = TokenUtils.getIdByJwtToken(request);//token解析出来的账号id

        if (accountId == null) {
            return iHrUserAccountService.unbindMobilePhone(Integer.valueOf(account_id));
        }

        return iHrUserAccountService.unbindMobilePhone(accountId);

    }

    @ApiOperation(value = "根据学校学号查询学校学生")
    @GetMapping("/enquireAboutSchoolStudents")
    public R enquireAboutSchoolStudents(@RequestParam Integer schoolId,@RequestParam String workNumber,@RequestParam(required = false) Integer applyFor) {
        QueryWrapper<HrUserAccount> wrapper = new QueryWrapper<>();
        wrapper.eq("school_id",schoolId);
        wrapper.eq("work_number",workNumber);
        wrapper.eq("platform_id",1);
        wrapper.eq("type",1);
        HrUserAccount one = iHrUserAccountService.getOne(wrapper);
        if (one==null && applyFor!=null && applyFor == 1){
            return R.ok();
        }
        if (one==null){
            return R.error("学生不存在");
        }
        if (one.getIsEnable()==0){
            return R.error("该学生已禁用，请启用后再进行操作");
        }
        if (applyFor!=null && applyFor == 1){
            return R.error("学生已存在");
        }
        HrUserInfo byId = iHrUserInfoService.getById(one.getUserId());
        one.setUserName(byId.getUserName());
        return R.ok().put("account", one);
    }

}

