package com.huoran.users.controller;

import com.huoran.common.constant.PlatformConstant;
import com.huoran.common.response.R;
import com.huoran.common.utils.MD5;
import com.huoran.common.utils.TokenUtils;
import com.huoran.users.entity.HrPersonalFile;
import com.huoran.users.entity.HrUserAccount;
import com.huoran.users.entity.HrUserInfo;
import com.huoran.users.entity.vo.UserPersonalEntityVo;
import com.huoran.users.service.IHrPersonalFileService;
import com.huoran.users.service.IHrUserAccountService;
import com.huoran.users.service.IHrUserInfoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * <p>
 * 用户信息表 前端控制器
 * </p>
 *
 * @author huoran
 * @since 2021-06-28
 */
@RestController
@RequestMapping("/users/userInfo")
@Api(value = "用户管理", tags = "用户管理")
public class HrUserInfoController {

    @Autowired
    private IHrUserInfoService iHrUserInfoService;

    @Autowired
    private IHrUserAccountService iHrUserAccountService;

    @Autowired
    private IHrPersonalFileService iHrPersonalFileService;

    @Autowired
    private IHrPersonalFileService personalFileService;

    /**
     * 查询用户信息
     *
     * @param request
     * @return
     */
    @GetMapping("/getUserInfo")
    @ApiOperation(value = "查询用户信息")
    public R userInfo(HttpServletRequest request) {

        String accountId = TokenUtils.getIdByJwtToken(request);
        Integer userId = iHrUserAccountService.getUserIdByAccountId(accountId);

        return R.ok(iHrUserInfoService.queryUserInfoById(userId, Integer.valueOf(accountId)));
    }

    /**
     * 修改更新用户
     */
    @PostMapping("/updateUser")
    @ApiOperation(value = "更新用户")
    public R updateUser(@RequestBody UserPersonalEntityVo vo) {

        HrUserInfo userInfo = vo.getHrUserInfo();
        HrUserAccount userAccount = vo.getHrUserAccount();
        Integer userId = userInfo.getUserId();

        //验证账号是否存在 -- 根据平台id
        int num = iHrUserInfoService.queryAccountExistByPlatformId(userAccount.getAccount(), userAccount.getUserId(), PlatformConstant.DATA_PLATFORM);
        if (num >= 1) {
            return R.error(500, "更新失败，账号不能重复");
        }

        //更新用户
        iHrUserInfoService.updateUserInfoById(userInfo);
        //更新账户
        Integer count = iHrUserAccountService.updateAccount(userAccount);
        if (count <= 0) {
            return R.error(500, "账号更新失败！");
        }
        if (vo.getHrPersonalFileEntities()!=null){
            List<HrPersonalFile> entityList = vo.getHrPersonalFileEntities();
            for (int i = 0; i < entityList.size(); i++) {
                HrPersonalFile personalFile = entityList.get(i);
                Integer personalFileId = personalFile.getPersonalFileId();
                if (personalFile.getPersonalCareerId() == null) {
                    return R.error(403, "false,personalCareerId can not be empty!!!");
                }
                if (personalFileId == null) {
                    iHrPersonalFileService.insertPersonalInfo(personalFile);
                } else {
                    iHrPersonalFileService.updateUserPersonal(personalFile);
                }
            }
        }
        return R.ok();
    }

    /**
     * @Description :   校验用户手机号是否存在
     * @Param phone
     * @Author Rong---2021/10/18
     */
    @GetMapping("/checkPhoneOrEmailExist")
    @ApiOperation(value = "校验用户手机号或邮箱是否存在")
    public boolean checkPhoneOrEmailExist(@ApiParam(name = "phone", value = "要校验的手机号码", required = false) @RequestParam("phone") String phone,
                                          @ApiParam(name = "email", value = "要校验的邮箱", required = false) @RequestParam("email") String email,
                                          @ApiParam(name = "type", value = "校验类型", required = false) @RequestParam("type") Integer type) {
        return iHrUserInfoService.checkPhoneOrEmailExist(phone, email, type);
    }

    @PostMapping("/deleteProfile")
    @ApiOperation(value = "删除个人档案")
    public R deleteProfile(@ApiParam(name = "personalFileIds", value = "个人档案id", required = true) @RequestParam("personalFileIds") List<Integer> personalFileIds) {
        boolean ret = false;
        if (personalFileIds.size() > 0) {
            ret = personalFileService.removeByIds(personalFileIds);
        }
        return ret ? R.ok() : R.error("删除个人档案失败");

    }

    /**
     * @Description :   查询用户名称
     * @Param id
     * @Author Rong---2021/12/30
     */
    @GetMapping("/getUserName")
    @ApiOperation(value = "查询用户名称")
    public String getUserName(@RequestParam("accountId") Integer accountId) {
        return iHrUserInfoService.getUserName(accountId);
    }
}
