package com.huoran.users.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.huoran.common.constant.DelConstant;
import com.huoran.common.response.R;
import com.huoran.common.utils.MD5;
import com.huoran.users.entity.HrUserAccount;
import com.huoran.users.entity.HrUserInfo;
import com.huoran.users.entity.req.DataUserListReq;
import com.huoran.users.entity.req.ModifyUserInfoReq;
import com.huoran.users.entity.req.SaveUserInfoReq;
import com.huoran.users.entity.res.DataUserInfoResp;
import com.huoran.users.service.IHrUserAccountService;
import com.huoran.users.service.IHrUserInfoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 用户信息表 前端控制器
 * </p>
 *
 * @author huoran
 * @since 2021-06-28
 */
@RestController
@RequestMapping("/users/occupationlab/user")
@Api(value = "API - 用户管理", tags = "职站-用户管理")
public class OccupationlabUserInfoController {

    @Autowired
    public IHrUserInfoService service;

    @Autowired
    public IHrUserAccountService accountService;

    /**
     * 按帐户获取详细信息
     *
     * @param
     * @return
     */
    @PostMapping("/getDetailByAccount")
    @ApiOperation(value = "查看用户详情")
    public R getDetailByAccount(@ApiParam(name = "account", value = "输入的学生账号", required = true) String account,
                                @ApiParam(name = "schoolId", value = "学校id", required = true) Integer schoolId,
                                @ApiParam(name = "platformId", value = "所属平台id(1.职站 2.数据平台 3.中台)", required = true) Integer platformId,
                                @ApiParam(name = "type", value = "平台端区分:0教师 1学生 2无端", required = true) Integer type) {
        //查询同校且同平台以及同端下
        QueryWrapper<HrUserAccount> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("school_id", schoolId);
        queryWrapper.eq("platform_id", platformId);
        queryWrapper.eq("type", type);
        queryWrapper.eq("account", account);
        queryWrapper.eq("is_enable", DelConstant.IS_ENABLE);
        return R.ok().put("data", accountService.getOne(queryWrapper));

    }

    /**
     * 批量逻辑删除职站平台用户(默认情况下只针对account表删除)
     * 删除账号的同时需要删除用户
     *
     * @param accountIds
     * @return
     */
    @PostMapping("/delStudent")
    @ApiOperation(value = "根据主键删除")
    public void delStudent(@ApiParam(name = "accountIds", value = "多个账号id", required = true) @RequestParam("accountIds") List<Integer> accountIds) {

        if (accountIds != null && accountIds.size() != 0) {
            for (Integer id : accountIds) {

                //查询该id是否有绑定相同的userId
                QueryWrapper<HrUserAccount> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("is_del", DelConstant.NOT_DEL);
                queryWrapper.eq("id", id);

                //获取到该绑定的userId
                HrUserAccount account = accountService.getOne(queryWrapper);

                //查询相同的userId
                QueryWrapper<HrUserAccount> queryWrapper1 = new QueryWrapper<>();
                queryWrapper1.eq("is_del", DelConstant.NOT_DEL);
                queryWrapper1.eq("user_id", account.getUserId());
                //该用户绑定的账户小于等于0时 同时删除user表和account表的数据
                HrUserAccount userAccount = accountService.getOne(queryWrapper1);
                if (userAccount != null) {
                    service.update(new HrUserInfo(), new UpdateWrapper<HrUserInfo>()
                            .set("is_del", DelConstant.IS_DEL)
                            .eq("user_id", account.getUserId()));
                }

                accountService.update(new HrUserAccount(), new UpdateWrapper<HrUserAccount>()
                        .set("is_del", DelConstant.IS_DEL)
                        .eq("id", id));
            }
        }
    }

    /**
     * @Description :   根据账号id删除账号、用户表信息
     * @Param accountId
     * @Author Rong---2021/10/14
     */
    @PostMapping("/delAccountAndUser")
    @ApiOperation(value = "删除账号、用户表信息")
    public void delAccountAndUser(@ApiParam(name = "accountId", value = "账号id", required = true) @RequestParam("accountId") Integer accountId) {

        //查询该id是否有绑定相同的userId
        QueryWrapper<HrUserAccount> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_del", DelConstant.NOT_DEL);
        queryWrapper.eq("id", accountId);

        //获取到该绑定的userId
        HrUserAccount account = accountService.getOne(queryWrapper);

        //查询相同的userId
        QueryWrapper<HrUserAccount> queryWrapper1 = new QueryWrapper<>();

        queryWrapper1.eq("is_del", DelConstant.NOT_DEL);
        queryWrapper1.eq("user_id", account.getUserId());

        //该用户绑定的账户小于等于0时 同时删除user表和account表的数据
        List<HrUserAccount> accountList = accountService.list(queryWrapper1);
        if (accountList.size() <= 1) {
            service.update(new HrUserInfo(), new UpdateWrapper<HrUserInfo>()
                    .set("is_del", DelConstant.IS_DEL)
                    .eq("user_id", account.getUserId()));
        }

        accountService.update(new HrUserAccount(), new UpdateWrapper<HrUserAccount>()
                .set("is_del", DelConstant.IS_DEL)
                .eq("id", accountId));

    }
}


