package com.huoran.users.controller;


import com.huoran.common.response.R;
import com.huoran.users.entity.Permission;
import com.huoran.users.service.PermissionService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * <p>
 * 权限表 前端控制器
 * </p>
 *
 * @author chen
 * @since 2021-09-29
 */
@Api(tags = "权限菜单管理")
@RestController
@RequestMapping("/users/permission")
public class PermissionController {

    @Autowired
    private PermissionService permissionService;

    @ApiOperation(value = "根据平台id查询所有菜单")
    @GetMapping("queryAllMenus")
    public R indexAllPermission(
            @ApiParam(name = "platformId", value = "平台id", required = true)
            @RequestParam Integer platformId,
            @ApiParam(name = "port", value = "默认pc端为0、小程序端为1", required = false)
            @RequestParam(required = false) Integer port) {
        List<Permission> list = permissionService.queryAllMenu(platformId,port);
        return R.ok().put("children",list);
    }

    @ApiOperation(value = "递归删除菜单")
    @PostMapping("remove")
    public R remove(@RequestParam Integer id) {
        permissionService.removeChildById(id);
        return R.ok();
    }

    @ApiOperation(value = "给角色分配权限")
    @PostMapping("/doAssign")
    public R doAssign(Integer roleId,Integer[] permissionId) {
        permissionService.saveRolePermissionRelationShip(roleId,permissionId);
        return R.ok();
    }

/*    @ApiOperation(value = "获取角色权限菜单")
    @GetMapping("obtain")
    public R obtain(@RequestParam String roleId) {
        List<Permission> list = permissionService.selectRoleMenu(roleId);
        return R.ok().put("permissionMenu", list);
    }*/

    @ApiOperation(value = "新增菜单")
    @PostMapping("save")
    public R save(@RequestBody Permission permission) {
        permissionService.save(permission);
        return R.ok();
    }

    @ApiOperation(value = "修改菜单")
    @PostMapping("update")
    public R updateById(@RequestBody Permission permission) {
        permissionService.updateById(permission);
        return R.ok();
    }

}

