package com.huoran.users.controller;

import com.huoran.api.UserClient;
import com.huoran.common.response.R;
import com.huoran.common.utils.TokenUtils;
import com.huoran.users.entity.UserAuthenticationInformation;
import com.huoran.users.service.UserAuthenticationInformationService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * @描述：用户实名认证信息记录控制类
 * @作者: Rong
 * @日期: 2023-12-14
 */
@RestController
@RequestMapping("/userAuthenticationInformation")
@Api(value = "用户实名认证信息记录:UserAuthenticationInformationController", tags = "用户实名认证信息记录")
public class UserAuthenticationInformationController {

    @Autowired
    public UserAuthenticationInformationService service;

    @Autowired
    private UserClient userClient;


    @PostMapping("/findById")
    @ApiOperation(value = "查询详情", response = UserAuthenticationInformation.class)
    public R findById(@ApiParam(name = "id", value = "主键", required = true) @RequestParam Integer id) {
        UserAuthenticationInformation userAuthenticationInformation = service.getById(id);
        return R.ok().put("data", userAuthenticationInformation);
    }


    @PostMapping("/save")
    @ApiOperation(value = "新增", response = UserAuthenticationInformation.class)
    public R save(@RequestBody @ApiParam(name = "用户实名认证信息记录对象", value = "传入json格式", required = true) UserAuthenticationInformation userAuthenticationInformation, HttpServletRequest request) {
        String accountId = TokenUtils.getIdByJwtToken(request);
        Integer schoolId = userClient.getSchoolIdByAccountId(accountId);
        userAuthenticationInformation.setAccountId(Integer.valueOf(accountId));
        boolean addState = service.save(userAuthenticationInformation);
        return addState ? R.ok() : R.error("新增失败");
    }





    @PostMapping("/batchDeletion")
    @ApiOperation(value = "批量删除", response = UserAuthenticationInformation.class)
    public R batchDeletion(@ApiParam(name = "ids", value = "主键", required = true) @RequestBody List<Integer> ids) {
        boolean delState = service.removeByIds(ids);
        return delState ? R.ok() : R.error("删除失败");
    }
}

