package com.huoran.users.controller;

import com.huoran.common.constant.LoginDistConstant;
import com.huoran.common.constant.OrderConstant;
import com.huoran.common.response.R;
import com.huoran.common.utils.TokenUtils;
import com.huoran.users.entity.HrUserAccount;
import com.huoran.users.entity.PlatformLogin;
import com.huoran.users.entity.dto.LoginDTO;
import com.huoran.users.entity.req.AccountRegisterReq;
import com.huoran.users.entity.req.AppletCallbackReq;
import com.huoran.users.entity.req.UnbindAccountsReq;
import com.huoran.users.entity.req.WxUserBindingReq;
import com.huoran.users.entity.vo.LoginVo;
import com.huoran.users.entity.vo.WxMssVo;
import com.huoran.users.service.IHrUserAccountService;
import com.huoran.users.service.IHrUserInfoService;
import com.huoran.users.service.PlatformLoginService;
import com.huoran.users.service.UserRoleService;
import com.huoran.users.weixin.mp.aes.AesException;
import com.huoran.users.weixin.mp.aes.SHA1;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.List;
import java.util.Map;

/**
 * @Author chen
 * @DATE 2021/7/7 15:16
 * @Version 1.0
 */
@Api(tags = "登录管理")
@RestController
@RequestMapping("/users/user")
public class UserLoginController {

    @Autowired
    private IHrUserInfoService userInfoService;

    @Autowired
    private IHrUserAccountService userAccountService;

    @Autowired
    private PlatformLoginService platformLoginService;

    @Autowired
    private UserRoleService userRoleService;

    @Autowired
    HrUserAccountController accountController;

    @ApiOperation(value = "根据手机号查询当前平台的其他账号用户信息")
    @PostMapping(value = "/getOtherAccountByPhone")
    public R getOtherAccountByPhone(
            @ApiParam(name = "phone", value = "手机号", required = true)
            @RequestParam String phone,
            @ApiParam(name = "platform", value = "平台id", required = true)
            @RequestParam String platform){
        List<HrUserAccount> hrUserAccounts = userInfoService.getOtherAccount(phone, platform);
        return R.ok().put("userAccounts",hrUserAccounts);
    }

    @ApiOperation(value = "根据手机号解除所有绑定账号且与当前用户进行绑定")
    @PostMapping(value = "/unbindAccounts")
    public R unbindAccounts(@RequestBody @Valid UnbindAccountsReq accountsReq){
        return userInfoService.unbindAccounts(accountsReq);
    }

    /**
     * 登录发送手机/邮箱验证码
     */
    @ApiOperation(value = "登录发送手机/邮箱验证码")
    @PostMapping("/sendPhoneVerificationCode")
    public R sendPhoneVerificationCode(
            @ApiParam(name = "phoneOrEmail", value = "手机或邮箱", required = true)
            @RequestParam String phoneOrEmail,
            @ApiParam(name = "loginOrBind", value = "登录状态为0、绑定状态为1", required = true)
            @RequestParam Integer loginOrBind,
            @ApiParam(name = "platform", value = "平台id", required = true)
            @RequestParam String platform) {
        return userInfoService.sendPhoneVerificationCode(phoneOrEmail,loginOrBind,platform);
    }

    /**
     * 根据账号id返回token
     */
    @ApiOperation(value = "根据账号id返回token")
    @PostMapping("/getToken")
    public R getToken(@RequestParam String id) {
        return userInfoService.getToken(id);
    }

    @ApiOperation(value = "账号是否禁用")
    @GetMapping(value = "/accountIsDisabled")
    public boolean accountIsDisabled(@RequestParam("token") String token){
        String id = TokenUtils.getIdByJwtToken(token);
        HrUserAccount hrUserAccount = userAccountService.getById(id);
        //是否禁用
        return hrUserAccount.getIsEnable().equals(OrderConstant.Not_enabled);
    }

    @ApiOperation(value = "平台登录统计")
    @GetMapping(value = "/platformLoginStatistics")
    public R platformLoginStatistics(
            @ApiParam(name = "platform", value = "平台id", required = true)
            @RequestParam Integer platform,HttpServletRequest request){
        String id = TokenUtils.getIdByJwtToken(request);
        Integer schoolId = null;
        if (id!=null){
            schoolId = userAccountService.querySchoolIdByAccountId(id);
        }
        return platformLoginService.platformLoginStatistics(platform,schoolId);
    }

    @ApiOperation(value = "幼教产品个人端账号快速注册登录")
    @PostMapping("/personalAccountApplication")
    public R personalAccountApplication(@RequestBody @Valid AccountRegisterReq registerReq) {
        return userInfoService.personalAccountApplication(registerReq);
    }



    /**
     * 微信小程序绑定回调
     */
    @ApiOperation(value = "微信小程序绑定回调(幼教产业版)")
    @PostMapping("/weChatAppletCallbackByKindergarten")
    public R weChatAppletCallbackByKindergarten(@RequestBody AppletCallbackReq appletCallbackReq, HttpServletResponse response) throws Exception {

        Map<String ,Object> result = userInfoService.weChatAppletCallbackByKindergarten(appletCallbackReq, response);

        return R.ok().put("data",result);
    }


    /**
     * 登录成功后返回token值，用于其他模块验证
     * @param loginVo 登录信息
     * @return loginVo
     */
    @ApiOperation(value = "登录认证")
    @PostMapping("/login")
    public R login(@RequestBody @Valid LoginVo loginVo) {
        LoginDTO data = userInfoService.login(loginVo);
        String id = TokenUtils.getIdByJwtToken(data.getToken());
        //统计平台登录次数
        if (!data.isCustomer()){
            Integer schoolId = userAccountService.querySchoolIdByAccountId(id);
            PlatformLogin platformLogin = new PlatformLogin();
            platformLogin.setPlatform(loginVo.getPlatform());
            platformLogin.setSchoolId(schoolId);
            platformLoginService.save(platformLogin);
        }
        //手机号登录返回选择数据
        if (loginVo.getDistinguish().equals(LoginDistConstant.PHONE_EMAIL_LOGIN)) {
            return R.ok().put("data", data);
        }
        HrUserAccount account = userAccountService.getById(id);
        String name = account.getAccount();
        List<Integer> roles = userRoleService.queryAccountRoles(id,null);
        if (loginVo.getType()!=1 && roles!=null && roles.isEmpty()){
            return R.error("该账号无角色权限，无法登录，请联系管理员添加角色！");
        }
        return R.ok("认证成功,欢迎 "+name+" 回来！").put("data", data);
    }

    /**
     * 是否为客户
     */
    @ApiOperation(value = "是否为客户")
    @GetMapping("/isClient")
    public R isClient(HttpServletRequest request) {
        String id = TokenUtils.getIdByJwtToken(request);
        HrUserAccount account = userAccountService.getById(id);
        //查询学校id
        Integer schoolId = accountController.getSchoolIdByAccountId(id);
        if (account.getCustomerId()==null){
            return R.ok().put("customer",false).put("schoolId",schoolId);
        }
        String clientName = userAccountService.queryClientName(account.getCustomerId());
        return R.ok().put("customer",true).
                put("customerName",clientName).
                put("customerId",account.getCustomerId()).
                put("schoolId",schoolId);
    }

    @ApiOperation(value = "登录验证码")
    @GetMapping(value = "/captcha",produces = "image/png")
    public void captcha(
            @ApiParam(name = "random", value = "随机数", required = true)
            @RequestParam String random, HttpServletResponse response){
        userInfoService.createCaptcha(random,response);
    }


    /**
     * 微信小程序绑定回调
     */
    @ApiOperation(value = "微信小程序绑定回调(合伙人版)")
    @PostMapping("/weChatAppletCallbackByPartner")
    public R weChatCallbackByPartner(@RequestBody AppletCallbackReq appletCallbackReq, HttpServletResponse response) throws Exception {

        Map<String ,Object> result = userInfoService.weChatAppletCallbackByPartner(appletCallbackReq, response);

        return R.ok().put("data",result);
    }

    @ApiOperation(value = "微信小程序绑定回调(运营版)")
    @PostMapping("/weChatAppletCallbackByOperation")
    public R weChatCallback(@RequestBody AppletCallbackReq appletCallbackReq, HttpServletResponse response) throws Exception {

        Map<String ,Object> result = userInfoService.weChatAppletCallbackByOperation(appletCallbackReq, response);

        return R.ok().put("data",result);
    }


    @ApiOperation(value = "微信小程序授权（获取sessionKey）")
    @PostMapping("/getSessionKey")
    public R getSessionKey(@RequestBody AppletCallbackReq appletCallbackReq, HttpServletResponse response) throws Exception {
        return userInfoService.getSessionKey(appletCallbackReq, response);
    }

    @ApiOperation(value = "微信小程序登录（根据openid）")
    @PostMapping("/loginByOpenid")
    public R loginByOpenid(@RequestParam String openid)  {
        return userInfoService.loginByOpenid(openid);
    }

    @ApiOperation(value = "微信小程序更新用户头像或名称")
    @PostMapping("/updateAvatars")
    public R updateAvatars(
            @ApiParam(name = "url", value = "头像地址")
            @RequestParam String url,
            @ApiParam(name = "openId", value = "微信id", required = true)
            @RequestParam String openId,
            @ApiParam(name = "userName", value = "用户名称")
            @RequestParam String userName,
            @ApiParam(name = "platformId", value = "平台id")
            @RequestParam String platformId){
        return userAccountService.updateAvatars(openId,url,userName,platformId);
    }

    /**
     * 用户绑定
     */
    @ApiOperation(value = "微信用户绑定登录")
    @PostMapping("/userBinding")
    public R userBinding(@RequestBody WxUserBindingReq userBindingReq) throws IOException {
        String token = userInfoService.userBinding(userBindingReq);
        return R.ok().put("token",token);
    }

    //发送消息通知
    @ApiOperation(value = "消息通知")
    @PostMapping("/pushMessage")
    public R pushMessage(@RequestBody WxMssVo wxMssVo){
        return userInfoService.messageNotification(wxMssVo);
    }

    /**
     * 后台保存用户同意消息推送的接口
     */
    /*@RequestMapping(value="/SaveMessageAuthority.do" ,produces = "application/json;charset=UTF-8")
    @ResponseBody
    public String SaveMessageAuthority(String openId,String templateId,String userName ,String idCard) throws Exception {
        try {
            WeChatMessageAuthorityPO weChatMessageAuthority = new WeChatMessageAuthorityPO();
            weChatMessageAuthority.setOpenId(openId);
            weChatMessageAuthority.setTemplateId(templateId);
            weChatMessageAuthority.setIdCard(idCard);
//            String temp = request.getParameter("studentName");
            if(StringUtils.isNotEmpty(userName)){
                userName = java.net.URLDecoder.decode(userName,"UTF-8");
                weChatMessageAuthority.setUserName(userName);
            }
            weChatMessageAuthority.setCreateDate(new Date());
            weChatMessageAuthority.setIsSend(BaseStaticParameter.NO);
            iWxMassageAuthorityService.saveOrUpdate(weChatMessageAuthority);

        }catch (Exception e){
            e.printStackTrace();
            return "false";
        }

        return "true";
    }*/

    /**
     * 后台发送推送信息
     */
    /*public static boolean sendNearRemindMessage(String openId,String serveName,String windowName,String lineNumber,String waitNumber,String msg) throws Exception {

        WxMaSubscribeMessage subscribeMessage = new WxMaSubscribeMessage();

        //跳转小程序页面路径
        subscribeMessage.setPage("pages/index/index");
        //模板消息id
        subscribeMessage.setTemplateId(WxMaConfiguration.templateId);
        //给谁推送 用户的openid （可以调用根据code换openid接口)
        subscribeMessage.setToUser(openId);
        //==========================================创建一个参数集合========================================================
        ArrayList<WxMaSubscribeData> wxMaSubscribeData = new ArrayList<>();

//        订阅消息参数值内容限制说明
//              ---摘自微信小程序官方：https://developers.weixin.qq.com/miniprogram/dev/api-backend/open-api/subscribe-message/subscribeMessage.send.html
//        参数类别 	参数说明 	参数值限制 	   说明
//        thing.DATA 	事物 	20个以内字符 	可汉字、数字、字母或符号组合
//        number.DATA 	数字 	32位以内数字 	只能数字，可带小数
//        letter.DATA 	字母 	32位以内字母 	只能字母
//        symbol.DATA 	符号 	5位以内符号 	只能符号
//        character_string.DATA 	字符串 	32位以内数字、字母或符号 	可数字、字母或符号组合
//        time.DATA 	时间 	24小时制时间格式（支持+年月日） 	例如：15:01，或：2019年10月1日 15:01
//        date.DATA 	日期 	年月日格式（支持+24小时制时间） 	例如：2019年10月1日，或：2019年10月1日 15:01
//        amount.DATA 	金额 	1个币种符号+10位以内纯数字，可带小数，结尾可带“元” 	可带小数
//        phone_number.DATA 	电话 	17位以内，数字、符号 	电话号码，例：+86-0766-66888866
//        car_number.DATA 	车牌 	8位以内，第一位与最后一位可为汉字，其余为字母或数字 	车牌号码：粤A8Z888挂
//        name.DATA 	姓名 	10个以内纯汉字或20个以内纯字母或符号 	中文名10个汉字内；纯英文名20个字母内；中文和字母混合按中文名算，10个字内
//        phrase.DATA 	汉字 	5个以内汉字 	5个以内纯汉字，例如：配送中

        WxMaSubscribeData wxMaSubscribeData1 = new WxMaSubscribeData();
        wxMaSubscribeData1.setName("thing1");
        wxMaSubscribeData1.setValue(serveName);
        wxMaSubscribeData.add(wxMaSubscribeData1);

        WxMaSubscribeData wxMaSubscribeData2 = new WxMaSubscribeData();
        wxMaSubscribeData2.setName("thing6");
        wxMaSubscribeData2.setValue(windowName);
        wxMaSubscribeData.add(wxMaSubscribeData2);

        WxMaSubscribeData wxMaSubscribeData3 = new WxMaSubscribeData();
        wxMaSubscribeData3.setName("character_string2");
        wxMaSubscribeData3.setValue(lineNumber);
        wxMaSubscribeData.add(wxMaSubscribeData3);

        WxMaSubscribeData wxMaSubscribeData4 = new WxMaSubscribeData();
        wxMaSubscribeData4.setName("number3");
        wxMaSubscribeData4.setValue(waitNumber);
        wxMaSubscribeData.add(wxMaSubscribeData4);

        WxMaSubscribeData wxMaSubscribeData5 = new WxMaSubscribeData();
        wxMaSubscribeData5.setName("thing5");
        wxMaSubscribeData5.setValue(msg);//"快到您了，请关注窗口和叫号情况"
        wxMaSubscribeData.add(wxMaSubscribeData5);

        //把集合给大的data
        subscribeMessage.setData(wxMaSubscribeData);
        //=========================================封装参数集合完毕========================================================

        try {
            System.out.println(JsonUtil.toJSONString(subscribeMessage));
            //获取微信小程序配置：

            final WxMaService wxService = WxMaConfiguration.getMaService(WxMaConfiguration.appid);
            //进行推送
            wxService.getMsgService().sendSubscribeMsg(subscribeMessage);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println(e.toString());
        }
        return false;
    }*/

    @ApiOperation(value = "公众号配置")
    @GetMapping("/publicConfiguration")
    protected void publicConfiguration(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        // TODO Auto-generated method stub

        String signature=request.getParameter("signature");
        String timestamp=request.getParameter("timestamp");
        String nonce=request.getParameter("nonce");
        String echostr=request.getParameter("echostr");
        String token="somelog";//这里填基本配置中的token
        String jiami="";
        try {
            jiami= SHA1.getSHA1(token, timestamp, nonce,"");//这里是对三个参数进行加密
        } catch (AesException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        System.out.println("加密"+jiami);
        System.out.println("本身"+signature);
        PrintWriter out=response.getWriter();
        if(jiami.equals(signature)) out.print(echostr);
    }


}
