package com.huoran.users.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huoran.common.response.R;
import com.huoran.common.utils.TokenUtils;
import com.huoran.users.entity.Permission;
import com.huoran.users.entity.RolePermission;
import com.huoran.users.entity.UserRole;
import com.huoran.users.entity.vo.DataPermissionVo;
import com.huoran.users.service.PermissionService;
import com.huoran.users.service.RolePermissionService;
import com.huoran.users.service.UserRoleService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * <p>
 * 用户角色表 前端控制器
 * </p>
 *
 * @author chen
 * @since 2021-09-29
 */
@Api(tags = "账号角色管理")
@RestController
@RequestMapping("/users/user-role")
public class UserRoleController {

    @Autowired
    private UserRoleService userRoleService;

    @Autowired
    private PermissionService permissionService;

    @Autowired
    private RolePermissionService rolePermissionService;

    @ApiOperation(value = "查询当前平台登录账号角色权限菜单")
    @GetMapping("/getUserRolesPermissionMenu")
    public R getUserRolesPermissionMenu(
            @ApiParam(name = "platformId", value = "平台id", required = true)
            @RequestParam Integer platformId,
            @ApiParam(name = "teamId", value = "团队id", required = false)
            @RequestParam(required = false) Integer teamId,HttpServletRequest request) {
        String accountId = TokenUtils.getIdByJwtToken(request);
        List<Integer> roles = userRoleService.queryAccountRoles(accountId,teamId);
        String roleIds = "";
        if (roles!=null && !roles.isEmpty()){
            for (Integer id : roles) {
                roleIds += id +",";
            }
            roleIds = roleIds.substring(0,roleIds.length()-1);
        }else {
            return R.error("该账号无权限！");
        }
        List<Permission> list = permissionService.selectRoleMenu(roleIds,platformId);

        List<DataPermissionVo> permissionVoList = null;
        //中台查询数据权限
        if (platformId==3){
            permissionVoList = permissionService.queryDataPermission(roleIds);
        }
        return R.ok().put("permissionMenu", list).put("dataPermissionList",permissionVoList);
    }

    @ApiOperation(value = "账号赋予角色")
    @PostMapping("/rolePermissionService")
    public R accountAssignmentRoles(
            @ApiParam(name = "accountId", value = "账号id", required = true)
            @RequestParam Integer accountId,
            @ApiParam(name = "roleIds", value = "角色ids", required = true)
            @RequestParam List<Integer> roleIds,
            @ApiParam(name = "teamId", value = "团队id", required = false)
            @RequestParam(value = "teamId",required = false) Integer teamId) {
        int save = 0;
        if (!roleIds.isEmpty()){
            for (Integer roleId:roleIds){
                UserRole userRole = new UserRole();
                userRole.setAccountId(accountId);
                userRole.setRoleId(roleId);
                if (teamId!=null){
                    userRole.setTeamId(teamId);
                }
                userRoleService.save(userRole);
                ++save;
            }
        }
        return save > 0 ? R.ok() : R.error();
    }

    @ApiOperation(value = "角色赋予所有权限")
    @GetMapping("/grantAllPermissions")
    public void grantAllPermissions(@RequestParam Integer roleId) {
        QueryWrapper<Permission> wrapper = new QueryWrapper<>();
        wrapper.select("id");
        List<Permission> list = permissionService.list(wrapper);
        list.forEach(permission -> {
            RolePermission rolePermission = new RolePermission();
            rolePermission.setRoleId(roleId);
            rolePermission.setPermissionId(permission.getId());
            rolePermissionService.save(rolePermission);
        });
    }

}

