package com.huoran.users.entity;

import com.baomidou.mybatisplus.annotation.*;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.io.Serializable;
import java.util.Date;

/**
 * <p>
 * 用户信息表
 * </p>
 *
 * @author huoran
 * @since 2021-06-28
 */
@Data
@TableName("hr_user_info")
public class HrUserInfo implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 用户id
     */
    @ApiModelProperty("用户id")
    @TableId(value = "user_id", type = IdType.AUTO)
    private Integer userId;

    /**
     * 用户姓名
     */
    @ApiModelProperty("用户姓名")
    private String userName;

    @ApiModelProperty("实名认证")
    @TableField(exist = false)
    private String authentication;

    @ApiModelProperty("手机号")
    @TableField(exist = false)
    private String phone;

    /**
     * 唯一标识账号
     */
    @ApiModelProperty("唯一标识账号")
    private String uniqueIdentification;

    /**
     * 证件类型（1、身份证）
     */
    @ApiModelProperty("证件类型（1、身份证）")
    private Integer documentType;

    /**
     * 证件号码
     */
    @ApiModelProperty("证件号码")
    private String idNumber;

    /**
     * 绑定省份id
     */
    @ApiModelProperty("绑定省份id")
    private Integer provinceId;

    @ApiModelProperty("省份名称")
    @TableField(exist = false)
    private String provinceName;

    /**
     * 绑定城市id
     */
    @ApiModelProperty("绑定城市id")
    private Integer cityId;

    @ApiModelProperty("城市名称")
    @TableField(exist = false)
    private String cityName;

    /**
     * 国家归属地
     */
    @ApiModelProperty("国家归属地")
    private String countries;

    /**
     * 出生日期
     */
    @ApiModelProperty("出生日期")
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    private Date dateBirth;

    /**
     * 教育程度
     */
    @ApiModelProperty("教育程度")
    private Integer educationDegree;

    /**
     * 绑定学校id
     */
    @ApiModelProperty("绑定学校id")
    private Integer schoolId;

    @ApiModelProperty("学校名称")
    @TableField(exist = false)
    private String schoolName;


    @ApiModelProperty("邮箱")
    private String email;

    @ApiModelProperty("微信号")
    private String wechatId;

    /**
     * 登陆次数
     */
    @ApiModelProperty("登陆次数")
    private Integer loginNumber;

    /**
     * 性别（1、男 2、女）
     */
    @ApiModelProperty("性别（1、男 2、女）")
    private Integer sex;

    /**
     * 用户头像路径
     */
    @ApiModelProperty("用户头像路径")
    private String userAvatars;

    /**
     * 是否删除(0->未删除；1->已删除)
     */
    @ApiModelProperty("是否删除(0->未删除；1->已删除)")
    @TableLogic
    private Integer isDel;

    /**
     * 最后登陆时间
     */
    @ApiModelProperty("最后登陆时间")
    private Date lastLoginTime;

    /**
     * 绑定多个角色id
     */
    @ApiModelProperty("绑定多个角色id")
    private String roleId;

    /**
     * 是否禁用（0->禁用，1->启用）
     */
    @ApiModelProperty("是否禁用（0->禁用，1->启用）")
    private Integer isEnable;

    /**
     * 组织关系：1->有组织关系；2->无组织关系；
     */
    @ApiModelProperty("组织关系：1->有组织关系；2->无组织关系；")
    private Integer organizationalRelation;

    /**
     * 更新时间
     */
    @ApiModelProperty("更新时间")
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private Date updateTime;

    /**
     * 创建时间
     */
    @ApiModelProperty("创建时间")
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    @TableField(fill = FieldFill.INSERT)
    private Date createTime;
}
