package com.huoran.users.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huoran.common.constant.DelConstant;
import com.huoran.common.constant.PlatformConstant;
import com.huoran.common.constant.SMSTemplate;
import com.huoran.common.entity.*;
import com.huoran.common.entity.Student;
import com.huoran.common.entity.resp.PhoneBindingrResp;
import com.huoran.common.exception.CustomException;
import com.huoran.common.exception.ExceptionEnum;
import com.huoran.common.response.R;
import com.huoran.common.utils.AliyunOssUtil;
import com.huoran.common.utils.DateUtil;
import com.huoran.common.utils.MD5;
import com.huoran.users.entity.*;
import com.huoran.users.entity.EnterpriseCertification;
import com.huoran.users.entity.UserAuthenticationInformation;
import com.huoran.users.entity.req.*;
import com.huoran.users.entity.res.*;
import com.huoran.users.entity.vo.CheckVo;
import com.huoran.users.mapper.*;
import com.huoran.users.service.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 各个平台账号表 服务实现类
 * </p>
 *
 * @author huoran
 * @since 2021-06-28
 */
@Service
public class HrUserAccountServiceImpl extends ServiceImpl<HrUserAccountMapper, HrUserAccount> implements IHrUserAccountService {

    @Autowired
    private HrUserAccountMapper hrUserAccountMapper;
    @Autowired
    private HrUserInfoMapper hrUserInfoMapper;

    @Autowired
    private UserAuthenticationInformationMapper userAuthenticationInformationMapper;
    @Autowired
    public EnterpriseCertificationService enterpriseCertificationService;
    @Autowired
    private HrPersonalFileMapper hrPersonalFileMapper;
    @Autowired
    private StringRedisTemplate stringRedisTemplate;

    @Autowired
    private BeanFactory beanFactory;

    @Autowired
    private IHrUserAccountService userAccountService;

    @Autowired
    private PlatformLoginService platformLoginService;

    //针对数据平台用户禁用启用判断
    public boolean disableEnableForDataPlatformUsersChecker(Integer accountId) {
        HrUserAccount account = hrUserAccountMapper.selectById(accountId);
        if (account.getPlatformId().contains(PlatformConstant.DATA_PLATFORM + "")) {//判断当前账号所属平台是否包含数据平台
            //如若为数据平台用户，启用前需要判断当前院校可用账户数量
            QueryWrapper<HrUserAccount> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("school_id", account.getSchoolId());
            queryWrapper.eq("is_enable", DelConstant.IS_ENABLE);
            List<HrUserAccount> getNum = baseMapper.selectList(queryWrapper);//查询现在该院校已启用的账号数量

            int availableNum = baseMapper.getAvailableNum(account.getSchoolId()) - 1;//统计该院校下可启用的账号数量  减1是因为客户也包含了算启用的用户

            //判断该院校已启用账户总数量是否小于该院校可启用的账户数量
            if (availableNum < getNum.size()) {
                //throw new CustomException(ExceptionEnum.DATA_PLATFORM_ACCOUNT_AVALIABLE_FAIL);
                return false;
            }

        }
        return true;
    }

    @Override
    public R queryPage(UserInfoReq userInfoReq) {
        Page<UserInfoListRes> page = new Page<>(userInfoReq.getPageNum(), userInfoReq.getPageSize());
        IPage<UserInfoListRes> pageList = hrUserAccountMapper.pageByCondition(page, userInfoReq);
        return R.ok().put("page", pageList);
    }

    @Override
    public int resetPwd(Integer userId) {
        HrUserAccount hrUserAccount = new HrUserAccount();
        hrUserAccount.setPassword(MD5.encrypt("1122aa"));
        return baseMapper.update(hrUserAccount, new QueryWrapper<HrUserAccount>().eq("user_id", userId));
    }

    @Override
    @Transactional
    public int delUserAccount(Integer userId, Integer isdel) {
        //设置用户表用户为已删除
        HrUserInfo hrUserInfo = new HrUserInfo();
        hrUserInfo.setIsDel(isdel);
        hrUserInfoMapper.update(hrUserInfo, new QueryWrapper<HrUserInfo>().eq("user_id", userId));
        //设置账号表与用户相关的所有账号都设置为已删除
        HrUserAccount hrUserAccount = new HrUserAccount();
        hrUserAccount.setIsDel(isdel);
        int userId1 = hrUserAccountMapper.update(hrUserAccount, new QueryWrapper<HrUserAccount>().eq("user_id", userId));
        return userId1;
    }

    @Override
    public R delUserAccounts(List<String> ids) {
        List<HrUserAccount> accountList = hrUserAccountMapper.selectList(new QueryWrapper<HrUserAccount>().in("app_open_id", ids));
        List<Integer> userIds = accountList.stream().map(HrUserAccount::getUserId).collect(Collectors.toList());
        //删除用户信息
        hrUserInfoMapper.deleteBatchIds(userIds);
        //删除账号表
        int delete = hrUserAccountMapper.delete(new QueryWrapper<HrUserAccount>().in("app_open_id", ids));
        //删除平台全部认证信息
        ids.forEach(openId -> {
            QueryWrapper<UserAuthenticationInformation> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("open_id", openId);
            userAuthenticationInformationMapper.delete(queryWrapper);

            QueryWrapper<EnterpriseCertification> queryWrapper1 = new QueryWrapper<>();
            queryWrapper1.eq("open_id", openId);
            enterpriseCertificationService.remove(queryWrapper1);
        });
        //删除组织信息，及组织下的用户信息
        for (HrUserAccount hrUserAccount : accountList) {
            List<OrganizationInfo> organizationInfoList = baseMapper.selectOrganizationInfo(hrUserAccount.getAppOpenId());
            if (organizationInfoList != null && organizationInfoList.size() > 0) {
                for (OrganizationInfo organizationInfo : organizationInfoList) {
                    //如果是团队负责人，则删除自身团队及成员团队信息
                    if (organizationInfo.getIsTeam() == 1) {
                        ArrayList<Integer> getTeamIds =  baseMapper.selectTeamId(organizationInfo.getTeamId());
                        for (Integer teamId : getTeamIds) {
                            baseMapper.deleteTeam(teamId);
                        }
                    }else {
                        //删除团队信息
                        baseMapper.deleteOrganization(organizationInfo.getAccountId());
                    }
                }
            }
        }

        return delete > 0 ? R.ok("删除成功！") : R.error();
    }

    @Override
    public UserSysDetailRes selectUserSysBind(Integer userId, Integer platformId) {
        //根据用户id以及平台id将对应的用户及账号信息查询出来
        List<UserAccountListRes> list = hrUserAccountMapper.selectUserSysBindById(userId, platformId);

        //先根据院校分组，再根据平台端分组
        Map<String, Map<Integer, List<UserAccountListRes>>> group2 = list.stream().collect(Collectors.groupingBy(UserAccountListRes::getSchoolName, Collectors.groupingBy(UserAccountListRes::getType)));
        UserSysDetailRes res = new UserSysDetailRes();
        res.setUserId(userId);
        res.setPlatformId(platformId);
        res.setMap(group2);
        return res;
    }

    @Override
    public HrUserAccount selectAccountDetail(Integer accountId) {

        return hrUserAccountMapper.selectOne(new QueryWrapper<HrUserAccount>().eq("id", accountId));
    }

    @Override
    public UserInfoDetailsRes queryUserInfoDetails(Integer userId) {

        UserInfoDetailsRes res = new UserInfoDetailsRes();

        //查询用户信息
        HrUserInfo hrUserInfo = hrUserInfoMapper.selectByUserId(userId);
        res.setHrUserInfo(hrUserInfo);

        //查询账号信息
        HrUserAccount userAccount = baseMapper.selectOne(new QueryWrapper<HrUserAccount>().eq("user_id", userId));

        //查询已绑定系统详情
        List<SysPlatform> platformList = hrUserInfoMapper.selectUserPlatformById(userId);

        //查询账号详情List
        List<HrUserAccount> userAccountList = hrUserAccountMapper.selectList(new QueryWrapper<HrUserAccount>()
                .eq("user_id", userId).select("account"));

        //查询个人档案List
        List<HrPersonalFile> personalFileList = hrPersonalFileMapper.selectList(new QueryWrapper<HrPersonalFile>()
                .eq("user_id", userId));

        res.setPlatformList(platformList);
        res.setUserAccountList(userAccountList);
        res.setPersonalFileList(personalFileList);
        res.setUserAccount(userAccount);
        return res;
    }

    @Override
    public R updateUserAvatars(Integer userId, MultipartFile file) {
        HrUserInfo hrUserInfo = new HrUserInfo();
        hrUserInfo.setUserId(userId);

        FilesResult result = null;
        int i = 0;
        String userAvatars = "";
        try {
            result = AliyunOssUtil.uploadFile(file);
            //用户图片路径
            userAvatars = result.getFileUrl();
            hrUserInfo.setUserAvatars(userAvatars);
            i = hrUserInfoMapper.updateById(hrUserInfo);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return i > 0 ? R.ok(userAvatars) : R.error();
    }

    @Override
    public int updatePersonCenter(UserInfoDetailsRes req) {

        //修改个人信息
        int i = hrUserInfoMapper.updateById(req.getHrUserInfo());

        //验证账号是否存在
        for (HrUserAccount hrUserAccount : req.getUserAccountList()) {
            int count = hrUserAccountMapper.queryAccountExist(hrUserAccount.getAccount(), req.getUserId());
            if (count > 0) {
                throw new CustomException(ExceptionEnum.EXIST_ACCOUNT);
            }
        }

        //修改账号信息
        for (HrUserAccount account : req.getUserAccountList()) {
            hrUserAccountMapper.updateById(account);
        }

        //根据用户id删除个人档案信息
        hrPersonalFileMapper.delete(new QueryWrapper<HrPersonalFile>().eq("user_id", req.getUserId()));
        //新增个人档案信息
        if (req.getPersonalFileList() != null && !req.getPersonalFileList().isEmpty()) {
            Boolean n = hrPersonalFileMapper.insertPersonalfileList(req.getPersonalFileList());
        }

        return 1;
    }

    @Override
    public IPage<DataUserInfoResp> getDataUserList(DataUserListReq req) {
        Page<DataUserInfoResp> page = new Page<DataUserInfoResp>(req.getPageNum(), req.getPageSize());
        Map<String, Object> map = new HashMap();
        map.put("createTime", req.getCreateTime());
        map.put("endTime", req.getEndTime());
        map.put("platformId", req.getPlatformId());
        map.put("roleId", req.getRoleId());
        map.put("isEnable", req.getIsEnable());
        map.put("keyWord", req.getKeyWord());
        map.put("schoolId", req.getSchoolId());

        map.put("pageNum", req.getPageNum());
        map.put("pageSize", req.getPageSize());
        return hrUserAccountMapper.getDataUserList(page, map);
    }

    @Override
    public R getUserTotalBySchoolId(Integer schoolId) {
        Integer totalCount = hrUserAccountMapper.selectCount(new QueryWrapper<HrUserAccount>()
                .eq("school_id", schoolId).eq("is_del", DelConstant.NOT_DEL).eq("platform_id", PlatformConstant.DATA_PLATFORM));
        return R.ok().put("totalCount", totalCount);
    }

    @Override
    public int updateAccountEnable(Integer id, Integer isEnable) {

        //是否禁用（0->禁用，1->启用）
        if (isEnable == 1) {
            boolean result = disableEnableForDataPlatformUsersChecker(id);
            if (result == false) {
                throw new CustomException(ExceptionEnum.ACCOUNT_AVALIABLE);
            }
        }

        HrUserAccount hrUserAccount = new HrUserAccount();
        hrUserAccount.setIsEnable(isEnable);
        hrUserAccount.setId(id);
        return hrUserAccountMapper.updateById(hrUserAccount);
    }

    @Override
    public String queryPassword(Integer accountId, Integer platformId) {

        HrUserAccount userAccount = hrUserAccountMapper.selectOne(new QueryWrapper<HrUserAccount>().eq("id", accountId).eq("platform_id", platformId));
        return userAccount.getPassword();
    }

    @Override
    public int updateAccountAllEnable(Integer userId, Integer isEnable) {

        UserInfoDetailsRes userInfoDetailsRes = this.viewUserDetailsforNakadai(userId);
        userInfoDetailsRes.getBindingPlatformAndUserList().forEach(res -> {
            //isEnable:是否禁用(0->禁用，1->启用)
            HrUserAccount userAccount = new HrUserAccount();
            userAccount.setIsEnable(isEnable);
            userAccount.setId(res.getAccountId());
            baseMapper.updateById(userAccount);
        });
        return 1;

    }

    @Override
    public int updateAccountAllAvaliable(Integer id, Integer isEnable) {
        //修改用户启用禁用状态
        HrUserInfo hrUserInfo = new HrUserInfo();
        hrUserInfo.setIsEnable(isEnable);
        hrUserInfo.setUserId(id);
        hrUserInfoMapper.updateById(hrUserInfo);
        //修改账号启用禁用状态
        HrUserAccount hrUserAccount = new HrUserAccount();
        hrUserAccount.setIsEnable(isEnable);
        hrUserAccount.setUserId(id);
        return hrUserAccountMapper.update(hrUserAccount, new QueryWrapper<HrUserAccount>().eq("user_id", id));
    }

    @Override
    public R examinePassword(ResetPwdReq resetPwdReq) {
        //校验新密码必须与新密码一致
        if (!resetPwdReq.getNewPassword().equals(resetPwdReq.getReNewPassword())) {
            return R.error("输入的新密码不一致，请重新确认哦!");
        }
        //校验原密码是否输入正确
        HrUserAccount hrUserAccount = this.baseMapper.selectById(resetPwdReq.getAccountId());
        if (MD5.encrypt(resetPwdReq.getPassword()).equals(hrUserAccount.getPassword())) {
            HrUserAccount account = new HrUserAccount();
            account.setPassword(MD5.encrypt(resetPwdReq.getNewPassword()));
            account.setId(resetPwdReq.getAccountId());
            int i = this.baseMapper.updateById(account);
            return i > 0 ? R.ok() : R.error("更新密码失败！");
        } else {
            return R.error("原密码错误，更新密码失败！");
        }
    }

    @Override
    public Integer getUserIdByAccountId(String accountId) {
        HrUserAccount hrUserAccount = hrUserAccountMapper.selectOne(new QueryWrapper<HrUserAccount>().eq("id", accountId));
        return hrUserAccount.getUserId();
    }

    @Override
    public Integer getAvailableNum(Integer schoolId) {
        return baseMapper.getAvailableNum(schoolId);
    }

    @Override
    public String getAccountId(CheckVo checkVo) {
        HrUserAccount hrUserAccount = hrUserAccountMapper.
                selectOne(new QueryWrapper<HrUserAccount>().
                        eq("school_id", checkVo.getSchoolId()).
                        eq("work_number", checkVo.getAccount()).
                        eq("type", checkVo.getMark()).
                        last("  and FIND_IN_SET( " + checkVo.getPlatform() + ", platform_id )").
                        select("id"));
        return hrUserAccount.getId().toString();
    }

    @Override
    public Integer updateAccount(HrUserAccount userAccount) {
        return hrUserAccountMapper.update(userAccount, new UpdateWrapper<HrUserAccount>().eq("id", userAccount.getId()));
    }

    @Override
    public String queryClientName(Integer customerId) {
        return baseMapper.selectClientName(customerId);
    }

    /**
     * @Description :   中台用户管理查看用户详情
     * @Param userId
     * @Author Rong---2022/4/19
     */
    @Override
    public UserInfoDetailsRes viewUserDetailsforNakadai(Integer userId) {
        UserInfoDetailsRes res = new UserInfoDetailsRes();

        //查询用户信息
        HrUserInfo hrUserInfo = hrUserInfoMapper.selectByUserId(userId);
        res.setHrUserInfo(hrUserInfo);

        //查询账号详情List
        HrUserAccount userAccount = hrUserAccountMapper.selectOne(new QueryWrapper<HrUserAccount>()
                .eq("user_id", userId));
        List<PhoneBindingrResp> bindingrRespList = null;

        HrUserAccount parameter = new HrUserAccount();
        if (userAccount.getPhone() == null || userAccount.getPhone().equals("")) {
            parameter.setUserId(hrUserInfo.getUserId());
        } else {
            parameter.setPhone(userAccount.getPhone());
        }
        bindingrRespList = hrUserInfoMapper.userBindingPhoneInfo(parameter);

        bindingrRespList.forEach(l -> {
            if (l.getType() == 1) {
                l.setRoleName("学生");
            }
        });

        res.setBindingPlatformAndUserList(bindingrRespList);
        res.setUserAccount(userAccount);
        return res;
    }

    @Override
    public UserInfoDetailsRes viewUserDetails(String openId) {
        UserInfoDetailsRes res = new UserInfoDetailsRes();

        //查询账号详情List
        List<HrUserAccount> userAccountList = hrUserAccountMapper.selectList(new QueryWrapper<HrUserAccount>()
                .eq("app_open_id", openId).orderByDesc("platform_id"));
        if (userAccountList.size() == 0) {
            res.setStatus(true);
            res.setMessage("用户不存在，请先重新登录注册！");
            return res;
        }

        //查询个人端用户信息
        Integer userId = null;
        String phone = null;
        for (HrUserAccount hrUserAccount : userAccountList) {
            if (hrUserAccount.getPlatformId().equals("7")) {
                userId = hrUserAccount.getUserId();
                phone = hrUserAccount.getPhone();
            }
        }
        HrUserInfo hrUserInfo = hrUserInfoMapper.selectByUserId(userId);
        hrUserInfo.setPhone(phone);
        hrUserInfo.setAuthentication("未认证");

        UserAuthenticationInformation authenticationInformation = userAuthenticationInformationMapper.selectOne(
                new QueryWrapper<UserAuthenticationInformation>().eq("open_id", openId));
        if (authenticationInformation != null) {
            hrUserInfo.setUserName(authenticationInformation.getRealName());
            hrUserInfo.setAuthentication("已认证");
            hrUserInfo.setIdNumber(authenticationInformation.getIdCardNo());
        }

        List<OrganizationInfo> organizationInfoList = baseMapper.selectOrganizationInfo(openId);

        res.setHrUserInfo(hrUserInfo);
        res.setOrganizationInfoList(organizationInfoList);
        res.setUserAccountList(userAccountList);
        return res;
    }

    @Override
    public R updateAvatars(String openId, String url, String userName, String platformId) {
        //查询账号详情
        HrUserAccount hrUserAccount = hrUserAccountMapper.selectOne(new QueryWrapper<HrUserAccount>()
                .eq("app_open_id", openId).eq("platform_id", platformId));

        //查询用户信息
        HrUserInfo hrUserInfo = hrUserInfoMapper.selectByUserId(hrUserAccount.getUserId());
        if (StringUtils.isNotBlank(url)) {
            hrUserInfo.setUserAvatars(url);
        }
        if (StringUtils.isNotBlank(userName)) {
            hrUserInfo.setUserName(userName);
            hrUserAccount.setAccount(userName);
            hrUserAccountMapper.updateById(hrUserAccount);
        }
        int update = hrUserInfoMapper.updateById(hrUserInfo);
        return update > 0 ? R.ok() : R.error();
    }

    @Override
    public int selectTeam(String id) {
        return baseMapper.selectTeamByOpenId(id);
    }

    @Override
    public R oneClickEnableVerification(Integer accountId) {

        HrUserAccount account = baseMapper.selectById(accountId);

        QueryWrapper<HrUserAccount> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("school_id", account.getSchoolId());
        queryWrapper.eq("is_enable", DelConstant.IS_ENABLE);
        List<HrUserAccount> getNum = baseMapper.selectList(queryWrapper);//查询现在该院校已启用的账号数量

        int availableNum = baseMapper.getAvailableNum(account.getSchoolId()) - 1;//统计该院校下可启用的账号数量  减1是因为客户也包含了算启用的用户

        //判断当前被启用的账户是否绑定手机号 如已绑定，查询全部绑定为一个手机号下的信息 单独判断数据平台下的账号
        UserInfoDetailsRes userInfoDetailsRes = this.viewUserDetailsforNakadai(account.getUserId());
        if (userInfoDetailsRes.getBindingPlatformAndUserList().size() > 0) {
            userInfoDetailsRes.getBindingPlatformAndUserList().forEach(res -> {
                if (res.getPlatformName().contains("数据前瞻")) {
                    //判断该院校已启用账户总数量是否小于该院校可启用的账户数量
                    if (availableNum < getNum.size()) {
                        throw new CustomException(ExceptionEnum.DATA_PLATFORM_ACCOUNT_AVALIABLE_FAIL);
                    }
                }
            });
        }

        return R.ok();
    }

    /**
     * @Description : 中台查看-个人档案
     * @Param userId
     * @Author Rong---2022/4/20
     */
    @Override
    public ProfileDetailsRes personalFile(Integer userId) {

        //查询用户信息
        ProfileDetailsRes personalFile = hrUserInfoMapper.personalFile(userId);
        //平台端区分：0->教师端 1->学生端 2->无端  增加判断平台端为1学生端的时候设置角色 因为学生是没有角色的!
        if (personalFile.getType() == 1) {
            personalFile.setRoleName("学生");
        }

        //查询个人档案
        List<PersonalFileResp> personalFileList = hrPersonalFileMapper.personalFileListByUserId(userId);

        personalFile.setNakadaiPersonalFileList(personalFileList);

        return personalFile;
    }

    @Override
    public R unbindMobilePhone(Integer accountId) {
        boolean result = baseMapper.unbindMobilePhone(accountId);
        return result ? R.ok() : R.error();
    }

    @Override
    public R changePhoneNumber(CheckVo checkVo, Integer accountId) {

        String key = StringUtils.join(checkVo.getPhone(), SMSTemplate.AUTHENTICATION_CODE);
        String opener = stringRedisTemplate.opsForValue().get(key);//获取验证码

        if (opener == null) {
            return R.error("验证码已过期或不存在,请重新发送验证码");
        }


        if (opener.equals(checkVo.getOpener())) {
            HrUserAccount account = new HrUserAccount();
            account.setPhone(checkVo.getPhone());
            account.setId(accountId);
            int updateCode = baseMapper.updateById(account);

            return updateCode > 0 ? R.ok() : R.error();
        }
        return R.error("更改失败！");
    }

    @Override
    public Integer querySchoolIdByAccountId(String accountId) {
        if (StringUtils.isEmpty(accountId)) {
            return null;
        }
        HrUserAccount hrUserAccount = baseMapper.selectById(accountId);
        return hrUserAccount.getSchoolId();
    }

    @Override
    public R studentAccountApplication(RegisterReq registerReq) {

        if (registerReq.getIdentification() == null || registerReq.getIdentification() != 1) {
//        手机验证码验证
            String key = StringUtils.join(registerReq.getPhone(), SMSTemplate.AUTHENTICATION_CODE);
            String value = stringRedisTemplate.opsForValue().get(key);
            if (StringUtils.isEmpty(value)) {
                throw new CustomException(ExceptionEnum.CODE_EXPIRED);
            }
            //验证验证码
            if (!registerReq.getCode().equals(value)) {
                return R.error("手机验证码错误！");
            }
        }

        //学号判重依据学校id、平台端(教师、学生端、无端(数据平台的用户)) 以及 平台来源(职站、数据平台)
        //账号组成条件:平台(职站/数据平台/中台)-端(教师、学生端、无端(数据平台的用户))-学校(学校id)-学号/工号(用户输入的工号学号)
        UserInfo userInfo = new UserInfo();
        BeanUtils.copyProperties(registerReq, userInfo);
        userInfo.setSchoolId(registerReq.getSchoolId());
        userInfo.setRoleId("0");
        boolean addUserRet = baseMapper.addUserInfo(userInfo);

        UserAccount account = new UserAccount();
        BeanUtils.copyProperties(registerReq, account);
        account.setSchoolId(registerReq.getSchoolId());
        account.setPassword(MD5.encrypt(registerReq.getPassword()));
        account.setPlatformId("1");//所属平台id 1.职站
        account.setType(1);//平台端区分：0->教师端 1->学生端 2->无端
        account.setUserId(userInfo.getUserId());
        account.setRoleId("0");

        account.setPhone(null);
        if (!registerReq.getPhone().equals("")) {
            account.setPhone(registerReq.getPhone());
            account.setPhoneBindingTime(DateUtil.currentDate());
        }
        boolean addAccountRet = baseMapper.addUserAccount(account);

        Student student = new Student();
        BeanUtils.copyProperties(registerReq, student);
        student.setAccountId(account.getId());
        student.setClassId("1");

        boolean save = baseMapper.save(student);

        //新增学生完之后进行登录操作
        /*LoginVo loginVo = new LoginVo();
         loginVo.setSchoolId(registerReq.getSchoolId());
         loginVo.setWorkNumber(account.getWorkNumber());
         loginVo.setType(1);
         loginVo.setPlatform(1);
        HashMap<String, Object> hashMap = hrUserInfoService.workNumberLogin(loginVo);

        //统计平台登录次数
        Integer schoolId = userAccountService.querySchoolIdByAccountId(account.getId().toString());
        PlatformLogin platformLogin = new PlatformLogin();
        platformLogin.setPlatform(loginVo.getPlatform());
        platformLogin.setSchoolId(schoolId);
        platformLoginService.save(platformLogin);

        LoginDTO data = new LoginDTO();
        data.setToken((String) hashMap.get("token"));
        data.setCustomer(false);
        return R.ok("认证成功,欢迎 "+registerReq.getUserName()+" 回来！").put("data", data);*/
        return save ? R.ok("账号申请已提交，需管理员审核，请10分钟后再登录").put("account", account.getId()) : R.error();
    }

}
