package com.huoran.users.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.log.Log;
import cn.hutool.log.LogFactory;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Maps;
import com.huoran.api.NakadaiClient;
import com.huoran.api.UserClient;
import com.huoran.common.constant.*;
import com.huoran.common.entity.UserAccount;
import com.huoran.common.entity.UserInfo;
import com.huoran.common.exception.CustomException;
import com.huoran.common.exception.ExceptionEnum;
import com.huoran.common.response.R;
import com.huoran.common.utils.*;
import com.huoran.users.config.WeChatPropertiesUtil;
import com.huoran.users.entity.*;
import com.huoran.users.entity.dto.LoginDTO;
import com.huoran.users.entity.req.*;
import com.huoran.users.entity.res.DataUserInfoResp;
import com.huoran.users.entity.vo.CheckVo;
import com.huoran.users.entity.vo.LoginVo;
import com.huoran.users.entity.vo.UseImportFailureVo;
import com.huoran.users.entity.vo.WxMssVo;
import com.huoran.users.mapper.HrPersonalFileMapper;
import com.huoran.users.mapper.HrUserAccountMapper;
import com.huoran.users.mapper.HrUserInfoMapper;
import com.huoran.users.mapper.RoleMapper;
import com.huoran.users.service.IHrUserInfoService;
import com.huoran.users.utils.EmailUtil;
import com.huoran.users.utils.ExcelImportHelper;
import com.huoran.common.utils.HttpClientUtils;
import com.huoran.users.utils.WeChatUtil;
import com.huoran.users.utils.poi.CommonCode;
import com.huoran.users.utils.poi.Constant;
import com.huoran.users.utils.poi.ExcelExportUtil;
import com.huoran.users.utils.poi.ExceptionCast;
import com.wf.captcha.ArithmeticCaptcha;
import com.wf.captcha.base.Captcha;
import lombok.SneakyThrows;
import org.apache.commons.collections4.map.HashedMap;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * <p>
 * 用户信息表 服务实现类
 * </p>
 *
 * @author huoran
 * @since 2021-06-28
 */
@Service
public class HrUserInfoServiceImpl extends ServiceImpl<HrUserInfoMapper, HrUserInfo> implements IHrUserInfoService {
    @Autowired
    private HrUserAccountMapper userAccountMapper;
    static Log log = LogFactory.get(HrUserInfoServiceImpl.class);
    @Resource
    StringRedisTemplate stringRedisTemplate;

    @Autowired
    private HrUserInfoMapper hrUserInfoMapper;

    @Autowired
    private NakadaiClient nakadaiClient;

    @Autowired
    private HrPersonalFileMapper hrPersonalFileMapper;

    @Autowired
    private HrUserAccountMapper hrUserAccountMapper;

    @Autowired
    private RoleMapper roleMapper;

    @Autowired
    private UserClient userClient;

    @Autowired
    private EmailUtil emailUtil;

    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    private static final String REQUEST_URL = "https://api.weixin.qq.com/sns/jscode2session";
    private static final String GRANT_TYPE = "authorization_code";

    public static boolean checkEmail(String email) {
        boolean flag = false;
        try {
            String check = "^([a-z0-9A-Z]+[-|_|\\.]?)+[a-z0-9A-Z]@([a-z0-9A-Z]+(-[a-z0-9A-Z]+)?\\.)+[a-zA-Z]{2,}$";
            Pattern regex = Pattern.compile(check);
            Matcher matcher = regex.matcher(email);
            flag = matcher.matches();
        } catch (Exception e) {
            flag = false;
        }
        return flag;
    }

    @Override
    public DataUserInfoResp getDetail(Integer userId) {
        return baseMapper.getDetail(userId);
    }

    @Override
    public Map<String, String> upload(MultipartFile file, Integer schoolId) throws IOException {
        List<ExcelImpUserInfoReq> users = ExcelImportHelper.readUser(file);
        if (users.size() <= 0) {
            ExceptionCast.cast(CommonCode.EXCEL_FILE_FORMAT_ERROR);
        }
        int count = 0;
        List<UseImportFailureVo> failVo1 = new ArrayList<>();
        // 参数合法性校验，只能上传.xlsx后缀的文件
        if (StringUtils.isBlank(file.getOriginalFilename())
                || !file.getOriginalFilename().substring(file.getOriginalFilename().lastIndexOf(".")).equals(".xlsx")) {
            ExceptionCast.cast(CommonCode.EXCEL_FILE_INVALID);
        }
        int size = users.size();
        Map<String, String> map = new HashMap<>();
        Long ii = 1L;//用于记录序列号
        try {
            for (int i = 0; i < users.size(); i++) {
                ExcelImpUserInfoReq req = users.get(i);
                // String account = req.getAccount().replaceAll("\\u00A0", "").trim();//账号
                String userName = req.getUserName().replaceAll("\\u00A0", "").trim();//用户名
                String roleName = req.getRoleId();//角色名称
                String email = req.getEmail();//邮箱
                String workNubmer = req.getWorkNumber().replaceAll("\\u00A0", "").trim();//学号/工号
                String phone = req.getPhone().replaceAll("\\u00A0", "").trim();
                SaveUserInfoReq userInfoReq = new SaveUserInfoReq();

                //职工账号，用户姓名，账号角色，工号为空 某列为空就提示错误
                // if (account.equals("")) {
                //     UseImportFailureVo vo = new UseImportFailureVo();
                //     vo.setIndex(ii).setUserName(userName)
                //             .setRoleId(roleName)
                //             .setAccount(account + " *必填项:(账号不能为空)")
                //             .setEmail(email)
                //             .setWorkNumber(workNubmer)
                //             .setFailureMsg("账号不能为空！");
                //
                //     users.remove(i);
                //     i--;
                //     failVo1.add(vo);
                //     continue;
                // }

                //校验姓名
                if (userName.equals("")) {
                    UseImportFailureVo vo = new UseImportFailureVo();
                    ++ii;
                    vo.setIndex(ii)
                            .setUserName(req.getUserName() + " 必填项:((姓名不能为空)")
                            // .setAccount(req.getAccount())
                            .setWorkNumber(req.getWorkNumber())
                            .setEmail(req.getEmail()).
                            setRoleId(req.getRoleId())
                            .setFailureMsg("姓名不能为空！");
                    users.remove(i);
                    i--;
                    failVo1.add(vo);
                    continue;
                }

                //校验角色
                if (roleName.equals("")) {
                    UseImportFailureVo vo = new UseImportFailureVo();
                    vo.setIndex(ii).setUserName(userName)
                            .setRoleId(roleName + " *必填项:(角色不能为空)")
                            // .setAccount(account)
                            .setEmail(email)
                            .setWorkNumber(workNubmer)
                            .setFailureMsg("角色不能为空！");

                    users.remove(i);
                    i--;
                    failVo1.add(vo);
                    continue;
                } else {
                    userInfoReq.setRoleId(roleName);
                }

                //校验工号
                if (workNubmer.equals("")) {
                    UseImportFailureVo vo = new UseImportFailureVo();
                    vo.setIndex(ii).setUserName(userName)
                            .setRoleId(roleName)
                            // .setAccount(account)
                            .setEmail(email)
                            .setWorkNumber(workNubmer + " *必填项:(学号/工号不能为空)")
                            .setFailureMsg("学号/工号不能为空！");

                    users.remove(i);
                    i--;
                    failVo1.add(vo);
                    continue;
                } else {
                    userInfoReq.setWorkNumber(workNubmer);
                }

                QueryWrapper<Role> wrapper = new QueryWrapper<>();
                wrapper.eq("school_id", schoolId);
                wrapper.eq("role_name", roleName);
                Role role = roleMapper.selectOne(wrapper);

                if (role == null) {
                    UseImportFailureVo vo = new UseImportFailureVo();
                    ++ii;
                    vo.setIndex(ii).setUserName(userName)
                            .setRoleId(roleName + "(不存在该角色！)")
                            // .setAccount(account)
                            .setEmail(email)
                            .setWorkNumber(workNubmer)
                            .setFailureMsg("不存在该角色！");

                    users.remove(i);
                    i--;
                    failVo1.add(vo);
                    continue;
                }

                //校验工号
                Map<String, Object> checkWorkNum = new HashMap<>();
                checkWorkNum.put("platformId", PlatformConstant.DATA_PLATFORM);
                checkWorkNum.put("workNumber", workNubmer);
                checkWorkNum.put("type", 2);
                checkWorkNum.put("schoolId", schoolId);
                HrUserAccount userAccount = baseMapper.getAccount(checkWorkNum);
                if (userAccount != null) {
                    UseImportFailureVo vo = new UseImportFailureVo();
                    ++ii;
                    vo.setIndex(ii).setUserName(userName)
                            .setRoleId(roleName)
                            // .setAccount(account)
                            .setEmail(email)
                            .setWorkNumber(workNubmer + "(重复的工号)！)")
                            .setFailureMsg("重复的工号！");

                    users.remove(i);
                    i--;
                    failVo1.add(vo);
                    continue;
                }

                if (email != null && email != "") {

                    if (checkEmail(email)) {
                        userInfoReq.setEmail(email);
                    } else {
                        UseImportFailureVo vo = new UseImportFailureVo();
                        ++ii;
                        vo.setIndex(ii).setUserName(userName)
                                .setRoleId(roleName)
                                // .setAccount(account)
                                .setEmail(email + "(请输入正确的邮箱格式)！)")
                                .setWorkNumber(workNubmer)
                                .setFailureMsg("请输入正确的邮箱格式！");

                        users.remove(i);
                        i--;
                        failVo1.add(vo);
                        continue;
                    }

                }

                ++ii;
                if (!users.isEmpty() && users.size() > 0) {

                    //查询角色
                    // DataRole getRole = baseMapper.getRole(roleName, String.valueOf(schoolId));

                    QueryWrapper<Role> wrapper1 = new QueryWrapper<>();
                    wrapper1.eq("school_id", schoolId);
                    wrapper1.eq("role_name", roleName);
                    Role getRole = roleMapper.selectOne(wrapper1);

                    req.setRoleId(getRole.getId() + "");

                    List<Integer> roleIds = new ArrayList<>();
                    roleIds.add(getRole.getId());

                    req.setUniqueIdentification(String.valueOf(System.currentTimeMillis()));
                    req.setSchoolId(schoolId);

                    //用户信息
                    HrUserInfo addUser = new HrUserInfo();
                    BeanUtils.copyProperties(req, addUser);

                    //账户信息
                    HrUserAccount addAccount = new HrUserAccount();
                    BeanUtils.copyProperties(req, addAccount);

                    //统计该院校下可启用的账号数量
                    int availableNum = userAccountMapper.getAvailableNum(req.getSchoolId());

                    //查询现在该院校已启用的账号数量
                    QueryWrapper<HrUserAccount> queryWrapper = new QueryWrapper<>();
                    queryWrapper.eq("school_id", req.getSchoolId());
                    queryWrapper.eq("is_enable", DelConstant.IS_ENABLE);
                    List<HrUserAccount> getNum = userAccountMapper.selectList(queryWrapper);

                    availableNum = availableNum - 1;//todo 减1是因为客户也包含了算启用的用户

                    //判断该院校已启用账户总数量是否小于该院校可启用的账户数量
                    if (getNum.size() <= availableNum) {
                        addUser.setIsEnable(DelConstant.IS_ENABLE);//设置为已启用
                    } else {
                        addUser.setIsEnable(DelConstant.NO_ENABLE);//设置为未启用
                    }
                    addUser.setRoleId("0");

                    int ret1 = baseMapper.insert(addUser);
                    addAccount.setPassword(MD5.encrypt("1122aa"));
                    addAccount.setUserId(addUser.getUserId());
                    addAccount.setPlatformId(PlatformConstant.DATA_PLATFORM.toString());
                    addAccount.setIsEnable(addUser.getIsEnable());
                    //22.4.13补充: 账号组成条件:平台(职站/数据平台/中台)-端(教师、学生端、无端(数据平台的用户))-学校(学校id)-学号/工号(用户输入的工号学号)
                    addAccount.setAccount(PlatformConstant.DATA_PLATFORM + "-" + PlatformConstant
                            .UNPROVOKED + "-" + schoolId + "-" + req.getWorkNumber());
                    addAccount.setRoleId("0");
                    addAccount.setPhone(null);
                    if (!phone.equals("")) {
                        addAccount.setPhone(phone);
                        addAccount.setPhoneBindingTime(DateUtils.getNowTime());
                    }
                    int ret2 = userAccountMapper.insert(addAccount);

                    //重新账号赋予角色
                    userClient.rolePermissionService(addAccount.getId(), roleIds,null);

                    if (StringUtils.isEmpty(String.valueOf(ret1)) && StringUtils.isEmpty(String.valueOf(ret2))) {
                        throw new RuntimeException();
                    }
                }
            }
        } catch (RuntimeException e) {
            map.put("failura", "数据有误，添加失败！");
            throw new RuntimeException();
        }
        String token = "";

        // 有导入失败的数据，才会存入redis
        if (failVo1.size() > 0) {
            //生成token
            token = "FAILURE_IMPORT" + UUID.randomUUID().toString().replace("-", "");
            ValueOperations<String, String> ops = stringRedisTemplate.opsForValue();
            String failureVOJson = JSON.toJSONString(failVo1);
            ops.set(token, failureVOJson, 30 * 60, TimeUnit.SECONDS);
        }
        map.put("token", token);
        map.put("successNum", (size - failVo1.size()) + "");
        map.put("failureNum", failVo1.size() + "");
        return map;
    }

    @Override
    public void exportFailureRecord(HttpServletResponse response, String token) throws Exception {
        if (StringUtils.isEmpty(token)) {
            return;
        }
        ValueOperations<String, String> ops = stringRedisTemplate.opsForValue();
        //获取数据
        String record = ops.get(token);
        if (StringUtils.isEmpty(record)) {
            return;
        }
        List<UseImportFailureVo> parse = JSON.parseObject(record, new TypeReference<List<UseImportFailureVo>>() {
        });

        parse.sort(Comparator.comparing(UseImportFailureVo::getIndex));

        ClassPathResource classPathResource = new ClassPathResource("excleTemplate/用户导入失败数据导出模板.xlsx");
        InputStream inputStream = classPathResource.getInputStream();

        //3、通过工具类下载文件
        new ExcelExportUtil(UseImportFailureVo.class, Constant.ROW_INDEX, Constant.STYLE_INDEX).
                export(response, inputStream, parse, "导入失败数据表.xlsx");

        if (inputStream != null) {
            inputStream.close();
        }
    }

    @Override
    public boolean queryPhone(String phone) {
        int count = hrUserAccountMapper.selectCount(new QueryWrapper<HrUserAccount>()
                .eq("phone", phone).eq("is_del", 0));
        return count > 0 ? true : false;
    }

    @Override
    public boolean queryEmail(String email) {
        int count = this.baseMapper.selectCount(new QueryWrapper<HrUserInfo>()
                .eq("email", email).eq("is_del", 0));
        return count > 0 ? true : false;
    }

    @Override
    public R sendCheckPhoneCode(CheckVo checkVo) {
        boolean re = true;
        boolean isSend;
        String phoneOrEmail = "";

        String msg = "";
        if (checkVo.getTypes().equals(1)) {//邮箱验证码
            phoneOrEmail = checkVo.getEmail();
/*            re = this.queryEmail(checkVo.getEmail());
            phoneOrEmail = checkVo.getEmail();
            msg = "邮箱已存在!";*/
        } else {//手机验证码
            phoneOrEmail = checkVo.getPhone();
/*            re = this.queryPhone(checkVo.getPhone());
            phoneOrEmail = checkVo.getPhone();
            msg = "手机号已存在!";*/
        }

/*        if (re) {
            return R.error(403, msg);
        }*/

        //先从redis中查询该手机号的验证码是否过期，未过期直接返回发送成功，防重放
        //为了保证该手机号5分钟内在其他场景下内正常使用，存入的key需要与短信模板进行拼接
        String key = StringUtils.join(phoneOrEmail, SMSTemplate.AUTHENTICATION_CODE);
        System.out.println(key);
        String code = stringRedisTemplate.opsForValue().get(key);
        if (!StringUtils.isEmpty(code)) {
            return R.ok("验证码已发，请勿重复操作！");
        }
        code = RandomUtil.getSixBitRandom();
        Map<String, Object> param = new HashMap<>();
        param.put("code", code);
        if (checkVo.getTypes().equals(1)) {//邮箱验证码
//            isSend = EmailUtils.sendEmail(phoneOrEmail, code);
            emailUtil.sendMessage(phoneOrEmail,
                    "职站",
                    "尊敬的用户，欢迎使用职站，您本次的验证码是: "+code+"，有效时间为5分钟");
            isSend = true;
        } else {//手机验证码
            isSend = SendSMSUtils.send(phoneOrEmail, SMSTemplate.AUTHENTICATION_CODE, param);
        }

        if (isSend) {
            stringRedisTemplate.opsForValue().set(key, code, 5, TimeUnit.MINUTES);
            CheckVo checkVo1 = new CheckVo();
            checkVo1.setOpener(key);
            return R.ok(checkVo1);
        } else {
            return R.error("验证码发送失败,请稍后重试!");
        }
    }

    @Override
    public R sendPhoneVerificationCode(String phoneOrEmail, Integer loginOrBind, String platform) {

        boolean isSend = false;
        //先从redis中查询该手机号的验证码是否过期，未过期直接返回发送成功，防重放
        //为了保证该手机号5分钟内在其他场景下内正常使用，存入的key需要与短信模板进行拼接
        String key = StringUtils.join(phoneOrEmail, SMSTemplate.AUTHENTICATION_CODE);
        String code = stringRedisTemplate.opsForValue().get(key);
        if (!StringUtils.isEmpty(code)) {
            return R.ok("验证码已发，请勿重复操作！");
        }
        code = RandomUtil.getSixBitRandom();
        Map<String, Object> param = new HashMap<>();
        param.put("code", code);

        if (EmailUtils.isEmail(phoneOrEmail)) {//邮箱验证码
            //发送前验证邮箱是否可以正常使用
            QueryWrapper<HrUserInfo> userInfoQueryWrapper = new QueryWrapper<>();
            userInfoQueryWrapper.eq("email", phoneOrEmail).
                    eq("is_enable", 1);
            List<HrUserInfo> hrUserInfos = hrUserInfoMapper.selectList(userInfoQueryWrapper);

            //邮箱不存在
            if (ObjectUtil.isNull(hrUserInfos) || hrUserInfos.size() <= 0) {
                throw new CustomException(ExceptionEnum.MAIL_NOT_EXIST);
            }
            isSend = EmailUtils.sendEmail(phoneOrEmail, code);
        } else {
            //登录验证手机是否存在，区分平台
            //TODO 判断用户是否禁用，给出对应提示
            if (loginOrBind.equals(0)) {
                //发送前验证手机号是否可以正常使用
                QueryWrapper<HrUserAccount> accountQueryWrapper = new QueryWrapper<>();
                accountQueryWrapper.eq("phone", phoneOrEmail).
                        eq("platform_id", platform).
                        eq("is_enable", 1);
                List<HrUserAccount> hrUserAccounts = hrUserAccountMapper.selectList(accountQueryWrapper);

                //手机号不存在
                if (ObjectUtil.isNull(hrUserAccounts) || hrUserAccounts.size() <= 0) {
                    throw new CustomException(ExceptionEnum.PHONE_NOT_EXIST);
                }
            }
            isSend = SendSMSUtils.send(phoneOrEmail, SMSTemplate.AUTHENTICATION_CODE, param);
        }

        if (isSend) {
            stringRedisTemplate.opsForValue().set(key, code, 2, TimeUnit.MINUTES);
            return R.ok("发送成功，请注意查收！");
        } else {
            return R.error("发送失败,请稍后重试!");
        }

    }

    @Transactional
    @Override
    public R bindPhone(CheckVo checkVo, Integer userId) {
        String opener = stringRedisTemplate.opsForValue().get(checkVo.getOpener());//获取验证码
        if (opener == null) {
            return R.error("验证码已过期或不存在,请重新发送验证码");
        } else if (opener.equals(checkVo.getCode())) {

            HrUserInfo hrUserInfo = new HrUserInfo();
            hrUserInfo.setUserId(userId);
            hrUserInfo.setEmail(checkVo.getEmail());

            int i = baseMapper.updateById(hrUserInfo);

            /*HrUserAccount userAccount = new HrUserAccount();
            userAccount.setUserId(userId);
            userAccount.setPhone(checkVo.getPhone());*/
            if (StringUtils.isNotEmpty(checkVo.getPhone())){
                UpdateWrapper<HrUserAccount> updateWrapper = new UpdateWrapper<>();
                updateWrapper.set("phone", checkVo.getPhone());
                updateWrapper.set("phone_binding_time", new Date());
                updateWrapper.eq("user_id", userId);
                hrUserAccountMapper.update(new HrUserAccount(), updateWrapper);
            }

            if (i > 0 ) {
                return R.ok();
            } else {
                return R.error("绑定失败！");
            }
        }
        return R.error("绑定失败！");
    }

    @Override
    public R queryUserInfoById(Integer userId, Integer accountId) {
        HrUserInfo userInfo = hrUserInfoMapper.selectOne(new QueryWrapper<HrUserInfo>().eq("user_id", userId));
        List<HrPersonalFile> personalFiles = hrPersonalFileMapper.selectList(new QueryWrapper<HrPersonalFile>().eq("user_id", userId));
        HrUserAccount hrUserAccount = hrUserAccountMapper
                .selectOne(new QueryWrapper<HrUserAccount>().eq("id", accountId));
//                        .last(" and FIND_IN_SET( " + PlatformConstant.DATA_PLATFORM + ", platform_id )"));

        String schoolName = hrUserAccountMapper.querySchoolNameBySchoolId(hrUserAccount.getSchoolId());
        return R.ok().put("userInfo", userInfo).put("personalFiles", personalFiles).put("userAccount", hrUserAccount).put("schoolName", schoolName);
    }

    @Override
    public Integer queryAccountExistByPlatformId(String account, Integer userId, Integer platformId) {
        return hrUserAccountMapper.queryAccountExistByPlatformId(account, userId, String.valueOf(platformId));
    }

    @Override
    public void updateUserInfoById(HrUserInfo entity) {
        hrUserInfoMapper.userInfoUpdateUserInfoById(entity);
    }

    @Override
    public LoginDTO login(LoginVo loginVo) {
        //校验参数、校验验证码、获取用户信息、校验密码、校验是否被禁用、返回token
        LoginDTO loginDTO = new LoginDTO();

        //根据手机号或邮箱登录
        if (loginVo.getDistinguish().equals(LoginDistConstant.PHONE_EMAIL_LOGIN)) {
            HashMap<String, Object> map = phoneOrEmailLogin(loginVo);
            if (ObjectUtil.isNull(map)) {
                throw new CustomException(ExceptionEnum.ACCOUNT_NOT_EXIST);
            }
            loginDTO.setUserAccounts((ArrayList<HrUserAccount>) map.get("userAccounts"));
            loginDTO.setToken((String) map.get("token"));
            return loginDTO;
        }

        //验证码校验
        Boolean check = check(loginVo.getRandom(), loginVo.getCode());
        if (check) {
            //账号、学号、工号登录
            if (loginVo.getDistinguish() == 1) {

                //登录集成判断，学号跟账号只能允许为空一个，都为空抛出异常
                if (loginVo.getWorkNumber() == null & loginVo.getAccount() == null) {
                    throw new CustomException(ExceptionEnum.EMPTY_PARAMETER);
                }

                //账号为空，根据学号、工号进行登录判断
                if (StrUtil.isBlank(loginVo.getAccount())) {
                    //根据学号、工号进行登录，验证相关基本信息
                    Integer schoolId = loginVo.getSchoolId();
                    String workNumber = loginVo.getWorkNumber();
                    if (schoolId == null) {
                        throw new CustomException(ExceptionEnum.EMPTY_SCHOOL);
                    }
                    if (workNumber == null) {
                        throw new CustomException(ExceptionEnum.EMPTY_WORK_NUMBER);
                    }
                    HashMap<String, Object> hashMap = workNumberLogin(loginVo);
                    loginDTO.setToken((String) hashMap.get("token"));
                    return loginDTO;
                }

                //账号登录
                HashMap<String, Object> hashMap = accountLogin(loginVo);
                if (ObjectUtil.isNull(hashMap)) {
                    throw new CustomException(ExceptionEnum.ACCOUNT_NOT_EXIST);
                }
                loginDTO.setToken((String) hashMap.get("token"));
                return loginDTO;
            }
        }
        return null;
    }

    /**
     * 根据学号工号登录
     *
     * @param loginVo 登录账号
     * @return token
     */
    public HashMap<String, Object> workNumberLogin(LoginVo loginVo) {
        QueryWrapper<HrUserAccount> wrapper = new QueryWrapper<>();
        wrapper.eq("school_id", loginVo.getSchoolId())
                .eq("work_number", loginVo.getWorkNumber())
                .eq("type", loginVo.getType())
                .eq("platform_id", loginVo.getPlatform());
        //判断学号、工号是否存在
        HrUserAccount hrUserAccount = hrUserAccountMapper.selectOne(wrapper);
        //学号不存在
        if (ObjectUtil.isNull(hrUserAccount) && loginVo.getType() == 1) {
            throw new CustomException(ExceptionEnum.STUDENT_ID_NOT_EXIST);
        }
        //工号不存在
        if (ObjectUtil.isNull(hrUserAccount) && loginVo.getType() == 0) {
            throw new CustomException(ExceptionEnum.JOB_ID_NOT_EXIST);
        }
        return this.queryAccountInfo(hrUserAccount, loginVo.getPassword(), true);
    }

    /**
     * 根据账号登录
     *
     * @param loginVo 登录账号
     * @return token
     */
    public HashMap<String, Object> accountLogin(LoginVo loginVo) {
        QueryWrapper<HrUserAccount> wrapper = new QueryWrapper<>();
        wrapper.eq("account", loginVo.getAccount()).last("and FIND_IN_SET(" + loginVo.getPlatform() + ",platform_id)");
        //判断账号是否存在
        HrUserAccount hrUserAccount = hrUserAccountMapper.selectOne(wrapper);
        if (ObjectUtil.isNull(hrUserAccount)) {
            throw new CustomException(ExceptionEnum.ACCOUNT_NOT_EXIST);
        }
        //无端用户无须判断
        /*if (hrUserAccount.getType()!=2){
            //查询平台权限
            if (!loginVo.getType().equals(hrUserAccount.getType())){
                throw new CustomException(ExceptionEnum.NOT_AUTHORITY);
            }
        }*/
        return this.queryAccountInfo(hrUserAccount, loginVo.getPassword(), false);
    }

    /**
     * 根据手机号或邮箱登录
     *
     * @param loginVo 手机号或邮箱
     * @return token
     */
    public HashMap<String, Object> phoneOrEmailLogin(LoginVo loginVo) {
        HashMap<String, Object> hashMap = Maps.newHashMap();
        //手机验证码登录验证
        //根据账号表查询手机号、用户表查询邮箱（用户表中手机号已弃用）
        if (EmailUtils.isEmail(loginVo.getAccount())) {
            //查询邮箱是否存在
            QueryWrapper<HrUserInfo> wrapper = new QueryWrapper<>();
            wrapper.eq("email", loginVo.getAccount());
            List<HrUserInfo> hrUserInfos = hrUserInfoMapper.selectList(wrapper);

            if (hrUserInfos.size() > 0) {
                ArrayList<HrUserAccount> accounts = new ArrayList<>();
                //获取用户id
                for (HrUserInfo userInfo : hrUserInfos) {
                    HrUserAccount account = hrUserAccountMapper.selectOne(new QueryWrapper<HrUserAccount>().
                            select("work_number", "school_id", "id", "is_enable").
                            eq("user_id", userInfo.getUserId()).
                            last("and FIND_IN_SET(" + loginVo.getPlatform() + ",platform_id)"));
                    account.setUserName(userInfo.getUserName());
                    accounts.add(account);
                }

                accounts.forEach(hrUserAccount -> {
                    hrUserAccount.setSchoolName(hrUserAccountMapper.querySchoolNameBySchoolId(hrUserAccount.getSchoolId()));
                });

                hashMap.put("userAccounts", accounts);
                return hashMap;
            }

            //根据所有用户查询账号数据
            /*//用户登录
            if (ObjectUtil.isNotNull(hrUserInfo)) {
                QueryWrapper<HrUserAccount> accountQueryWrapper = new QueryWrapper<>();
                accountQueryWrapper.
                        eq("user_id", hrUserInfo.getUserId()).
                        last("and FIND_IN_SET(" + loginVo.getPlatform() + ",platform_id)");
                //判断账号是否存在
                QueryWrapper<HrUserAccount> wrapperExist = new QueryWrapper<>();
                wrapperExist.eq("user_id", hrUserInfo.getUserId());
                HrUserAccount hrUserAccount = hrUserAccountMapper.selectOne(wrapperExist);
                if (ObjectUtil.isNull(hrUserAccount)) {
                    if (EmailUtils.isEmail(loginVo.getAccount())) {
                        throw new CustomException(ExceptionEnum.MAIL_NOT_EXIST);
                    }
                    throw new CustomException(ExceptionEnum.PHONE_NOT_EXIST);
                }

                return this.queryAccountInfo(accountQueryWrapper, loginVo.getPassword(), loginVo.getPlatform());
            }*/

            /*//客户登录，用户查询不到，查询客户手机号和邮箱
            Integer customerId = hrUserInfoMapper.selectCustomerId(loginVo.getAccount());
            //客户存在，提示不允许使用手机号和邮箱登录
            if (customerId != null) {
                throw new CustomException(ExceptionEnum.NOT_ALLOWED);
            }*/

            //邮箱不存在
            throw new CustomException(ExceptionEnum.MAIL_NOT_EXIST);
        }

        //查询手机号是否存在,发送之前已经查询一次
        /*QueryWrapper<HrUserAccount> accountQueryWrapper = new QueryWrapper<>();
        accountQueryWrapper.eq("phone", loginVo.getAccount()).
                eq("is_enable",1).
                last("and FIND_IN_SET(" + loginVo.getPlatform() + ",platform_id)");
        List<HrUserAccount> hrUserAccounts = hrUserAccountMapper.selectList(accountQueryWrapper);

        //手机号不存在
        if (ObjectUtil.isNull(hrUserAccounts) || hrUserAccounts.size()<=0) {
            throw new CustomException(ExceptionEnum.PHONE_NOT_EXIST);
        }*/

        return this.phoneVerification(loginVo);

    }

    public HashMap<String, Object> phoneVerification(LoginVo loginVo) {
        HashMap<String, Object> hashMap = Maps.newHashMap();

        //中台手机号登录验证登录密码
        if (loginVo.getPlatform()==3){
            HrUserAccount hrUserAccount = hrUserAccountMapper.selectOne(new QueryWrapper<HrUserAccount>().
                    eq("phone", loginVo.getAccount()).eq("platform_id",loginVo.getPlatform()));

            if (ObjectUtil.isNull(hrUserAccount)) {
                throw new CustomException(ExceptionEnum.PHONE_NOT_EXIST);
            }else {
                //校验密码
                if (!hrUserAccount.getPassword().equals(MD5.encrypt(loginVo.getPassword()))) {
                    throw new CustomException(ExceptionEnum.WRONG_PASSWORD);
                }
                //是否禁用
                if (hrUserAccount.getIsEnable().equals(OrderConstant.Not_enabled)) {
                    throw new CustomException(ExceptionEnum.ACCOUNT_DISABLED);
                }
                //更新用户信息
                updateUserInfo(hrUserAccount.getUserId(), false, hrUserAccount.getId());

                hashMap.put("token", JwtUtils.getJwtToken(StringUtils.join(hrUserAccount.getId()),0L, loginVo.getPlatform()));
                return hashMap;
            }

        }
        String key = StringUtils.join(loginVo.getAccount(), SMSTemplate.AUTHENTICATION_CODE);
        String value = stringRedisTemplate.opsForValue().get(key);
        if (StringUtils.isEmpty(value)) {
            throw new CustomException(ExceptionEnum.CODE_EXPIRED);
        }
        //验证验证码
        if (loginVo.getCode().equals(value)) {
            List<HrUserAccount> hrUserAccounts = getOtherAccount(loginVo.getAccount(), loginVo.getPlatform().toString());
            hashMap.put("userAccounts", hrUserAccounts);
            if (hrUserAccounts.size()==1){
                hashMap.put("token", JwtUtils.getJwtToken(StringUtils.join(hrUserAccounts.get(0).getId()),0L, loginVo.getPlatform()));
            }
            return hashMap;
        }

        throw new CustomException(ExceptionEnum.CODE_ERROR);
    }

    @Override
    public R unbindAccounts(UnbindAccountsReq accountsReq) {

        String key = StringUtils.join(accountsReq.getPhone(), SMSTemplate.AUTHENTICATION_CODE);
        String value = stringRedisTemplate.opsForValue().get(key);
        if (StringUtils.isEmpty(value)) {
            throw new CustomException(ExceptionEnum.CODE_EXPIRED);
        }
        //验证验证码
        if (accountsReq.getCode().equals(value)) {
            List<Integer> accountIds = accountsReq.getAccountIds();
            if (accountIds.size() > 0) {
                for (Integer accountId : accountIds) {
/*                    HrUserAccount userAccount = new HrUserAccount();
                    userAccount.setId(accountId);
                    userAccount.setPhone(null);
                    hrUserAccountMapper.updateById(userAccount);*/
                    //解绑手机号
                    hrUserAccountMapper.unbindMobilePhone(accountId);
                }
            }
            
            QueryWrapper<HrUserAccount> wrapper = new QueryWrapper<>();
            wrapper.eq("school_id", accountsReq.getSchoolId())
                    .eq("work_number", accountsReq.getWorkNumber())
                    .eq("type", accountsReq.getType())
                    .eq("platform_id", accountsReq.getPlatform());
            //手机绑定
            HrUserAccount hrUserAccount = hrUserAccountMapper.selectOne(wrapper);
            hrUserAccount.setPhone(accountsReq.getPhone());
            int update = hrUserAccountMapper.updateById(hrUserAccount);
            if (update > 0) {
                LoginDTO loginDTO = new LoginDTO();
                //登录验证
                HashMap<String, Object> hashMap = queryAccountInfo(hrUserAccount, accountsReq.getPassword(), true);
                if (ObjectUtil.isNull(hashMap)) {
                    throw new CustomException(ExceptionEnum.ACCOUNT_NOT_EXIST);
                }
                loginDTO.setToken((String) hashMap.get("token"));
                return R.ok("认证成功,欢迎 " + hrUserAccount.getWorkNumber() + " 回来！").put("data", loginDTO);
            }
        }
        return R.error();
    }

    /*
    根据手机号或邮箱登录，@param loginVo 手机号或邮箱
    public HashMap<String, Object> phoneOrEmailLogin(LoginVo loginVo) {
        //根据账号表查询手机号、用户表查询邮箱（用户表中手机号已弃用）
        if (EmailUtils.isEmail(loginVo.getAccount())) {
            //邮箱查询
            QueryWrapper<HrUserInfo> wrapper = new QueryWrapper<>();
            wrapper.eq("email", loginVo.getAccount());
            HrUserInfo hrUserInfo = hrUserInfoMapper.selectOne(wrapper);

            //用户登录
            if (ObjectUtil.isNotNull(hrUserInfo)) {
                QueryWrapper<HrUserAccount> accountQueryWrapper = new QueryWrapper<>();
                accountQueryWrapper.
                        eq("user_id", hrUserInfo.getUserId()).
                        last("and FIND_IN_SET(" + loginVo.getPlatform() + ",platform_id)");
                //判断账号是否存在
                QueryWrapper<HrUserAccount> wrapperExist = new QueryWrapper<>();
                wrapperExist.eq("user_id", hrUserInfo.getUserId());
                HrUserAccount hrUserAccount = hrUserAccountMapper.selectOne(wrapperExist);
                if (ObjectUtil.isNull(hrUserAccount)) {
                    if (EmailUtils.isEmail(loginVo.getAccount())) {
                        throw new CustomException(ExceptionEnum.MAIL_NOT_EXIST);
                    }
                    throw new CustomException(ExceptionEnum.PHONE_NOT_EXIST);
                }
                //无端用户无须判断
            *//*if (hrUserAccount.getType()!=2){
                //查询平台权限
                if (!loginVo.getType().equals(hrUserAccount.getType())){
                    throw new CustomException(ExceptionEnum.NOT_AUTHORITY);
                }
            }*//*
                return this.queryAccountInfo(accountQueryWrapper, loginVo.getPassword(), loginVo.getPlatform());
            }

            //客户登录，用户查询不到，查询客户手机号和邮箱
            Integer customerId = hrUserInfoMapper.selectCustomerId(loginVo.getAccount());
            //客户存在，提示不允许使用手机号和邮箱登录
            if (customerId != null) {
                throw new CustomException(ExceptionEnum.NOT_ALLOWED);
            }

            //邮箱不存在
            throw new CustomException(ExceptionEnum.MAIL_NOT_EXIST);
        }

        //手机号进行查询，查询多个账号
        QueryWrapper<HrUserAccount> accountQueryWrapper = new QueryWrapper<>();
        accountQueryWrapper.eq("phone", loginVo.getAccount()).
                last("and FIND_IN_SET(" + loginVo.getPlatform() + ",platform_id)");
        //账号表不存在，查询客户表
        if (ObjectUtil.isNull(accountQueryWrapper)) {
            //客户登录，用户查询不到，查询客户手机号和邮箱
            Integer customerId = hrUserInfoMapper.selectCustomerId(loginVo.getAccount());
            //客户存在，提示不允许使用手机号和邮箱登录
            if (customerId != null) {
                throw new CustomException(ExceptionEnum.NOT_ALLOWED);
            }
        }else {
            return this.queryAccountInfo(accountQueryWrapper, loginVo.getPassword(), loginVo.getPlatform());
        }

        throw new CustomException(ExceptionEnum.PHONE_NOT_EXIST);
    }*/

    /**
     * 账号信息效验，生成token
     *
     * @param password            密码
     * @return token
     */
    public HashMap<String, Object> queryAccountInfo(HrUserAccount hrUserAccount, String password, boolean verify) {
        HashMap<String, Object> hashMap = Maps.newHashMap();
        if (ObjectUtil.isNotNull(hrUserAccount)) {
            //校验密码
            if (!hrUserAccount.getPassword().equals(MD5.encrypt(password))) {
                throw new CustomException(ExceptionEnum.WRONG_PASSWORD);
            }
            //是否禁用
            if (hrUserAccount.getIsEnable().equals(OrderConstant.Not_enabled)) {
                throw new CustomException(ExceptionEnum.ACCOUNT_DISABLED);
            }
            //验证手机号是否绑定
            if (verify) {
                if (ObjectUtil.isEmpty(hrUserAccount.getPhone())) {
                    throw new CustomException(ExceptionEnum.NO_BIND_PHONE);
                }
            }
            //客户为空，更新用户信息
            if (hrUserAccount.getUserId() != null) {
                boolean customer = false;
                if (hrUserAccount.getCustomerId() != null) {
                    customer = true;
                }
                //更新用户信息
                updateUserInfo(hrUserAccount.getUserId(), customer, hrUserAccount.getId());
                // hashMap.put("customer", false);
            }
            //客户不为为空，判断有无订单，订单时间未过期，产品是否启用，是否发货(取消验证)
            /*if (hrUserAccount.getCustomerId() != null) {
                customerVerification(hrUserAccount.getCustomerId(), platform);
                String customerName = baseMapper.selectCustomerName(hrUserAccount.getCustomerId());
                hashMap.put("customer", true);
                hashMap.put("customerName", customerName);
            }*/
            hashMap.put("token", JwtUtils.getJwtToken(StringUtils.join(hrUserAccount.getId()),0L, Integer.valueOf(hrUserAccount.getPlatformId())));
            return hashMap;
        }
        throw new CustomException(ExceptionEnum.NOT_AUTHORITY);
    }

    @Override
    public R getToken(String id) {
//        String token = JwtUtils.getJwtToken(id);
        LoginDTO data = new LoginDTO();
//        data.setToken(token);
        HrUserAccount account = hrUserAccountMapper.selectById(id);
        String name = account.getAccount();
/*        List<Integer> roles = userRoleService.queryAccountRoles(id);
        if (loginVo.getType()!=1 && roles!=null && roles.size()<=0){
            return R.error("该账号无角色权限，无法登录，请联系管理员添加角色！");
        }*/
        return R.ok("认证成功,欢迎 " + name + " 回来！").put("data", data);
        // return R.ok().put("token",token);
    }

    /**
     * 客户账号验证
     *
     * @param customerId 客户id
     * @param platform   平台
     */
    public void customerVerification(Integer customerId, Integer platform) {
        R customerOrder = nakadaiClient.getCustomerOrder(customerId);
        Object orderList = customerOrder.get("orderList");
        String jsonString = JSON.toJSONString(orderList);
        List<Order> orders = JSON.parseArray(jsonString, Order.class);

        //客户无订单
        if (orders.size() == 0) {
            throw new CustomException(ExceptionEnum.NO_ORDER);
        }
        Integer authority = 0;
        //数据平台登录
        if (platform.equals(PlatformConstant.DATA_PLATFORM)) {
            authority = 0;
        }
        //职站登录
        if (platform.equals(PlatformConstant.POST_STATION)) {
            authority = 1;
        }
        List<OrderOther> orderOthers = new ArrayList<>();
        for (Order order : orders) {
            R orderOther = nakadaiClient.getOrderOther(order.getOrderId(), authority);
            Object orderOtherList = orderOther.get("orderOtherList");
            String toJSONString = JSON.toJSONString(orderOtherList);
            List<OrderOther> otherList = JSON.parseArray(toJSONString, OrderOther.class);
            if (otherList.size() != 0) {
                orderOthers.addAll(otherList);
            }
        }
        List<Integer> ship = new ArrayList<>();
        List<Integer> isEnable = new ArrayList<>();
        orderOthers.forEach(orderOther -> {
            ship.add(orderOther.getShip());
            isEnable.add(orderOther.getIsEnable());
        });
        //订单都未发货
        boolean contains = ship.contains(OrderConstant.SHIPPED);
        if (!contains) {
            throw new CustomException(ExceptionEnum.NO_SHIPPED);
        }
        //订单都未启用
        boolean enable = isEnable.contains(OrderConstant.Enable);
        if (!enable) {
            throw new CustomException(ExceptionEnum.NO_ENABLE);
        }
        //过滤订单信息
        List<OrderOther> collect = orderOthers.stream().filter(orderOther -> {
            return orderOther.getIsEnable() == 1;
        }).collect(Collectors.toList());
        List<Boolean> maturity = new ArrayList<>();
        //查询订单是否到期
        collect.forEach(orderOther -> {
            Date startTime = orderOther.getStartTime();
            Date endTime = orderOther.getEndTime();
            boolean in = DateUtil.isIn(new Date(), startTime, endTime);
            maturity.add(in);
        });
        //订单都过期
        boolean end = maturity.contains(true);
        if (!end) {
            throw new CustomException(ExceptionEnum.MATURITY);
        }
    }

    /**
     * 更新用户信息，登录次数和时间
     */
    public void updateUserInfo(Integer userId, boolean customer, Integer accountId) {
        QueryWrapper<HrUserInfo> wrapper = new QueryWrapper<>();
        wrapper.eq("user_id", userId);
        HrUserInfo userInfo = baseMapper.selectOne(wrapper);

//        HrUserAccount userAccount = userAccountMapper.selectById(accountId);

        if (ObjectUtil.isNull(userInfo)) {
            throw new CustomException(ExceptionEnum.USER_NOT_EXIST);
        }
        //手机号验证
        /*if (!customer){
            //更新用户信息之前，判断用户是否绑定手机号
            if (StrUtil.isEmpty(userInfo.getPhone()) || StrUtil.isBlank(userInfo.getPhone()) || StrUtil.isNullOrUndefined(userInfo.getPhone())) {
                throw new CustomException(ExceptionEnum.NO_BIND_PHONE);
            }
        }*/
        userInfo.setLastLoginTime(new Date());
        //新用户首次登录跳转至日志通知
        if (userInfo.getLoginNumber() == 0) {
            redisTemplate.opsForValue().set(accountId.toString(), "true");
        }
        userInfo.setLoginNumber(userInfo.getLoginNumber() + 1);
        baseMapper.updateById(userInfo);
    }

    /**
     * 校验验证码
     *
     * @param random 随机数
     * @param code   验证码
     * @return boolean
     */
    public Boolean check(String random, String code) {
        String value = redisTemplate.opsForValue().get(random);
        //验证码已过期
        if (StringUtils.isEmpty(value)) {
            throw new CustomException(ExceptionEnum.CODE_EXPIRED);
        }
        //验证码错误
        if (!code.equals(value)) {
            throw new CustomException(ExceptionEnum.CODE_ERROR);
        }
        return redisTemplate.delete(random);
    }

    /**
     * 生成算数图形验证码
     *
     * @param random   随机数
     * @param response 图形验证码
     */
    @Override
    public void createCaptcha(String random, HttpServletResponse response) {
        Captcha captcha = new ArithmeticCaptcha();
        //生成验证码
        String text = captcha.text();
        //缓存验证码
        redisTemplate.opsForValue().set(random, text, 5, TimeUnit.MINUTES);
        response.setContentType(MediaType.IMAGE_PNG_VALUE);
        response.setHeader(HttpHeaders.PRAGMA, "No-cache");
        response.setHeader(HttpHeaders.CACHE_CONTROL, "No-cache");
        response.setDateHeader(HttpHeaders.EXPIRES, 0L);
        try {
            captcha.out(response.getOutputStream());
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Override
    public List<HrUserAccount> getOtherAccount(String phone, String platform) {

        //根据手机号查询当前平台的其他账号用户信息
        List<HrUserAccount> hrUserAccounts = hrUserAccountMapper.selectList(new QueryWrapper<HrUserAccount>().
                select("work_number", "school_id", "id", "user_id", "is_enable").
                eq("phone", phone).
                last("and FIND_IN_SET(" + platform + ",platform_id)"));

        if (hrUserAccounts != null && hrUserAccounts.size() > 0) {
            hrUserAccounts.forEach(hrUserAccount -> {
                hrUserAccount.setSchoolName(hrUserAccountMapper.querySchoolNameBySchoolId(hrUserAccount.getSchoolId()));
                HrUserInfo userInfo = hrUserInfoMapper.selectById(hrUserAccount.getUserId());
                hrUserAccount.setUserName(userInfo.getUserName());
            });

            return hrUserAccounts;
        }
        return null;
    }

    /**
     * @Description :   校验手机号码
     * @Param phone
     * @Author Rong---2021/10/18
     */
    @Override
    public boolean checkPhoneOrEmailExist(String phone, String email, Integer type) {

        switch (type) {
            case 1:
                QueryWrapper<HrUserAccount> accountQueryWrapper = new QueryWrapper<>();
                accountQueryWrapper.eq("is_del", DelConstant.NOT_DEL);
                accountQueryWrapper.eq("phone", phone);
                Integer count = hrUserAccountMapper.selectCount(accountQueryWrapper);
                return count > 0;
            case 2:
                QueryWrapper<HrUserInfo> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("is_del", DelConstant.NOT_DEL);
                queryWrapper.eq("email", email);
                Integer count1 = baseMapper.selectCount(queryWrapper);
                return count1 > 0;
        }
        return false;
    }

    @Override
    public String getUserName(Integer accountId) {
        return baseMapper.getUserName(accountId);
    }

    private R getAccountList(Integer userId) {
        //根据用户id查询账号
        List<HrUserAccount> accountList = hrUserAccountMapper.selectList(new QueryWrapper<HrUserAccount>().
                eq("user_id", userId).select("account", "school_id", "platform_id", "type"));
        //只有一个账号无选择列表进行选择
        if (accountList.size() == 1) {
            return R.ok();
        }
        accountList.forEach(account -> {
            Integer schoolId = account.getSchoolId();
            String schoolName = baseMapper.selectSchoolName(schoolId);
            account.setSchoolName(schoolName);
            if (account.getPlatformId().contains(PlatformConstant.POST_STATION.toString())) {
                account.setPlatformName("职站");
            }
            if (account.getPlatformId().contains(PlatformConstant.DATA_PLATFORM.toString())) {
                account.setPlatformName("数据平台");
            }
            if (account.getPlatformId().contains(PlatformConstant.NAKADAI.toString())) {
                account.setPlatformName("中台");
            }
            if (account.getType().equals(0)) {
                account.setTypeName("教师端");
            }
            if (account.getType().equals(1)) {
                account.setTypeName("学生端");
            }
            if (account.getType().equals(2)) {
                account.setTypeName("该平台不区分端口！");
            }
        });
        return R.ok().put("accountList", accountList);
    }

    /**
     * 根据code获取用户openid
     * @param code 微信登录code
     * @return OpenId
     * @throws Exception
     */
    private JSONObject getSessionKeyOrOpenIdByPartner(String code) throws Exception {
        Map<String, String> requestUrlParam = new HashMap<>();
        // 小程序appId，自己补充
        requestUrlParam.put("appid", WeChatPropertiesUtil.PARTNER_WX_OPEN_APP_ID);
        // 小程序secret，自己补充
        requestUrlParam.put("secret", WeChatPropertiesUtil.PARTNER_WX_OPEN_APP_SECRET);
        // 小程序端返回的code
        requestUrlParam.put("js_code", code);
        // 默认参数
        requestUrlParam.put("grant_type", GRANT_TYPE);

        // 发送post请求读取调用微信接口获取openid用户唯一标识
        String result = HttpClientUtils.doPost(REQUEST_URL, requestUrlParam);
        return JSON.parseObject(result);
    }

    private JSONObject getSessionKeyOrOpenIdByOperation(String code) throws Exception {
        Map<String, String> requestUrlParam = new HashMap<>();
        // 小程序appId，自己补充
        requestUrlParam.put("appid", WeChatPropertiesUtil.OPERATION_WX_OPEN_APP_ID);
        // 小程序secret，自己补充
        requestUrlParam.put("secret", WeChatPropertiesUtil.OPERATION_WX_OPEN_APP_SECRET);
        // 小程序端返回的code
        requestUrlParam.put("js_code", code);
        // 默认参数
        requestUrlParam.put("grant_type", GRANT_TYPE);

        // 发送post请求读取调用微信接口获取openid用户唯一标识
        String result = HttpClientUtils.doPost(REQUEST_URL, requestUrlParam);
        return JSON.parseObject(result);
    }


    private JSONObject getSessionKeyOrOpenIdByKindergarten(String code) throws Exception {
        Map<String, String> requestUrlParam = new HashMap<>();
        // 小程序appId，自己补充
        requestUrlParam.put("appid", WeChatPropertiesUtil.KINDERGARTEN_WX_OPEN_APP_ID);
        // 小程序secret，自己补充
        requestUrlParam.put("secret", WeChatPropertiesUtil.KINDERGARTEN_WX_OPEN_APP_SECRET);
        // 小程序端返回的code
        requestUrlParam.put("js_code", code);
        // 默认参数
        requestUrlParam.put("grant_type", GRANT_TYPE);

        // 发送post请求读取调用微信接口获取openid用户唯一标识
        String result = HttpClientUtils.doPost(REQUEST_URL, requestUrlParam);
        return JSON.parseObject(result);
    }

    /**
     * 微信接口请求令牌：生产环境下 要用redis缓存起来
     * @return AccessToken
     */
    /*public String getAccessToken() {
        String url = "https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid="
                + WeChatPropertiesUtil.WX_OPEN_APP_ID + "&secret=" + WeChatPropertiesUtil.WX_OPEN_APP_SECRET;
        String accessToken = null;
        try {
            URL urlGet = new URL(url);
            HttpURLConnection http = (HttpURLConnection) urlGet.openConnection();
            http.setRequestMethod("GET"); // 必须是get方式请求
            http.setRequestProperty("Content-Type","application/x-www-form-urlencoded");
            http.setDoOutput(true);
            http.setDoInput(true);
            System.setProperty("sun.net.client.defaultConnectTimeout", "30000");// 连接超时30秒
            System.setProperty("sun.net.client.defaultReadTimeout", "30000"); // 读取超时30秒
            http.connect();
            InputStream is = http.getInputStream();
            int size = is.available();
            byte[] jsonBytes = new byte[size];
            is.read(jsonBytes);
            String message = new String(jsonBytes, "UTF-8");
            JSONObject demoJson = JSONObject.parseObject(message);
            accessToken = demoJson.getString("access_token");
            is.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return accessToken;
    }*/

    @Override
    public Map<String ,Object> weChatAppletCallbackByPartner(AppletCallbackReq appletCallbackReq, HttpServletResponse response) throws Exception {
        String code = appletCallbackReq.getCode();

        Map<String, Object> data = new HashedMap<String, Object>();
        // try {
            JSONObject sessionKeyOrOpenId = this.getSessionKeyOrOpenIdByPartner(code);
            Object openid = sessionKeyOrOpenId.get("openid");
            //根据openid查询用户，存在则返回登录状态
            QueryWrapper<HrUserAccount> wrapper = new QueryWrapper<>();
            wrapper.eq("app_open_id",openid);
            wrapper.eq("platform_id",4);
            HrUserAccount userAccount = hrUserAccountMapper.selectOne(wrapper);
            if (ObjectUtil.isNull(userAccount)){
                data.put("state", "bind");
            }else {
                //登录查询是否有团队
                Integer exist = hrUserAccountMapper.selectTeam(userAccount.getId());
                if (exist>0){
                    //更新登录时间
                    updateUserInfo(userAccount.getUserId(),false,userAccount.getId());
                    data.put("state", "login");
//                    data.put("token", JwtUtils.getJwtToken(userAccount.getId().toString()));
                    data.put("accountId", userAccount.getId());
                }else {
                    throw new CustomException(ExceptionEnum.TEAM_NOT_EXIST);
                }
            }
            System.out.println(sessionKeyOrOpenId);
            data.put("sessionKey", sessionKeyOrOpenId);
        return data;
    }


    @Override
    public R getSessionKey(AppletCallbackReq appletCallbackReq, HttpServletResponse response) throws Exception {
        String code = appletCallbackReq.getCode();
        JSONObject sessionKeyOrOpenId = this.getSessionKeyOrOpenIdByPartner(code);
        return R.ok().put("sessionKey", sessionKeyOrOpenId);
    }

    @Override
    public R loginByOpenid(String openid) {
        Map<String, Object> data = new HashedMap<String, Object>();
        //根据openid查询用户，存在则返回登录状态
        QueryWrapper<HrUserAccount> wrapper = new QueryWrapper<>();
        wrapper.eq("app_open_id",openid);
        wrapper.eq("platform_id",4);
        HrUserAccount userAccount = hrUserAccountMapper.selectOne(wrapper);
        if (ObjectUtil.isNull(userAccount)){
            return R.error("账号不存在！");
        }else {
            //更新登录时间
            updateUserInfo(userAccount.getUserId(),false,userAccount.getId());
            data.put("state", "login");
//            data.put("token", JwtUtils.getJwtToken(userAccount.getId().toString()));
            data.put("accountId", userAccount.getId());
        }
        return R.ok().put("data",data);
    }

    @Override
    public R updateAvatars(String url,String id) {
        //更新用户头像
        QueryWrapper<HrUserAccount> wrapper = new QueryWrapper<>();
        wrapper.eq("platform_id",4);
        wrapper.eq("id",id);
        HrUserAccount userAccount = hrUserAccountMapper.selectOne(wrapper);

        HrUserInfo userInfo = hrUserInfoMapper.selectById(userAccount.getUserId());
        if (url!=null){
            userInfo.setUserAvatars(url);
            hrUserInfoMapper.updateById(userInfo);
        }
        return R.ok();
    }

    @Override
    public Map<String ,Object> weChatAppletCallbackByOperation(AppletCallbackReq appletCallbackReq, HttpServletResponse response) throws Exception {
        String code = appletCallbackReq.getCode();

        Map<String, Object> data = new HashedMap<String, Object>();
        JSONObject sessionKeyOrOpenId = this.getSessionKeyOrOpenIdByOperation(code);
        Object openid = sessionKeyOrOpenId.get("openid");
        //根据openid查询用户，存在则返回登录状态
        QueryWrapper<HrUserAccount> wrapper = new QueryWrapper<>();
        wrapper.eq("app_open_id",openid);
        wrapper.eq("platform_id",3);
        HrUserAccount userAccount = hrUserAccountMapper.selectOne(wrapper);
        if (ObjectUtil.isNull(userAccount)){
            data.put("state", "bind");
        }else {
            //更新登录时间
            updateUserInfo(userAccount.getUserId(),false,userAccount.getId());
            data.put("state", "login");
//            data.put("token", JwtUtils.getJwtToken(userAccount.getId().toString()));
            data.put("accountId", userAccount.getId());
        }
        System.out.println(sessionKeyOrOpenId);
        data.put("sessionKey", sessionKeyOrOpenId);
        return data;
    }


    @Override
    public Map<String, Object> weChatAppletCallbackByKindergarten(AppletCallbackReq appletCallbackReq, HttpServletResponse response) throws Exception {
        String code = appletCallbackReq.getCode();

        Map<String, Object> data = new HashedMap<String, Object>();
        JSONObject sessionKeyOrOpenId = this.getSessionKeyOrOpenIdByKindergarten(code);
        Object openid = sessionKeyOrOpenId.get("openid");
        //根据openid查询用户，存在则返回登录状态
        QueryWrapper<HrUserAccount> wrapper = new QueryWrapper<>();
        wrapper.eq("app_open_id",openid);
        wrapper.eq("platform_id",appletCallbackReq.getPlatform());
        HrUserAccount userAccount = hrUserAccountMapper.selectOne(wrapper);
        //为空注册用户信息
        if (ObjectUtil.isNull(userAccount)){
            data.put("state", "bind");
        }else {
            //更新登录时间
            updateUserInfo(userAccount.getUserId(),false,userAccount.getId());
//            data.put("token", JwtUtils.getJwtToken(userAccount.getId().toString()));
            data.put("accountId", userAccount.getId());
        }
        System.out.println(sessionKeyOrOpenId);
        data.put("sessionKey", sessionKeyOrOpenId);
        return data;
    }

    @Override
    public String userBinding(WxUserBindingReq userBindingReq) {
        if (userBindingReq.getPlatformId()==4){
            //根据手机号查询用户是否存在
            QueryWrapper<HrUserAccount> wrapper = new QueryWrapper<>();
            wrapper.eq("phone",userBindingReq.getPhone());
            wrapper.eq("platform_id",4);
            wrapper.eq("is_enable",1);
            HrUserAccount userAccount = hrUserAccountMapper.selectOne(wrapper);
            if (ObjectUtil.isNull(userAccount)){
                throw new CustomException(ExceptionEnum.USER_NOT_EXIST);
            }
            //更新登录时间
            updateUserInfo(userAccount.getUserId(),false,userAccount.getId());
            //查询是否有团队
            Integer exist = hrUserAccountMapper.selectTeam(userAccount.getId());
            if (exist>0){
                userAccount.setAppOpenId(userBindingReq.getOpenid());
                userAccount.setUnionid(userBindingReq.getUnionid());
                hrUserAccountMapper.updateById(userAccount);
                return JwtUtils.getJwtToken(userAccount.getId().toString(),0L, userBindingReq.getPlatformId());
            }
            throw new CustomException(ExceptionEnum.TEAM_NOT_EXIST);
        }else {
            //根据手机号查询用户是否存在
            QueryWrapper<HrUserAccount> wrapper = new QueryWrapper<>();
            wrapper.eq("phone",userBindingReq.getPhone());
            wrapper.eq("platform_id",3);
            wrapper.eq("is_enable",1);
            HrUserAccount userAccount = hrUserAccountMapper.selectOne(wrapper);
            if (ObjectUtil.isNull(userAccount)){
                throw new CustomException(ExceptionEnum.USER_NOT_EXIST);
            }
            //更新登录时间
            updateUserInfo(userAccount.getUserId(),false,userAccount.getId());
            userAccount.setAppOpenId(userBindingReq.getOpenid());
            userAccount.setUnionid(userBindingReq.getUnionid());
            hrUserAccountMapper.updateById(userAccount);
            return JwtUtils.getJwtToken(userAccount.getId().toString(),0L, userBindingReq.getPlatformId());
        }
    }

    /**
     * 参数以raw的方式做post请求
     * @param url
     * @param stringJson
     * @param headers
     * @param encode
     * @return
     */
    public static String httpPostRaw(String url, String stringJson, Map<String, String> headers, String encode) {
        String str = "";
        if (encode == null) {
            encode = "utf-8";
        }
        // HttpClients.createDefault()等价于 HttpClientBuilder.create().build();
        CloseableHttpClient closeableHttpClient = HttpClients.createDefault();
        HttpPost httpost = new HttpPost(url);

        // 设置header
        httpost.setHeader("Content-type", "application/json");
        if (headers != null && headers.size() > 0) {
            for (Map.Entry<String, String> entry : headers.entrySet()) {
                httpost.setHeader(entry.getKey(), entry.getValue());
            }
        }
        // 组织请求参数
        StringEntity stringEntity = new StringEntity(stringJson, encode);
        httpost.setEntity(stringEntity);
        String content = null;
        CloseableHttpResponse httpResponse = null;
        try {
            // 响应信息
            httpResponse = closeableHttpClient.execute(httpost);
            HttpEntity entity = httpResponse.getEntity();
            content = EntityUtils.toString(entity, encode);
            System.out.println(content);
            str = content;
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                httpResponse.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        try { // 关闭连接、释放资源
            closeableHttpClient.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return str;
    }

    @Override
    @SneakyThrows
    public R messageNotification(WxMssVo wxMssVo) {
        //miniprogram_state	string	是	跳转小程序类型：developer为开发版；trial为体验版；formal为正式版；默认为正式版
        //lang	string	是	进入小程序查看”的语言类型，支持zh_CN(简体中文)、en_US(英文)、zh_HK(繁体中文)、zh_TW(繁体中文)，默认为zh_CN
        //拼接消息内容
//        wxMssVo.setTemplate_id("8pDVfWYqh9c-nn3CeA1NXM58pmoyQXZzkVnRFKy_l2A");
        wxMssVo.setMiniprogram_state("trial");
        wxMssVo.setLang("zh_CN");
        String accessToken=null;
        try {
            //设置openId
//            wxMssVo.setTouser("osj9X5LWbk5nPxxw0AJ5un5XJ8cs");
//            设置accessToken
            accessToken= WeChatUtil.getAccessToken();
        } catch (IOException e) {
            e.printStackTrace();
        }
//        设置小程序 跳转首页
//        wxMssVo.setPage("pages/index/index");

        wxMssVo.setRequest_url("https://api.weixin.qq.com/cgi-bin/message/subscribe/send?access_token=" + accessToken);
//        将模板中的值一一赋值 发送到小程序的数据要转化为json格式 可以使用下面的方法直接拼接成JSONObject
        cn.hutool.json.JSONObject jsonObject = new cn.hutool.json.JSONObject();
        cn.hutool.json.JSONObject dataInfo = new cn.hutool.json.JSONObject();
        dataInfo.putOpt("value", new Date());
        jsonObject.putOpt("time5", dataInfo);

//        使用map结合模板 后续再加遍历拼接成JSONObject
//        HashMap<String, Object> map = new HashMap<>(3);
//        //根据模板写值
//        map.put("thing2", "测试学校");
//        map.put("thing8","下班打卡");
//        map.put("phrase10","下班打卡");
//        wxMssVo.setMap(map);
        String s = WeChatUtil.sendTemplateMessage(wxMssVo);
        return R.ok().put("s",s);

        /*// 微信小程序ID
        String appid = WeChatPropertiesUtil.PARTNER_WX_OPEN_APP_ID;
        // 微信小程序秘钥
        String secret =WeChatPropertiesUtil.PARTNER_WX_OPEN_APP_SECRET;

        String urlAt="https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid="+appid+"&secret="+secret;
        // 发送请求，返回Json字符串
        String strAt = WeChatUtil.httpRequest(urlAt, "GET", null);
        // 转成Json对象 获取openid
        JSONObject fromObject = JSONObject.parseObject(strAt);
        //获取at
        String accessToken = fromObject.getString("access_token");
        System.out.println("后台获取的"+accessToken);

        String url = "https://api.weixin.qq.com/cgi-bin/message/subscribe/send?access_token="+accessToken;
        // 发送请求，返回Json字符串
        String str1 = WeChatUtil.httpRequest(url, "POST", format);
        // 转成Json对象 获取openid
        JSONObject fromObject1 = JSONObject.parseObject(str1);

        System.out.println("jsonObject____============"+fromObject1.toString());
*/
    }

    @Override
    public R personalAccountApplication(AccountRegisterReq registerReq) {
        registerReq.setPlatformId("7");
        //创建账号，用户信息
        //查看平台账号信息，账号不为空直接返回token登录
        HrUserAccount hrUserAccount = hrUserAccountMapper.selectOne(new QueryWrapper<HrUserAccount>().
                    eq("phone", registerReq.getPhone()).eq("platform_id",registerReq.getPlatformId()));

        if (ObjectUtil.isNull(hrUserAccount)) {
            String randomNumbers = cn.hutool.core.util.RandomUtil.randomNumbers(6);
            //用户相关信息表
            UserInfo userInfo = new UserInfo();
            BeanUtils.copyProperties(registerReq, userInfo);
            userInfo.setUserName("U"+randomNumbers);
            userInfo.setLoginNumber(1);
            userInfo.setLastLoginTime(new Date());
            int addUserInfo = baseMapper.addUserInfo(userInfo);

            UserAccount userAccount = new UserAccount();
            BeanUtils.copyProperties(registerReq, userAccount);
            userAccount.setPassword(MD5.encrypt("1122aa"));
            userAccount.setUserId(userInfo.getUserId());
            userAccount.setType(2);//平台端区分：0->教师端 1->学生端 2->无端
            userAccount.setPlatformId(registerReq.getPlatformId());
            userAccount.setAppOpenId(registerReq.getAppOpenId());
            userAccount.setAccount("A"+randomNumbers);
            int addAccount = baseMapper.addUserAccount(userAccount);

            //注册成功返回token
            if (addUserInfo > 0 && addAccount > 0){
                LoginDTO loginDTO = new LoginDTO();
//                loginDTO.setToken(JwtUtils.getJwtToken(userAccount.getId().toString()));
                return R.ok().put("data", loginDTO);
            }

        }else {
            //是否禁用
            if (hrUserAccount.getIsEnable().equals(OrderConstant.Not_enabled)) {
                throw new CustomException(ExceptionEnum.ACCOUNT_DISABLED);
            }
            //更新用户信息
            updateUserInfo(hrUserAccount.getUserId(), false, hrUserAccount.getId());
            LoginDTO loginDTO = new LoginDTO();
//            loginDTO.setToken(JwtUtils.getJwtToken(hrUserAccount.getId().toString()));
            return R.ok().put("data", loginDTO);
        }

        return R.error();
    }
}
