package com.huoran.users.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huoran.users.entity.DataPermission;
import com.huoran.users.entity.Permission;
import com.huoran.users.entity.RolePermission;
import com.huoran.users.entity.vo.DataPermissionVo;
import com.huoran.users.mapper.PermissionMapper;
import com.huoran.users.mapper.SupplierMapper;
import com.huoran.users.service.DataPermissionService;
import com.huoran.users.service.PermissionService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huoran.users.service.RolePermissionService;
import com.huoran.users.service.SupplierService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 权限表 服务实现类
 * </p>
 *
 * @author chen
 * @since 2021-09-29
 */
@Service
public class PermissionServiceImpl extends ServiceImpl<PermissionMapper, Permission> implements PermissionService {

    @Autowired
    private RolePermissionService rolePermissionService;

    @Autowired
    private DataPermissionService dataPermissionService;

    @Autowired
    private SupplierService supplierService;

    @Override
    public List<Permission> queryAllMenu(Integer platformId,Integer port) {
        QueryWrapper<Permission> wrapper = new QueryWrapper<>();
        wrapper.eq("platform_id",platformId);
        if (port!=null){
            wrapper.eq("port",port);
        }else {
            wrapper.eq("port",0);
        }
        List<Permission> permissionList = baseMapper.selectList(wrapper);
        return build(permissionList);
    }

    @Override
    public void removeChildById(Integer id) {
        List<Integer> idList = new ArrayList<>();
        this.selectChildListById(id, idList);
        idList.add(id);
        baseMapper.deleteBatchIds(idList);
    }

    /**
     * 递归获取子节点
     */
    private void selectChildListById(Integer id, List<Integer> idList) {
        List<Permission> childList = baseMapper.selectList(new QueryWrapper<Permission>().eq("pid", id).select("id"));
        childList.forEach(item -> {
            idList.add(item.getId());
            this.selectChildListById(item.getId(), idList);
        });
    }

    /**
     * 给角色分配权限
     * @param roleId 角色
     * @param permissionIds 权限值
     */
    @Override
    public void saveRolePermissionRelationShip(Integer roleId, Integer[] permissionIds) {
        //给角色分配权限之前，删除之前所有角色权限
        rolePermissionService.remove(new QueryWrapper<RolePermission>().eq("role_id", roleId));

        List<RolePermission> rolePermissionList = new ArrayList<>();
        for(Integer permissionId : permissionIds) {
            if(StringUtils.isEmpty(permissionId)) {
                continue;
            }
            RolePermission rolePermission = new RolePermission();
            rolePermission.setRoleId(roleId);
            rolePermission.setPermissionId(permissionId);
            rolePermissionList.add(rolePermission);
        }
        rolePermissionService.saveBatch(rolePermissionList);
    }

    @Override
    public void assigningDataPermissions(Integer roleId, List<DataPermission> dataPermissionList) {
        //给角色数据权限之前，删除之前所有角色数据权限
        dataPermissionService.remove(new QueryWrapper<DataPermission>().eq("role_id", roleId));
        //添加数据权限
        for (DataPermission dataPermission : dataPermissionList) {
            dataPermission.setRoleId(roleId);
            dataPermissionService.save(dataPermission);
        }

    }

    /**
     * 根据角色id获取权限菜单
     * @param roleIds 角色id
     * @return 权限菜单
     */
    @Override
    public List<Permission> selectRoleMenu(String roleIds,Integer platformId) {
        List<Permission> allPermissionList = baseMapper.selectRolePermission(roleIds,platformId);
        return build(allPermissionList);
    }

    @Override
    public List<DataPermissionVo> queryDataPermission(String roleIds) {
        List<DataPermissionVo> dataPermissionList = dataPermissionService.selectDataPermission(roleIds);
        int count = supplierService.count(null);
        for (DataPermissionVo dataPermissionVo : dataPermissionList) {
//            客户权限id
//            if (dataPermissionVo.getPermissionId()==135){
//                客户拥有所有供应商，默认查询全部客户，包括未下单客户
                if (dataPermissionVo.getSupplierId().length() == count*2-1){
                    dataPermissionVo.setAll(true);
                }
//            }
        }
        return dataPermissionList;
    }

    /**
     * 使用递归方法建菜单
     * @param treeNodes 权限菜单
     * @return 权限菜单
     */
    private static List<Permission> build(List<Permission> treeNodes) {
        List<Permission> trees = new ArrayList<>();
        for (Permission treeNode : treeNodes) {
            if (treeNode.getPid()==0) {
                treeNode.setLevel(1);
                trees.add(findChildren(treeNode,treeNodes));
            }
        }
        return trees;
    }

    /**
     * 递归查找子节点
     * @param treeNodes 权限子菜单
     * @return 权限子菜单
     */
    private static Permission findChildren(Permission treeNode,List<Permission> treeNodes) {
        treeNode.setChildren(new ArrayList<Permission>());

        for (Permission it : treeNodes) {
            if(treeNode.getId().equals(it.getPid())) {
                int level = treeNode.getLevel() + 1;
                it.setLevel(level);
                if (treeNode.getChildren() == null) {
                    treeNode.setChildren(new ArrayList<>());
                }
                treeNode.getChildren().add(findChildren(it,treeNodes));
            }
        }
        return treeNode;
    }
}
