package com.huoran.users.service.impl;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.huoran.common.response.R;
import com.huoran.users.entity.*;
import com.huoran.users.mapper.*;
import com.huoran.users.service.RolePermissionService;
import com.huoran.users.service.RoleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 角色表 服务实现类
 * </p>
 *
 * @author chen
 * @since 2021-09-29
 */
@Service
public class RoleServiceImpl extends ServiceImpl<RoleMapper, Role> implements RoleService {

    @Autowired
    private RolePermissionMapper rolePermissionMapper;

    @Autowired
    private RolePermissionService rolePermissionService;

    @Autowired
    private PermissionMapper permissionMapper;

    @Autowired
    private UserRoleMapper userRoleMapper;

    @Autowired
    private HrUserAccountMapper accountMapper;

    @Override
    public boolean repeat(Role role) {
        QueryWrapper<Role> roleQueryWrapper = new QueryWrapper<>();
        roleQueryWrapper.eq("role_name",role.getRoleName());
        roleQueryWrapper.eq("school_id",role.getSchoolId());
        roleQueryWrapper.eq("platform_id",role.getPlatformId());
        roleQueryWrapper.last("and id not in ("+role.getId()+")");
        Integer count = baseMapper.selectCount(roleQueryWrapper);
        return count>0;
    }

    @Override
    public boolean batchRemove(List<Integer> roleIds) {
        //删除该角色对应的权限
        for (Integer roleId : roleIds) {
            QueryWrapper<RolePermission> wrapper = new QueryWrapper<>();
            wrapper.eq("role_id",roleId);
            rolePermissionMapper.delete(wrapper);

            QueryWrapper<UserRole> userRoleQueryWrapper = new QueryWrapper<>();
            userRoleQueryWrapper.eq("role_id",roleId);
            userRoleMapper.delete(userRoleQueryWrapper);
        }
        //删除角色
        int deleteBatchIds = baseMapper.deleteBatchIds(roleIds);
        return deleteBatchIds > 0;
    }

    @Override
    public Role getRoleInfo(Integer schoolId, String roleName,Integer platformId) {
        QueryWrapper<Role> roleQueryWrapper = new QueryWrapper<>();
        roleQueryWrapper.eq("role_name",roleName);
        roleQueryWrapper.eq("school_id",schoolId);
        roleQueryWrapper.eq("platform_id",platformId);
        return baseMapper.selectOne(roleQueryWrapper);
    }

    @Override
    public boolean addCustomerAdmin(Integer schoolId, String accountId) {
        /*QueryWrapper<Role> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("school_id",schoolId).eq("platform_id",platformId).eq("role_name","超级管理员");
        Role one = baseMapper.selectOne(queryWrapper);*/

        ArrayList<Integer> platformIds = Lists.newArrayList();
        platformIds.add(1);
        platformIds.add(2);
        for (Integer platformId : platformIds) {
            //添加该学校角色
            Role role = new Role();
            role.setRoleName("超级管理员");
            role.setSchoolId(schoolId);
            role.setPlatformId(platformId);
            role.setRoleCode(DateUtil.format(new Date(), "yyyy-MM-dd"));
            role.setRemark("系统默认角色，用于系统及用户的整体管理，可查看和操作所有功能，不可删除");
            int insert = baseMapper.insert(role);
            if (insert>0){
                // 角色赋予所有权限
                QueryWrapper<Permission> wrapper = new QueryWrapper<>();
                wrapper.select("id").eq("platform_id",platformId);
                List<Permission> list = permissionMapper.selectList(wrapper);
                list.forEach(permission -> {
                    RolePermission rolePermission = new RolePermission();
                    rolePermission.setRoleId(role.getId());
                    rolePermission.setPermissionId(permission.getId());
                    rolePermissionMapper.insert(rolePermission);
                });
                UserRole userRole = new UserRole();
                userRole.setRoleId(role.getId());
                userRole.setAccountId(Integer.parseInt(accountId));
                //账号赋予角色
                userRoleMapper.insert(userRole);
            }
        }
        //添加该学校管理员角色
        Role role = new Role();
        role.setRoleName("管理员");
        role.setSchoolId(schoolId);
        role.setPlatformId(1);
        role.setRoleCode(DateUtil.format(new Date(), "yyyy-MM-dd"));
        role.setRemark("系统默认角色，用于系统及用户的整体管理，可查看和操作所有功能");
        int insert = baseMapper.insert(role);
        if (insert>0){
            // 角色赋予所有权限
            QueryWrapper<Permission> wrapper = new QueryWrapper<>();
            wrapper.select("id").eq("platform_id",1);
            List<Permission> list = permissionMapper.selectList(wrapper);
            list.forEach(permission -> {
                RolePermission rolePermission = new RolePermission();
                rolePermission.setRoleId(role.getId());
                rolePermission.setPermissionId(permission.getId());
                rolePermissionMapper.insert(rolePermission);
            });
            UserRole userRole = new UserRole();
            userRole.setRoleId(role.getId());
            userRole.setAccountId(Integer.parseInt(accountId));
            //账号赋予角色
            userRoleMapper.insert(userRole);
        }

        //添加该学校员工角色
        Role role1 = new Role();
        role1.setRoleName("用户");
        role1.setSchoolId(schoolId);
        role1.setPlatformId(2);
        role1.setRoleCode(DateUtil.format(new Date(), "yyyy-MM-dd"));
        role1.setRemark("系统默认角色，普通用户所属，可查看数据");
        int insert1 = baseMapper.insert(role1);
        if (insert1>0){
            // 角色赋予数据权限
            List<Integer> list = new ArrayList<>();
            list.add(110);
            list.add(129);
            list.add(130);
            list.add(131);
            list.forEach(permissionId -> {
                RolePermission rolePermission = new RolePermission();
                rolePermission.setRoleId(role1.getId());
                rolePermission.setPermissionId(permissionId);
                rolePermissionMapper.insert(rolePermission);
            });
            UserRole userRole = new UserRole();
            userRole.setRoleId(role1.getId());
            userRole.setAccountId(Integer.parseInt(accountId));
            //账号赋予角色
            userRoleMapper.insert(userRole);
        }
        return true;
    }

    @Override
    public R reassignmentPermissions() {
        //查询所有客户，更新他们所在职站的超管权限
        List<HrUserAccount> accountList = accountMapper.selectList(new QueryWrapper<HrUserAccount>().isNotNull("customer_id"));
        //查询所有权限
        List<Permission> permissions = permissionMapper.selectList(new QueryWrapper<Permission>().
                eq("platform_id", 1).eq("port", 0));

        for (HrUserAccount userAccount : accountList) {
            Integer roleId = baseMapper.selectCustomerSuper(userAccount.getId());
            //给角色分配权限之前，删除之前所有角色权限
            rolePermissionMapper.delete(new QueryWrapper<RolePermission>().eq("role_id", roleId));

            List<RolePermission> rolePermissionList = new ArrayList<>();
            for(Permission permission : permissions) {
                if(StringUtils.isEmpty(permission.getId())) {
                    continue;
                }
                RolePermission rolePermission = new RolePermission();
                rolePermission.setRoleId(roleId);
                rolePermission.setPermissionId(permission.getId());
                rolePermissionList.add(rolePermission);
            }
            rolePermissionService.saveBatch(rolePermissionList);
        }

        return null;
    }
}
