package com.huoran.users.utils;

import com.huoran.users.entity.req.ExcelImpUserInfoReq;
import com.huoran.users.utils.poi.CommonCode;
import com.huoran.users.utils.poi.ExceptionCast;
import org.apache.commons.lang.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class ExcelImportHelper {
    //判断row是否为空
    public static boolean isRowEmpty(Row row) {
        if (null == row) {
            return true;
        }
        int firstCellNum = row.getFirstCellNum();   //第一个列位置
        int lastCellNum = row.getLastCellNum();     //最后一列位置
        int nullCellNum = 0;    //空列数量
        for (int c = firstCellNum; c < lastCellNum; c++) {
            Cell cell = row.getCell(c);
            if (null == cell || CellType.BLANK == cell.getCellType()) {
                nullCellNum++;
                continue;
            }
            cell.setCellType(CellType.STRING);
            String cellValue = cell.getStringCellValue().trim();
            if (StringUtils.isEmpty(cellValue)) {
                nullCellNum++;
            }
        }
        //所有列都为空
        if (nullCellNum == (lastCellNum - firstCellNum)) {
            return true;
        }
        return false;
    }

    /**
     * 读取用户管理
     *
     * @param file
     * @return
     */
    public static List<ExcelImpUserInfoReq> readUser(MultipartFile file) {
        List<ExcelImpUserInfoReq> list = new ArrayList<ExcelImpUserInfoReq>();

        Workbook workbook = getWorkbook(file);
        ExcelImpUserInfoReq impUserInfoReq = null;
        // 循环工作表Sheet
        for (int numSheet = 0; numSheet < workbook.getNumberOfSheets(); numSheet++) {
            Sheet hssfSheet = workbook.getSheetAt(numSheet);
            if (hssfSheet == null) {
                continue;
            }

            int count = 1;
            // 循环行Row//开始行2
            for (int rowNum = 2; rowNum <= hssfSheet.getLastRowNum(); rowNum++) {
                Row row = hssfSheet.getRow(rowNum);

                if (isRowEmpty(row)) {
                    count++;
                    if (count == hssfSheet.getLastRowNum()) {
                        ExceptionCast.cast(CommonCode.EXCEL_FILE_NULL);
                    }
                    continue;
                }

                // Cell account;
                Cell name;
                Cell roleId;
                Cell email;
                Cell workNumber;
                Cell phone;
                if (row != null) {

                    impUserInfoReq = new ExcelImpUserInfoReq();

                    // if (row.getCell(0) != null) {
                    //     row.getCell(0).setCellType(CellType.STRING);
                    //     account = row.getCell(0);
                    // } else {
                    //     account = row.createCell(0);
                    // }

                    if (row.getCell(0) != null) {
                        row.getCell(0).setCellType(CellType.STRING);
                        name = row.getCell(0);
                    } else {
                        name = row.createCell(0);
                    }

                    if (row.getCell(1) != null) {
                        row.getCell(1).setCellType(CellType.STRING);
                        roleId = row.getCell(1);
                    } else {
                        roleId = row.createCell(1);
                    }

                    if (row.getCell(2) != null) {
                        row.getCell(2).setCellType(CellType.STRING);
                        email = row.getCell(2);
                    } else {
                        email = row.createCell(2);
                    }

                    if (row.getCell(3) != null) {
                        row.getCell(3).setCellType(CellType.STRING);
                        workNumber = row.getCell(3);
                    } else {
                        workNumber = row.createCell(3);
                    }

                    if (row.getCell(4) != null) {
                        row.getCell(4).setCellType(CellType.STRING);
                        phone = row.getCell(4);
                    } else {
                        phone = row.createCell(4);
                    }


                    /*row.getCell(0).setCellType(CellType.STRING);
                    account = row.getCell(0);

                    row.getCell(1).setCellType(CellType.STRING);
                    name = row.getCell(1);
                    row.getCell(2).setCellType(CellType.STRING);
                    roleId = row.getCell(2);

                    row.getCell(3).setCellType(CellType.STRING);
                    email = row.getCell(3);

                    row.getCell(4).setCellType(CellType.STRING);
                    workNumber = row.getCell(4);*/

                    // 账号
                    // impUserInfoReq.setAccount(account.getStringCellValue());
                    // 用户姓名
                    impUserInfoReq.setUserName(name.getStringCellValue());
                    // 账户角色
                    impUserInfoReq.setRoleId(roleId.getStringCellValue());
                    //邮箱
                    impUserInfoReq.setEmail(email.getStringCellValue());
                    //学号工号
                    impUserInfoReq.setWorkNumber(workNumber.getStringCellValue());

                    impUserInfoReq.setPhone(phone.getStringCellValue());

                    list.add(impUserInfoReq);
                }
            }

        }

        return list;
    }

    private static Workbook getWorkbook(MultipartFile file) {
        String fileName = file.getOriginalFilename();
        Workbook workbook = null;

        if (fileName.endsWith("xlsx")) {
            try {
                workbook = new XSSFWorkbook(file.getInputStream());
            } catch (IOException e) {
                e.printStackTrace();
            }
        } else if (fileName.endsWith("xls")) {
            try {
                workbook = new HSSFWorkbook(file.getInputStream());
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return workbook;
    }
}
