package com.huoran.websocket.config.result;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.io.Serializable;

/**
 * 返回的数据格式
 *
 * @author 王松
 * @WX-QQ 1720696548
 * @date 2019/11/14 14:55
 */

@Data
@ApiModel(value = "WebSocketR", description = "统一返回格式")
public class WebSocketR<T> implements Serializable {

	private static final long serialVersionUID = -5666504070515657048L;

	@ApiModelProperty(value = "状态码")
	private Integer code;

	@ApiModelProperty(value = "状态描叙")
	private String msg;

	/**
	 * 注解 @JsonInclude(value = JsonInclude.Include.NON_EMPTY) 设置空不返回
	 */
	@ApiModelProperty(value = "返回数据")
	private T data;

	@ApiModelProperty(value = "错误的栈内存信息")
	private String errorMsg;


	private WebSocketR(Integer code, String msg, T data, String errorMsg) {
		this.code = code;
		this.data = data;
		this.msg = msg;
		this.errorMsg = errorMsg;
	}


	public static <T> WebSocketR<Void> success() {
		return new WebSocketR<>(WebSocketRType.SYS_SUCCESS.getValue(), WebSocketRType.SYS_SUCCESS.getMsg(), null, null);
	}

	public static <T> WebSocketR<T> success(T data) {
		return new WebSocketR<>(WebSocketRType.SYS_SUCCESS.getValue(), WebSocketRType.SYS_SUCCESS.getMsg(), data, null);
	}

	public static <T> WebSocketR<String> error(Integer code, String msg) {
		return new WebSocketR<>(code, msg, null, null);
	}
}
