package com.huoran.websocket.model.vo;

import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.io.Serializable;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;


/**
 *  websocket客户端-接收端-返回参数
 *
 * @author 王松
 * @mail 1720696548@qq.com
 * @date 2020/6/30 0030 18:24
 */
@Data
public class SendMsgVO implements Serializable {
    private static final long serialVersionUID = -9051438901021877148L;

    /**
     * 发送人用户Id (来源Id,上线为上线线人的用户Id)
     */
    @ApiModelProperty("发送人账号Id")
    private String from;
    /**
     * 发送人用户名 (上下线为上线线人的用户名)
     */
    @ApiModelProperty("发送人用户名 (上下线为上线线人的用户名)")
    private String username;

    /**
     * 接收人用户Id (目标ID,逗号分隔) (所有人使用-ALL,包括自己在内也能接收)
     */
    @ApiModelProperty("接收人用户Id (目标ID,逗号分隔) (所有人使用-ALL,包括自己在内也能接收)")
    private String to;
    /**
     * 发送内容(text/json)
     */
    @ApiModelProperty("发送内容(text/json)，content定义：1赛事、2创业、3考核、4模型。-号拼接携带id。例：3-2：表示属于考核消息，2为考核id")
    private String content;
    /**
     * 扩展消息字段(text/json)
     */
    @ApiModelProperty("扩展消息字段(默认为空)")
    private String extras;
    /**
     * 当前在线人数
     */
    @ApiModelProperty("当前在线人数")
    private Integer onlineNum;
    /**
     * 消息创建时间(YYYY-MM-DD )
     */
    @ApiModelProperty("消息创建时间(YYYY-MM-DD )")
    private String createTime;


    public SendMsgVO() {
        DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        this.createTime = df.format(LocalDateTime.now());
    }
}
