package com.huoran.websocket.server;

import com.alibaba.fastjson.JSON;
import com.huoran.websocket.model.entity.OnlineUser;
import com.huoran.websocket.model.vo.SendMsgVO;
import lombok.extern.slf4j.Slf4j;

import javax.websocket.Session;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;
import java.util.Set;

/**
 * wensocket 提供的 api 方法
 *
 * @author wangsong
 * @date 2023/05/24
 */
@Slf4j
public class OpenWebSocketApi {

    /**
     * 消息发送（ 遍历用户Id , 在通过sendMsg方法发送消息）
     * <P>
     *     提供给外部或当前监听消息发送消息
     * </P>
     *
     * @param sendMsg：消息内容
     */
    public static void send(SendMsgVO sendMsg) {
        List<String> all = Arrays.asList("all", "ALL");
        if (all.contains(sendMsg.getTo())) {
            // 发送消息给所有人
            Set<String> userIds = WebsocketServer.getClients().keySet();
            for (String userId : userIds) {
                sendMsg(userId, sendMsg);
            }
        } else {
            // 发送消息给指定人
            String[] userIds = sendMsg.getTo().split(",");
            for (String userId : userIds) {
                sendMsg(userId, sendMsg);
            }
        }
    }


    /**
     * 消息发送(最后发送, 在send方法中循环用户Id 列表依次发送消息给指定人)
     * <p>
     * // 消息发送（同步:getBasicRemote 异步:getAsyncRemote）
     * </P>
     *
     * @param userId  消息接收人ID , onlineUsers 的 key
     * @param sendMsg 消息内容
     */
    private static boolean sendMsg(String userId, SendMsgVO sendMsg) {
        // 判断用户是否在线, 在线发送消息推送
        if (!WebsocketServer.getClients().containsKey(userId)) {
            log.info("websocket用户ID:{} 不在线: 推送信息失败, 消息：{} ", userId, JSON.toJSONString(sendMsg.toString()));
        }
        List<OnlineUser> onlineUsers = WebsocketServer.getClients().get(userId);
        try {
            for (OnlineUser onlineUser : onlineUsers) {
                Session session = onlineUser.getSession();
                if (session == null) {
                    continue;
                }
                if (!session.isOpen()) {
                    session.close();
                }
                // 此处存在同一个 session 被多个线程调用问题, 使用 synchronized 锁定排队执行
               synchronized (session){
                   onlineUser.getSession().getBasicRemote().sendText(JSON.toJSONString(sendMsg));
               }
            }
            return true;
        } catch (IOException e) {
            log.error(e.toString());
            return false;
        }
    }
}
