package com.huoran.websocket.topic;

import com.huoran.websocket.server.WebsocketServer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.data.redis.connection.RedisConnectionFactory;
import org.springframework.data.redis.listener.ChannelTopic;
import org.springframework.data.redis.listener.RedisMessageListenerContainer;
import org.springframework.data.redis.listener.adapter.MessageListenerAdapter;

/**
 * 订阅者配置
 * @author wangsong
 * @date 2023/05/17
 */
@Configuration
public class SubConfig {

    @Autowired
    private WebsocketServer websocketServer;

    @Bean
    public MessageListenerAdapter messageListener() {
        return new MessageListenerAdapter(new WebsocketMsgListener(websocketServer));
    }

    @Bean
    @Primary
    public RedisMessageListenerContainer redisContainer(RedisConnectionFactory factory) {
        RedisMessageListenerContainer container = new RedisMessageListenerContainer();
        container.setConnectionFactory(factory);
        container.addMessageListener(messageListener(), new ChannelTopic("pubsub:queue-websocket"));
        return container;
    }
}
